import { AppConfig } from '../app/vitis/app.model';
import {
  vitisUsersMode,
  vitisLogsMode,
  vitisConfigVitisObject,
  vitisConfigSharedDirObject,
  vitisConfigEmailTemplatesObject,
  vitisAPIVitisWebService,
  vitisUserHeader,
  vitisHelpHeader,
  vitisTranslateConfig,
  vitisConfigInterfaceObject,
  vitisConfigCustomTranslationsObject,
  vitisAdditionalRouteObject
} from '../app/vitis/app.struct';
import {
  vmapConfigVmapObject,
  vmapTranslateConfig,
  vmapAPIVmapWebService,
  vmapAdminDatasourceObject,
  vmapAdminCartoObject,
  vmapMapsLayersMode,
  vmapSymbolsFontsMode,
  vmapCartoWidget,
  vmapPrintWidget,
  vmapMapPrintingsMode,
  vmapUsersSection,
  vmapUsersGroupsSpecSection,
  vmapFluxMode,
  vmapMapReportsMode
} from '../modules/vmap/struct/vmap.struct';

/**
 * Mode users revisité pour vMap
 */
const userObject = vitisUsersMode.objects[0];
userObject.sections = [{
  name: 'vitis_users_users_details',
  label: 'FORM_GENERAL_SECTION_CONFIGURATION',
  route: 'details',
  type: 'simpleform',
  privilege: 'vitis_admin',
  options: vitisUsersMode.objects[0].options,
}, vmapUsersSection];

const groupObject = vitisUsersMode.objects[2];
groupObject.sections = [{
  name: 'vitis_users_groups_details',
  label: 'FORM_GENERAL_SECTION_CONFIGURATION',
  route: 'details',
  type: 'simpleform',
  privilege: 'vitis_admin',
  options: vitisUsersMode.objects[2].options,
}, vmapUsersGroupsSpecSection];

export const appStruct: AppConfig = {
  'translate': [vitisTranslateConfig, vmapTranslateConfig],
  'name': 'vMap',
  'logo': {
    'type': 'icon',
    'class': 'icon-vitis-vitis'
  },
  'favicon': 'conf/images/Veremes-Leaf-Frame-W.png',
  'modes': [{
    'type': 'separator',
    'label': 'TITLE_SEPARATOR_MAP',
    'privilege': 'vmap_user'
  },
  vmapAdminCartoObject,
  {
    'type': 'separator',
    'label': 'TITLE_MODE_VMAP_ADMIN',
    'privilege': 'vmap_admin'
  },
  vmapAdminDatasourceObject,
  vmapMapsLayersMode,
  vmapSymbolsFontsMode,
  vmapFluxMode,
  {
    'type': 'separator',
    'label': 'Administration des impressions',
    'privilege': 'vitis_admin'
  },
  vmapMapPrintingsMode,
  vmapMapReportsMode,
  {
    'type': 'separator',
    'label': 'TITLE_MODE_VMAP_ADMINISTRATION',
    'privilege': 'vitis_admin'
  },
  vitisUsersMode,
  vitisConfigSharedDirObject,
  {
    'name': 'vitis_config',
    'label': 'TITLE_MODE_CONFIGURATION',
    'icon': 'icon-vitis-cogs',
    'route': 'config',
    'privilege': 'vitis_admin',
    'objects': [
      vitisConfigVitisObject,
      vmapConfigVmapObject,
      vitisConfigEmailTemplatesObject,
      vitisConfigInterfaceObject,
      vitisConfigCustomTranslationsObject,
      vitisAdditionalRouteObject]
  }, {
    'name': 'vitis_api',
    'label': 'TITLE_MODE_CONFIGURATION_API',
    'icon': 'icon-vitis-code',
    'route': 'api_doc',
    'type': 'api_doc',
    'privilege': 'vitis_admin',
    'options': {
      'web_services': [vitisAPIVitisWebService, vmapAPIVmapWebService],
      'base_path': '/v2'
    }
  }, vitisLogsMode],
  'headers': [vitisUserHeader, vitisHelpHeader],
  'widgets': [vmapCartoWidget, vmapPrintWidget],
  'documentation': [
    {
      'name': 'vMap',
      'url': 'http://documentation.veremes.net/vmap/fr/index.html'
    }
  ],
  'credits': {
    'authors': [
      'Alexandre Miromenkov',
      'Anthony Borghi',
      'Armand Bahi',
      'Frederic Carretero',
      'Laurent Panabières',
      'Laurie Ambroise',
      'Ludivine Premont',
      'Mickael Fernandez',
      'Olivier Gayte',
      'Sofian Pujo',
    ],
    'graphics': [
      'Alexandre Roy'
    ],
    'libs': [
      'Apache',
      'Angular',
      'Bootstrap',
      'Postgres',
      'PostGIS',
      'Mapserver',
      'Symfony',
      'OpenLayers',
      'Open Street Maps'
    ],
    'contact': 'www.veremes.com',
    'licence': '© Veremes'
  }
};
