# Installation de vMap avec Docker

Veremes propose également la possibilité d'installer vMap via Docker.

Cela rend possible une première utilisation du produit (découverte, familiarisation ou approche commerciale) rapidement, et sans nécessité de connaissances particulières (aucune configuration).  

```{admonition} Précautions
:class: tip
L'image de vMap, ainsi que la place nécessaire étant assez conséquent, il est recommandé d'avoir **au moins 10 Go d'espace disque disponible** afin de ne pas rencontrer de problèmes d'espace durant l'installation.
```

```{admonition} Attention
:class: danger
Il est à noter qu'il ne faut **en aucun cas utiliser l'image brute en production**.
N'ayant pas été conçu dans ce but, la configuration reste générique (même mot de passe, ...) et certaines configurations sont omises (certificat SSL, ...).

Si votre but est de déployer une application en production, vous pouvez retrouver la documentation pour une installation en production [au lien suivant](../installation/installation.md)
```

## Installation sous Windows

[Publication prochaine]

### Installation de Docker

```{admonition} Lien vers le site de docker
:class: tip
[Site de Docker](https://www.docker.com/get-started/)
```

Une fois sur le site, il faut cliquer sur "Download for Windows"  
![telechargement_docker](../images/installation/docker/windows/telechargement_docker.jpg)  

Une fois le téléchargement terminé, vous pouvez exécuter l'exécutable, et suivre les différentes étapes :  
![installation_docker_1](../images/installation/docker/windows/setup_docker_1.jpg)  

Cocher "Use WSL 2 instead of Hyper-V"
![installation_docker_2](../images/installation/docker/windows/setup_docker_2.jpg)

Docker demande de redémarrer l'ordinateur à la fin de l'installation, **c'est une étape obligatoire, sinon le moteur de docker ne peut pas démarrer**
![installation_docker_3](../images/installation/docker/windows/setup_docker_3.jpg)

Après le redémarrage de l'ordinateur, vous deviez voir une interface, il faut attendre que le rectangle en bas à gauche tourne au vert, une fois ceci fait, votre installation de docker est prête.
![installation_docker_4](../images/installation/docker/windows/setup_docker_5.jpg)

### Installation et démarrage de l'instance vMap sur Docker

```{admonition} Image vMap2 sur Docker Hub
:class: tip
Notre image Docker pour vMap2, ainsi que la documentation relative à cette image est disponible [à l'adresse suivante](https://hub.docker.com/r/veremes/vmap).
```

Il faut dans un premier temps ouvrir une fenêtre d'invite de commandes
```{admonition} Ouvrir un invite de commande
:class: note
Vous pouvez utiliser le raccourci "touche windows + r" et taper dans la fenêtre "cmd.exe" afin de démarrer un invite de commandes.
![ouverture_invite_de_commandes](../images/installation/docker/windows/ouvrir_cmd.jpg)
```

Et copier la commande suivante :

```batch
docker run -dit --name vmap -p 8080:8080 -e BIND_PORT=8080 -d veremes/vmap:latest
```

```{admonition} Précautions en cas de lancement de multiples instances
:class: danger
Si jamais plusieurs instances vMap2 venaient à être lancées, il faut absolument faire attention à ne **pas utiliser le même nombre que pour la précédente** (8080 dans le cas d'exemple), dans le cas contraire, des conflits pourraient rendre les machines inutilisables.
```

La commande devrait dans un premier temps récupérer l'image de vMap, puis vous démarrer l'instance, le temps nécéssaire est variable en fonction de la puissance de l'ordinateur, et la connexion réseau.

Une fois le processus terminé (un bon moyen de le savoir est de voir un conteneur démarré et en vert sur l'interface) :
![conteneur_fonctionnel](../images/installation/docker/windows/conteneur_vmap_on.jpg)

Une fois ceci fait, vous pouvez vous rendre à l'adresse suivante [localhost:8080/vmap](http://localhost:8080/vmap) (changer le 8080 dans le cas ou un nombre différent est précisé dans la commande précédente).

Vous devriez arriver devant l'interface suivante :
![page_connexion_vmap](../images/installation/docker/windows/connexion_vmap.jpg)

Vous pouvez donc vous connecter et commencer à utiliser vMap

```{admonition} Identifiants vMap Docker par défaut
:class: tip
**Nom d'utilisateur : admin**  
**Mot de passe : admin**
```

---

## Installation sous Linux

### Installation de Docker

```{admonition} Exemple pour Ubuntu / Debian
:class: warning
Les indications suivantes sont un résumé de l'installation de la documentation oficielle, l'installation de Docker sous Linux étant plus compliquée et dépendante du système Linux.
Il est recommandé d'utiliser la documentation oficielle disponible [au lien suivant](https://docs.docker.com/desktop/install/linux-install/) pour installer Docker sous Linux.
```

Pour installer Docker sur Linux, il faut dans un premier temps, ajouter les dépôts de Docker à la distribution

Mise à jour d'apt, et installation des paquets nécéssaires :
```bash
 sudo apt-get update

 sudo apt-get install \
    ca-certificates \
    curl \
    gnupg \
    lsb-release
```

Ajout de la clef GPG de Docker :  
```bash
sudo mkdir -p /etc/apt/keyrings
curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /etc/apt/keyrings/docker.gpg
```

Ajout du dépôt :  
```bash
echo \
  "deb [arch=$(dpkg --print-architecture) signed-by=/etc/apt/keyrings/docker.gpg] https://download.docker.com/linux/ubuntu \
  $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null
```

Mise à jour de la liste des paquets apt :
```bash
sudo apt-get update
```

Installation du moteur Docker :  
```bash
sudo apt-get install docker-ce docker-ce-cli containerd.io docker-compose-plugin
```

````{admonition} Exemple pour Ubuntu / Debian
:class: note
Vous pouvez aussi installer l'interface desktop de docker en passant par le paquet fourni par Docker, exemple avec le paquet DEB pour Ubuntu / Debian disponible à [l'adresse suivante](https://desktop.docker.com/linux/main/amd64/docker-desktop-4.16.0-amd64.deb?utm_source=docker&utm_medium=webreferral&utm_campaign=docs-driven-download-linux-amd64)

Il suffit de lancer les commandes suivantes :

```bash
sudo apt-get update
sudo apt-get install ./docker-desktop-<version>-<arch>.deb
```

Et à la fin de l'installation, vous devriez avoir l'application "Docker Desktop" dans les applications installées.

Cela vous permet d'avoir une interface pour gérer les conteneurs de ce type :  
![installation_docker_4](../images/installation/docker/windows/setup_docker_5.jpg)
````

### Lancement de vMap avec Docker

```{admonition} Image vMap sur Docker Hub
:class: tip
Notre image Docker pour vMap2, ainsi que la documentation relative à cette image est disponible [à l'addresse suivante](https://hub.docker.com/r/veremes/vmap).
```

Il faut copier la commande suivante dans un terminal :  

```bash
sudo docker run -dit --name vmap -p 8080:8080 -e BIND_PORT=8080 -d veremes/vmap:latest
```

```{admonition} Précautions en cas de lancement de multiples instances
:class: danger
Si jamais plusieurs instances vMap2 venaient à être lancées, il faut absolument faire attention à ne **pas utiliser le même nombre que pour la précédente** (8080 dans le cas d'exemple), dans le cas contraire, des conflits pourraient rendre les machines inutilisables.
```

La commande devrait dans un premier temps récupérer l'image de vMap, puis vous démarrer l'instance, le temps nécéssaire est variable en fonction de la puissance de l'ordinateur, et la connexion réseau.

Vous pouvez voir si la machine est démarée avec la commande suivante :

```bash
sudo docker container ls
```

Voici un exemple de sortie de la commande dans la console :  
```bash
CONTAINER ID   IMAGE                 COMMAND                  CREATED          STATUS          PORTS                                       NAMES
716f1bc79690   veremes/vmap:latest   "bin/bash -c /usr/lo…"   12 minutes ago   Up 12 minutes   0.0.0.0:8080->8080/tcp, :::8080->8080/tcp   vmap
```

Une fois ceci fait, vous pouvez vous rendre à l'adresse suivante [localhost:8080/vmap](http://localhost:8080/vmap) (changer le 8080 dans le cas ou un nombre différent est précisé dans la commande précédente).

Vous devriez arriver devant l'interface suivante :
![page_connexion_vmap](../images/installation/docker/windows/connexion_vmap.jpg)

Vous pouvez donc vous connecter et commencer à utiliser vMap

```{admonition} Identifiants vMap Docker par défaut
:class: tip
**Nom d'utilisateur : admin**  
**Mot de passe : admin**
```
