<?php

namespace App\Module\Cadastre\WebService\Collection;

/**
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 */

class AbstractSufCollection extends MajicCollection {
  /**
   * @return string|null
   * Cette méthode permet de récupérer la valeur de l'attribut 's_tar' dans le tableau aSuf.
   */
  public function getSTar(): ?string {
    if ($this->count() == 1) {
      return $this->first()->getCcostn();
    }
    return null;
  }

  /**
   * @return string|null
   * Cette méthode permet de récupérer la valeur de l'attribut 'suf' dans le tableau aSuf.
   */
  public function getSuf(): ?string {
    if ($this->count() == 1) {
      return $this->first()->getCcosub();
    }
    return null;
  }

  /**
   * @return string|null
   * Cette méthode permet de récupérer la valeur de l'attribut 'grss_gr' dans le tableau aSuf.
   */
  public function getGrssGr(): ?string {
    if ($this->count() == 1) {
      return $this->first()->getDsgrpf();
    }
    return null;
  }
  /**
   * @return string|null
   * Cette méthode permet de récupérer la valeur de l'attribut 'clas' dans le tableau aSuf.
   */
  public function getClas(): ?string {
    if ($this->count() == 1) {
      return $this->first()->getDclssf();
    }
    return null;
  }
  /**
   * @return string|null
   * Cette méthode permet de récupérer la valeur de l'attribut 'nat_cult' dans le tableau aSuf.
   */
  public function getNatCult(): ?string {
    if ($this->count() == 1) {
      return $this->first()->getCnatsp();
    }
    return null;
  }

  /**
   * @return integer
   * Cette méthode permet de récupérer la valeur de l'attribut 'contenance' pour des parcelles avec exonération dans le tableau aSuf.
   */
  public function getContenance(): int {
    $iContenance = 0;
    foreach ($this->toArray() as $oSuf) {
      $iContenance += $oSuf->getContenance();
    }
    return $iContenance;
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' dans le tableau aSuf.
   */
  public function getRevenuCadastral(): float {
    $fRevenuCadastral = 0;
    foreach ($this->toArray() as $oSuf) {
      $fRevenuCadastral += $oSuf->getRevenuCadastral();
    }
    return $fRevenuCadastral;
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu reference' dans le tableau aSuf.
   * Méthode corrigée.
   */
  public function getRevenuReference(): float {
    $fRevenuReference = 0;
    foreach ($this->toArray() as $oSuf) {
      $fRevenuReference += $oSuf->getRevenuReference();
    }
    return $fRevenuReference;
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf.
   * Méthode corrigée.
   */
  public function getRevenuCadastralAvecExo(): float {
    $fRevenuCadastralAvecExo = 0;
    foreach ($this->toArray() as $oSuf) {
      if (preg_match("/TA/", $oSuf->getNatExo())) {
        $fRevenuCadastralAvecExo += $oSuf->getRevenuCadastral();
      }
    }
    return $fRevenuCadastralAvecExo;
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf.
   * Lorsque l'exoneration est procuré par une commune, un groupement de commune ou toutes collectivité.
   * Méthode corrigée.
   */
  public function getRevenuCadastralComAvecExo(): float {
    $fRevenuCadastralAvecExo = 0;
    foreach ($this->toArray() as $oSuf) {
      $fRevenuCadastralExo = $oSuf->getRevenuCadastralAvecExo();
      if ($fRevenuCadastralExo !== null) {
        $fRevenuCadastralAvecExo += $fRevenuCadastralExo;
      }
    }
    return $fRevenuCadastralAvecExo;
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf.
   * Lorsque l'exoneration est procuré par une commune, un groupement de commune ou toutes collectivité.
   * Méthode corrigée.
   */
  public function getRevenuCadastralTaxeADAvecExo(): float {
    $fRevenuCadastralAvecExo = 0;
    foreach ($this->toArray() as $oSuf) {
      $fRevenuCadastralExo = $oSuf->getRevenuCadastralTaxeADAvecExo();
      if ($fRevenuCadastralExo !== null) {
        $fRevenuCadastralAvecExo += $fRevenuCadastralExo;
      }
    }
    return $fRevenuCadastralAvecExo;
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf.
   * Lorsque l'exoneration est procuré par une commune, un groupement de commune ou toutes collectivité.
   * Méthode corrigée.
   */
  public function getRevenuCadastralDepAvecExo(): float {
    $fRevenuCadastralAvecExo = 0;
    foreach ($this->toArray() as $oSuf) {
      $fRevenuCadastralExo = $oSuf->getRevenuCadastralDepAvecExo();
      if ($fRevenuCadastralExo !== null) {
        $fRevenuCadastralAvecExo += $fRevenuCadastralExo;
      }
    }
    return $fRevenuCadastralAvecExo;
  }
  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf.
   * Lorsque l'exoneration est procuré par la région ou toutes collectivité.
   * Méthode corrigée.
   */
  public function getRevenuCadastralRegAvecExo(): float {
    $fRevenuCadastralAvecExo = 0;
    foreach ($this->toArray() as $oSuf) {
      $fRevenuCadastralExo = $oSuf->getRevenuCadastralRegAvecExo();
      if ($fRevenuCadastralExo !== null) {
        $fRevenuCadastralAvecExo += $fRevenuCadastralExo;
      }
    }
    return $fRevenuCadastralAvecExo;
  }
}
