<?php

namespace App\Module\Cadastre\WebService\Collection;

/**
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 */

class BatiCollection extends MajicCollection {
  /**
   * @return integer
   * Cette méthode permet de calculer le revenu imposable total du bâti
   */
  public function getRevImpoBati(): int {
    $iRevImpoBati = 0;
    foreach ($this->toArray() as $oBat) {
      $iRevImpoBati += $oBat->getPevCollection()->getRevenuCadastral();
    }
    return $iRevImpoBati;
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur locative totale du bâti
   */
  public function getValeurLocativeBati(): float {
    $iValeurLocativeBati = 0;
    foreach ($this->toArray() as $oBat) {
      $iValeurLocativeBati += $oBat->getPevCollection()->getDvlpera();
    }
    return $iValeurLocativeBati;
  }

  /**
   * @param string $LCcodem attribut l_ccodem de proprietaire
   * @return integer
   * Cette méthode permet de calculer l'exonération communale totale du bâti
   * Méthode corrigée
   */
  public function getExoComBati($LCcodem): int {
    $iExoComBati = 0;
    foreach ($this->toArray() as $oBat) {
      $iExoComBati += $oBat->getPevCollection()->getExoneration($LCcodem);
    }
    return $iExoComBati;
  }

  /**
   * @param string $LCcodem attribut l_ccodem de proprietaire
   * @return integer
   * Cette méthode permet de calculer l'exonération départementale totale du bâti
   * Méthode corrigée
   */
  public function getExoDepBati($LCcodem): int {
    $iExoDepBati = 0;
    foreach ($this->toArray() as $oBat) {
      $iExoDepBati += $oBat->getPevCollection()->getExoneration($LCcodem);
    }
    return $iExoDepBati;
  }

  /**
   * @param string $LCcodem attribut l_ccodem de proprietaire
   * @return integer
   * Cette méthode permet de calculer l'exonération régionale totale du bâti
   */
  public function getExoRegBati($LCcodem): int {
    $iExoRegBati = 0;
    foreach ($this->toArray() as $oBat) {
      $iExoRegBati += $oBat->getPevCollection()->getExoneration($LCcodem);
    }
    return $iExoRegBati;
  }

  /**
   * @return float
   * Cette méthode permet de calculer le revenu imposable total du bâti
   */
  public function getRevImpoBatifloat(): float {
    $fRevImpoBati = 0;
    foreach ($this->toArray() as $oBat) {
      $fRevImpoBati += floatval($oBat->getPevCollection()->getRevenuCadastralfloat());
    }
    return $fRevImpoBati;
  }

  /**
   * @param string $LCcodem attribut l_ccodem de proprietaire
   * @return float
   * Cette méthode permet de calculer l'exonération communale totale du bâti
   * Méthode corrigée
   */
  public function getExoComBatifloat($LCcodem): float {
    $fExoComBati = 0;
    foreach ($this->toArray() as $oBat) {
      $fExoComBati += $oBat->getPevCollection()->getExonerationfloat($LCcodem);
    }
    return $fExoComBati;
  }

  /**
   * @param string $LCcodem attribut l_ccodem de proprietaire
   * @return float
   * Cette méthode permet de calculer l'exonération départementale totale du bâti
   * Méthode corrigée
   */
  public function getExoDepBatifloat($LCcodem): float {
    $fExoDepBati = 0;
    foreach ($this->toArray() as $oBat) {
      $fExoDepBati += $oBat->getPevCollection()->getExonerationfloat($LCcodem);
    }
    return $fExoDepBati;
  }

  /**
   * @param string $LCcodem attribut l_ccodem de proprietaire
   * @return float
   * Cette méthode permet de calculer l'exonération régionale totale du bâti
   */
  public function getExoRegBatifloat($LCcodem): float {
    $fExoRegBati = 0;
    foreach ($this->toArray() as $oBat) {
      $fExoRegBati += $oBat->getPevCollection()->getExonerationfloat($LCcodem);
    }
    return $fExoRegBati;
  }
}
