<?php

namespace App\Module\Cadastre\WebService\Collection;

use Doctrine\ORM\EntityManager;

/**
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 */

class CompteCommunalParcelleCollection extends MajicCollection {
  /**
   * @return float
   * Récupération des revenus de la subdivision fiscale des parcelles
   */
  public function getRevenuCadastral(): float {
    $data = 0;
    foreach ($this->toArray() as $oCc) {
      $data += floatval($oCc->getRevenuCadastralTotal());
    }
    return $data;
  }

  /**
   * @return integer
   * Récupération de la surface totale des parcelles
   */
  public function getSurfaceCadastrale(): int {
    $data = 0;
    foreach ($this->toArray() as $oCc) {
      $data += $oCc->getTotalContenance();
    }
    return $data;
  }

  /**
   * @return integer
   * récupération de la surface totale des parcelles en mettre carré.
   */
  public function getSurfaceCadastraleM2(): int {
    $dSurfaceCadastraleM2 = 0;
    foreach ($this->toArray() as $oCc) {
      $dSurfaceCadastraleM2 += intval($oCc->getTotalContenanceM2());
    }
    return $dSurfaceCadastraleM2;
  }

  /**
   * @return float
   * récupération des valeurs locatives
   */
  public function getValeurLocative(): float {
    $dValeurLocative = 0;
    foreach ($this->toArray() as $oCc) {
      $dValeurLocative += $oCc->getValeurLocativeBati();
    }
    return $dValeurLocative;
  }

  /**
   * @return void
   * récupération des valeurs locatives
   */
  public function getEmprise(EntityManager $oEntityManager): void {
    foreach ($this->toArray() as $oCc) {
      $oCc->getEmprises($oEntityManager);
    }
  }

  /**
   * @return integer
   * Récupération de la emprises bâties totale des parcelles
   */
  public function getSurfaceCadastraleBati(): int {
    $data = 0;

    foreach ($this->toArray() as $oCc) {
      foreach ($oCc->getParcelleCollection() as $oPar) {
        $intersect = $oPar->getIntersect();
        if (isset($intersect)) {
          foreach ($intersect as $empriseCollection) {
            if ($empriseCollection->getTitle() == "Emprise Bati") {
              $data += $empriseCollection->getEmpriseTotale();
            }
          }
        }
      }
    }
    return $data;
  }

  /**
   * @return string|null
   * Cette méthode permet de récupérer l'année de référence
   */
  public function getAnneeReference(): ?string {
    if ($this->isEmpty()) {
      return null;
    }
    return $this->first()->getAnneeReference();
  }

  /**
   * @return array
   * cette méthode permet de construire un tableau pour BatiParcelle.
   */
  public function getMajicData(): array{
    return [
      'Sum_revenu' => $this->getRevenuCadastral(),
      'Sum_contenance' => $this->getSurfaceCadastrale(),
      'Sum_contenance_m2' => $this->getSurfaceCadastraleM2(),
      'Sum_valeur_locative' => $this->getValeurLocative(),
      'Sum_contenance_bati' => $this->getSurfaceCadastraleBati(),
      'aCompteCommunalParcelle' => $this->toMajicData(),
    ];
  }
}
