<?php

namespace App\Module\Cadastre\WebService\Collection;

/**
 * Permet de gérer des Entité type majic
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 */

class EmpriseCollection extends MajicCollection {
  /**
   * @var array|null
   */
  private $definition;

/**
 * @return array
 * Formatage de la collection
 */
  protected function toMajicData(): array{
    $aMajicData = parent::toMajicData();
    // modification par référence
    foreach ($aMajicData as $i => &$aEmpriseData) {
      $aEmpriseData['id_emprise'] = $i;
    }
    return $aMajicData;
  }

  /**
   * @param array|null $definition
   * @return self
   */
  public function setDefinition(?array $definition): self {
    $this->definition = $definition;
    return $this;
  }

  /**
   * @return string|null
   * Récupération du titre
   */
  public function getTitle(): ?string {
    return $this->definition['title'] ?? null;
  }

  /**
   * @return string|null
   * Récupération du type de géométrie
   */
  public function getTypeGeom(): ?string {
    return $this->definition['type'] ?? null;
  }

  /**
   * @return array|null
   * Récupération des colums
   */
  public function getColumns(): ?array{
    return $this->definition['columns'] ?? null;
  }

  /**
   * @return float
   * Récupére l'emprise totale d'un ensemble d'emprise (bati ou pos ou servitude etc)
   */
  public function getEmpriseTotale(): float {
    $oData = 0;
    foreach ($this->toArray() as $oEmprise) {
      $oData += $oEmprise->getIntersect();
    }
    return $oData;
  }

  /**
   * @return float
   * Récupére le ratio de l'emprise totale d'un ensemble d'emprise (bati ou pos ou servitude etc)
   */
  public function getEmpriseRatioTotale(): float {
    $oData = 0;
    foreach ($this->toArray() as $oEmprise) {
      $oData += $oEmprise->getRatio();
    }
    return $oData;
  }

  /**
   * @return array
   * cette méthode permet de construire un tableau pour EmpriseCollection.
   */
  public function getMajicData(): array{
    return [
      'titre' => $this->getTitle(),
      'type_geom' => $this->getTypeGeom(),
      'emprise_totale_intersect' => $this->getEmpriseTotale(),
      'emprise_totale_ratio' => $this->getEmpriseRatioTotale(),
      'columns' => $this->getColumns(),
      'aEmprise' => $this->toMajicData(),
    ];
  }
}
