<?php

namespace App\Module\Cadastre\WebService\Collection;

use App\Module\Cadastre\WebService\Service\MajicUtils;
use Doctrine\ORM\EntityManager;

/**
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 */

class ParcelleCollection extends MajicCollection {
  /**
   * @return integer
   * Cette méthode permet de récupérer la valeur de l'attribut 'contenance' pour le compte communal avec exonération dans le tableau aParcelle
   */
  public function getContenance(): int {
    $iContenance = 0;
    foreach ($this->toArray() as $oPar) {
      $iContenance += $oPar->getContenance();
    }
    return $iContenance;
  }

/**
 * @return integer
 * Cette méthode permet de calculer le revenu imposable total des parcelles
 * Méthode modifiée
 */
  public function getRevImpoPar(): int {
    $fRevImpoPar = 0;
    foreach ($this->toArray() as $oPar) {
      $fRevImpoPar += floatval($oPar->getRevenuCadastral());
    }
    return intval(round($fRevImpoPar));
  }

  /**
   * @return integer
   * Cette méthode permet de calculer la contenance totale des parcelles
   */
  public function getTotalContenance(): int {
    $iContenance = 0;
    foreach ($this->toArray() as $oPar) {
      $iContenance += $oPar->getDcntpa();
    }
    return $iContenance;
  }

  /**
   * @return integer
   * Cette méthode permet de calculer la contenance totale des parcelles en mettre carré
   */
  public function getTotalContenanceM2(): int {
    $iContenanceM2 = 0;
    foreach ($this->toArray() as $oPar) {
      $iContenanceM2 += $oPar->getContenance();
    }
    return $iContenanceM2;
  }

  /**
   * @return float
   * Cette méthode permet de récupérer le revenu cadastral total des parcelles
   */
  public function getRevenuCadastralTotal(): float {
    $dRevCadastralTotal = 0;
    foreach ($this->toArray() as $oPar) {
      $dRevCadastralTotal += $oPar->getRevenuCadastral();
    }
    $dRevCadastralTotal = MajicUtils::setFloatToText($dRevCadastralTotal);
    return floatval($dRevCadastralTotal);
  }

  /**
   * @return integer
   * Cette méthode permet de calculer l'éxonération communale totale des parcelles
   */
  public function getExoComPar(): int {
    $fExoComPar = 0;
    foreach ($this->toArray() as $oPar) {
      $fExoComPar += $oPar->getRevenuCadastralComAvecExo();
    }
    return intval(round($fExoComPar));
  }

  /**
   * @return integer
   * Cette méthode permet de calculer l'exonération départementale totale des parcelles
   */
  public function getExoDepPar(): int {
    $fExoDepPar = 0;
    foreach ($this->toArray() as $oPar) {
      $fExoDepPar += $oPar->getRevenuCadastralDepAvecExo();
    }
    return intval(round($fExoDepPar));
  }

  /**
   * @return integer
   * Cette méthode permet de calculer l'exonération régionale totale des parcelles
   */
  public function getExoRegPar(): int {
    $fExoRegPar = 0;
    foreach ($this->toArray() as $oPar) {
      $fExoRegPar += $oPar->getRevenuCadastralRegAvecExo();
    }
    return intval(round($fExoRegPar));
  }

  /**
   * @return float
   * Cette méthode permet de calculer le revenu imposable total des parcelles
   */
  public function getRevImpoParfloat(): float {
    $fRevImpoPar = 0;
    foreach ($this->toArray() as $oPar) {
      $fRevImpoPar += floatval($oPar->getRevenuCadastral());
    }
    $fRevImpoPar = MajicUtils::setFloatToText($fRevImpoPar);
    return floatval($fRevImpoPar);
  }

  /**
   * @return float
   * Cette méthode permet de calculer l'éxonération communale totale des parcelles
   * Méthode modifiée
   */
  public function getExoComParfloat(): float {
    $fExoComPar = 0;
    foreach ($this->toArray() as $oPar) {
      $fExoComPar += $oPar->getRevenuCadastralComAvecExo();
    }
    $fExoComPar = MajicUtils::setFloatToText($fExoComPar);
    return floatval($fExoComPar);
  }

  /**
   * @return float
   * Cette méthode permet de calculer l'exonération départementale totale des parcelles
   */
  public function getExoDepParfloat(): float {
    $fExoDepPar = 0;
    foreach ($this->toArray() as $oPar) {
      $fExoDepPar += $oPar->getRevenuCadastralDepAvecExo();
    }
    return $fExoDepPar;
  }

  /**
   * @return float
   * Cette méthode permet de calculer l'exonération régionale totale des parcelles
   */
  public function getExoRegParfloat(): float {
    $fExoRegPar = 0;
    foreach ($this->toArray() as $oPar) {
      $fExoRegPar += $oPar->getRevenuCadastralRegAvecExo();
    }
    return $fExoRegPar;
  }

  /**
   * @return float
   * Cette méthode permet de calculer l'éxonération des taxes additionnelles totale des parcelles
   */
  public function getExoTaxeADParfloat(): float {
    $fExoTaxeADPar = 0;
    foreach ($this->toArray() as $oPar) {
      $fExoTaxeADPar += $oPar->getRevenuCadastralTaxeADAvecExo();
    }
    $fExoTaxeADPar = MajicUtils::setFloatToText($fExoTaxeADPar);
    return floatval($fExoTaxeADPar);
  }

  /**
   * @return float
   * Cette méthode permet de calculer le revenu imposable des taxes additionnelles des parcelles en fonction du revenu imposable total (parcelles) et de l'exonération communale totale (parcelles)
   * Méthode modifiée
   */
  public function getExoTaxeADPar(): float {
    $fExoTaxeADPar = 0;
    foreach ($this->toArray() as $oPar) {
      $fExoTaxeADPar += $oPar->getRevenuCadastralTaxeADAvecExo();
    }
    return round($fExoTaxeADPar);
  }

  /**
   * @return void
   * Cette méthode permet de récupérer les emprises
   */
  public function getEmprises(EntityManager $oEntityManager): void {
    foreach ($this->toArray() as $oPar) {
      $oPar->getEmprises($oEntityManager);
    }
  }

  /**
   * @return float
   * Cette méthode permet de calculer l'emprise bâtie totale des parcelles.
   * Méthode dans ParcelleCollection
   * Métode modifiée
   */
  public function getTotalContenanceBati(): float {
    $iContenanceBati = 0;
    foreach ($this->toArray() as $oPar) {
      foreach ($oPar->getIntersect() as $empriseCollection) {
        $iContenanceBati += $empriseCollection->getEmpriseTotale();
      }
    }
    return $iContenanceBati;
  }
}
