<?php

namespace App\Module\Cadastre\WebService\Collection;

use App\Module\Cadastre\WebService\Entity\Majic\Pev;

/**
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 */

class PevCollection extends MajicCollection {
  /**
   * @return integer|null
   * Cette méthode permet de récupérer le total des valeur locative des PEV
   * Getter modifié
   */
  public function getDvlpera(): ?int{
    $dvlpera = 0;
    foreach ($this->toArray() as $pev) {
      $dvlpera += $pev->getDvlpera();
    }
    return $dvlpera;
  }
  /**
   * @return string|null
   * Cette méthode permet de récupérer la valeur de l'attribut 's_tar' du premier aPev.
   */
  public function getSTar(): ?string {
    if ($this->count() == 1) {
      return $this->first()->getCcostb();
    }
    return null;
  }

  /**
   * @return string|null
   * Cette méthode permet de récupérer la valeur de l'attribut 'af' dans le tableau aPev.
   */
  public function getAf(): ?string {
    if ($this->count() == 1) {
      return $this->first()->Ccoaff();
    } else {
      return null;
    }
  }

  /**
   * @return string|null
   * Cette méthode permet de récupérer la valeur de l'attribut 'cat' dans le tableau aPev.
   */
  public function getCat(): ?string {
    if ($this->count() == 1) {
      return $this->first()->getDcapec();
    } else {
      return null;
    }
  }

  /**
   * @return integer
   * Cette méthode permet de récupérer la somme du revenu cadastral pour un bâti.
   */
  public function getRevenuCadastral(): int {
    $iRevenuCadastral = 0;
    foreach ($this->toArray() as $oPev) {
      $iRevenuCadastral += $oPev->getRevenuCadastral();
    }
    return $iRevenuCadastral;
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la somme du revenu cadastral pour un bâti.
   */
  public function getRevenuCadastralfloat(): float {
    $fRevenuCadastral = 0;
    foreach ($this->toArray() as $oPev) {
      $fRevenuCadastral += floatval($oPev->getRevenuCadastral());
    }
    return $fRevenuCadastral;
  }

  /**
   * @param string|null $sLccodem
   * @return integer
   * Cette méthode permet de récupérer la somme des exonérations pour un bâti
   * Méthode corrigée
   */
  public function getExoneration(?string $sLccodem = null): int {
    $iExoneration = 0;
    foreach ($this->toArray() as $oPev) {
      if ((($sLccodem != null) && (preg_match('/PP/', $oPev->getNatExo())) && (intval($oPev->getDnupev()) > 1))
        || ($oPev->getNatExo() != null)) {
        $iExoneration += $oPev->getRevenuCadastral();
      }
    }
    return $iExoneration;
  }

  /**
   * @param string|null $sLccodem
   * @return float
   * Cette méthode permet de récupérer la somme des exonérations pour un bâti
   */
  public function getExonerationfloat(?string $sLccodem = null): float {
    $fExoneration = 0;
    foreach ($this->toArray() as $oPev) {
      if ((($sLccodem != null) && (preg_match('/PP/', $oPev->getNatExo())) && (intval($oPev->getDnupev()) > 1))
        || ($oPev->getNatExo() != null)) {
        $fExoneration = $fExoneration + $oPev->getRevenuCadastralfloat();
      }
    }
    return $fExoneration;
  }
}
