<?php

namespace App\Module\Cadastre\WebService\Controller;

use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Serializer\Normalizer\AbstractNormalizer;

/**
 * Controller permettant la génération de la fiche descriptive d'une parcelle
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 */

class FicheDescriptiveParcelleController extends AbstractCadastreController {
  /**
   * Récupère toutes les entités pour une route de type GET Module/Objet.
   * @return Response réponse à retourner par l'API
   */
  public function getData($id): Response{
    $aData = $this->requestToArray();
    $sIdCom = substr($id, 0, 5);
    if (!$this->haveCommuneRights($sIdCom)) {
      $aReturn = [
        'message' => 'Erreur lors de la génération de la fiche descriptive de la parcelle (Vous n\'avez pas les droits sur la commune).',
        'errorMessage' => 'ERROR_GENERIC_CONTROLLER_GET',
        'status' => 0];
      return $this->formatResponse($aReturn, Response::HTTP_UNAUTHORIZED);
    }
    // chargement de la connexion à la base de donnée
    $oDataAccessor = $this->getDataAccessor();
    $oEntity = $oDataAccessor->getEntity($id, $aData);
    if ($oEntity['status'] == 0) {
      $oEntity['message'] = 'Error during get';
      $oEntity['errorMessage'] = 'ERROR_GENERIC_CONTROLLER_GET';
      return $this->formatResponse($oEntity, Response::HTTP_UNAUTHORIZED);
    }
    $aSerializerContext = [];
    if (isset($aData['attributs'])) {
      $aSerializerContext[AbstractNormalizer::ATTRIBUTES] = explode('|', $aData['attributs']);
    }

    // Recuperation de la géométrie de la parcelle
    $qb = $this->oDatabaseConnection->getQueryAccessor()->createQueryBuilder();
    $qb->select('ST_AsEWKT(m.geom) as geom')
      ->from('[SCHEMA].[TABLE]', 'm')
      ->where($qb->expr()->eq('m.id_par', '[ID_PAR]'));

    $aParams = [
      'SCHEMA' => ['value' => $this->oProperties->get('cadastre_schema_cadastre'), 'type' => 'schema_name'],
      'TABLE' => ['value' => $this->oProperties->get('cadastre_views:parcelle'), 'type' => 'table_name'],
      'ID_PAR' => ['value' => $id, 'type' => 'string'],

    ];

    $geom = null;

    try {
      $oGeom = $qb->executeWithParams($aParams)->fetchAllAssociative();
      if ($oGeom != null) {
        $geom = $oGeom[0]['geom'];
      }
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('The data could not be retrieved.' . $e->getMessage());
      $aReturn = ['status' => 0];
      $aReturn['message'] = 'The data could not be retrieved.';
      return $this->formatResponse($aReturn, Response::HTTP_NOT_ACCEPTABLE);
    }

    // Infos complémentaires (paramétrées par l'administrateur)
    $iTolerance = $this->oProperties->get('cadastre_descr_parcel_intersect_tolerance');
    $bInterectIdCom = $this->oProperties->get('cadastre_descr_parcel_intersect_id_com');
    $aIntersectionViews = $this->oProperties->get('cadastre_descr_parcel_intersect_tables');
    $intersection = [];

    if (!empty($aIntersectionViews)) {
      $intersection = $this->getIntersection($bInterectIdCom, $geom, $aIntersectionViews, $iTolerance, $id);
    }
    // Construction du JSON
    $oEntity['data']['aIntersections'] = $intersection;
    return $this->formatResponse($oEntity, Response::HTTP_OK, $aSerializerContext);
  }

  /**
   * récupère les intersections
   * @param bool $bInterectIdCom cadastre_fiche_urba_intersect_id_com dans les properties
   * @param string $geom géométrie de la parcelle
   * @param array $aIntersectionViews cadastre_fiche_urba_intersect_tables dans les properties
   * @param integer $iTolerance cadastre_fiche_urba_intersect_tolerance dans le sproperties
   * @param string $idPar paramètres de la route
   * @return array
   */
  protected function getIntersection($bInterectIdCom, $geom, $aIntersectionViews, $iTolerance, $idPar): array{
    return $this->getGeomIntersections($bInterectIdCom, $geom, $aIntersectionViews, $iTolerance, $idPar);
  }
}
