<?php

namespace App\Module\Cadastre\WebService\Controller;

use App\Module\Cadastre\WebService\Entity\Majic\BatiParcelle;
use App\Module\Cadastre\WebService\Entity\Majic\CompteCommunalParcelle;
use App\Module\Cadastre\WebService\Repository\BatiParcelleRepository;
use App\Module\Cadastre\WebService\Repository\CompteCommunalRepository;
use App\Module\Vitis\WebService\Controller\GenericController;
use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Configuration\Route;
use App\Module\Vitis\WebService\Service\DataAccessor\DatabaseConnection;
use App\Module\Vitis\WebService\Service\FileAccessor\FileManager;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use App\Module\Vitis\WebService\Service\PrintService\PrintTemplate;
use App\Module\Vitis\WebService\Service\VitisRequest;
use Ramsey\Uuid\Uuid;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller permettant la génération du rapport cadastre de parcelle
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 */

class RapportCadastreParcelleController extends GenericController {
  /**
   * @var PrintTemplate
   */
  protected $oPrintService;

  /**
   * Initialise le controler et instancie les services requis
   * @param Properties $oPropertiesSrvc Service d'accès aux properties
   * @param VitisRequest $oVitisRequest Service d'abstraction de la requête Symfony
   * @param Route $oRouteConfig Service d'accès à la configuration de la route
   * @param LoggerManager $oLoggerManager Service d'utilsation des logs
   * @param DatabaseConnection $oDatabaseConnection Service d'accès à la base de données
   * @param FileManager $oFileManager Service de gestion des fichiers
   * @param PrintTemplate $oTemplate Service de manipulation des token JWE
   */
  public function __construct(Properties $oPropertiesSrvc, VitisRequest $oVitisRequest, Route $oRouteConfig, LoggerManager $oLoggerManager, DatabaseConnection $oDatabaseConnection, FileManager $oFileManager, PrintTemplate $oTemplate) {
    parent::__construct($oPropertiesSrvc, $oVitisRequest, $oRouteConfig, $oLoggerManager, $oDatabaseConnection, $oFileManager);
    $this->oPrintService = $oTemplate;
  }

  /**
   * récupère les ids de parcelle
   * @return array
   */
  private function getIds(): array{
    $aData = $this->requestToArray();
    $idsPar = explode("|", $aData['parcelles']);
    return $idsPar;
  }

  /**
   * génère le rapport cadastre parcelle pour les locaux
   * @return Response
   */
  public function gererateReportLocaux(): Response {
    $aidPar = $this->getIds();
    $aBati = $this->getBatiParcelle($aidPar);
    $aData = $aBati->getMajicData();
    /* Génération du fichier Excel */
    $oFile = $this->oFileManager->getLocalFileAccessor();
    $sDirPath = '@prop(cadastre_report_dir)/' . $aBati[0]->getAnneeReference() . '/Rapport_Parcelle_Locaux/' . Uuid::uuid1()->toString() . '/';
    $sDirPath = $this->oProperties->replacePropertiesInString($sDirPath);
    $sPath = $sDirPath . 'Locaux' . '.xlsx';
    if (!$oFile->file_exists($sDirPath)) {
      $oFile->mkdir($sDirPath, 0777, true);
    }
    $sTemplatePath = $this->oProperties->replacePropertiesInString("@prop(cadastre_template_dir)/json_tpl_locaux.json");
    /* Génération de l'excel */
    $aReturn = $this->transformExcel($sTemplatePath, $aData, $sPath, $oFile);
    return $this->formatResponse($aReturn);
  }

  /**
   * génère le rapport cadastre parcelle pour les propriétaires locaux
   * @return Response
   */
  public function gererateReportProprietaireLocaux(): Response {
    $aidPar = $this->getIds();
    $aBati = $this->getBatiParcelle($aidPar);
    $aData = $aBati->getMajicData();
    /* Génération du fichier Excel */
    $oFile = $this->oFileManager->getLocalFileAccessor();
    $sDirPath = '@prop(cadastre_report_dir)/' . $aBati[0]->getAnneeReference() . '/Rapport_Parcelle_ProprietaireLocaux/' . Uuid::uuid1()->toString() . '/';
    $sDirPath = $this->oProperties->replacePropertiesInString($sDirPath);
    $sPath = $sDirPath . 'ProprietaireLocaux' . '.xlsx';
    if (!$oFile->file_exists($sDirPath)) {
      $oFile->mkdir($sDirPath, 0777, true);
    }
    $sTemplatePath = $this->oProperties->replacePropertiesInString("@prop(cadastre_template_dir)/json_tpl_propLocaux.json");
    /* Génération de l'excel */
    $aReturn = $this->transformExcel($sTemplatePath, $aData, $sPath, $oFile);
    return $this->formatResponse($aReturn);
  }

  /**
   * génère le rapport cadastre parcelle pour les baties et non-baties
   * @return Response
   */
  public function gererateReportBatieNonBatie(): Response {
    $aidPar = $this->getIds();
    $aBati = $this->getCompteCommunalParcelleEmprise($aidPar);
    $aData = $aBati->getMajicData();
    /* Génération du fichier Excel */
    $oFile = $this->oFileManager->getLocalFileAccessor();
    $sDirPath = '@prop(cadastre_report_dir)/' . $aBati[0]->getAnneeReference() . '/Rapport_Parcelle_BatieNonBatie/' . Uuid::uuid1()->toString() . '/';
    $sDirPath = $this->oProperties->replacePropertiesInString($sDirPath);
    $sPath = $sDirPath . 'BatieNonBatie' . '.xlsx';
    if (!$oFile->file_exists($sDirPath)) {
      $oFile->mkdir($sDirPath, 0777, true);
    }
    $sTemplatePath = $this->oProperties->replacePropertiesInString("@prop(cadastre_template_dir)/json_tpl_batieNonBatie.json");
    /* Génération de l'excel */
    $aReturn = $this->transformExcel($sTemplatePath, $aData, $sPath, $oFile);
    return $this->formatResponse($aReturn);
  }

  /**
   * génère le rapport cadastre parcelle pour les emprises posplu
   * @return Response
   */
  public function gererateReportEmprisePosPlu(): Response {
    $aidPar = $this->getIds();
    $aBati = $this->getCompteCommunalParcelleEmprise($aidPar);
    $aData = $aBati->getMajicData();
    /* Génération du fichier Excel */
    $oFile = $this->oFileManager->getLocalFileAccessor();
    $sDirPath = '@prop(cadastre_report_dir)/' . $aBati[0]->getAnneeReference() . '/Rapport_Emprise_PosPlu/' . Uuid::uuid1()->toString() . '/';
    $sDirPath = $this->oProperties->replacePropertiesInString($sDirPath);
    $sPath = $sDirPath . 'EmprisePosPlu' . '.xlsx';
    if (!$oFile->file_exists($sDirPath)) {
      $oFile->mkdir($sDirPath, 0777, true);
    }
    $sTemplatePath = $this->oProperties->replacePropertiesInString("@prop(cadastre_template_dir)/json_tpl_posPlu.json");
    /* Génération de l'excel */
    $aReturn = $this->transformExcel($sTemplatePath, $aData, $sPath, $oFile);
    return $this->formatResponse($aReturn);
  }

  /**
   * génère le rapport cadastre parcelle pour les subdivisions fiscales
   * @return Response
   */
  public function gererateReportSubdivisionFiscale(): Response {
    $aidPar = $this->getIds();
    $aBati = $this->getCompteCommunalParcelleEmprise($aidPar);
    $aData = $aBati->getMajicData();
    /* Génération du fichier Excel */
    $oFile = $this->oFileManager->getLocalFileAccessor();
    $sDirPath = '@prop(cadastre_report_dir)/' . $aBati[0]->getAnneeReference() . '/Rapport_Parcelle_SubdivisionFiscale/' . Uuid::uuid1()->toString() . '/';
    $sDirPath = $this->oProperties->replacePropertiesInString($sDirPath);
    $sPath = $sDirPath . 'SubdivisionFiscale' . '.xlsx';
    if (!$oFile->file_exists($sDirPath)) {
      $oFile->mkdir($sDirPath, 0777, true);
    }
    $sTemplatePath = $this->oProperties->replacePropertiesInString("@prop(cadastre_template_dir)/json_tpl_subdivisionFiscale.json");
    /* Génération de l'excel */
    $aReturn = $this->transformExcel($sTemplatePath, $aData, $sPath, $oFile);
    return $this->formatResponse($aReturn);
  }

  /**
   * génère le rapport cadastre parcelle pour les propriétaires
   * @return Response
   */
  public function gererateReportProprietaire(): Response {
    $aidPar = $this->getIds();
    $aBati = $this->getCompteCommunalParcelleNoEmprise($aidPar);
    $aData = $aBati->getMajicData();
    /* Génération du fichier Excel */
    $oFile = $this->oFileManager->getLocalFileAccessor();
    $sDirPath = '@prop(cadastre_report_dir)/' . $aBati[0]->getAnneeReference() . '/Rapport_Parcelle_Proprietaire/' . Uuid::uuid1()->toString() . '/';
    $sDirPath = $this->oProperties->replacePropertiesInString($sDirPath);
    $sPath = $sDirPath . 'Proprietaire' . '.xlsx';
    if (!$oFile->file_exists($sDirPath)) {
      $oFile->mkdir($sDirPath, 0777, true);
    }
    $sTemplatePath = $this->oProperties->replacePropertiesInString("@prop(cadastre_template_dir)/json_tpl_proprietaire.json");
    /* Génération de l'excel */
    $aReturn = $this->transformExcel($sTemplatePath, $aData, $sPath, $oFile);
    return $this->formatResponse($aReturn);
  }

  /**
   * génère le rapport cadastre parcelle pour toutes les informations
   * @return Response
   */
  public function gererateReportToutesInfos(): Response {
    $aidPar = $this->getIds();
    $aBati = $this->getCompteCommunalParcelleEmprise($aidPar);
    $aData = $aBati->getMajicData();
    /* Génération du fichier PDF */
    $oFile = $this->oFileManager->getLocalFileAccessor();
    $sDirPath = '@prop(cadastre_report_dir)/' . $aBati[0]->getAnneeReference() . '/Rapport_cadastre_parcelle_Toutes_Infos/' . Uuid::uuid1()->toString() . '/';
    $sDirPath = $this->oProperties->replacePropertiesInString($sDirPath);
    $sPath = $sDirPath . 'ToutesInfos' . '.pdf';
    if (!$oFile->file_exists($sDirPath)) {
      $oFile->mkdir($sDirPath, 0777, true);
    }
    $sTemplatePath = $this->oProperties->replacePropertiesInString("@prop(cadastre_template_dir)/tpl_toutesInfos.html");
    /* Vérification de l'existence du template */
    if (file_exists($sTemplatePath)) {
      $sTemplate = $oFile->file_get_contents($sTemplatePath);
      $this->oPrintService->print($sTemplate, 'A4', 'pdf', 'portrait', $aData, [], [], $sPath, false);
    } else {
      $aReturn = [
        'message' => 'Le template n\'a pas été trouvé.',
        'errorMessage' => 'ERROR_GENERIC_CONTROLLER_GET',
        'status' => 0];
      return $this->formatResponse($aReturn, Response::HTTP_NOT_FOUND);
    }
    /* Retourne le lien de téléchargement du PDF */
    $aReturn = [
      'data' => [
        'rapport' => $oFile->getProxyPassUrl($sPath),
      ],
      'status' => 1,
    ];
    return $this->formatResponse($aReturn);
  }

  /**
   * @param array $aidPar liste des identifiants de parcelle
   * récupère les bâtis pour les parcelles
   * @return Response|Object
   */
  private function getBatiParcelle($aidPar) {
    // chargement de la connexion à la base de donnée
    $oEntityManager = $this->oDatabaseConnection->getAdminEntityManager();
    $oBatiParcelleRepository = $oEntityManager->getRepository(BatiParcelle::class);
    if (!$oBatiParcelleRepository instanceof BatiParcelleRepository) {
      $aReturn = [
        'message' => 'Repository is not a BatiParcelleRepository.',
        'errorMessage' => 'ERROR_GENERIC_CONTROLLER_GET',
        'status' => 0,
      ];
      return $this->formatResponse($aReturn, Response::HTTP_NOT_FOUND);
    }
    $aBatiParcelle = $oBatiParcelleRepository->findByIdParInRange($aidPar);
    if (count($aBatiParcelle) == 0) {
      $aReturn = [
        'message' => 'Erreur lors de la g\e00E9n\e00E9ration du rapport cadastre parcelle (Absence de bati parcelle).',
        'errorMessage' => 'ERROR_CADASTRE_MISSING_DATA',
        'status' => 0];
      return $this->formatResponse($aReturn, Response::HTTP_NOT_FOUND);
    }
    return $aBatiParcelle;
  }

  /**
   * @param array $aidPar liste des identifiants de parcelle
   * récupère les bâtis pour les parcelles
   * @return Response|Object
   */
  private function getCompteCommunalParcelleEmprise($aidPar) {
    // chargement de la connexion à la base de donnée
    $oUserEntityManager = $this->oDatabaseConnection->getUserEntityManager();

    $oEntityManager = $this->oDatabaseConnection->getAdminEntityManager();
    $oCompteCommunalRepository = $oEntityManager->getRepository(CompteCommunalParcelle::class);
    if (!$oCompteCommunalRepository instanceof CompteCommunalRepository) {
      $aReturn = [
        'message' => 'Repository is not a CompteCommunalRepository.',
        'errorMessage' => 'ERROR_GENERIC_CONTROLLER_GET',
        'status' => 0,
      ];
      return $this->formatResponse($aReturn, Response::HTTP_NOT_FOUND);
    }
    $aCcParcelle = $oCompteCommunalRepository->findByIdParInRange($aidPar);
    if (count($aCcParcelle) == 0) {
      $aReturn = [
        'message' => 'Erreur lors de la g\e00E9n\e00E9ration du rapport cadastre parcelle (Absence de bati parcelle).',
        'errorMessage' => 'ERROR_CADASTRE_MISSING_DATA',
        'status' => 0];
      return $this->formatResponse($aReturn, Response::HTTP_NOT_FOUND);
    }
    $aCcParcelle->getEmprise($oUserEntityManager);
    return $aCcParcelle;
  }

  /**
   * @param array $aidPar liste des identifiants de parcelle
   * récupère les bâtis pour les parcelles
   * @return Response|Object
   */
  private function getCompteCommunalParcelleNoEmprise($aidPar) {
    // chargement de la connexion à la base de donnée
    // $oUserEntityManager = $this->oDatabaseConnection->getUserEntityManager();
    $oEntityManager = $this->oDatabaseConnection->getAdminEntityManager();
    $oCompteCommunalRepository = $oEntityManager->getRepository(CompteCommunalParcelle::class);
    if (!$oCompteCommunalRepository instanceof CompteCommunalRepository) {
      $aReturn = [
        'message' => 'Repository is not a CompteCommunalRepository.',
        'errorMessage' => 'ERROR_GENERIC_CONTROLLER_GET',
        'status' => 0,
      ];
      return $this->formatResponse($aReturn, Response::HTTP_NOT_FOUND);
    }
    $aCcParcelle = $oCompteCommunalRepository->findByIdParInRange($aidPar);
    if (count($aCcParcelle) == 0) {
      $aReturn = [
        'message' => 'Erreur lors de la g\e00E9n\e00E9ration du rapport cadastre parcelle (Absence de bati parcelle).',
        'errorMessage' => 'ERROR_CADASTRE_MISSING_DATA',
        'status' => 0];
      return $this->formatResponse($aReturn, Response::HTTP_NOT_FOUND);
    }
    return $aCcParcelle;
  }

  /**
   * @param string $sTemplatePath le chemin du template
   * @param array $aData les données
   * @param string $sPath le chemin du l'excel
   * @param Object $oFile management des fichiers
   * Methode pour générer l'excel et retourner le lien de téléchargement
   * @return array|Response
   */
  private function transformExcel($sTemplatePath, $aData, $sPath, $oFile) {
    if (file_exists($sTemplatePath)) {
      $sTemplate = $oFile->file_get_contents($sTemplatePath);
      $this->oPrintService->printExcel($sTemplate, $aData, $sPath);
    } else {
      $aReturn = [
        'message' => 'Le template n\'a pas \e00E9t\e00E9 trouv\e00E9.',
        'errorMessage' => 'ERROR_CADASTRE_MISSING_DATA',
        'status' => 0];
      return $this->formatResponse($aReturn, Response::HTTP_NOT_FOUND);
    }
    /* Retourne le lien de téléchargement de l'Excel */
    $aReturn = [
      'data' => [
        'rapport' => $oFile->getProxyPassUrl($sPath),
      ],
      'status' => 1,
    ];
    return $aReturn;
  }
}
