<?php

namespace App\Module\Cadastre\WebService\Entity\FicheDescriptive\Invariant;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\OneToMany;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour v_vmap_maj_fiche_inv_descr
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="v_vmap_maj_fiche_inv_descr")
 */
class Description extends AbstractEntity {
  /**
   * @var string
   * @Column(type="string", name="id_local")
   * @Id
   */
  private $id_local;

  /**
   * @var string|null
   * @Column(type="string", name="dvoilib")
   */
  private $dvoilib;

  /**
   * @var string|null
   * @Column(type="string", name="dindic")
   */
  private $dindic;

  /**
   * @var string|null
   * @Column(type="string", name="dnvoiri")
   */
  private $dnvoiri;

  /**
   * @var string|null
   * @Column(type="string", name="l_nature_voie")
   */
  private $l_nature_voie;

  /**
   * @var string|null
   * @Column(type="string", name="ccoriv")
   */
  private $ccoriv;

  /**
   * @var string|null
   * @Column(type="string", name="libcom")
   */
  private $libcom;

  /**
   * @var string
   * @Column(type="string", name="id_com")
   */
  private $id_com;

  /**
   * @var string|null
   * @Column(type="string", name="l_dteloc")
   */
  private $l_dteloc;

  /**
   * @var string|null
   * @Column(type="string", name="l_cconlc")
   */
  private $l_cconlc;

  /**
   * @var string|null
   * @Column(type="string", name="l_dnatlc")
   */
  private $l_dnatlc;

  /**
   * @var string|null
   * @Column(type="string", name="jdatat")
   */
  private $jdatat;

  /**
   * @var string|null
   * @Column(type="string", name="jannat")
   */
  private $jannat;

  /**
   * @var string|null
   * @Column(type="string", name="cchpr")
   */
  private $cchpr;

  /**
   * @var integer|null
   * @Column(type="integer", name="bipeva")
   */
  private $bipeva;

  /**
   * @var string|null
   * @Column(type="string", name="dnbniv")
   */
  private $dnbniv;

  /**
   * @var string|null
   * @Column(type="string", name="dnubat")
   */
  private $dnubat;

  /**
   * @var string|null
   * @Column(type="string", name="desc")
   */
  private $desc;

  /**
   * @var string|null
   * @Column(type="string", name="dniv")
   */
  private $dniv;

  /**
   * @var integer|null
   * @Column(type="integer", name="dvltrt")
   */
  private $dvltrt;

  /**
   * @var string|null
   * @Column(type="string", name="invar")
   */
  private $invar;

  /**
   * @var ArrayCollection<Pev>
   * @OneToMany(targetEntity="App\Module\Cadastre\WebService\Entity\FicheDescriptive\Invariant\Pev", mappedBy="InvariantDescription")
   */
  private $aPev;

  /**
   * @var ArrayCollection<Habitation>
   * @OneToMany(targetEntity="App\Module\Cadastre\WebService\Entity\FicheDescriptive\Invariant\Habitation", mappedBy="InvariantDescription")
   */
  private $aHab;

  /**
   * @var ArrayCollection<LocalPro>
   * @OneToMany(targetEntity="App\Module\Cadastre\WebService\Entity\FicheDescriptive\Invariant\LocalPro", mappedBy="InvariantDescription")
   */
  private $aLocalPro;

  /**
   * @var ArrayCollection<Dependance>
   * @OneToMany(targetEntity="App\Module\Cadastre\WebService\Entity\FicheDescriptive\Invariant\Dependance", mappedBy="InvariantDescription")
   */
  private $aDep;

  /**
   * @var ArrayCollection<Proprietaire>
   * @OneToMany(targetEntity="App\Module\Cadastre\WebService\Entity\FicheDescriptive\Invariant\Proprietaire", mappedBy="InvariantDescription")
   */
  private $aProp;

  /**
   * @var ArrayCollection<Lot>
   * @OneToMany(targetEntity="App\Module\Cadastre\WebService\Entity\FicheDescriptive\Invariant\Lot", mappedBy="InvariantDescription")
   */
  private $aLot;

  public function __construct() {
    $this->aPev = new ArrayCollection();
    $this->aHab = new ArrayCollection();
    $this->aLocalPro = new ArrayCollection();
    $this->aDep = new ArrayCollection();
    $this->aProp = new ArrayCollection();
    $this->aLot = new ArrayCollection();
  }

  /**
   * @return string
   */
  public function getIdLocal(): string {
    return $this->id_local;
  }

  /**
   * @return string|null
   */
  public function getDvoilib(): ?string {
    return $this->dvoilib;
  }

  /**
   * @return string|null
   */
  public function getDnvoiri(): ?string {
    return $this->dnvoiri;
  }

  /**
   * @return string|null
   */
  public function getLNatureVoie(): ?string {
    return $this->l_nature_voie;
  }

  /**
   * @return string|null
   */
  public function getCcoriv(): ?string {
    return $this->ccoriv;
  }

  /**
   * @return string|null
   */
  public function getDindic(): ?string {
    return $this->dindic;
  }

  /**
   * @return string|null
   */
  public function getLibcom(): ?string {
    return $this->libcom;
  }

  /**
   * @return string
   */
  public function getIdCom(): string {
    return $this->id_com;
  }

  /**
   * @return string|null
   */
  public function getLDteloc(): ?string {
    return $this->l_dteloc;
  }

  /**
   * @return string|null
   */
  public function getLCconlc(): ?string {
    return $this->l_cconlc;
  }

  /**
   * @return string|null
   */
  public function getLDnatlc(): ?string {
    return $this->l_dnatlc;
  }

  /**
   * @return string|null
   */
  public function getJdatat(): ?string {
    return $this->jdatat;
  }

  /**
   * @return string|null
   */
  public function getJannat(): ?string {
    return $this->jannat;
  }

  /**
   * @return string|null
   */
  public function getCchpr(): ?string {
    return $this->cchpr;
  }

  /**
   * @return integer|null
   */
  public function getBipeva(): ?int {
    return $this->bipeva;
  }

  /**
   * @return string|null
   */
  public function getDnbniv(): ?string {
    return $this->dnbniv;
  }

  /**
   * @return string|null
   */
  public function getDnubat(): ?string {
    return $this->dnubat;
  }

  /**
   * @return string|null
   */
  public function getDesc(): ?string {
    return $this->desc;
  }

  /**
   * @return string|null
   */
  public function getDniv(): ?string {
    return $this->dniv;
  }

  /**
   * @return integer|null
   */
  public function getDvltrt(): ?int {
    return $this->dvltrt;
  }

  /**
   * @return string|null
   */
  public function getInvar(): ?string {
    return $this->invar;
  }

  /**
   * @return array
   */
  public function getPev(): array{
    $aPev = [];
    foreach ($this->aPev as $pev) {
      $aPev[] = $pev->getInvariantPevData();
    }
    return $aPev;
  }

  /**
   * @return array
   */
  public function getHab(): array{
    $aHab = [];
    foreach ($this->aHab as $hab) {
      $aHab[] = $hab->getInvariantHabData();
    }
    return $aHab;
  }

  /**
   * @return array
   */
  public function getLocalPro(): array{
    $aLocalPro = [];
    foreach ($this->aLocalPro as $localPro) {
      $aLocalPro[] = $localPro->getInvariantLocalProData();
    }
    return $aLocalPro;
  }

  /**
   * @return array
   */
  public function getDep(): array{
    $aDep = [];
    foreach ($this->aDep as $Dep) {
      $aDep[] = $Dep->getInvariantDepData();
    }
    return $aDep;
  }

  /**
   * @return array
   */
  public function getProp(): array{
    $aProp = [];
    foreach ($this->aProp as $Prop) {
      $aProp[] = $Prop->getInvariantPropData();
    }
    return $aProp;
  }

  /**
   * @return array
   */
  public function getLot(): array{
    $aLot = [];
    foreach ($this->aLot as $Lot) {
      $aLot[] = $Lot->getInvariantLotData();
    }
    return $aLot;
  }
}
