<?php

namespace App\Module\Cadastre\WebService\Entity\FicheDescriptive\Parcelle;

use App\Module\Cadastre\WebService\Entity\FicheDescriptive\Parcelle\Bati;
use App\Module\Cadastre\WebService\Entity\FicheDescriptive\Parcelle\Proprietaire;
use App\Module\Cadastre\WebService\Entity\FicheDescriptive\Parcelle\SubdivisionFiscale;
use App\Module\Vitis\WebService\Entity\AbstractEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\OneToMany;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour v_vmap_maj_fiche_prc_descr
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="v_vmap_maj_fiche_prc_descr")
 */
class Description extends AbstractEntity {
  /**
   * @var string
   * @Column(type="string", name="id_par")
   * @Id
   */
  private $id_par;

  /**
   * @var string
   * @Column(type="string", name="id_com")
   */
  private $id_com;

  /**
   * @var string|null
   * @Column(type="string", name="parcelle")
   */
  private $parcelle;

  /**
   * @var string|null
   * @Column(type="string", name="commune")
   */
  private $commune;

  /**
   * @var float|null
   * @Column(type="float", name="sup_fiscale")
   */
  private $sup_fiscale;

  /**
   * @var string|null
   * @Column(type="string", name="dcntpa")
   */
  private $dcntpa;

  /**
   * @var string|null
   * @Column(type="string", name="dnvoiri")
   */
  private $dnvoiri;

  /**
   * @var string|null
   * @Column(type="string", name="dindic")
   */
  private $dindic;

  /**
   * @var string|null
   * @Column(type="string", name="l_nature_voie", nullable=true)
   */
  private $l_nature_voie;

  /**
   * @var string|null
   * @Column(type="string", name="dvoilib")
   */
  private $dvoilib;

  /**
   * @var string|null
   * @Column(type="string", name="ccoriv")
   */
  private $ccoriv;

  /**
   * @var string|null
   * @Column(type="string", name="gparbat")
   */
  private $gparbat;

  /**
   * @var string|null
   * @Column(type="string", name="gurbpa")
   */
  private $gurbpa;

  /**
   * @var ArrayCollection<Proprietaire>
   * @OneToMany(targetEntity="App\Module\Cadastre\WebService\Entity\FicheDescriptive\Parcelle\Proprietaire", mappedBy="ParcelleDescription")
   */
  private $aProp;

  /**
   * @var ArrayCollection<SubdivisionFiscale>
   * @OneToMany(targetEntity="App\Module\Cadastre\WebService\Entity\FicheDescriptive\Parcelle\SubdivisionFiscale", mappedBy="ParcelleDescription")
   */
  private $aSubFisc;

  /**
   * @var ArrayCollection<Bati>
   * @OneToMany(targetEntity="App\Module\Cadastre\WebService\Entity\FicheDescriptive\Parcelle\Bati", mappedBy="ParcelleDescription")
   */
  private $aBat;

  public function __construct() {
    $this->aProp = new ArrayCollection();
    $this->aSubFisc = new ArrayCollection();
    $this->aBat = new ArrayCollection();
  }

  /**
   * @return string
   */
  public function getIdPar(): string {
    return $this->id_par;
  }

  /**
   * @return string
   */
  public function getIdCom(): string {
    return $this->id_com;
  }

  /**
   * @return string|null
   */
  public function getParcelle(): ?string {
    return $this->parcelle;
  }

  /**
   * @return string|null
   */
  public function getCommune(): ?string {
    return $this->commune;
  }

  /**
   * @return float|null
   */
  public function getSupFiscale(): ?float {
    return $this->sup_fiscale;
  }

  /**
   * @return string|null
   */
  public function getDcntpa(): ?string {
    return $this->dcntpa;
  }

  /**
   * @return string|null
   */
  public function getDnvoiri(): ?string {
    return $this->dnvoiri;
  }

  /**
   * @return string|null
   */
  public function getDindic(): ?string {
    return $this->dindic;
  }

  /**
   * @return string|null
   */
  public function getLNatureVoie(): ?string {
    return $this->l_nature_voie;
  }

  /**
   * @return string|null
   */
  public function getDvoilib(): ?string {
    return $this->dvoilib;
  }

  /**
   * @return string|null
   */
  public function getCcoriv(): ?string {
    return $this->ccoriv;
  }

  /**
   * @return string|null
   */
  public function getGparbat(): ?string {
    return $this->gparbat;
  }

  /**
   * @return string|null
   */
  public function getGurbpa(): ?string {
    return $this->gurbpa;
  }

  /**
   * @return array
   */
  public function getProp(): array{
    $aProp = [];
    foreach ($this->aProp as $prop) {
      $aProp[] = $prop->getParcellePropData();
    }
    return $aProp;
  }

  /**
   * @return array
   */
  public function getSuf(): array{
    $aSubFisc = [];
    foreach ($this->aSubFisc as $suf) {
      $aSubFisc[] = $suf->getParcelleSubFiscData();
    }
    return $aSubFisc;
  }

  /**
   * @return array
   */
  public function getBat(): array{
    $aBat = [];
    foreach ($this->aBat as $bat) {
      $aBat[] = $bat->getParcelleBatiData();
    }
    return $aBat;
  }
}
