<?php

namespace App\Module\Cadastre\WebService\Entity\FicheUrbanisme;

use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour cadastre_views.fiche_urba
 * Cette classe permet de sélectionner les attributs à la fiche d'urbanisme.
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\FicheUrbanismeRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="@prop(cadastre_views:fiche_urba)")
 */
class FicheUrbanisme {
  /**
   * @var string non utilisé juste présent pour la récupérations de toutes les entités
   * @Column(type="string", name="id")
   * @Id
   */
  private $id;

  /**
   * @var string
   * @Column(type="string", name="id_par")
   */
  private $id_par;

  /**
   * @var string|null
   * @Column(type="string", name="libdep")
   */
  private $libdep;

  /**
   * @var string|null
   * @Column(type="string", name="libcom")
   */
  private $libcom;

  /**
   * @var string|null
   * @Column(type="string", name="dnupla")
   */
  private $dnupla;

  /**
   * @var string|null
   * @Column(type="string", name="ccosec")
   */
  private $ccosec;

  /**
   * @var string|null
   * @Column(type="string", name="dcntpa")
   */
  private $dcntpa;

  /**
   * @var string|null
   * @Column(type="string", name="ddenom")
   */
  private $ddenom;

  /**
   * @var string|null
   * @Column(type="string", name="dlign3")
   */
  private $dlign3;

  /**
   * @var string|null
   * @Column(type="string", name="dlign4")
   */
  private $dlign4;

  /**
   * @var string|null
   * @Column(type="string", name="dlign6")
   */
  private $dlign6;

  /**
   * @var string|null
   * @Column(type="string", name="l_dsgrpf")
   */
  private $l_dsgrpf;

  /**
   * @var string|null
   * @Column(type="string", name="dnvoiri")
   */
  private $dnvoiri;

  /**
   * @var string|null
   * @Column(type="string", name="dindic")
   */
  private $dindic;

  /**
   * @var string|null
   * @Column(type="string", name="cconvo")
   */
  private $cconvo;

  /**
   * @var string|null
   * @Column(type="string", name="dvoilib")
   */
  private $dvoilib;

  /**
   * récupération de l'adresse en concaténant dnvoiri, dindic, cconvo, dvoilib
   * @return string|null
   * Méthode modifiée : ajout d'un trim fait par le SQL dans l'ancien framework
   */
  public function getAdresse(): ?string {
    return trim($this->dnvoiri . $this->dindic . ' ' . $this->cconvo . ' ' . $this->dvoilib);
  }

  /**
   * @return string|null
   */
  public function getId(): ?string {
    return $this->id;
  }

  /**
   * @return array
   * Cette méthode permet de construire un tableau correspondant à la Fiche d'urbanisme
   */
  public function getMajicData(): array{
    return [
      'ID_PAR' => $this->id_par,
      'LIBDEP' => $this->libdep,
      'LIBCOM' => $this->libcom,
      'DNUPLA' => $this->dnupla,
      'CCOSEC' => $this->ccosec,
      'DCNTPA' => $this->dcntpa,
      'DDENOM' => $this->ddenom,
      'DLIGN3' => $this->dlign3,
      'DLIGN4' => $this->dlign4,
      'DLIGN6' => $this->dlign6,
      'DSGRPF' => $this->l_dsgrpf,
      'ADRESSE' => $this->getAdresse(),
    ];
  }
}
