<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Collection\ExonSufCollection;
use App\Module\Cadastre\WebService\Entity\Majic\ExonSuf;
use App\Module\Cadastre\WebService\Repository\ExonSufRepository;
use App\Module\Cadastre\WebService\Service\MajicUtils;
use App\Module\Vitis\WebService\Entity\AbstractEntity;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\MappedSuperclass;
use Doctrine\ORM\Mapping\Table;
use Doctrine\Persistence\Mapping\ClassMetadata;
use Doctrine\Persistence\ObjectManager;

/**
 *  Objet abstrait pour les surfaces des subdivisions fiscales : Suf et SufBnd
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @MappedSuperclass
 */
abstract class AbstractSuf extends AbstractEntity implements MajicInterface {
  /**
   * @var EntityManager|null
   */
  protected $oEntityManager;
  /**
   * @var LoggerManager|null
   */
  protected $oLoggerManager;

  /**
   * @var string
   * @Column(type="string", name="id_dnupro")
   */
  protected $id_dnupro;

  /**
   * @var string
   * @Column(type="string", name="id_suf")
   * @Id
   */
  protected $id_suf;

  /**
   * @var string
   * @Column(type="string", name="dnupro")
   */
  protected $dnupro;

  /**
   * @var string
   * @Column(type="string", name="ccostn")
   */
  protected $ccostn;

  /**
   * @var string
   * @Column(type="string", name="ccosub")
   */
  protected $ccosub;

  /**
   * @var string
   * @Column(type="string", name="gnexps")
   */
  protected $gnexps;

  /**
   * @var string
   * @Column(type="string", name="dsgrpf")
   */
  protected $dsgrpf;

  /**
   * @var string
   * @Column(type="string", name="l_dsgrpf")
   */
  protected $l_dsgrpf;

  /**
   * @var string
   * @Column(type="string", name="dclssf")
   */
  protected $dclssf;

  /**
   * @var string
   * @Column(type="string", name="cnatsp")
   */
  protected $cnatsp;

  /**
   * @var string
   * @Column(type="string", name="dcntsf")
   */
  protected $dcntsf;

  /**
   * @var int
   * @Column(type="integer", name="drcsuba")
   */
  protected $drcsuba;

  /**
   * @var int
   * @Column(type="integer", name="drcsub")
   */
  protected $drcsub;

  /**
   * @var string
   * @Column(type="string", name="cgrnum")
   */
  protected $cgrnum;

  /**
  * @var string
  * @Column(type="string", name="l_cgrnum")
  */
 protected $l_cgrnum;

  /**
   * @var ExonSufCollection
   */
  protected $aExonSuf;

  public function __construct() {
    $this->aExonSuf = new ExonSufCollection();
  }

  /**
   * @return string|null
   */
  public function getCcostn(): ?string {
    return $this->ccostn;
  }
  /**
   * @return string|null
   */
  public function getCcosub(): ?string {
    return $this->ccosub;
  }
  /**
   * @return string|null
   */
  public function getDsgrpf(): ?string {
    return $this->dsgrpf;
  }
  /**
   * @return string|null
   */
  public function getDclssf(): ?string {
    return $this->dclssf;
  }
  /**
   * @return string|null
   */
  public function getCnatsp(): ?string {
    return $this->cnatsp;
  }

  /**
   * remplis la liste des exonération des surfaces des subdivisions fiscales si necessaire
   * @return ExonSufCollection|array
   */
  private function fillExonSufArray() {
    $aExonSuf = [];
    if ($this->aExonSuf instanceof ExonSufCollection && !$this->aExonSuf->isEmpty()) { 
      return $this->aExonSuf;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $exonSufRepository = $this->oEntityManager->getRepository(ExonSuf::class);
      if (!$exonSufRepository instanceof ExonSufRepository) {
        throw new \Exception("Repository is not a ExonSufRepository", 1);
      }
      $aExonSuf = $exonSufRepository->findByIdSuf($this->id_suf);
      $this->aExonSuf = new ExonSufCollection();
      foreach ($aExonSuf as $exonSuf) {
        $this->aExonSuf->add($exonSuf);
      }
      return $this->aExonSuf;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get ExonSuf list for Suf ', [$this, $e]);
      return [];
    }
  }

  /**
   * Cette méthode permet de récupérer les exonération des surfaces des subdivisions fiscales
   * @return ExonSufCollection|null
   */
  public function getExonSufCollection(): ?ExonSufCollection {
    return $this->fillExonSufArray();
  }

  /**
   * Cette méthode permet de récupérer les caractères correspondants à la colonne HA depuis l'attribut dcntsf
   * @return int|null null équivaut à la valeur 0
   */
  public function getContenanceHA(): ?int{
    $sContenanceHA = substr($this->dcntsf, 0, 5);
    $iContenanceHA = intval($sContenanceHA);
    return ($iContenanceHA === 0) ? null : $iContenanceHA;
  }

  /**
   * @return string|null
   * Cette méthode permet de récupérer les caractères correspondants à la colonne A depuis l'attribut dcntsf
   */
  public function getContenanceA(): ?string{
    $sContenanceA = substr($this->dcntsf, 5, 2);
    $sContenanceHA = $this->getContenanceHA();
    if ($sContenanceHA == null && intval($sContenanceA) === 0) {
      return null;
    }
    if ($sContenanceHA == null && strlen($sContenanceA) == 2 && intval(substr($sContenanceA, 0, 1)) === 0) {
      return ' ' . substr($sContenanceA, 1, 1);
    }
    if ($sContenanceHA != null && strlen($sContenanceA) == 1) {
      return '0' . $sContenanceA;
    }
    return $sContenanceA;
  }

  /**
   * @return string
   * Cette méthode permet de récupérer les caractères correspondants à la colonne CA depuis l'attribut dcntsf
   */
  public function getContenanceCA(): string {
    $sContenanceCA = substr($this->dcntsf, 7, 2);
    // intval de null vaut 0
    $iContenanceA = intval($this->getContenanceA());
    if ($iContenanceA != 0 && strlen($sContenanceCA) == 1) {
      return '0' . $sContenanceCA;
    }
    return $sContenanceCA;
  }

  /**
   * Cette méthode permet de diviser l'attribut drcsuba par 100
   * @return float
   */
  public function getRevenuCadastral(): float {
    return floatval($this->drcsuba / 100);
  }

  /**
   * Cette méthode permet de diviser l'attribut drcsub par 100
   * @return float
   */
  public function getRevenuReference(): float {
    return floatval($this->drcsub / 100);
  }

  /**
   * @param array $aReg tableau des regex pour l'attribut gnexps
   * @param array $aCollectivite tableau des exonérations de la collectivité
   * @return float|null
   * Cette méthode abstraite pour récupérer les valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf
   * Méthode corrigée
   */
  private function getAbstractRevenuCadastral($aReg, $aCollectivite): ?float {
    foreach ($aReg as $sReg) {
      if (strpos($this->gnexps, $sReg) > -1) {
        return $this->getRevenuCadastral();
      }
    }
    return $this->getExonSufCollection()->getRevenuCadastralExo($aCollectivite);
  }

  /**
   * @return float|null
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf
   * Lorsque l'exoneration est procuré par une commune, un groupement de commune ou toutes collectivité
   * Méthode corrigée
   */
  public function getRevenuCadastralAvecExo(): ?float{
    $aReg = ['EP', 'CD', 'CR', 'NI', 'RT'];
    $aCollectivite = ['C', 'TC', 'GC'];
    return $this->getAbstractRevenuCadastral($aReg, $aCollectivite);
  }

  /**
   * @return float|null
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf
   * Méthode corrigée
   */
  public function getRevenuCadastralTaxeADAvecExo(): ?float{
    $aReg = ['EP', 'CD', 'CR', 'NI', 'RT'];
    $aCollectivite = ['A'];
    return $this->getAbstractRevenuCadastral($aReg, $aCollectivite);
  }

  /**
   * @return float|null
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf
   * Lorsque l'exoneration est procuré par le département ou toutes collectivité
   * Méthode corrigée
   */
  public function getRevenuCadastralDepAvecExo(): ?float{
    $aReg = ['EP', 'CD', 'DR', 'NI', 'RT'];
    $aCollectivite = ['D', 'TC'];
    return $this->getAbstractRevenuCadastral($aReg, $aCollectivite);
  }

  /**
   * @return float|null
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf
   * Lorsque l'exoneration est procuré par la région ou toutes collectivité
   * Méthode corrigée
   */
  public function getRevenuCadastralRegAvecExo(): ?float{
    $aReg = ['EP', 'DR', 'CR', 'NI', 'RT'];
    $aCollectivite = ['R', 'TC'];
    return $this->getAbstractRevenuCadastral($aReg, $aCollectivite);
  }

  /**
   * Cette méthode permet de récupérer la valeur de l'attribut 'nat_exo' du tableau aExonSuf
   * Provient de ExonSufCollection
   * Méthode corrigée
   * @return string|null
   */
  public function getNatExo(): ?string {
    return $this->getExonSufCollection()->getNatExo();
  }

  /**
   * Cette méthode permet de récupérer la contenance totale d'une suf
   * @return integer
   */
  public function getContenance(): int {
    $iContenanceHA = intval($this->getContenanceHA());
    $iContenanceA = intval($this->getContenanceA());
    $iContenanceCA = intval($this->getContenanceCA());
    return ($iContenanceHA * 10000) + ($iContenanceA * 100) + ($iContenanceCA);
  }

  /**
   * {@inheritDoc}
   * Cette méthode permet de construire un tableau correspondant aux suf dans le relevé de propriété
   * Il contient ExonSuf
   */
  public function getMajicData(): array{
    return [
      'dnupro' => $this->dnupro,
      'id_dnupro' => $this->id_dnupro,
      's_tar' => $this->ccostn,
      'suf' => $this->ccosub,
      'gnexps' => $this->gnexps,
      'grss_gr' => $this->dsgrpf,
      'occupation' => $this->l_dsgrpf,
      'nature' => $this->l_dsgrpf,
      'groupe' => $this->l_cgrnum,
      'clas' => $this->dclssf,
      'nat_cult' => $this->cnatsp,
      'ha' => $this->getContenanceHA(),
      'a' => $this->getContenanceA(),
      'ca' => $this->getContenanceCA(),
      'revenu_cadastral' => $this->getRevenuCadastral(),
      'revenu_cadastral_txt' => MajicUtils::setFloatToText($this->getRevenuCadastral()),
      'revenu_cadastral_exo' => $this->getExonSufCollection()->getRevenuCadastralExo(),
      'revenu_reference' => $this->getRevenuReference(),
      'revenu_reference_txt' => MajicUtils::setFloatToText($this->getRevenuReference()),
      'nat_exo' => $this->getNatExo(),
      'ccolloc' => $this->getExonSufCollection()->getCcolloc(),
      'contenance_suf' => $this->getContenance(),
      'aExonSuf' => $this->getExonSufCollection()->getMajicData(),
    ];
  }

  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet la récupération d'un gestionnaire d'entité pour interagir avec la base
   * @return void
   */
  public function injectObjectManager(ObjectManager $objectManager, ClassMetadata $classMetadata): void {
    if ($objectManager instanceof EntityManager) {
      $this->oEntityManager = $objectManager;
    }
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void {
    $this->oLoggerManager = $oService;
  }
}
