<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Collection\LotLocalCollection;
use App\Module\Cadastre\WebService\Collection\PevCollection;
use App\Module\Cadastre\WebService\Collection\ProprietaireCollection;
use App\Module\Cadastre\WebService\Entity\Majic\LotLocal;
use App\Module\Cadastre\WebService\Entity\Majic\Pev;
use App\Module\Cadastre\WebService\Entity\Majic\Proprietaire;
use App\Module\Cadastre\WebService\Repository\BatiRepository;
use App\Module\Cadastre\WebService\Repository\LotLocalRepository;
use App\Module\Cadastre\WebService\Repository\PevRepository;
use App\Module\Cadastre\WebService\Repository\ProprietaireRepository;
use App\Module\Cadastre\WebService\Service\MajicUtils;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;
use Doctrine\Persistence\Mapping\ClassMetadata;
use Doctrine\Persistence\ObjectManager;

/**
 *  Objet ORM pour cadastre_rp.rp_bati
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\BatiRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="@prop(cadastre_rp:rp_bati)")
 */
class Bati implements MajicInterface {
  /**
   * @var array
   */
  private const NAT_LOC_MAP = [
    'MA' => 'MAIS', // Cas validé
    'AP' => 'APP', // Cas validé
    'DC' => 'DLC', // cas non validé
    'LC' => 'L.COM', // Cas validé
    'CM' => 'CM', // Cas validé
    'CA' => 'CA', // Cas validé
    'CD' => 'DEP.C', // Cas validé
    'CB' => 'L.DIV', // Cas validé
    'ME' => 'M.EXC', // Cas validé
    'MP' => 'M.PAR', // Cas validé
    'SM' => 'SOLM', // Cas non validé
    'AU' => 'AU', // Cas validé
    'CH' => 'CHANT', // Cas validé
    'U ' => 'E.IND', // Cas validé
    'US' => 'US', // Cas validé
    'UG' => 'AGAZ', // Cas non validé
    'UE' => 'UE', // Cas validé
    'U1' => 'GARE', // Cas non validé
    'U2' => 'GTRI', // Cas non validé
    'U3' => 'GAMAT', // Cas non validé
    'U4' => 'GAMAG', // Cas non validé
    'U5' => 'GDETI', // Cas non validé
    'U6' => 'GDERE', // Cas non validé
    'U7' => 'GMATR', // Cas non validé
    'U8' => 'GEMR', // Cas non validé
    'U9' => 'GSTUS', // Cas non validé
    'AT' => 'AT', // Cas validé
    //'DE' => $this->getNatureDep(), DE sera géré spécifiquement
  ];

  /**
   * @var array
   */
  private const NATURE_DEP_MAP = [
    'GA' => 'GAR', // Cas validé
    'CV' => 'CAVE', // Cas validé
    'GR' => 'GREN', // Cas validé
    'TR' => 'TERR', // Cas non validé
    'GP' => 'GPARK', // Cas validé
    'GC' => 'GRCA', // Cas non validé
    'BX' => 'BOX', // Cas validé
    'PK' => 'PARK', // Cas validé
    'CL' => 'CELL', // Cas non validé
    'BD' => 'BUAN', // Cas non validé
    'BC' => 'BUCH', // Cas non validé
    'RS' => 'REMI', // Cas non validé
    'TT' => 'TT', // Cas validé
    'PI' => 'P.IND', // Cas validé
    'PA' => 'AGREM', // Cas validé
    'CD' => 'CHDO', // Cas non validé
    'DC' => 'DEP.C', // Cas validé
    'JH' => 'J.HIV', // Cas validé
    'PS' => 'PISC', // Cas validé
    'SR' => 'SERR', // Cas non validé
  ];

  /**
   * @var array
   */
  private const OCCUPATION_MAP = [
    'P' => 'Proprietaire ou  Usufruitier',
    'L' => 'Location autre que propriétaire ou usufruitier',
    'V' => 'Vacant',
    'T' => 'Location, non affectée à l\'habitation, soumise à TVA',
    'D' => 'Habitation principale occupée par le propriétaire',
  ];

  /**
   * @var EntityManager|null
   */
  private $oEntityManager;

  /**
   * @var LoggerManager|null
   */
  private $oLoggerManager;

  /**
   * @var string
   * @Column(type="string", name="id_local")
   * @Id
   */
  private $id_local;

  /**
   * @var string
   * @Column(type="string", name="id_par")
   */
  private $id_par;

  /**
   * @var string|null
   * @Column(type="string", name="ccosec")
   */
  private $ccosec;

  /**
   * @var integer|null
   * @Column(type="integer", name="dnupla")
   */
  private $dnupla;

  /**
   * @var string|null
   * @Column(type="string", name="ccoplc")
   */
  private $ccoplc;

  /**
   * @var string|null
   * @Column(type="string", name="dvoilib")
   */
  private $dvoilib;

  /**
   * @var string|null
   * @Column(type="string", name="ccoriv")
   */
  private $ccoriv;

  /**
   * @var string|null
   * @Column(type="string", name="dnubat")
   */
  private $dnubat;

  /**
   * @var string|null
   * @Column(type="string", name="nesc")
   */
  private $nesc;

  /**
   * @var string|null
   * @Column(type="string", name="dniv")
   */
  private $dniv;

  /**
   * @var string|null
   * @Column(type="string", name="dpor")
   */
  private $dpor;

  /**
   * @var string|null
   * @Column(type="string", name="ccoeva")
   */
  private $ccoeva;

  /**
   * @var string|null
   * @Column(type="string", name="gtauom")
   */
  private $gtauom;

  /**
   * @var integer|null
   * @Column(type="integer", name="dvlpera")
   */
  private $dvlpera;

  /**
   * @var string|null
   * @Column(type="string", name="b_dnupro")
   */
  private $b_dnupro;

  /**
   * @var string
   * @Column(type="string", name="b_iddnupro")
   */
  private $b_iddnupro;

  /**
   * @var string|null
   * @Column(type="string", name="dteloc")
   */
  private $dteloc;

  /**
   * @var integer|null
   * @Column(type="integer", name="dvltrt")
   */
  private $dvltrt;

  /**
   * @var integer|null
   * @Column(type="integer", name="dnumql")
   */
  private $dnumql;

  /**
   * @var integer|null
   * @Column(type="integer", name="ddenql")
   */
  private $ddenql;

  /**
   * @var string|null
   * @Column(type="string", name="invar")
   */
  private $invar;

  /**
   * @var string|null
   * @Column(type="string", name="l_dteloc")
   */
  private $l_dteloc;

  /**
   * @var string|null
   * @Column(type="string", name="cconlc")
   */
  private $cconlc;

  /**
   * @var string|null
   * @Column(type="string", name="l_cconlc")
   */
  private $l_cconlc;

  /**
   * @var string|null
   * @Column(type="string", name="dnatlc")
   */
  private $dnatlc;

  /**
   * @var string|null
   * @Column(type="string", name="jannat")
   */
  private $jannat;

  /**
   * @var string|null
   * @Column(type="string", name="jdatat")
   */
  private $jdatat;

  /**
   * @var string|null
   * @Column(type="string", name="dnbniv")
   */
  private $dnbniv;

  /**
   * @var string|null
   * @Column(type="string", name="dnvoiri")
   */
  private $dnvoiri;

  /**
   * @var string|null
   * @Column(type="string", name="dindic")
   */
  private $dindic;

  /**
   * @var string|null
   * @Column(type="string", name="libcom")
   */
  private $libcom;

  /**
   * @var string|null
   * @Column(type="string", name="cconad")
   */
  private $cconad;

  /**
   * @var PevCollection
   */
  private $aPev;

  /**
   * @var LotLocalCollection
   */
  private $aLotLocal;

  /**
   * @var ProprietaireCollection
   */
  private $aProp;

  public function __construct() {
    $this->aPev = new PevCollection();
    $this->aLotLocal = new LotLocalCollection();
    $this->aProp = new ProprietaireCollection();
  }

  /**
   * remplis la liste des lots locaux si necessaire
   * @return LotLocalCollection|array
   */
  private function fillLotLocalArray() {
    $aLot = [];
    // passage en mode singleton
    if ($this->aLotLocal instanceof LotLocalCollection && !$this->aLotLocal->isEmpty()) { 
      return $this->aLotLocal;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $lotLocalRepository = $this->oEntityManager->getRepository(LotLocal::class);
      if (!$lotLocalRepository instanceof LotLocalRepository) {
        throw new \Exception("Repository is not a LotLocalRepository", 1);
      }
      $aLot = $lotLocalRepository->findByIdLocal($this->id_local);
      $this->aLotLocal = new LotLocalCollection();
      foreach ($aLot as $lot) {
        $this->aLotLocal->add($lot);
      }
      return $this->aLotLocal;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get pev list for bati ', [$this, $e]);
      return [];
    }
  }

  /**
   * Cette méthode permet de récupérer les LotLocaux
   * @return LotLocalCollection|null
   */
  public function getLotLocalCollection(): ?LotLocalCollection {
    return $this->fillLotLocalArray();
  }

  /**
   * remplis la liste des proprietaires si necessaire
   * @return ProprietaireCollection|array
   */
  private function fillProprietaireArray() {
    $aProp = [];
    if ($this->aProp instanceof ProprietaireCollection && !$this->aProp->isEmpty()) { 
      return $this->aProp;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $proprietaireRepository = $this->oEntityManager->getRepository(Proprietaire::class);
      if (!$proprietaireRepository instanceof ProprietaireRepository) {
        throw new \Exception("Repository is not a ProprietaireRepository", 1);
      }
      $aProp = $proprietaireRepository->findByIdDnupro($this->b_iddnupro);
      $this->aProp = new ProprietaireCollection();
      foreach ($aProp as $prop) {
        $this->aProp->add($prop);
      }
      return $this->aProp;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get proprietaire list for bati ', [$this, $e]);
      return [];
    }
  }

  /**
   * Cette méthode permet de récupérer les Proprietaires
   * @return ProprietaireCollection|null
   */
  public function getProprietaireCollection(): ?ProprietaireCollection {
    return $this->fillProprietaireArray();
  }

  /**
   * remplis la liste de la partie d'évaluation si necessaire
   * @return PevCollection|array
   */
  private function fillPevArray() {
    $aPev = [];
    // passage en mode singleton
    if ($this->aPev instanceof PevCollection && !$this->aPev->isEmpty()) { 
      return $this->aPev;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $pevRepository = $this->oEntityManager->getRepository(Pev::class);

      if (!$pevRepository instanceof PevRepository) {
        throw new \Exception("Repository is not a PevRepository", 1);
      }
      $aPev = $pevRepository->findByIdLocal($this->id_local);
      $this->aPev = new PevCollection();
      foreach ($aPev as $pev) {
        $this->aPev->add($pev);
      }
      return $this->aPev;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get pev list for bati ', [$this, $e]);
      return [];
    }
  }

  /**
   * Cette méthode permet de récupérer les PevCollection
   * @return PevCollection|null
   */
  public function getPevCollection(): ?PevCollection {
    return $this->fillPevArray();
  }

  /**
   * Cette méthode permet de récupérer les deux derniers caractères de l'attribut Jdatat et d'y ajouter 1.
   * @return string|null
   */
  public function getJdatat(): ?string{
    $sNewJdatat = $this->jdatat;
    if (preg_match('`^\d{1,2}/\d{1,2}/\d{4}$`', $sNewJdatat)) {
      return substr(strval(intval(substr($sNewJdatat, 6, 4)) + 1), 2, 2);
    } else if (preg_match('`^\d{1,2}/\d{1,2}/\d{2}$`', $sNewJdatat)) {
      $sNewJdatat = strval(intval(substr($sNewJdatat, 6, 2)) + 1);
      if (strlen($sNewJdatat) == 1) {
        $sNewJdatat = '0' . $sNewJdatat;
      }
      return substr($sNewJdatat, -2, 2);
    }
    return substr(strval(intval(substr($sNewJdatat, 4, 4)) + 1), 2, 2);
  }

  /**
   * @return string|null
   * Cette méthode permet de concaténer les attributs dnvoiri et indic.
   */
  public function getVoirie(): ?string {
    return ltrim($this->dnvoiri, '0') . $this->dindic;
  }

  /**
   * @return string|null
   * @deprecated
   * Cette méthode permet de supprimer le code commune de l'attribut invar.
   */
  public function getNumeroInvar(): ?string {
    return substr($this->invar, 3, 7);
  }

  /**
   * @return string|null
   */
  public function getIdPar(): ?string {
    return $this->id_par;
  }

  /**
   * @return int|null
   */
  public function getDvlpera(): ?int {
    return $this->dvlpera;
  }

  /**
   * @return string|null
   * Cette méthode permet de supprimer le code commune de l'attribut invar.
   */
  public function getNatLoc(): ?string {
    // si la une correspondance existe dans la constante on la retourne
    if (array_key_exists($this->cconlc, self::NAT_LOC_MAP)) {
      return self::NAT_LOC_MAP[$this->cconlc];
    }
    // cas particulier de DE
    if ($this->cconlc === 'DE') {
      return $this->getNatureDep(); // Cf fonction getNatureDep()
    }
    // retour par défaut
    return substr($this->cconlc, 0, 4);
  }

  /**
   * Cette méthode permet de retourner la nature de dépendance.
   * @return string|null
   */
  public function getNatureDep(): ?string {
    // si la une correspondance existe dans la constante on la retourne
    if (array_key_exists($this->cconad, self::NATURE_DEP_MAP)) {
      return self::NATURE_DEP_MAP[$this->cconad];
    }

    return substr($this->cconad, 0, 4);
  }

  /**
   * @return string|null
   * Cette méthode permet de retourner l'occupation du logement
   */
  public function getOccupation(): ?string {
    // cherche une correspondance dans la table de correspondance
    if (array_key_exists($this->dnatlc, self::OCCUPATION_MAP)) {
      return self::OCCUPATION_MAP[$this->dnatlc];
    }
    // retour par défaut
    return substr($this->dnatlc, 0, 1);

  }

  /**
   * Cette méthode permet de récupérer les deux derniers caractères de l'attribut Jdatat et d'y ajouter 1.
   * @return string|null
   */
  public function getJdatat_full(): ?string {
    $sDate = trim($this->jdatat);
    if (!empty($sDate)) {
      if (preg_match('/^\d{1,2}\/\d{1,2}\/\d{2,4}$/', $sDate)) {
        return $sDate;
      }
      return substr($sDate, 0, 2) . '/' . substr($sDate, 2, 2) . '/' . substr($sDate, 4, 4);
    }
    return null;
  }

/**
 * {@inheritDoc}
 * Cette méthode permet de construire un tableau correspondant au Bati dans le relevé de propriété
 * Il contient PEV, Lot local et Propriétaire
 */
  public function getMajicData(): array{
    return [
      'date' => $this->getJdatat(),
      'section' => $this->ccosec,
      'num_plan' => $this->dnupla,
      'cp' => $this->ccoplc,
      'voirie' => $this->getVoirie(),
      'adresse' => $this->dvoilib,
      'code_rivoli' => $this->ccoriv,
      'bat' => $this->dnubat,
      'ent' => $this->nesc,
      'niv' => $this->dniv,
      'num_de_porte' => $this->dpor,
      'm_eva' => $this->ccoeva,
      'nat_loc' => $this->getNatLoc(),
      'tx_om' => $this->gtauom,
      's_tar' => $this->getPevCollection()->getSTar(),
      'af' => $this->getPevCollection()->getAf(),
      'cat' => $this->getPevCollection()->getCat(),
      'revenu_cadastral' => $this->getPevCollection()->getRevenuCadastral(),
      'revenu_cadastral_txt' => MajicUtils::setFloatToText($this->getPevCollection()->getRevenuCadastral()),
      'dvlpera' => $this->getPevCollection()->getDvlpera(),
      'b_dnupro' => $this->b_dnupro,
      'b_iddnupro' => $this->b_iddnupro,
      'dteloc' => $this->dteloc,
      'dvltrt' => $this->dvltrt,
      'dnumql' => $this->dnumql,
      'ddenql' => $this->ddenql,
      'num_invar' => $this->invar,
      'local_type' => $this->l_dteloc,
      'nature' => $this->cconlc,
      'lib_nature' => $this->l_cconlc,
      'occupation' => $this->dnatlc,
      'lib_occupation' => $this->getOccupation(),
      'date_mut' => $this->getJdatat_full(),
      'annee_constr' => $this->jannat,
      'dnbniv' => $this->dnbniv,
      'libcom' => $this->libcom,
      'klot' => null, //klot non présent avant et risque d'erreur
      'aPev' => $this->getPevCollection()->getMajicData(),
      'aLotLocal' => $this->getLotLocalCollection()->getMajicData(),
      'aProp' => $this->getProprietaireCollection()->getMajicData(),
    ];
  }

  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet la récupération d'un gestionnaire d'entité pour interagir avec la base
   * @return void
   */
  public function injectObjectManager(ObjectManager $objectManager, ClassMetadata $classMetadata): void {
    if ($objectManager instanceof EntityManager) {
      $this->oEntityManager = $objectManager;
    }
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void {
    $this->oLoggerManager = $oService;
  }
}
