<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Collection\BatiCollection;
use App\Module\Cadastre\WebService\Entity\Majic\Bati;
use App\Module\Cadastre\WebService\Repository\MajicCommuneRepository;
use App\Module\Cadastre\WebService\Repository\BatiRepository;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;
use Doctrine\Persistence\Mapping\ClassMetadata;
use Doctrine\Persistence\ObjectManager;

/**
 *  Objet ORM pour cadastre_views.bati_parcelle
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\BatiParcelleRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="@prop(cadastre_views:bati_parcelle)")
 */
class BatiParcelle implements MajicInterface {
  /**
   * @var EntityManager|null
   */
  private $oEntityManager;

  /**
   * @var LoggerManager|null
   */
  private $oLoggerManager;

  /**
   * @var string
   * @Column(type="string", name="id")
   */
  private $id;

  /**
   * @var string
   * @Column(type="string", name="id_par")
   * @Id
   */
  private $id_par;

  /**
   * @var string|null
   * @Column(type="string", name="ccodep")
   */
  private $ccodep;

  /**
   * @var string|null
   * @Column(type="string", name="ccocom")
   */
  private $ccocom;

  /**
   * @var string|null
   * @Column(type="string", name="ccodir")
   */
  private $ccodir;

  /**
   * @var string|null
   * @Column(type="string", name="libcom")
   */
  private $libcom;

  /**
   * @var string|null
   * @Column(type="string", name="dnvoiri")
   */
  private $dnvoiri;

  /**
   * @var string|null
   * @Column(type="string", name="dindic")
   */
  private $dindic;

  /**
   * @var string|null
   * @Column(type="string", name="dvoilib")
   */
  private $dvoilib;

  /**
   * @var string|null
   * @Column(type="string", name="cconvo")
   */
  private $cconvo;

  /**
   * @var string|null
   * @Column(type="string", name="dcntpa")
   */
  private $dcntpa;

  /**
   * @var BatiCollection
   */
  protected $aBati;

  public function __construct() {
    $this->aBati = new BatiCollection();
  }

  /**
   * remplis la liste des batis si necessaire
   * @return BatiCollection|array
   */
  protected function fillBatiArray() {
    // passage en mode singleton
    if ($this->aBati instanceof BatiCollection && !$this->aBati->isEmpty()) { 
      return $this->aBati;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $batiRepository = $this->oEntityManager->getRepository(Bati::class);
      if (!$batiRepository instanceof BatiRepository) {
        throw new \Exception("Repository is not a BatiRepository", 1);
      }
      $aBati = $batiRepository->findByIdPar($this->id_par);
      $this->aBati = new BatiCollection();
      foreach ($aBati as $bati) {
        $this->aBati->add($bati);
      }
      return $this->aBati;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get bati list for compte communal ', [$this, $e]);
      return [];
    }
  }

  /**
   * @return string|null
   */
  public function getId(): ?string {
    return $this->id;
  }
  /**
   * Cette méthode permet de récupérer la liste des bâtis
   * @return BatiCollection|null
   */
  public function getBatiCollection(): ?BatiCollection {
    return $this->fillBatiArray();
  }
  /**
   * @return string|null
   * Cette méthode permet de concaténer les attributs ccodep et ccodir.
   */
  public function getDepdir(): ?string {
    return $this->ccodep . " " . $this->ccodir;
  }
  /**
   *  @return string|null
   * Cette méthode permet de concaténer les attributs ccocom et libcom.
   */
  public function getCom(): ?string {
    return $this->ccocom . " " . $this->libcom;
  }
  /**
   * @return string|null
   * Cette méthode permet de concaténer les attributs cconvo et dvoilib.
   */
  public function getAdresseParcelle(): ?string {
    return ltrim(rtrim($this->dnvoiri), '0') . rtrim($this->dindic) . " " . rtrim($this->cconvo) . " " . rtrim($this->dvoilib);
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur locative totale du bâti
   * Méthode dans BatiCollection
   */
  public function getValeurLocativeBati(): float {
    return $this->getBatiCollection()->getValeurLocativeBati();
  }

  /**
   * @return string|null
   */
  public function getDcntpa(): ?string {
    return $this->dcntpa;
  }

  /**
   * @return string|null
   * Cette méthode permet de récupérer l'année de référence pour une commune
   * Méthode modifiée
   */
  public function getAnneeReference(): ?string {
    if (!isset($this->oEntityManager)) {
      return null;
    }
    try {
      $MajicCommuneRepository = $this->oEntityManager->getRepository(Commune::class);
      $sIdCom = $this->ccodep . $this->ccocom;
      if (!$MajicCommuneRepository instanceof MajicCommuneRepository) {
        throw new \Exception("Repository is not a MajicCommuneRepository", 1);
      }
      return $MajicCommuneRepository->getAnneeReference($sIdCom);
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get year ', [$this, $e]);
      return null;
    }
  }

  /**
   * {@inheritDoc}
   * Cette méthode permet de construire un tableau correspondant au Bati de la Parcelle dans rapport cadastre parcelle
   * Il contient les bâtis
   */
  public function getMajicData(): array{
    return [
      'id_par' => $this->id_par,
      'annee_de_maj' => $this->getAnneeReference(),
      'dep_dir' => $this->getDepdir(),
      'code_dep' => $this->ccodep,
      'code_com' => $this->ccocom,
      'lib_dep' => null, // $this->libdep ABS
      'com' => $this->getCom(),
      'libcom' => $this->libcom,
      'role' => null,
      'vue' => null,
      'contenance' => $this->getDcntpa(),
      'adresse' => $this->getAdresseParcelle(),
      'total_valeur_locative' => $this->getValeurLocativeBati(),
      'aBat' => $this->getBatiCollection()->getMajicData(),
    ];
  }

  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet la récupération d'un gestionnaire d'entité pour interagir avec la base
   * @return void
   */
  public function injectObjectManager(ObjectManager $objectManager, ClassMetadata $classMetadata): void {
    if ($objectManager instanceof EntityManager) {
      $this->oEntityManager = $objectManager;
    }
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void {
    $this->oLoggerManager = $oService;
  }
}
