<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Collection\BatiCollection;
use App\Module\Cadastre\WebService\Collection\ParcelleCollection;
use App\Module\Cadastre\WebService\Entity\Majic\AbstractCompteCommunal;
use App\Module\Cadastre\WebService\Repository\BatiRepository;
use App\Module\Cadastre\WebService\Repository\ParcelleRepository;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour cadastre_rp.rp_compte_communal
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="@prop(cadastre_rp:rp_compte_communal)")
 */
class CompteCommunal extends AbstractCompteCommunal {
  /**
   * {@inheritdoc}
   */
  protected function fillBatiArray() {
    // passage en mode singleton
    if ($this->aBati instanceof BatiCollection && !$this->aBati->isEmpty()) { 
      return $this->aBati;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $batiRepository = $this->oEntityManager->getRepository(Bati::class);
      if (!$batiRepository instanceof BatiRepository) {
        throw new \Exception("Repository is not a BatiRepository", 1);
      }
      $aBati = $batiRepository->findByIdDnupro($this->id_dnupro);
      $this->aBati = new BatiCollection();
      foreach ($aBati as $bati) {
        $this->aBati->add($bati);
      }
      return $this->aBati;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get bati list for compte communal ', [$this, $e]);
      return [];
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function fillParcelleArray() {
    $IdPar = [];
    // passage en mode singleton
    if ($this->aParcelle instanceof ParcelleCollection && !$this->aParcelle->isEmpty()) { 
      return $this->aParcelle;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $parRepository = $this->oEntityManager->getRepository(Parcelle::class);
      if (!$parRepository instanceof ParcelleRepository) {
        throw new \Exception("Repository is not a ParcelleRepository", 1);
      }
      $aIdPar = $parRepository->findByIdPar($this->dnupro, ($this->ccodep . $this->ccocom));
      $aIdParBnd = $parRepository->findByIdParBnd($this->id_dnupro, substr($this->id_dnupro, 5), ($this->ccodep . $this->ccocom));
      $IdPar = array_merge($aIdPar, $aIdParBnd);
      $parcelle = $parRepository->findByIdInRange($IdPar);
      $this->aParcelle = new ParcelleCollection();
      foreach ($parcelle as $par) {
        $this->aParcelle->add($par);
        $par->setCcDnupro($this->id_dnupro);
        if (in_array($par->getIdPar(), $aIdParBnd) && !in_array($par->getIdPar(), $aIdPar)) {
          $par->setBnd(true);
        }
      }
      return $this->aParcelle;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get parcelle list for compte communal', [$e]);
      return [];
    }
  }

  /**
   * {@inheritDoc}
   * Cette méthode permet de construire un tableau correspondant au CompteCommunal dans le relevé de propriété
   */
  public function getMajicData(): array{
    $aData = parent::getMajicData();
    $aData['lib_dep'] = null;
    return $aData;
  }
}
