<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Collection\BatiCollection;
use App\Module\Cadastre\WebService\Collection\ParcelleCollection;
use App\Module\Cadastre\WebService\Entity\Majic\AbstractCompteCommunal;
use App\Module\Cadastre\WebService\Repository\BatiRepository;
use App\Module\Cadastre\WebService\Repository\CompteCommunalRepository;
use App\Module\Cadastre\WebService\Repository\ParcelleRepository;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour cadastre_rp.rp_compte_communal_parcelle
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\CompteCommunalRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="@prop(cadastre_rp:rp_compte_communal_parcelle)")
 */
class CompteCommunalParcelle extends AbstractCompteCommunal {
  /**
   * @var string
   * @Column(type="string", name="id_par")
   * @Id
   */
  private $id_par;

  /**
   * @var string
   * @Column(type="string", name="iddnupro")
   */
  private $iddnupro;

  /**
   * @var string
   * @Column(type="string", name="libdep")
   */
  private $libdep;

  /**
   * @var string
   * @Column(type="string", name="id_com")
   */
  private $id_com;

  /**
   * {@inheritdoc}
   */
  protected function fillBatiArray() {
    // passage en mode singleton
    if ($this->aBati instanceof BatiCollection && !$this->aBati->isEmpty()) { 
      return $this->aBati;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $batiRepository = $this->oEntityManager->getRepository(Bati::class);
      if (!$batiRepository instanceof BatiRepository) {
        throw new \Exception("Repository is not a BatiRepository", 1);
      }
      $aBati = $batiRepository->findByIdDnuproIdPar($this->iddnupro, $this->id_par);
      $this->aBati = new BatiCollection();
      foreach ($aBati as $bati) {
        $this->aBati->add($bati);
      }
      return $this->aBati;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get bati list for compte communal Parcelle ', [$this, $e]);
      return [];
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function fillParcelleArray() {
    // passage en mode singleton
    if ($this->aParcelle instanceof ParcelleCollection && !$this->aParcelle->isEmpty()) { 
      return $this->aParcelle;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $parRepository = $this->oEntityManager->getRepository(Parcelle::class);
      if (!$parRepository instanceof ParcelleRepository) {
        throw new \Exception("Repository is not a ParcelleRepository", 1);
      }
      $parcelle = $parRepository->findByIdDnuproIdPar($this->iddnupro, $this->id_par);
      $this->aParcelle = new ParcelleCollection();
      foreach ($parcelle as $par) {
        $this->aParcelle->add($par);
        $par->setCcDnupro($this->iddnupro);
      }
      return $this->aParcelle;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get parcelle list for compte communal Parcelle ', [$e]);
      return [];
    }
  }

  /**
   * {@inheritDoc}
   * Cette méthode permet de construire un tableau correspondant au CompteCommunalParcelle dans le relevé de propriété
   */
  public function getMajicData(): array{
    $aData = parent::getMajicData();
    $aData['lib_dep'] = $this->libdep;
    $aData['id_com'] = $this->id_com;
    return $aData;
  }
}
