<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Repository\DependDescrRepository;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour cadastre_rp.rp_depend_descr
 *  Cette entité permet de sélectionner les attributs nécessaires afin de traiter la partie Description dépendance
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\DependDescrRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="@prop(cadastre_rp:rp_depend_descr)")
 */
class DependDescr implements MajicInterface {
  /**
   * @var string
   * @Column(type="string", name="id_dep")
   * @Id
   */
  private $id_dep;

  /**
   * @var string
   * @Column(type="string", name="id_pev")
   */
  private $id_pev;

  /**
   * @var string
   * @Column(type="string", name="dnudes")
   */
  private $dnudes;

  /**
   * @var string
   * @Column(type="string", name="dnupev")
   */
  private $dnupev;

  /**
   * @var string
   * @Column(type="string", name="invar")
   */
  private $invar;

  /**
   * @var string
   * @Column(type="string", name="cconad")
   */
  private $cconad;

  /**
   * @var string
   * @Column(type="string", name="l_cconad")
   */
  private $l_cconad;

  /**
   * @var integer
   * @Column(type="integer", name="dsudep")
   */
  private $dsudep;

  /**
   * @var integer
   * @Column(type="integer", name="dcimlc")
   */
  private $dcimlc;

  /**
   * @var string
   * @Column(type="string", name="geaulc")
   */
  private $geaulc;

  /**
   * @var string
   * @Column(type="string", name="gelelc")
   */
  private $gelelc;

  /**
   * @var string
   * @Column(type="string", name="gchclc")
   */
  private $gchclc;

  /**
   * @var string
   * @Column(type="string", name="dnbbai")
   */
  private $dnbbai;

  /**
   * @var string
   * @Column(type="string", name="dnbdou")
   */
  private $dnbdou;

  /**
   * @var string
   * @Column(type="string", name="dnblav")
   */
  private $dnblav;

  /**
   * @var string
   * @Column(type="string", name="dnbwc")
   */
  private $dnbwc;

  /**
   * @var string
   * @Column(type="string", name="dmatgm")
   */
  private $dmatgm;

  /**
   * @var string
   * @Column(type="string", name="dmatto")
   */
  private $dmatto;

  /**
   * @var string
   * @Column(type="string", name="detent")
   */
  private $detent;
  /**
   * @var string
   * @Column(type="string", name="l_dmatgm")
   */
  private $l_dmatgm;
  /**
   * @var string
   * @Column(type="string", name="l_dmatto")
   */
  private $l_dmatto;
  /**
   * @var string
   * @Column(type="string", name="l_detent")
   */
  private $l_detent;

  /**
   * @return string|null
   */
  public function getIdDep(): ?string {
    return $this->id_dep;
  }
  /**
   * @return string|null
   */
  public function getIdPev(): ?string {
    return $this->id_pev;
  }
  /**
   * @return string|null
   */
  public function getCconad(): ?string {
    return $this->cconad;
  }
  /**
   * @return string|null
   */
  public function getEau(): ?string {
    return ($this->geaulc == 'O') ? 'EAU' : null;
  }

  /**
   * @return string|null
   */
  public function getElect(): ?string {
    return ($this->gelelc == 'O') ? 'ELECT' : null;
  }

  /**
   * @return string|null
   */
  public function getChauf(): ?string {
    return ($this->gchclc == 'O') ? 'CHAUF' : null;
  }

  /**
   * {@inheritDoc}
   * Cette méthode permet de construire un tableau correspondant à DependDescr dans le relevé de propriété
   */
  public function getMajicData(): array{
    return [
      'num_invar' => $this->invar,
      'num_pev' => $this->dnupev,
      'num_depend' => $this->dnudes,
      'nature' => $this->l_cconad,
      'surface' => $this->dsudep,
      'ponderation' => $this->dcimlc,
      'Etat' => $this->detent,
      'l_Etat' => $this->l_detent,
      'mat_gros_mur' => $this->dmatgm,
      'lib_mat_gros_mur' => $this->l_dmatgm,
      'mat_toiture' => $this->dmatto,
      'lib_mat_toiture' => $this->l_dmatto,
      'nb_baignoires' => $this->dnbbai,
      'nb_douches' => $this->dnbdou,
      'nb_lavabos' => $this->dnblav,
      'nb_wc' => $this->dnbwc,
      'eau' => $this->getEau(),
      'elect' => $this->getElect(),
      'chauf' => $this->getChauf(),
    ];
  }
}
