<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Repository\ExonSufRepository;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour s_majic.nb_30_suf_exo
 *  Cette entité permet de sélectionner les attributs nécessaires afin de traiter la partie éxonération des surfaces des subdivisions fiscales
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\ExonSufRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="@prop(cadastre_rp:rp_suf_exo)")
 */
class ExonSuf implements MajicInterface {
  /**
   * @var string
   * @Column(type="string", name="rnuexn")
   * @Id
   */
  private $rnuexn;

  /**
   * @var string
   * @Column(type="string", name="id_suf")
   * @Id
   */
  private $id_suf;

  /**
   * @var string
   * @Column(type="string", name="ccolloc")
   */
  private $ccolloc;

  /**
   * @var string
   * @Column(type="string", name="gnexts")
   */
  private $gnexts;
  
  /**
   * @var string
   * @Column(type="string", name="l_gnexts")
   */
  private $l_gnexts;

  /**
   * @var string
   * @Column(type="string", name="jfinex")
   */
  private $jfinex;

  /**
   * @var string
   * @Column(type="string", name="pexn")
   */
  private $pexn;

  /**
   * @var string
   * @Column(type="string", name="rcexnba")
   */
  private $rcexnba;

  /**
   * @var string
   * @Column(type="string", name="ccosub")
   */
  private $ccosub;

  /**
   * @var string
   * @Column(type="string", name="jandeb")
   */
  private $jandeb;

  /**
   * @var int
   * @Column(type="integer", name="vecexn")
   */
  private $vecexn;

  /**
   * @return string|null
   */
  public function getIdSuf(): ?string {
    return $this->id_suf;
  }

  /**
   * @return string|null
   */
  public function getCcolloc(): ?string {
    return $this->ccolloc;
  }

  /**
   * @return string|null
   */
  public function getRcexnba(): ?string {
    return $this->rcexnba;
  }

  /**
   * @return string|null
   */
  public function getNatExo(): ?string {
    return $this->gnexts;
  }

  /**
   * {@inheritDoc}
   * Cette méthode permet de construire un tableau correspondant a ExonSuf dans le relevé de propriété
   */
  public function getMajicData(): array{
    return [
      'rnuexn' => $this->rnuexn,
      'ccolloc' => $this->ccolloc,
      'nat_exo' => $this->gnexts,
      'nature_label' => $this->l_gnexts,
      'jfinex' => $this->jfinex,
      'pexn' => $this->pexn,
      'rcexnba' => $this->rcexnba,
      'ccosub' => $this->ccosub,
      'jandeb' => $this->jandeb,
      'vecexn' => $this->vecexn,
    ];
  }
}
