<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Repository\ExonePevRepository;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour s_majic.bati_30_pev_exo
 *  Cette entité permet de sélectionner les attributs nécessaires afin de traiter la partie d'éxonération de la partie d'évaluation
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\ExonePevRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="@prop(cadastre_rp:rp_pev_exo)")
 */
class ExonePev implements MajicInterface {
  /**
   * @var string
   * @Column(type="string", name="id_ex")
   * @Id
   */
  private $id_ex;

  /**
   * @var string
   * @Column(type="string", name="id_pev")
   */
  private $id_pev;

  /**
   * @var string
   * @Column(type="string", name="ccolloc")
   */
  private $ccolloc;

  /**
   * @var string
   * @Column(type="string", name="gnextl")
   */
  private $gnextl;

  /**
   * @var string
   * @Column(type="string", name="janimp")
   */
  private $janimp;

  /**
   * @var string
   * @Column(type="string", name="jandeb")
   */
  private $jandeb;

  /**
   * @var integer
   * @Column(type="integer", name="rcexba2")
   */
  private $rcexba2;

  /**
   * @var integer
   * @Column(type="integer", name="dvldif2a")
   */
  private $dvldif2a;

  /**
   * @var integer
   * @Column(type="integer", name="fcexba2")
   */
  private $fcexba2;

  /**
   * @var string
   * @Column(type="string", name="pexb")
   */
  private $pexb;

  /**
   * @var string
   * @Column(type="string", name="dnuexb")
   */
  private $dnuexb;

  /**
   * @var string
   * @Column(type="string", name="dnupev")
   */
  private $dnupev;

  /**
   * @var string
   * @Column(type="string", name="janbil")
   */
  private $janbil;

  /**
   * @var string
   * @Column(type="string", name="l_gnextl")
   */
  private $l_gnextl;

  /**
   * @return string|null
   */
  public function getIdEx(): ?string {
    return $this->id_ex;
  }
  /**
   * @return string|null
   */
  public function getIdPev(): ?string {
    return $this->id_pev;
  }

  /**
   * @return string|null
   */
  public function getGnextl(): ?string {
    return $this->gnextl;
  }

  /**
   * @return string
   * Cette méthode permet de récupérer les deux derniers caractères de l'attribut janimp
   */
  public function getAnret(): string {
    return substr($this->janimp, 2, 3);
  }

  /**
   * @return string
   * Cette méthode permet de récupérer les deux derniers caractères de l'attribut jandeb
   */
  public function getAndeb(): string {
    return substr($this->jandeb, 2, 3);
  }

  /**
   * @return integer
   * Cette méthode permet de récupérer le pourcentage d'éxonération à partir du champ 'pexb'
   */
  public function getPourcExo(): int {
    $iPourcExo = (int) $this->pexb / 100;
    return intval($iPourcExo);
  }

  /**
   * {@inheritDoc}
   * Cette méthode permet de construire un tableau correspondant à ExonePev dans le relevé de propriété
   */
  public function getMajicData(): array{
    return [
      'coll' => $this->ccolloc,
      'dnuexb' => $this->dnuexb,
      'dnupev' => $this->dnupev,
      'nat_exo' => $this->gnextl,
      'nature_label' => $this->l_gnextl,
      'an_ret' => $this->getAnret(),
      'an_deb' => $this->getAndeb(),
      'fraction_rc_exo' => $this->rcexba2,
      'dvldif2a' => $this->dvldif2a,
      'fcexba2' => $this->fcexba2,
      'janbil' => $this->janbil,
      'jandeb' => $this->jandeb,
      'janimp' => $this->janimp,
      'pourc_exo' => $this->getPourcExo(),
      // à changer au moment de la génération des rapports
      'tx_om' => '<font color=red><b>Info Manquante</b></font>',
      // à changer au moment de la génération des rapports
      'coef' => '<font color=red><b>Info Manquante</b></font>',
    ];
  }
}
