<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Repository\LotLocalRepository;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour s_majic.lot_local
 *  Cette entité permet de sélectionner les attributs nécessaires afin de traiter la partie lot-local.
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\LotLocalRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="lot_local")
 */
class LotLocal implements MajicInterface {
  /**
   * @var string
   * @Column(type="string", name="id_local")
   * @Id
   */
  private $id_local;

  /**
   * @var string
   * @Column(type="string", name="klot")
   * @Id
   */
  private $klot;

  /**
   * @var string|null
   * @Column(type="string", name="dnupdl")
   */
  private $dnupdl;

  /**
   * @var string|null
   * @Column(type="string", name="dnulot")
   */
  private $dnulot;

  /**
   * @var integer|null
   * @Column(type="integer", name="dnumql")
   */
  private $dnumql;

  /**
   * @var integer|null
   * @Column(type="integer", name="ddenql")
   */
  private $ddenql;

  /**
   * @return string
   */
  public function getIdLocal(): string {
    return $this->id_local;
  }

  /**
   * @return string|null
   * Cette méthode permet de concaténer les differents attributs d'un lot-local afin d'obtenir une seul chaine de caractères.
   */
  public function getLot(): ?string{
    $NumLot = intval($this->dnulot);
    return $this->dnupdl . ' LOT ' . $NumLot . ' ' . $this->dnumql . ' /' . $this->ddenql;
  }

  /**
   * {@inheritDoc}
   * Cette méthode permet de construire un tableau correspondant à LotLocal dans le relevé de propriété
   */
  public function getMajicData(): array{
    return [
      'klot' => $this->klot,
      'lot' => $this->getLot(),
    ];
  }
}
