<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Collection\AbstractSufCollection;
use App\Module\Cadastre\WebService\Entity\Majic\Suf;
use App\Module\Cadastre\WebService\Entity\Majic\SufBnd;
use App\Module\Cadastre\WebService\Repository\EmpriseRepository;
use App\Module\Cadastre\WebService\Repository\ParcelleRepository;
use App\Module\Cadastre\WebService\Repository\SufRepository;
use App\Module\Cadastre\WebService\Service\MajicUtils;
use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;
use Doctrine\Persistence\Mapping\ClassMetadata;
use Doctrine\Persistence\ObjectManager;

/**
 *  Objet ORM pour cadastre_rp.rp_parcelle
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\ParcelleRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="@prop(cadastre_rp:rp_parcelle)")
 */
class Parcelle implements MajicInterface {
  /**
   * @var Properties|null
   */
  private $oProperties;
  /**
   * @var EntityManager|null
   */
  private $oEntityManager;
  /**
   * @var LoggerManager|null
   */
  private $oLoggerManager;

  /**
   * @var string
   * @Column(type="string", name="id_par")
   * @Id
   */
  private $id_par;

  /**
   * @var string
   * @Column(type="string", name="jdatat")
   */
  private $jdatat;

  /**
   * @var string
   * @Column(type="string", name="ccosec")
   */
  private $ccosec;

  /**
   * @var string
   * @Column(type="string", name="dnupla")
   */
  private $dnupla;

  /**
   * @var string
   * @Column(type="string", name="dnvoiri")
   */
  private $dnvoiri;

  /**
   * @var string
   * @Column(type="string", name="dindic")
   */
  private $dindic;

  /**
   * @var string
   * @Column(type="string", name="cconvo")
   */
  private $cconvo;

  /**
   * @var string
   * @Column(type="string", name="dvoilib")
   */
  private $dvoilib;

  /**
   * @var string
   * @Column(type="string", name="ccoriv")
   */
  private $ccoriv;

  /**
   * @var string
   * @Column(type="string", name="dparpi")
   */
  private $dparpi;

  /**
   * @var string
   * @Column(type="string", name="gparnf")
   */
  private $gparnf;

  /**
   * @var string
   * @Column(type="string", name="dcntpa")
   */
  private $dcntpa;

  /**
   * @var string
   * @Column(type="string", name="dnupro")
   */
  private $dnupro;

  /**
   * @var string
   * @Column(type="string", name="gparbat")
   */
  private $gparbat;

  /**
   * @var string
   * @Column(type="string", name="gurbpa")
   */
  private $gurbpa;

  /**
   * @var string
   * @Column(type="string", name="typevoie")
   */
  private $typevoie;

  /**
   * @var string
   * @Column(type="string", name="id_dnupro")
   */
  private $id_dnupro;

  /**
   * @var AbstractSufCollection
   */
  private $aSuf;

  /**
   * @var array
   */
  private $aIntersect = [];

  /**
   * @var bool
   */
  private $bnd = false;

  /**
   * @var string
   */
  private $ccDnupro = null;

  public function __construct() {
    $this->aSuf = new AbstractSufCollection();
  }

  /**
   * @return string
   */
  public function getIdPar(): string {
    return $this->id_par;
  }

  /**
   * @return string|null
   */
  public function getDcntpa(): ?string {
    return $this->dcntpa;
  }

  /**
   * @return string|null
   */
  public function getIdDnupro(): ?string {
    return $this->id_dnupro;
  }

  /**
   * @return bool
   */
  public function isBnd(): bool {
    return $this->bnd;
  }

  /**
   * @return void
   */
  public function setBnd(bool $Bnd): void {
    $this->bnd = $Bnd;
  }

  /**
   * @return void
   */
  public function setCcDnupro(string $CcDnupro): void {
    $this->ccDnupro = $CcDnupro;
  }

  /**
   * remplis la liste des suf si necessaire
   * @return AbstractSufCollection|null
   */
  private function fillSufArray(): ?AbstractSufCollection {
    // passage en mode singleton
    if ($this->aSuf instanceof AbstractSufCollection && !$this->aSuf->isEmpty()) { 
      return $this->aSuf;
    }

    if (!isset($this->oEntityManager)) {
      return null;
    }

    $aIdSuf = [];
    $aIdSufBnd = [];
    $substrCcDnupro = substr($this->ccDnupro, 5);
    try {
      $sufRepository = $this->oEntityManager->getRepository(Suf::class);
      $sufBndRepository = $this->oEntityManager->getRepository(SufBnd::class);
      if (!$sufRepository instanceof SufRepository || !$sufBndRepository instanceof SufRepository) {
        throw new \Exception("Repository is not a SufRepository", 1);
      }
      // on récupère tous les identifiant suf en lien avec la parcelle
      // et le dnupro pour faire la répartition
      $aIdSufIntersected = $sufRepository->findByIdPar($this->id_par, $this->ccDnupro, $substrCcDnupro);
      foreach ($aIdSufIntersected as $aSufIntersected) {
        // répartition suf/suf_bnd
        if ($aSufIntersected['dnupro'] == $substrCcDnupro) {
          $aIdSuf[] = $aSufIntersected['id_suf'];
        } else {
          $aIdSufBnd[] = $aSufIntersected['id_suf'];
        }
      }
      // récupération des entités suf/suf_bnd
      $aSuf = $sufRepository->findByIdInRange($aIdSuf);
      $aSufBnd = $sufBndRepository->findByIdInRange($aIdSufBnd);
      // on fusionne les deux tableaux pour avoir un tableau d'abstractSuf
      $aMergeSuf = array_merge($aSuf, $aSufBnd);
      $this->aSuf = new AbstractSufCollection();
      foreach ($aMergeSuf as $oAbstractSuf) {
        $this->aSuf->add($oAbstractSuf);
      }
      return $this->aSuf;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get Suf list for Parcelle ', [$this, $e]);
      return null;
    }
  }

  /**
   * Cette méthode permet de récupérer les AbstractSufCollection
   * @return AbstractSufCollection|null
   */
  public function getAbstractSufCollection(): ?AbstractSufCollection {
    return $this->fillSufArray();
  }

  /**
   * @return string
   * Cette méthode permet de récupérer les deux derniers caractères de l'attribut Jdatat et d'y ajouter 1.
   */
  public function getJdatat(): string {
    $sNewJdatat = $this->jdatat;
    if (preg_match('`^\d{1,2}/\d{1,2}/\d{4}$`', $sNewJdatat)) {
      return substr(strval(intval(substr($sNewJdatat, 6, 4)) + 1), 2, 2);
    } else if (preg_match('`^\d{1,2}/\d{1,2}/\d{2}$`', $sNewJdatat)) {
      $sNewJdatat = strval(intval(substr($sNewJdatat, 6, 2)) + 1);
      if (strlen($sNewJdatat) == 1) {
        $sNewJdatat = '0' . $sNewJdatat;
      }
      return substr($sNewJdatat, -2, 2);
    }
    return substr(strval(intval(substr($sNewJdatat, 4, 4)) + 1), 2, 2);
  }

  /**
   * @return string|null
   * Cette méthode permet d'avoir l'attribut dnupla sur 4 caractères.
   */
  public function getNumeroParcelle(): ?string{
    $sDnupla = $this->dnupla;
    if ((strlen($sDnupla) == 0 || strlen($sDnupla) > 4)) {
      return null;
    }
    // on ajoute des 0 devant la valeur de façon à être sur 4 char
    while (strlen($sDnupla) < 4) {
      $sDnupla = '0' . $sDnupla;
    }
    return $sDnupla;
  }

  /**
   * @return string
   * Cette méthode permet de concaténer les attributs dnvoiri et indic.
   */
  public function getNumeroVoirie(): string {
    return ltrim($this->dnvoiri, '0') . $this->dindic;
  }

  /**
   * @return string
   * Cette méthode permet de concaténer les attributs cconvo et dvoilib.
   */
  public function getAdresseParcelle(): string {
    return ltrim(rtrim($this->dnvoiri), '0') . rtrim($this->dindic) . ' ' . rtrim($this->cconvo) . ' ' . rtrim($this->dvoilib);
  }

  /**
   * @return string
   * Cette méthode permet de trouver la valeur de gparbat.
   */
  public function getGparbat(): string {
    return (empty($this->gparbat) || $this->gparbat == '0') ? 'Non' : 'Oui';
  }
  /**
   * @return string
   * Cette méthode permet de trouver la valeur de gparbat.
   */
  public function getGurbpa(): string {
    return (empty($this->gurbpa) || $this->gurbpa == '0') ? 'Non' : 'Oui';
  }

  /**
   * @return string|null
   * Cette méthode permet de trouver la valeur de lieudit.
   */
  public function getLieudit(): ?string {
    return ($this->typevoie == '3') ? $this->getAdresseParcelle() : null;
  }

  /**
   * @return integer
   * Cette méthode permet de récupérer la valeur de l'attribut 'contenance' pour des parcelles avec exonération dans le tableau aSuf.
   */
  public function getContenance(): int {
    return $this->getAbstractSufCollection()->getContenance();
  }

  /**
   * @return integer|null
   * Cette méthode permet de récupérer les caractères correspondants à la colonne HA depuis l'attribut contenance_suf du tableau aSuf.
   */
  public function getContenanceHA(): ?int{
    $sContenanceHA = intval($this->getContenance() / 10000);
    if ($sContenanceHA == 0) {
      return null;
    }
    return $sContenanceHA;
  }

  /**
   * @return string|null
   * Cette méthode permet de récupérer les caractères correspondants à la colonne A depuis l'attribut contenance_suf du tableau aSuf.
   */
  public function getContenanceA(): ?string{
    $sContenanceA = strval(intval(($this->getContenance() % 10000) / 100));
    $iContenanceHA = $this->getContenanceHA();
    if ($iContenanceHA == null && $sContenanceA == 0) {
      return null;
    }
    if ($iContenanceHA == null && strlen($sContenanceA) == 2 && substr($sContenanceA, 0, 1) == 0) {
      return ' ' . substr($sContenanceA, 1, 1);
    }
    if ($iContenanceHA != null && strlen($sContenanceA) == 1) {
      return '0' . $sContenanceA;
    }
    return $sContenanceA;
  }

  /**
   * @return string
   * Cette méthode permet de récupérer les caractères correspondants à la colonne CA depuis l'attribut contenance_suf du tableau aSuf.
   */
  public function getContenanceCA(): string {
    $sContenanceCA = strval(intval($this->getContenance() % 100));
    if (intval($this->getContenanceA()) != 0) {
      if (strlen($sContenanceCA) == 1) {
        return '0' . $sContenanceCA;
      }
    }
    return $sContenanceCA;
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' dans le tableau aSuf
   * Provient de AbstractSufCollection
   */
  public function getRevenuCadastral(): float {
    return $this->getAbstractSufCollection()->getRevenuCadastral();
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu reference' dans le tableau aSuf
   * Provient de AbstractSufCollection
   * Méthode corrigée.
   */
  public function getRevenuReference(): float {
    return $this->getAbstractSufCollection()->getRevenuReference();

  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf
   * Provient de AbstractSufCollection
   * Méthode corrigée.
   */
  public function getRevenuCadastralAvecExo(): float {
    return $this->getAbstractSufCollection()->getRevenuCadastralAvecExo();

  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf
   * Provient de AbstractSufCollection
   * Lorsque l'exoneration est procuré par une commune, un groupement de commune ou toutes collectivité.
   * Méthode corrigée.
   */
  public function getRevenuCadastralComAvecExo(): float {
    return $this->getAbstractSufCollection()->getRevenuCadastralComAvecExo();

  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf
   * Provient de AbstractSufCollection
   * Lorsque l'exoneration est procuré par une commune, un groupement de commune ou toutes collectivité.
   * Méthode corrigée.
   */
  public function getRevenuCadastralTaxeADAvecExo(): float {
    return $this->getAbstractSufCollection()->getRevenuCadastralTaxeADAvecExo();
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf
   * Provient de AbstractSufCollection
   * Lorsque l'exoneration est procuré par une commune, un groupement de commune ou toutes collectivité
   * Méthode corrigée.
   */
  public function getRevenuCadastralDepAvecExo(): float {
    return $this->getAbstractSufCollection()->getRevenuCadastralDepAvecExo();
  }

  /**
   * @return float
   * Cette méthode permet de récupérer la valeur de l'attribut 'revenu cadastral' pour des parcelles avec exonération dans le tableau aSuf
   * Provient de AbstractSufCollection
   * Lorsque l'exoneration est procuré par la région ou toutes collectivité.
   * Méthode corrigée.
   */
  public function getRevenuCadastralRegAvecExo(): float {
    return $this->getAbstractSufCollection()->getRevenuCadastralRegAvecExo();
  }

  /**
   * @return void
   * Cette méthode permet de réupérer l'ensemble des emprises de la parcelles à partir des properties de la fiche d'urbanisme
   */
  public function getEmprises(EntityManager $oEntityManager): void {
    $empriseRepository = $oEntityManager->getRepository(Emprise::class);
    if (!$empriseRepository instanceof EmpriseRepository) {
      throw new \Exception("Repository is not a EmpriseRepository", 1);
    }
    $aIntersect = $this->oProperties->get('cadastre_fiche_urba_intersect_tables') ?? [];
    $iTolerance = $this->oProperties->get('cadastre_fiche_urba_intersect_tolerance') ?? 1;
    $aViewBati = $this->oProperties->get('cadastre_fiche_urba_view_bati') ?? [];
    $aIntersect = array_merge($aViewBati, $aIntersect);
    $aIdParIntersected = $empriseRepository->findByIdPar($this->id_par, $aIntersect, $iTolerance);
    $this->aIntersect = $aIdParIntersected;
  }

  /**
   * @return array
   * Cette méthode permet de récupérer les intersections de emprise
   */
  public function getIntersect(): array{
    return !empty($this->aIntersect) ? $this->aIntersect : [];
  }

  /**
   * {@inheritDoc}
   * Cette méthode permet de construire un tableau correspondant à Parcelle dans le relevé de propriété
   * Il contient Suf
   */
  public function getMajicData(): array{
    $aData = [
      'dateheure' => date('d/m/Y') . ' à ' . date('H:m'),
      'id_par' => $this->id_par,
      'date' => $this->getJdatat(),
      'section' => $this->ccosec,
      'num_plan' => $this->dnupla,
      'num_parc' => $this->getNumeroParcelle(),
      'num_voirie' => $this->getNumeroVoirie(),
      'adresse' => $this->getAdresseParcelle(),
      'code_rivoli' => $this->ccoriv,
      'num_parc_prim' => $this->dparpi,
      'fp_dp' => $this->gparnf,
      'gurbpa' => $this->getGurbpa(),
      'gparbat' => $this->getGparbat(),
      'typevoie' => $this->typevoie,
      'lieudit' => $this->getLieudit(),
      'surface' => $this->dcntpa,
      'dnupro' => $this->dnupro,
      's_tar' => $this->getAbstractSufCollection()->getSTar(),
      'suf' => $this->getAbstractSufCollection()->getSuf(),
      'grss_gr' => $this->getAbstractSufCollection()->getGrssGr(),
      'clas' => $this->getAbstractSufCollection()->getClas(),
      'nat_cult' => $this->getAbstractSufCollection()->getNatCult(),
      'ha' => $this->getContenanceHA(),
      'a' => $this->getContenanceA(),
      'ca' => $this->getContenanceCA(),
      'revenu_cadastral_avec_exo' => $this->getRevenuCadastralAvecExo(),
      'revenu_cadastral_com_avec_exo' => $this->getRevenuCadastralComAvecExo(),
      'revenu_cadastral_dep_avec_exo' => $this->getRevenuCadastralDepAvecExo(),
      'revenu_cadastral_reg_avec_exo' => $this->getRevenuCadastralRegAvecExo(),
      'revenu_cadastral' => $this->getRevenuCadastral(),
      'revenu_cadastral_txt' => MajicUtils::setFloatToText($this->getRevenuCadastral()),
      'revenu_reference' => $this->getRevenuReference(),
      'revenu_reference_txt' => MajicUtils::setFloatToText($this->getRevenuReference()),
      'contenance' => $this->getContenance(),
      'is_cadastre_user' => true, // Ne valait jamais false dans l'ancien VMAP
      'aSuf' => $this->getAbstractSufCollection()->getMajicData(),
    ];
    if (!empty($this->aIntersect)) {
      $aIntersection = [];
      foreach ($this->aIntersect as $intersect) {
        $data = $intersect->getMajicData();
        array_push($aIntersection, $data);
      }
      $aData['intersect'] = $aIntersection;
    }
    return $aData;
  }

  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet la récupération d'un gestionnaire d'entité pour interagir avec la base
   * @return void
   */
  public function injectObjectManager(ObjectManager $objectManager, ClassMetadata $classMetadata): void {
    if ($objectManager instanceof EntityManager) {
      $this->oEntityManager = $objectManager;
    }
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void {
    $this->oLoggerManager = $oService;
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @param Properties $oService
   * @return void
   */
  public function setPropertiesService(Properties $oService): void {
    $this->oProperties = $oService;
  }
}
