<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Collection\MajicCollection;
use App\Module\Cadastre\WebService\Entity\Majic\DependDescr;
use App\Module\Cadastre\WebService\Entity\Majic\ExonePev;
use App\Module\Cadastre\WebService\Entity\Majic\HabitDescr;
use App\Module\Cadastre\WebService\Entity\Majic\ProfDescr;
use App\Module\Cadastre\WebService\Repository\DependDescrRepository;
use App\Module\Cadastre\WebService\Repository\ExonePevRepository;
use App\Module\Cadastre\WebService\Repository\HabitDescrRepository;
use App\Module\Cadastre\WebService\Repository\PevRepository;
use App\Module\Vitis\WebService\Repository\GenericRepository;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;
use Doctrine\Persistence\Mapping\ClassMetadata;
use Doctrine\Persistence\ObjectManager;

/**
 * Objet ORM pour cadastre_rp.rp_pev
 * Cette classe permet de sélectionner les attributs nécessaires afin de traiter la partie d'évaluation.
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\PevRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="@prop(cadastre_rp:rp_pev)")
 */
class Pev implements MajicInterface {
  /**
   * @var EntityManager|null
   */
  private $oEntityManager;

  /**
   * @var LoggerManager|null
   */
  private $oLoggerManager;

  /**
   * @var string
   * @Column(type="string", name="id_pev")
   * @Id
   */
  private $id_pev;

  /**
   * @var string
   * @Column(type="string", name="id_local")
   */
  private $id_local;

  /**
   * @var string
   * @Column(type="string", name="ccostb")
   */
  private $ccostb;

  /**
   * @var string
   * @Column(type="string", name="ccoaff")
   */
  private $ccoaff;

  /**
   * @var string
   * @Column(type="string", name="dcapec")
   */
  private $dcapec;

  /**
   * @var string
   * @Column(type="string", name="dnupev")
   */
  private $dnupev;

  /**
   * @var integer
   * @Column(type="integer", name="dvlper")
   */
  private $dvlper;

  /**
   * @var integer
   * @Column(type="integer", name="dvlpera")
   */
  private $dvlpera;

  /**
   * @var string
   * @Column(type="string", name="gnexpl")
   */
  private $gnexpl;

  /**
   * @var string
   * @Column(type="string", name="dnuref")
   */
  private $dnuref;

  /**
   * @var integer
   * @Column(type="integer", name="bipeva")
   */
  private $bipeva;


  /**
   * @var integer
   * @Column(type="integer", name="dsupot")
   */
  private $dsupot;

  /**
   * @var string
   * @Column(type="string", name="l_ccoaff")
   */
  private $l_ccoaff;

  /**
   * @var string
   * @Column(type="string", name="l_gnexpl")
   */
  private $l_gnexpl;

  /**
   * @var string
   * @Column(type="string", name="dcsglc")
   */
  private $dcsglc;

  /**
   * @var string
   * @Column(type="string", name="dcsplc")
   */
  private $dcsplc;

  /**
   * @var string
   * @Column(type="string", name="dcralc")
   */
  private $dcralc;

  /**
   * @var integer
   * @Column(type="integer", name="dcetlc")
   */
  private $dcetlc;


  /**
   * @var MajicCollection
   * Tableau associatif comportant la liste des objets Exone_Pev du bati.
   */
  private $aExonePev;

  /**
   * @var MajicCollection
   * Tableau associatif comportant la liste des objets Descr de(s) habitation(s)
   */
  private $aHabitDescr;

  /**
   * @var MajicCollection
   * Tableau associatif comportant la liste des objets Descr de(s) dépendance(s)
   */
  private $aDependDescr;

  /**
   * @var MajicCollection
   * Tableau associatif comportant la liste des objets Descr professionel
   */
  private $aProfDescr;

  public function __construct() {
    $this->aExonePev = new MajicCollection();
    $this->aHabitDescr = new MajicCollection();
    $this->aDependDescr = new MajicCollection();
    $this->aProfDescr = new MajicCollection();
  }
  /**
   * @return string|null
   */
  public function Ccoaff(): ?string {
    return $this->ccoaff;
  }
  /**
   * @return string|null
   */
  public function getCcostb(): ?string {
    return $this->ccostb;
  }

  /**
   * @return string|null
   */
  public function getDcapec(): ?string {
    return $this->dcapec;
  }
  
  /**
   * @return string|null
   */
  public function getIdLocal(): ?string {
    return $this->id_local;
  }

  /**
   * @return string|null
   */
  public function getDnupev(): ?string {
    return $this->dnupev;
  }

  /**
   * @return integer
   */
  public function getDvlpera(): int {
    return $this->dvlpera;
  }

  /**
   * remplis la liste des exonérations de la partie d'évaluation si necessaire
   * @return MajicCollection|array
   */
  private function fillExonePevArray() : MajicCollection|array {
    $aExonePev = [];
    if ($this->aExonePev instanceof MajicCollection && !$this->aExonePev->isEmpty()) {
      return $this->aExonePev;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $exonePevRepository = $this->oEntityManager->getRepository(ExonePev::class);
      if (!$exonePevRepository instanceof ExonePevRepository) {
        throw new \Exception("Repository is not a ExonePevRepository", 1);
      }
      $aExonePev = $exonePevRepository->findByIdPev($this->id_pev);
      $this->aExonePev = new MajicCollection();
      foreach ($aExonePev as $exonePev) {
        $this->aExonePev->add($exonePev);
      }
      return $this->aExonePev;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get ExonePev list for Pev ', [$this, $e]);
      return [];
    }
  }

  /**
   * Cette méthode permet de récupérer les éxonérations de la partie d'évaluation (PEV)
   * @return MajicCollection|null
   */
  public function getExonePevCollection(): ?MajicCollection {
    return $this->fillExonePevArray();
  }

  /**
   * remplis la liste des habitations si necessaire
   * @return MajicCollection|array
   */
  private function fillHabitDescrArray() {
    $aHabitDescr = [];
    if ($this->aHabitDescr instanceof MajicCollection && !$this->aHabitDescr->isEmpty()) {
      return $this->aHabitDescr;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $habitDescrRepository = $this->oEntityManager->getRepository(HabitDescr::class);
      if (!$habitDescrRepository instanceof HabitDescrRepository) {
        throw new \Exception("Repository is not a HabitDescrRepository", 1);
      }
      $aHabitDescr = $habitDescrRepository->findByIdPev($this->id_pev);
      $this->aHabitDescr = new MajicCollection();
      foreach ($aHabitDescr as $habitDescr) {
        $this->aHabitDescr->add($habitDescr);
      }
      return $this->aHabitDescr;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get habitDescr list for Pev ', [$this, $e]);
      return [];
    }
  }

  /**
   * Cette méthode permet de récupérer les descriptions des habitations
   * @return MajicCollection|null
   */
  public function getHabitDescrCollection(): ?MajicCollection {
    return $this->fillHabitDescrArray();
  }

  /**
   * remplis la liste des dépendances si necessaire
   * @return MajicCollection|array
   */
  private function fillDependDescrArray() {
    $aDependDescr = [];
    if ($this->aDependDescr instanceof MajicCollection && !$this->aDependDescr->isEmpty()) {
      return $this->aDependDescr;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $dependDescrRepository = $this->oEntityManager->getRepository(DependDescr::class);
      if (!$dependDescrRepository instanceof DependDescrRepository) {
        throw new \Exception("Repository is not a DependDescrRepository", 1);
      }
      $aDependDescr = $dependDescrRepository->findByIdPev($this->id_pev);
      $this->aDependDescr = new MajicCollection();
      foreach ($aDependDescr as $dependDescr) {
        $this->aDependDescr->add($dependDescr);
      }
      return $this->aDependDescr;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get DependDescr list for Pev ', [$this, $e]);
      return [];
    }
  }

  /**
   * Cette méthode permet de récupérer les descriptions des dépendances
   * @return MajicCollection|null
   */
  public function getDependDescrCollection(): ?MajicCollection {
    return $this->fillDependDescrArray();
  }

  /**
   * remplis la liste des locaux professionnels si necessaire
   * @return MajicCollection|array
   */
  private function fillProfDescrArray() {
    $aProfDescr = [];
    if ($this->aProfDescr instanceof MajicCollection && !$this->aProfDescr->isEmpty()) {
      return $this->aProfDescr;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $genericRepository = $this->oEntityManager->getRepository(ProfDescr::class);
      if (!$genericRepository instanceof GenericRepository) {
        throw new \Exception("Repository is not a GenericRepository", 1);
      }
      $aProfDescr = $genericRepository->findById($this->id_pev);
      $this->aProfDescr = new MajicCollection();
      foreach ($aProfDescr as $profDescr) {
        $this->aProfDescr->add($profDescr);
      }
      return $this->aProfDescr;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get ProfDescr list for Pev ', [$this, $e]);
      return [];
    }
  }

  /**
   * Cette méthode permet de récupérer les descriptions des locaux professionnels
   * @return MajicCollection|null
   */
  public function getProfDescrCollection(): ?MajicCollection {
    return $this->fillProfDescrArray();
  }

  /**
   * @return string|null
   */
  public function getNatExo(): ?string {
    if (!empty($this->gnexpl)) {
      return $this->gnexpl;
    }
    if ($this->getExonePevCollection()->count() > 0) {
      return $this->getExonePevCollection()->first()->getGnextl();
    }
    return null;
  }

  /**
   * @return integer|float
   */
  public function getRevenuCadastral() {
    if ($this->getNatExo() != null) {
      $iRevenuCadastral = floatval($this->dvlpera) / 2;
      return round($iRevenuCadastral);
    }
    return $this->bipeva;
  }

  /**
   * @return float
   */
  public function getRevenuCadastralfloat(): float {
    if ($this->getNatExo() != null) {
      $iRevenuCadastral = floatval($this->dvlpera) / 2;
      return $iRevenuCadastral;
    }
    return floatval($this->bipeva);
  }

  /**
   * {@inheritDoc}
   * Cette méthode permet de construire un tableau correspondant à Pev dans le relevé de propriété
   * Il contient ExonePev, HabitDescr, DependDescr et ProfDescr
   */
  public function getMajicData(): array{
    return [
      's_tar' => $this->ccostb,
      'af' => $this->ccoaff,
      'l_nature' => $this->l_ccoaff, // Pointe sur un champs qui n'existe pas
      'cat' => $this->dcapec,
      'nat_exo' => $this->getNatExo(),
      'revenu_cadastral' => $this->getRevenuCadastral(),
      'num_pev' => $this->dnupev,
      'val_loc_ref' => $this->dvlper,
      'val_loc_ann' => $this->getDvlpera(),
      'nature' => $this->gnexpl,
      'l_nat_exon' => $this->l_gnexpl, // Pointe sur un champs qui n'existe pas
      'local_type' => $this->dnuref,
      'dcsglc' => $this->dcsglc,
      'dcsplc' => $this->dcsplc,
      'dcralc' => $this->dcralc,
      'dcetlc' => $this->dcetlc,
      'dsupot' => $this->dsupot,
      'aExonePev' => $this->getExonePevCollection()->getMajicData(),
      'aHabitDescr' => $this->getHabitDescrCollection()->getMajicData(),
      'aDependDescr' => $this->getDependDescrCollection()->getMajicData(),
      'aProfDescr' => $this->getProfDescrCollection()->getMajicData(),
    ];
  }

  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet la récupération d'un gestionnaire d'entité pour interagir avec la base
   * @return void
   */
  public function injectObjectManager(ObjectManager $objectManager, ClassMetadata $classMetadata): void {
    if ($objectManager instanceof EntityManager) {
      $this->oEntityManager = $objectManager;
    }
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void {
    $this->oLoggerManager = $oService;
  }
}
