<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Repository\ProprietaireRepository;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour cadastre_rp.rp_proprietaire
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\ProprietaireRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="@prop(cadastre_rp:rp_proprietaire)")
 */
class Proprietaire implements MajicInterface {
  /**
   * @var string
   * @Column(type="string", name="id_dnupro")
   * @Id
   */
  private $id_dnupro;

  /**
   * @var string
   * @Column(type="string", name="id_pro")
   * @Id
   */
  private $id_pro;

  /**
   * @var string
   * @Column(type="string", name="dnupro")
   */
  private $dnupro;

  /**
   * @var string
   * @Column(type="string", name="dnuper")
   */
  private $dnuper;

  /**
   * @var string
   * @Column(type="string", name="dqualp")
   */
  private $dqualp;

  /**
   * @var string
   * @Column(type="string", name="ddenom")
   */
  private $ddenom;

  /**
   * @var string
   * @Column(type="string", name="dnomlp")
   */
  private $dnomlp;

  /**
   * @var string
   * @Column(type="string", name="dprnlp")
   */
  private $dprnlp;

  /**
   * @var string
   * @Column(type="string", name="epxnee")
   */
  private $epxnee;

  /**
   * @var string
   * @Column(type="string", name="dnomcp")
   */
  private $dnomcp;

  /**
   * @var string
   * @Column(type="string", name="dprncp")
   */
  private $dprncp;

  /**
   * @var string
   * @Column(type="string", name="jdatnss")
   */
  private $jdatnss;

  /**
   * @var string
   * @Column(type="string", name="dldnss")
   */
  private $dldnss;

  /**
   * @var string
   * @Column(type="string", name="dlign3")
   */
  private $dlign3;

  /**
   * @var string
   * @Column(type="string", name="dlign4")
   */
  private $dlign4;

  /**
   * @var string
   * @Column(type="string", name="dlign5")
   */
  private $dlign5;

  /**
   * @var string
   * @Column(type="string", name="dlign6")
   */
  private $dlign6;

  /**
   * @var string
   * @Column(type="string", name="gtoper")
   */
  private $gtoper;

  /**
   * @var string
   * @Column(type="string", name="l_ccodro")
   */
  private $l_ccodro;

  /**
   * @var string
   * @Column(type="string", name="l_ccodem")
   */
  private $l_ccodem;

  /**
   * @var string
   * @Column(type="string", name="gdesip")
   */
  private $gdesip;

  /**
   * @return string|null
   */
  public function getIdDnupro(): ?string {
    return $this->id_dnupro;
  }
  /**
   * @return string|null
   */
  public function getIdPro(): ?string {
    return $this->id_pro;
  }

  /**
   * @return string|null
   */
  public function getLCcodem(): ?string {
    return $this->l_ccodem;
  }

  /**
   * Cette méthode permet de récupérer le nom du propriétaire en fonction de la valeur de l'attribut epxnee.
   * @return string
   */
  public function getProprietaireNom(): string {
    switch ($this->epxnee) {
      case 'EPX':
        return rtrim($this->dqualp) . ' ' . rtrim($this->ddenom) . ' EP ' . rtrim($this->dnomcp) . ' ' . rtrim($this->dprncp);
      case 'NEE':
        return rtrim($this->dqualp) . ' ' . rtrim($this->ddenom) . ' EP ' . rtrim($this->dnomlp) . ' ' . rtrim($this->dprnlp);
      default:
        return rtrim($this->dqualp) . ' ' . rtrim($this->ddenom);
    }
  }

  /**
   * Cette méthode permet de récupérer le nom du propriétaire correspondant à un relevé de propriété destiné aux tiers.
   * @return string
   */
  public function getProprietaireNomTiers(): string {
    return rtrim($this->dqualp) . ' ' . rtrim($this->ddenom);
  }

  /**
   * @return string
   * Cette méthode permet de récupérer l'adresse complète du propriétaire en concaténant les attributs dlign3, dlign4, dlign5 et dlign6.
   */
  public function getProprietaireAdresse(): string {
    return rtrim($this->dlign3) . ' ' . ltrim(rtrim($this->dlign4), '0') . ' ' . rtrim($this->dlign5) . ' ' . rtrim($this->dlign6);
  }

  /**
   * @return string|null
   */
  public function getJdatnss(): ?string {
    if ($this->gtoper == '2') {
      return null;
    }
    return 'NE(E) le ' . $this->jdatnss;
  }
  /**
   * @return string|null
   */
  public function getDldnss(): ?string {
    if ($this->gtoper == '2') {
      return null;
    }
    return 'A ' . $this->dldnss;
  }

  /**
   * {@inheritDoc}
   * Cette méthode permet de construire un tableau correspondant à Propriétaire dans le relevé de propriété
   */
  public function getMajicData(): array{
    return [
      'l_ccodro' => $this->l_ccodro,
      'l_ccodem' => $this->l_ccodem,
      'dnuper' => $this->dnuper,
      'dnupro' => $this->dnupro,
      'gdesip' => $this->gdesip,
      'proprietaire_nom' => $this->getProprietaireNom(),
      'proprietaire_nom_tiers' => $this->getProprietaireNomTiers(),
      'proprietaire_adresse' => $this->getProprietaireAdresse(),
      'jdatnss' => $this->getJdatnss(),
      'dldnss' => $this->getDldnss(),
    ];
  }
}
