<?php

namespace App\Module\Cadastre\WebService\Entity\Majic;

use App\Module\Cadastre\WebService\Collection\ProprietaireCollection;
use App\Module\Cadastre\WebService\Entity\Majic\AbstractSuf;
use App\Module\Cadastre\WebService\Entity\Majic\Proprietaire;
use App\Module\Cadastre\WebService\Repository\ProprietaireRepository;
use App\Module\Cadastre\WebService\Repository\SufRepository;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour v_vmap_rp_suf_bnd_v2
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\SufRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_majic)",name="v_vmap_rp_suf_bnd_v2")
 */
class SufBnd extends AbstractSuf {
  /**
   * @var string
   * @Column(type="string", name="dnupdl")
   */
  private $dnupdl;

  /**
   * @var string
   * @Column(type="string", name="dnulot")
   */
  private $dnulot;

  /**
   * @var int
   * @Column(type="integer", name="dnumql")
   */
  private $dnumql;

  /**
   * @var int
   * @Column(type="integer", name="ddenql")
   */
  private $ddenql;
  /**
   * @var ProprietaireCollection
   */
  private $aProp;

  public function __construct() {
    parent::__construct();
    $this->aProp = new ProprietaireCollection();

  }
  /**
   * remplis la liste des proprietaires si necessaire
   * @return ProprietaireCollection|array
   */
  private function fillProprietaireArray() {
    $aProp = [];
    if (empty($this->dnulot)) {
      return [];
    }
    if ($this->aProp instanceof ProprietaireCollection && !$this->aProp->isEmpty()) { 
      return $this->aProp;
    }
    if (!isset($this->oEntityManager)) {
      return [];
    }
    try {
      $proprietaireRepository = $this->oEntityManager->getRepository(Proprietaire::class);
      if (!$proprietaireRepository instanceof ProprietaireRepository) {
        throw new \Exception("Repository is not a ProprietaireRepository", 1);
      }
      $aProp = $proprietaireRepository->findByIdDnupro($this->id_dnupro);
      $this->aProp = new ProprietaireCollection();
      foreach ($aProp as $prop) {
        $this->aProp->add($prop);
      }
      return $this->aProp;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot get proprietaire list for SufBnd ', [$this, $e]);
      return [];
    }
  }

  /**
   * Cette méthode permet de récupérer les Proprietaires
   * @return ProprietaireCollection|null
   */
  public function getProprietaireCollection(): ?ProprietaireCollection {
    return $this->fillProprietaireArray();
  }

  /**
   * Cette méthode permet de construire un tableau correspondant au SufBnd dans le relevé de propriété
   * {@inheritDoc}
   */
  public function getMajicData(): array{
    $aData = parent::getMajicData();

    $aData['aProp'] = (!empty($this->dnulot)) ? $this->getProprietaireCollection()->getMajicData() : [];
    $aData['num_voirie'] = (!empty($this->dnulot)) ? '*****' : null;
    $aData['dnulot'] = (!empty($this->dnulot)) ? $this->dnulot : null;
    $aData['dnupdl'] = (!empty($this->dnulot)) ? $this->dnupdl : null;
    $aData['dnumql'] = (!empty($this->dnulot)) ? $this->dnumql : null;
    $aData['ddenql'] = (!empty($this->dnulot)) ? $this->ddenql : null;
    return $aData;
  }
}
