<?php

namespace App\Module\Cadastre\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractGeomEntity;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour la vue "cadastre_views.parcelle" dans les properties.
 *
 * @author Alizée Buckel <alizee.buckel@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Cadastre\WebService\Repository\GeometryRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(cadastre_schema_cadastre)",name="@prop(cadastre_views:parcelle)")
 */
class Parcelle extends AbstractGeomEntity {
  /**
   * @var string
   * @Column(type="string", name="id_par")
   * @Id
   */
  private $id_par;

  /**
   * @var string
   * @Column(type="string", name="id_com")
   */
  private $id_com;

  /**
   * @var string
   * @Column(type="string", name="id_sec")
   */
  private $id_sec;

  /**
   * @var string
   * @Column(type="string", name="commune")
   */
  private $commune;

  /**
   * @var string
   * @Column(type="string", name="parcelle")
   */
  private $parcelle;

  /**
   * @var string
   * @Column(type="string", name="section")
   */
  private $section;

  /**
   * @var string
   * @Column(type="string", name="pre")
   */
  private $pre;

  /**
   * @var string|null
   * @Column(type="string", name="coar")
   */
  private $coar;

  /**
   * @var string
   * @Column(type="string", name="indp")
   */
  private $indp;

  /**
   * @var integer
   * @Column(type="integer", name="sup_fiscale")
   */
  private $sup_fiscale;

  /**
   * @var string
   * @Column(type="string", name="feuille")
   */
  private $feuille;

  /**
   * @var string
   * @Column(type="geometry", name="geom")
   */
  private $geom;

  /**
   * @return string
   */
  public function getIdPar(): string {
    return $this->id_par;
  }

  /**
   * @return string
   */
  public function getIdCom(): string {
    return $this->id_com;
  }

  /**
   * @return string
   */
  public function getSec(): string {
    return $this->id_sec;
  }

  /**
   * @return string
   */
  public function getCodeCom(): string {
    return substr($this->id_com, 2, 3);
  }

  /**
   * @return string
   */
  public function getCommune(): string {
    return $this->commune;
  }

  /**
   * @return string
   */
  public function getParcelle(): string {
    return $this->parcelle;
  }

  /**
   * @return string
   */
  public function getSection(): string {
    return $this->section;
  }

  /**
   * @return string
   */
  public function getPre(): string {
    return $this->pre;
  }

  /**
   * @return string|null
   */
  public function getCoar(): ?string {
    return $this->coar;
  }

  /**
   * @return string
   */
  public function getIndp(): string {
    return $this->indp;
  }

  /**
   * @return string
   */
  public function getIdu(): string {
    return substr($this->id_par, 2, 12);
  }

  /**
   * @return integer
   */
  public function getSupFiscale(): int {
    return $this->sup_fiscale;
  }

  /**
   * @return string
   */
  public function getFeuille(): string {
    return $this->feuille;
  }

  /**
   * @return string geometry ewkt 2154
   */
  public function getGeom(): string {
    return $this->getReprojectedGeomForAPI($this->geom);
  }

}
