<?php

namespace App\Module\Cadastre\WebService\Repository;

use App\Module\Cadastre\WebService\Collection\EmpriseCollection;
use App\Module\Cadastre\WebService\Entity\Majic\Emprise;
use App\Module\Vitis\WebService\Repository\GenericRepository;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Mapping\ClassMetadata;

class EmpriseRepository extends GenericRepository {
  /**
   * {@inheritdoc}
   */
  public function __construct(EntityManagerInterface $em, ClassMetadata $class) {
    parent::__construct($em, $class);
  }

  /**
   * @param string $sIdPar id_par de BatiParcelle
   * @param array|null $aIntersect
   * @param int $iTolerance
   * @return array|null
   * Cette méthode permet d'obtenir les emprises collections en fonction de l'id_par
   */
  public function findByIdPar($sIdPar, ?array $aIntersect, int $iTolerance): ?array{
    if (empty($aIntersect)) {
      return [];
    }
    $aEmprise = parent::findById($sIdPar);
    if (empty($aEmprise)) {
      return [];
    }
    $aFinalCollection = [];
    $oEmprise = $aEmprise[0];
    foreach ($aIntersect as $oIntersect) {
      $aCollection = new EmpriseCollection();
      $aCollection->setDefinition($oIntersect);
      $aData = $this->getNativeDataForEmprise($oIntersect, $oEmprise);
      foreach ($aData as $oRow) {
        if (strtoupper($oIntersect['type']) != 'POINT' && round($oRow['intersect']) >= $iTolerance) {
          $oEmpriseTmp = clone $oEmprise;
          $oEmpriseTmp->setDefinition($oIntersect);
          $oEmpriseTmp->setAttribute($oRow);
          $aCollection->add($oEmpriseTmp);
        }
      }
      $aFinalCollection[] = $aCollection;
    }
    return $aFinalCollection;
  }

  /**
   * @param array $oIntersect
   * @param Emprise $oEmprise
   * @return array|null
   * Cette méthode permet d'obtenir les intersections
   */
  private function getNativeDataForEmprise(array $oIntersect, Emprise $oEmprise): ?array{
    $sView = $oIntersect['schema'] . '.' . $oIntersect['table'];
    $sFunction = 'st_area2d';
    switch (strtoupper($oIntersect['type'])) {
      case "LINE":
      case "LINESTRING":
      case "MULTILINESTRING":
        $sFunction = "st_length2d";
        break;
    }
    $qb = $this->getEntityManager()->getConnection()->createQueryBuilder();
    $qb->select('st_srid(a.geom) as "viewsrid"')
      ->from($sView, 'a')
      ->setMaxResults(1);
    try {
      $resultSrid = $qb->execute()->fetchAllAssociative();
      $sIntersectGeomSql = 'st_transform(b.geom, st_srid(a.geom))';
      if (!empty($resultSrid)) {
        if ($resultSrid[0]['viewsrid'] === $oEmprise->getSrid()) {
          $sIntersectGeomSql = 'b.geom';
        }
      }
      $qbData = $this->getEntityManager()->getConnection()->createQueryBuilder();
      $qbData->select('CAST(' . $sFunction . '(st_intersection(a.geom, ' . $sIntersectGeomSql . ' )) AS DECIMAL(10,1)) AS "intersect", b.*')
        ->from('s_cadastre.v_vmap_parcelle', 'a')
        ->from($sView, 'b')
        ->where('st_intersects(a.geom, ' . $sIntersectGeomSql . ') AND a.id_par = :IdPar')
        ->setParameter('IdPar', $oEmprise->getIdPar());
      $result = $qbData->execute()->fetchAllAssociative();
    } catch (\Exception $e) {
      error_log('EmpriseRepository : can\'t get geometry for Emprise : ' . $e->getMessage());
      $result = null;
    }
    return $result;
  }
}
