<?php

namespace App\Module\Cadastre\WebService\Repository;

use App\Module\Vitis\WebService\Repository\GenericRepository;
use App\Module\Cadastre\WebService\Collection\ParcelleCollection;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Mapping\ClassMetadata;

class ParcelleRepository extends GenericRepository {
  /**
   * {@inheritdoc}
   */
  public function __construct(EntityManagerInterface $em, ClassMetadata $class) {
    parent::__construct($em, $class);
  }

  /**
   * @param string $sDnupro dnupro de CompteCommunal
   * @param string $sIdCom ccodep et ccocom de CompteCommunal
   * @return array|null
   * Cette méthode permet d'obtenir les parcelles par id_par
   */
  public function findByIdPar($sDnupro, $sIdCom): ?array{
    $qb = $this->getEntityManager()->getConnection()->createQueryBuilder();
    $qb->distinct()
      ->select('a.id_par')
      ->from('s_majic.nb_10_parcelle', 'a')
      ->leftJoin('a', 's_majic.nb_21_suf_descr', 'b', 'a.id_par = b.id_par')
      ->where('(a.dnupro=:sDnupro OR b.dnupro=:sDnupro) AND b.id_com=:sIdCom')
      ->orderBy('a.id_par')
      ->setParameter('sDnupro', $sDnupro, 'string')
      ->setParameter('sIdCom', $sIdCom, 'string');
    try {
      $result = $qb->execute()->fetchAllAssociative();
      return $this->formatResult($result);
    } catch (\Exception $e) {
      error_log('CompteCommunal : can\'t get id_par for parcelle : ' . $e->getMessage());
      return null;
    }
  }

  /**
   * @param string $sIdDnupro id_dnupro de CompteCommunal
   * @param string $sDnupro substr(id_dnupro) de CompteCommunal
   * @param string $sIdCom ccodep et ccocom de CompteCommunal
   * @return array|null
   * Cette méthode permet d'obtenir les parcelles par id_parBnd
   */
  public function findByIdParBnd($sIdDnupro, $sDnupro, $sIdCom): ?array{
    $qb = $this->getEntityManager()->getConnection()->createQueryBuilder();
    $qb->select('b.id_par')
      ->from('s_majic.nb_21_suf_descr', 'a')
      ->innerJoin('a', 's_majic.nb_10_parcelle', 'b', 'a.id_par = b.id_par')
      ->where('a.dnupro=:sDnupro AND b.id_dnupro!=:sIdDnupro AND b.id_com=:sIdCom')
      ->setParameter('sIdDnupro', $sIdDnupro, 'string')
      ->setParameter('sDnupro', $sDnupro, 'string')
      ->setParameter('sIdCom', $sIdCom, 'string');
    try {
      $result = $qb->execute()->fetchAllAssociative();
      return $this->formatResult($result);
    } catch (\Exception $e) {
      error_log('CompteCommunal : can\'t get id_par for parcelle Bnd : ' . $e->getMessage());
      return null;
    }
  }

  /**
   * @param string $sIdDnupro id_dnupro de CompteCommunalParcelle
   * @param string $sIdPar ccodep et ccocom de CompteCommunalParcelle
   * @return ParcelleCollection|null
   * Cette méthode permet d'obtenir les parcelles pour CompteCommunalParcelle
   */
  public function findByIdDnuproIdPar($sIdDnupro, $sIdPar): ?ParcelleCollection{
    $qb = $this->createQueryBuilder('a');
    $qb->where('a.id_dnupro=:IdDnupro AND a.id_par=:IdPar')
      ->setParameter('IdDnupro', $sIdDnupro)
      ->setParameter('IdPar', $sIdPar);
    return new ParcelleCollection($qb->getQuery()->getResult());
  }

  /**
   * @param array $Tab
   * @return array
   * Cette méthode permet de formatter le resultat
   */
  private function formatResult(array $Tab): array{
    $aIdPar = [];
    foreach ($Tab as $idPar) {
      $aIdPar[] = $idPar['id_par'];
    }
    return $aIdPar;
  }
}
