// Fichier contenant les variables communes pour les formulaires

export const optionYesNoRadio = {
  'choices': [{
    'label': 'FORM_YES',
    'value': true
  },
  {
    'label': 'FORM_NO',
    'value': false
  }]
};

/**
 * Render d'une checkbox pour afficher/cacher une couche
 * @param data
 * @param type
 * @param row
 * @param meta
 */
export const renderVisibleLayerColumn = (data: any, type: any, row: any, meta: any) => {
  if (!row.onError) {
    const checked = data === true ? 'checked' : '';
    return '<div class="checkbox checkbox-info margin-2 col-12" style="position: relative">' +
      '<input id="visible_layer_' + row.layer_grid_id + '" ' +
      'type="checkbox" ' +
      'style="cursor:pointer; position: absolute;" ' +
      checked +
      '/>' +
      '<label for="visible_layer_' + row.layer_grid_id + '" class="checkbox_label" style="cursor:pointer"></label>' +
      '</div>';
  }
  return '';
};

/**
 * Affiche les symboles en fonction du status de la couche
 * row.isLoaded => La couche est chargé
 * row.onError => La couche est chargé mais n'est plus dans le flux.
 * @param data
 * @param type
 * @param row
 * @param meta
 */
export const renderSymbolesLayerColumn = (data: any, type: any, row: any, meta: any) => {
  let sReturn = '';
  if (row.isLoaded === true) {
    sReturn += '<span class="fas fa-check text-success"></span>';
  }
  if (row.onError === true) {
    sReturn += '<span class="fas fa-times text-error"></span>';
  }
  return sReturn;
};

/**
 * Map option, sans proj, center et view.extent, avec openstreetmap
 */
export const defaultEmptyMapOptions = {
  'type': 'vmap',
  'controls': {
    'mouse_position': true,
    'zoom_controls': true,
    'scale_line': true,
    'map_projection': true
  },
  'interactions': {
    'multi_geometry': false,
    'full_screen': true,
    'remove_all': false,
    'remove': false,
    'modify': false,
    'draw_point': false,
    'draw_line': false,
    'draw_polygon': false
  },
  'layers': [],
  'draw_color': 'rgba(54,184,255,0.6)',
  'contour_color': 'rgba(0,0,0,0.4)',
  'contour_size': 2,
  'circle_radius': 6,
  'features': [],
  'coord_accuracy': 8,
  'tree': {
    'name': 'Tree',
    'children': [
      {
        'view': {
          'zoom': 6,
          'maxZoom': 28,
          'minZoom': 0,
          'extent': [
            -1304719.742131629,
            5123230.307147879,
            1615938.3074785857,
            6823759.011433303
          ]
        }
      },
      {
        'name': 'Open Street Map',
        'children': [
          {
            'name': 'Open Street Maps',
            'visible': true,
            'layerType': 'osm',
            'url': 'https://{a-c}.tile.openstreetmap.org/{z}/{x}/{y}.png',
            'select': true,
            'index': 0
          }
        ],
        'visible': true
      }
    ]
  }
};
