import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';
declare let $: any;

export class DatasourceTreeviewPgTableMapFormJS {

  private oObject: any;
  // Services
  private vitisCheck: VitisCheck;
  private mapCoreManagerService: MapCoreManagerService;

  /**
   * Constructeur lancée au chargement du formulaire
   * @param {object} oFormReader Formreader
   * @param {object} oObject objet de l'arbre dans lequel il y a les fonctions nécessaires
   * @param {object} injector
   */
  constructor(
    private oFormReader: FormreaderComponent,
    oObject: any,
    injector: any
  ) {
    this.oObject = oObject;

    this.vitisCheck = injector.get(VitisCheck);
    this.mapCoreManagerService = injector.get(MapCoreManagerService);
  }

  /**
   * Evenement lancé par AdminDatasourceFormComponent juste avant l'envoi du formulaire
   */
  public prepareData(): Promise <boolean> {
    return new Promise((resolve, reject) => {
      this.mapCoreManagerService.getProjections().subscribe((oProjections: any) => {
        const oMapField = this.oFormReader.getFormField('table_map_layers');
        // Vérifie la présence des informations requises
        if (this.vitisCheck.isDefAndNotNull(this.oObject.oTableInfo)
          && this.vitisCheck.isDefAndNotNull(this.oObject.aValues)
          && this.vitisCheck.isDefAndNotNull(this.oObject.oTableInfo.geom_column_finded)) {

          let sProj = oProjections.projections[0].code;
          // Si possible utilise le srid de la table
          if (this.vitisCheck.isDefAndNotNull(this.oObject.oTableInfo.srid) && this.oObject.oTableInfo.srid != 0) {
            const projIndex = oProjections.projections.findIndex((proj) => proj.srid == this.oObject.oTableInfo.srid.toString());
            if (projIndex !== -1) {
              sProj = oProjections.projections[projIndex].code;
            }
          }
        }
      });
    });
  }
}
