import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { vitisAlert, VitisCheck, VitisClone, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { WmsService } from 'src/modules/vmap/components/admin-datasource/services/wms.service';
import { WMSLayerRequireData } from 'src/app/modules/vmap/admin-datasource/models/DatasourceTreeviewWMS/WMSLayer.model';
import { environment } from 'src/environments/environment';
import { TranslateService } from '@ngx-translate/core';
import { AdminDatasourceService } from '../../services/admin-datasource.service';
import { DatasourceTreeviewWMSLayer } from '../../models/DatasourceTreeviewWMS/DatasourceTreeviewWMSLayer';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';
import { MapCoreFormreaderService } from 'src/app/vitis/map-core/services/map-core-fromreader/map-core-formreader.service';
declare let $: any;

export class DatasourceTreeviewWMSConnectionLayersFormJS {

  private oObject: any;
  // Services
  private vitisCheck: VitisCheck;
  private vitisClone: VitisClone;
  private mapCoreManagerService: MapCoreManagerService;
  private mapCoreFormreaderService: MapCoreFormreaderService;
  private wmsService: WmsService;
  private vitisRequest: VitisRequest;
  private translate: TranslateService;
  private adminDatasourceService: AdminDatasourceService;

  private oDatagridField;

  /**
   * Constructeur lancée au chargement du formulaire
   * @param {object} oFormReader Formreader
   * @param {object} oObject objet de l'arbre dans lequel il y a les fonctions nécessaires
   * @param {object} injector
   */
  constructor(
    private oFormReader: FormreaderComponent,
    oObject: any,
    injector: any
  ) {
    this.oObject = oObject;

    this.vitisCheck = injector.get(VitisCheck);
    this.vitisClone = injector.get(VitisClone);
    this.mapCoreManagerService = injector.get(MapCoreManagerService);
    this.mapCoreFormreaderService = injector.get(MapCoreFormreaderService);
    this.wmsService = injector.get(WmsService);
    this.vitisRequest = injector.get(VitisRequest);
    this.translate = injector.get(TranslateService);
    this.adminDatasourceService = injector.get(AdminDatasourceService);

  }

  /**
   * Evenement lancé par AdminDatasourceFormComponent juste avant l'envoi du formulaire
   */
  public prepareData(): Promise<boolean> {
    return new Promise((resolve, reject) => {
      if (this.vitisCheck.isDefAndNotNull(this.oObject) && this.vitisCheck.isDefAndNotNull(this.oObject.oCapabilities)) {
        const oMapField = this.oFormReader.getFormField('wms_map_layers');
        this.oDatagridField = this.oFormReader.getFormField('wms_datagrid_layers');

        const oCapabilities = this.oObject.oCapabilities;
        if (this.vitisCheck.isDefAndNotNull(oCapabilities) &&
          this.vitisCheck.isDefAndNotNull(oCapabilities.json) &&
          this.vitisCheck.isDefAndNotNull(oCapabilities.json.Capability) &&
          this.vitisCheck.isDefAndNotNull(oCapabilities.json.Capability.Layer)) {

          const oCapabilitiesLayers = oCapabilities.json.Capability.Layer;

          // Récupère la liste des couches
          const aLayers = this.wmsService.searchLayers(oCapabilitiesLayers);

          // Récupère la projection des couches
          this.getMapProjFromWMSLayers(aLayers).then((sProj: string) => {

            // Prépare les données pour la carte et la grille à partir des couches du flux WMS.
            const oData = this.prepareLayers_(aLayers);

            // Suppression service si existe déjà
            const iChildIndex = oMapField.map_options.tree.children.findIndex((oChild: any) => oChild.name === oData.oChildrenService.name);
            if (iChildIndex !== -1) {
              oMapField.map_options.tree.children.splice(iChildIndex, 1);
            }

            // Ajout du service
            oMapField.map_options.tree.children.push(oData.oChildrenService);

            setTimeout(() => {
              if (this.vitisCheck.isDefAndNotNull(this.oDatagridField)) {
                this.oDatagridField.setValue(oData.aDatagridValues);
                // init event
                this.setEventDatagrid_(this.oDatagridField, oMapField);
                // Refresh Components
                oMapField.refreshMap();
                this.oDatagridField.refreshGrid();
              }
            });
          });
        }
      }
    });
  }

  /**
   * Prépare les couches pour la carte et les données pour la grille
   * @param aLayers
   * @returns
   */
  private prepareLayers_(aLayers: Array<any>): any {
    const oChildrenService = {
      'name': 'Service WMS',
      'children': []
    };

    const aDatagridValues = [];
    aLayers = this.mergeLayers(aLayers);
    for (let i = 0; i < aLayers.length; i++) {
      const oLayerData = aLayers[i];
      const oDatagridValues = this.createDataGridLine_(oLayerData);
      aDatagridValues.push(oDatagridValues);
      // Ajout la couche à la carte que si la couche est dans les capabilities
      if (oLayerData.bIsInCapabilities !== false) {
        const oChildrenLayer = this.createWMSLayer_(oLayerData, i);
        oChildrenService.children.push(oChildrenLayer);
      }
    }

    return {
      oChildrenService: oChildrenService,
      aDatagridValues: aDatagridValues
    };
  }

  /**
   * Créer une ligne pour la grille
   * @param oData
   */
  private createDataGridLine_(oData: WMSLayerRequireData) {
    return {
      layer_grid_id: oData.layerGridId,
      layer_id: oData.layerId,
      name: this.vitisCheck.isDefAndNotNull(oData.oLayer.Name) ? oData.oLayer.Name : '',
      title: this.vitisCheck.isDefAndNotNull(oData.oLayer.Title) ? oData.oLayer.Title : '',
      visible: oData.bVisible,
      style: oData.oStyle.Title ? oData.oStyle.Title : (oData.oStyle.Name ? oData.oStyle.Name : ''),
      style_value: oData.oStyle.Name,
      format: oData.sFormat,
      crs_list: oData.crs_list,
      isLoaded: oData.bIsLoaded,
      onError: (oData.bIsLoaded && !oData.bIsInCapabilities) // Couche chargé qui n'existe plus dans le getCapabilities
    };
  }

  /**
   * Créer une couche wms
   * @param oData
   * @param index
   */
  private createWMSLayer_(oData: WMSLayerRequireData, index: number) {
    // Définition de la couche.
    return {
      'layer_grid_id': oData.layerGridId,
      'name': oData.oLayer.Name,
      'layerType': 'imagewms',
      'visible': oData.bVisible,
      'select': true,
      'url': this.oObject.flux_url,
      'params': {
        'LAYERS': oData.oLayer.Name,
        'VERSION': '1.3.0',
        'STYLES': oData.oStyle.Name,
        'TIMESTAMP': new Date().getTime()
      },
      'service_options': this.oObject.flux_options,
      'service_login': this.oObject.flux_login,
      'service_password': this.oObject.flux_password,
      'index': index
    };
  }

  /**
   * Initie les évenements sur la grille qui auront un impacte sur la carte
   * @param oDatagridField
   * @param oMapField
   */
  private setEventDatagrid_(oDatagridField: any, oMapField: any) {
    const self = this;
    // Attend que oDataGrid soit instancié
    setTimeout(() => {
      const updateMap = (oDatas) => {
        // Evenement pour modifier la carte
        self.mapCoreFormreaderService.emitMapFormreaderEvent(oMapField.id, {
          action: 'setVisibility',
          layerGridId: oDatas.layer_grid_id,
          visible: oDatas.visible
        });
      };

      this.oDatagridField.oDataGrid.deselectRowEvent.subscribe((oDatas) => {
        oDatas.visible = false;
        updateMap(oDatas);
      });

      this.oDatagridField.oDataGrid.selectRowEvent.subscribe((oDatas) => {
        oDatas.visible = true;
        updateMap(oDatas);
      });
    });
  }

  /**
   * Récupère la projection de la carte à partir de la liste des couches
   * @param aLayers
   */
  public getMapProjFromWMSLayers(aLayers: any): Promise<any> {
    return new Promise((resolve, reject) => {
      this.mapCoreManagerService.getProjections().subscribe((oProjections: any) => {
        // Récupère l'index de projection à utiliser (en priorité le crs qui a le meilleur index dans la couche)
        let iUsedProjectionIndex = 0;
        let iLastCrsIndex: number;
        let iLastLayerCrsIndex: number;
        for (let i = 0; i < oProjections.projections.length; i++) {
          let bIsAllowedProj = true;
          iLastCrsIndex = null;
          for (let ii = 0; ii < aLayers.length; ii++) {
            const aCrs = this.vitisCheck.isArray(aLayers[ii].CRS) ? aLayers[ii].CRS : this.vitisCheck.isArray(aLayers[ii].SRS) ? aLayers[ii].SRS : [];
            const iCrsIndex = aCrs.indexOf(oProjections.projections[i].code);
            if (iCrsIndex === -1) {
              bIsAllowedProj = false;
            } else if (!this.vitisCheck.isDefAndNotNull(iLastCrsIndex) || iLastCrsIndex > iCrsIndex) {
              iLastCrsIndex = iCrsIndex;
            }
          }
          if (bIsAllowedProj && (!this.vitisCheck.isDefAndNotNull(iLastLayerCrsIndex) || iLastLayerCrsIndex > iLastCrsIndex)) {
            iUsedProjectionIndex = this.vitisClone.clone(i);
            iLastLayerCrsIndex = iLastCrsIndex;
          }
        }
        // Par sécurité si la valeur vaut -1 on prend l'element 0
        if (iUsedProjectionIndex === -1) {
          iUsedProjectionIndex = 0;
        }
        resolve(oProjections.projections[iUsedProjectionIndex].code);
      });
    });
  }


  /**
   * Merge la liste des couches avec la liste des couches chargées
   * @param aLayers
   */
  private mergeLayers(aLayers: Array<any>): Array<any> {
    const aWMSLoadedLayers: Array<DatasourceTreeviewWMSLayer> = this.oObject.getChildrens();
    const aMergedLayers: Array<WMSLayerRequireData> = [];

    for (let i = 0; i < aLayers.length; i++) {
      const oLayer = aLayers[i];
      const oData: WMSLayerRequireData = {
        layerGridId: this.getUniqueLayerGridId_(aMergedLayers),
        oLayer,
        bVisible: false,
        sFormat: null,
        oStyle: null,
        bIsLoaded: false,
        bIsInCapabilities: true
      };
      const layerCharged = aWMSLoadedLayers.find(layer => layer.layer_name === oLayer.Name);
      if (layerCharged) {
        oData.layerGridId = i.toString();
        oData.oStyle = {
          Name: layerCharged.layer_style
        };
        oData.sFormat = layerCharged.layer_format;
        oData.bIsLoaded = true;
        oData.layerId = layerCharged.layer_id;
        aMergedLayers.push(oData);
      } else {
        const aCrs = this.vitisCheck.isArray(oLayer.CRS) ? oLayer.CRS : this.vitisCheck.isArray(oLayer.SRS) ? oLayer.SRS : [];
        const sCrs = aCrs.join('|');
        oData.crs_list = sCrs;

        const combinaisons = this.adminDatasourceService.buildCombinationsObjects({ label: 'style', value: oLayer.Style || [] },
          { label: 'format', value: oLayer.sFormat || [] });

        combinaisons.forEach((combination) => {
          const currentCombination = { ...oData };
          if (this.vitisCheck.isDefAndNotNull(combination.style)) {
            currentCombination.oStyle = { Name: combination.style.Name };
          }
          if (this.vitisCheck.isDefAndNotNull(combination.format)) {
            currentCombination.sFormat = combination.format;
          }
          aMergedLayers.push(currentCombination);
        });
      }
    }
    return aMergedLayers;
  }

  /**
   * Supprime les couches de vmap
   */
  public deleteLayers() {
    const aDeletedValues = this.oDatagridField.oDataGrid.getSelectedRows_();
    const aDeletedLayerIds = [];
    for (let i = 0; i < aDeletedValues.length; i++) {
      const oDeletedValue = aDeletedValues[i];
      if (oDeletedValue.layer_id) {
        aDeletedLayerIds.push(oDeletedValue.layer_id);
      }
    }

    if (aDeletedLayerIds.length > 0) {
      this.sendRequestGetMap_(aDeletedLayerIds).then((aLayers) => {
        let sHtml = '';
        for (let i = 0; i < aLayers.length; i++) {
          const oLayer = aLayers[i];
          if (this.vitisCheck.isDefAndNotNull(oLayer.layer_id) && this.vitisCheck.isDefAndNotNull(oLayer.map) && oLayer.map.length > 0) {
            sHtml += '<div>';
            sHtml += '<p>' + oLayer.name + ' (' + oLayer.layer_id + ') :';
            for (let ii = 0; ii < oLayer.map.length; ii++) {
              const oMap = oLayer.map[ii];
              sHtml += ' <span>' + oMap.name + ' (' + oMap.map_id + ')</span>';
            }
            sHtml += '</p>';

            sHtml += '</div>';
          }
        }
        // Si au moins une couche sélectionné fait partie d'une carte on demande une confirmation de suppression
        if (sHtml !== '') {
          this.translate.get('DATASOURCE_TREEVIEW_FORM_LAYER_IN_MAP').subscribe((sTranslate: string) => {
            sHtml = '<p>' + sTranslate + '</p>' + sHtml;
            vitisAlert.alert({
              'type': 'warning',
              'width': '800px',
              'html': sHtml,
              'showCancelButton': true
            }).then((result) => {
              if (result.isConfirmed) {
                this.sendRequestDeleteLayers_(aDeletedLayerIds);
              }
            });
          });
        } else {
          this.sendRequestDeleteLayers_(aDeletedLayerIds);
        }
      });
    } else {
      this.translate.get('LIST_DELETE_NO_SELECTION_IMPORTED_LAYERS').subscribe((sTranslate: string) => {
        vitisAlert.fireToast({
          type: 'warning',
          title: sTranslate
        });
      });
    }
  }

  /**
   * Importe les couches du flux dans vmap
   */
  public addLayers() {
    const aSelectedValues = this.oDatagridField.oDataGrid.getSelectedRows_();
    const aInsertValues = [];
    const layerOptions = JSON.stringify(this.oObject.oCapabilities.json);
    for (let i = 0; i < aSelectedValues.length; i++) {
      const oSelectedValues = aSelectedValues[i];
      if (!oSelectedValues.layer_id) {

        let name = oSelectedValues.name + ' (' + oSelectedValues.style;
        name += oSelectedValues.format ? ' ' + oSelectedValues.format : '';
        name += ')';

        aInsertValues.push({
          datasource_id: this.oObject.datasourceId,
          name: name.length > 100 ? name.substring(0, 97) + '...' : name,
          theme_id: null,
          source_id: null,
          active: true,
          is_background_layer: false,
          layer_name: oSelectedValues.name,
          layer_style: oSelectedValues.style_value,
          layer_format: oSelectedValues.format || 'image/png',
          crs_list: oSelectedValues.crs_list,
          layer_options: layerOptions,
          is_tiled: false
        });
      }
    }

    if (aInsertValues.length > 0) {
      this.sendRequestAddLayers_(aInsertValues);
    } else if (aSelectedValues.length > 0) {
      this.translate.get('LIST_INSERT_SELECTION_ALLREADY_INSERT_LAYERS').subscribe((sTranslate: string) => {
        vitisAlert.fireToast({
          icon: 'warning',
          title: sTranslate
        });
      });
    } else {
      this.translate.get('LIST_INSERT_NO_SELECTION_LAYERS').subscribe((sTranslate: string) => {
        vitisAlert.fireToast({
          icon: 'warning',
          title: sTranslate
        });
      });
    }
  }

  /**
   * Lance la requête de suppression
   * @param aDatas
   */
  private sendRequestAddLayers_(aDatas: Array<any>): Promise<any> {
    return new Promise((resolve, reject) => {
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/wmslayers/adds`;

      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        data: {
          data: aDatas
        },
        method: 'POST'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            this.translate.get('LIST_ADD_OK').subscribe((sTranslate: string) => {
              // Rafraichi la liste des enfants
              this.oObject.loadChildrens().then((aChildrens) => {
                this.adminDatasourceService.setChildrensMetadata(aChildrens, this.oObject);
                this.adminDatasourceService.refreshRightComponent(this.oObject.uId);
              });

              vitisAlert.fireToast({
                type: 'success',
                title: sTranslate
              });
            });
            resolve(oResult.data);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Lance la requête de suppression
   * @param aDeletedLayerIds
   */
  private sendRequestDeleteLayers_(aDeletedLayerIds: Array<string>): Promise<any> {
    return new Promise((resolve, reject) => {
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/wmslayers`;

      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          idList: aDeletedLayerIds.join('|')
        },
        method: 'DELETE',
        displayErrorMessage: true
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            this.translate.get('LIST_DELETE_OK').subscribe((sTranslate: string) => {
              // Rafraichi la liste des enfants
              this.oObject.loadChildrens().then((aChildrens) => {
                this.adminDatasourceService.setChildrensMetadata(aChildrens, this.oObject);
                this.adminDatasourceService.refreshRightComponent(this.oObject.uId);
              });

              vitisAlert.fireToast({
                type: 'success',
                title: sTranslate
              });
            });
            resolve(oResult.data);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Lance une requête pour vérifier que les couches ne soit pas inclus dans une carte
   * @param aDeletedLayerIds
   */
  private sendRequestGetMap_(aDeletedLayerIds: Array<string>): Promise<any> {
    return new Promise((resolve, reject) => {
      const sUrl = `${environment.properties.app_api_url}/vmap/layers`;

      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          filter: { 'relation': 'AND', 'operators': [{ 'column': 'layer_id', 'compare_operator': 'IN', 'value': aDeletedLayerIds }] },
          attributs: 'layer_id|name|map'
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            resolve(oResult.data);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Trouve un identifiant unique à une ligne de la grille
   * @returns string
   */
  private getUniqueLayerGridId_(aLayerGrid: Array<WMSLayerRequireData>): string {
    let id = '_' + Math.random().toString(36).substring(2, 11);
    if (aLayerGrid.findIndex((layer: any) => layer.layerGridId == id) !== -1) {
      id = this.getUniqueLayerGridId_(aLayerGrid);
    }
    return id;
  }
}
