import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { WmsService } from 'src/modules/vmap/components/admin-datasource/services/wms.service';
import { VitisCheck, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { defaultEmptyMapOptions } from 'src/app/modules/vmap/admin-datasource/forms/DatasourceTreeviewFormVar';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';
import { MapCoreFormreaderService } from 'src/app/vitis/map-core/services/map-core-fromreader/map-core-formreader.service';

export class DatasourceTreeviewWMSLayerMapFormJS {

  private oObject: any;
  // Services
  private vitisCheck: VitisCheck;
  private vitisClone: VitisClone;
  private mapCoreManagerService: MapCoreManagerService;
  private mapCoreFormreaderService: MapCoreFormreaderService;
  private wmsService: WmsService;

  /**
   * Constructeur lancée au chargement du formulaire
   * @param {object} oFormReader Formreader
   * @param {object} oObject objet de l'arbre dans lequel il y a les fonctions nécessaires
   * @param {object} injector
   */
  constructor(
    private oFormReader: FormreaderComponent,
    oObject: any,
    injector: any
  ) {
    this.oObject = oObject;

    this.vitisCheck = injector.get(VitisCheck);
    this.vitisClone = injector.get(VitisClone);
    this.mapCoreManagerService = injector.get(MapCoreManagerService);
    this.mapCoreFormreaderService = injector.get(MapCoreFormreaderService);
    this.wmsService = injector.get(WmsService);
  }

  /**
   * Evenement lancé par AdminDatasourceFormComponent juste avant l'envoi du formulaire
   */
  public prepareData() {
    const oMapField = this.oFormReader.getFormField('wms_map_layer');
    // Réinitialise la carte
    oMapField.map_options = this.vitisClone.clone(defaultEmptyMapOptions);

    this.mapCoreManagerService.getProjections().subscribe((oProjections: any) => {
      if (this.vitisCheck.isDefAndNotNull(this.oObject.crs_list) &&
        this.vitisCheck.isDefAndNotNull(this.oObject.layer_name)) {

        // Ajoute la projection à la définition du champs
        const sProj = oProjections.projections[0].code;

        const aCrs = this.oObject.crs_list.split('|');
        let iUsedProjectionIndex = 0;
        let iLastCrsIndex: number;
        for (let i = 0; i < oProjections.projections.length; i++) {
          const iCrsIndex = aCrs.indexOf(oProjections.projections[i].code);
          if (iCrsIndex !== -1) {
            if (!this.vitisCheck.isDefAndNotNull(iLastCrsIndex) || iLastCrsIndex > iCrsIndex) {
              iUsedProjectionIndex = this.vitisClone.clone(i);
              iLastCrsIndex = iCrsIndex;
            }
          }
        }

        // Prépare les données pour la carte et la grille à partir des couches du flux WMS.
        const oService = this.prepareLayer_();

        // Ajout du service
        oMapField.map_options.tree.children.push(oService);

        // Rafraichi la carte
        this.mapCoreFormreaderService.emitMapFormreaderEvent(oMapField.id, {
          action: 'reload'
        });

        const oLayerOptionsField = this.oFormReader.getFormField('layer_options');
        if (this.vitisCheck.isDefAndNotNull(oLayerOptionsField)) {
          const oJsonCapabilities = oLayerOptionsField.getValue();
          if (this.vitisCheck.isDefAndNotNull(oJsonCapabilities) &&
            this.vitisCheck.isDefAndNotNull(oJsonCapabilities.Capability) &&
            this.vitisCheck.isDefAndNotNull(oJsonCapabilities.Capability.Layer)) {

            const oCapabilitiesLayers = oJsonCapabilities.Capability.Layer;

            // Récupère la liste des couches dans le getCapabilities
            const aLayers = this.wmsService.searchLayers(oCapabilitiesLayers);
            // Récupère la couche
            const oLayer = aLayers.find((layer: any) => layer.Name === this.oObject.layer_name);
            const oBoundingBox = oLayer.BoundingBox.find((bbox: any) => bbox.crs === oMapField.map_options.proj);

            if (this.vitisCheck.isDefAndNotNull(oBoundingBox)) {
              setTimeout(() => {
                // Evenement pour modifier la carte
                this.mapCoreFormreaderService.emitMapFormreaderEvent(oMapField.id, {
                  action: 'zoomOnExtent',
                  extent: oBoundingBox.extent
                });
              });
            }
          }
        }
      }
    });
  }

  /**
   * Prépare la couche pour la carte
   * @returns
   */
  private prepareLayer_(): any {
    const oChildrenService = {
      'name': 'Service WMS',
      'children': [{
        'layer_id': this.oObject.layer_id,
        'name': this.oObject.layer_name,
        'layerType': 'imagewms',
        'visible': true,
        'select': true,
        'url': this.oObject.oParent.flux_url,
        'params': {
          'LAYERS': this.oObject.layer_name,
          'VERSION': '1.3.0',
          'STYLES': this.vitisCheck.isDefAndNotNull(this.oObject.layer_style) ? this.oObject.layer_style : '',
          'TIMESTAMP': new Date().getTime()
        },
        'service_options': this.oObject.oParent.flux_options,
        'service_login': this.oObject.oParent.flux_login,
        'service_password': this.oObject.oParent.flux_password,
        'headers': this.oObject.oParent.headers,
        'index': 1
      }]
    };
    return oChildrenService;
  }
}
