import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { vitisAlert, VitisCheck, VitisClone, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { WmtsService } from 'src/modules/vmap/components/admin-datasource/services/wmts.service';
import { WMTSLayerRequireData } from '../../models/DatasourceTreeviewWMTS/WMTSLayer.model';
import { DatasourceTreeviewWMTSConnection } from 'src/modules/vmap/components/admin-datasource/models/DatasourceTreeviewWMTS/DatasourceTreeviewWMTSConnection';
import { DatasourceTreeviewWMTSLayer } from 'src/modules/vmap/components/admin-datasource/models/DatasourceTreeviewWMTS/DatasourceTreeviewWMTSLayer';
import { environment } from 'src/environments/environment';
import { TranslateService } from '@ngx-translate/core';
import { AdminDatasourceService } from '../../services/admin-datasource.service';
import { Injector } from '@angular/core';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';
import { MapCoreFormreaderService } from 'src/app/vitis/map-core/services/map-core-fromreader/map-core-formreader.service';
declare let $: any;

export class DatasourceTreeviewWMTSConnectionLayersFormJS {

  private oObject: DatasourceTreeviewWMTSConnection;
  // Services
  private vitisCheck: VitisCheck;
  private vitisRequest: VitisRequest;
  private vitisClone: VitisClone;
  private mapCoreManagerService: MapCoreManagerService;
  private mapCoreFormreaderService: MapCoreFormreaderService;
  private wmtsService: WmtsService;
  private translate: TranslateService;
  private adminDatasourceService: AdminDatasourceService;
  private formReaderService: FormreaderService;

  private oDatagridField;

  /**
   * Constructeur lancée au chargement du formulaire
   * @param {object} oFormReader Formreader
   * @param {DatasourceTreeviewWMTSConnection} oObject objet de l'arbre dans lequel il y a les fonctions nécessaires
   * @param {object} injector
   */
  constructor(
    private oFormReader: FormreaderComponent,
    oObject: DatasourceTreeviewWMTSConnection,
    injector: Injector
  ) {
    this.oObject = oObject;

    this.vitisCheck = injector.get(VitisCheck);
    this.vitisRequest = injector.get(VitisRequest);
    this.vitisClone = injector.get(VitisClone);
    this.mapCoreManagerService = injector.get(MapCoreManagerService);
    this.mapCoreFormreaderService = injector.get(MapCoreFormreaderService);
    this.wmtsService = injector.get(WmtsService);
    this.translate = injector.get(TranslateService);
    this.adminDatasourceService = injector.get(AdminDatasourceService);

    this.formReaderService = injector.get(FormreaderService);

  }

  /**
   * Evenement lancé par AdminDatasourceFormComponent juste avant l'envoi du formulaire
   */
  public prepareData(): Promise<boolean> {
    return new Promise((resolve, reject) => {
      if (this.vitisCheck.isDefAndNotNull(this.oObject.oCapabilities)) {
        const oMapField = this.oFormReader.getFormField('wmts_map_layers');
        this.oDatagridField = this.oFormReader.getFormField('wmts_datagrid_layers');

        const oCapabilities = this.oObject.oCapabilities;
        if (this.vitisCheck.isDefAndNotNull(oCapabilities) &&
          this.vitisCheck.isDefAndNotNull(oCapabilities.Contents) &&
          this.vitisCheck.isDefAndNotNull(oCapabilities.Contents.Layer)) {

          const oCapabilitiesLayers = oCapabilities.Contents.Layer;

          // Récupère la liste des couches
          const aLayers = this.wmtsService.searchLayers(oCapabilitiesLayers);

          // Récupère la projection des couches
          this.getMapProjFromWMTSLayers(aLayers).then((sProj: string) => {

            // Prépare les données pour la carte et la grille à partir des couches du flux WMTS.
            const oData = this.prepareLayers_(aLayers);

            // Suppression service si existe déjà
            const iChildIndex = oMapField.map_options.tree.children.findIndex((oChild: any) => oChild.name === oData.oChildrenService.name);
            if (iChildIndex !== -1) {
              oMapField.map_options.tree.children.splice(iChildIndex, 1);
            }

            // Ajout du service
            oMapField.map_options.tree.children.push(oData.oChildrenService);

            setTimeout(() => {
              if (this.vitisCheck.isDefAndNotNull(this.oDatagridField)) {
                this.oDatagridField.setValue(oData.aDatagridValues);
                // init event
                this.setEventDatagrid_(this.oDatagridField, oMapField);
                // Refresh Components
                oMapField.refreshMap();
                this.oDatagridField.refreshGrid();
              }
            });
          });
        }
      }
    });
  }

  /**
   * Prépare les couches pour la carte et les données pour la grille
   * @param aLayers
   * @returns
   */
  private prepareLayers_(aLayers: Array<any>): any {
    const oChildrenService = {
      'name': 'Service WMTS',
      'children': []
    };
    const aDatagridValues = [];
    aLayers = this.mergeLayers(aLayers);
    for (let i = 0; i < aLayers.length; i++) {
      const oLayerData = aLayers[i];
      const oDatagridValues = this.createDataGridLine_(oLayerData);
      aDatagridValues.push(oDatagridValues);
      // Ajout la couche à la carte que si la couche est dans les capabilities
      if (oLayerData.bIsInCapabilities !== false) {
        const oChildrenLayer = this.createWMTSLayer_(oLayerData, i);
        oChildrenService.children.push(oChildrenLayer);
      }
    }

    return {
      oChildrenService: oChildrenService,
      aDatagridValues: aDatagridValues
    };
  }

  /**
   * Créer la définition d'une couche wmts
   * @param oData
   * @param index
   */
  private createWMTSLayer_(oData: WMTSLayerRequireData, index: number): any {
    // Définition de la couche.
    return {
      'layer_grid_id': oData.layerGridId,
      'name': oData.oLayer.Identifier,
      'index': index,
      'layer': oData.oLayer.Identifier,
      'visible': oData.bVisible,
      'layerType': 'wmts',
      'matrixSet': oData.oMatrix.Identifier,
      'style': oData.oStyle ? oData.oStyle.Identifier : '',
      'format': oData.sFormat,
      // 'requestEncoding': oLayer.requestEncoding,
      'service_options': JSON.stringify(this.oObject.oCapabilities),
      'version': this.oObject.flux_version,
      'url': this.oObject.flux_url,
      'service_login': this.oObject.flux_login,
      'service_password': this.oObject.flux_password
    };
  }

  /**
   * Créer une ligne pour la grille
   * @param oData
   */
  private createDataGridLine_(oData: WMTSLayerRequireData) {
    return {
      layer_grid_id: oData.layerGridId,
      layer_id: oData.layerId,
      name: this.vitisCheck.isDefAndNotNull(oData.oLayer.Identifier) ? oData.oLayer.Identifier : '',
      title: this.vitisCheck.isDefAndNotNull(oData.oLayer.Title) ? oData.oLayer.Title : '',
      visible: oData.bVisible,
      style: oData.oStyle ? (oData.oStyle.Title ? oData.oStyle.Title : oData.oStyle.Identifier) : '',
      style_value: oData.oStyle ? oData.oStyle.Identifier : '',
      format: oData.sFormat,
      matrix: oData.oMatrix.Identifier + ' (' + oData.oMatrix.SupportedCRS + ')',
      matrix_value: oData.oMatrix.Identifier,
      isLoaded: oData.bIsLoaded,
      onError: (oData.bIsLoaded && !oData.bIsInCapabilities) // Couche chargé qui n'existe plus dans le getCapabilities
    };
  }

  /**
   * Initie les évenements sur la grille qui auront un impacte sur la carte
   * @param oDatagridField
   * @param oMapField
   */
  private setEventDatagrid_(oDatagridField: any, oMapField: any) {
    const self = this;
    setTimeout(() => {
      const updateMap = (oDatas) => {
        // Evenement pour modifier la carte
        self.mapCoreFormreaderService.emitMapFormreaderEvent(oMapField.id, {
          action: 'setVisibility',
          layerGridId: oDatas.layer_grid_id,
          visible: oDatas.visible
        });
      };

      this.oDatagridField.oDataGrid.deselectRowEvent.subscribe((oDatas) => {
        oDatas.visible = false;
        updateMap(oDatas);
      });

      this.oDatagridField.oDataGrid.selectRowEvent.subscribe((oDatas) => {
        oDatas.visible = true;
        updateMap(oDatas);
      });
    });
  }

  /**
   * Récupère la projection de la carte à partir de la liste des couches
   * @param aLayers
   */
  public getMapProjFromWMTSLayers(aLayers): Promise<any> {
    return new Promise((resolve, reject) => {
      this.mapCoreManagerService.getProjections().subscribe((oProjections: any) => {
        // Récupère l'index de projection à utiliser
        let sUsedProjectionIndex = oProjections.projections.findIndex((oProj) => oProj.code === 'EPSG:3857'); // par défaut 3857
        for (let i = 0; i < oProjections.projections.length; i++) {
          let bIsAllowedProj = true;
          for (let ii = 0; ii < aLayers.length; ii++) {
            const index = this.wmtsService.getTileMatrixSetsIndex(aLayers[ii], this.oObject.oCapabilities, oProjections.projections[i]);

            if (index === -1) {
              bIsAllowedProj = false;
            }
          }
          if (bIsAllowedProj) {
            sUsedProjectionIndex = this.vitisClone.clone(i);
            break;
          }
        }
        // Par sécurité si la valeur vaut -1 on prend l'element 0
        if (sUsedProjectionIndex === -1) {
          sUsedProjectionIndex = 0;
        }
        resolve(oProjections.projections[sUsedProjectionIndex].code);
      });
    });
  }

  /**
   * Merge la liste des couches avec la liste des couches chargées
   * @param aLayers
   */
  private mergeLayers(aLayers: Array<any>): Array<any> {
    const aWMTSLoadedLayers: Array<DatasourceTreeviewWMTSLayer> = this.oObject.getChildrens();
    const aMergedLayers: Array<WMTSLayerRequireData> = [];

    for (let i = 0; i < aLayers.length; i++) {
      const oLayer = aLayers[i];
      const oData: WMTSLayerRequireData = {
        layerGridId: this.getUniqueLayerGridId_(aMergedLayers),
        oLayer,
        oMatrix: null,
        sFormat: null,
        oStyle: null,
        bVisible: false,
        bIsLoaded: false,
        bIsInCapabilities: true
      };
      const layerCharged = aWMTSLoadedLayers.find(layer => layer.layer_name === oLayer.Identifier);
      if (layerCharged) {
        oData.layerGridId = i.toString();
        oData.oStyle = {
          Identifier: layerCharged.layer_style
        };
        oData.sFormat = layerCharged.layer_format;
        // Récupération de la matrice utlisé par la couche.
        let oLayerTileMatrix: any;
        if (this.vitisCheck.isDefAndNotNull(layerCharged.matrix_set) &&
          this.vitisCheck.isDefAndNotNull(this.oObject) &&
          this.vitisCheck.isDefAndNotNull(this.oObject.oCapabilities) &&
          this.vitisCheck.isDefAndNotNull(this.oObject.oCapabilities.Contents) &&
          this.vitisCheck.isArray(this.oObject.oCapabilities.Contents.TileMatrixSet)) {
          oLayerTileMatrix = this.oObject.oCapabilities.Contents.TileMatrixSet.find((oTileMatrix: any) => oTileMatrix.Identifier == layerCharged.matrix_set);
        }
        oData.oMatrix = {
          Identifier: layerCharged.matrix_set,
          SupportedCRS: oLayerTileMatrix ? oLayerTileMatrix.SupportedCRS : ''
        };
        oData.bIsLoaded = true;
        oData.layerId = layerCharged.layer_id;
        aMergedLayers.push(oData);
      } else {
        const aTileMatrixSets = this.wmtsService.getTileMatrixSets(oLayer, this.oObject.oCapabilities);
        const combinaisons = this.adminDatasourceService.buildCombinationsObjects({ label: 'style', value: oLayer.Style || [] },
          { label: 'format', value: oLayer.Format || [] },
          { label: 'matrix', value: aTileMatrixSets || [] });

        combinaisons.forEach((combination) => {
          const currentCombination = { ...oData };
          if (this.vitisCheck.isDefAndNotNull(combination.matrix)) {
            currentCombination.oMatrix = combination.matrix;
          }
          if (this.vitisCheck.isDefAndNotNull(combination.style)) {
            currentCombination.oStyle = combination.style;
          }
          if (this.vitisCheck.isDefAndNotNull(combination.format)) {
            currentCombination.sFormat = combination.format;
          }
          aMergedLayers.push(currentCombination);
        });
      }
    }
    return aMergedLayers;
  }

  /**
   * Supprime les couches de vmap
   */
  public deleteLayers() {
    const aDeletedValues = this.oDatagridField.oDataGrid.getSelectedRows_();
    const aDeletedLayerIds = [];
    for (let i = 0; i < aDeletedValues.length; i++) {
      const oDeletedValue = aDeletedValues[i];
      if (oDeletedValue.layer_id) {
        aDeletedLayerIds.push(oDeletedValue.layer_id);
      }
    }

    if (aDeletedLayerIds.length > 0) {
      this.sendRequestGetMap_(aDeletedLayerIds).then((aLayers) => {
        let sHtml = '';
        for (let i = 0; i < aLayers.length; i++) {
          const oLayer = aLayers[i];
          if (this.vitisCheck.isDefAndNotNull(oLayer.layer_id) && this.vitisCheck.isDefAndNotNull(oLayer.map) && oLayer.map.length > 0) {
            sHtml += '<div>';
            sHtml += '<p>' + oLayer.name + ' (' + oLayer.layer_id + ') :';
            for (let ii = 0; ii < oLayer.map.length; ii++) {
              const oMap = oLayer.map[ii];
              sHtml += ' <span>' + oMap.name + ' (' + oMap.map_id + ')</span>';
            }
            sHtml += '</p>';

            sHtml += '</div>';
          }
        }
        // Si au moins une couche sélectionné fait partie d'une carte on demande une confirmation de suppression
        if (sHtml !== '') {
          this.translate.get('DATASOURCE_TREEVIEW_FORM_LAYER_IN_MAP').subscribe((sTranslate: string) => {
            sHtml = '<p>' + sTranslate + '</p>' + sHtml;
            vitisAlert.alert({
              'type': 'warning',
              'width': '800px',
              'html': sHtml,
              'showCancelButton': true
            }).then((result) => {
              if (result.isConfirmed) {
                this.sendRequestDeleteLayers_(aDeletedLayerIds);
              }
            });
          });
        } else {
          this.sendRequestDeleteLayers_(aDeletedLayerIds);
        }
      });
    } else {
      this.translate.get('LIST_DELETE_NO_SELECTION_IMPORTED_LAYERS').subscribe((sTranslate: string) => {
        vitisAlert.fireToast({
          type: 'warning',
          title: sTranslate
        });
      });
    }
  }

  /**
   * Importe les couches du flux dans vmap
   */
  public addLayers() {
    const aSelectedValues = this.oDatagridField.oDataGrid.getSelectedRows_();
    const aInsertValues = [];
    for (let i = 0; i < aSelectedValues.length; i++) {
      const oSelectedValues = aSelectedValues[i];
      if (!oSelectedValues.layer_id) {
        const name = oSelectedValues.name + ' (' + oSelectedValues.style + ' ' + oSelectedValues.format + ' ' + oSelectedValues.matrix + ')';

        aInsertValues.push({
          datasource_id: this.oObject.datasourceId,
          name: name.length > 100 ? name.substring(0, 97) + '...' : name,
          theme_id: null,
          source_id: null,
          active: true,
          is_background_layer: false,
          layer_name: oSelectedValues.name,
          layer_style: oSelectedValues.style_value,
          layer_format: oSelectedValues.format,
          matrix_set: oSelectedValues.matrix_value,
          layer_options: JSON.stringify(this.oObject.oCapabilities)
        });
      }
    }

    if (aInsertValues.length > 0) {
      this.sendRequestAddLayers_(aInsertValues);
    } else if (aSelectedValues.length > 0) {
      this.translate.get('LIST_INSERT_SELECTION_ALLREADY_INSERT_LAYERS').subscribe((sTranslate: string) => {
        vitisAlert.fireToast({
          icon: 'warning',
          title: sTranslate
        });
      });
    } else {
      this.translate.get('LIST_INSERT_NO_SELECTION_LAYERS').subscribe((sTranslate: string) => {
        vitisAlert.fireToast({
          icon: 'warning',
          title: sTranslate
        });
      });
    }
  }

  /**
   * Lance la requête de suppression
   * @param aDatas
   */
  private sendRequestAddLayers_(aDatas: Array<any>): Promise<any> {
    return new Promise((resolve, reject) => {
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/wmtslayers/adds`;

      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        data: {
          data: aDatas
        },
        method: 'POST'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            this.translate.get('LIST_ADD_OK').subscribe((sTranslate: string) => {
              // Rafraichi la liste des enfants
              this.oObject.loadChildrens().then((aChildrens) => {
                this.adminDatasourceService.setChildrensMetadata(aChildrens, this.oObject);
                this.adminDatasourceService.refreshRightComponent(this.oObject.uId);
              });

              vitisAlert.fireToast({
                type: 'success',
                title: sTranslate
              });
            });
            resolve(oResult.data);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Lance la requête de suppression
   * @param aDeletedLayerIds
   */
  private sendRequestDeleteLayers_(aDeletedLayerIds: Array<string>): Promise<any> {
    return new Promise((resolve, reject) => {
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/wmtslayers`;

      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          idList: aDeletedLayerIds.join('|')
        },
        method: 'DELETE',
        displayErrorMessage: true
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            this.translate.get('LIST_DELETE_OK').subscribe((sTranslate: string) => {
              // Rafraichi la liste des enfants
              this.oObject.loadChildrens().then((aChildrens) => {
                this.adminDatasourceService.setChildrensMetadata(aChildrens, this.oObject);
                this.adminDatasourceService.refreshRightComponent(this.oObject.uId);
              });

              vitisAlert.fireToast({
                type: 'success',
                title: sTranslate
              });
            });
            resolve(oResult.data);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Lance une requête pour vérifier que les couches ne soit pas inclus dans une carte
   * @param aDeletedLayerIds
   */
  private sendRequestGetMap_(aDeletedLayerIds: Array<string>): Promise<any> {
    return new Promise((resolve, reject) => {
      const sUrl = `${environment.properties.app_api_url}/vmap/layers`;

      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          filter: { 'relation': 'AND', 'operators': [{ 'column': 'layer_id', 'compare_operator': 'IN', 'value': aDeletedLayerIds }] },
          attributs: 'layer_id|name|map'
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            resolve(oResult.data);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Trouve un identifiant unique à une ligne de la grille
   * @returns string
   */
  private getUniqueLayerGridId_(aLayerGrid: Array<WMTSLayerRequireData>): string {
    let id = '_' + Math.random().toString(36).substring(2, 11);
    if (aLayerGrid.findIndex((layer: any) => layer.layerGridId == id) !== -1) {
      id = this.getUniqueLayerGridId_(aLayerGrid);
    }
    return id;
  }
}
