import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { VitisCheck, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { DatasourceTreeviewWMTSLayer } from '../../models/DatasourceTreeviewWMTS/DatasourceTreeviewWMTSLayer';
import { defaultEmptyMapOptions } from 'src/app/modules/vmap/admin-datasource/forms/DatasourceTreeviewFormVar';
import { MapCoreFormreaderService } from 'src/app/vitis/map-core/services/map-core-fromreader/map-core-formreader.service';

export class DatasourceTreeviewXYZLayerMapFormJS {

  private oObject: DatasourceTreeviewWMTSLayer;
  // Services
  private vitisCheck: VitisCheck;
  private vitisClone: VitisClone;
  private mapCoreFormreaderService: MapCoreFormreaderService;

  /**
   * Constructeur lancée au chargement du formulaire
   * @param {object} oFormReader Formreader
   * @param {object} oObject objet de l'arbre dans lequel il y a les fonctions nécessaires
   * @param {object} injector
   */
  constructor(
    private oFormReader: FormreaderComponent,
    oObject: any,
    injector: any
  ) {
    this.oObject = oObject;

    this.vitisCheck = injector.get(VitisCheck);
    this.vitisClone = injector.get(VitisClone);
    this.mapCoreFormreaderService = injector.get(MapCoreFormreaderService);
  }

  /**
   * Evenement lancé par AdminDatasourceFormComponent juste avant l'envoi du formulaire
   */
  public prepareData() {
    // Si pas en insert (impossible en insert car this.oObject n'existe pas)
    if (this.oFormReader.sFormDefinitionName !== 'insert' && this.vitisCheck.isDefAndNotNull(this.oObject)) {
      const oMapField = this.oFormReader.getFormField('xyz_map_layer');

      // Réinitialise la carte
      oMapField.map_options = this.vitisClone.clone(defaultEmptyMapOptions);

      // Prépare les données pour la carte et la grille à partir des couches du flux WMTS.
      const oService = this.prepareLayer_();

      // Suppression service si existe déjà
      const iChildIndex = oMapField.map_options.tree.children.findIndex((oChild: any) => oChild.name === oService.name);
      if (iChildIndex !== -1) {
        oMapField.map_options.tree.children.splice(iChildIndex, 1);
      }

      // Ajout du service
      oMapField.map_options.tree.children.push(oService);

      this.mapCoreFormreaderService.emitMapFormreaderEvent(oMapField.id, {
        action: 'reload'
      });
    }
  }

  /**
   * Prépare la couche pour la carte
   * @returns
   */
  private prepareLayer_(): any {
    const oChildrenService = {
      'name': 'Service XYZ',
      'children': [{
        'index': 1000000,
        'name': this.oObject.name,
        'visible': 'true',
        'layerType': 'xyz',
        'url': this.oObject.oParent.flux_url
      }]
    };
    return oChildrenService;
  }
}
