import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { DatasourceTreeviewPostgis } from './DatasourceTreeviewPg/DatasourceTreeviewPostgis';
import { DatasourceTreeviewFiles } from './DatasourceTreeviewFiles/DatasourceTreeviewFiles';
import { DatasourceTreeviewWMS } from './DatasourceTreeviewWMS/DatasourceTreeviewWMS';
import { DatasourceTreeviewWMTS } from './DatasourceTreeviewWMTS/DatasourceTreeviewWMTS';
import { DatasourceTreeviewXYZ } from './DatasourceTreeviewXYZ/DatasourceTreeviewXYZ';
import { DatasourceTreeviewOSM } from './DatasourceTreeviewOSM/DatasourceTreeviewOSM';
import { DatasourceTreeviewVectortiles } from './DatasourceTreeviewVectortiles/DatasourceTreeviewVectortiles';
import { Injector } from '@angular/core';

export class DatasourceTreeviewDatasourceTypes {

  aChildrenTypes = [
    DatasourceTreeviewPostgis,
    DatasourceTreeviewFiles,
    DatasourceTreeviewOSM,
    DatasourceTreeviewWMS,
    DatasourceTreeviewWMTS,
    DatasourceTreeviewXYZ,
    DatasourceTreeviewVectortiles
  ];
  aChildrenObjects = [];

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector
  ) { }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetype`;

      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              for (let i = 0; i < oResult.data.length; i++) {
                const datasourceType = oResult.data[i];

                switch (datasourceType.datasourcetype_id) {
                  case 1: // Postgis (connexion)
                    const child = new DatasourceTreeviewPostgis(this.vitisCheck, this.vitisRequest, this.injector, datasourceType);
                    this.aChildrenObjects.push(child);
                    break;
                  case 2: // File
                    this.aChildrenObjects.push(new DatasourceTreeviewFiles(this.vitisCheck, this.vitisRequest, this.injector, datasourceType));
                    break;
                  case 3: // IGN

                    break;
                  case 4: // Open Street Map
                    this.aChildrenObjects.push(new DatasourceTreeviewOSM(this.vitisCheck, this.vitisRequest, this.injector, datasourceType));

                    break;
                  case 5: // WMS
                    this.aChildrenObjects.push(new DatasourceTreeviewWMS(this.vitisCheck, this.vitisRequest, this.injector, datasourceType));
                    break;
                  case 6: // WMTS
                    this.aChildrenObjects.push(new DatasourceTreeviewWMTS(this.vitisCheck, this.vitisRequest, this.injector, datasourceType));
                    break;
                  case 7: // XYZ
                    this.aChildrenObjects.push(new DatasourceTreeviewXYZ(this.vitisCheck, this.vitisRequest, this.injector, datasourceType));
                    break;
                  case 8: // XYZ
                    this.aChildrenObjects.push(new DatasourceTreeviewVectortiles(this.vitisCheck, this.vitisRequest, this.injector, datasourceType));
                    break;
                  default:
                    break;
                }
              }

              resolve(this.aChildrenObjects);
            } else {
              reject(oResult);
            }
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Appelé à la sélection
   */
  public onSelect() {

  }
}
