import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { DatasourceTreeviewFileLayerList } from './DatasourceTreeviewFileLayerList';
import { DatasourceTreeviewFilePublicationList } from './DatasourceTreeviewFilePublicationList';
import { DatasourceTreeviewFilesConnection } from './DatasourceTreeviewFilesConnection';
import { Injector } from '@angular/core';

export class DatasourceTreeviewFile {
  icon: string = 'icon-vmap-image'; // icon par défaut
  collapsed: boolean = false;
  aChildrenTypes = [DatasourceTreeviewFileLayerList, DatasourceTreeviewFilePublicationList];
  aChildrenObjects = [];

  // propriétés file
  name: string;
  directoryPath: string;
  fileextension: string;
  filesize: string;

  route: string = DatasourceTreeviewFile.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'file';
  }

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    public oFileConnectionParent: DatasourceTreeviewFilesConnection,
    file: any,
  ) {
    this.name = file.filename;
    this.directoryPath = file.directory_path;
    this.fileextension = file.extention.toLowerCase();
    if (this.fileextension === 'shp') {
      this.icon = 'icon-vmap-file';
    }
    this.filesize = file.size;

  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];
      this.aChildrenObjects.push(new DatasourceTreeviewFileLayerList(this.vitisCheck, this.vitisRequest, this.injector, this));
      this.aChildrenObjects.push(new DatasourceTreeviewFilePublicationList(this.vitisCheck, this.vitisRequest, this.injector, this));
      resolve(this.aChildrenObjects);
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourceFileForm {
    const oFormValues: DatasourceFileForm = {
      filename: this.name,
      directoryPath: this.directoryPath,
      fileextension: this.fileextension,
      icon: this.icon,
      filesize: this.filesize
    };
    return oFormValues;
  }
}

export class DatasourceFileForm {
  filename: string;
  directoryPath: string;
  fileextension: string;
  filesize: string;
  icon: string;
}
