import { LocalStorage, VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { DatasourceTreeviewFileLayerList } from './DatasourceTreeviewFileLayerList';
import { Injector } from '@angular/core';
import { AdminDatasourceService } from '../../services/admin-datasource.service';
import { environment } from 'src/environments/environment';
import { AdminDatasourceComponent } from '../../admin-datasource.component';

export class DatasourceTreeviewFileLayer {
  uId: number;
  icon: string = 'icon-vmap-layers-half';
  aChildrenTypes = [];

  // Propriété de la couche
  layer_id: number;
  name: string;
  datasource_id: number;
  datemaj: Date;
  theme_id: number;
  source_id: number;
  source_name: string;
  min_layer_scale: number;
  max_layer_scale: number;
  active: boolean;
  file_path: string;
  fileformat_id: number;
  coordsys_id: number;
  definition_sym_json: string;
  pictogram: string;
  isPictogramExist: boolean;
  legend: string;
  isLegendExist: boolean;

  // Services
  private adminDatasourceService: AdminDatasourceService;
  private localStorage: LocalStorage;

  aRightClickMenu = [{
    icon: 'icon-vmap-layers-half',
    label: 'ADMIN_DATASOURCE_BTN_UPDATE_PG_LAYER',
    event: 'updateFileLayer',
  }, {
    icon: 'fa fa-clone',
    label: 'ADMIN_DATASOURCE_BTN_CLONE_LAYER',
    event: 'cloneFileLayer'
  }, {
    icon: 'fa-trash',
    label: 'ADMIN_DATASOURCE_BTN_DELETE_PG_LAYER',
    event: 'deleteFileLayer',
  },
  {
    icon: 'fa fa-edit',
    label: 'ADMIN_DATASOURCE_BTN_EDIT_SYMB_PG_LAYER',
    event: 'openSymbology',
  },
  {
    icon: 'fa fa-refresh',
    label: 'ADMIN_DATASOURCE_BTN_REFRESH',
    event: 'refreshData',
  }];

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    public oFileLayerListParent: DatasourceTreeviewFileLayerList,
    datas: any,
  ) {
    this.setData(datas);
    this.adminDatasourceService = this.injector.get(AdminDatasourceService);
    this.localStorage = this.injector.get(LocalStorage);
  }

  route: string = DatasourceTreeviewFileLayer.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'filelayer/info';
  }

  /**
  * Rafraichit les données de l'objet
  */
  public refreshData() {
    const sUrl = `${environment.properties.app_api_url}/vmap/layers/${this.layer_id}`;
    this.vitisRequest.ajaxRequestPromise({
      url: sUrl,
      method: 'GET'
    }).then((resp: any) => {
      if (this.vitisCheck.isObject(resp.data)) {
        const oResult = resp.data;
        if (oResult.status == 1 && oResult.data) {
          this.setData(oResult.data);
          if (this.vitisCheck.isDefAndNotNull(this.adminDatasourceService)) {
            this.adminDatasourceService.refreshRightComponent(this.uId); // Demande à rafraichir le composant de droite
          }
        }
      }
    });
  }

  /**
   * Met les valeurs du webservice dans l'objet
   * @param datas
   */
  private setData(datas: any) {
    this.layer_id = datas.layer_id;
    this.name = datas.name;
    this.datasource_id = datas.datasource_id;
    this.datemaj = datas.datemaj;
    this.source_id = datas.source_id;
    this.theme_id = datas.theme_id;
    this.source_name = datas.source_name;
    this.min_layer_scale = datas.min_layer_scale;
    this.max_layer_scale = datas.max_layer_scale;
    this.active = datas.active;
    this.file_path = datas.file_path;
    this.fileformat_id = datas.fileformat_id;
    this.coordsys_id = datas.coordsys_id;
    this.definition_sym_json = datas.definition_sym_json;
    this.pictogram = datas.pictogram ? datas.pictogram : null;
    this.isPictogramExist = this.vitisCheck.isDefAndNotNull(datas.pictogram);
    this.legend = datas.legend ? datas.legend : null;
    this.isLegendExist = this.vitisCheck.isDefAndNotNull(datas.legend);
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourceFileLayerForm {
    const oFormValues: DatasourceFileLayerForm = {
      layer_id: this.layer_id,
      name: this.name,
      datasource_id: this.datasource_id,
      datemaj: this.datemaj,
      theme_id: this.theme_id,
      source_id: this.source_id,
      source_name: this.source_name,
      min_layer_scale: this.min_layer_scale,
      max_layer_scale: this.max_layer_scale,
      active: this.active,
      file_path: this.file_path,
      fileformat_id: this.fileformat_id,
      coordsys_id: this.coordsys_id,
      definition_sym_json: this.definition_sym_json,
      pictogram: this.pictogram,
      isPictogramExist: this.isPictogramExist,
      legend: this.legend,
      isLegendExist: this.isLegendExist
    };
    return oFormValues;
  }

  /**
   * Lance le formulaire de mise à jour de l'objet
   * @param {AdminDatasourceComponent} scope
   */
  public updateFileLayer(scope: AdminDatasourceComponent) {
    scope.openLayerRoute(this.layer_id, 'informations_generales');
  }

  /**
   * Sauvegarde le uId et renvoie vers le formulaire d'ajout
   */
  public cloneFileLayer(scope: AdminDatasourceComponent) {
    this.localStorage.set('idClone', this.uId);
    const oObject = this.adminDatasourceService.getObjectById(this.uId);
    scope.openRoute({
      route: DatasourceTreeviewFileLayer.getRoute(),
      formName: 'insert',
      uId: oObject.oParent.uId
    });
  }

  /**
   * Ouvre la symbologie
   * @param {AdminDatasourceComponent} scope
   */
  public openSymbology(scope: AdminDatasourceComponent) {
    scope.openLayerRoute(this.layer_id, 'symbology');
  }

  /**
   * Supprimer une couche File
   * @param scope
   */
  public deleteFileLayer(scope: AdminDatasourceComponent) {
    this.adminDatasourceService.deleteLayer(this.layer_id, this.uId);
  }
}

export class DatasourceFileLayerForm {
  layer_id: number;
  name: string;
  datasource_id: number;
  datemaj: Date;
  theme_id: number;
  source_id: number;
  source_name: string;
  min_layer_scale: number;
  max_layer_scale: number;
  active: boolean;
  file_path: string;
  fileformat_id: number;
  coordsys_id: number;
  definition_sym_json: string;
  pictogram: string;
  isPictogramExist: boolean;
  legend: string;
  isLegendExist: boolean;
}
