import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { DatasourceTreeviewFileLayer } from './DatasourceTreeviewFileLayer';
import { DatasourceTreeviewFile } from './DatasourceTreeviewFile';
import { Injector } from '@angular/core';
import { AdminDatasourceComponent } from '../../admin-datasource.component';

export class DatasourceTreeviewFileLayerList {
  uId: number;
  collapsed: boolean = false;
  icon: string = 'icon-vmap-bxs-layer';
  name: string;
  filePath: string;
  aChildrenTypes = [DatasourceTreeviewFileLayer];
  aChildrenObjects = [];
  aRightClickMenu = [{
    icon: 'icon-vmap-layers-half',
    label: 'ADMIN_DATASOURCE_BTN_ADD_PG_LAYER',
    event: 'createFileLayer',
  }];
  messageMapping: any = {
    '=0': 'SUBTITLE_DATASOURCE_TREE_LAYERS',
    'other': 'SUBTITLE_DATASOURCE_TREE_LAYERS_MULTI'
  };

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    public oFileParent: DatasourceTreeviewFile,
  ) {
    this.filePath = this.oFileParent.directoryPath + '/' + this.oFileParent.name;

  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/filelayers`;
      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          order_by: 'name',
          filter: {
            'relation': 'AND',
            'operators': [
              { 'column': 'file_path', 'compare_operator': '=', 'value': this.filePath },
              {
                'column': 'datasource_id',
                'compare_operator': '=',
                'value': this.oFileParent.oFileConnectionParent.datasourceId
              }
            ]
          }
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              // Ajoute les connexions issue de la base
              for (let i = 0; i < oResult.data.length; i++) {
                const fileLayer = oResult.data[i];

                const child = new DatasourceTreeviewFileLayer(this.vitisCheck, this.vitisRequest, this.injector, this, fileLayer);
                this.aChildrenObjects.push(child);
              }
            }

            resolve(this.aChildrenObjects);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Lance le formulaire de création de l'objet enfant
   * @param {AdminDatasourceComponent} scope
   */
  public createFileLayer(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewFileLayer.getRoute(),
      formName: 'insert',
      uId: this.uId
    });
  }

  /**
   * Initialise les valeurs du formulaire insert enfant
   */
  public getChildrenInitialFormValues(): any {
    const oFormValues: any = {
      file_path: this.filePath,
      datasource_id: this.oFileParent.oFileConnectionParent.datasourceId
    };

    return oFormValues;
  }
}
