import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { DatasourceTreeviewFilePublicationList } from './DatasourceTreeviewFilePublicationList';
import { Injector } from '@angular/core';
import { AdminDatasourceService } from '../../services/admin-datasource.service';
import { environment } from 'src/environments/environment';
import { AdminDatasourceComponent } from '../../admin-datasource.component';

export class DatasourceTreeviewFilePublication {
  uId: number;
  icon: string = 'icon-vmap-bx-log-in-circle';

  publicationId: number;
  title: string;
  name: string;
  datasourceId: number;
  dateMaj: Date;
  extent: string;
  active: boolean;
  minLayerScale: number;
  maxLayerScale: number;
  filePath: string;
  fileformatId: number;
  coordsysId: number;
  definitionSymJson: string;

  private adminDatasourceService: AdminDatasourceService;

  aRightClickMenu = [{
    icon: 'icon-vmap-layers-half',
    label: 'ADMIN_DATASOURCE_BTN_UPDATE_FILE_PUBLICATION',
    event: 'updateFilePublication',
  },
  {
    icon: 'fa-trash',
    label: 'ADMIN_DATASOURCE_BTN_DELETE_FILE_PUBLICATION',
    event: 'deleteFilePublication',
  },
  {
    icon: 'fa fa-refresh',
    label: 'ADMIN_DATASOURCE_BTN_REFRESH',
    event: 'refreshData',
  }];

  route: string = DatasourceTreeviewFilePublication.getRoute();

  /**
   * Route pour les infos d'une publication de type fichier
   */
  static getRoute() {
    return 'filepublication/info';
  }

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    public oFilePublicationListParent: DatasourceTreeviewFilePublicationList,
    datas: any,
  ) {
    this.setData(datas);
    this.adminDatasourceService = this.injector.get(AdminDatasourceService);
  }

  /**
   * Rafraichit les données de l'objet
   */
  public refreshData() {
    const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/filepublications/${this.publicationId}`;
    this.vitisRequest.ajaxRequestPromise({
      url: sUrl,
      method: 'GET'
    }).then((resp: any) => {
      if (this.vitisCheck.isObject(resp.data)) {
        const oResult = resp.data;
        if (oResult.status == 1 && oResult.data) {
          this.setData(oResult.data);
          if (this.vitisCheck.isDefAndNotNull(this.adminDatasourceService)) {
            this.adminDatasourceService.refreshRightComponent(this.uId); // Demande à rafraichir le composant de droite
          }
        }
      }
    });
  }

  /**
   * Met les valeurs du webservice dans l'objet
   * @param datas
   */
  private setData(datas: any) {
    this.publicationId = datas.publication_id;
    this.title = datas.title;
    this.name = datas.name;
    this.datasourceId = datas.datasource_id;
    this.dateMaj = datas.datemaj;
    this.extent = datas.extent;
    this.active = datas.active;
    this.minLayerScale = datas.min_layer_scale;
    this.maxLayerScale = datas.max_layer_scale;
    this.filePath = datas.file_path;
    this.fileformatId = datas.fileformat_id;
    this.definitionSymJson = datas.definition_sym_json;
    this.coordsysId = datas.coordsys_id;
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourceFilePublicationForm {
    const oFormValues: DatasourceFilePublicationForm = {
      publication_id: this.publicationId,
      title: this.title,
      name: this.name,
      datasource_id: this.datasourceId,
      datemaj: this.dateMaj,
      extent: this.extent,
      active: this.active,
      min_layer_scale: this.minLayerScale,
      max_layer_scale: this.maxLayerScale,
      file_path: this.filePath,
      fileformat_id: this.fileformatId,
      definition_sym_json: this.definitionSymJson,
      coordsys_id: this.coordsysId,
    };

    return oFormValues;
  }

  /**
   * Lance le formulaire de mise à jour de l'objet
   * @param {AdminDatasourceComponent} scope
   */
  public updateFilePublication(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewFilePublication.getRoute(),
      formName: 'update',
      uId: this.uId
    });
  }

  /**
   * Supprimer une couche postgres
   * @param scope
   */
  public deleteFilePublication(scope: AdminDatasourceComponent) {
    this.adminDatasourceService.deletePublication(this.publicationId, this.uId);
  }
}

export class DatasourceFilePublicationForm {
  publication_id: number;
  title: string;
  name: string;
  datasource_id: number;
  datemaj: Date;
  extent: string;
  active: boolean;
  min_layer_scale: number;
  max_layer_scale: number;
  file_path: string;
  fileformat_id: number;
  definition_sym_json: string;
  coordsys_id: number;
}
