import { LocalStorage, VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { AdminDatasourceComponent } from 'src/modules/vmap/components/admin-datasource/admin-datasource.component';
import { Injector } from '@angular/core';
import { AdminDatasourceService } from '../../services/admin-datasource.service';
import { DatasourceTreeviewOSMConnection } from './DatasourceTreeviewOSMConnection';

export class DatasourceTreeviewOSMLayer {
  uId: number;
  oParent: DatasourceTreeviewOSMConnection;
  icon: string = 'icon-vmap-layers-half';

  layer_id: number;
  name: string;
  datasource: any;
  datasource_id: number;
  datemaj: Date;
  source_id: number;
  source_name: string;
  extent: string;
  min_layer_scale: number;
  max_layer_scale: number;
  active: boolean;
  is_background_layer: boolean;
  theme_label: string;
  theme_id: number;
  pictogram: string;
  isPictogramExist: boolean;
  legend: string;
  isLegendExist: boolean;

  // Services
  private adminDatasourceService: AdminDatasourceService;
  private localStorage: LocalStorage;

  aRightClickMenu = [{
    icon: 'icon-vmap-layers-half',
    label: 'ADMIN_DATASOURCE_BTN_UPDATE_OSM_LAYER',
    event: 'updateOSMLayer',
  }, {
    icon: 'fa fa-clone',
    label: 'ADMIN_DATASOURCE_BTN_CLONE_LAYER',
    event: 'cloneOSMLayer'
  }, {
    icon: 'fa-trash',
    label: 'ADMIN_DATASOURCE_BTN_DELETE_OSM_LAYER',
    event: 'deleteOSMLayer',
  }];
  route: string = DatasourceTreeviewOSMLayer.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'osmlayer/info';
  }

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: any,
  ) {
    this.setData_(datas);
    this.adminDatasourceService = this.injector.get(AdminDatasourceService);
    this.localStorage = this.injector.get(LocalStorage);
  }

  /**
   * Rafraichit les données de l'objet
   */
  public refreshData() {
    const sUrl = `${environment.properties.app_api_url}/vmap/layers/${this.layer_id}`;
    this.vitisRequest.ajaxRequestPromise({
      url: sUrl,
      method: 'GET'
    }).then((resp: any) => {
      if (this.vitisCheck.isObject(resp.data)) {
        const oResult = resp.data;
        if (oResult.status == 1 && oResult.data) {
          this.setData_(oResult.data);
          if (this.vitisCheck.isDefAndNotNull(this.adminDatasourceService)) {
            this.adminDatasourceService.refreshRightComponent(this.uId); // Demande à rafraichir le composant de droite
          }
        }
      }
    });
  }

  /**
   * Met les valeurs du webservice dans l'objet
   * @param datas
   */
  private setData_(datas: any) {
    this.layer_id = datas.layer_id;
    this.name = datas.name;
    this.datasource_id = datas.datasource_id;
    this.datasource = datas.datasource;
    this.datemaj = datas.datemaj;
    this.source_id = datas.source ? datas.source.source_id : null;
    this.source_name = datas.source ? datas.source.name : null;
    this.extent = datas.extent;
    this.min_layer_scale = datas.min_layer_scale;
    this.max_layer_scale = datas.max_layer_scale;
    this.active = datas.active;
    this.is_background_layer = datas.is_background_layer;
    this.theme_label = datas.theme ? datas.theme.label : null;
    this.theme_id = datas.theme ? datas.theme.theme_id : null;
    this.pictogram = datas.pictogram ? datas.pictogram : null;
    this.isPictogramExist = this.vitisCheck.isDefAndNotNull(datas.pictogram);
    this.legend = datas.legend ? datas.legend : null;
    this.isLegendExist = this.vitisCheck.isDefAndNotNull(datas.legend);
  }


  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): Promise<DatasourceOSMLayerForm> {
    return new Promise((resolve, reject) => {
      const oFormValues: DatasourceOSMLayerForm = {
        layer_id: this.layer_id,
        name: this.name,
        datasource: this.datasource,
        datasource_id: this.datasource_id,
        datemaj: this.datemaj,
        source_id: this.source_id,
        source_name: this.source_name,
        extent: this.extent,
        min_layer_scale: this.min_layer_scale,
        max_layer_scale: this.max_layer_scale,
        active: this.active,
        is_background_layer: this.is_background_layer,
        theme_label: this.theme_label,
        theme_id: this.theme_id,
        pictogram: this.pictogram,
        isPictogramExist: this.isPictogramExist,
        legend: this.legend,
        isLegendExist: this.isLegendExist
      };
      resolve(oFormValues);
    });
  }

  /**
   * Lance le formulaire de mise à jour de l'objet
   * @param {AdminDatasourceComponent} scope
   */
  public updateOSMLayer(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewOSMLayer.getRoute(),
      formName: 'update',
      uId: this.uId
    });
  }

  /**
   * Sauvegarde le uId et renvoie vers le formulaire d'ajout
   */
  public cloneOSMLayer(scope: AdminDatasourceComponent) {
    this.localStorage.set('idClone', this.uId);
    const oObject = this.adminDatasourceService.getObjectById(this.uId);
    scope.openRoute({
      route: DatasourceTreeviewOSMLayer.getRoute(),
      formName: 'insert',
      uId: oObject.oParent.uId
    });
  }

  /**
   * Supprimer une couche OSM
   * @param scope
   */
  public deleteOSMLayer(scope: AdminDatasourceComponent) {
    this.adminDatasourceService.deleteLayer(this.layer_id, this.uId);
  }

}

export class DatasourceOSMLayerForm {
  layer_id: number;
  name: string;
  datasource: any;
  datasource_id: number;
  datemaj: Date;
  source_id: number;
  source_name: string;
  extent: string;
  min_layer_scale: number;
  max_layer_scale: number;
  active: boolean;
  is_background_layer: boolean;
  theme_label: string;
  theme_id: number;
  pictogram: string;
  isPictogramExist: boolean;
  legend: string;
  isLegendExist: boolean;
}
