import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { DatasourceTreeviewPgConnection } from './DatasourceTreeviewPgConnection';
import { DatasourceTreeviewPgSchema } from './DatasourceTreeviewPgSchema';
import { Injector } from '@angular/core';
import { AdminDatasourceComponent } from '../../admin-datasource.component';

export class DatasourceTreeviewPgDatabase {
  uId: number;
  oParent: DatasourceTreeviewPgConnection;
  collapsed: boolean = false;
  icon: string = 'icon-vmap-database';
  name: string;
  aChildrenTypes = [DatasourceTreeviewPgSchema];
  aChildrenObjects = [];
  route: string = DatasourceTreeviewPgDatabase.getRoute();
  aRightClickMenu = [{
    icon: 'fa fa-refresh',
    label: 'ADMIN_DATASOURCE_BTN_REFRESH',
    event: 'refreshData',
  }];

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'pgdatabase';
  }

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: string,
  ) {
    this.name = datas;
  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];

      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/schemas`;

      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          filter: {
            'relation': 'AND',
            'operators': [
              { 'column': 'table_schema', 'compare_operator': 'NOT IN', 'value': ['pg_catalog', 'information_schema'] }
            ]
          },
          order_by: 'schema_name',
          datasource_id: this.oParent.datasourceId,
          database: this.name
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              // Ajoute les connexions issue de la base
              for (let i = 0; i < oResult.data.length; i++) {
                const schema = oResult.data[i].schema_name;
                const child = new DatasourceTreeviewPgSchema(this.vitisCheck, this.vitisRequest, this.injector, schema);
                this.aChildrenObjects.push(child);
              }
            }
            resolve(this.aChildrenObjects);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourcePgDatabaseForm {
    const oFormValues: DatasourcePgDatabaseForm = {
      name: this.name
    };

    return oFormValues;
  }

  /**
   * Rafraichit les données de l'objet
   * @param scope
   */
  public refreshData(scope: AdminDatasourceComponent) {
    // Rechargement de la base de données.
    this.aChildrenObjects = null;
    scope.getChildrens(this);
    // Page d'accueil de la base de données.
    scope.openRoute({
      route: this.route,
      formName: 'display',
      uId: this.uId
    });
  }
}

export class DatasourcePgDatabaseForm {
  name: string;
}
