import { LocalStorage, VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { AdminDatasourceComponent } from '../../admin-datasource.component';
import { DatasourceTreeviewPgLayerList } from './DatasourceTreeviewPgLayerList';
import { Injector } from '@angular/core';
import { AdminDatasourceService } from '../../services/admin-datasource.service';


export class DatasourceTreeviewPgLayer {
  uId: number;
  oParent: DatasourceTreeviewPgLayerList;
  icon: string = 'icon-vmap-layers-half';

  layerId: number;
  name: string;
  datemaj: Date;
  sourceId: number;
  sourceName: string;
  extent: string;
  minLayerScale: number;
  maxLayerScale: number;
  active: boolean;
  isBackgroundLayer: boolean;
  themeLabel: string;
  themeId: number;
  definitionSymJson: string;
  definitionSymMapserver: string;
  isSymDefinitionJson: boolean;
  database: string;
  schema: string;
  pgTable: string;
  idfield: string;
  geomfield: string;
  coordsysId: number;
  coordsysLabel: string;
  msGeomtypeId: number;
  msGeomtypeLabel: string;
  jsonColumns: string;
  isLocation: boolean;
  isFilter: boolean;
  isDisectable: boolean;
  locationSearchField: string;
  locationPlaceholder: string;
  locationResultField: string;
  locationSearchModeId: number;
  locationSearchModeLabel: string;
  isMapSelection: boolean;
  mapSelectionBuffer: number;
  isDataGrid: boolean;
  minEditionScale: number;
  maxEditionScale: number;
  formTitle: string;
  formDisplaySizeId: number;
  formDisplaySizeLabel: string;
  formInsertSizeId: number;
  formInsertSizeLabel: string;
  formUpdateSizeId: number;
  formUpdateSizeLabel: string;
  isClone: boolean;
  isAutomaticallyEdit: boolean;
  isMultiEdit: boolean;
  isMultiGeom: boolean;
  selectField: string;
  defaultFormJsonDef: string;
  defaultFormJsDef: string;
  defaultFormCssDef: string;
  publishedFormJsonDef: string;
  publishedFormJsDef: string;
  publishedFormCssDef: string;
  customFormJsonDef: string;
  customFormJsDef: string;
  customFormCssDef: string;
  isGeomSelection: boolean;
  isGeomUpdatable: boolean;
  datasourceId: number;
  isGeometryLayer: boolean;
  isRequestorLayer: boolean;
  isSpecificTableForm: boolean;
  schemaForm: string;
  tableForm: string;
  isSpecificTableRequestor: boolean;
  schemaRequestor: string;
  tableRequestor: string;
  pictogram: string;
  isPictogramExist: boolean;
  legend: string;
  isLegendExist: boolean;

  // Services
  private adminDatasourceService: AdminDatasourceService;
  private localStorage: LocalStorage;

  aRightClickMenu = [{
    icon: 'icon-vmap-layers-half',
    label: 'ADMIN_DATASOURCE_BTN_UPDATE_PG_LAYER',
    event: 'updatePgLayer',
  },
  {
    icon: 'fa fa-clone',
    label: 'ADMIN_DATASOURCE_BTN_CLONE_LAYER',
    event: 'clonePgLayer'
  },
  {
    icon: 'fa-trash',
    label: 'ADMIN_DATASOURCE_BTN_DELETE_PG_LAYER',
    event: 'deletePgLayer',
  },
  {
    icon: 'fa fa-edit',
    condition: '= {{isLocation}} == 1 || {{isMapSelection}} == 1 || {{isRequestorLayer}} == 1',
    label: 'ADMIN_DATASOURCE_BTN_EDIT_FORM_PG_LAYER',
    event: 'openStudio',
  },
  {
    icon: 'fa fa-edit',
    condition: '= {{isGeometryLayer}} == 1',
    label: 'ADMIN_DATASOURCE_BTN_EDIT_SYMB_PG_LAYER',
    event: 'openSymbologie',
  },
  {
    icon: 'fa fa-refresh',
    label: 'ADMIN_DATASOURCE_BTN_REFRESH',
    event: 'refreshData',
  }];
  route: string = DatasourceTreeviewPgLayer.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'pglayer/info';
  }

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: any,
  ) {
    this.setData(datas);
    this.adminDatasourceService = this.injector.get(AdminDatasourceService);
    this.localStorage = this.injector.get(LocalStorage);
  }

  /**
   * Rafraichit les données de l'objet
   */
  public refreshData() {
    const sUrl = `${environment.properties.app_api_url}/vmap/layers/${this.layerId}`;
    this.vitisRequest.ajaxRequestPromise({
      url: sUrl,
      method: 'GET'
    }).then((resp: any) => {
      if (this.vitisCheck.isObject(resp.data)) {
        const oResult = resp.data;
        if (oResult.status == 1 && oResult.data) {
          this.setData(oResult.data);
          if (this.vitisCheck.isDefAndNotNull(this.adminDatasourceService)) {
            this.adminDatasourceService.refreshRightComponent(this.uId); // Demande à rafraichir le composant de droite
          }
        }
      }
    });
  }

  /**
   * Met les valeurs du webservice dans l'objet
   * @param datas
   */
  private setData(datas: any) {
    this.layerId = datas.layer_id;
    this.name = datas.name;
    this.datemaj = datas.datemaj;
    this.sourceId = datas.source ? datas.source.source_id : null;
    this.sourceName = datas.source ? datas.source.name : null;
    this.extent = datas.extent;
    this.minLayerScale = datas.min_layer_scale;
    this.maxLayerScale = datas.max_layer_scale;
    this.active = datas.active;
    this.isBackgroundLayer = datas.is_background_layer;
    this.themeLabel = datas.theme ? datas.theme.label : null;
    this.themeId = datas.theme ? datas.theme.theme_id : null;
    this.definitionSymJson = datas.definition_sym_json ? JSON.parse(datas.definition_sym_json) : null;
    this.definitionSymMapserver = datas.definition_sym_mapserver;
    this.isSymDefinitionJson = datas.is_sym_definition_json ? JSON.parse(datas.is_sym_definition_json) : null;
    this.database = datas.database;
    this.schema = datas.schema;
    this.pgTable = datas.pg_table;
    this.idfield = datas.idfield;
    this.geomfield = datas.geomfield;
    this.coordsysId = datas.coordsys ? datas.coordsys.coordsys_id : null;
    this.coordsysLabel = datas.coordsys ? datas.coordsys.label : null;
    this.msGeomtypeId = datas.ms_geomtype ? datas.ms_geomtype.ms_geomtype_id : null;
    this.msGeomtypeLabel = datas.ms_geomtype ? datas.ms_geomtype.label : null;
    this.jsonColumns = datas.json_columns;
    this.isLocation = datas.is_location;
    this.isFilter = datas.is_filter;
    this.isDisectable = datas.is_disectable;
    this.locationSearchField = datas.location_search_field;
    this.locationPlaceholder = datas.location_placeholder;
    this.locationResultField = datas.location_result_field;
    this.locationSearchModeId = datas.location_search_mode ? datas.location_search_mode.search_mode_id : null;
    this.locationSearchModeLabel = datas.location_search_mode ? datas.location_search_mode.label : null;
    this.isMapSelection = datas.is_map_selection;
    this.mapSelectionBuffer = datas.map_selection_buffer;
    this.isDataGrid = datas.is_data_grid;
    this.minEditionScale = datas.min_edition_scale;
    this.maxEditionScale = datas.max_edition_scale;
    this.formTitle = datas.form_title;
    this.formDisplaySizeId = datas.form_display_size ? datas.form_display_size.form_display_size_id : null;
    this.formDisplaySizeLabel = datas.form_display_size ? datas.form_display_size.label : null;
    this.formInsertSizeId = datas.form_insert_size ? datas.form_insert_size.form_insert_size_id : null;
    this.formInsertSizeLabel = datas.form_insert_size ? datas.form_insert_size.label : null;
    this.formUpdateSizeId = datas.form_update_size ? datas.form_update_size.form_update_size_id : null;
    this.formUpdateSizeLabel = datas.form_update_size ? datas.form_update_size.label : null;
    this.isClone = datas.is_clone;
    this.isAutomaticallyEdit = datas.is_automatically_edit;
    this.isMultiEdit = datas.is_multi_edit;
    this.isMultiGeom = datas.is_multi_geom;
    this.selectField = datas.select_field;
    this.defaultFormJsonDef = datas.default_form_json_def ? JSON.parse(datas.default_form_json_def) : null;
    this.defaultFormJsDef = datas.default_form_js_def;
    this.defaultFormCssDef = datas.default_form_css_def;
    this.publishedFormJsonDef = datas.published_form_json_def ? JSON.parse(datas.published_form_json_def) : null;
    this.publishedFormJsDef = datas.published_form_js_def;
    this.publishedFormCssDef = datas.published_form_css_def;
    this.customFormJsonDef = datas.custom_form_json_def ? JSON.parse(datas.custom_form_json_def) : null;
    this.customFormJsDef = datas.custom_form_js_def;
    this.customFormCssDef = datas.custom_form_css_def;
    this.isGeomSelection = datas.is_geom_selection;
    this.isGeomUpdatable = datas.is_geom_updatable;
    this.datasourceId = datas.datasource_id;
    this.isGeometryLayer = datas.is_geometry_layer;
    this.isRequestorLayer = datas.is_requestor_layer;
    this.isSpecificTableForm = datas.is_specific_table_form;
    this.schemaForm = datas.schema_form;
    this.tableForm = datas.table_form;
    this.isSpecificTableRequestor = datas.is_specific_table_requestor;
    this.schemaRequestor = datas.schema_requestor;
    this.tableRequestor = datas.table_requestor;
    this.pictogram = datas.pictogram ? datas.pictogram : null;
    this.isPictogramExist = this.vitisCheck.isDefAndNotNull(datas.pictogram);
    this.legend = datas.legend ? datas.legend : null;
    this.isLegendExist = this.vitisCheck.isDefAndNotNull(datas.legend);
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourcePgLayerForm {
    const oFormValues: DatasourcePgLayerForm = {
      layer_id: this.layerId,
      name: this.name,
      datemaj: this.datemaj,
      source_id: this.sourceId,
      source_label: this.sourceName,
      extent: this.extent,
      min_layer_scale: this.minLayerScale,
      max_layer_scale: this.maxLayerScale,
      active: this.active,
      is_background_layer: this.isBackgroundLayer,
      theme_id: this.themeId,
      theme_label: this.themeLabel,
      definition_sym_json: this.definitionSymJson,
      definition_sym_mapserver: this.definitionSymMapserver,
      is_sym_definition_json: this.isSymDefinitionJson,
      database: this.database,
      schema: this.schema,
      pg_table: this.pgTable,
      idfield: this.idfield,
      geomfield: this.geomfield,
      coordsys_id: this.coordsysId,
      coordsys_label: this.coordsysLabel,
      ms_geomtype_id: this.msGeomtypeId,
      ms_geomtype_label: this.msGeomtypeLabel,
      json_columns: this.jsonColumns,
      is_location: this.isLocation,
      is_filter: this.isFilter,
      is_disectable: this.isDisectable,
      location_search_field: this.locationSearchField,
      location_placeholder: this.locationPlaceholder,
      location_result_field: this.locationResultField,
      location_search_mode_id: this.locationSearchModeId,
      location_search_mode_label: this.locationSearchModeLabel,
      is_map_selection: this.isMapSelection,
      map_selection_buffer: this.mapSelectionBuffer,
      is_data_grid: this.isDataGrid,
      min_edition_scale: this.minEditionScale,
      max_edition_scale: this.maxEditionScale,
      form_title: this.formTitle,
      form_display_size_id: this.formDisplaySizeId,
      form_insert_size_id: this.formInsertSizeId,
      form_update_size_id: this.formUpdateSizeId,
      is_clone: this.isClone,
      is_automatically_edit: this.isAutomaticallyEdit,
      is_multi_edit: this.isMultiEdit,
      is_multi_geom: this.isMultiGeom,
      select_field: this.selectField,
      default_form_json_def: this.defaultFormJsonDef,
      default_form_js_def: this.defaultFormJsDef,
      default_form_css_def: this.defaultFormCssDef,
      published_form_json_def: this.publishedFormJsonDef,
      published_form_js_def: this.publishedFormJsDef,
      published_form_css_def: this.publishedFormCssDef,
      custom_form_json_def: this.customFormJsonDef,
      custom_form_js_def: this.customFormJsDef,
      custom_form_css_def: this.customFormCssDef,
      is_geom_selection: this.isGeomSelection,
      is_geom_updatable: this.isGeomUpdatable,
      datasource_id: this.datasourceId,
      is_geometry_layer: this.isGeometryLayer,
      is_requestor_layer: this.isRequestorLayer,
      is_specific_table_form: this.isSpecificTableForm,
      schema_form: this.schemaForm,
      table_form: this.tableForm,
      is_specific_table_requestor: this.isSpecificTableRequestor,
      schema_requestor: this.schemaRequestor,
      table_requestor: this.tableRequestor,
      pictogram: this.pictogram,
      isPictogramExist: this.isPictogramExist,
      legend: this.legend,
      isLegendExist: this.isLegendExist
    };
    return oFormValues;
  }


  /**
   * Redirige vers component de mise à jour de la couche
   * @param {AdminDatasourceComponent} scope
   */
  public updatePgLayer(scope: AdminDatasourceComponent) {
    scope.openLayerRoute(this.layerId, 'informations_generales');
  }

  /**
   * Sauvegarde le uId et renvoie vers le formulaire d'ajout
   */
  public clonePgLayer(scope: AdminDatasourceComponent) {
    this.localStorage.set('idClone', this.uId);
    const oObject = this.adminDatasourceService.getObjectById(this.uId);

    // Met en JSON les onglets non affichés
    this.definitionSymJson = JSON.stringify(this.definitionSymJson);
    this.defaultFormJsonDef = JSON.stringify(this.defaultFormJsonDef);
    this.publishedFormJsonDef = JSON.stringify(this.publishedFormJsonDef);
    this.customFormJsonDef = JSON.stringify(this.customFormJsonDef);

    scope.openRoute({
      route: DatasourceTreeviewPgLayer.getRoute(),
      formName: 'insert',
      uId: oObject.oParent.uId
    });
  }

  /**
   * Ouvre le studio pour éditer le formulaire
   * @param {AdminDatasourceComponent} scope
   */
  public openStudio(scope: AdminDatasourceComponent) {
    scope.openLayerRoute(this.layerId, 'form');
  }
  /**
   * Ouvre la symbologie
   * @param {AdminDatasourceComponent} scope
   */
  public openSymbologie(scope: AdminDatasourceComponent) {
    scope.openLayerRoute(this.layerId, 'symbology');
  }
  /**
   * Supprimer une couche postgres
   * @param scope
   */
  public deletePgLayer(scope: AdminDatasourceComponent) {
    this.adminDatasourceService.deleteLayer(this.layerId, this.uId);
  }
}


export class DatasourcePgLayerForm {
  layer_id: number;
  name: string;
  datemaj: Date;
  source_id: number;
  source_label?: string;
  extent: string;
  min_layer_scale: number;
  max_layer_scale: number;
  active: boolean;
  is_background_layer: boolean;
  theme_id: number;
  theme_label?: string;
  definition_sym_json: string;
  definition_sym_mapserver: string;
  is_sym_definition_json: boolean;
  database: string;
  schema: string;
  pg_table: string;
  idfield: string;
  geomfield: string;
  coordsys_id: number;
  coordsys_label?: string;
  ms_geomtype_id: number;
  ms_geomtype_label?: string;
  json_columns: string;
  is_location: boolean;
  is_filter: boolean;
  is_disectable: boolean;
  location_search_field: string;
  location_placeholder: string;
  location_result_field: string;
  location_search_mode_id: number;
  location_search_mode_label?: string;
  is_map_selection: boolean;
  map_selection_buffer: number;
  is_data_grid: boolean;
  min_edition_scale: number;
  max_edition_scale: number;
  form_title: string;
  form_display_size_id: number;
  form_insert_size_id: number;
  form_update_size_id: number;
  is_clone: boolean;
  is_automatically_edit: boolean;
  is_multi_edit: boolean;
  is_multi_geom: boolean;
  select_field: string;
  default_form_json_def: string;
  default_form_js_def: string;
  default_form_css_def: string;
  published_form_json_def: string;
  published_form_js_def: string;
  published_form_css_def: string;
  custom_form_json_def: string;
  custom_form_js_def: string;
  custom_form_css_def: string;
  is_geom_selection: boolean;
  is_geom_updatable: boolean;
  datasource_id: number;
  is_geometry_layer: boolean;
  is_requestor_layer: boolean;
  is_specific_table_form: boolean;
  schema_form: string;
  table_form: string;
  is_specific_table_requestor: boolean;
  schema_requestor: string;
  table_requestor: string;
  pictogram: string;
  isPictogramExist: boolean;
  legend: string;
  isLegendExist: boolean;
}
