import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { DatasourceTreeviewPgSchema } from './DatasourceTreeviewPgSchema';
import { DatasourceTreeviewPgTable } from './DatasourceTreeviewPgTable';
import { DatasourceTreeviewPgTableList } from './DatasourceTreeviewPgTableList';
import { Injector } from '@angular/core';

export class DatasourceTreeviewPgMaterializedViewList extends DatasourceTreeviewPgTableList {

  /**
   * Constructeur
   */
  constructor(
    vitisCheck: VitisCheck,
    vitisRequest: VitisRequest,
    injector: Injector,
  ) {
    super(vitisCheck, vitisRequest, injector);
    this.name = 'SUBTITLE_DATASOURCE_TREE_MATERIALIZED_VIEWS';
  }
  uId: number;
  oParent: DatasourceTreeviewPgSchema;
  collapsed: boolean = false;
  icon: string = 'icon-vmap-materialized-views';
  name: string;
  aChildrenTypes = [DatasourceTreeviewPgTable];
  aChildrenObjects = [];
  route: string = DatasourceTreeviewPgMaterializedViewList.getRoute();

  messageMapping: any = {
    '=0': 'SUBTITLE_DATASOURCE_TREE_MATERIALIZED_VIEWS',
    'other': 'SUBTITLE_DATASOURCE_TREE_MATERIALIZED_VIEWS_MULTI'
  };

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'pgmatviewlist';
  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/matviews`;

      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          order_by: 'matviewname',
          datasource_id: this.oParent.oParent.oParent.datasourceId,
          database: this.oParent.oParent.name,
          schema: this.oParent.name,
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              // Ajoute les connexions issue de la base
              for (let i = 0; i < oResult.data.length; i++) {
                const matview: any = oResult.data[i];
                matview.icon = 'icon-vmap-materialized-view';
                const child = new DatasourceTreeviewPgTable(this.vitisCheck, this.vitisRequest, this.injector, matview);
                this.aChildrenObjects.push(child);
              }
            }

            resolve(this.aChildrenObjects);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourcePgMaterializedViewListForm {
    const oFormValues: DatasourcePgMaterializedViewListForm = {};

    return oFormValues;
  }
}


export class DatasourcePgMaterializedViewListForm {
}
