import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { DatasourceTreeviewPgPublicationList } from './DatasourceTreeviewPgPublicationList';
import { Injector } from '@angular/core';
import { AdminDatasourceService } from '../../services/admin-datasource.service';
import { environment } from 'src/environments/environment';
import { AdminDatasourceComponent } from '../../admin-datasource.component';

export class DatasourceTreeviewPgPublication {
  uId: number;
  icon: string = 'icon-vmap-bx-log-in-circle';

  publicationId: number;
  title: string;
  name: string;
  datasourceId: number;
  dateMaj: Date;
  extent: string;
  active: boolean;
  minLayerScale: number;
  maxLayerScale: number;
  definitionSymJson: string;
  definitionSymMapserver: string;
  isSymDefinitionJson: boolean;
  database: string;
  schema: string;
  pgTable: string;
  idfield: string;
  geomfield: string;
  coordsysId: number;
  msGeomtypeId: number;

  // Services
  private adminDatasourceService: AdminDatasourceService;

  aRightClickMenu = [{
    icon: 'icon-vmap-layers-half',
    label: 'ADMIN_DATASOURCE_BTN_UPDATE_PG_PUBLICATION',
    event: 'updatePgPublication',
  },
  {
    icon: 'fa-trash',
    label: 'ADMIN_DATASOURCE_BTN_DELETE_PG_PUBLICATION',
    event: 'deletePgPublication',
  },
  {
    icon: 'fa fa-refresh',
    label: 'ADMIN_DATASOURCE_BTN_REFRESH',
    event: 'refreshData',
  }];

  route: string = DatasourceTreeviewPgPublication.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'pgpublication/info';
  }

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: any,
  ) {
    this.setData(datas);
    this.adminDatasourceService = this.injector.get(AdminDatasourceService);
  }

  /**
   * Rafraichit les données de l'objet
   */
  public refreshData() {
    const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/pgpublications/${this.publicationId}`;
    this.vitisRequest.ajaxRequestPromise({
      url: sUrl,
      method: 'GET'
    }).then((resp: any) => {
      if (this.vitisCheck.isObject(resp.data)) {
        const oResult = resp.data;
        if (oResult.status == 1 && oResult.data) {
          this.setData(oResult.data);
          if (this.vitisCheck.isDefAndNotNull(this.adminDatasourceService)) {
            this.adminDatasourceService.refreshRightComponent(this.uId); // Demande à rafraichir le composant de droite
          }
        }
      }
    });
  }

  /**
   * Met les valeurs du webservice dans l'objet
   * @param datas
   */
  private setData(datas: any) {
    this.publicationId = datas.publication_id;
    this.title = datas.title;
    this.name = datas.name;
    this.datasourceId = datas.datasource_id;
    this.dateMaj = datas.datemaj;
    this.extent = datas.extent;
    this.active = datas.active;
    this.minLayerScale = datas.min_layer_scale;
    this.maxLayerScale = datas.max_layer_scale;
    this.definitionSymJson = datas.definition_sym_json;
    this.definitionSymMapserver = datas.definition_sym_mapserver;
    this.isSymDefinitionJson = datas.is_sym_definition_json;
    this.database = datas.database;
    this.schema = datas.schema;
    this.pgTable = datas.pg_table;
    this.idfield = datas.idfield;
    this.geomfield = datas.geomfield;
    this.coordsysId = datas.coordsys_id;
    this.msGeomtypeId = datas.ms_geomtype_id;
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourcePgPublicationForm {
    const oFormValues: DatasourcePgPublicationForm = {
      publication_id: this.publicationId,
      title: this.title,
      name: this.name,
      datasource_id: this.datasourceId,
      datemaj: this.dateMaj,
      extent: this.extent,
      active: this.active,
      min_layer_scale: this.minLayerScale,
      max_layer_scale: this.maxLayerScale,
      definition_sym_json: this.definitionSymJson,
      definition_sym_mapserver: this.definitionSymMapserver,
      is_sym_definition_json: this.isSymDefinitionJson,
      database: this.database,
      schema: this.schema,
      pg_table: this.pgTable,
      idfield: this.idfield,
      geomfield: this.geomfield,
      coordsys_id: this.coordsysId,
      ms_geomtype_id: this.msGeomtypeId
    };

    return oFormValues;
  }

  /**
   * Lance le formulaire de mise à jour de l'objet
   * @param {AdminDatasourceComponent} scope
   */
  public updatePgPublication(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewPgPublication.getRoute(),
      formName: 'update',
      uId: this.uId
    });
  }

  /**
   * Supprimer une couche postgres
   * @param scope
   */
  public deletePgPublication(scope: AdminDatasourceComponent) {
    this.adminDatasourceService.deletePublication(this.publicationId, this.uId);
  }
}


export class DatasourcePgPublicationForm {
  publication_id: number;
  title: string;
  name: string;
  datasource_id: number;
  datemaj: Date;
  extent: string;
  active: boolean;
  min_layer_scale: number;
  max_layer_scale: number;
  definition_sym_json: string;
  definition_sym_mapserver: string;
  is_sym_definition_json: boolean;
  database: string;
  schema: string;
  pg_table: string;
  idfield: string;
  geomfield: string;
  coordsys_id: number;
  ms_geomtype_id: number;
}
