import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { DatasourceTreeviewPgPublication } from './DatasourceTreeviewPgPublication';
import { DatasourceTreeviewPgTable } from './DatasourceTreeviewPgTable';
import { Injector } from '@angular/core';
import { AdminDatasourceComponent } from '../../admin-datasource.component';

export class DatasourceTreeviewPgPublicationList {

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
  ) {
    this.name = 'Publications';
  }
  uId: number;
  oParent: DatasourceTreeviewPgTable;
  collapsed: boolean = false;
  icon: string = 'icon-vmap-arrow-redo';
  name: string;
  aChildrenTypes = [DatasourceTreeviewPgPublication];
  aChildrenObjects = [];
  aRightClickMenu = [{
    icon: 'icon-vmap-layers-half',
    label: 'ADMIN_DATASOURCE_BTN_ADD_PG_PUBLICATION',
    event: 'createPgPublication',
  }];
  route: string = DatasourceTreeviewPgPublicationList.getRoute();

  messageMapping: any = {
    '=0': 'SUBTITLE_DATASOURCE_TREE_PUBLICATIONS',
    'other': 'SUBTITLE_DATASOURCE_TREE_PUBLICATIONS_MULTI'
  };

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'pgpublicationlist';
  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/pgpublications`;
      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          order_by: 'name',
          filter: {
            'relation': 'AND',
            'operators': [
              { 'column': 'database', 'compare_operator': '=', 'value': this.oParent.oParent.oParent.oParent.name },
              { 'column': 'schema', 'compare_operator': '=', 'value': this.oParent.oParent.oParent.name },
              { 'column': 'pg_table', 'compare_operator': '=', 'value': this.oParent.name },
              {
                'column': 'datasource_id',
                'compare_operator': '=',
                'value': this.oParent.oParent.oParent.oParent.oParent.datasourceId
              }
            ]
          }
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              // Ajoute les connexions issue de la base
              for (let i = 0; i < oResult.data.length; i++) {
                const pgPublication = oResult.data[i];

                const child = new DatasourceTreeviewPgPublication(this.vitisCheck, this.vitisRequest, this.injector, pgPublication);
                this.aChildrenObjects.push(child);
              }
            }

            resolve(this.aChildrenObjects);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourcePgPublicationListForm {
    const oFormValues: DatasourcePgPublicationListForm = {};

    return oFormValues;
  }

  /**
   * Créé une publication de type postgres
   */
  public createPgPublication(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewPgPublication.getRoute(),
      formName: 'insert',
      uId: this.uId
    });
  }

  /**
   * Initialise les valeurs du formulaire insert enfant
   */
  public getChildrenInitialFormValues(): any {
    const oFormValues: any = {
      database: this.oParent.oParent.oParent.oParent.name,
      schema: this.oParent.oParent.oParent.name,
      pg_table: this.oParent.name,
      datasource_id: this.oParent.oParent.oParent.oParent.oParent.datasourceId
    };

    return oFormValues;
  }
}


export class DatasourcePgPublicationListForm {}
