import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { DatasourceTreeviewPgSchema } from './DatasourceTreeviewPgSchema';
import { DatasourceTreeviewPgTable } from './DatasourceTreeviewPgTable';
import { Injector } from '@angular/core';
import { AdminDatasourceComponent } from '../../admin-datasource.component';

export class DatasourceTreeviewPgTableList {

  public aRightClickMenu = [{
    icon: 'fa fa-refresh',
    label: 'ADMIN_DATASOURCE_BTN_REFRESH',
    event: 'refreshPgTableList',
  }];

  /**
   * Constructeur
   */
  constructor(
    protected vitisCheck: VitisCheck,
    protected vitisRequest: VitisRequest,
    protected injector: Injector,
  ) {
    this.name = 'SUBTITLE_DATASOURCE_TREE_TABLES';
  }
  uId: number;
  oParent: DatasourceTreeviewPgSchema;
  collapsed: boolean = false;
  icon: string = 'icon-vmap-tables';
  name: string;
  aChildrenTypes = [DatasourceTreeviewPgTable];
  aChildrenObjects = [];
  route: string = DatasourceTreeviewPgTableList.getRoute();

  messageMapping: any = {
    '=0': 'SUBTITLE_DATASOURCE_TREE_TABLES',
    'other': 'SUBTITLE_DATASOURCE_TREE_TABLES_MULTI'
  };

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'pgtablelist';
  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/tables`;

      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          order_by: 'tablename',
          datasource_id: this.oParent.oParent.oParent.datasourceId,
          database: this.oParent.oParent.name,
          schema: this.oParent.name,
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              // Ajoute les connexions issue de la base
              for (let i = 0; i < oResult.data.length; i++) {
                const table = oResult.data[i];

                const child = new DatasourceTreeviewPgTable(this.vitisCheck, this.vitisRequest, this.injector, table);
                this.aChildrenObjects.push(child);
              }
            }
            resolve(this.aChildrenObjects);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourcePgTableListForm {
    const oFormValues: DatasourcePgTableListForm = {};

    return oFormValues;
  }

  /**
   * Rafraichit la liste des tables.
   * @param scope
   */
  public refreshPgTableList(scope: AdminDatasourceComponent) {
    // Rechargement de la liste des tables.
    this.aChildrenObjects.length = 0;
    scope.getChildrens(this);
    // Page d'accueil de la liste des tables.
    scope.openRoute({
      route: this.route,
      formName: 'display',
      uId: this.uId
    });
  }
}

export class DatasourcePgTableListForm {
}
