
import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { DatasourceTreeviewPgSchema } from './DatasourceTreeviewPgSchema';
import { DatasourceTreeviewPgTable } from './DatasourceTreeviewPgTable';
import { DatasourceTreeviewPgTableList } from './DatasourceTreeviewPgTableList';
import { Injector } from '@angular/core';

export class DatasourceTreeviewPgViewList extends DatasourceTreeviewPgTableList {

  /**
   * Constructeur
   */
  constructor(
    vitisCheck: VitisCheck,
    vitisRequest: VitisRequest,
    injector: Injector,
  ) {
    super(vitisCheck, vitisRequest, injector);
    this.name = 'SUBTITLE_DATASOURCE_TREE_VIEW';
  }
  uId: number;
  oParent: DatasourceTreeviewPgSchema;
  collapsed: boolean = false;
  icon: string = 'icon-vmap-views';
  name: string;
  aChildrenTypes = [DatasourceTreeviewPgTable];
  aChildrenObjects = [];
  route: string = DatasourceTreeviewPgViewList.getRoute();

  messageMapping: any = {
    '=0': 'SUBTITLE_DATASOURCE_TREE_VIEWS',
    'other': 'SUBTITLE_DATASOURCE_TREE_VIEWS_MULTI'
  };

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'pgviewlist';
  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/views`;

      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          order_by: 'viewname',
          datasource_id: this.oParent.oParent.oParent.datasourceId,
          database: this.oParent.oParent.name,
          schema: this.oParent.name,
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              // Ajoute les connexions issue de la base
              for (let i = 0; i < oResult.data.length; i++) {
                const view: any = oResult.data[i];
                view.icon = 'icon-vmap-view';

                const child = new DatasourceTreeviewPgTable(this.vitisCheck, this.vitisRequest, this.injector, view);
                this.aChildrenObjects.push(child);
              }
            }

            resolve(this.aChildrenObjects);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): DatasourcePgViewListForm {
    const oFormValues: DatasourcePgViewListForm = {
      title: 'DATASOURCE_FORM_PG_VIEW_TITLE'
    };

    return oFormValues;
  }
}


export class DatasourcePgViewListForm {
}
