import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { AdminDatasourceComponent } from '../../admin-datasource.component';
import { DatasourceTreeviewPgConnection } from './DatasourceTreeviewPgConnection';
import { Injector } from '@angular/core';

export class DatasourceTreeviewPostgis {
  uId: number;
  name: string;
  id: number;
  icon: string = 'icon-vmap-postgresql';
  collapsed: boolean = false;
  aChildrenTypes = [DatasourceTreeviewPgConnection];
  aChildrenObjects = [];
  aRightClickMenu = [{
    icon: 'icon-vmap-link-1',
    label: 'ADMIN_DATASOURCE_BTN_ADD_PG_CONNECTION',
    event: 'createPgConnection',
  }];
  route: string = DatasourceTreeviewPostgis.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'postgis';
  }

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: any,
  ) {
    this.name = datas.label;
    this.id = datas.datasourcetype_id;
  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];
      const sUrl = `${environment.properties.app_api_url}/vmap/datasources`;
      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          order_by: 'name',
          attributs: 'datasource_id|name|description|pg_server|pg_port|pg_login|pg_password|pg_database|pg_public',
          filter: { 'relation': 'AND', 'operators': [{ 'column': 'datasourcetype_id', 'compare_operator': '=', 'value': this.id }] },
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              const datasources = {};
              // Ajoute les connexions issue de la base
              for (let i = 0; i < oResult.data.length; i++) {
                const datasource = oResult.data[i];

                if (datasource.pg_public === false) {
                  datasource.icon = 'icon-vmap-link';
                }

                const child = new DatasourceTreeviewPgConnection(this.vitisCheck, this.vitisRequest, this.injector, datasource);
                this.aChildrenObjects.push(child);
              }
            }

            resolve(this.aChildrenObjects);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Lance le formulaire de création de l'objet enfant
   * @param {AdminDatasourceComponent} scope
   */
  public createPgConnection(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewPgConnection.getRoute(),
      formName: 'insert',
      uId: this.uId
    });
  }

  /**
   * Appelé à la sélection
   */
  public onSelect() {

  }
}
