import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { Injector } from '@angular/core';
import { AdminDatasourceComponent } from '../../admin-datasource.component';
import { DatasourceTreeviewVectortilesConnection } from './DatasourceTreeviewVectortilesConnection';

export class DatasourceTreeviewVectortiles {
  uId: number;
  name: string;
  id: number;
  icon: string = 'icon-vmap-arrow-redo';
  collapsed: boolean = false;
  aChildrenTypes = [DatasourceTreeviewVectortilesConnection];
  aChildrenObjects = [];
  aRightClickMenu = [{
    icon: '',
    label: 'ADMIN_DATASOURCE_BTN_ADD_VECTORTILES_CONNECTION',
    event: 'insertConnection',
  }];
  route: string = DatasourceTreeviewVectortiles.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'vectortiles';
  }

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: any,
  ) {
    this.name = datas.label;
    this.id = datas.datasourcetype_id;
  }

  /**
   * Charge les objets enfants
   * @returns {Promise}
   */
  public loadChildrens(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.aChildrenObjects = [];
      const sUrl = `${environment.properties.app_api_url}/vmap/datasources`;
      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          attributs: 'datasource_id|datasource_name|description|flux_url|flux_options|flux_login|flux_password|name|headers',
          order_by: 'name',
          filter: { 'relation': 'AND', 'operators': [{ 'column': 'datasourcetype_id', 'compare_operator': '=', 'value': this.id }] }
        },
        method: 'GET'
      }).then((resp: any) => {
        if (this.vitisCheck.isObject(resp.data)) {
          const oResult = resp.data;
          if (oResult.status == 1 && oResult.data) {
            if (oResult.data && oResult.data.length > 0) {
              for (let i = 0; i < oResult.data.length; i++) {
                const service = oResult.data[i];
                this.aChildrenObjects.push(new DatasourceTreeviewVectortilesConnection(this.vitisCheck, this.vitisRequest, this.injector, service));
              }
            }

            resolve(this.aChildrenObjects);
          } else {
            reject(oResult);
          }
        } else {
          reject(resp);
        }
      });
    });
  }

  /**
   * Returne la liste des objets enfants
   * @returns {Array<any>}
   */
  public getChildrens(): Array<any> {
    return this.aChildrenObjects;
  }

  /**
   * Lance le formulaire de création de l'objet enfant
   * @param {AdminDatasourceComponent} scope
   */
  public insertConnection(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewVectortilesConnection.getRoute(),
      formName: 'insert',
      uId: this.uId
    });
  }
}
