import { LocalStorage, VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { DatasourceTreeviewWMTSConnection } from './DatasourceTreeviewWMTSConnection';
import { Injector } from '@angular/core';
import { AdminDatasourceComponent } from '../../admin-datasource.component';
import { environment } from 'src/environments/environment';
import { AdminDatasourceService } from '../../services/admin-datasource.service';

export class DatasourceTreeviewWMTSLayer {
  uId: number;
  oParent: DatasourceTreeviewWMTSConnection;
  icon: string = 'icon-vmap-layers-half';

  public layer_id: number;
  public name: string;
  public datasource_id: number;
  public datasource: number;
  public datemaj: Date;
  public source_id: number;
  public source_name: string;
  public extent: string;
  public min_layer_scale: number;
  public max_layer_scale: number;
  public active: boolean;
  public is_background_layer: boolean;
  public theme_label: string;
  public theme_id: number;
  public layer_name: string;
  public matrix_set: string;
  public layer_style: string;
  public layer_format: string;
  public layer_options: string;
  pictogram: string;
  isPictogramExist: boolean;
  legend: string;
  isLegendExist: boolean;

  private adminDatasourceService: AdminDatasourceService;
  private localStorage: LocalStorage;

  aRightClickMenu = [{
    icon: 'icon-vmap-layers-half',
    label: 'ADMIN_DATASOURCE_BTN_UPDATE_WMTS_LAYER',
    event: 'updateWMTSLayer',
  }, {
    icon: 'fa fa-clone',
    label: 'ADMIN_DATASOURCE_BTN_CLONE_LAYER',
    event: 'cloneWMTSLayer'
  }, {
    icon: 'fa-trash',
    label: 'ADMIN_DATASOURCE_BTN_DELETE_WMTS_LAYER',
    event: 'deleteWMTSLayer',
  }];
  route: string = DatasourceTreeviewWMTSLayer.getRoute();

  /**
   * Retourne la route
   * @returns
   */
  static getRoute() {
    return 'wmtslayer/info';
  }

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private injector: Injector,
    datas: any,
  ) {
    this.setData_(datas);
    this.adminDatasourceService = this.injector.get(AdminDatasourceService);
    this.localStorage = this.injector.get(LocalStorage);
  }

  /**
   * Rafraichit les données de l'objet
   * @param scope
   */
  public refreshData() {
    const sUrl = `${environment.properties.app_api_url}/vmap/layers/${this.layer_id}`;
    this.vitisRequest.ajaxRequestPromise({
      url: sUrl,
      method: 'GET'
    }).then((resp: any) => {
      if (this.vitisCheck.isObject(resp.data)) {
        const oResult = resp.data;
        if (oResult.status == 1 && oResult.data) {
          this.setData_(oResult.data);
          if (this.vitisCheck.isDefAndNotNull(this.adminDatasourceService)) {
            this.adminDatasourceService.refreshRightComponent(this.uId); // Demande à rafraichir le composant de droite
          }
        }
      }
    });
  }

  /**
   * Met les valeurs du webservice dans l'objet
   * @param datas
   */
  private setData_(datas: any) {
    this.layer_id = datas.layer_id;
    this.name = datas.name;
    this.datasource_id = datas.datasource_id;
    this.datasource = datas.datasource;
    this.datemaj = datas.datemaj;
    this.source_id = datas.source ? datas.source.source_id : null;
    this.source_name = datas.source ? datas.source.name : null;
    this.extent = datas.extent;
    this.min_layer_scale = datas.min_layer_scale;
    this.max_layer_scale = datas.max_layer_scale;
    this.active = datas.active;
    this.is_background_layer = datas.is_background_layer;
    this.theme_label = datas.theme ? datas.theme.label : null;
    this.theme_id = datas.theme ? datas.theme.theme_id : null;
    this.layer_name = datas.layer_name;
    this.matrix_set = datas.matrix_set;
    this.layer_style = datas.layer_style;
    this.layer_format = datas.layer_format;
    this.layer_options = JSON.parse(datas.layer_options);
    this.pictogram = datas.pictogram ? datas.pictogram : null;
    this.isPictogramExist = this.vitisCheck.isDefAndNotNull(datas.pictogram);
    this.legend = datas.legend ? datas.legend : null;
    this.isLegendExist = this.vitisCheck.isDefAndNotNull(datas.legend);
  }


  /**
   * Récupère les valeurs pour le formulaire
   */
  public getFormValues(): Promise<DatasourceWMTSLayerForm> {
    return new Promise((resolve, reject) => {
      const oFormValues: DatasourceWMTSLayerForm = {
        layer_id: this.layer_id,
        name: this.name,
        datasource_id: this.datasource_id,
        datasource: this.datasource,
        datemaj: this.datemaj,
        source_id: this.source_id,
        source_name: this.source_name,
        extent: this.extent,
        min_layer_scale: this.min_layer_scale,
        max_layer_scale: this.max_layer_scale,
        active: this.active,
        is_background_layer: this.is_background_layer,
        theme_label: this.theme_label,
        theme_id: this.theme_id,
        layer_name: this.layer_name,
        matrix_set: this.matrix_set,
        layer_style: this.layer_style,
        layer_format: this.layer_format,
        layer_options: this.layer_options,
        pictogram: this.pictogram,
        isPictogramExist: this.isPictogramExist,
        legend: this.legend,
        isLegendExist: this.isLegendExist
      };

      if ((this.vitisCheck.isDefAndNotNull(oFormValues.layer_options) && oFormValues.layer_options != '')
        || !this.vitisCheck.isDefAndNotNull(this.oParent) || !this.vitisCheck.isFunction(this.oParent.getCapabilitiesWMTS)) {
        resolve(oFormValues);
      } else {
        this.oParent.getCapabilitiesWMTS().then((oCapabilities) => {
          oFormValues.layer_options = oCapabilities.json_capabilities;
          resolve(oFormValues);
        });
      }
    });
  }

  /**
   * Lance le formulaire de mise à jour de l'objet
   * @param {AdminDatasourceComponent} scope
   */
  public updateWMTSLayer(scope: AdminDatasourceComponent) {
    scope.openRoute({
      route: DatasourceTreeviewWMTSLayer.getRoute(),
      formName: 'update',
      uId: this.uId
    });
  }

  /**
   * Sauvegarde le uId et renvoie vers le formulaire d'ajout
   */
  public cloneWMTSLayer(scope: AdminDatasourceComponent) {
    this.localStorage.set('idClone', this.uId);
    const oObject = this.adminDatasourceService.getObjectById(this.uId);
    scope.openRoute({
      route: DatasourceTreeviewWMTSLayer.getRoute(),
      formName: 'insert',
      uId: oObject.oParent.uId
    });
  }

  /**
   * Supprime une couche WMTS
   * @param scope
   */
  public deleteWMTSLayer(scope: AdminDatasourceComponent) {
    this.adminDatasourceService.deleteLayer(this.layer_id, this.uId);
  }

}

export class DatasourceWMTSLayerForm {
  layer_id: number;
  name: string;
  datasource_id: number;
  datasource: any;
  datemaj: Date;
  source_id: number;
  source_name: string;
  extent: string;
  min_layer_scale: number;
  max_layer_scale: number;
  active: boolean;
  is_background_layer: boolean;
  theme_label: string;
  theme_id: number;
  layer_name: string;
  matrix_set: string;
  layer_style: string;
  layer_format: string;
  layer_options: string;
  pictogram: string;
  isPictogramExist: boolean;
  legend: string;
  isLegendExist: boolean;
}
