import { Component, OnDestroy, OnInit } from '@angular/core';
import { Subscription } from 'rxjs';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';

@Component({
  selector: 'app-vmap-controls-maps-button',
  templateUrl: './vmap-controls-maps-button.component.html',
  styleUrls: ['./vmap-controls-maps-button.component.scss']
})
export class VmapControlsMapsButtonComponent implements OnInit, OnDestroy {

  public bPanelOpen: boolean = false;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsService: VmapToolsService,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    this.subscriptions.push(this.vmapToolsService.getActiveLeftPanelEmiter().subscribe((sTool: string) => {
      this.bPanelOpen = (sTool === 'control-maps');
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Bascule l'ouverture du panneau
   */
  public togglePanel() {
    this.bPanelOpen = !this.bPanelOpen;

    // Panneau de gauche ouvert à la taille 2 ou 0 si fermé
    if (this.bPanelOpen) {
      this.vmapToolsService.setLeftPanel(2, 'control-maps');
    } else {
      this.vmapToolsService.setLeftPanel(0, '');
    }
  }
}
