import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { Map } from 'ol';
import { Subscription } from 'rxjs';
import { IsSmallScreenService } from 'src/app/vitis/core/services/utils/is-small-screen/is-small-screen.service';
import { environment } from 'src/environments/environment';

@Component({
  selector: 'app-vmap-controls-left-buttons',
  templateUrl: './vmap-controls-left-buttons.component.html',
  styleUrls: ['./vmap-controls-left-buttons.component.scss']
})
export class VmapControlsLeftButtonsComponent implements OnInit, OnDestroy {

  @Input() olMap: Map;
  @Input() isWidget: boolean;
  @Input() haveLayersPanel: boolean;
  @Input() haveMapsPanel: boolean;

  layers_tool: boolean = false;

  public isSmallScreen;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  /**
   * Constructeur
   */
  constructor(
    private route: ActivatedRoute,
    private isSmallScreenService: IsSmallScreenService
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {

    this.isSmallScreen = this.isSmallScreenService.getIsSmallScreen();
    this.subscriptions.push(this.isSmallScreenService.isSmallScreenObservable.subscribe(isSmallScreen => {
      this.isSmallScreen = isSmallScreen;
    }));

    this.subscriptions.push(this.route.queryParams.subscribe(oParams => {
      // Affiche les outils de couches si layers_tool === 'true'
      if (oParams.layers_tool === 'true') {
        this.layers_tool = true;
      }
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }
}
