import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Map } from 'ol';
import { Subscription } from 'rxjs';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { VmapCartoService } from 'src/modules/vmap/components/carto/services/vmap-carto/vmap-carto.service';

@Component({
  selector: 'app-vmap-maps-panel',
  templateUrl: './vmap-maps-panel.component.html',
  styleUrls: ['./vmap-maps-panel.component.scss']
})
export class VmapMapsPanelComponent implements OnInit, OnDestroy {

  @Input() public olMap: Map;
  @Input() isWidget: boolean;

  public bPanelOpen: boolean = false;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  /**
   * Retourne le map_id d'après l'URL
   */
  get mapId() {
    return this.route.snapshot.queryParams['map_id'];
  }

  /**
   * Constructeur
   */
  constructor(
    private route: ActivatedRoute,
    private vmapToolsService: VmapToolsService,
    private vmapCartoService: VmapCartoService
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    this.subscriptions.push(this.vmapToolsService.getActiveLeftPanelEmiter().subscribe((sTool: string) => {
      this.bPanelOpen = (sTool === 'control-maps');
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Ferme le panneau de gauche
   */
  public closePanel() {
    this.bPanelOpen = false;

    this.vmapToolsService.setLeftPanel(0, '');
  }

  /**
   * Load une carte
   */
  public async loadMap(map_id: string | number) {
    // Charge la carte sélectionnée.
    this.vmapCartoService.loadMap(map_id, this.olMap);

    // Ferme l'outil
    this.closePanel();
  }
}
