import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { MapCoreDrawService } from 'src/app/vitis/map-core/services/map-core-draw/map-core-draw.service';
import { VmapToolsMeasureService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-measure/vmap-tools-measure.service';

@Component({
  selector: 'app-vmap-add-measure-mobile',
  templateUrl: './vmap-add-measure-mobile.component.html',
  styleUrls: ['./vmap-add-measure-mobile.component.scss']
})
export class VmapAddMeasureMobileComponent implements OnInit {

  @Input() olMap;

  @Output() cancelEditMeasure = new EventEmitter();

  /**
   * Constructeur
   */
  constructor(
    private mapCoreDrawService: MapCoreDrawService,
    private vmapToolsMeasureService: VmapToolsMeasureService
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    if (!this.olMap.get('measure_layer')) {
      this.initDrawing();
    }
  }

  /**
   * Ajout des mesures dans la couche measure_layer
   * @param feature
   */
  public addMeasure(feature) {
    this.olMap.get('measure_layer').getSource().addFeature(feature);
    this.olMap.set('measure_features', this.vmapToolsMeasureService.infosMeasures);
    this.cancelEditMeasure.emit();
  }

  /**
   * Ferme l'outil d'insertion des mesures
   * @param e
   */
  public cancelInsertMeasure(e) {
    this.cancelEditMeasure.emit();
  }

  /**
   * Initialise le vector Layer pour ajouter les mesures à la carte
   */
  private initDrawing(): void {
    this.olMap.set('measure_layer', this.mapCoreDrawService.initDrawLayer(this.olMap, null, this.vmapToolsMeasureService.olMeasureStyle));
    this.olMap.set('measure_features', this.vmapToolsMeasureService.infosMeasures);
  }

}
