import { Component, OnInit, Input, Output, EventEmitter } from '@angular/core';
import { Feature } from 'ol';
import { Geometry } from 'ol/geom';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { LayerToolsService } from '../../../services/layer-tools/layer-tools.service';

@Component({
  selector: 'app-vmap-edit-geometry-mobile',
  templateUrl: './vmap-edit-geometry-mobile.component.html',
  styleUrls: ['./vmap-edit-geometry-mobile.component.scss']
})
export class VmapEditGeometryMobileComponent implements OnInit {

  @Input() olMap;
  @Input() oLayer;
  @Input() open: boolean;

  @Output() cancelEditGeometry = new EventEmitter();

  public openFormFeature = false;
  public sDatasFeature = {};
  public sDefinitionFeature;

  private ewktFeature;
  private lastFeatureEdit;
  public updateFeature: Feature<Geometry>;

  /**
   * Constructeur
   */
  constructor(
    private mapUtils: MapCoreUtilsService,
    private layerToolsService: LayerToolsService
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    this.sDefinitionFeature = this.oLayer.published_form_json_def;
    this.sDefinitionFeature.insert.event = 'insertFeature()';
    this.sDefinitionFeature.insert.eventContainer = this;

    const isFormButtonsSet = this.sDefinitionFeature.insert.rows.find(field => field.fields[0].type === 'button');
    if (!isFormButtonsSet) {
      this.sDefinitionFeature.insert.rows.push({
        fields: [{
          type: 'button',
          class: 'btn-ungroup btn-group-sm',
          nb_cols: 12,
          buttons: [{
            type: 'submit',
            name: 'form_submit',
            label: 'FORM_CREATE',
            class: 'btn-primary'
          }]
        }]
      });
    }
  }

  /**
   * Retour de la feature précédemment crééer et ouverture de son formulaire de saisie
   * @param feature
   */
  public addFeature(feature: Feature<Geometry>) {
    if (feature && feature.getGeometry()) {
      this.ewktFeature = this.mapUtils.getEWKTFromGeomAndProj(feature.clone().getGeometry(), this.olMap, this.oLayer.coordsys_id);
      this.lastFeatureEdit = feature;

      // Ouverture du formulaire pour l'insertion d'une nouvelle feature dans la couche
      this.openFormFeature = true;
    } else {

    }
  }

  /**
   * Event appelé à chaque changement du formulaire de saisie
   * @param changes
   */
  public formFeatureChanges(changes: any) {
    this.sDatasFeature = changes;
  }

  /**
   * Validation du formulaire et insertion de la feature en base de donnée
   */
  public insertFeature() {
    if (this.oLayer.is_geometry_layer && this.oLayer.is_geom_selection) {
      this.sDatasFeature = {
        ...this.sDatasFeature,
        [this.oLayer.geomfield]: this.ewktFeature
      };
    }
    this.layerToolsService.insertFeatureLayer(this.sDatasFeature, this.oLayer.layer_id, this.olMap)
      .then(e => this.cancelInsertGeometry());
  }

  /**
   * Event appelé lors de la fermeture du formulaire sans validation
   */
  public formEventClose() {
    this.openFormFeature = false;
    this.updateFeature = new Feature({ geometry: this.lastFeatureEdit.getGeometry() });
  }

  /**
   * Ferme l'outil d'insertion d'une nouvelle géométrie à la couche
   */
  public cancelInsertGeometry() {
    this.cancelEditGeometry.emit();
    this.openFormFeature = false;
    this.sDatasFeature = {};
    this.ewktFeature = null;
  }
}

