import { Component, OnInit, Input, ViewChild, AfterViewInit, Output, EventEmitter, OnDestroy } from '@angular/core';
import { FormJS, FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { VitisClone, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { Subscription } from 'rxjs';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';

@Component({
  selector: 'app-vmap-fullscreen-form-mobile',
  templateUrl: './vmap-fullscreen-form-mobile.component.html',
  styleUrls: ['./vmap-fullscreen-form-mobile.component.scss']
})
export class VmapFullscreenFormMobileComponent implements OnInit, AfterViewInit, OnDestroy {

  @ViewChild(FormreaderComponent) oFormReader: FormreaderComponent;

  @Input() open: boolean;
  @Input() sDefinition;
  @Input() sFormName;
  @Input() oFormJS: FormJS;
  @Input() sDatas;

  @Output() formChanges = new EventEmitter();
  @Output() eventClose = new EventEmitter();

  public definition;
  private datas;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  /**
   * Constructeur
   */
  constructor(
    private vitisClone: VitisClone,
    private formreaderSrvc: FormreaderService,
    private vitisRequest: VitisRequest,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {
  }

  /**
   * Appelé après le chargement du DOM
   */
  ngAfterViewInit(): void {
    setTimeout(() => {
      this.definition = this.vitisClone.clone(this.sDefinition);
      this.datas = this.vitisClone.clone(this.sDatas);

      if (this.oFormJS) {
        this.vitisRequest.loadExternalJs(this.oFormJS.sJsPath).then(() => {
          this.oFormReader.initForm(this.definition, this.sFormName, this.datas, null, null, this.oFormJS);
          this.subscriptions.push(this.oFormReader.oFormValues.valueChanges.subscribe(form => {
            this.formChanges.emit(form);
          }));
        });

      } else {
        this.oFormReader.initForm(this.definition, this.sFormName, this.datas);
        this.subscriptions.push(this.oFormReader.oFormValues.valueChanges.subscribe(form => {
          this.formChanges.emit(form);
        }));
      }
    });
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Ferme le panel de formulaire pour les mobiles
   */
  public closePanel() {
    if (this.eventClose) {
      this.eventClose.emit();
    } else {
      this.open = false;
    }
  }
}
