import { Component, OnInit, Input, OnDestroy } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Map } from 'ol';
import { Subscription } from 'rxjs';
import { VmapCartoService } from 'src/modules/vmap/components/carto/services/vmap-carto/vmap-carto.service';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';

@Component({
  selector: 'app-vmap-layers-maps-panel-mobile',
  templateUrl: './vmap-layers-maps-panel-mobile.component.html',
  styleUrls: ['./vmap-layers-maps-panel-mobile.component.scss']
})
export class VmapLayersMapsPanelMobileComponent implements OnInit, OnDestroy {

  @Input() public olMap: Map;
  @Input() public isWidget: boolean;
  @Input() public haveLayersPanel: boolean;

  public layers_tool: boolean = false;
  public maps_tool: boolean = false;

  public bPanelOpen: boolean;
  public selectedTab: number;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsService: VmapToolsService,
    private route: ActivatedRoute,
    private vmapCartoService: VmapCartoService,
  ) { }

  /**
   * Retourne le map_id
   */
  get mapId() {
    return this.olMap ? this.olMap.get('map_id') : null;
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    this.subscriptions.push(this.route.queryParams.subscribe(oParams => {
      if (oParams.layers_tool === 'true') {
        this.layers_tool = true;
      }
      if (oParams.maps_tool === 'true') {
        this.maps_tool = true;
      }
    }));

    // Écoute l'ouverture des menus
    this.subscriptions.push(this.vmapToolsService.getActiveLeftPanelEmiter().subscribe((sTool: string) => {
      this.bPanelOpen = (sTool === 'control-layers-maps-mobile');

      // Menu ouvert par défaut
      if (this.bPanelOpen) {
        if (this.isWidget) {
          if (this.maps_tool) {
            this.selectedTab = 0;
          } else {
            if (this.haveLayersPanel) {
              this.selectedTab = 1;
            } else {
              this.selectedTab = 2;
            }
          }
        } else {
          this.selectedTab = 0;
        }
      }
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Ferme le panneau de gauche
   */
  public closePanel() {
    this.bPanelOpen = false;
    this.vmapToolsService.setLeftPanel(0, '');
  }

  /**
   * Sélection d'un ongle
   * @param index
   */
  public selectTab(index) {
    this.selectedTab = index;
  }

  /**
   * Load une carte
   */
  public loadMap(map_id: string | number) {

    // Change de carte
    this.vmapCartoService.loadMap(map_id, this.olMap);

    // Ferme l'outil
    this.closePanel();
  }
}
