import { Component, OnInit, Input, OnDestroy } from '@angular/core';
import { Map } from 'ol';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { VmapSelectService } from 'src/modules/vmap/components/carto/services/vmap-select/vmap-select.service';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import usersForm from 'src/assets/forms/user_infos/user_infos.json';
import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { Subscription } from 'rxjs';
import { VmapLayer } from 'src/app/vitis/map-core/services/map-models.model';
import { EventsKey } from 'ol/events';
import { unByKey } from 'ol/Observable';
import { VmapToolsMeasureService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-measure/vmap-tools-measure.service';
import { MapCoreDrawService } from 'src/app/vitis/map-core/services/map-core-draw/map-core-draw.service';
import { VmapToolsGeolocationService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-geolocation/vmap-tools-geolocation.service';
import { vitisHelpHeader } from 'src/app/vitis/app.struct';
import { FormJS } from 'src/app/vitis/formreader/formreader.component';
import { ActivatedRoute } from '@angular/router';

interface ModalOptions {
  form_path?: string;
  js_path?: string;
  oFormDefinition?: any;
  oFormValues?: any;
  sModalFormName?: string;
}

@Component({
  selector: 'app-vmap-tools-menu-mobile',
  templateUrl: './vmap-tools-menu-mobile.component.html',
  styleUrls: ['./vmap-tools-menu-mobile.component.scss']
})
export class VmapToolsMenuMobileComponent implements OnInit, OnDestroy {

  @Input() public olMap: Map;
  @Input() public isWidget: boolean;

  public bPanelOpen: boolean;
  public helper_tool: boolean = false; // True en cas de widget si helper_tool dans l'url

  public openUserInfos = false;
  public sDefinitionUser: any;
  public sDatasUser: any = {};

  public oUserModalObject: ModalOptions;

  public sModalFormDefinition: any;
  public oModalFormJS: FormJS = null;
  public sModalFormName: string = 'update';
  public sModalFormData: any = {};
  public bOpenModalForm = false;

  public dropDownHelp = false;
  public dropdownHelpId = `dropdown-help-${new Date().getTime()}`;

  public dropDownUser = false;
  public dropdownUserId = `dropdown-user-${new Date().getTime()}`;

  public openInsertLayer = false;
  public dropDownInsertion = false;
  public dropdownInsertionId = `dropdown-insertion-${new Date().getTime()}`;

  public aInsertLayers: Array<VmapLayer>;
  public currLayerEditGeom: VmapLayer;

  public openDrawMeasures = false;

  public dropDownLocalisation = false;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];
  private mapSubscriptions: EventsKey[] = [];

  /**
   * Retourne true si le tracking GPS est actif
   */
  get trackingGps() {
    return this.geolocationSrvc.isTrackingEnable();
  }

  /**
   * Retourne true s'il faut afficher la précision
   */
  get accuracyGps() {
    return this.geolocationSrvc.isAccuracyEnable();
  }

  /**
   * Retourne les objets de vitisHelpHeader
   */
  get helpObjects() {
    return vitisHelpHeader.objects;
  }

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsService: VmapToolsService,
    private mapSelectService: VmapSelectService,
    private authService: AuthService,
    private vitisRequest: VitisRequest,
    private vitisCheck: VitisCheck,
    private formreaderSrvc: FormreaderService,
    private mapCoreDrawService: MapCoreDrawService,
    private vmapToolsMeasureService: VmapToolsMeasureService,
    private geolocationSrvc: VmapToolsGeolocationService,
    private route: ActivatedRoute,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {
    this.sDefinitionUser = usersForm;
    this.sDefinitionUser.update.eventContainer = this;
    this.sDatasUser = this.authService.getUserInfos();

    // Écoute l'ouverture des menus
    this.subscriptions.push(this.vmapToolsService.getActiveLeftPanelEmiter().subscribe((sTool: string) => {
      this.bPanelOpen = (sTool === 'menu-mobile');
    }));

    // Ecoute les couches sélectionnable de la couche
    this.subscriptions.push(this.mapSelectService.getObservableSelectLayers().subscribe((oSelectedLayers) => {
      // Sélectionne les couches éditables
      this.aInsertLayers = oSelectedLayers.filter(layer => layer.layer_id && layer.have_insert_right === true);
    }));

    this.mapSubscriptions.push(this.olMap.on('change', () => {
      // Ferme les popups de la couche
      this.vmapToolsMeasureService.emptiedInfosMeasures();
      this.olMap.set('measure_features', []);
      this.olMap.set('measure_layer', this.mapCoreDrawService.initDrawLayer(this.olMap, null, this.vmapToolsMeasureService.olMeasureStyle));
    }));

    this.subscriptions.push(this.route.queryParams.subscribe(oParams => {
      if (oParams.helper_tool === 'true') {
        this.helper_tool = true;
      }
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
    this.mapSubscriptions.forEach(mapSubscription => unByKey(mapSubscription));
  }

  /**
   * Ferme le panneau de gauche
   */
  public closePanel() {
    this.dropDownInsertion = false;
    this.dropDownUser = false;
    this.bPanelOpen = false;
    this.vmapToolsService.setLeftPanel(0, '');
  }

  /**
   * Déconnectes l'utilisateur en cours
   */
  public disconnect() {
    this.authService.disconnect();
  }

  /**
   * Evenement appelé avant la validation du formulaire de modification de l'utilisateur
   * @returns
   */
  public beforeEvent() {
    return true;
  }

  /**
   * Evenement appelé lors de la validation du formulaire de modification de l'utilisateur
   */
  public sendSimpleForm() {
    this.vitisRequest.ajaxRequestPromise({
      method: 'PUT',
      url: `${environment.properties.app_api_url}/vitis/users/${this.sDatasUser.user_id}`,
      data: this.formreaderSrvc.transformToDataForm(this.sDatasUser)
    }).catch(e => {
      console.error(e);
    });
  }

  /**
   * Evement appelé après la validation du formulaire de modification de l'utilisateur
   */
  public afterEvent() {
    this.authService.setUserInfos(this.sDatasUser);
  }

  /**
   * Listener lors de la modification du formulaire de l'utilisateur
   * @param form
   */
  public formUsersChanges(form) {
    this.sDatasUser = form;
  }

  /**
   * Listener lors de la modification du formulaire
   * @param form
   */
  public modalFormChanges(form) {
    this.sModalFormData = form;
  }

  /**
   * Ouvre l'outil d'insertion d'une géométrie à la couche
   */
  public drawGeometries(oLayer: VmapLayer) {
    this.closePanel();
    this.openInsertLayer = true;
    this.currLayerEditGeom = oLayer;
  }

  /**
   * Ferme l'outil d'insertion
   */
  public cancelEditGeometry() {
    this.openInsertLayer = false;
    this.currLayerEditGeom = null;
  }

  /**
   * Ouvre l'outil d'insertion d'une mesure
   */
  public drawMeasures() {
    this.closePanel();
    this.openDrawMeasures = true;
  }

  /**
   * Ferme l'outil d'insertion des mesures
   */
  public cancelEditMeasure() {
    this.openDrawMeasures = false;
  }

  /**
   * Change l'état du traceur gps
   */
  public changeTrackingState() {
    this.geolocationSrvc.setTrackingGps(!this.trackingGps);
  }

  /**
   * Change l'état de la précision gps
   */
  public changeAccuraryState() {
    this.geolocationSrvc.setAccuracyGps(!this.accuracyGps);
  }

  /**
   * Prépare les données et ouvre la modale infos utilisateur avec displayModalForm
   */
  public displayUserModalForm() {
    this.oUserModalObject = {
      oFormDefinition: this.sDefinitionUser,
      oFormValues: this.sDatasUser,
    };
    this.displayModalForm(this.oUserModalObject);
  }

  /**
   * Affiche un formulaire en modale pleine page
   * @param oModalObject
   */
  public displayModalForm(oModalObject: ModalOptions) {
    if (!this.bOpenModalForm) {
      if (oModalObject) {

        this.sModalFormData = oModalObject.oFormValues ?? {};
        this.sModalFormName = oModalObject.sModalFormName ?? 'update';

        if (oModalObject.oFormDefinition) {
          this.sModalFormDefinition = oModalObject.oFormDefinition;
          this.bOpenModalForm = true;
        } else if (oModalObject.form_path) {

          // Récupère le formulaire
          this.vitisRequest.ajaxRequestPromise({
            method: 'GET',
            url: oModalObject.form_path,
          }).then((resp) => {
            if (this.vitisCheck.isDefAndNotNull(resp.data)) {
              if (oModalObject.js_path) {
                this.oModalFormJS = {
                  sJsPath: oModalObject.js_path
                };
              } else {
                this.oModalFormJS = null;
              }
              this.sModalFormDefinition = resp.data;
              this.bOpenModalForm = true;
            } else {
              console.error('Error getting JSON form');
            }
          });
        }
      }


    } else {
      this.bOpenModalForm = false;
    }
  }

  /**
   * Ferme la modale pleine page ouverte par displayModalForm
   */
  public modalFormClose() {
    this.bOpenModalForm = false;
  }
}
