import { Component, OnInit, Input, OnDestroy } from '@angular/core';
import { Map } from 'ol';
import { Subscription } from 'rxjs';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';

@Component({
  selector: 'app-vmap-tools-selection-mobile',
  templateUrl: './vmap-tools-selection-mobile.component.html',
  styleUrls: ['./vmap-tools-selection-mobile.component.scss']
})
export class VmapToolsSelectionMobileComponent implements OnInit, OnDestroy {

  @Input() public olMap: Map;

  public bPanelOpen: boolean;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsService: VmapToolsService
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {
    // Écoute l'ouverture des menus
    this.subscriptions.push(this.vmapToolsService.getActiveRightPanelEmiter().subscribe((sTool: string) => {
      this.bPanelOpen = (sTool === 'selection-mobile');
    }));
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Ferme le panneau de gauche
   */
  public closePanel() {
    this.bPanelOpen = false;
    this.vmapToolsService.setRightPanel(0, '');
  }
}
