import { Component, OnInit, EventEmitter, Input, OnDestroy } from '@angular/core';
import { environment } from 'src/environments/environment';
import { Subscription } from 'rxjs';
import { Map } from 'ol';

@Component({
  selector: 'app-vmap-cadastre-tools-bati',
  templateUrl: './vmap-cadastre-tools-bati.component.html',
  styleUrls: ['./vmap-cadastre-tools-bati.component.scss']
})

export class VmapCadastreToolsBatiComponent implements OnInit, OnDestroy {

  public oDepartement: string;
  public oCommune: string;
  public oProprietaire: string;
  public oInvariant: Array<string>;
  public aCurrentCompte: Array<string>;
  public aCurrentParcelle: Array<string>;
  public oPropertiesDepartement: boolean;

  public oSelectedDepartementEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedCommuneEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedProprietaireEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedInvariantEmiter: EventEmitter<Array<string>> = new EventEmitter();

  // Subscriptions
  private subscriptionDepartement: Subscription;
  private subscriptionCommune: Subscription;
  private subscriptionProprietaire: Subscription;
  private subscriptionInvariant: Subscription;

  @Input() olMap: Map;

  /**
   * Constructeur
   */
  constructor() {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    // True ou false (properties) selon la présence de départements
    this.oPropertiesDepartement = environment.properties.cadastre_use_departement;

    // Ecoute les départements
    this.subscriptionDepartement = this.oSelectedDepartementEmiter.subscribe((next: string) => {
      this.oDepartement = next;
    });
    // Ecoute les communes
    this.subscriptionCommune = this.oSelectedCommuneEmiter.subscribe((next: string) => {
      this.oCommune = next;
    });

    // Ecoute les communes
    this.subscriptionProprietaire = this.oSelectedProprietaireEmiter.subscribe((next) => {
      this.oProprietaire = next;
    });

    // Ecoute les communes
    this.subscriptionInvariant = this.oSelectedInvariantEmiter.subscribe((next) => {
      this.oInvariant = next;
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    this.unsubscribDepartement();
    this.unsubscribCommune();
    this.unsubscribProprietaire();
    this.unsubscribInvariant();
  }

  /**
 * Permet de ne plus observer les changements de département
 */
  private unsubscribDepartement(): void {
    if (this.subscriptionDepartement) {
      this.subscriptionDepartement.unsubscribe();
    }
  }

  /**
 * Permet de ne plus observer les changements de commune
 */
  private unsubscribCommune(): void {
    if (this.subscriptionCommune) {
      this.subscriptionCommune.unsubscribe();
    }
  }

  /**
* Permet de ne plus observer les changements de proprietaire
*/
  private unsubscribProprietaire(): void {
    if (this.subscriptionProprietaire) {
      this.subscriptionProprietaire.unsubscribe();
    }
  }

  /**
* Permet de ne plus observer les changements d'invariant
*/
  private unsubscribInvariant(): void {
    if (this.subscriptionInvariant) {
      this.subscriptionInvariant.unsubscribe();
    }
  }

  /**
  * Réception des changements de proprietaire
  * @param $event string nouveaux proprietaires
  */
  public newProprietaire($event: string): void {
    this.oProprietaire = $event;
  }

  /**
  * Réception des changements de numéro d'invariants
  * @param $event Array<string> nouveaux invariants
  */
  public newParcelle($event: Array<string>): void {
    this.oInvariant = $event;
  }

  /**
  * Réception des changements de parcelles en cours de sélection
  * @param $event Array<string> nouvelles parcelles
  */
  public getParcelle($event: Array<string>): void {
    this.aCurrentParcelle = $event;
  }

  /**
  * Réception des changements de comptes
  * @param $event Array<string> nouveaux comptes
  */
  public getCompte($event: Array<string>): void {
    this.aCurrentCompte = $event;
  }
}
