import { Component, OnInit, EventEmitter, Input, OnDestroy } from '@angular/core';
import { environment } from 'src/environments/environment';
import { Subscription } from 'rxjs';
import { Map } from 'ol';

@Component({
  selector: 'app-vmap-cadastre-tools-commune',
  templateUrl: './vmap-cadastre-tools-commune.component.html',
  styleUrls: ['./vmap-cadastre-tools-commune.component.scss']
})

export class VmapCadastreToolsCommuneComponent implements OnInit, OnDestroy {

  public oDepartement: string;
  public oPropertiesDepartement: boolean;
  public oSelectedDepartementEmiter: EventEmitter<string> = new EventEmitter();

  // Subscriptions
  private subscriptionDepartement: Subscription;

  @Input() olMap: Map;

  /**
   * Constructeur
   */
  constructor() {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    // True ou false (properties) selon la présence de départements
    this.oPropertiesDepartement = environment.properties.cadastre_use_departement;
    // Ecoute les départements
    this.subscriptionDepartement = this.oSelectedDepartementEmiter.subscribe((next: string) => {
      this.oDepartement = next;
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    this.unsubscribDepartement();
  }

  /**
  * Permet de ne plus observer les changements de département
  */
  private unsubscribDepartement(): void {
    if (this.subscriptionDepartement) {
      this.subscriptionDepartement.unsubscribe();
    }
  }
}
