import { Component, OnInit, Input, OnChanges } from '@angular/core';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { VmapToolsCadastreMapService } from '../../service/vmap-tools-cadastre-map.service';
import { Map } from 'ol';
import { VmapToolsCadastreService } from '../../service/vmap-tools-cadastre.service';


@Component({
  selector: 'app-vmap-cadastre-tools-fiche-descriptive-modal',
  templateUrl: './vmap-cadastre-tools-fiche-descriptive-modal.component.html',
  styleUrls: ['./vmap-cadastre-tools-fiche-descriptive-modal.component.scss']
})

export class VmapCadastreToolsFicheDescriptiveModalComponent implements OnInit, OnChanges {

  @Input() public aDescrParcelle: any;
  @Input() olMap: Map;

  public privilege: string;
  public isOpen: boolean = false;
  public modalSize: number;
  public oLabel: string;
  public bRPFicheDescr: boolean = false;
  public bInvarFicheDescr: boolean = false;
  public bUrbaFicheDescr: boolean = false;

  // DATA relevé de propriété
  public aParcelle: any;

  // DATA modifiée de la fiche descriptive
  public aModifyDescrParcelle: any;

  // DATA de la fiche d'invariant
  public aInvariant: any;

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsCadastreService: VmapToolsCadastreService,
    private vmapToolsCadastreMapService: VmapToolsCadastreMapService,
    private vitisCheck: VitisCheck) {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    this.privilege = this.vmapToolsCadastreService.getPrivilege();
  }

  /**
   * Appelé lorsqu'il y a des changements sur les input
   */
  ngOnChanges(): void {
    // Vide les données modifiées à chaque sélection dans tools-grid
    this.aModifyDescrParcelle = [];
    setTimeout(() => {
      this.getDatas();
    });
  }

  /**
  * Prépare les données pour la fiche descriptive
  */
  private getDatas(): void {
    this.aModifyDescrParcelle = this.aDescrParcelle;
    // Surface Géographique
    this.aModifyDescrParcelle['sup_fiscale'] ? this.aModifyDescrParcelle['sup_fiscale'] = parseFloat(this.aModifyDescrParcelle['sup_fiscale']) + ' m²' : this.aModifyDescrParcelle['sup_fiscale'] = null;
    // Contenance
    this.aModifyDescrParcelle['DCNTPA'] ? this.aModifyDescrParcelle['DCNTPA'] = parseFloat(this.aModifyDescrParcelle['DCNTPA']) + ' m²' : this.aModifyDescrParcelle['DCNTPA'] = null;
    // Batie
    if (this.aModifyDescrParcelle['GPARBAT'] === '' || this.aModifyDescrParcelle['GPARBAT'] === ' ' || this.aModifyDescrParcelle['GPARBAT'] === 0 || this.aModifyDescrParcelle['GPARBAT'] === null) {
      this.aModifyDescrParcelle['GPARBAT'] = 'N';
    } else {
      this.aModifyDescrParcelle['GPARBAT'] = 'O';
    }
    // Urbaine
    this.aModifyDescrParcelle['GURBPA'] !== 'U' ? this.aModifyDescrParcelle['GURBPA'] = 'N' : this.aModifyDescrParcelle['GURBPA'] = 'O';

    // Propriétaires
    if (this.aModifyDescrParcelle['aProprietaires']) {
      for (let i = 0; i < this.aModifyDescrParcelle['aProprietaires'].length; i++) {
        this.aModifyDescrParcelle['aProprietaires'][i]['accordSexe'] = '(e)';
        this.aModifyDescrParcelle['aProprietaires'][i]['DQUALP'] = this.vitisCheck.isDefAndNotNull(this.aModifyDescrParcelle['aProprietaires'][i]['DQUALP']) ? this.aModifyDescrParcelle['aProprietaires'][i]['DQUALP'] : '';

        if (this.aModifyDescrParcelle['aProprietaires'][i]['DQUALP']) {
          this.aModifyDescrParcelle['aProprietaires'][i]['DQUALP'] = this.aModifyDescrParcelle['aProprietaires'][i]['DQUALP'].trim();
          if (this.aModifyDescrParcelle['aProprietaires'][i]['DQUALP'] === 'M') {
            this.aModifyDescrParcelle['aProprietaires'][i]['accordSexe'] = '';
          }
          if (this.aModifyDescrParcelle['aProprietaires'][i]['DQUALP'] === 'MLE' || this.aModifyDescrParcelle['aProprietaires'][i]['DQUALP'] === 'MME') {
            this.aModifyDescrParcelle['aProprietaires'][i]['accordSexe'] = 'e';
          }
        }
      }
    }

    // Subdivision Fiscales
    if (this.aModifyDescrParcelle['aSubdivisionFiscale']) {
      this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDcntsf'] = 0;
      this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDrcsuba'] = 0;
      this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDrcsub'] = 0;

      for (let i = 0; i < this.aModifyDescrParcelle['aSubdivisionFiscale'].length; i++) {
        if (this.vitisCheck.isDefAndNotNull(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DCNTSF'])) {
          this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DCNTSF'] = parseFloat(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DCNTSF']) + ' m²';
        }
        if (this.vitisCheck.isDefAndNotNull(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DRCSUBA'])) {
          this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DRCSUBA'] = (Math.pow(10, -2) * parseFloat(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DRCSUBA'])).toFixed(2) + ' €';
        }
        if (this.vitisCheck.isDefAndNotNull(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DRCSUB'])) {
          this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DRCSUB'] = (Math.pow(10, -2) * parseFloat(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DRCSUB'])).toFixed(2) + ' €';
        }

        if (this.vitisCheck.isDefAndNotNull(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DCNTSF'])) {
          this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDcntsf'] += parseFloat(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DCNTSF']);
        }
        if (this.vitisCheck.isDefAndNotNull(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DRCSUBA'])) {
          this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDrcsuba'] += parseFloat(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DRCSUBA']);
        }
        if (this.vitisCheck.isDefAndNotNull(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DRCSUB'])) {
          this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDrcsub'] += parseFloat(this.aModifyDescrParcelle['aSubdivisionFiscale'][i]['DRCSUB']);
        }
      }

      if (this.vitisCheck.isDefAndNotNull(this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDcntsf'])) {
        this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDcntsf'] = parseFloat(this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDcntsf']) + ' m²';
      }
      if (this.vitisCheck.isDefAndNotNull(this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDrcsuba'])) {
        this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDrcsuba'] = parseFloat(this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDrcsuba']).toFixed(2) + ' €';
      }
      if (this.vitisCheck.isDefAndNotNull(this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDrcsub'])) {
        this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDrcsub'] = parseFloat(this.aModifyDescrParcelle['aSubdivisionFiscale']['totalDrcsub']).toFixed(2) + ' €';
      }
    }
  }

  /**
  * Imprime la fiche descriptive
  */
  public selectPrint(): void {
    const printWindow = window.open('', '', 'height=800, width=800');
    const css = '<style> @media print{.container-btn{display: none;} .white-space{white-space: normal;} .odd{background-color: #E9E9E9;} .even{background-color: #F5F5F5;} .subtitle{background-color:#212121; height:42px;} .header-parcelle{position: sticky; width: 100%;top: 0; background-color: #212121; height: 42px;text-align: center; color: white; padding-top:8px;} .body-modal{flex-direction: column;} .title{line-height: 42px; float: left; margin-left: 8px; font-size: 21px !important; color: white;} table,th,td{border: 1px solid black; border-collapse: collapse; white-space: pre-wrap;} table{margin-top: 8px; margin-left: 5%; margin-bottom: 5px; width: 90%;} .table-header{background-color: #464646; color: white; text-align: center;} *{ color-adjust: exact; -webkit-print-color-adjust: exact; print-color-adjust: exact; } .icone{display: none;} .no-break{break-inside: avoid;}} </style>';
    const data = document.getElementById('header-parcelle').innerHTML;
    printWindow.document.write('<html><head>' + css + '</head><body>' + data + '</body></html>');
    printWindow.print();
    printWindow.close();
  }

  /**
  * Prépare la modale du relevé de propriété via la fiche descriprive
  * @returns Promise
  */
  public selectedReleveDePropriete() {
    this.bRPFicheDescr = !this.bRPFicheDescr;
    if (this.bRPFicheDescr) {
      this.vmapToolsCadastreMapService.descriptionParcelle(this.aDescrParcelle.ID_PAR).then(descParcelle => {
        this.aParcelle = descParcelle;
        this.oLabel = 'Génération du relevé de propriété';
        this.isOpen = true;
        this.modalSize = 2;
      });
    }
  }

  /**
 * Ouvre la modale de l'invariant via la fiche descriprive
 * @returns Promise
 */
  public selectedInvariant(element): Promise<any> {
    return new Promise(async (resolve, reject) => {
      this.bInvarFicheDescr = !this.bInvarFicheDescr;
      try {
        const aInvariant = await this.vmapToolsCadastreService.loadInvariantByIdBat_(element.ID_BAT);
        this.aInvariant = aInvariant;
        this.oLabel = 'Fiche descriptive de l\'invariant';
        this.isOpen = true;
        this.modalSize = 3;
        resolve(true);
      } catch (error) {
        console.error(error);
        reject(error);
      }
    });
  }

  /**
  * Permet de télécharger la fiche d'urbanisme
  */
  public selectedFicheUrbanisme(): void {
    this.bUrbaFicheDescr = !this.bUrbaFicheDescr;
    if (this.bUrbaFicheDescr) {
      this.vmapToolsCadastreMapService.downloadFicheUrba(this.aDescrParcelle.ID_PAR, this.olMap);
      this.bUrbaFicheDescr = false;
    }
  }

  /**
  * Permet de fermer la modale
  * @param $event false quand on ferme la modale
  */
  public closeModal($event): void {
    this.isOpen = $event;
    this.bRPFicheDescr = false;
    this.bInvarFicheDescr = false;
  }
}
