import { Component, OnInit, EventEmitter, Input, OnDestroy } from '@angular/core';
import { environment } from 'src/environments/environment';
import { Subscription } from 'rxjs';
import { Map } from 'ol';

@Component({
  selector: 'app-vmap-cadastre-tools-parcelle-dgfip',
  templateUrl: './vmap-cadastre-tools-parcelle-dgfip.component.html',
  styleUrls: ['./vmap-cadastre-tools-parcelle-dgfip.component.scss']
})

export class VmapCadastreToolsParcelleDgfipComponent implements OnInit, OnDestroy {

  public oDepartement: string;
  public oCommune: string;
  public oVoie: string;
  public oAdresses: Array<string>;
  public aCurrentCompte: Array<string>;
  public aCurrentParcelle: Array<string>;
  public oPropertiesDepartement: boolean;

  public oSelectedDepartementEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedCommuneEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedVoieEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedAdresseEmiter: EventEmitter<Array<string>> = new EventEmitter();

  // Subscriptions
  private subscriptionDepartement: Subscription;
  private subscriptionCommune: Subscription;
  private subscriptionVoie: Subscription;
  private subscriptionAdresse: Subscription;

  @Input() olMap: Map;

  /**
   * Constructeur
   */
  constructor() {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    // True ou false (properties) selon la présence de départements
    this.oPropertiesDepartement = environment.properties.cadastre_use_departement;
    // Ecoute les départements
    this.subscriptionDepartement = this.oSelectedDepartementEmiter.subscribe((next) => {
      this.oDepartement = next;
    });

    // Ecoute les communes
    this.subscriptionCommune = this.oSelectedCommuneEmiter.subscribe((next) => {
      this.oCommune = next;
    });

    // Ecoute les voies
    this.subscriptionVoie = this.oSelectedVoieEmiter.subscribe((next) => {
      this.oVoie = next;
    });

    // Ecoute les adresses
    this.subscriptionAdresse = this.oSelectedAdresseEmiter.subscribe((next) => {
      this.oAdresses = next;
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    this.unsubscribDepartement();
    this.unsubscribCommune();
    this.unsubscribVoie();
    this.unsubscribAdresse();
  }

  /**
 * Permet de ne plus observer les changements de département
 */
  private unsubscribDepartement(): void {
    if (this.subscriptionDepartement) {
      this.subscriptionDepartement.unsubscribe();
    }
  }

  /**
 * Permet de ne plus observer les changements de commune
 */
  private unsubscribCommune(): void {
    if (this.subscriptionCommune) {
      this.subscriptionCommune.unsubscribe();
    }
  }

  /**
* Permet de ne plus observer les changements de voie
*/
  private unsubscribVoie(): void {
    if (this.subscriptionVoie) {
      this.subscriptionVoie.unsubscribe();
    }
  }

  /**
* Permet de ne plus observer les changements d'adresse
*/
  private unsubscribAdresse(): void {
    if (this.subscriptionAdresse) {
      this.subscriptionAdresse.unsubscribe();
    }
  }

  /**
  * Réception des changements de voies
  * @param $event string nouveaux invariants
  */
  public newVoie($event: string): void {
    this.oVoie = $event;
  }

  /**
  * Réception des changements de parcelles en cours de sélection
  * @param $event Array<string> nouvelles parcelles
  */
  public getParcelle($event: Array<string>): void {
    this.aCurrentParcelle = $event;
  }

  /**
  * Réception des changements de comptes
  * @param $event Array<string> nouveaux comptes
  */
  public getCompte($event: Array<string>): void {
    this.aCurrentCompte = $event;
  }
}
