import { Component, OnInit, EventEmitter, Input, OnDestroy } from '@angular/core';
import { environment } from 'src/environments/environment';
import { Subscription } from 'rxjs';
import { Map } from 'ol';

@Component({
  selector: 'app-vmap-cadastre-tools-parcelle-proprietaire',
  templateUrl: './vmap-cadastre-tools-parcelle-proprietaire.component.html',
  styleUrls: ['./vmap-cadastre-tools-parcelle-proprietaire.component.scss']
})

export class VmapCadastreToolsParcelleProprietaireComponent implements OnInit, OnDestroy {

  public oDepartement: string;
  public oCommune: string;
  public oProprietaire: string;
  public oParcelle: Array<string>;
  public aCurrentCompte: Array<string>;
  public aCurrentParcelle: Array<string>;
  public oPropertiesDepartement: boolean;

  public oSelectedDepartementEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedCommuneEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedProprietaireEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedParcelleEmiter: EventEmitter<Array<string>> = new EventEmitter();

  // Subscriptions
  private subscriptionDepartement: Subscription;
  private subscriptionCommune: Subscription;
  private subscriptionProprietaire: Subscription;
  private subscriptionParcelle: Subscription;

  @Input() olMap: Map;

  /**
   * Constructeur
   */
  constructor() {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    // True ou false (properties) selon la présence de départements
    this.oPropertiesDepartement = environment.properties.cadastre_use_departement;

    // Ecoute les départements
    this.subscriptionDepartement = this.oSelectedDepartementEmiter.subscribe((next: string) => {
      this.oDepartement = next;
    });

    // Ecoute les communes
    this.subscriptionCommune = this.oSelectedCommuneEmiter.subscribe((next: string) => {
      this.oCommune = next;
    });

    // Ecoute les propriétaires
    this.subscriptionProprietaire = this.oSelectedProprietaireEmiter.subscribe((next) => {
      this.oProprietaire = next;
    });

    // Ecoute les parcelles
    this.subscriptionParcelle = this.oSelectedParcelleEmiter.subscribe((next) => {
      this.oParcelle = next;
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    this.unsubscribDepartement();
    this.unsubscribCommune();
    this.unsubscribProprietaire();
    this.unsubscribParcelle();
  }

  /**
 * Permet de ne plus observer les changements de département
 */
  private unsubscribDepartement(): void {
    if (this.subscriptionDepartement) {
      this.subscriptionDepartement.unsubscribe();
    }
  }

  /**
 * Permet de ne plus observer les changements de commune
 */
  private unsubscribCommune(): void {
    if (this.subscriptionCommune) {
      this.subscriptionCommune.unsubscribe();
    }
  }

  /**
* Permet de ne plus observer les changements de proprietaire
*/
  private unsubscribProprietaire(): void {
    if (this.subscriptionProprietaire) {
      this.subscriptionProprietaire.unsubscribe();
    }
  }

  /**
* Permet de ne plus observer les changements de parcelle
*/
  private unsubscribParcelle(): void {
    if (this.subscriptionParcelle) {
      this.subscriptionParcelle.unsubscribe();
    }
  }


  /**
  * Réception des changements de propriétaire
  * @param $event string nouveau proprietaire
  */
  public newProprietaire($event: string): void {
    this.oProprietaire = $event;
  }

  /**
  * Réception des changements de parcelles
  * @param $event Array<string> nouvelles parcelles
  */
  public newParcelle($event: Array<string>): void {
    this.oParcelle = $event;
  }

  /**
  * Réception des changements de parcelles en cours de sélection
  * @param $event Array<string> nouvelles parcelles
  */
  public getParcelle($event: Array<string>): void {
    this.aCurrentParcelle = $event;
  }

  /**
  * Réception des changements de comptes
  * @param $event Array<string> nouveaux comptes
  */
  public getCompte($event: Array<string>): void {
    this.aCurrentCompte = $event;
  }
}
