import { Component, OnInit, EventEmitter, Input, OnDestroy } from '@angular/core';
import { environment } from 'src/environments/environment';
import { Subscription } from 'rxjs';
import { Map } from 'ol';

@Component({
  selector: 'app-vmap-cadastre-tools-parcelle-section',
  templateUrl: './vmap-cadastre-tools-parcelle-section.component.html',
  styleUrls: ['./vmap-cadastre-tools-parcelle-section.component.scss']
})

export class VmapCadastreToolsParcelleSectionComponent implements OnInit, OnDestroy {

  public oDepartement: string;
  public oCommune: string;
  public oSection: string;
  public aCurrentCompte: Array<string>;
  public aCurrentParcelle: Array<string>;
  public oPropertiesDepartement: boolean;

  public oSelectedDepartementEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedCommuneEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedSectionEmiter: EventEmitter<string> = new EventEmitter();

  // Subscriptions
  private subscriptionDepartement: Subscription;
  private subscriptionCommune: Subscription;
  private subscriptionSection: Subscription;

  @Input() olMap: Map;

  /**
   * Constructeur
   */
  constructor() {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    // True ou false (properties) selon la présence de départements
    this.oPropertiesDepartement = environment.properties.cadastre_use_departement;

    // Ecoute les départements
    this.subscriptionDepartement = this.oSelectedDepartementEmiter.subscribe((next: string) => {
      this.oDepartement = next;
    });

    // Ecoute les communes
    this.subscriptionCommune = this.oSelectedCommuneEmiter.subscribe((next: string) => {
      this.oCommune = next;
    });

    // Ecoute les sections
    this.subscriptionSection = this.oSelectedSectionEmiter.subscribe((next: string) => {
      this.oSection = next;
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    this.unsubscribDepartement();
    this.unsubscribCommune();
    this.unsubscribSection();
  }

  /**
  * Permet de ne plus observer les changements de département
  */
  private unsubscribDepartement(): void {
    if (this.subscriptionDepartement) {
      this.subscriptionDepartement.unsubscribe();
    }
  }

  /**
 * Permet de ne plus observer les changements de commune
 */
  private unsubscribCommune(): void {
    if (this.subscriptionCommune) {
      this.subscriptionCommune.unsubscribe();
    }
  }

  /**
* Permet de ne plus observer les changements de section
*/
  private unsubscribSection(): void {
    if (this.subscriptionSection) {
      this.subscriptionSection.unsubscribe();
    }
  }

  /**
  * Réception des changements de parcelles en cours de sélection
  * @param $event Array<string> nouvelles parcelles
  */
  public getParcelle($event: Array<string>): void {
    this.aCurrentParcelle = $event;
  }

  /**
  * Réception des changements de comptes
  * @param $event Array<string> nouveaux comptes
  */
  public getCompte($event: Array<string>): void {
    this.aCurrentCompte = $event;
  }
}
