import { Component, OnInit, Input, OnDestroy } from '@angular/core';
import { VmapToolsCadastreService } from 'src/modules/vmap/components/carto/components/vmap-tools/components/vmap-tools-cadastre/service/vmap-tools-cadastre.service';
import { VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { VmapToolsCadastreMapService } from '../../service/vmap-tools-cadastre-map.service';
import { Subscription } from 'rxjs';

@Component({
  selector: 'app-vmap-cadastre-tools-releve-de-propriete-modal',
  templateUrl: './vmap-cadastre-tools-releve-de-propriete-modal.component.html',
  styleUrls: ['./vmap-cadastre-tools-releve-de-propriete-modal.component.scss']
})

export class VmapCadastreToolsReleveDeProprieteModalComponent implements OnInit, OnDestroy {

  public idDnupro: number;
  public idPar: number;

  @Input() public aParcelle: any;

  // Subscription fiche descriptive
  private subscriptionReleveDePropriete: Subscription;

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsCadastreService: VmapToolsCadastreService,
    private vitisRequest: VitisRequest,
    private vmapToolsCadastreMapService: VmapToolsCadastreMapService) {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    // Changement de parcelle
    this.subscriptionReleveDePropriete = this.vmapToolsCadastreMapService.getRPParcelle().subscribe(async (next: any) => {
      this.idDnupro = next[0].ID_DNUPRO;
      this.idPar = next[0].ID_PAR;
    });
    this.idDnupro = this.aParcelle[0].ID_DNUPRO;
    this.idPar = this.aParcelle[0].ID_PAR;
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    if (this.subscriptionReleveDePropriete) {
      this.subscriptionReleveDePropriete.unsubscribe();
    }
  }

  /**
  * Télécharge le relevé de propriété standard
  */
  public getRPStandard(): void {
    this.vmapToolsCadastreService.downloadRPStandard_(this.idDnupro).then(response => {
      this.vitisRequest.ajaxDownloadFile(response.rapport, 'Relevé_de_propriete_standard' + this.idDnupro + '.pdf');
    });
  }

  /**
  * Télécharge le relevé de propriété de la parcelle
  */
  public getRPParcelle(): void {
    this.vmapToolsCadastreService.downloadRPParcelle_(this.idPar).then(response => {
      this.vitisRequest.ajaxDownloadFile(response.rapport, 'Relevé_de_propriete_parcelle' + this.idPar + '.pdf');
    });
  }

  /**
  * Télécharge le relevé de propriété de la parcelle tier
  */
  public getParcelleTier(): void {
    this.vmapToolsCadastreService.downloadRPParcelleTier_(this.idPar).then(response => {
      this.vitisRequest.ajaxDownloadFile(response.rapport, 'Relevé_de_propriete_parcelle_tier' + this.idPar + '.pdf');
    });
  }

  /**
  * Télécharge le relevé de propriété d'un tier
  */
  public getRPTier() {
    this.vmapToolsCadastreService.downloadRPTier_(this.idDnupro).then(response => {
      this.vitisRequest.ajaxDownloadFile(response.rapport, 'Relevé_de_propriete_tier' + this.idDnupro + '.pdf');
    });
  }
}
