import { Component, OnInit, EventEmitter, Input, OnDestroy } from '@angular/core';
import { environment } from 'src/environments/environment';
import { Subscription } from 'rxjs';
import { Map } from 'ol';

@Component({
  selector: 'app-vmap-cadastre-tools-section',
  templateUrl: './vmap-cadastre-tools-section.component.html',
  styleUrls: ['./vmap-cadastre-tools-section.component.scss']
})

export class VmapCadastreToolsSectionComponent implements OnInit, OnDestroy {

  public oDepartement: string;
  public oCommune: string;
  public oPropertiesDepartement: boolean;

  public oSelectedDepartementEmiter: EventEmitter<string> = new EventEmitter();
  public oSelectedCommuneEmiter: EventEmitter<string> = new EventEmitter();

  // Subscriptions
  private subscriptionDepartement: Subscription;
  private subscriptionCommune: Subscription;


  @Input() olMap: Map;

  /**
   * Constructeur
   */
  constructor() {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    this.oPropertiesDepartement = environment.properties.cadastre_use_departement;

    this.subscriptionDepartement = this.oSelectedDepartementEmiter.subscribe((next: string) => {
      this.oDepartement = next;
    });

    this.subscriptionCommune = this.oSelectedCommuneEmiter.subscribe((next: string) => {
      this.oCommune = next;
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    this.unsubscribDepartement();
    this.unsubscribCommune();
  }

  /**
  * Permet de ne plus observer les changements de département
  */
  private unsubscribDepartement(): void {
    if (this.subscriptionDepartement) {
      this.subscriptionDepartement.unsubscribe();
    }
  }

  /**
 * Permet de ne plus observer les changements de commune
 */
  private unsubscribCommune(): void {
    if (this.subscriptionCommune) {
      this.subscriptionCommune.unsubscribe();
    }
  }
}
