import { Component, OnInit, Input, OnChanges } from '@angular/core';
import { VmapToolsCadastreService } from 'src/modules/vmap/components/carto/components/vmap-tools/components/vmap-tools-cadastre/service/vmap-tools-cadastre.service';
import { VmapToolsCadastreMapService } from '../../service/vmap-tools-cadastre-map.service';
import { VitisRequest, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';

@Component({
  selector: 'app-vmap-cadastre-tools-button-rapport',
  templateUrl: './vmap-cadastre-tools-button-rapport.component.html',
  styleUrls: ['./vmap-cadastre-tools-button-rapport.component.scss']
})

export class VmapCadastreToolsButtonRapportComponent implements OnInit, OnChanges {

  @Input() public aCurrentParcelle: Array<string>;
  @Input() public aCurrentCompte: Array<string>;
  @Input() olMap;

  public isOpenMainList: boolean = false;
  public isOpenSubList: boolean = false;
  public privilege: string;

  // Modale
  public isOpen: boolean = false;
  public modalSize: number;
  public oLabel: string;

  // Bouton
  public bSelectingRapport: string;
  public bSelectingFicheUrba: boolean = false;
  public bSelectingReleveDePropriete: string = '';
  public bSelectingFicheDescr: boolean = false;

  // DATA
  public aDescrParcelle: any;

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsCadastreService: VmapToolsCadastreService,
    private vmapToolsCadastreMapService: VmapToolsCadastreMapService,
    private vitisRequest: VitisRequest) {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    this.privilege = this.vmapToolsCadastreService.getPrivilege();
  }

  /**
  * Suppression des features
  * Changement des parcelles
  */
  ngOnChanges(): void {
    if (!this.aCurrentParcelle) {
      this.aCurrentParcelle = [];
    }
  }

  /**
 * Ouvre le panneau des principaux rapports
 */
  public openMainList(): void {
    this.isOpenMainList = !this.isOpenMainList;
  }

  /**
  * Ouvre un second panneau pour les autres rapports
  */
  public openSubList(): void {
    this.isOpenSubList = !this.isOpenSubList;
  }

  /**
  * Permet d'ouvrir la fiche descriptive
  * @returns Promise
  */
  public selectedFicheDescriptive(): Promise<any> {
    if (this.aCurrentParcelle.length == 1) {
      this.bSelectingFicheDescr = !this.bSelectingFicheDescr;
      this.bSelectingReleveDePropriete = '';
      this.bSelectingFicheUrba = false;
      return new Promise(async (resolve, reject) => {
        try {
          if (this.bSelectingFicheDescr) {
            const descParcelle = await this.vmapToolsCadastreMapService.ficheDescriptive(this.aCurrentParcelle[0]['id_par']);
            this.aDescrParcelle = descParcelle;
            this.oLabel = 'Fiche descriptive de la parcelle';
            this.isOpen = true;
            this.modalSize = 1;
          }
          resolve(true);
        } catch (error) {
          console.error(error);
          reject(error);
        }
      });
    } else {
      vitisAlert.fireToast({
        type: 'warning',
        title: 'Veuillez sélectionner une unique parcelle'
      });
    }
  }

  /**
  * Permet de télécharger la fiche d'urbanisme
  * @returns Promise
  */
  public selectedFicheUrba(): Promise<any> {
    if (this.aCurrentParcelle.length == 1) {
      this.bSelectingFicheUrba = true;
      this.bSelectingReleveDePropriete = '';
      this.bSelectingFicheDescr = false;
      if (this.bSelectingFicheUrba) {
        return new Promise((resolve, reject) => {
          try {
            const descParcelle = this.vmapToolsCadastreMapService.downloadFicheUrba(this.aCurrentParcelle[0]['id_par'], this.olMap);
            this.bSelectingFicheUrba = false;
            resolve(descParcelle);
          } catch (error) {
            console.error(error);
            reject(error);
          }
        });
      }
    } else {
      vitisAlert.fireToast({
        type: 'warning',
        title: 'Veuillez sélectionner une unique parcelle'
      });
    }
  }

  /**
   * Permet de télécharger le relevé de propriété
   * Uniquement pour le plus au niveau d'autorisation dans la module cadastre
   * @param element string permet de sélectionner le type de relevé de propriété
  */
  public selectedReleveDePropriete(element: string): void {
    this.bSelectingReleveDePropriete = element;
    if (element == 'parcelle') {
      if (this.aCurrentParcelle.length == 1) {
        this.bSelectingFicheUrba = false;
        this.bSelectingFicheDescr = false;
        if (this.bSelectingReleveDePropriete) {
          this.vmapToolsCadastreService.downloadRPParcelle_(this.aCurrentParcelle[0]['id_par']).then(response => {
            this.vitisRequest.ajaxDownloadFile(response.rapport, 'Relevé_de_propriete_parcelle' + this.aCurrentParcelle[0]['id_par'] + '.pdf');
            this.bSelectingReleveDePropriete = '';
          });
        }
      } else {
        vitisAlert.fireToast({
          type: 'warning',
          title: 'Veuillez sélectionner une unique parcelle'
        });
      }
    } else if (element == 'standard') {
      if (this.aCurrentCompte.length == 1) {
        const aDnupro = this.aCurrentCompte[0]['ID_COM'] + this.aCurrentCompte[0]['DNUPRO'];
        this.vmapToolsCadastreService.downloadRPStandard_(aDnupro).then(response => {
          this.vitisRequest.ajaxDownloadFile(response.rapport, 'Relevé_de_propriete_standard' + aDnupro + '.pdf');
          this.bSelectingReleveDePropriete = '';
        });
      } else {
        vitisAlert.fireToast({
          type: 'warning',
          title: 'Veuillez sélectionner un unique compte'
        });
      }
    } else if (element == 'tiers') {
      if (this.aCurrentCompte.length == 1) {
        const aDnupro = this.aCurrentCompte[0]['ID_COM'] + this.aCurrentCompte[0]['DNUPRO'];
        this.vmapToolsCadastreService.downloadRPTier_(aDnupro).then(response => {
          this.vitisRequest.ajaxDownloadFile(response.rapport, 'Relevé_de_propriete_tier' + aDnupro + '.pdf');
          this.bSelectingReleveDePropriete = '';
        });
      } else {
        vitisAlert.fireToast({
          type: 'warning',
          title: 'Veuillez sélectionner un unique compte'
        });
      }
    }
  }

  /**
  * Permet de formatter les id des parcelles
  * @returns string id des parcelles separées par des |
  */
  private getIds(): string {
    let aIdPar: string = '';
    for (let i = 0; i < this.aCurrentParcelle.length; i++) {
      aIdPar = aIdPar + this.aCurrentParcelle[i]['id_par'] + '|';
    }
    return aIdPar.slice(0, -1);
  }

  /**
  * Telecharge les fichiers excels ou pdf
  * @param sType string qui permet de déterminer l'excel souhaité
  */
  public selectedExcel(sType: string): void {
    this.bSelectingRapport = sType;
    const aIds = this.getIds();
    if (this.aCurrentParcelle.length > 0) {
      if (sType == 'informations') {
        this.vmapToolsCadastreService.downloadExcelToutesInfos_(aIds).then(response => {
          this.vitisRequest.ajaxDownloadFile(response.rapport, 'ToutesInfos_' + Date.now() + '.pdf');
        });

      } else if (sType == 'baties') {
        this.vmapToolsCadastreService.downloadExcelBatiesNonBatie_(aIds).then(response => {
          this.vitisRequest.ajaxDownloadFile(response.rapport, 'BatieNonBatie_' + Date.now() + '.xlsx');
        });
      } else if (sType == 'subFiscales') {
        this.vmapToolsCadastreService.downloadExcelSubdFiscales_(aIds).then(response => {
          this.vitisRequest.ajaxDownloadFile(response.rapport, 'SubdivisionFiscale_' + Date.now() + '.xlsx');
        });
      } else if (sType == 'locaux') {
        this.vmapToolsCadastreService.downloadExcelLocaux_(aIds).then(response => {
          this.vitisRequest.ajaxDownloadFile(response.rapport, 'Locaux_' + Date.now() + '.xlsx');
        });
      } else if (sType == 'proprietaires') {
        this.vmapToolsCadastreService.downloadExcelProprietaires_(aIds).then(response => {
          this.vitisRequest.ajaxDownloadFile(response.rapport, 'Proprietaires_' + Date.now() + '.xlsx');
        });
      } else if (sType == 'emprises') {
        this.vmapToolsCadastreService.downloadExcelEmprises_(aIds).then(response => {
          this.vitisRequest.ajaxDownloadFile(response.rapport, 'EmprisePosPlu_' + Date.now() + '.xlsx');
        });
      } else if (sType == 'proprietairesLocaux') {
        this.vmapToolsCadastreService.downloadExcelProprietairesLocaux_(aIds).then(response => {
          this.vitisRequest.ajaxDownloadFile(response.rapport, 'ProprietairesLocaux_' + Date.now() + '.xlsx');
        });
      }
      this.bSelectingRapport = null;
    } else {
      vitisAlert.fireToast({
        type: 'warning',
        title: 'Veuillez sélectionner au moins une parcelle'
      });
    }
  }

  /**
   *  Ferme la modale
   * @param $event statut de la modale
  */
  public closeModal($event): void {
    this.isOpen = $event;
    this.oLabel = null;
    this.bSelectingReleveDePropriete = '';
    this.bSelectingFicheDescr = false;
    this.bSelectingFicheUrba = false;
  }
}
