import { Component, Input, OnInit, EventEmitter, OnChanges, OnDestroy } from '@angular/core';
import { VmapToolsCadastreService } from 'src/modules/vmap/components/carto/components/vmap-tools/components/vmap-tools-cadastre/service/vmap-tools-cadastre.service';
import { Subscription } from 'rxjs';
import { Map } from 'ol';
import { TranslateService } from '@ngx-translate/core';

interface Attribut {
  label: string;
  value: string;
}

@Component({
  selector: 'app-vmap-cadastre-tools-drop-down-list',
  templateUrl: './vmap-cadastre-tools-drop-down-list.component.html',
  styleUrls: ['./vmap-cadastre-tools-drop-down-list.component.scss']
})

export class VmapCadastreToolsDropDownListComponent implements OnInit, OnChanges, OnDestroy {
  @Input() sType: any;
  @Input() oAttribute: string;
  @Input() oDepartement: string;
  @Input() oCommune: string;
  @Input() oSection: string;
  @Input() oVoie: string;
  @Input() reloadObject: boolean;

  @Input() oSelectedAttributeEmiter: EventEmitter<Array<string>> = new EventEmitter();
  @Input() oSelectedParentEmiter: EventEmitter<string>;

  @Input() olMap: Map;

  // titre de la liste déroulante
  public title: string;
  public placeholder: string;

  // Elements de la liste déroulante
  public attributes: Array<Attribut> = [];
  public aParcelle: Array<string> = [];
  public privilege: string;

  // Subscription de l'attribut
  private subscriptionParent: Subscription;

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsCadastreService: VmapToolsCadastreService,
    private translate: TranslateService
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    this.privilege = this.vmapToolsCadastreService.getPrivilege();
    this.initField();
    if (this.oSelectedParentEmiter) {
      this.subscriptionParent = this.oSelectedParentEmiter.subscribe((val) => {
        setTimeout(() => { // Timeout le temps que l'info revienne sur oDepartement, oCommune etc.
          this.initField();
          this.oSelectedAttributeEmiter.next(null);
        });
      });
    }
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    this.unsubscribeParent();
  }

  /**
  * Permet de ne plus observer les changements de données de la fiche descriptive
  */
  private unsubscribeParent(): void {
    // désinscription de la fiche descriptive si subscription
    if (this.subscriptionParent) {
      this.subscriptionParent.unsubscribe();
    }
  }


  /**
   * Appelé au changement du select
   * @param attributeValue
   */
  public onChange(attributeValue: any) {
    if (attributeValue == 'toutes') {
      this.oSelectedAttributeEmiter.next(this.aParcelle);
    } else {
      this.oSelectedAttributeEmiter.next(attributeValue);
    }
  }

  /**
   * Reload les données en cas de changement dans une variable
   */
  ngOnChanges() {
    // Reload de la recherche par objet métier
    if (this.reloadObject == true) {
      this.attributes = [];
      this.getTypeOfSearch();
    }

    // Reload les données si changement de voie
    if (this.sType == 'adresse') {
      this.getElementOfAdress();
    }
  }

  /**
   * Initialisation du contenu des listes
   */
  private initField() {
    if (this.sType == 'attribut') {
      this.getTypeOfSearch();
    } else if (this.sType == 'parcelleAttribute') {
      this.getTypeOfParcelle();
    } else if (this.sType == 'departement') {
      this.getElementOfDepartements();
    } else if (this.sType == 'commune') {
      this.getElementOfCommunes();
    } else if (this.sType == 'section') {
      this.getElementOfSections();
    }

    if (this.sType == 'adresse') {
      this.placeholder = this.translate.instant('CADASTRE_TOOL_SEARCH_ALL_ADDRESS');
    } else {
      this.placeholder = this.translate.instant('CADASTRE_TOOL_SEARCH_PLACEHOLDER');
    }
  }

  /**
  * Méthode permettant la liste déroulante principale
  */
  private getTypeOfSearch(): void {
    this.title = 'Objet :';
    this.attributes = [
      { label: 'CADASTRE_TOOL_SEARCH_TYPE_CITY', value: 'commune' },
      { label: 'CADASTRE_TOOL_SEARCH_TYPE_SECTION', value: 'section' },
      { label: 'CADASTRE_TOOL_SEARCH_TYPE_LOCATION', value: 'lieuDit' },
      { label: 'CADASTRE_TOOL_SEARCH_TYPE_PARCEL', value: 'parcelle' }
    ];
    if (this.privilege != 'vmap_cadastre_light_user') {
      this.attributes.push({ label: 'CADASTRE_TOOL_SEARCH_TYPE_BUILDING', value: 'bati' });
    }

    // Traduction.
    for (let i = 0; i < this.attributes.length; i++) {
      this.attributes[i].label = this.translate.instant(this.attributes[i].label);
    }
  }

  /**
  * Méthode permettant la liste déroulante pour les parcelles
  */
  private getTypeOfParcelle(): void {
    this.title = 'Recherche par :';
    this.attributes = [
      { label: 'Section', value: 'section' },
      { label: this.translate.instant('COMPONENT_CADASTRE_TOOLS_DROP_DOWN_LIST_DGFIP_ADDRESS'), value: 'dgfip' }
    ];
    if (this.privilege != 'vmap_cadastre_light_user') {
      this.attributes.push({ label: 'Propriétaire', value: 'proprietaire' });
    }
  }

  /**
  * Méthode permettant de remplir la liste déroulante des départements
  */
  private getElementOfDepartements(): void {
    this.title = 'Département :';
    this.getDepartements();
  }

  /**
  * Méthode permettant de remplir la liste déroulante des communes
  */
  private getElementOfCommunes(): void {
    this.title = 'Commune :';
    if (this.oDepartement) {
      this.getCommuneParDepartement();
    } else {
      this.getCommune();
    }
  }



  /**
  * Méthode permettant de remplir la liste déroulante des sections
  */
  private getElementOfSections(): void {
    this.title = 'Section :';
    if (this.oCommune) {
      this.getSectionParCommune();
    }
  }

  /**
* Méthode permettant de remplir la liste déroulante des adresses
*/
  private getElementOfAdress(): void {
    this.title = 'Adresse :';
    if (this.oVoie) {
      this.getAdresseParVoie();
    }
  }

  /**
   * Structure les données des départements
   */
  private getDepartements(): void {
    this.attributes = [];
    this.vmapToolsCadastreService.loadDepartements_().then(response => {
      for (let i = 0; i < response.length; i++) {
        this.attributes.push({ label: response[i].nom, value: response[i].dep_code });
      }
    });
  }

  /**
   * Structure les données des communes par département
   */
  private getCommuneParDepartement(): void {
    this.attributes = [];
    this.vmapToolsCadastreService.loadCommunesParDepartement_(this.oDepartement, this.olMap).then(response => {
      for (let i = 0; i < response.length; i++) {
        this.attributes.push({ label: response[i].nom, value: response[i].id_com });
      }
    });
  }

  /**
   * Structure les données des communes en l'absence de département
   */
  private getCommune(): void {
    this.attributes = [];
    this.vmapToolsCadastreService.loadCommunes_(this.olMap).then(response => {
      for (let i = 0; i < response.length; i++) {
        this.attributes.push({ label: response[i].nom, value: response[i].id_com });
      }
    });
  }

  /**
   * Structure les données des sections
   */
  private getSectionParCommune(): void {
    this.attributes = [];
    if (this.oCommune) {
      this.vmapToolsCadastreService.loadSections_(this.oCommune, this.olMap).then(response => {
        for (let i = 0; i < response.length; i++) {
          this.attributes.push({ label: response[i].section, value: response[i].id_sec });
        }
      });
    }
  }

  /**
  * Structure les données des adresses
  */
  private getAdresseParVoie(): void {
    this.attributes = [];
    this.aParcelle = [];
    if (this.oVoie) {
      this.vmapToolsCadastreService.loadAdressesByVoie_(this.oVoie).then(response => {
        this.attributes.push({ label: 'Toutes', value: 'toutes' });
        for (let i = 0; i < response.length; i++) {
          if (response[i].DNVOIRI) {
            this.attributes.push({ label: response[i].DNVOIRI, value: response[i].ID_PAR });
            this.aParcelle.push(response[i].ID_PAR);
          }
        }
        this.oSelectedAttributeEmiter.next(this.aParcelle);
      });
    }
  }
}
