import { Component, Input, OnInit, EventEmitter, OnChanges, OnDestroy, Output, ViewChild, ElementRef } from '@angular/core';
import { VmapToolsCadastreService } from 'src/modules/vmap/components/carto/components/vmap-tools/components/vmap-tools-cadastre/service/vmap-tools-cadastre.service';
import { VmapToolsCadastreMapService } from '../../service/vmap-tools-cadastre-map.service';
import { Feature, Map } from 'ol';
import { Subscription } from 'rxjs';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { TranslateService } from '@ngx-translate/core';

interface Attribut {
  label: string;
  value: string;
}

@Component({
  selector: 'app-vmap-cadastre-tools-grid',
  templateUrl: './vmap-cadastre-tools-grid.component.html',
  styleUrls: ['./vmap-cadastre-tools-grid.component.scss']
})

export class VmapCadastreToolsGridComponent implements OnInit, OnChanges, OnDestroy {
  @ViewChild('searchInput') searchInput: ElementRef;

  @Input() sType: any;
  @Input() oDepartement: string;
  @Input() oCommune: string;
  @Input() oInvariant: Array<string>;
  @Input() oSection: string;
  @Input() oParcelle: Array<string>;
  @Input() oAdresses: Array<string>;
  @Input() oElement: any;
  @Input() bAutoSelect: string; // Sélection par défaut : "one" pour sélectionner uniquement s'il y a un seul élément "all" pour tout sélectionner, rien pour ne rien sélectionner

  @Input() oSelectedAttributeEmiter: EventEmitter<Array<string>> = new EventEmitter();
  @Input() oSelectedParentEmiter: EventEmitter<Array<string>>;

  @Input() olMap: Map;

  @Output() aParcellesEvent = new EventEmitter<Array<string>>();

  // Header du component
  public headerLabel: string;

  // En-tête du tableau
  public attributes: Attribut[];

  // Elements du tableau
  public rows: Array<any>;
  public items: Array<any>;

  // Mot pour filtrer les données
  public searchWord: string = '';
  public oCurrentElement: Array<any> = [];

  // Subscription de l'attribut
  private subscriptionParent: Subscription;

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsCadastreService: VmapToolsCadastreService,
    private vmapToolsCadastreMapService: VmapToolsCadastreMapService,
    private mapUtils: MapCoreUtilsService,
    private translate: TranslateService
  ) {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    this.items = [];
    if (this.oSelectedParentEmiter) {
      this.subscriptionParent = this.oSelectedParentEmiter.subscribe((val) => {
        setTimeout(() => { // Timeout le temps que l'info revienne sur oDepartement, oCommune etc.
          this.oSelectedAttributeEmiter.next(null);
        });
      });
    }
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    this.unsubscribeParent();
    // Suppression des features
    this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
  }

  /**
  * Permet de ne plus observer les changements de données de la fiche descriptive
  */
  private unsubscribeParent(): void {
    // désinscription de la fiche descriptive si subscription
    if (this.subscriptionParent) {
      this.subscriptionParent.unsubscribe();
    }
  }

  /**
  * Initialisation du contenu des tableaux
  */
  private initField(): void {
    if (this.sType == 'commune') {
      this.getElementOfCommunes();
    } else if (this.sType == 'section') {
      this.getElementOfSections();
    } else if (this.sType == 'lieuDit') {
      this.getElementOfLieuxDits();
    } else if (this.sType == 'parcelle') {
      this.getElementOfParcelle();
    } else {
      this.getElementOfParcelle();
    }
    // Met en forme les données obtenues par recherche cartographique
    if (this.oElement) {
      setTimeout(() => {
        this.rows = this.oElement;
        this.items = [...this.rows];

        // Sélection auto (one ou all)
        setTimeout(() => {
          if (this.bAutoSelect === 'one' && this.oElement.length === 1) {
            this.selectedElement(this.oElement[0], false);
          } else if (this.bAutoSelect === 'all' && this.oElement.length > 1) {
            for (let i = 0; i < this.oElement.length; i++) {
              const element = this.oElement[i];
              this.selectedElement(element, false);
            }
          }
        });
      });
    }
  }

  /**
  * Suppression des features
  * Changement des parcelles
  */
  ngOnChanges(): void {
    // Vide les éléments sélectionné à chaque reload des data et renvoi l'information à tool-button-rapport
    this.oCurrentElement = [];
    this.aParcellesEvent.emit(this.oCurrentElement);
    this.initField();
    // Si ça ne concerne pas la recherche cartographique
    if (!this.oElement) {
      if (this.olMap.get('interaction_layer')) {
        this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
      }
    }

    if (this.searchInput) {
      this.searchInput.nativeElement.value = '';
    }
  }

  /**
  * Méthode permettant de remplir le tableau des communes
  */
  private getElementOfCommunes(): void {
    this.headerLabel = this.translate.instant('CADASTRE_TOOL_CITIES_INFORMATIONS');
    this.attributes = [
      { label: 'id_com', value: 'id_com' },
      { label: 'code_com', value: 'code_com' },
      { label: 'nom', value: 'nom' }
    ];

    if (!this.oElement) {
      if (this.oDepartement) {
        this.getCommuneParDepartement();
      } else {
        this.getCommunes();
      }
    }
  }

  /**
  * Méthode permettant de remplir le tableau des sections
  */
  private getElementOfSections(): void {
    this.headerLabel = this.translate.instant('CADASTRE_TOOL_SECTIONS_INFORMATIONS');
    this.attributes = [
      { label: 'section', value: 'section' },
      { label: 'id_sec', value: 'id_sec' },
    ];
    if (this.oCommune) {
      this.getSectionParCommune();
    }
  }

  /**
  * Méthode permettant de remplir le tableau des lieux-dits
  */
  private getElementOfLieuxDits(): void {
    this.headerLabel = this.translate.instant('CADASTRE_TOOL_BUILDING_INFORMATIONS');
    this.attributes = [
      { label: 'tex', value: 'tex' },
      { label: 'section', value: 'section' },
    ];
    if (this.oCommune) {
      this.getLieuDitParCommune();
    }
  }

  /**
  * Méthode permettant de remplir le tableau des parcelles
  * Pour les résultats des parcelles et des bâtis
  */
  private getElementOfParcelle(): void {
    this.headerLabel = this.translate.instant('CADASTRE_TOOL_PARCELS_INFORMATIONS');
    this.attributes = [
      { label: 'id_par', value: 'id_par' },
      { label: 'parcelle', value: 'parcelle' },
      { label: 'sup_fiscale', value: 'sup_fiscale' },
      { label: 'id_sec', value: 'id_sec' }
    ];

    // oInvariant provenant de bati
    if (this.oInvariant) {
      this.getParcelle(this.oInvariant);
    }

    // oParcelle provenant de parcelle-proprietaire
    if (this.oParcelle) {
      this.getParcelle(this.oParcelle);
    }

    // oSection provenant de parcelle section
    if (this.oSection) {
      this.getParcelleParSection();
    }

    // oSection provenant de parcelle dgfip
    if (this.oAdresses) {
      this.getParcelle(this.oAdresses);
    }
  }

  /**
  * Méthode permettant de faire une recherche dans le tableau par mot
  * @param event récupération du mot dans le input
  */
  public searchInGrid(event: any): void {
    if (!event) {
      setTimeout(() => {
        this.items = [...this.rows];
      });
    }
    this.searchWord = event;
    if (this.sType == 'commune') {
      this.items = this.rows.filter(item => item.nom.toLowerCase().includes(this.searchWord.toLowerCase()));
    } else if (this.sType == 'section') {
      this.items = this.rows.filter(item => item.section.toLowerCase().includes(this.searchWord.toLowerCase()));
    } else if (this.sType == 'lieuDit') {
      this.items = this.rows.filter(item => item.tex.toLowerCase().includes(this.searchWord.toLowerCase()));
    } else if (this.sType == 'parcelle' || this.sType == 'bati') {
      this.items = this.rows.filter(item => item.id_par.toLowerCase().includes(this.searchWord.toLowerCase()));
    }
    // Vide les éléments sélectionné à chaque reload des data et renvoi l'information à tool-button-rapport
    this.oCurrentElement = [];
    this.aParcellesEvent.emit(this.oCurrentElement);
    this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();

    // Remet le contour cartographique aux nouveaux élements après recherche
    if (this.oElement) {
      this.items.forEach(element => {
        this.vmapToolsCadastreMapService.selectedElement(this.olMap, element.geom);
        this.vmapToolsCadastreMapService.selectedFeatureContour(element.geom, 'carto', element.id_par);
      });
    }
  }

  /**
  * Structure les données des communes par département
  */
  private getCommuneParDepartement(): void {
    this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
    this.vmapToolsCadastreService.loadCommunesParDepartement_(this.oDepartement, this.olMap).then(response => {
      setTimeout(() => {
        this.rows = response;
        this.items = [...this.rows];
      });
    });
  }

  /**
* Structure les données des communes
*/
  private getCommunes(): void {
    this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
    this.vmapToolsCadastreService.loadCommunes_(this.olMap).then(response => {
      setTimeout(() => {
        this.rows = response;
        this.items = [...this.rows];
      });
    });
  }

  /**
  * Structure les données des sections
  * @returns Promise
  */
  private getSectionParCommune(): void {
    this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
    this.vmapToolsCadastreService.loadSections_(this.oCommune, this.olMap).then(response => {
      setTimeout(() => {
        this.rows = response;
        this.items = [...this.rows];
      });
    });
  }

  /**
  * Structure les données des lieux-dits
  */
  private getLieuDitParCommune(): void {
    this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
    this.vmapToolsCadastreService.loadLieuxDits_(this.oCommune, this.olMap).then(response => {
      setTimeout(() => {
        this.rows = response;
        this.items = [...this.rows];
      });
    });
  }

  /**
  * Structure les données des parcelles pour la recherche par bati | parcelle-proprietaire | parcelle-dgfip
  * @param aParcelle Array<string> liste des parcelles
  */
  private getParcelle(aParcelle: Array<string>): void {
    this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
    if (aParcelle.length > 0) {
      this.vmapToolsCadastreService.loadParcelles_(aParcelle, this.olMap).then(response => {
        setTimeout(() => {
          this.rows = response;
          this.items = [...response];
        });
        // Vide les éléments sélectionné à chaque reload des data et renvoi l'information à tool-button-rapport
        this.oCurrentElement = [];
        this.aParcellesEvent.emit(this.oCurrentElement);
      });
    } else {
      this.items = [];
      this.oCurrentElement = [];
      this.aParcellesEvent.emit(this.oCurrentElement);
    }
  }

  /**
  * Structure les données des parcelles pour la recherche par parcelle-section
  */
  private getParcelleParSection(): void {
    this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
    this.vmapToolsCadastreService.loadParcellesBySection_(this.oCommune, this.oSection, this.olMap).then(response => {
      setTimeout(() => {
        this.rows = response;
        this.items = [...response];
      });
      // Vide les éléments sélectionné à chaque reload des data et renvoi l'information à tool-button-rapport
      this.oCurrentElement = [];
      this.aParcellesEvent.emit(this.oCurrentElement);
    });
  }

  /**
  * Sélectionne une parcelle spécifique
  * @param selection any element selectionné
  * @param bCenterMap boolean si false alors n'effectue pas de zoom sur la carte
  */
  public selectedElement(selection: any, bCenterMap = true): void {
    this.vmapToolsCadastreMapService.selectedElement(this.olMap, selection.geom);
    this.vmapToolsCadastreMapService.refreshVectorLayer(this.olMap, true, false);

    // Elements sélectionnés et désélectionnés
    if (this.oCurrentElement.includes(selection)) {
      for (let i = 0; i < this.oCurrentElement.length; i++) {
        if (selection == this.oCurrentElement[i]) {
          // désélectionner
          this.oCurrentElement.splice(i, 1);
        }
      }
    } else {
      // Sélectionner
      this.oCurrentElement.push(selection);
      // Si c'est une parcelle on ajoute l'id_par pour identifier le contour
      if (selection.id_par) {
        this.vmapToolsCadastreMapService.addFeatureMetadata(selection.geom, 'select', selection.id_par);
      } else {
        this.vmapToolsCadastreMapService.addFeatureMetadata(selection.geom, 'select', null);
      }
    }
    // Centre la carte sur les géométries des élements sélectionnés
    const aGeom = this.getCurrentGeom();
    if (aGeom.length > 0 && bCenterMap === true) {
      this.vmapToolsCadastreMapService.centerGeom(this.olMap, aGeom);
    }
    // Affichage des autres éléments sélectionnés
    if (this.oCurrentElement.length > 0) {
      for (let i = 0; i < this.oCurrentElement.length; i++) {
        const oElement = this.oCurrentElement[i];
        if (oElement.geom) {
          const geometry = this.mapUtils.getGeomFromEWKT(oElement.geom, this.olMap);
          const feature = new Feature({ geometry });
          this.vmapToolsCadastreMapService.addFeatureContour(feature, 'select');
        }
      }
    }
    // Envoi les élements sélectionnés à tools-button-rapport
    this.aParcellesEvent.emit(this.oCurrentElement);
  }

  /**
  * Récupère toutes les géométries des élements sélectionnés
  * @return Array<string> liste des géométries
  */
  private getCurrentGeom(): Array<string> {
    const currentGeom: Array<string> = [];
    this.oCurrentElement.forEach(function (value) {
      currentGeom.push(value.geom);
    });
    return currentGeom;
  }
}
