import { Component, Input, OnInit, EventEmitter, Output, OnChanges, OnDestroy } from '@angular/core';
import { VmapToolsCadastreService } from 'src/modules/vmap/components/carto/components/vmap-tools/components/vmap-tools-cadastre/service/vmap-tools-cadastre.service';
import { Subscription } from 'rxjs';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

interface Attribut {
  label: string;
  value: string;
}

@Component({
  selector: 'app-vmap-cadastre-tools-list-bati',
  templateUrl: './vmap-cadastre-tools-list-bati.component.html',
  styleUrls: ['./vmap-cadastre-tools-list-bati.component.scss']
})

export class VmapCadastreToolsListBatiComponent implements OnInit, OnChanges, OnDestroy {
  @Input() sType: any;
  @Input() oAttribute: string;

  @Input() oDepartement: string;
  @Input() oCommune: string;
  @Input() oProprietaire: string;

  @Input() oSelectedAttributeEmiter: EventEmitter<string> = new EventEmitter();
  @Input() oSelectedParentEmiter: EventEmitter<string>;

  // envoi du numéro d'invariant
  @Output() oInvariantEvent = new EventEmitter<Array<string>>();
  @Output() oParcelleEvent = new EventEmitter<Array<string>>();
  @Output() oVoieEvent = new EventEmitter<string>();
  @Output() aCompteEvent = new EventEmitter<Array<string>>();

  // Liste des attributs
  public attributesProprietaires: Attribut[];
  public attributesComptes: Attribut[];
  public attributesInvariants: Attribut[];
  public attributesVoies: Attribut[];

  public itemsProprietaires: Array<any>;
  public itemsComptes: Array<any>;
  public itemsInvariants: Array<any>;
  public itemsVoies: Array<any>;
  public aCurrentDnupro: Array<any> = [];

  public searchWord: string = '';

  public oCurrentProprietaire: string = null;
  public oCurrentCompte: Array<string> = [];
  public oCurrentInvariant: Array<any> = [];
  public oCurrentParcelle: Array<any> = [];
  public oCurrentVoie: string = null;

  // Subscription de l'attribut
  private subscriptionParent: Subscription;

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsCadastreService: VmapToolsCadastreService,
    private vitisCheck: VitisCheck) {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    this.initField();
    if (this.oSelectedParentEmiter) {
      this.subscriptionParent = this.oSelectedParentEmiter.subscribe((val) => {
        setTimeout(() => { // Timeout le temps que l'info revienne sur oDepartement, oCommune etc.
          this.initField();
          this.oSelectedAttributeEmiter.next(null);
        });
      });
    }
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    this.unsubscribeParent();
  }

  /**
* Permet de ne plus observer les changements de données de la fiche descriptive
*/
  private unsubscribeParent(): void {
    // désinscription de la fiche descriptive si subscription
    if (this.subscriptionParent) {
      this.subscriptionParent.unsubscribe();
    }
  }

  /**
   * Initialisation du contenu des tableaux
   */
  private initField(): void {
    this.attributesProprietaires = [
      { label: 'Liste des proprietaires :', value: 'value' }
    ];
    this.attributesComptes = [
      { label: 'Liste des comptes :', value: 'value' }
    ];
    this.attributesInvariants = [
      { label: 'Liste des invariants :', value: 'value' }
    ];
    this.attributesVoies = [
      { label: 'Liste des voies :', value: 'value' }
    ];
  }

  /**
   * Appelé lorsqu'il y a des changements sur les input
   */
  ngOnChanges(): void {
    if (this.oProprietaire && this.oCommune) {
      this.searchDataproprietaire().then(() => {
        this.reloadData();
        this.getCompteParProprietaire();
      });
    }

    if (this.sType == 'parcelle-dgfip') {
      this.getVoirie().then(() => {
      });
    }
  }

  /**
  * Permet de reload les listes d'objets
  */
  private reloadData(): void {
    const aParcelle: Array<string> = [];
    // Liste d'objets
    this.itemsComptes = [];
    this.itemsInvariants = [];
    // Elements en cours de sélection
    this.oCurrentInvariant = [];
    this.oCurrentParcelle = [];
    // Parcelles dans tools-grid
    this.oInvariantEvent.emit(aParcelle);
    this.oParcelleEvent.emit(aParcelle);
  }

  /**
  * Requête pour récupérer les propriétaires
  * @returns Promise
  */
  private searchDataproprietaire(): Promise<any> {
    this.itemsProprietaires = [];
    this.oCurrentProprietaire = '';
    return new Promise((resolve, reject) => {
      this.vmapToolsCadastreService.loadProprietaires_(this.oCommune, this.oProprietaire).then(response => {
        // Vérifie sous quelle forme le WS renvoi la réponse et formatte en array
        if (this.vitisCheck.isArray(response)) {
          if (response.length > 0) {
            this.itemsProprietaires = [...response];
            this.oCurrentProprietaire = this.itemsProprietaires[0].DDENOM;
            resolve(true);
          } else {
            // Vide le compte
            this.getCompteParProprietaire();
          }
        } else {
          if (this.vitisCheck.isDefAndNotNull(response)) {
            Object.keys(response).map(index => {
              this.itemsProprietaires.push(response[index]);
              this.oCurrentProprietaire = this.itemsProprietaires[0].DDENOM;
              resolve(true);
            });
          } else {
            // Vide le compte
            this.getCompteParProprietaire();
          }
        }
      }).catch((error) => {
        reject(error);
      });
    });
  }

  /**
  * Méthode permettant de récupérer les comptes en fonction des propriétaires
  */
  private getCompteParProprietaire(): void {
    this.itemsComptes = [];
    this.oCurrentCompte = [];
    if (this.oCurrentProprietaire !== '') {
      this.vmapToolsCadastreService.loadComptesParProprietaires_(this.oCommune, this.oCurrentProprietaire).then(response => {
        // Vérifie sous quelle forme le WS renvoi la réponse et formatte en array
        if (this.vitisCheck.isArray(response)) {
          if (response.length > 0) {
            this.itemsComptes = [...response];
          }
        } else {
          if (this.vitisCheck.isDefAndNotNull(response)) {
            Object.keys(response).map(index => {
              this.itemsComptes.push(response[index]);
            });
          }
        }
        // Vérification de la présence de comptes
        if (this.itemsComptes.length > 0) {
          for (let i = 0; i < this.itemsComptes.length; i++) {
            this.oCurrentCompte.push(this.itemsComptes[i].DNUPRO);
          }
        }
        // Rempli la liste des invariants et des parcelles pour les batis et parcelle-proprietaire
        if (this.sType == 'bati') {
          this.getInvariantParCompte();
        } else if (this.sType == 'parcelle-proprietaire') {
          this.getDescriptionParcelle();
        }

      });
    } else {
      this.itemsComptes = [];
      // Rempli la liste des invariants et des parcelles pour les batis et parcelle-proprietaire
      if (this.sType == 'bati') {
        this.getInvariantParCompte();
      } else if (this.sType == 'parcelle-proprietaire') {
        this.getDescriptionParcelle();
      }
    }
    this.aCompteEvent.emit(this.itemsComptes);
  }

  /**
  * Méthode permettant de récupérer les invariants en fonction des comptes
  */
  private getInvariantParCompte(): void {
    this.itemsInvariants = [];
    this.oCurrentInvariant = [];
    this.getDnupro();
    if (this.aCurrentDnupro.length > 0) {
      this.vmapToolsCadastreService.loadInvariants_(this.aCurrentDnupro).then(response => {
        // Vérifie sous quelle forme le WS renvoi la réponse et formatte en array
        if (this.vitisCheck.isArray(response)) {
          if (response.length > 0) {
            this.itemsInvariants = [...response];
          }
        } else {
          if (this.vitisCheck.isDefAndNotNull(response)) {
            Object.keys(response).map(index => {
              this.itemsInvariants.push(response[index]);
            });
          }
        }
        if (this.itemsInvariants.length > 0) {
          this.oCurrentInvariant = [...this.itemsInvariants];
        }
        this.getIdPar();
      });
    } else {
      this.getIdPar();
      this.oParcelleEvent.emit([]);
    }
  }

  /**
  * Méthode permettant de récupérer les parcelles en fonction des comptes
  */
  private getDescriptionParcelle(): void {
    this.oCurrentParcelle = [];
    this.getDnupro();
    if (this.oCurrentCompte.length > 0) {
      this.vmapToolsCadastreService.loadDescriptionParcelleParProprietaires_(this.oCommune, this.oCurrentCompte).then(response => {
        // Vérifie sous quelle forme le WS renvoi la réponse et formatte en array
        if (this.vitisCheck.isArray(response)) {
          if (response.length > 0) {
            this.oCurrentParcelle = [...response];
          }
        } else {
          if (this.vitisCheck.isDefAndNotNull(response)) {
            Object.keys(response).map(index => {
              this.oCurrentParcelle.push(response[index]);
            });
          }
        }
        this.getIdPar();
      });
    } else {
      this.getIdPar();
    }
  }

  /**
  * Envoie le dnupro à tools-button-rapport
  */
  private getDnupro(): void {
    this.aCurrentDnupro = [];
    if (this.oCurrentCompte.length > 0 && this.itemsComptes.length > 0) {
      for (let i = 0; i < this.oCurrentCompte.length; i++) {
        for (let j = 0; j < this.itemsComptes.length; j++) {
          if (this.itemsComptes[j].DNUPRO == this.oCurrentCompte[i]) {
            this.aCurrentDnupro.push(this.itemsComptes[j]);
          }
        }
      }
    }
    this.aCompteEvent.emit(this.aCurrentDnupro);
  }

  /**
   * Méthode permettant de sélectionner le propriétaire
   * @param selection any propriétaires selectionné (click)
  */
  public selectedProprietaire(selection: any): void {
    this.oCurrentProprietaire = selection;
    this.getCompteParProprietaire();
  }

  /**
   * Méthode permettant de sélectionner les comptes
    * @param selection any compte selectionné (click)
  */
  public selectedCompte(selection: any): void {
    this.itemsInvariants = [];
    this.oCurrentInvariant = [];
    if (this.oCurrentCompte.includes(selection)) {
      for (let i = 0; i < this.oCurrentCompte.length; i++) {
        if (selection == this.oCurrentCompte[i]) {
          this.oCurrentCompte.splice(i, 1);
        }
      }
      if (this.sType == 'bati') {
        this.getInvariantParCompte();
      } else if (this.sType == 'parcelle-proprietaire') {
        this.getDescriptionParcelle();
      }
    } else {
      this.oCurrentCompte.push(selection);
      if (this.sType == 'bati') {
        this.getInvariantParCompte();
      } else if (this.sType == 'parcelle-proprietaire') {
        this.getDescriptionParcelle();
      }
    }
  }

  /**
  * Méthode permettant de sélectionner les invariants
  * @param selection any invariant selectionné (click)
  */
  public selectedInvariant(selection: any): void {
    if (this.oCurrentInvariant.includes(selection)) {
      for (let i = 0; i < this.oCurrentInvariant.length; i++) {
        if (selection == this.oCurrentInvariant[i]) {
          this.oCurrentInvariant.splice(i, 1);
        }
      }
      this.getIdPar();
    } else {
      this.oCurrentInvariant.push(selection);
      this.getIdPar();
    }
  }

  /**
 * Méthode permettant de remplir la liste des parcelles selon les invariants sélectionnés
 */
  private getIdPar(): void {
    const aParcelle: Array<string> = [];
    if (this.sType == 'bati') {
      if (this.oCurrentInvariant.length > 0) {
        for (let i = 0; i < this.oCurrentInvariant.length; i++) {
          aParcelle.push(this.oCurrentInvariant[i].ID_PAR);
        }
      }
      // Envoi les informations sur les parcelles à la grid
      this.oInvariantEvent.emit(aParcelle);
    } else if (this.sType == 'parcelle-proprietaire') {
      if (this.oCurrentParcelle.length > 0) {
        for (let i = 0; i < this.oCurrentParcelle.length; i++) {
          aParcelle.push(this.oCurrentParcelle[i].ID_PAR);
        }
      }
      // Envoi les informations sur les parcelles à la grid
      this.oParcelleEvent.emit(aParcelle);
    }
  }

  /**
   * Requête pour récupérer les voies par commune
   * @returns Promise
   */
  private getVoirie(): Promise<any> {
    this.itemsVoies = [];
    this.oCurrentVoie = '';
    return new Promise((resolve, reject) => {
      this.vmapToolsCadastreService.loadVoiesByCommunes_(this.oCommune).then(response => {
        // Vérifie sous quelle forme le WS renvoi la réponse et formatte en array
        if (this.vitisCheck.isDefAndNotNull(response)) {
          if (this.vitisCheck.isArray(response)) {
            this.itemsVoies = [...response];
          } else {
            Object.keys(response).map(index => {
              this.itemsVoies.push(response[index]);
            });
          }
          this.oCurrentVoie = this.itemsVoies[0];
          this.oVoieEvent.emit(this.itemsVoies[0].ID_RIVOLI);
          resolve(true);
        } else {
          this.itemsVoies = [];
          this.oCurrentVoie = '';
          this.oVoieEvent.emit();
          resolve(false);
        }
      }).catch((error) => {
        reject(error);
      });
    });
  }

  /**
  * Méthode permettant de sélectionner une voie
  * @param selection any voirie selectionnée (click)
  */
  public selectedVoirie(selection: any): void {
    this.oCurrentVoie = selection;
    this.oVoieEvent.emit(selection.ID_RIVOLI);
  }
}
