import { Component, Input, Output, OnInit, EventEmitter, OnDestroy } from '@angular/core';
import { vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { Subscription } from 'rxjs';


@Component({
  selector: 'app-vmap-cadastre-tools-search-input',
  templateUrl: './vmap-cadastre-tools-search-input.component.html',
  styleUrls: ['./vmap-cadastre-tools-search-input.component.scss']
})

export class VmapCadastreToolsSearchInputComponent implements OnInit, OnDestroy {

  @Input() sType: string;

  @Input() oDepartement: string;
  @Input() oCommune: string;

  @Input() oSelectedAttributeEmiter: EventEmitter<string> = new EventEmitter();
  @Input() oSelectedParentEmiter: EventEmitter<string>;

  // Envoi du mot recherché
  @Output() oProprietaireEvent = new EventEmitter<string>();

  // Subscription de l'attribut
  private subscriptionParent: Subscription;

  // Mot recherché
  public searchWord: string;

  /**
   * Constructeur
   */
  constructor() {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    if (this.oSelectedParentEmiter) {
      this.subscriptionParent = this.oSelectedParentEmiter.subscribe((val) => {
        setTimeout(() => { // Timeout le temps que l'info revienne sur oDepartement, oCommune etc.
          this.oSelectedAttributeEmiter.next(null);
        });
      });
    }
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    this.unsubscribeParent();
  }

  /**
* Permet de ne plus observer les changements de données de la fiche descriptive
*/
  private unsubscribeParent(): void {
    // désinscription de la fiche descriptive si subscription
    if (this.subscriptionParent) {
      this.subscriptionParent.unsubscribe();
    }
  }

  /**
  * Méthode permettant de faire une recherche dans le tableau par mot
  * @param event récupération du mot dans le input
  */
  public searchFilter() {
    if (this.counter() === true) {
      if (this.sType == 'proprietaire') {
        this.oProprietaireEvent.emit(this.searchWord);
      }
    }
  }

  /**
 * Méthode permettant de compter le nombre de caractères dans le mot
 * @Return boolean true si plus de 2 caractères
 */
  private counter(): boolean {
    if (this.searchWord.length > 2) {
      return true;
    } else {
      vitisAlert.fireToast({
        type: 'warning',
        title: 'Veuillez entrer au moins 3 caractères'
      });
    }
  }
}
