import { EventEmitter, Injectable, Output } from '@angular/core';
import { Vector as VectorLayer } from 'node_modules/ol/layer.js';
import { Vector as VectorSource } from 'ol/source.js';
import Feature from 'ol/Feature';
import { MapCoreDrawService } from 'src/app/vitis/map-core/services/map-core-draw/map-core-draw.service';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { Circle as CircleStyle, Stroke, Style, Fill } from 'ol/style.js';
import { Geometry, LineString, MultiLineString, MultiPoint, MultiPolygon, Point, Polygon } from 'ol/geom';
import { VmapToolsCadastreService } from './vmap-tools-cadastre.service';
import { Map } from 'ol';
import { MapCoreJSONService } from 'src/app/vitis/map-core/services/map-core-json/map-core-json.service';
import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import * as olExtent from 'ol/extent';
import { environment } from 'src/environments/environment';
import { VmapToolsPrintService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-print/vmap-tools-print.service';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { VmapToolsSetCircleRadiusService } from 'src/app/modules/vmap/carto/services/vmap-tools/vmap-tools-set-circle-radius/vmap-tools-set-circle-radius.service';
import { fromCircle } from 'ol/geom/Polygon';
import { EventsKey } from 'ol/events';
import { unByKey } from 'ol/Observable';

@Injectable({
  providedIn: 'root'
})

export class VmapToolsCadastreMapService {
  private feature: Feature<Geometry>;
  private vectorSource: VectorSource<Geometry> = new VectorSource();
  private drawVectorSource: VectorSource<Geometry> = new VectorSource();
  private circleVectorSource: VectorSource<Geometry> = new VectorSource();
  private vectorLayer: VectorLayer<VectorSource<Geometry>>;
  public aParcelles: any;
  public mapScale: number;
  public iResizeFactor = 1; // facteur de resize de la zone d'impression (sert pour les petits écrans)

  public oGeomElementEmiter: EventEmitter<any> = new EventEmitter();
  public oEWKTEmitter: EventEmitter<string> = new EventEmitter();
  public oRPEmitter: EventEmitter<string> = new EventEmitter();
  public oFicheDescrlleEmitter: EventEmitter<string> = new EventEmitter();

  private aListenerKeyInteraction: Array<EventsKey> = [];

  private olInteraction: any;

  /**
   * Constructeur
   */
  constructor(
    private mapCoreDrawService: MapCoreDrawService,
    private mapUtils: MapCoreUtilsService,
    private mapCoreJSONService: MapCoreJSONService,
    private vmapToolsCadastreService: VmapToolsCadastreService,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private printService: VmapToolsPrintService,
    private vmapToolsService: VmapToolsService,
    private vmapToolsSetCircleRadiusService: VmapToolsSetCircleRadiusService
  ) { }

  /**
  * Style pointillé pour le tracé du dessin
  */
  private olSelectStyle = new Style({
    fill: new Fill({
      color: 'rgba(255, 255, 255, 0.2)',
    }),
    stroke: new Stroke({
      color: 'rgba(0, 0, 0, 0.5)',
      lineDash: [10, 10],
      width: 2,
    }),
    image: new CircleStyle({
      radius: 5,
      stroke: new Stroke({
        color: 'rgba(0, 0, 0, 0.7)',
      }),
      fill: new Fill({
        color: 'rgba(255, 255, 255, 0.2)',
      }),
    }),
  });

  /**
  * Initiation du mode dessin
  * @param olMap carte en cours
  * @param bClear si false alors ne nettoie pas la couche si elle existe déjà
  */
  public refreshVectorLayer(olMap: any, bClear = true, bClearCarto = true): any {
    if (this.vectorLayer) {
      if (bClear) {
        if (bClearCarto === true) {
          // Vide toutes les features
          this.vectorLayer.getSource().clear();
        } else {
          // Vide toutes les features sauf les type carto
          const aFeatures = this.vectorLayer.getSource().getFeatures();
          aFeatures.forEach((feature: Feature<Geometry>) => {
            if (feature.get('type') !== 'carto') {
              this.vectorLayer.getSource().removeFeature(feature);
            }
          });
        }
      }
    } else {
      this.vectorLayer = this.mapCoreDrawService.initDrawLayer(olMap, this.vectorSource);
    }
  }

  /**
  * Dessine un contour sur la carte de this.feature
  * @param geom géométrie au format EWKT de l'élément
  * @param type type de geometrie à effacer (carto|select)
  * @param idPar
  */
  public selectedFeatureContour(geom: string, type: string, idPar: string): void {
    this.addFeatureMetadata(geom, type, idPar);
    this.addFeatureContour(this.feature, type);
  }

  /**
   * Ajoute les métadonnées à this.feature
   * @param geom
   * @param type
   * @param idPar
   */
  public addFeatureMetadata(geom: string, type: string, idPar: string): void {
    this.feature.set('isSelectFeatureContour', true);
    this.feature.set('geom_id', geom);
    this.feature.set('type', type);
    this.feature.set('idPar', idPar);
  }

  /**
   * Ajoute la feature à la carte
   * @param feature
   * @param type
   */
  public addFeatureContour(feature: Feature<Geometry>, type: string): void {
    try {
      if (type == 'carto') {
        const selectStyle = new Style({
          fill: new Fill({
            color: 'rgba(255, 255, 255, 0.2)',
          }),
          stroke: new Stroke({
            color: '#D9534F',
            width: 2,
          }),
          image: new CircleStyle({
            radius: 3,
            fill: new Fill({
              color: '#D9534F',
            }),
          }),
        });
        feature.setStyle(selectStyle);
      } else {
        const selectStyle = new Style({
          fill: new Fill({
            color: 'rgba(255, 255, 255, 0.2)',
          }),
          stroke: new Stroke({
            color: '#337ab7',
            width: 2,
          }),
          image: new CircleStyle({
            radius: 3,
            fill: new Fill({
              color: '#337ab7',
            }),
          }),
        });
        feature.setStyle(selectStyle);
      }
      feature.set('type', type);
      this.vectorLayer.getSource().addFeature(feature);
    } catch (error) {
      console.error(error);
    }
  }

  /**
   * Supprime le contour sur la carte
   * @param geom géométrie au format EWKT de l'élément
   * @param type type de geometrie à effacer (carto|select)
   */
  public hideSpecificSelectedFeatureContour(geom: string, type: string): void {
    const aFeatures = this.vectorLayer.getSource().getFeatures();
    for (let i = aFeatures.length - 1; i >= 0; i--) {
      const olFeature = aFeatures[i];
      if (olFeature.get('isSelectFeatureContour') === true) {
        if (geom == olFeature.get('geom_id') && olFeature.get('type') == type) {
          this.vectorLayer.getSource().removeFeature(olFeature);
        }
      }
    }
  }

  /**
  * Supprime tous les contours sur la carte
  */
  public hideAllSelectedFeatureContour(): void {
    if (this.vectorLayer) {
      const aFeatures = this.vectorLayer.getSource().getFeatures();
      for (let i = aFeatures.length - 1; i >= 0; i--) {
        const olFeature = aFeatures[i];
        if (olFeature.get('isSelectFeatureContour') === true) {
          this.vectorLayer.getSource().removeFeature(olFeature);
        }
      }
    }
  }

  /**
  * Initialise le vector Layer pour ajouter les sélections à la carte
  */
  private initDrawing(olMap: Map): void {
    olMap.set('interaction_layer', this.mapCoreDrawService.initDrawLayer(olMap, this.drawVectorSource, this.olSelectStyle));
  }

  /**
   * Sélection d'une parcelle par le dessin
   * Relevé de propriété et fiche urbanisme en un clic
   * @param olMap
   * @param sGeomType
   */
  public selectMapParcelle(olMap: Map, sGeomType: string, type: string): Promise<string> {
    let currentIdPar: any;
    let olInteraction: any;
    return new Promise(async (resolve, reject) => {
      try {
        // VectorLayer contenant les parcelles
        this.refreshVectorLayer(olMap, true);
        // VectorLayer du dessin
        this.initDrawing(olMap);
        if (olMap.get('interaction_layer')) {
          const interaction_layer = olMap.get('interaction_layer');
          olInteraction = await this.mapCoreDrawService.addDrawInteraction(sGeomType, olMap, interaction_layer);
        }
        // A chaque fin de dessin
        olInteraction.on('drawend', async (evt: any) => {
          // Clean les parcelles et le dessin à chaque nouveau clic
          setTimeout(() => {
            if (olMap.get('interaction_layer')) {
              const interaction_layer = olMap.get('interaction_layer');
              interaction_layer.getSource().clear();
            }
          });

          const mapProj: string = olMap.getView().getProjection().getCode().substring(5);
          const geometryAdded: Polygon | Point | LineString = evt.feature.getGeometry();
          const EWKT = this.mapUtils.getEWKTFromGeom(geometryAdded, olMap);
          const scale = this.mapUtils.getScale(olMap, { pretty: false });
          const intersect_buffer: number = mapProj == '4326' ? 0.1 : (5 * scale) /  1000;
          currentIdPar = await this.intersectParcelle(EWKT, intersect_buffer, olMap);

          if (type == 'ficheDescriptive') {
            const parcelleFicheDescr = await this.ficheDescriptive(currentIdPar);
            resolve(parcelleFicheDescr);
          } else if (type == 'releveDePropriete') {
            const parcelleRP = await this.descriptionParcelle(currentIdPar);
            this.oRPEmitter.next(parcelleRP);
            resolve(parcelleRP);
          }
        });
        if (olMap.get('interaction_layer')) {
          const interaction_layer = olMap.get('interaction_layer');
          interaction_layer.getSource().clear();
        }
      } catch (error) {
        console.error(error);
        reject(error);
      }
    });
  }

  /**
  * Intersection d'une parcelle pour ouverture de la modale
  * @param geom
  * @returns Promise
  */
  private intersectParcelle(geom: string, buffer: number, olMap: any): Promise<any> {
    return new Promise(async (resolve, reject) => {
      try {
        let currentIdsPar: string;
        const aParcelle = await this.vmapToolsCadastreService.loadParcellesByGeom_(geom, buffer, 'POINT', olMap.getView().getProjection().getCode().substring(5));
        this.aParcelles = aParcelle;
        // Si une parcelle est détectée
        if (this.aParcelles.length > 0) {
          this.selectedElement(olMap, aParcelle[0].geom);
          this.selectedFeatureContour(aParcelle[0].geom, 'select', aParcelle[0].id_par);
          currentIdsPar = aParcelle[0].id_par;
          resolve(currentIdsPar);
        }
      } catch (error) {
        console.error(error);
        reject(error);
      }
    });
  }

  /**
   * Termine les interactions en cours sur la carte
   * @param olMap carte en cours
   */
  public removeInteractions(olMap: Map) {
    if (this.aListenerKeyInteraction) {
      this.aListenerKeyInteraction.forEach((key, index) => {
        if (key) {
          unByKey(key);
        }
      });
      this.aListenerKeyInteraction = [];
    }
    this.vmapToolsSetCircleRadiusService.eventDrawstartUnbind();
    this.vmapToolsSetCircleRadiusService.eventInnerDrawstartUnbind();
    this.olInteraction = null;

    this.mapCoreDrawService.removeInteractions(olMap);
    this.vectorSource.clear();
  }

  /**
   * Permet de déterminer le système de projection en fonction des properties
   * @returns string
   */
  private getProjection(): string {
    let projection: string;
    if (!environment.properties.cadastre_database_projection) {
      projection = '2154';
    } else {
      projection = environment.properties.cadastre_database_projection.substring(5, 9);
    }
    return projection;
  }
  /**
   * Sélection d'une commune | section | lieu-dit | parcelle avec la géométrie
   * @param olMap carte en cours
   * @param sGeomType string type de geometrie sélectionnée
   * @param sType string object sur lequel porte la recherche
   * @returns Promise
   */
  public getGeom(olMap: Map, sGeomType: string, sType: string): Promise<any> {
    let olInteraction: any;
    const aTypeOfElement = ['bati', 'parcelle', 'urbanisme'];
    const aReleveDePropriete = ['releveDeProprieteParcelle', 'releveDeProprieteParcelleTier', 'releveDeProprieteStandard', 'releveDeProprieteTier'];
    return new Promise(async (resolve, reject) => {

      // VectorLayer contenant les parcelles
      this.refreshVectorLayer(olMap, true);
      // VectorLayer du dessin
      this.initDrawing(olMap);
      if (olMap.get('interaction_layer')) {
        const interaction_layer = olMap.get('interaction_layer');
        olInteraction = await this.mapCoreDrawService.addDrawInteraction(sGeomType, olMap, interaction_layer);
        this.olInteraction = olInteraction;
      }

      // Pour les cercles : permet l'affichage de la popup
      this.vmapToolsSetCircleRadiusService.setCircleListener(olInteraction, olMap, 'setCircle', this);

      // A chaque fin de dessin
      const drawendInteraction = olInteraction.on('drawend', async (evt: any) => {
        setTimeout(() => {
          // Clean les parcelles et le dessin à chaque nouveau clic
          if (olMap.get('interaction_layer')) {
            const interaction_layer = olMap.get('interaction_layer');
            interaction_layer.getSource().clear();
            this.hideAllSelectedFeatureContour();
          }
        });

        // Met en forme les nouvelles coordonnées
        const mapProj: string = olMap.getView().getProjection().getCode().substring(5);
        const geometryAdded: Polygon | Point | LineString = evt.feature.getGeometry().getType() == 'Circle' ? fromCircle(evt.feature.getGeometry()) : evt.feature.getGeometry();
        const EWKT = this.mapUtils.getEWKTFromGeom(geometryAdded, olMap);
        const scale = this.mapUtils.getScale(olMap, { pretty: false });
        const intersect_buffer: number = mapProj == '4326' ? 0.1 : (5 * scale) /  1000;

        // Récupération des données par coordonnées
        let currentElement: any;
        try {
          if (sType == 'commune') {
            currentElement = await this.vmapToolsCadastreService.loadCommunesByGeom_(EWKT, intersect_buffer, sGeomType, mapProj);
          } else if (sType == 'section') {
            currentElement = await this.vmapToolsCadastreService.loadSectionsByGeom_(EWKT, intersect_buffer, sGeomType, mapProj);
          } else if (sType == 'lieuDit') {
            currentElement = await this.vmapToolsCadastreService.loadLieuxDitsByGeom_(EWKT, intersect_buffer, sGeomType, mapProj);
          } else if (aTypeOfElement.includes(sType) || aReleveDePropriete.includes(sType)) {
            currentElement = await this.vmapToolsCadastreService.loadParcellesByGeom_(EWKT, intersect_buffer, sGeomType, mapProj);
          }

          for (let i = 0; i < currentElement.length; i++) {
            this.selectedElement(olMap, currentElement[i].geom);
            if (sType == 'urbanisme') {
              // Créer des contours bleus pour chaque élements
              this.selectedFeatureContour(currentElement[i].geom, 'select', currentElement[i].id_par);
              this.downloadFicheUrba(currentElement[i].id_par, olMap);
            } else if (aReleveDePropriete.includes(sType)) {
              this.selectedFeatureContour(currentElement[i].geom, 'select', currentElement[i].id_par);
              currentElement = await this.descriptionParcelle(currentElement[i].id_par);
              this.downloadReleveDePropriete(currentElement, sType);
            } else {
              // Créer des contours rouges pour chaque élements
              this.selectedFeatureContour(currentElement[i].geom, 'carto', null);
            }
          }
        } catch (error) {
          console.error(error);
          reject(error);
        }
        this.oGeomElementEmiter.next(currentElement);
        resolve(currentElement);

        // Supprime les interactions pour faire un cercle avec un rayon
        this.vmapToolsSetCircleRadiusService.eventInnerDrawstartUnbind();
      });
      this.aListenerKeyInteraction.push(drawendInteraction);
    });
  }

  /**
  * Sélection d'une commune | section | lieu-dit | parcelle avec le géométrie
  * @param geom géométrie au format EWKT de l'élément sélectionné dans tools-grid
  * @param olMap carte en cours
  */
  public selectedElement(olMap: any, geom: string) {
    const geometry = this.mapUtils.getGeomFromEWKT(geom, olMap);
    this.feature = new Feature({ geometry });
    this.mapScale = Math.round(this.mapUtils.getScale(olMap, { pretty: false }));
  }

  /**
  * Centre sur les géométries
  * @param geom géométrie au format EWKT de l'élément sélectionné dans tools-grid
  * @param olMap carte en cours
  */
  public centerGeom(olMap: any, geom: Array<string>) {
    const aGeom: Array<Array<number>> = [];
    let finalExtent = olExtent.createEmpty();
    for (let i = 0; i < geom.length; i++) {
      const geometry = this.mapUtils.getGeomFromEWKT(geom[i], olMap);
      aGeom.push(geometry.getExtent());
    }
    for (const aExtent of aGeom) {
      finalExtent = olExtent.extend(finalExtent, aExtent);
    }
    olMap.getView().fit(finalExtent, {
      padding: this.vmapToolsService.getCenterPadding()
    });
  }

  /**
  * Transmets les éléments séléctionnés si ils changent
  * @returns EventEmitter<any>
  */
  public getCurrentElement(): EventEmitter<any> {
    return this.oGeomElementEmiter;
  }

  /**
* Transmets les changements de géométrie au format EWKT
* @returns EventEmitter<any>
*/
  public getRPParcelle(): EventEmitter<string> {
    return this.oRPEmitter;
  }

  /**
* Transmets les changements de géométrie au format EWKT
* @returns EventEmitter<any>
*/
  public getFicheDescrParcelle(): EventEmitter<string> {
    return this.oFicheDescrlleEmitter;
  }

  /**
  * Téléchargement de la fiche d'urbanisme
  * @param idPar string id des parcelles
  * @returns Promise
  */
  public downloadFicheUrba(idPar: string, olMap: any): Promise<any> {
    return new Promise((resolve, reject) => {

      // Récupère la définition HTML du template
      this.getFicheUrbaTemplate_().then((sPrintTemplate: string) => {

        const oPrintParams = this.printService.getPrintParams({
          olMap: olMap,
          templateDef: sPrintTemplate,
          mapScale: this.mapScale / this.iResizeFactor,
          mapDef: this.mapCoreJSONService.getMapDef(olMap),
          mapFeature: this.feature,
        });

        if (oPrintParams &&
          oPrintParams.aScope &&
          oPrintParams.aIncludes &&
          oPrintParams.aMaps) {

          const sScope = JSON.stringify(oPrintParams.aScope);
          const sIncludes = JSON.stringify(oPrintParams.aIncludes);
          const sMaps = JSON.stringify(oPrintParams.aMaps);

          const data = {
            ID_PAR: idPar,
            scope_json: sScope,
            maps_json: sMaps,
            includes_json: sIncludes,
          };
          try {
            this.vmapToolsCadastreService.downloadFicheUrba_(data).then(response => {
              if (response &&
                response.rapport) {
                this.vitisRequest.ajaxDownloadFile(response.rapport, 'Fiche_Urbanisme_' + idPar + '_' + Date.now() + '.pdf');
              }
            });
            resolve(true);
          } catch (error) {
            console.error(error);
            reject(error);
          }
        }
      });
    });
  }

  /**
  * Téléchargement de le relevé de propriété pour le bouton mobile
  * @param data any la description de la parcelle
  * @param type string le type de relevé de propriété
  * @returns Promise
  */
  public downloadReleveDePropriete(data: any, type: string): Promise<void> {
    const idDnupro = data[0].ID_DNUPRO;
    const idPar = data[0].ID_PAR;
    return new Promise((resolve, reject) => {
      try {
        if (type === 'releveDeProprieteStandard') {
          this.vmapToolsCadastreService.downloadRPStandard_(idDnupro).then(response => {
            this.vitisRequest.ajaxDownloadFile(response.rapport, 'Relevé_de_propriete_standard_' + idDnupro + '.pdf');
          });
        } else if (type === 'releveDeProprieteParcelle') {
          this.vmapToolsCadastreService.downloadRPParcelle_(idPar).then(response => {
            this.vitisRequest.ajaxDownloadFile(response.rapport, 'Relevé_de_propriete_parcelle_' + idPar + '.pdf');
          });
        } else if (type === 'releveDeProprieteTier') {
          this.vmapToolsCadastreService.downloadRPTier_(idDnupro).then(response => {
            this.vitisRequest.ajaxDownloadFile(response.rapport, 'Relevé_de_propriete_tier_' + idDnupro + '.pdf');
          });
        } else if (type === 'releveDeProprieteParcelleTier') {
          this.vmapToolsCadastreService.downloadRPParcelleTier_(idPar).then(response => {
            this.vitisRequest.ajaxDownloadFile(response.rapport, 'Relevé_de_propriete_parcelle_tier_' + idPar + '.pdf');
          });
        }
      } catch (error) {
        console.error(error);
        reject(error);
      }
    });
  }

  /**
  * Permet d'obtenir la desciption des parcelles sélectionnées
  * @param currentIdPar Liste des id_par des parcelles
  */
  public ficheDescriptive(currentIdPar: string): Promise<string> {
    return new Promise((resolve, reject) => {
      try {
        this.vmapToolsCadastreService.loadFicheDescriptiveByIdPar_(currentIdPar).then(response => {
          const aDescriptionParcelle = response;
          this.oFicheDescrlleEmitter.next(aDescriptionParcelle);
          resolve(aDescriptionParcelle);
        });
      } catch (error) {
        console.error(error);
        reject(error);
      }
    });
  }

  /**
  * Permet d'obtenir la description des parcelles sélectionnées pour les relevés de propriété
  * @param currentIdPar Liste des id_par des parcelles
  */
  public descriptionParcelle(currentIdPar: string): Promise<any> {
    return new Promise((resolve, reject) => {
      try {
        this.vmapToolsCadastreService.loadDescriptionParcelleByIdPar_(currentIdPar).then(response => {
          const aDescriptionParcelle = [...response];
          resolve(aDescriptionParcelle);
        });
      } catch (error) {
        console.error(error);
        reject(error);
      }
    });
  }

  /**
   * Récupère la définition du modèle d'impression de la fiche urba
   * @returns Promise
   */
  private getFicheUrbaTemplate_(): Promise<any> {
    return new Promise((resolve, reject) => {
      const url = `${environment.properties.app_api_url}/cadastre/printficheurbanisme`;

      // Récupère le formulaire
      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url,
      }).then((resp) => {
        if (this.vitisCheck.isDefAndNotNull(resp.data) &&
          this.vitisCheck.isString(resp.data.data)) {
          resolve(resp.data.data);
        } else {
          console.error('Error getting print templates');
          reject(false);
        }
      });
    });
  }

  /**
   * Lance le dessin du cercle quand on lui a spécifié une valeur dans la popup
   * @param feature
   */
  public setCircle(feature: Feature) {
    if (this.olInteraction) {
      this.olInteraction.finishDrawing();
    }
  }
}
