import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { Map } from 'ol';
import { Subscription } from 'rxjs';
import { MapToolDef } from '../../../map-tools.model';
import { VmapToolsCadastreMapService } from '../service/vmap-tools-cadastre-map.service';
import { VmapToolsCadastreService } from '../service/vmap-tools-cadastre.service';

@Component({
  selector: 'app-vmap-cadastre-tools-fiche-descriptive',
  templateUrl: './vmap-cadastre-tools-fiche-descriptive.component.html',
  styleUrls: ['./vmap-cadastre-tools-fiche-descriptive.component.scss']
})

export class VmapCadastreToolsFicheDescriptiveComponent implements OnInit, OnDestroy {

  @Input() olMap: Map;
  @Input() oTool: MapToolDef;

  // Statut du panel
  public bToolOpen: boolean;
  public privilege: string;

  // Subscription fiche descriptive
  private subscription: Subscription;

  // Modale
  public isOpen: boolean = false;
  public modalSize: number = null;
  public oLabel: string = null;

  // DATA
  public aDescrParcelle: any;

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsCadastreMapService: VmapToolsCadastreMapService,
    private vmapToolsCadastreService: VmapToolsCadastreService) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    // Ouverture de l'outil
    this.oTool.onOpen.subscribe(() => {
      this.bToolOpen = true;
      this.privilege = this.vmapToolsCadastreService.getPrivilege();
      if (this.privilege === 'vmap_cadastre_user' || this.privilege === 'vmap_cadastre_medium_user' || this.privilege === 'vmap_cadastre_light_user') {
        this.locateCadastreOnMap();
      }
    });

    // Fermeture de l'outil
    this.oTool.onClose.subscribe(() => {
      // désinscription de la fiche descriptive si subscription
      if (this.subscription) {
        this.subscription.unsubscribe();
      }
      this.bToolOpen = false;
      this.closeModal(false);
      this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    if (this.subscription) {
      this.subscription.unsubscribe();
    }
    this.bToolOpen = false;
    this.closeModal(false);
    this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
  }

  /**
  * Sélectionne une parcelle sur la carte
  * Ouvre la fiche descriptive
  * @returns Promise
  */
  private async locateCadastreOnMap(): Promise<void> {
    this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
    this.aDescrParcelle = [];
    setTimeout(() => {
      try {
        this.vmapToolsCadastreMapService.selectMapParcelle(this.olMap, 'point', 'ficheDescriptive').then(descParcelle => {
          // Changement de parcelle
          this.subscription = this.vmapToolsCadastreMapService.getFicheDescrParcelle().subscribe(async (next: any) => {
            this.aDescrParcelle = next;
            this.oLabel = 'Fiche descriptive de la parcelle';
            this.isOpen = true;
            this.modalSize = 1;
          });

          this.aDescrParcelle = descParcelle;
          this.oLabel = 'Fiche descriptive de la parcelle';
          this.isOpen = true;
          this.modalSize = 1;
        });
      } catch (error) {
        console.error(error);
      }
    });
  }

  /**
  * Permet de fermer les modales
  * @param $event boolean renvoi false quand la modale doit être fermée
  */
  public closeModal($event: boolean): void {
    this.isOpen = $event;
    // Vide toutes les variables
    this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
    this.modalSize = null;
    this.oLabel = null;
  }
}
