import { Component, Input, OnInit, EventEmitter, OnDestroy } from '@angular/core';
import { Map } from 'ol';
import { MapToolDef } from '../../../map-tools.model';
import { VmapToolsCadastreMapService } from '../service/vmap-tools-cadastre-map.service';
import { VmapToolsCadastreService } from '../service/vmap-tools-cadastre.service';


@Component({
  selector: 'app-vmap-cadastre-tools-fiche-urbanisme',
  templateUrl: './vmap-cadastre-tools-fiche-urbanisme.component.html',
  styleUrls: ['./vmap-cadastre-tools-fiche-urbanisme.component.scss']
})
export class VmapCadastreToolsFicheUrbanismeComponent implements OnInit, OnDestroy {

  @Input() olMap: Map;
  @Input() oTool: MapToolDef;

  public bToolOpen: boolean = false;
  public privilege: string;

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsCadastreMapService: VmapToolsCadastreMapService,
    private vmapToolsCadastreService: VmapToolsCadastreService) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    // Ouverture de l'outil
    this.oTool.onOpen.subscribe(() => {
      this.bToolOpen = true;
      this.privilege = this.vmapToolsCadastreService.getPrivilege();
      if (this.privilege === 'vmap_cadastre_user' || this.privilege === 'vmap_cadastre_medium_user') {
        this.locateCadastreOnMap();
      }
    });

    // Fermeture de l'outil
    this.oTool.onClose.subscribe(() => {
      this.bToolOpen = false;
      this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    this.bToolOpen = false;
    this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
  }

  /**
  * Sélectionne une parcelle sur la carte
  * Lance le telechargement de la fiche d'urbanisme
  * @returns Promise
  */
  private async locateCadastreOnMap(): Promise<void> {
    try {
      await this.vmapToolsCadastreMapService.getGeom(this.olMap, 'point', 'urbanisme');
    } catch (error) {
      console.error(error);
    }
  }
}
