import { Component, Input, OnInit, EventEmitter, OnDestroy } from '@angular/core';
import { Map } from 'ol';
import { MapToolDef } from '../../map-tools.model';
import { VmapToolsCadastreMapService } from './service/vmap-tools-cadastre-map.service';
import { VmapToolsCadastreService } from './service/vmap-tools-cadastre.service';
import { vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { Subscription } from 'rxjs';

@Component({
  selector: 'app-vmap-tools-cadastre',
  templateUrl: './vmap-tools-cadastre.component.html',
  styleUrls: ['./vmap-tools-cadastre.component.scss']
})

export class VmapToolsCadastreComponent implements OnInit, OnDestroy {

  @Input() olMap: Map;
  @Input() oTool: MapToolDef;

  private oSelectedAttributeEmiter: EventEmitter<string> = new EventEmitter();
  public oAttribute: string;

  // Elements sélectionnés dans tools-grid
  public aCurrentCompte: Array<string>;
  public aCurrentParcelle: Array<string>;
  public oElement: any;

  // Index de l'onglet
  public selectedTab = 0;
  // Permet de reload la recherche attributaire
  public reloadObject: boolean;

  // Statut du panel
  public bToolOpen: boolean;

  // Modal
  public isOpen: boolean = false;
  public modalSize: number;
  public oLabel: string;

  // Subscription fiche descriptive
  private subscriptionFicheDescr: Subscription;
  // Subscription au relevé de propriété
  private subscriptionReleveDePropriete: Subscription;
  // Subscription de l'élement courant
  private subscriptionCurrentElement: Subscription;
  // Subscription de l'attribut
  private subscriptionAttribute: Subscription;

  // Boutons
  public bSelectingFicheUrba: boolean = false;
  public bSelectingReleveDePropriete: boolean = false;
  public bSelectingFicheDescr: boolean = false;
  public bSelectingObject: string = null;
  public bSelectingType: string = null;

  public privilege: string;

  // DATA
  public aDescrParcelle: any;
  public aParcelle: any;


  /**
   * Constructeur
   */
  constructor(
    private vmapToolsCadastreMapService: VmapToolsCadastreMapService,
    private vmapToolsCadastreService: VmapToolsCadastreService) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    // Ouverture de l'outil
    this.oTool.onOpen.subscribe(() => {
      this.bToolOpen = true;
      this.privilege = this.vmapToolsCadastreService.getPrivilege();

      // Unscribe dans le destroy
      this.subscriptionAttribute = this.oSelectedAttributeEmiter.subscribe((next) => {
        this.oAttribute = next;
      });
    });

    // Fermeture de l'outil
    this.oTool.onClose.subscribe(() => {
      // Désinscriptions
      this.unsubscribeReleveDePropriete();
      this.unsubscribeFicheDescr();
      this.unsubscribeCurrentElement();
      this.unsubscribeAttribute();
      this.vmapToolsCadastreMapService.removeInteractions(this.olMap);

      this.oAttribute = null;
      this.oElement = null;
      this.deselectButton();
      this.closeModal(false);
      this.bToolOpen = false;
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    // Désinscriptions
    this.unsubscribeReleveDePropriete();
    this.unsubscribeFicheDescr();
    this.unsubscribeCurrentElement();
    this.unsubscribeAttribute();
    this.vmapToolsCadastreMapService.removeInteractions(this.olMap);

    this.oAttribute = null;
    this.deselectButton();
    this.closeModal(false);
    this.bToolOpen = false;
  }

  /**
 * Permet de ne plus observer les changements de données de la fiche descriptive
 */
  private unsubscribeFicheDescr(): void {
    // désinscription de la fiche descriptive si subscription
    if (this.subscriptionFicheDescr) {
      this.subscriptionFicheDescr.unsubscribe();
    }
  }

  /**
* Permet de ne plus observer les changements d'attribut
*/
  private unsubscribeAttribute(): void {
    // désinscription de la fiche descriptive si subscription
    if (this.subscriptionAttribute) {
      this.subscriptionAttribute.unsubscribe();
    }
  }

  /**
  * Permet de ne plus observer les changements de données du relevé de propriété
  */
  private unsubscribeReleveDePropriete(): void {
    // désinscription du relevé de propriété si subscription
    if (this.subscriptionReleveDePropriete) {
      this.subscriptionReleveDePropriete.unsubscribe();
    }
  }

  /**
* Permet de ne plus observer les changements de données géographique
*/
  private unsubscribeCurrentElement(): void {
    // désinscription de la fiche descriptive si subscription
    if (this.subscriptionCurrentElement) {
      this.subscriptionCurrentElement.unsubscribe();
    }
  }

  /**
   * Sélection d'un onglet
   * @param index number numéro de l'onglet
   */
  public selectTab(index: number): void {
    this.selectedTab = index;
    // Désinscriptions
    this.unsubscribeReleveDePropriete();
    this.unsubscribeFicheDescr();

    // Reload des données si changement d'index
    if (index == 0) {
      this.bSelectingObject = null;
      this.bSelectingType = null;
      this.oElement = null;
      this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
      this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
    } else {
      this.oAttribute = null;
      this.reloadObject = true;
      this.deselectButton();
      this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
      this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
    }
  }

  /**
  * Désélectionne les rapports en un clic
  */
  public deselectButton(): void {
    this.bSelectingReleveDePropriete = false;
    this.bSelectingFicheDescr = false;
    this.bSelectingFicheUrba = false;
    this.bSelectingObject = null;
    this.bSelectingType = null;
    this.modalSize = null;
    this.oLabel = null;
    this.isOpen = false;
  }

  /**
  * Sélection de la fiche d'urbanisme (rapport en un clic)
  */
  public selectedFicheUrba(): void {
    // Désinscriptions
    this.unsubscribeReleveDePropriete();
    this.unsubscribeFicheDescr();

    this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
    this.bSelectingFicheUrba = !this.bSelectingFicheUrba;
    // Reload le formulaire
    this.oAttribute = null;
    this.reloadObject = true;
    this.aParcelle = [];
    this.aDescrParcelle = [];

    this.bSelectingReleveDePropriete = false;
    this.bSelectingFicheDescr = false;
    if (this.bSelectingFicheUrba) {
      try {
        this.vmapToolsCadastreMapService.getGeom(this.olMap, 'point', 'urbanisme');
      } catch (error) {
        console.error(error);
      }
    } else {
      this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
    }
  }

  /**
  * Sélection du relevé de propriété (rapport en un clic)
  */
  public selectedReleveDePropriete(): void {
    // Désinscriptions
    this.unsubscribeReleveDePropriete();
    this.unsubscribeFicheDescr();

    this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
    setTimeout(() => {
      this.aParcelle = [];
      this.bSelectingReleveDePropriete = !this.bSelectingReleveDePropriete;
      this.bSelectingFicheUrba = false;
      this.bSelectingFicheDescr = false;
      this.oAttribute = null;
      this.reloadObject = true;
      if (this.bSelectingReleveDePropriete) {
        this.vmapToolsCadastreMapService.selectMapParcelle(this.olMap, 'point', 'releveDePropriete').then(descParcelle => {
          // Changement de parcelle
          this.subscriptionReleveDePropriete = this.vmapToolsCadastreMapService.getRPParcelle().subscribe(async (next: any) => {
            this.aParcelle = next;
            this.oLabel = 'Génération du relevé de propriété';
            this.isOpen = true;
            this.modalSize = 2;
          });
          this.aParcelle = descParcelle;
          this.oLabel = 'Génération du relevé de propriété';
          this.isOpen = true;
          this.modalSize = 2;
        });
      } else {
        this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
      }
    });

  }

  /**
 * Sélection de la fiche descriptive (rapport en un clic)
 * @returns Promise<void>
 */
  public async selectedFicheDescriptive(): Promise<void> {
    // Désinscriptions
    this.unsubscribeReleveDePropriete();
    this.unsubscribeFicheDescr();

    this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
    setTimeout(() => {
      this.aDescrParcelle = [];
      this.bSelectingFicheDescr = !this.bSelectingFicheDescr;
      this.bSelectingFicheUrba = false;
      this.bSelectingReleveDePropriete = false;
      this.oAttribute = null;
      this.reloadObject = true;
      try {
        if (this.bSelectingFicheDescr) {
          this.vmapToolsCadastreMapService.selectMapParcelle(this.olMap, 'point', 'ficheDescriptive').then(descParcelle => {
            // Changement de parcelle
            this.subscriptionFicheDescr = this.vmapToolsCadastreMapService.getFicheDescrParcelle().subscribe(async (next: any) => {
              this.aDescrParcelle = next;
              this.oLabel = 'Fiche descriptive de la parcelle';
              this.isOpen = true;
              this.modalSize = 1;
            });
            this.aDescrParcelle = descParcelle;
            this.oLabel = 'Fiche descriptive de la parcelle';
            this.isOpen = true;
            this.modalSize = 1;
          });
        } else {
          this.vmapToolsCadastreMapService.removeInteractions(this.olMap);
        }
      } catch (error) {
        console.error(error);
      }
    });
  }

  /**
  * Permet de fermer les modales (Relevé de propriété et fiche descriptive)
  * @param $event boolean renvoi false quand la modale doit être fermée
  */
  public closeModal($event: boolean): void {
    this.isOpen = $event;
    // Vide toutes les variables
    this.vmapToolsCadastreMapService.hideAllSelectedFeatureContour();
    this.modalSize = null;
    this.oLabel = null;
  }

  /**
  * Objet sur lequel porte la recherche, utilise le point comme mode de recherche par default
  * @param object string element sur lequel porte la recherche (commune | section | lieuDit | bati | parcelle)
  * @returns Promise
  */
  public async searchOnMapInit(object: string): Promise<void> {
    // Active ou desactive le bouton
    if (this.bSelectingObject == object) {
      this.bSelectingObject = null;
      this.bSelectingType = null;
    } else {
      this.bSelectingObject = object;
    }

    // Vide les élements et la carte à chaque changement d'objet
    this.oElement = null;
    this.vmapToolsCadastreMapService.removeInteractions(this.olMap);

    /**
    * Détermine le point comme géométrie par default
    * Récupération des données
    */
    if (this.bSelectingObject) {
      this.bSelectingType = 'point';
      this.getCurrentElement();
    }
  }

  /**
  * Permet de choisir le type de sélection cartographique (point | linestring | polygon)
  * @param type string type de tracé géométrique
  * @returns Promise<void>
  */
  public async locateCadastreOnMap(type: string): Promise<void> {
    // Declick le bouton
    if (this.bSelectingType == type) {
      this.bSelectingType = null;
    } else {
      this.bSelectingType = type;
    }

    // Vide la carte à chaque changement de type de sélection cartographique
    this.vmapToolsCadastreMapService.removeInteractions(this.olMap);

    // Récupération des données
    if (this.bSelectingObject && this.bSelectingType) {
      this.getCurrentElement();
    } else {
      // Si aucun objet n'a été sélectionné
      vitisAlert.fireToast({
        type: 'warning',
        title: 'Veuillez sélectionner un objet'
      });
    }
  }

  /**
 * Permet de faire une sélection cartographique
 * @returns Promise<void>
 */
  private async getCurrentElement(): Promise<void> {
    try {
      const descParcelle = await this.vmapToolsCadastreMapService.getGeom(this.olMap, this.bSelectingType, this.bSelectingObject);
      this.oElement = descParcelle;
      // Ecoute les changements de coordonnées géographiques
      this.subscriptionCurrentElement = this.vmapToolsCadastreMapService.getCurrentElement().subscribe((next: any) => {
        this.oElement = next;
      });
    } catch (error) {
      console.error(error);
    }
  }

  /**
  * Permet d'écouter le changement dans les parcelles sélectionnées (tools-grid)
  * Résultat de la recherche cartographique
  * @param $event Array<string> nouvelles parcelles
  */
  public getParcelle($event: Array<string>): void {
    this.aCurrentParcelle = $event;
  }

  /**
  * Permet d'écouter le changement dans les comptes sélectionnés (tools-grid)
  * Résultat de la recherche cartographique
  * @param $event Array<string> nouveaux comptes
  */
  public getCompte($event: Array<string>): void {
    this.aCurrentCompte = $event;
  }
}
