import { Component, createNgModule, Injector, ViewChild, ViewContainerRef } from '@angular/core';
import { SimpleformComponent } from 'src/app/vitis/simpleform/simpleform.component';
import { vitisHelpHeader } from 'src/app/vitis/app.struct';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';

declare let $: any;

@Component({
  selector: 'app-vmap-tools-helper',
  templateUrl: './vmap-tools-helper.component.html',
  styleUrls: ['./vmap-tools-helper.component.scss']
})
export class VmapToolsHelperComponent {

  public dashboardHelperModal: any = 'dashboard-helper-modal' + Date.now();
  public modalFormMode: any;
  public modalFormSize: string = 'medium';

  @ViewChild('container', { read: ViewContainerRef }) container: ViewContainerRef;

  /**
   * Constructeur
   */
  constructor(
    private injector: Injector,
    private vitisCheck: VitisCheck,
  ) { }

  /**
   * Retourne les objets de vitisHelpHeader
   */
  get objects() {
    return vitisHelpHeader.objects;
  }

  /**
   * Affiche un simple-form dans une modale
   * @param  {any} oObject
   */
  public displayModalForm(oObject: any) {

    this.modalFormMode = oObject;
    this.modalFormSize = 'medium';
    if (oObject && oObject.options && oObject.options.size) {
      this.modalFormSize = oObject.options.size;
    }

    if (oObject.name !== 'vitis_help_documentation' || (this.vitisCheck.isDefAndNotNull(environment.properties.app_documentation) &&
      environment.properties.app_documentation.replace(' ', '') !== '')) {
      // Import SimpleFormModule
      import('../../../../../../../vitis/simpleform/simpleform.module').then(({ SimpleformModule }) => {

        // Compile le module
        const moduleRef = createNgModule(SimpleformModule, this.injector);
        const { instance } = this.container.createComponent(SimpleformComponent, { ngModuleRef: moduleRef });

        // set oInputMode du formreader
        setTimeout(() => {
          instance.oInputMode = this.modalFormMode;
        }, 100);

        // Affiche la modale
        $('#' + this.dashboardHelperModal).modal('show');
      });
    } else {
      window.open(environment.appStruct.documentation[0].url);
    }
  }

  /**
   * Ferme la modale
   */
  public closeModalForm() {
    $('#' + this.dashboardHelperModal).modal('hide');
    this.handleModalForm_();
  }

  /**
   * Supprime le formulaire en cours lorsque la modale se ferme
   */
  public handleModalForm_() {
    const emptyForm = {
      'display': {
        'name': 'vitis_dashboard_modal_display_form',
        'title': '',
        'input_size': 'xs',
        'nb_cols': 8,
        'rows': []
      },
      'insert': {
        'name': 'vitis_dashboard_modal_insert_form',
        'title': '',
        'input_size': 'xs',
        'nb_cols': 8,
        'rows': []
      },
      'update': {
        'name': 'vitis_dashboard_modal_update_form',
        'title': '',
        'input_size': 'xs',
        'nb_cols': 8,
        'rows': []
      }
    };
    this.modalFormMode = emptyForm;

    // Vide le container
    for (let i = 0; i < this.container.length; i++) {
      this.container.remove(i);
    }
  }
}
