import { Component, Input, Output, OnChanges, OnInit, EventEmitter, ViewChild, OnDestroy } from '@angular/core';
import { environment } from 'src/environments/environment';
import { VitisCheck, VitisClone, vitisAlert, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { MapSelection, VmapSelectService } from 'src/modules/vmap/components/carto/services/vmap-select/vmap-select.service';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { VmapToolsFormComponent } from 'src/app/modules/vmap/carto/components/vmap-tools/components/vmap-tools-select-right-panel/vmap-tools-form/vmap-tools-form.component';

import { Geometry } from 'ol/geom';
import { Feature } from 'ol';
import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { Subscription } from 'rxjs';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { VmapToolsReportService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-report/vmap-tools-report.service';
import { VmapGtfWorkspaceModalComponent } from '../vmap-gtf-workspace-modal/vmap-gtf-workspace-modal.component';
import { VmapFormService } from 'src/modules/vmap/components/carto/services/vmap-form/vmap-form.service';

@Component({
  selector: 'app-vmap-tools-modify-right-panel',
  templateUrl: './vmap-tools-modify-right-panel.component.html',
  styleUrls: ['./vmap-tools-modify-right-panel.component.scss']
})
export class VmapToolsModifyRightPanelComponent implements OnInit, OnChanges, OnDestroy {

  @ViewChild(VmapToolsFormComponent) oFormComponent: VmapToolsFormComponent;

  @Input() map;
  @Input() selections: Array<MapSelection>;
  @Input() indexSelection: number;
  @Input() isGeomExist: boolean;

  @Input() isSmallScreen: boolean;

  @Output() subscribeFormName = new EventEmitter();
  @Output() updatePopup = new EventEmitter();
  @Output() subscribeFormSize = new EventEmitter();


  public oLayer;

  private oParamsId;
  private oParamsLayerId;

  public bOpenActions = false;
  public bOpenReports = false;
  public bOpenWorkspaces = false;

  public sFormName = 'display';
  public sDefinition;
  public sDatasForm;

  private feature: Feature<Geometry>;
  private DisectionFeature: Array<Feature<Geometry>>;
  public geomEventValue: string = '';
  private oFormReader: FormreaderComponent;

  // Affichage des boutons d'actions.
  public oActions: any = {
    google_street_view: true
  };

  // Affichage des rapports
  public aReports: Array<any> = [];

  // GTF workspaces
  public aGTFWorkspaces: Array<any> = [];

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  /**
   * Constructeur
   */
  constructor(
    private vitisClone: VitisClone,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private mapSelectService: VmapSelectService,
    private mapUtils: MapCoreUtilsService,
    private mapToolsService: VmapToolsService,
    private translate: TranslateService,
    private route: ActivatedRoute,
    private router: Router,
    private formreaderSrvc: FormreaderService,
    private reportService: VmapToolsReportService,
    private mapFormService: VmapFormService,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    this.subscriptions.push(this.route.data.subscribe(mode => {
      this.subscriptions.push(this.route.queryParams.subscribe(oParams => {
        this.oParamsId = oParams.id;
        this.oParamsLayerId = oParams.layer_id;
      }));
    }));


    if (this.selections && this.vitisCheck.isNumber(this.indexSelection) && this.selections[this.indexSelection]) {
      const aSelection = this.selections[this.indexSelection];
      this.oLayer = aSelection.oLayer;
      this.initFeature(aSelection);
      const def = this.vitisCheck.isString(aSelection.oLayer.published_form_json_def) ?
        JSON.parse(aSelection.oLayer.published_form_json_def) : aSelection.oLayer.published_form_json_def;

      this.sDefinition = this.vitisClone.clone(def);
      this.sDatasForm = aSelection.data;
    }
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Appelé lorsqu'il y a des changements sur les input
   */
  ngOnChanges(changes): void {
    let aSelections = this.selections;
    let indexSelection = this.indexSelection;
    if (changes) {
      if (changes.selections && changes.selections.currentValue) {
        aSelections = changes.selections.currentValue;
      }
      if (changes.indexSelection && changes.indexSelection.currentValue) {
        indexSelection = changes.indexSelection.currentValue;
      }
      const aSelection = aSelections[indexSelection];
      if (aSelection) {
        this.oLayer = aSelection.oLayer;
        this.initFeature(aSelection);
        const def = this.vitisCheck.isString(aSelection.oLayer.published_form_json_def) ?
          JSON.parse(aSelection.oLayer.published_form_json_def) : aSelection.oLayer.published_form_json_def;

        this.sDefinition = this.vitisClone.clone(def);
        this.sDatasForm = aSelection.data;

        // Affichage du bouton "Voir sur Google Street View".
        if (this.vitisCheck.isDefAndNotNull(this.oLayer)) {
          if (this.vitisCheck.isDefAndNotNull(this.oLayer.google_street_view)) {
            this.oActions.google_street_view = this.oLayer.google_street_view;
          }
        }

        // Récupération des rapports propres à la couche sélectionnée
        if (this.vitisCheck.isDefAndNotNull(this.oLayer)) {
          this.getUserPrintReports_().then((reports) => {
            this.aReports = reports;
          }).catch((error) => {
            console.error(error);
          });
        }

        // Récupération des traitements FME / GTF
        if (environment.properties.enable_gtf_api == true) {
          this.getFMEWorkspaces();
        }
      }
    }
  }

  /**
   * Initialise la feature pour sa modification
   * @param aSelection
   */
  private initFeature(aSelection): void {
    const EWKT = aSelection.data[aSelection.oLayer.geomfield];
    const geometry = this.vitisCheck.isDefAndNotNull(EWKT) ? this.mapUtils.getGeomFromEWKT(EWKT, this.map) : null;
    this.feature = new Feature({ geometry });
    this.DisectionFeature = null; // Réinitialise la feature créée par la disection pour éviter des duplications
  }

  /**
   * Permet de récupérer l'update de la geométrie de la feature
   * @param feature
   */
  public subscribeFeature(feature) {
    this.feature = feature;
  }

  /**
   * Permet de récupérer l'update de la nouvelle géométrie de la feature créée par la disection
   * @param feature
   */
  public subscribeDisectionFeature(feature) {
    this.DisectionFeature = feature;
  }

  /**
   * Ouvre le menu des actions
   */
  public openActions() {
    this.bOpenActions = !this.bOpenActions;
    if (!this.bOpenActions) {
      this.bOpenReports = false;
      this.bOpenWorkspaces = false;
    }
  }

  /**
   * Ouvre le menu des rapports
   */
  public openReports() {
    this.bOpenReports = !this.bOpenReports;
  }

  /**
   * Ouvre le menu des workspaces GTF
   */
  public openWorkspaces() {
    this.bOpenWorkspaces = !this.bOpenWorkspaces;
  }

  /**
   * Ouvre la fenêtre google street view
   */
  public googleStreetView(): void {
    const clickedMapCoordinates = this.mapSelectService.getClickedCoordinate();
    const EWKT = this.selections[this.indexSelection].data[this.selections[this.indexSelection].oLayer.geomfield];
    const url = this.mapUtils.getOpenStreetViewUrlFromEWKT(EWKT, this.map, clickedMapCoordinates);
    window.open(url);
  }

  /**
   * Retourne si geom est de type multi
   * @returns
   */
  public isGeometryTypeMulti(): boolean {
    if (this.feature) {
      const geom = this.feature.getGeometry();
      if (geom) {
        const geometryType = geom.getType();
        return geometryType === 'MultiPoint' || geometryType === 'MultiLineString' || geometryType === 'MultiPolygon';
      }
    }
    return false;
  }

  /**
  * Copie du lien vers cet objet
  */
  public copyPressObjectLink(): void {
    let link = this.router.url;

    if (!this.oParamsLayerId) {
      link += `&layer_id=${this.selections[this.indexSelection].oLayer.layer_id}`;
    }

    if (!this.oParamsId) {
      link += `&id=${this.selections[this.indexSelection].data[this.selections[this.indexSelection].oLayer.idfield]}`;
    }

    const txt = `${environment.properties.app_client_url}${link}`;

    if (navigator.clipboard) {
      navigator.clipboard.writeText(txt).then(() => {
        this.translate.get('URL_COPY_PRESS_OK').subscribe((sTranslate: string) => {
          vitisAlert.fireToast({
            type: 'success',
            title: `${sTranslate} :\n${environment.properties.app_client_url}${link}`
          });
        });
      });
    }
  }

  /**
   * Ouvre le formulaire d'édition
   */
  public editForm(): void {
    this.sFormName = 'update';
    this.subscribeFormName.next(this.sFormName);
    this.subscribeFormSize.next(this.selections[this.indexSelection].oLayer.form_update_size_id ?? 1);
    // Permet de déclencher la recharge du formulaire par FormReader
    this.sDefinition = this.vitisClone.clone(
      this.vitisCheck.isString(this.selections[this.indexSelection].oLayer.published_form_json_def) ?
        JSON.parse(this.selections[this.indexSelection].oLayer.published_form_json_def) : this.selections[this.indexSelection].oLayer.published_form_json_def);
    this.sDatasForm = this.vitisClone.clone(this.sDatasForm);
  }

  /**
 * Event appelé si on redéfinit la géométrie et aux actions présent dans la redéfinition de la géométrie
 */
  public updateGeometry(): void {
    this.sFormName = 'geometry';
    this.subscribeFormName.next(this.sFormName);
    this.sDatasForm = this.selections[this.indexSelection].data;
  }

  /**
   * Sauvegarde la nouvelle géométrie de l'object
   */
  public saveGeometryModifications(): void {
    const oLayer = this.selections[this.indexSelection].oLayer;
    const sEWKTGeom = this.mapUtils.getEWKTFromFeatures([this.feature], this.map);


    if (this.vitisCheck.isDefAndNotNull(this.DisectionFeature)) {
      const sMultiInsertForm = {
        'insert': this.DisectionFeature.map((entry: any) => {
          if (entry != sEWKTGeom) {
            return {
              ...this.sDatasForm,
              [oLayer.geomfield]: this.mapUtils.getEWKTFromFeatures([entry], this.map)
            };
          };
        }), 'update': {
          [oLayer.geomfield]: sEWKTGeom,
          [oLayer.idfield]: this.sDatasForm[oLayer.idfield],
        }
      };
      this.sendMultipleInsertRequest(sMultiInsertForm);
    } else {
      this.sendUpdateForm({
        [oLayer.geomfield]: sEWKTGeom,
        [oLayer.idfield]: this.sDatasForm[oLayer.idfield],
      }, true);
    }
  }

  /**
   * Annule la redéfinition de la géométrie et renvoie sur le formulaire de description
   */
  public cancelGeometryModifications(): void {
    const aSelection = this.selections[this.indexSelection];
    this.initFeature(aSelection);
    this.sendCloseForm();
  }

  /**
   * Envoi de la requête pour le formulaire d'édition
   * @param event
   */
  public sendUpdateForm(event: any, updateGeometry?: any): void {
    const data = this.vitisCheck.isDefAndNotNull(event) ? event : this.oFormComponent.getFormValues();
    const sendFunction = (resData: any): Promise<void> => {
      return this.formreaderSrvc.manageLargesFiles(resData).then((managedData: any) => {
        return new Promise<void>((resolve, reject) => {
          this.vitisRequest.ajaxRequestPromise({
            method: 'PUT',
            url: `${environment.properties.app_api_url}/vmap/layers/${this.oLayer.layer_id}/query/${this.sDatasForm[this.oLayer.idfield]}`,
            data: this.formreaderSrvc.transformToDataForm(managedData),
            displayErrorMessage: false
          }).then((resp: any) => {
            if (resp && resp.data && resp.data.data) {
              if (resp.data.refreshAllLayers) {
                this.mapToolsService.refreshMap(this.map);
              } else {
                this.refreshLayer(this.oLayer.layer_id);
              }
              if (resp.data.closeFormPanel) {
                this.mapToolsService.setRightPanel(0, '');
              } else {
                this.sendCloseForm(resp.data.data);
                this.updatePopup.next(resp.data.data);
              }
              // Add Toast
              vitisAlert.fireToast({
                icon: 'success',
                title: this.translate.instant('COMPONENT_MAP_ENABLE_UTILS_SUCCESS_TOAST')
              });
              resolve();
            }
          }).catch(e => {
            console.error(e);
            this.translate.get(['ERROR_UPDATE_LAYER_OBJECT', 'ERROR_UPDATE_LAYER_OBJECT_DETAIL']).subscribe(async (translation) => {
              vitisAlert.alert({
                type: 'error',
                title: translation['ERROR_UPDATE_LAYER_OBJECT'],
                text: translation['ERROR_UPDATE_LAYER_OBJECT_DETAIL'],
              });
            });
            reject();
          });
        });
      });
    };

    if (updateGeometry) {
      sendFunction(data);
    } else {
      if (this.oFormComponent.oFormReader.isFormValid()) {
        this.oFormComponent.oFormReader['updateObjectLayerForm'] = sendFunction;
        this.sDefinition[this.sFormName].event = 'updateObjectLayerForm()';
        this.oFormComponent.oFormReader.submitForm(this.sDefinition, this.sFormName, data);
      }
    }
  }

  /**
   * Insert plusieurs nouvelles géométrie en base
   */
  private sendMultipleInsertRequest(evtData: Object): any {
    let data = null;
    const oLayer = this.selections[this.indexSelection].oLayer;

    if (this.vitisCheck.isDefAndNotNull(evtData)) {
      data = evtData;
    } else {
      return;
    }
    const sendFunction = (resData: any): Promise<void> => {
      return this.formreaderSrvc.manageLargesFiles(resData).then((managedData: any) => {
        return new Promise<void>((resolve, reject) => {
          this.vitisRequest.ajaxRequestPromise({
            method: 'POST',
            url: `${environment.properties.app_api_url}/vmap/layers/${this.oLayer.layer_id}/queries`,
            data: {
              insert: managedData['insert'].map((dataEntry: any) => {
                Object.keys(dataEntry).forEach((k) => dataEntry[k] == null && delete dataEntry[k]);
                if (this.vitisCheck.isDefAndNotNull(dataEntry[oLayer.idfield])) {
                  delete dataEntry[oLayer.idfield];
                }
                return dataEntry;
              }), update: managedData['update']
            },
            displayErrorMessage: true
          }).then((resp: any) => {
            if (resp && resp.data && resp.data.data && resp.data.data.update) {
              this.refreshLayer(this.oLayer.layer_id);
              this.sendCloseForm(resp.data.data.update);
              this.updatePopup.next(resp.data.data.update);
              resolve();
            }
          }).catch((e) => {
            this.translate.get(['ERROR_INSERT_LAYER_OBJECT', 'ERROR_INSERT_LAYER_OBJECT_DETAIL']).subscribe(async (translation) => {
              vitisAlert.alert({
                type: 'error',
                title: translation['ERROR_INSERT_LAYER_OBJECT'],
                text: translation['ERROR_INSERT_LAYER_OBJECT_DETAIL'],
              });
              reject('Error inserting');
            });
          });
        });
      });
    };

    sendFunction(data);
  }

  /**
   * Ferme le formulaire actuel
   */
  public sendCloseForm(oNewData?: any): void {
    this.sFormName = 'display';
    this.subscribeFormSize.next(this.selections[this.indexSelection].oLayer.form_display_size_id ?? 1);
    this.subscribeFormName.next(this.sFormName);
    this.sDefinition = this.vitisClone.clone(this.vitisCheck.isString(this.selections[this.indexSelection].oLayer.published_form_json_def) ?
      JSON.parse(this.selections[this.indexSelection].oLayer.published_form_json_def) : this.selections[this.indexSelection].oLayer.published_form_json_def);
    if (oNewData) {
      this.sDatasForm = this.vitisClone.clone(oNewData);
    } else {
      this.sDatasForm = this.vitisClone.clone(this.sDatasForm);
    }
    this.bOpenActions = false;
    if (this.vitisCheck.isDefAndNotNull(this.map.get('popup_layer')) && !this.map.get('popup_layer').getVisible()) {
      this.map.get('popup_layer').setVisible(true);
    }
  }

  /**
   * Rafraîchit la couche
   * @param iLayerId
   */
  private refreshLayer(iLayerId: number): void {
    this.mapToolsService.refreshLayer(this.map, iLayerId, true);
  }

  /**
   * Envoi la requête de suppression d'un object
   */
  public deleteLayerObject(): void {
    this.displayDeleteWarning().then((bResponse: boolean) => {
      if (bResponse === true) {
        const oLayer = this.selections[this.indexSelection].oLayer;
        this.vitisRequest.ajaxRequestPromise({
          method: 'DELETE',
          url: `${environment.properties.app_api_url}/vmap/layers/${oLayer.layer_id}/query/${this.sDatasForm[oLayer.idfield]}`,
        }).then((resp: any) => {
          if (resp && resp.data && resp.data.data) {
            this.refreshLayer(oLayer.layer_id);
            this.mapToolsService.closeAllPanels();
            this.updatePopup.next(null);
          }
        }).catch(e => {
          console.error(e);
          const sErrorMessage = (this.vitisCheck.isDefAndNotNull(e.data) && this.vitisCheck.isDefAndNotNull(e.data.errorMessage)) ? e.data.errorMessage : 'ERROR_DELETE_LAYER_OBJECT';
          this.translate.get(sErrorMessage).subscribe(async (translation) => {
            vitisAlert.alert({
              text: translation,
              showCancelButton: true
            });
          });
        });
      }
    });
  }

  /**
   * Affiche le message de warning à la suppression
   * @returns
   */
  private displayDeleteWarning(): Promise<boolean> {
    return new Promise((resolve, reject) => {
      this.translate.get(['FORM_DELETE_WARNING',
        'CONFIRM_CANCEL',
        'CONFIRM_CONFIRM']
      ).subscribe((translations) => {
        vitisAlert.alert({
          title: translations['FORM_DELETE_WARNING'],
          type: 'warning',
          showCancelButton: true,
          confirmButtonText: translations['CONFIRM_CONFIRM'],
          cancelButtonText: translations['CONFIRM_CANCEL'],
        }).then((result: any) => {
          if (result &&
            result.isConfirmed === true) {
            resolve(true);
          } else {
            resolve(false);
          }
        });
      });
    });
  }

  /**
   * Sélectionne le mode insert avec la couche qui va avec
   */
  public selectInsertMode() {
    this.mapSelectService.closeSelections();
    this.mapSelectService.setSelectMode('add');
    this.mapSelectService.setSelection(this.oLayer);
  }

  /**
   * Recharge le javascript lorsque le formreader est modifié
   * @param oFormReader
   */
  public updateFormReader(oFormReader: FormreaderComponent) {
    const aSelection = this.selections[this.indexSelection];

    // Charge le JS du formulaire
    if (this.vitisCheck.isDefAndNotNull(aSelection.oLayer.published_form_js_def)) {
      if (!this.vitisCheck.isSameObject(this.oFormReader, oFormReader)) {
        const oScope = {
          map: this.map,
          ...aSelection,
        };
        oFormReader.scope = oScope;
        this.mapFormService.loadJs(aSelection.oLayer.published_form_js_def, aSelection.oLayer.layer_id, oFormReader, oScope);
        this.oFormReader = this.vitisClone.clone(oFormReader);
      }
    }

    // Contenu CSS
    if (this.vitisCheck.isDefAndNotNull(aSelection.oLayer.published_form_css_def)) {
      setTimeout(() => { // Attend que l'HTML soit chargé
        this.mapFormService.loadCss(aSelection.oLayer.published_form_css_def);
      });
    }
  }

  /**
   * Récupération des rapports d'impression par rapport à la couche en cours
   * @returns
   */
  private getUserPrintReports_(): Promise<Array<any>> {
    return new Promise((resolve, reject) => {
      const url = `${environment.properties.app_api_url}/vmap/printreport`;

      // Récupère le formulaire
      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url,
        params: {
          order_by: 'printreport_id',
          filter: {
            relation: 'AND',
            operators: [{
              column: 'layer_id',
              compare_operator: '=',
              value: this.oLayer.layer_id
            }]
          }
        }

      }).then((resp) => {
        if (this.vitisCheck.isDefAndNotNull(resp.data) && resp.data.status === 1 && this.vitisCheck.isArray(resp.data.data)) {
          resolve(resp.data.data);
        } else {
          console.error('Error getting print reports');
          reject(false);
        }
      });
    });
  }

  /**
   * Lance l'impression d'un rapport donné
   * @param oPrintReport
   */
  public submitPrintReport(oPrintReport: any): void {
    if (this.vitisCheck.isDefAndNotNull(oPrintReport)) {
      const aSelections = this.selections.map(element => element.data);
      this.reportService.printReport(this.map, oPrintReport, aSelections).then(() => {
        this.bOpenActions = false;
      });
    }
  }

  /**
   * Récupère les traitements GTF associée à la couche et à l'utilisateur
   */
  public getFMEWorkspaces(): void {
    this.mapToolsService.getLayerFMEWorkspaces(this.oLayer.layer_id).then((aWorkspaces: Array<any>) => {
      this.aGTFWorkspaces = aWorkspaces.filter((val) => {
        return val.allow_display_card === true || val.allow_display_card === 1;
      });
    }).catch(() => {
      this.aGTFWorkspaces = [];
      vitisAlert.alert({
        type: 'error',
        text: this.translate.instant('ERROR_CALLING_GTF_API'),
      });
    });
  }

  /**
   * Affichage de la fenêtre modale de lancement de traitements GTF.
   */
  public displayWorkbenchModal(oWorkspace: any): void {
    const oSelection = this.selections[this.indexSelection];
    this.mapToolsService.displayGTFWorkspaceModal(VmapGtfWorkspaceModalComponent, oWorkspace, this.oLayer, oSelection.data);
  }
}
