import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { MapSelection } from 'src/modules/vmap/components/carto/services/vmap-select/vmap-select.service';
import { VmapToolsFormComponent } from 'src/app/modules/vmap/carto/components/vmap-tools/components/vmap-tools-select-right-panel/vmap-tools-form/vmap-tools-form.component';
import { vitisAlert, VitisCheck, VitisClone, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { TranslateService } from '@ngx-translate/core';
import { Layer } from 'ol/layer';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { VmapFormService } from 'src/modules/vmap/components/carto/services/vmap-form/vmap-form.service';


@Component({
  selector: 'app-vmap-tools-multiedit-right-panel',
  templateUrl: './vmap-tools-multiedit-right-panel.component.html',
  styleUrls: ['./vmap-tools-multiedit-right-panel.component.scss']
})
export class VmapToolsMultieditRightPanelComponent implements OnInit {
  @ViewChild(VmapToolsFormComponent) oFormComponent: VmapToolsFormComponent;

  @Input() map;
  @Input() selections: Array<MapSelection>;

  @Input() isSmallScreen: boolean;

  public oLayer: any;
  public sDefinition: string;
  public sFormName = 'multi_update';
  private oFormReader: FormreaderComponent;

  /**
   * Constructeur
   */
  constructor(
    private vitisClone: VitisClone,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private translate: TranslateService,
    private mapToolsService: VmapToolsService,
    private formreaderSrvc: FormreaderService,
    private mapFormService: VmapFormService,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    if (this.selections) {
      // On selectionne le premier élément du tableau car la couche est la même pour tous
      const aSelection = this.selections[0];
      this.oLayer = aSelection.oLayer;
      const def = this.vitisCheck.isString(aSelection.oLayer.published_form_json_def) ?
        JSON.parse(aSelection.oLayer.published_form_json_def) : aSelection.oLayer.published_form_json_def;

      this.sDefinition = this.vitisClone.clone(def);
    }
  }

  /**
   * Envoi de la requête pour le formulaire d'édition
   * @param event
   */
  public sendUpdateForm(event?: any): void {
    const data = this.vitisCheck.isDefAndNotNull(event) ? event : this.oFormComponent.getFormValues();
    if (this.oFormComponent.getIsFormValid()) {

      const aIds = [];
      for (const selection in this.selections) {
        if (Object.prototype.hasOwnProperty.call(this.selections, selection)) {
          aIds.push(this.selections[selection].data[this.selections[selection].oLayer.idfield]);
        }
      }
      data['ids'] = aIds;
      const sendFunction = (resData: any): Promise<void> => {
        return this.formreaderSrvc.manageLargesFiles(this.handleUpdateValues(resData)).then((managedData: any) => {
          return new Promise<void>((resolve, reject) => {
            this.vitisRequest.ajaxRequestPromise({
              method: 'PUT',
              url: `${environment.properties.app_api_url}/vmap/layers/${this.oLayer.layer_id}/query`,
              data: this.formreaderSrvc.transformToDataForm(managedData),
            }).then((resp: any) => {
              if (resp && resp.data && resp.data.data) {
                this.refreshLayer(this.oLayer.layer_id);
                this.sendCloseForm();
                // Add Toast
                vitisAlert.fireToast({
                  icon: 'success',
                  title: this.translate.instant('COMPONENT_MAP_ENABLE_UTILS_SUCCESS_TOAST')
                });
                resolve();
              }
            }).catch(e => {
              console.error(e);
              this.translate.get(['ERROR_UPDATE_LAYER_OBJECT', 'ERROR_UPDATE_LAYER_OBJECT_DETAIL']).subscribe(async (translation) => {
                vitisAlert.alert({
                  type: 'error',
                  title: translation['ERROR_UPDATE_LAYER_OBJECT'],
                  text: translation['ERROR_UPDATE_LAYER_OBJECT_DETAIL'],
                });
              });
              reject();
            });
          });
        });
      };

      if (this.oFormComponent.oFormReader.isFormValid()) {
        this.oFormComponent.oFormReader['updateObjectLayerForm'] = sendFunction;
        this.sDefinition[this.sFormName].event = 'updateObjectLayerForm()';
        this.oFormComponent.oFormReader.submitForm(this.sDefinition, this.sFormName, data);
      }
    } else {
      vitisAlert.fireToast({
        icon: 'error',
        title: this.translate.instant('COMPONENT_MAP_SYMBOLOGY_ERROR_INVALID_FORM')
      });
    }
  }

  /**
   * Rafraîchit la couche
   * @param iLayerId
   */
  private refreshLayer(iLayerId: number): void {
    this.mapToolsService.refreshLayer(this.map, iLayerId, true);
  }

  /**
   * Ferme le paneau de droite
   */
  public sendCloseForm() {
    this.mapToolsService.setRightPanel(0, '');
  }

  /**
   * Enlève les valeurs vides
   * @param oValues
   * @returns
   */
  private handleUpdateValues(oValues: any): any {
    const oReturn = {};
    for (const key in oValues) {
      if (Object.prototype.hasOwnProperty.call(oValues, key)) {
        const element = oValues[key];
        if (!(this.vitisCheck.isString(element) && element.length === 0)) {
          oReturn[key] = element;
        }
      }
    }
    return oReturn;
  }

  /**
   * Recharge le javascript lorsque le formreader est modifié
   * @param oFormReader
   */
  public updateFormReader(oFormReader: FormreaderComponent) {
    // On selectionne le premier élément du tableau car la couche est la même pour tous
    const aSelection = this.selections[0];

    // Charge le JS du formulaire
    if (this.vitisCheck.isDefAndNotNull(aSelection.oLayer.published_form_js_def)) {
      if (!this.vitisCheck.isSameObject(this.oFormReader, oFormReader)) {
        const oScope = {
          map: this.map,
          ...aSelection,
        };
        oFormReader.scope = oScope;
        this.mapFormService.loadJs(aSelection.oLayer.published_form_js_def, aSelection.oLayer.layer_id, oFormReader, oScope);
        this.oFormReader = this.vitisClone.clone(oFormReader);
      }
    }

    // Contenu CSS
    if (this.vitisCheck.isDefAndNotNull(aSelection.oLayer.published_form_css_def)) {
      setTimeout(() => { // Attend que l'HTML soit chargé
        this.mapFormService.loadCss(aSelection.oLayer.published_form_css_def);
      });
    }
  }
}
