import { Component, OnInit, Input, EventEmitter, ViewChild, Optional, Inject, ElementRef, OnDestroy, AfterViewInit, ViewContainerRef } from '@angular/core';
import { Feature, Map } from 'ol';
import { MapToolDef } from '../../map-tools.model';
import { DataGridComponent } from 'src/app/vitis/data-grid/data-grid.component';
import { DataGridOptions } from 'src/app/vitis/data-grid/data-grid.model';
import { WorkspaceListOptions } from 'src/app/vitis/workspacelist/workspacelist.model';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { vitisAlert, VitisCheck, VitisRequest, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { VmapSelectService } from 'src/modules/vmap/components/carto/services/vmap-select/vmap-select.service';
import { DashboardComponent } from 'src/app/vitis/core/components/dashboard/dashboard.component';
import { environment } from 'src/environments/environment';
import { TranslateService } from '@ngx-translate/core';
import { Subscription } from 'rxjs';
import { EventsKey } from 'ol/events';
import { unByKey } from 'ol/Observable';
import { MapCorePopupService, Popup } from 'src/app/vitis/map-core/services/map-core-popup/map-core-popup.service';
import { Geometry } from 'ol/geom';
import { MapCoreDrawService } from 'src/app/vitis/map-core/services/map-core-draw/map-core-draw.service';
import { UntypedFormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { NgbActiveModal, NgbModal } from '@ng-bootstrap/ng-bootstrap';
import exportFromJSON from 'export-from-json';
import { ActivatedRoute, Router } from '@angular/router';
import { VmapGtfWorkspaceModalComponent } from '../vmap-gtf-workspace-modal/vmap-gtf-workspace-modal.component';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { VmapToolsReportService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-report/vmap-tools-report.service';
import { WklDateComponent } from 'src/app/vitis/workspacelist/template/wkl-date/wkl-date.component';
import { VmapLayer } from 'src/app/vitis/map-core/services/map-models.model';

declare let $: any;
@Component({
  selector: 'app-vmap-tools-requestor',
  templateUrl: './vmap-tools-requestor.component.html',
  styleUrls: ['./vmap-tools-requestor.component.scss']
})
export class VmapToolsRequestorComponent implements OnInit, OnDestroy {

  @Input() olMap: Map;
  @Input() oTool: MapToolDef;

  @Input() oTabsEmitter: EventEmitter<Array<any>>;
  @Input() oSelectedTabEmitter: EventEmitter<number>;

  @ViewChild(DataGridComponent, { static: false }) oDataGrid: DataGridComponent;
  @ViewChild('exportModal') public exportModalContent: ElementRef;
  private exportModal: NgbActiveModal;
  private workbenchModal: NgbActiveModal;

  public iSelectedTab: number;
  private aTabs: Array<any> = [];

  public aLayers: Array<any> = [];
  public currentLayer: string;
  public aReports: Array<any> = [];

  public workbenchArray: Array<any> = [];
  protected workbenchName: string = 'default_name';

  private layer: VmapLayer;
  private bOpen: boolean = false;

  public aFilterColumns = [];
  public sFilterGeom: string;
  public olFilterGeom: Geometry;
  public bFilterGeomEnabled: boolean = true;
  public bIsMultiUpdateLayer: boolean = false;
  public bHaveDeleteRight: boolean = false;
  public bIsInsertLayer: boolean = false;
  public bIsReportableLayer: boolean = true;

  // Filtre en cours
  public oFilter: any = {
    type: 'custom',
    nodes: []
  };

  // Types de filtre disponible
  public oFilterTypes: any = {
    custom: 'VITIS_WORKSPACELIST_FILTER_CUSTOM',
    form: 'VITIS_WORKSPACELIST_FILTER_FORM'
  };

  // Largeur de page
  public pageLength: Number = environment.properties.rows_per_page;

  // Options de la grille de données.
  public oDataGridOptions: DataGridOptions = {
    datatableOptions: {},
    datagridOptions: WorkspaceListOptions
  };

  private locationPopup: Popup;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];
  private mapSubscriptions: EventsKey[] = [];

  // Formats d'exportation de la liste.
  public exportFormats: any = {
    'csv': {
      replace_label: true,
      no_data_value: ''
    },
    'xls': {
      replace_label: true,
      no_data_value: ''
    },
    'json': {
      replace_label: false,
      no_data_value: null
    },
    'xml': {
      replace_label: false,
      no_data_value: ''
    }
  };
  public exportTypes: any = [{
    label: 'REQUESTOR_EXTRACT_TYPE_SELECTION',
    value: 'selected_rows',
  }, {
    label: 'REQUESTOR_EXTRACT_TYPE_CURRENT_PAGE',
    value: 'current_page',
  }, {
    label: 'REQUESTOR_EXTRACT_TYPE_ALL_DATA',
    value: 'all_rows',
  }];
  public exportForm: UntypedFormGroup;

  private oParamsIds: Array<number>;
  private oParamsLayerId: number;

  /**
   * Retourne les tabs visible
   */
  public get aVisibleTabs(): any {
    return this.aTabs.filter(tab => tab.visible === true);
  }

  /**
   * Constructeur
   */
  constructor(
    private mapManagerService: MapCoreManagerService,
    protected vitisRequest: VitisRequest,
    protected vitisCheck: VitisCheck,
    private auth: AuthService,
    protected translate: TranslateService,
    public vmapToolsService: VmapToolsService,
    public mapCoreUtilsService: MapCoreUtilsService,
    public vmapSelectService: VmapSelectService,
    private reportService: VmapToolsReportService,
    private mapPopup: MapCorePopupService,
    private mapCoreDrawService: MapCoreDrawService,
    private formBuilder: UntypedFormBuilder,
    private modalService: NgbModal,
    private route: ActivatedRoute,
    private router: Router,
    private vitisClone: VitisClone,
    protected formReaderService: FormreaderService,
    public viewContainer: ViewContainerRef,
    @Optional() @Inject(DashboardComponent) private dashboardComponent: DashboardComponent,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    // Rafraîchit les données du requêteur en cas de changement de carte
    this.mapSubscriptions.push(this.olMap.on('change', () => {
      this.initRequestor();
      this.initDrawLayer();
    }));

    this.initRequestor();
    this.initDrawLayer();

    // Récupération des traitements FME / GTF
    if (environment.properties.enable_gtf_api == true) {
      this.getFMEWorkspaces();
    }

    this.subscriptions.push(this.route.data.subscribe(() => {
      this.subscriptions.push(this.route.queryParams.subscribe((oParams: any) => {
        this.oParamsIds = null;
        this.oParamsLayerId = null;
        if (oParams.ids && oParams.layer_id) {
          this.oParamsIds = oParams.ids.split(';');
          this.oParamsLayerId = oParams.layer_id;
        }
        setTimeout(() => {
          // Ouvre le requêteur s'il y a des paramètres ids et layer_id dans l'url
          this.openFromIdsSelection_();
        });
      }));
    }));

    // Nombre d'éléments à afficher sur la grille
    this.pageLength = Number(environment.properties.rows_per_page);

    // Subscribe aux évènements de la barre d'onglets de l'outil
    this.subscriptions.push(this.oTabsEmitter.subscribe((next: Array<any>) => {
      this.aTabs = next;
      this.oSelectedTabEmitter.next(0);
    }));

    this.subscriptions.push(this.oSelectedTabEmitter.subscribe((next: number) => {
      this.iSelectedTab = next;

      if (this.aVisibleTabs.length > 0) {

        // Vide le filtre
        this.oFilter = {
          type: 'custom',
          nodes: []
        };

        // Couche en cours
        this.currentLayer = this.aVisibleTabs[this.iSelectedTab].label;

        // Ferme la popup de localisation
        this.closeLocationPopup_();

        // Initialise la grille et charge la donnée
        this.initLayerDataGrid_();

        setTimeout(() => {
          this.calcTableHeight();
        });

        // Recharge les traitements GTF de la couche (si l'application est configurée pour utiliser l'API de GTF)
        if (environment.properties.enable_gtf_api == true) {
          this.getFMEWorkspaces();
        }
      }
    }));

    // Actualise la taille du tableau
    this.subscriptions.push(this.vmapToolsService.getBottomPanelEmiter().subscribe(() => {
      /**
       * Réinitialise la hauteur du data grid pour obtenir la hauteur du composant
       */
      this.oDataGridOptions.datatableOptions.tableHeight = 0;
      if (this.vitisCheck.isDefAndNotNull(this.oDataGrid)) {
        this.oDataGrid.ajustTableHeight();
      }
      if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datatableOptions)) {
        setTimeout(() => {
          this.calcTableHeight();
        }, 500);
      }
    }));

    this.subscriptions.push(this.vmapToolsService.getRefreshMapEmitter().subscribe(() => {
      if (this.bOpen) {
        this.refreshGrid();
      }
    }));

    // Ouverture du requêteur après une requête géométrique
    this.subscriptions.push(this.vmapSelectService.getRequestorSelectionObs().subscribe((selectionOptions) => {
      this.openFromGeomSelection_(selectionOptions);
    }));

    // Ouverture de l'outil
    this.subscriptions.push(this.oTool.onOpen.subscribe(() => {
      this.bOpen = true;
      if (!this.vitisCheck.isDefAndNotNull(this.iSelectedTab)) {
        // Lance le chargement de la première tab
        this.oSelectedTabEmitter.next(0);
      } else {
        this.refreshGrid();
      }
      this.displayGeomSelectionFilter();
    }));

    // Fermeture de l'outil
    this.subscriptions.push(this.oTool.onClose.subscribe(() => {
      this.bOpen = false;
      // Ferme la popup de localisation
      this.closeLocationPopup_();

      // Supprime le feature de filtre
      this.displayGeomSelectionFilter(true);
    }));

    // formulaire d'exportation de la liste.
    this.exportForm = this.formBuilder.group({
      export_file_name: ['export', Validators.required],
      export_file_format: ['csv', Validators.required],
      export_rows_number: ['20'],
      export_type: ['selected_rows'],
    });
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
    this.mapSubscriptions.forEach(mapSubscription => unByKey(mapSubscription));
    this.displayGeomSelectionFilter(true);
  }

  /**
   * Calcul la hauteur de la table
   */
  private calcTableHeight() {
    const containerHeight = this.viewContainer.element.nativeElement.offsetHeight;
    const filterHeight = $('.grid-filter').height() + 10 || 0;
    const buttonsHeight = 31;
    const footerHeight = $('.tableInfos').height() || 21;
    const margins = 60;
    const dataGridHeight = containerHeight - (filterHeight + buttonsHeight + footerHeight + margins);
    this.oDataGridOptions.datatableOptions.tableHeight = dataGridHeight;
    if (this.vitisCheck.isDefAndNotNull(this.oDataGrid)) {
      this.oDataGrid.ajustTableHeight();
    }
  }


  /**
   * Applique le filtre à la grille
   * @param bReloadData reload oDataGrid (default true)
   */
  public applyFilter(bReloadData = true): void {
    // Nettoyage du filtre
    this.oFilter.nodes = this.oFilter.nodes.map(e => {
      if (e.compareOperator === '~' && e.column.searchType === 'number') {
        e.compareOperator = '=';
      }
      return e;
    });

    // Filtre formaté pour la requête API
    this.oFilter.value = this.vitisRequest.getAPIFilterFromNodes(this.oFilter.nodes, this.oFilter.relationOperator);

    // Rafraichit la vue
    if (this.oDataGrid) {
      this.oDataGrid.reloadTable(false, { filter: this.oFilter });
    }
  }

  /**
   * Bouton rafraîchir
   *
   * On n'utilise pas celui de datagrid car tous les boutons sont surchargés
   */
  public refreshGrid() {
    if (this.oDataGrid) {
      this.oDataGrid.reloadTable();
    }
  }

  /**
   * Active ou desactive le filtre géométrique
   */
  public togglegeomfilter() {
    this.bFilterGeomEnabled = !this.bFilterGeomEnabled;
    this.refreshGrid();
  }

  /**
   * Affichage de la fenêtre modale d'exportation de la liste.
   */
  public displayExportModal() {
    this.exportModal = this.modalService.open(this.exportModalContent);
  }

  /**
   * Affichage de la fenêtre modale de lancement de traitements GTF.
   */
  public async displayWorkbenchModal(workspace: any) {
    this.vmapToolsService.displayGTFWorkspaceModal(VmapGtfWorkspaceModalComponent, workspace, this.layer, null, this.oDataGrid, this.oDataGridOptions);
  }

  /**
   * Ferme de la fenêtre modale d'exportation de la liste.
   */
  public hideExportModal() {
    if (this.exportModal) {
      this.exportModal.close();
    }
  }

  /**
   * Lance la fenêtre d'export
   */
  public exportGrid() {

    // Récupération des valeurs du formulaire ou valeur par défaut
    const sFormat: string = (this.exportForm.value.export_file_format) ? this.exportForm.value.export_file_format : 'csv';
    const sFileName: string = (this.exportForm.value.export_file_name) ? this.exportForm.value.export_file_name : 'export';
    const sExportType: string = (this.exportForm.value.export_type) ? this.exportForm.value.export_type : 'selected_rows';

    this.getDataForExport(sExportType, this.exportFormats[sFormat]).then((aData: any[]) => {
      switch (sFormat) {
        case 'csv':
        case 'json':
        case 'xml':
        case 'xls':
          const aConfig: any = {
            data: aData,
            fileName: sFileName,
            exportType: sFormat,
          };
          // transforme le json et télécharge le résultat
          exportFromJSON(aConfig);
          break;
        default:
          console.error('format not available : ' + sFormat);
          break;
      }
    }).catch(() => {
      console.error('error occured while exporting data');
    }).finally(() => {
      this.hideExportModal();
    });
  }

  /**
   * Rafraîchit les données du requêteur
   */
  private initRequestor() {

    // Récupération des couches requêtables
    const requestableLayers = this.getRequestableLayers_(this.olMap);
    this.aLayers = requestableLayers;

    if (this.aLayers.length > 0) {
      this.currentLayer = this.aLayers[0].name;
      this.layer = this.aLayers[0];
      this.bIsMultiUpdateLayer = (this.layer.is_multi_edit && this.layer.have_update_right);
      this.bHaveDeleteRight = this.layer.have_delete_right;
      this.bIsInsertLayer = this.layer.have_insert_right;

      // Récupération des modèles de rapport
      this.getUserPrintReports_().then((aUserPrintReports: Array<any>) => {
        this.aReports = aUserPrintReports;
        this.bIsReportableLayer = this.aReports.length > 0;
      });
    }

    // Réinit variables de fonctionnement
    delete this.sFilterGeom;
    delete this.olFilterGeom;
    this.bFilterGeomEnabled = true;

    // Initialisation des tabs
    setTimeout(() => {
      this.initTabsFromLayers_();
      this.iSelectedTab = null;
    });
  }

  /**
   * Initialise les onglets de l'outil à partir des couches requêtables
   */
  private initTabsFromLayers_() {
    this.aTabs = [];

    for (const layer of this.aLayers) {
      this.aTabs.push({
        label: layer.name,
        value: layer.table,
        layer_id: layer.layer_id,
        selected: false,
        visible: true
      });
    }

    this.oTabsEmitter.next(this.aTabs);
  }

  /**
   * Liste des couches requêtables
   * @param olMap
   * @returns
   */
  private getRequestableLayers_(olMap: Map): Array<any> {
    const aRequestableLayers = [];
    const aLayers = this.mapManagerService.getLayersArray(olMap);

    for (let i = 0; i < aLayers.length; i++) {
      const oLayer = aLayers[i];
      if (oLayer.is_data_grid === true && oLayer.have_requestor_select_right === true) {
        if (this.vitisCheck.isDefAndNotNull(oLayer.json_columns)) {
          try {
            const aJsonColumns = JSON.parse(oLayer.json_columns);

            // Si au moins une des colonnes possède requestor === true
            if (this.vitisCheck.isDefAndNotNull(aJsonColumns.find((column: any) => {
              return column.requestor === true;
            }))) {
              aRequestableLayers.push(oLayer);
            }
          } catch (error) {
            console.error('json_columns not valid', oLayer.json_columns, oLayer);
          }
        }
      }
    }

    return aRequestableLayers.sort((a, b) => a.name > b.name && 1 || -1);
  }

  /**
   * Récupération des rapports d'impression par rapport à la couche en cours
   * @returns
   */
  private getUserPrintReports_(): Promise<Array<any>> {
    return new Promise((resolve, reject) => {
      const url = `${environment.properties.app_api_url}/vmap/printreport`;

      // Récupère le formulaire
      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url,
        params: {
          order_by: 'printreport_id',
          filter: {
            relation: 'AND',
            operators: [{
              column: 'layer_id',
              compare_operator: '=',
              value: this.layer.layer_id
            }]
          }
        }

      }).then((resp) => {
        if (this.vitisCheck.isDefAndNotNull(resp.data) && resp.data.status === 1 && this.vitisCheck.isArray(resp.data.data)) {
          resolve(resp.data.data);
        } else {
          console.error('Error getting print reports');
          reject(false);
        }
      });
    });
  }

  /**
   * Fonction déclenchée lors de la sélection d'un rapport
   */
  public submitPrintReport(printreport_id: number) {
    const aSelectedRows = this.oDataGrid.getSelectedRows_();

    if (aSelectedRows.length > 0) {
      const oPrintReport = this.aReports.find((element) => element.printreport_id === printreport_id);

      // Lance l'impression du rapport
      this.reportService.printReport(this.olMap, oPrintReport, aSelectedRows);

    } else {
      vitisAlert.fireToast({
        icon: 'warning',
        title: this.translate.instant('REQUESTOR_PRINT_REPORT_NO_SELECTION')
      });
    }
  }

  /**
   * Permet d'initialiser les colonnes de la grille de données
   */
  private async initLayerDataGrid_(): Promise<void> {
    if (this.aLayers && this.aLayers.length) {
      let columns = [];
      this.layer = this.aLayers.find(layer => layer.name === this.currentLayer);

      this.bIsMultiUpdateLayer = (this.layer.is_multi_edit && this.layer.have_update_right);
      this.bHaveDeleteRight = this.layer.have_delete_right;
      this.bIsInsertLayer = this.layer.have_insert_right;

      // Récupération des modèles de rapport
      this.getUserPrintReports_().then((aUserPrintReports: Array<any>) => {
        this.aReports = aUserPrintReports;
        this.bIsReportableLayer = this.aReports.length > 0;
      });

      if (this.vitisCheck.isDefAndNotNull(this.layer.json_columns)) {

        // Colonnes de la grille
        columns = JSON.parse(this.layer.json_columns).filter(e => e && e.column).map((column, index) => {
          const attributs: any = {
            align: 'left',
            field: column.column,
            title: column.name,
            translateTitle: column.name,
            resizeable: true,
            searchType: column.requestor_type ? column.requestor_type : (column.type ? column.type : 'string'),
            searchable: true,
            sortable: true,
            visible: column.requestor,
            index
          };
          if (column.requestor_type === 'date' || column.type === 'date') {
            attributs.template = WklDateComponent;
          }
          return attributs;
        });

        // Colonnes pour le filtre
        this.aFilterColumns = columns.filter((e: any) => e && e.field && e.visible == true);
      } else {
        return;
      }

      this.oDataGridOptions.datagridOptions = {
        module: 'vmap',
        ressource: 'vmap/layers/:id/query',
        id_field: this.layer.idfield,
        display: true,
        columns,
        refresh: false,
        export: false,
      };

      this.initDataGrid_();
    }
  }

  /**
   * Permet d'initialiser la grille de données
   */
  private initDataGrid_(): void {

    // Paramètres de la grille de données.
    this.oDataGridOptions.datatableOptions = {
      order: [0, 'desc'], // Ordre
      displayStart: 0,  // Offset
      rowId: this.oDataGridOptions.datagridOptions.id_field,
      autorefresh: null,
      select: true,
      selectMode: 'multi',
      tableHeight: 0,
      ajax: (data: any, callback: any, settings: any) => {
        this.getResourceData_(data).then((oResponse: any) => {
          callback(oResponse); // Remplace les données
        }).catch(() => {
          callback({
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          });
        });
      }
    };

    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions)) {
      if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.columns)) {

        let oDatagridColumns = [];
        if (this.vitisCheck.isArray(this.oDataGridOptions.datagridOptions.columns)) {
          oDatagridColumns = this.oDataGridOptions.datagridOptions.columns.filter(column => {
            // Privilège(s) obligatoire(s) pour afficher la colonne.
            if (this.vitisCheck.isDefAndNotNull(column.privilege)) {
              if (this.vitisCheck.isString(column.privilege)) {
                column.privilege = [column.privilege];
              }
              if (this.vitisCheck.isArray(column.privilege)) {
                const aUserPrivileges = this.auth.getPrivileges();
                for (let i = 0; i < column.privilege.length; i++) {
                  const sPrivilege = column.privilege[i];
                  if (this.vitisCheck.isString(sPrivilege) &&
                    aUserPrivileges.indexOf(sPrivilege) !== -1) {
                    return true;
                  }
                }
              } else {
                console.error('Column privilege not string or array', column.privilege);
              }
              return false;
            }
            return true;
          });
        }

        // Traitement des colonnes
        for (let i = 0; i < oDatagridColumns.length; i++) {

          const column = oDatagridColumns[i];

          // Index par défaut de la colonne
          column.index = i;

          // Traduction
          column.translateTitle = this.translate.instant(column.title);

          column.data = column.field;

          // Visibilité de la colonne
          column.visible = column.visible === false ? false : true;

          // Colonne type
          column.type = column.template ? 'html' : 'string';

          // Alignement du contenu de la colonne.
          column.className = 'table-align-' + (column.align ? column.align : 'left');

          // Service web pour filtres
          if (column.searchWebService === undefined &&
            column.searchSuggestions !== false &&
            !this.vitisCheck.isArray(column.searchSuggestions)) {
            column.searchWebService = {
              resource: this.getRessourceUrl(),
              filterOnComplete: true,
              params: {
                attributs: column.field,
                order_by: column.field,
                distinct: true,
                sort_order: 'ASC',
                limit: 10,
              },
              label: column.field,
              value: column.field,
            };
          }
        }

        this.oDataGridOptions.datagridOptions.columns = oDatagridColumns;
      }
    }

    // Initialise les boutons icons
    this.initIconButtons();

    setTimeout(() => {
      // Création de la liste.
      if (this.oDataGrid) {
        this.oDataGrid.initTable(this.oDataGridOptions);
      }
    });
  }

  /**
   * Initialise les boutons présent dans chaque ligne du tableau
   */
  public initIconButtons(): void {

    const zoomOnGeometry = {
      type: 'display',
      icon: 'icon-vitis-search',
      event: (data: any) => {
        // Géométrie
        let geometry = null;
        if (this.vitisCheck.isDefAndNotNull(this.layer) &&
          this.vitisCheck.isDefAndNotNull(this.layer.geomfield) &&
          this.vitisCheck.isDefAndNotNull(data[this.layer.geomfield])) {
          geometry = this.mapCoreUtilsService.getGeomFromEWKT(data[this.layer.geomfield], this.olMap);
        }

        this.locateGeometry_(data, geometry);
      }
    };

    const openDisplayForm = {
      type: 'update',
      icon: 'fas fa-file',
      event: (data: any) => {
        // Géométrie
        let geometry = null;
        if (this.vitisCheck.isDefAndNotNull(this.layer) &&
          this.vitisCheck.isDefAndNotNull(this.layer.geomfield) &&
          this.vitisCheck.isDefAndNotNull(data[this.layer.geomfield])) {
          geometry = this.mapCoreUtilsService.getGeomFromEWKT(data[this.layer.geomfield], this.olMap);
        }

        this.openDisplayForm_(data, geometry);
      }
    };

    this.oDataGridOptions.iconButtons = [zoomOnGeometry, openDisplayForm];
  }

  /**
   * Génére l'url pour récupérer les données
   * @returns {string}
   */
  protected getRessourceUrl(): string {
    const sRessourceUrl = this.oDataGridOptions.datagridOptions.ressource;
    return environment.properties.app_api_url + '/' + sRessourceUrl.replace(':id', this.layer.layer_id);
  }

  /**
   * Récupère les donéées au format DataTables
   * @param  {any} params
   * @returns Promise
   */
  private getResourceData_(params: any): Promise<any> {
    return new Promise((resolve, reject) => {
      this.oDataGridOptions.loadingData_ = true;

      // Affiche la géométrie d'intersection si besoin
      this.displayGeomSelectionFilter();

      // Lance la requête
      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: this.getRessourceUrl(),
        ajaxLoader: false,
        params: this.getRequestParams_(params),
        success: (resp: any) => {
          this.oDataGridOptions.loadingData_ = false;

          const oReturn = {
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          };

          if (this.vitisCheck.isDefAndNotNull(resp.data)) {
            if (this.vitisCheck.isDefAndNotNull(resp.data.status === 1)) {
              if (this.vitisCheck.isArray(resp.data.data)) {
                oReturn.data = resp.data.data;
              }
              if (this.vitisCheck.isDefAndNotNull(resp.data.total_row_number)) {
                oReturn.recordsTotal = resp.data.total_row_number;
                oReturn.recordsFiltered = resp.data.total_row_number;
              }
            }
          }

          resolve(oReturn);
        },
        error: (err: any) => {
          console.error('error while loading values :', err);

          this.oDataGridOptions.loadingData_ = false;
          const oReturn = {
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          };

          resolve(oReturn);
        }
      });
    });
  }

  /**
   * Parse les params envoyés par DataTables pour créer des paramètres interprétables par l'API Vitis
   * @param  {any} params
   * @returns any
   */
  private getRequestParams_(params: any): any {
    const requestParams: any = {};

    // Order
    if (this.vitisCheck.isDefAndNotNull(params.order)) {
      if (this.vitisCheck.isDefAndNotNull(params.order[0])) {
        if (this.vitisCheck.isString(params.order[0].dir)) {
          requestParams.sort_order = params.order[0].dir.toUpperCase();
        }
        if (this.vitisCheck.isNumber(params.order[0].column)) {
          const iColumnId = params.order[0].column;
          if (this.vitisCheck.isDefAndNotNull(params.columns[iColumnId])) {
            if (this.vitisCheck.isString(params.columns[iColumnId].data)) {
              requestParams.order_by = params.columns[iColumnId].data;
            }
          }
        }
      }
    }

    // Limit
    if (this.vitisCheck.isNumber(params.length)) {
      requestParams.limit = params.length;
    }

    // Offset
    if (this.vitisCheck.isNumber(params.start)) {
      requestParams.offset = params.start;
    }

    // Filtrage de la couche à appliquer.
    const oRequestFilter: any = this.vitisClone.clone(this.oFilter);
    const aLayersFilters: any = this.olMap.get('layers_filters');

    // Merge le filtre avec celui appliqué sur les couches
    if (this.vitisCheck.isArray(aLayersFilters) && aLayersFilters.length > 0) {
      const oLayerFilter = aLayersFilters.find((oLayer: any) => {
        return oLayer.layer_id === this.layer.layer_id;
      });
      if (oLayerFilter &&
        this.vitisCheck.isArray(oLayerFilter.filter.operators) &&
        oLayerFilter.filter.operators.length > 0) {

        // Nouveau filtre ou ajout aux filtres du requêteur.
        if (oRequestFilter.nodes.length == 0) {
          oRequestFilter.value = oLayerFilter.filter;
        } else {
          oRequestFilter.value = {
            relation: 'AND',
            operators: [oLayerFilter.filter, oRequestFilter.value],
          };
        }
      }
    }

    // Applique le filtre à la requête.
    if (this.vitisCheck.isDefAndNotNull(oRequestFilter.value) &&
      Object.keys(oRequestFilter.value).length > 0) {
      requestParams.filter = oRequestFilter.value;
    }

    // Filtre géométrique
    if (this.bFilterGeomEnabled && this.vitisCheck.isDefAndNotNull(this.sFilterGeom)) {
      requestParams.intersect_geom = this.sFilterGeom;
      requestParams.intersect_buffer = this.vmapSelectService.getRealScaleBuffer_(this.layer, this.sFilterGeom, this.olMap);
    }
    requestParams.result_srid = this.olMap.getView().getProjection().getCode().substring(5);

    // Attributs
    if (this.vitisCheck.isDefAndNotNull(params.columns)) {
      // détecte la présence d'attribut supplementaire requis par la liste
      const extra_attributs = (this.oDataGridOptions.datagridOptions.extra_attributs) ? this.oDataGridOptions.datagridOptions.extra_attributs : [];
      requestParams.attributs = params.columns.filter(
        (aColumn: any) => this.vitisCheck.isString(aColumn.data)
      ).map(
        (aColumn: any) => aColumn.data
      ).concat(extra_attributs).join('|');
    }

    return requestParams;
  }

  /**
   * Affiche la géométrie sur la carte et zoom dessus
   * @param data
   */
  private locateGeometry_(data: any, geometry: Geometry) {

    // Texte à afficher
    let popupText = null;
    if (this.vitisCheck.isDefAndNotNull(this.layer) &&
      this.vitisCheck.isDefAndNotNull(this.layer.select_field) &&
      this.vitisCheck.isDefAndNotNull(data[this.layer.select_field])) {
      popupText = data[this.layer.select_field];
    }

    // Ferme la popup de localisation
    this.closeLocationPopup_();

    // Nouvelle popup
    this.locationPopup = this.mapPopup.displayPopup(popupText, new Feature({ geometry: geometry }), this.olMap, { hideCloseBtn: true });

    // Centrer la carte sur la géométrie
    this.mapCoreUtilsService.centerMapOnGeom(this.olMap, geometry, this.vmapToolsService.getCenterPadding());
  }

  /**
   * Ouvre le formulaire de consultation
   * @param data
   */
  private openDisplayForm_(data: any, geometry: Geometry) {

    // Ferme la popup de localisation
    this.closeLocationPopup_();

    // Id de l'enregistrement
    let id = null;
    if (this.vitisCheck.isDefAndNotNull(this.layer) &&
      this.vitisCheck.isDefAndNotNull(this.layer.idfield) &&
      this.vitisCheck.isDefAndNotNull(data[this.layer.idfield])) {
      id = data[this.layer.idfield];
    } else {
      console.error('cannot get selection id', data, this.layer);
      return null;
    }

    this.vmapSelectService.selectById(id, this.layer, this.olMap).then((aData) => {

      // Set les nouvelles sélections
      const aSelections = [];
      if (aData && aData[0]) {
        for (let i = 0; i < aData.length; i++) {
          aSelections.push({
            data: aData[i],
            oLayer: this.layer,
            geometry: geometry,
          });
        }
      }

      // Affiche les popups
      if (this.vitisCheck.isDefAndNotNull(geometry) && this.layer.is_geometry_layer) {
        this.vmapSelectService.displayPopups(this.olMap, aSelections);
      }

      // Sélectionne le mode (modify/insert) avec la sélection qui va avec
      this.vmapSelectService.closeSelections();
      this.vmapSelectService.setSelectMode('modify');
      this.vmapSelectService.setSelection(aSelections);

      // Centrer la carte sur la géométrie
      if (this.vitisCheck.isDefAndNotNull(geometry) && this.layer.is_geometry_layer) {
        this.mapCoreUtilsService.centerMapOnGeom(this.olMap, geometry, this.vmapToolsService.getCenterPadding());
      }
    });
  }

  /**
   * Ferme la popup de localisation
   */
  private closeLocationPopup_() {
    // Ferme la précédente popup
    if (this.locationPopup) {
      this.locationPopup.close();
      this.locationPopup = null;
    }
  }

  /**
   * Retourne la tab en fonction du layer_id passé en paramètre
   * @param iLayerId
   * @returns
   */
  private getTabByLayerId_(iLayerId: string | number) {
    return this.aTabs.find(tab => tab && tab.layer_id == iLayerId);
  }

  /**
   * Ouvre le requêteur après une requête géométrique
   */
  private openFromGeomSelection_(options: any) {
    if (options.layer &&
      options.geom) {

      if (options.layer.layer_id) {
        const tab = this.getTabByLayerId_(options.layer.layer_id);
        const tabIndex = this.aVisibleTabs.indexOf(tab);

        if (tab && tabIndex !== -1) {
          this.iSelectedTab = tabIndex;
        }
      }

      if (!this.vitisCheck.isDefAndNotNull(this.iSelectedTab)) {
        this.iSelectedTab = 0;
      }

      this.bFilterGeomEnabled = true;
      this.sFilterGeom = options.geom;
      this.olFilterGeom = options.olGeometry;
      this.vmapToolsService.setBottomPanel(1, 'requestor');
      this.oTool.open();
      this.oSelectedTabEmitter.next(this.iSelectedTab);
    }
  }

  /**
   * Initialise le vector Layer pour ajouter les sélections à la carte
   */
  private initDrawLayer(): void {
    this.olMap.set('requestor_draw_layer', this.mapCoreDrawService.initDrawLayer(this.olMap, null));
  }

  /**
   * Affiche la géométrie de this.olFilterGeom si bFilterGeomEnabled vaut true
   */
  private displayGeomSelectionFilter(bForceDelete = false): void {

    const olDrawLayer = this.olMap.get('requestor_draw_layer');
    olDrawLayer.getSource().clear();

    if (this.bFilterGeomEnabled && this.vitisCheck.isDefAndNotNull(this.olFilterGeom) && bForceDelete !== true) {
      olDrawLayer.getSource().addFeature(new Feature({ geometry: this.olFilterGeom }));
    }
  }

  /**
   * Récupère la donnée pour l'export
   * @param sExportType
   * @param aConfig
   * @returns
   */
  private getDataForExport(sExportType: string, aConfig: any = {}): Promise<any[]> {
    return new Promise((resolve, reject) => {
      let aSelectedRows = [];
      let iNbRows: number;
      switch (sExportType) {
        case 'current_page':
          aSelectedRows = [...this.oDataGrid.oDataGridOptionsRecords].slice(
            (this.oDataGrid.page - 1) * this.oDataGrid.pageLength, this.oDataGrid.page * this.oDataGrid.pageLength);
          this.oDataGrid.getDataGridDataForExport(0, { ...aConfig, ...{ aSelectedRows } }).then(aData => {
            resolve(aData);
          });
          break;
        case 'all_rows':
          vitisAlert.alert({
            text: this.translate.instant('REQUESTOR_EXTRACT_TYPE_ALL_DATA_WARNING'),
            type: 'warning',
            showCancelButton: true,
            confirmButtonText: this.translate.instant('REQUESTOR_EXTRACT_TYPE_ALL_DATA_WARNING_OK'),
            cancelButtonText: this.translate.instant('REQUESTOR_EXTRACT_TYPE_ALL_DATA_WARNING_CANCEL'),
          }).then((result) => {
            if (result.isDismissed === false) {
              iNbRows = this.oDataGrid.oDataGridOptionsRecordsTotal;
              this.oDataGrid.getDataGridDataForExport(iNbRows, aConfig).then(aData => {
                resolve(aData);
              });
            }
          });
          break;
        case 'selected_rows':
          aSelectedRows = this.oDataGrid.getSelectedRows_();
          this.oDataGrid.getDataGridDataForExport(0, { ...aConfig, ...{ aSelectedRows } }).then(aData => {
            resolve(aData);
          });
          break;
        default:
          break;
      }
    });
  }

  /**
   * Ouvre le requêteur d'après les ids dans l'url
   */
  private openFromIdsSelection_() {
    if (this.oParamsIds && this.oParamsLayerId) {

      const tab = this.getTabByLayerId_(this.oParamsLayerId);
      const tabIndex = this.aVisibleTabs.indexOf(tab);

      if (tab && tabIndex !== -1) {
        this.iSelectedTab = tabIndex;
      }

      // Ouvre le requêteur
      if (!this.vitisCheck.isDefAndNotNull(this.iSelectedTab)) {
        this.iSelectedTab = 0;
      }
      if (this.bOpen === false) {
        this.vmapToolsService.setBottomPanel(1, 'requestor');
        this.oTool.open();
      }

      this.oSelectedTabEmitter.next(this.iSelectedTab);

      // Gère le filtre
      this.bFilterGeomEnabled = false;
      const aSelectLayers = this.vmapSelectService.getSelectLayers();
      const initLayer = this.oParamsLayerId ? aSelectLayers.find(layer => layer.layer_id == +this.oParamsLayerId) : aSelectLayers.find(layer => !layer.layer_id);
      if (initLayer) {
        this.oFilter.nodes = [{
          column: {
            field: initLayer.idfield
          },
          compareOperator: 'IN',
          value: this.oParamsIds
        }];
      }

      this.applyFilter(false);

      setTimeout(() => {
        this.clearUrl_();
      });
    }
  }

  /**
   * Nettoie ids de l'URL
   */
  private clearUrl_() {
    this.router.navigate([],
      {
        relativeTo: this.route,
        queryParams: { layer_id: null, ids: null },
        queryParamsHandling: 'merge'
      });
  }

  /**
   * Ouvre le formulaire d'édition multiple avec les enregistrements sélectionné.
   */
  public displayMultiUpdate() {

    const aSelectedRows = this.oDataGrid.getSelectedRows_();

    if (aSelectedRows.length > 0) {
      if (this.vitisCheck.isDefAndNotNull(aSelectedRows)) {
        // Ferme la popup de localisation
        this.closeLocationPopup_();

        // Ids des enregistrements
        const ids = [];
        for (let i = 0; i < aSelectedRows.length; i++) {
          if (this.vitisCheck.isDefAndNotNull(this.layer) &&
            this.vitisCheck.isDefAndNotNull(this.layer.idfield) &&
            this.vitisCheck.isDefAndNotNull(aSelectedRows[i][this.layer.idfield])) {
            ids.push(aSelectedRows[i][this.layer.idfield]);
          } else {
            console.error('cannot get selection id', aSelectedRows[i], this.aVisibleTabs[this.iSelectedTab]);
            return null;
          }
        }

        const filter = {
          relation: 'AND',
          operators: [{
            column: this.layer.idfield,
            compare_operator: 'IN',
            value: ids,
          }]
        };
        this.vmapSelectService.selectById(ids, this.layer, this.olMap, { filter }).then((aData) => {

          // Set les nouvelles sélections
          const aSelections = [];
          const aGeometries: Array<Geometry> = [];
          if (aData && aData[0]) {
            let olGeom: Geometry;
            for (let i = 0; i < aData.length; i++) {
              olGeom = null;
              if (this.layer.is_geometry_layer) {
                olGeom = this.mapCoreUtilsService.getGeomFromEWKT(aData[i][this.layer.geomfield], this.olMap);
                aGeometries.push(olGeom);
              }
              aSelections.push({
                data: aData[i],
                oLayer: this.layer,
                geometry: olGeom
              });
            }
          }

          // Sélectionne le mode (multiedit) avec la sélection qui va avec
          this.vmapSelectService.closeSelections();
          if (aSelectedRows.length > 1) {
            this.vmapSelectService.setSelectMode('multiedit');
          } else {
            this.vmapSelectService.setSelectMode('modify');
          }
          this.vmapSelectService.setSelection(aSelections);
        });
      }
    } else {
      vitisAlert.fireToast({
        icon: 'warning',
        title: this.translate.instant('REQUESTOR_MULTI_EDIT_SELECTION_WARNING')
      });
    }
  }

  /**
   * Suppression des enregistrements sélectionné dans le requeteur
   */
  public multiDelete() {
    this.displayDeleteWarning().then((bResponse: boolean) => {
      if (bResponse === true) {
        const aSelectedRows = this.oDataGrid.getSelectedRows_();
        // Renvoi un message d'erreur si aucune ligne est sélectionnée
        if (aSelectedRows.length == 0) {
          vitisAlert.alert({
            type: 'error',
            text: this.translate.instant('ERROR_DELETE_NO_LAYER_SELECTED'),
          });
          return null;
        }
        // Ferme la popup de localisation
        this.closeLocationPopup_();

        // Ids des enregistrements
        const ids = [];
        for (let i = 0; i < aSelectedRows.length; i++) {
          if (this.vitisCheck.isDefAndNotNull(this.layer) &&
            this.vitisCheck.isDefAndNotNull(this.layer.idfield) &&
            this.vitisCheck.isDefAndNotNull(aSelectedRows[i][this.layer.idfield])) {
            ids.push(aSelectedRows[i][this.layer.idfield]);
          } else {
            console.error('cannot get selection id', aSelectedRows[i], this.aVisibleTabs[this.iSelectedTab]);
            return null;
          }
        }

        this.vitisRequest.ajaxRequestPromise({
          method: 'DELETE',
          url: `${environment.properties.app_api_url}/vmap/layers/${this.layer.layer_id}/query`,
          data: { 'ids': ids },
        }).then((resp: any) => {
          if (resp && resp.data && resp.data.data) {
            // Rafraîchit la couche
            this.vmapToolsService.refreshLayer(this.olMap, this.layer.layer_id, true);
            this.vmapToolsService.closeAllPanels();
            // Add Toast
            vitisAlert.fireToast({
              icon: 'success',
              title: this.translate.instant('SUCCESS_DELETE_LAYER_OBJECTS')
            });
          }
        }).catch(e => {
          console.error(e);
          vitisAlert.alert({
            type: 'error',
            text: this.translate.instant('ERROR_DELETE_LAYER_OBJECTS'),
          });
        });
      }
    });
  }

  /**
   * Affiche le message de warning à la suppression
   * @returns
   */
  private displayDeleteWarning(): Promise<boolean> {
    return new Promise((resolve, reject) => {
      this.translate.get(['FORM_DELETE_OBJECTS_WARNING',
        'CONFIRM_CANCEL',
        'CONFIRM_CONFIRM']
      ).subscribe((translations) => {
        vitisAlert.alert({
          title: translations['FORM_DELETE_OBJECTS_WARNING'],
          type: 'warning',
          showCancelButton: true,
          confirmButtonText: translations['CONFIRM_CONFIRM'],
          cancelButtonText: translations['CONFIRM_CANCEL'],
        }).then((result: any) => {
          if (result &&
            result.isConfirmed === true) {
            resolve(true);
          } else {
            resolve(false);
          }
        });
      });
    });
  }

  /**
   * Ouvre le formulaire d'insertion
   */
  public displayInsert() {
    // Ferme la popup de localisation
    this.closeLocationPopup_();
    this.vmapSelectService.closeSelections();
    this.vmapSelectService.setSelectMode('add');
    this.vmapSelectService.setSelection(this.layer);
  }

  /**
   * Récupère les traitements GTF associée à la couche et à l'utilisateur
   */
  public getFMEWorkspaces(): void {
    this.vmapToolsService.getLayerFMEWorkspaces(this.layer.layer_id).then((aWorkspaces: Array<any>) => {
      this.workbenchArray = aWorkspaces.filter((val) => {
        return val.allow_requestor === true || val.allow_requestor === 1;
      });
    }).catch(() => {
      this.workbenchArray = [];
      vitisAlert.alert({
        type: 'error',
        text: this.translate.instant('ERROR_CALLING_GTF_API'),
      });
    });
  }
}
