import { Component, OnInit, Input, Output, EventEmitter, OnDestroy } from '@angular/core';
import { MapCoreDrawService } from 'src/app/vitis/map-core/services/map-core-draw/map-core-draw.service';
import { VitisClone, VitisRequest, vitisAlert, VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { BehaviorSubject, Subject } from 'rxjs';
import VectorSource from 'ol/source/Vector';
import { Circle, Geometry, LineString, MultiLineString, MultiPoint, MultiPolygon, Point, Polygon } from 'ol/geom';
import VectorLayer from 'ol/layer/Vector';
import { environment } from 'src/environments/environment';
import { Feature, Map } from 'ol';
import { VmapSelectService } from 'src/modules/vmap/components/carto/services/vmap-select/vmap-select.service';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { VmapToolsMeasureService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-measure/vmap-tools-measure.service';
import { Fill, Stroke, Style } from 'ol/style';
import CircleStyle from 'ol/style/Circle';
import { Snap } from 'ol/interaction';
import { fromExtent } from 'ol/geom/Polygon';
import { transform } from 'ol/proj';
import { TranslateService } from '@ngx-translate/core';
import { unByKey } from 'ol/Observable';
import { fromCircle } from 'ol/geom/Polygon';
import Collection from 'ol/Collection';
import moment from 'moment-timezone';

import * as modif_cord_json from './../forms/modif_coord.json';
import * as snap_form from './../forms/snap_form.json';
import { MapCorePopupService, Popup } from 'src/app/vitis/map-core/services/map-core-popup/map-core-popup.service';
import { VmapLayer } from 'src/app/vitis/map-core/services/map-models.model';
import { EventsKey } from 'ol/events';
import { VmapToolsSetCircleRadiusService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-set-circle-radius/vmap-tools-set-circle-radius.service';

class InfosCircle {
  geometry: any;
  sPopUpData: string;
  popup: Popup;
}

@Component({
  selector: 'app-vmap-tools-geometry-form',
  templateUrl: './vmap-tools-geometry-form.component.html',
  styleUrls: ['./vmap-tools-geometry-form.component.scss']
})
export class VmapToolsGeometryFormComponent implements OnInit, OnDestroy {

  @Input() map: Map;
  @Input() oLayer: VmapLayer;
  @Input() currFeature: Feature;
  @Input() geomEventValue: string;
  @Input() selection: any;
  @Input() isGeomExist: boolean;

  @Output() geomEventValueChange: EventEmitter<string> = new EventEmitter<string>();
  @Output() subscribeFeature = new EventEmitter();
  @Output() subscribeDisectionFeature = new EventEmitter();

  public sFormName = 'insert';
  public sDefinition: any;
  public sDatasForm: any = {};

  private geomType;
  public isPolygon = false; // C'est une géométrie de type polygon
  public isMultiGeom = false;
  public geomEvent = new BehaviorSubject<string | any>('');
  public isGeometryLayer = false;

  private feature: Feature<Geometry>;
  private vectorSource: VectorSource<Geometry>;
  private vectorLayer: VectorLayer<VectorSource<Geometry>>;
  private vectorLayerSnap: VectorLayer<VectorSource<Geometry>>;

  private layersSnapping: Array<number>;
  private featuresSnap: Array<Feature<Geometry>>;
  private loadSnap: boolean = false;
  public isLoadingSnap: boolean;
  private xhrSnap: any;
  private currXhrExist = new Subject<boolean>();
  private limitFeaturesSnap = 2000;

  private interactionModify: any;
  private interactionRemoveVertex: any;
  private interactionSelect: any;
  private interactionSnap: any;
  private interactionRemove: any;
  private interactionAdd: any;

  private geomEventSubscribe;
  private selectModeSubscribe;
  public mode;
  private rightPanelEmitterSubscribe;

  public listGeomtypesId = 'listGeomtypesId_' + Date.now();
  public listGeomtypesPolyId = 'listGeomtypesPolyId_' + Date.now();

  private listenerKeyInteractionModify: EventsKey;
  private listenerKeyInteractionRemoveVertex: EventsKey;
  private listenerKeyInteractionAddFeature: EventsKey;
  private listenerKeyInteractionRemoveFeature: EventsKey;

  private aSpecialTypePolygon: Array<string> = ['circle', 'box', 'square'];

  private infosCircle: InfosCircle;
  private bCursorInPopup: boolean;

  private removeVertexPointsLayer = new VectorLayer({
    source: new VectorSource(),
    zIndex: 2000,
  });

  /**
   * Constructeur
   */
  constructor(
    private vitisClone: VitisClone,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private mapCoreDrawService: MapCoreDrawService,
    private mapSelectService: VmapSelectService,
    private mapUtils: MapCoreUtilsService,
    private mapToolsService: VmapToolsService,
    private translate: TranslateService,
    private vmapToolsSetCircleRadiusService: VmapToolsSetCircleRadiusService
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {
    if (this.currFeature) {
      this.feature = this.currFeature.clone();
      setTimeout(() => {
        this.geomEvent.next('modify_object');
      });
    }
    if (this.oLayer) {
      this.mode = this.mapSelectService.getSelectMode();
      if (this.vitisCheck.isDefAndNotNull(this.oLayer['ms_geomtype']) &&
        this.vitisCheck.isDefAndNotNull(this.oLayer['ms_geomtype']['label'])) {
        let geomType = '';
        if (this.oLayer.is_multi_geom === true) {
          this.isMultiGeom = true;
          geomType += 'MULTI';
        }
        if (this.oLayer['ms_geomtype']['label'] === 'GEOMETRY') { // La géométrie de la couche est de type GEOMETRY
          this.isGeometryLayer = true;
          geomType += 'POINT';
        } else { // La géométrie de la couche est de type POINT, LIGNE OU POLYGONE
          geomType += this.oLayer['ms_geomtype']['label'];
        }
        this.geomType = geomType;
        if (geomType == 'POLYGON' || geomType == 'MULTIPOLYGON') {
          this.isPolygon = true;
        }
      } else { // La couche ne possède pas de géométrie
        this.translate.get('ERROR_INSERT_LAYER_OBJECT').subscribe(async (translation) => {
          vitisAlert.alert({
            text: translation,
            showCancelButton: true
          });
        });
      }
      setTimeout(() => {
        this.geomEvent.next(this.mode === 'modify' ? 'modify_object' : 'add_object');
      });

      this.selectModeSubscribe = this.mapSelectService.getSelectModeObs().subscribe((mode: string) => {
        this.mode = mode;
      });

      this.geomEventSubscribe = this.geomEvent.subscribe(geomvalue => {
        if (this.geomEventValue != geomvalue) {
          this.geomEventValue = geomvalue;
          if (geomvalue) {
            this.geomEventValueChange.emit(this.geomEventValue);
            switch (geomvalue) {
              case 'modify_object':
                this.deleteCurrentInteraction(['select', 'modify', 'remove', 'removeVertex']);
                this.initDrawing();
                this.modifyGeometry();
                break;
              case 'add_object':
                this.deleteCurrentInteraction(['select', 'modify', 'remove', 'removeVertex']);
                this.initDrawing();
                this.addGeometry();
                break;
              case 'del_object':
                this.deleteCurrentInteraction(['select', 'add', 'removeVertex']);
                this.removeGeometry();
                break;
              case 'snap':
                this.deleteCurrentInteraction(['modify', 'remove', 'add', 'removeVertex']);
                this.snap();
                break;
              case 'modify_coord_object':
                this.deleteCurrentInteraction(['select', 'modify', 'add', 'remove', 'removeVertex']);
                this.modifyCoordGeometry();
                break;
              case 'divide_geometry_object':
                this.deleteCurrentInteraction(['select', 'modify', 'add', 'remove', 'removeVertex']);
                this.initDrawing();
                this.divideGeometry();
                break;
              case 'remove_vertex':
                this.deleteCurrentInteraction(['select', 'modify', 'add', 'remove', 'removeVertex']);
                this.initDrawing();
                this.removeVertex();
                break;
            }
          }
        }
      });

      const this_ = this;
      this.map.on('moveend', function () {
        if (this_.loadSnap && this_.vectorLayerSnap) {
          this_.loadSnappingDatas();
        }
      });
    }
    this.rightPanelEmitterSubscribe = this.mapToolsService.getRightPanelEmiter().subscribe(formSize => {
      if (formSize === 0) {
        this.deleteAllInteraction();
      }
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy() {
    this.deleteAllInteraction();
  }

  /**
   * Initialise le vector Layer et ajoute la feature si elle existe
   */
  private initDrawing(): void {
    this.vectorSource = new VectorSource();
    this.vectorLayer = this.mapCoreDrawService.initDrawLayer(this.map, this.vectorSource);
    if (this.feature) {
      this.vectorSource.addFeature(this.feature);
    } else {
      this.getInitFeature();
    }
  }

  /**
   * Initialise une nouvelle feature grace au type de la layer
   */
  private getInitFeature(): void {
    let geometry;
    const geomType = this.checkSpecialMultiPolygonType(this.geomType);
    switch (geomType) {
      case 'polygon':
        geometry = new Polygon([]);
        break;
      case 'multipolygon':
        geometry = new MultiPolygon([]);
        break;
      case 'point':
        geometry = new Point([]);
        break;
      case 'multipoint':
        geometry = new MultiPoint([]);
        break;
      case 'linestring':
        geometry = new LineString([]);
        break;
      case 'multilinestring':
        geometry = new MultiLineString([]);
    }
    this.feature = new Feature({ geometry });
    this.subscribeFeature.next(this.feature);
  }

  /**
   * Retourne si une/des feature(s) sont présente sur le vector source de la map
   * @returns
   */
  public featureExist(): boolean {
    if (this.vectorSource) {
      if (this.vectorSource.getFeatures().length > 0) {
        return true;
      }
    }
    return false;
  }

  /**
   * Event appelé lors du clique sur 'Modifier la géométrie'
   */
  private modifyGeometry(): void {
    if(this.feature && this.vitisCheck.isDefAndNotNull(this.feature.getGeometry())) {
      const { select, interaction, snap } = this.mapCoreDrawService.addDrawInteraction('modify', this.map, this.vectorLayer, { selectFeature: this.feature, snap: this.interactionSnap, getInteractions: true });
      this.interactionModify = interaction;
      this.interactionSelect = select;
      this.interactionSnap = snap;

      this.listenerKeyInteractionModify = this.interactionModify.on('modifyend', (evt: any) => {
        const feature = evt.features.getArray()[0];
        this.feature = new Feature({ geometry: feature.getGeometry() });
        this.subscribeFeature.next(this.feature);
      });
    } else {
      if (this.geomType) {
        this.addGeometry();
      }
    }
  }

  /**
   * Event appelé lors du clique sur 'Ajouter un object'
   */
  private addGeometry(): void {
    if (this.geomType) {
      let interactionFormat = this.isMultiGeom ? this.geomType.toLowerCase().substring(5) : this.geomType.toLowerCase();
      const { interaction, snap } = this.mapCoreDrawService.addDrawInteraction(interactionFormat, this.map, this.vectorLayer, { snap: this.interactionSnap, getInteractions: true });
      this.interactionAdd = interaction;
      this.interactionSnap = snap;

      // Pour les cercles : permet l'affichage de la popup
      this.vmapToolsSetCircleRadiusService.setCircleListener(interaction, this.map, 'setCircle', this);

      this.listenerKeyInteractionAddFeature = this.vectorSource.on('addfeature', (evt: any) => {
        if (this.infosCircle && this.infosCircle.popup) {
          this.infosCircle.popup.close();
        }
        const geometryAdded: Polygon | Point | LineString = evt.feature.getGeometry().getType() == 'Circle' ? fromCircle(evt.feature.getGeometry()) : evt.feature.getGeometry();

        if (this.isMultiGeom) {
          const multiGeom: any = this.feature.getGeometry();
          interactionFormat = this.checkSpecialPolygonType(interactionFormat);
          if (multiGeom) {
            switch (interactionFormat) {
              case 'polygon':
                multiGeom.appendPolygon(geometryAdded);
                break;
              case 'point':
                multiGeom.appendPoint(geometryAdded);
                break;
              case 'linestring':
                multiGeom.appendLineString(geometryAdded);
                break;
            }
            this.feature = new Feature({ geometry: multiGeom });
            this.subscribeFeature.next(this.feature);
          }
        } else {
          const features = this.vectorSource.getFeatures();
          for(let i: number = 0; i < features.length; i++) {
            if (!this.vitisCheck.isDefAndNotNull(features[i].getGeometry())) {
              this.vectorSource.removeFeature(features[i]);
              features.splice(i, 1);
            }
          }
          if (features && features.length > 1) {
            const currFeature = features[0];
            this.vectorSource.removeFeature(currFeature);
          }
          this.feature = new Feature({ geometry: geometryAdded });
          this.subscribeFeature.next(this.feature);
        }
      });
    }
  }

  /**
   * Event appelé lors du clique sur 'Supprimer un object'
   */
  private removeGeometry(): void {
    const interactionFormat = this.checkSpecialPolygonType(this.geomType.toLowerCase().substring(5));
    let geometries = [];
    const currGeom: any = this.feature.getGeometry();
    if (currGeom) {
      switch (interactionFormat) {
        case 'polygon':
          geometries = currGeom.getPolygons();
          break;
        case 'point':
          geometries = currGeom.getPoints();
          break;
        case 'linestring':
          geometries = currGeom.getLineStrings();
          break;
      }
    }
    this.vectorSource = new VectorSource();
    this.vectorLayer = this.mapCoreDrawService.initDrawLayer(this.map, this.vectorSource);
    if (geometries.length > 0) {
      this.vectorSource.addFeatures(geometries.map(geom => new Feature({ geometry: geom })));
    }
    this.interactionRemove = this.mapCoreDrawService.addDrawInteraction('remove', this.map, this.vectorLayer, { snap: this.interactionSnap });

    this.listenerKeyInteractionRemoveFeature = this.vectorSource.on('removefeature', (evt: any) => {
      const features = this.vectorLayer.getSource().getFeatures();
      const geomtype = this.feature.getGeometry().getType();
      const interactionFormatFeature = this.checkSpecialPolygonType(geomtype.toLowerCase().substring(5));
      let geometry;
      switch (interactionFormatFeature) {
        case 'polygon':
          geometry = new MultiPolygon([]);
          features.forEach(feature => geometry.appendPolygon(feature.getGeometry()));
          break;
        case 'point':
          geometry = new MultiPoint([]);
          features.forEach(feature => geometry.appendPoint(feature.getGeometry()));
          break;
        case 'linestring':
          geometry = new MultiLineString([]);
          features.forEach(feature => geometry.appendLineString(feature.getGeometry()));
          break;
      }
      if (features.length) {
        this.feature = new Feature({ geometry });
      } else {
        this.feature = null;
      }
      this.subscribeFeature.next(this.feature);
    });
  }

  /**
   * Event appelé lors du clique sur 'Accrochage universel'
   */
  private snap(): void {
    const sDatas = {
      type: 1,
      pixelTolerance: 15,
      limit: this.limitFeaturesSnap
    };

    if (this.interactionSnap) {
      sDatas.pixelTolerance = this.interactionSnap.pixelTolerance_;
      if (this.interactionSnap.vertex_) {
        sDatas.type = 2;
        if (this.interactionSnap.edge_) {
          sDatas.type = 3;
        }
      }
    }
    if (this.vectorLayerSnap) {
      const style: any = this.vectorLayerSnap.getStyle();
      if (style.getFill() || style.getStroke() || style.getImage()) {
        sDatas['visibility'] = true;
      }
    }
    const definition = this.vitisClone.clone(snap_form['default']);


    if (this.interactionSnap) {
      sDatas.pixelTolerance = this.interactionSnap.pixelTolerance_;
    }

    const layersFields = {
      'fields': [
      ]
    };
    const templateLayer = {
      'type': 'checkbox',
      'value': false,
      'nb_cols': 12
    };
    this.mapSelectService.getSelectLayers().forEach(layer => {
      const layerRow = this.vitisClone.clone({ ...this.vitisClone.clone(templateLayer), 'name': layer.layer_id, 'label': layer.name });
      if (this.layersSnapping && this.layersSnapping.length) {
        sDatas[layer.layer_id] = this.layersSnapping.includes(layer.layer_id);
      }
      layersFields.fields.push(this.vitisClone.clone(layerRow));
    });
    definition.update.rows.unshift(this.vitisClone.clone(layersFields));
    this.loadForm({
      sDefinition: this.vitisClone.clone(definition),
      sFormName: 'update',
      sDatasForm: this.vitisClone.clone(sDatas)
    });
  }

  /**
   * Event appelé lors de la validation du formulaire 'Accrochage universel'
   * @param event
   */
  private createSnap(event): any {
    this.limitFeaturesSnap = event.limit;
    this.layersSnapping = Object.keys(event).filter(key => Number.isInteger(+key) && event[key]).map(layer_id => +layer_id);

    if (this.interactionSnap) {
      this.map.removeInteraction(this.interactionSnap);
      this.interactionSnap = null;
      if (this.vectorLayerSnap) {
        this.vectorLayerSnap.setStyle(null);
        this.map.removeLayer(this.vectorLayerSnap);
        this.vectorLayerSnap = null;
      }
    }

    const vectorSourceSnap = new VectorSource();

    const selectStyle = new Style({
      fill: new Fill({
        color: 'rgba(255, 255, 255, 0.2)',
      }),
      stroke: new Stroke({
        color: '#337ab7',
        width: 2,
      }),
      image: new CircleStyle({
        radius: 3,
        fill: new Fill({
          color: '#337ab7',
        }),
      }),
    });

    this.vectorLayerSnap = new VectorLayer({
      source: vectorSourceSnap,
      style: event.visibility ? selectStyle : new Style(),
      zIndex: 1000,
    });
    this.map.addLayer(this.vectorLayerSnap);

    this.interactionSnap = new Snap({
      source: this.vectorLayerSnap.getSource(),
      pixelTolerance: event.pixelTolerance,
      vertex: event.type == 3 || event.type == 2,
      edge: event.type == 3
    });

    this.loadSnap = this.layersSnapping.length > 0;
    if (this.loadSnap === true) {
      this.loadSnappingDatas();
    } else {
      this.isLoadingSnap = null;
    }
  }

  /**
   * Event appelé lors du clique sur 'Modifier les coordonnées'
   */
  private modifyCoordGeometry() {
    const featureClone = this.feature.clone();
    const geometry: any = featureClone.getGeometry();

    this.geomType = this.checkSpecialMultiPolygonType(this.geomType).toUpperCase();
    const sDatas = {
      format: 'WKT',
      projection: 'EPSG:4326',
      geomtype: this.geomType,
      type: geometry.getType()
    };
    if (geometry.getCoordinates().length > 0) {
      const EWKT = this.mapUtils.getEWKTFromGeomAndProj(geometry, this.map, '4326');
      const WKT = this.mapUtils.getWKTFromEWKT(EWKT);
      sDatas['wkt'] = WKT.geom;
      sDatas['projection'] = WKT.proj;
    }
    const sDefinition = modif_cord_json['default'];
    sDefinition.update.initEvent = 'initEventModifCord()';
    this.loadForm({
      sDefinition: this.vitisClone.clone(sDefinition),
      sFormName: 'update',
      sDatasForm: this.vitisClone.clone(sDatas)
    });
  }

  /**
   * Récupère la géométrie résultante du découpage
   * @returns Promise
   */
  private getDividedGeom(geom1: string, geom2: string, r_srid: number, type: string): Promise<any> {
    return new Promise((resolve, reject) => {
      const url = `${environment.properties.app_api_url}/vmap/disect/polygon`;

      // Récupère la géométrie
      this.vitisRequest.ajaxRequestPromise({
        method: 'POST',
        url,
        data: {
          'geom1': geom1,
          'geom2': geom2,
          'result_srid': r_srid
        }

      }).then((resp) => {
        if (this.vitisCheck.isDefAndNotNull(resp.data) && resp.data.length > 1) {
          resolve(resp.data);
        } else {
          console.error('Error divided geometry');
          reject(false);
        }
      })
        .catch((e) => {
          console.error(e);
          if (e && e.data && e.data.format && e.data.errorMessage === 'WRONG_GEOM_FORMAT') {
            this.translate.get(['ERROR_DISECTING_GEOMETRY', 'ERROR_DISECTING_GEOMETRY_DETAIL_WRONG_GEOM_FORMAT']).subscribe(async (translation) => {
              vitisAlert.alert({
                type: 'error',
                title: translation['ERROR_DISECTING_GEOMETRY'],
                text: translation['ERROR_DISECTING_GEOMETRY_DETAIL_WRONG_GEOM_FORMAT'].replace('[sGeomType]', e.data.format),
              });
            });
          } else {
            this.translate.get(['ERROR_DISECTING_GEOMETRY', 'ERROR_DISECTING_GEOMETRY_DETAIL']).subscribe(async (translation) => {
              vitisAlert.alert({
                type: 'error',
                title: translation['ERROR_DISECTING_GEOMETRY'],
                text: translation['ERROR_DISECTING_GEOMETRY_DETAIL'],
              });
            });
          }
          reject();
        });
    });
  }

  /**
   * Fonction pour diviser une géométrie
   */
  private divideGeometry() {
    const initGeomType = this.geomType.toLowerCase();
    const initFeature: Geometry = this.feature.getGeometry();
    let disectedFeature: Feature<Geometry>[];
    const mapProj: string = this.map.getView().getProjection().getCode().substring(5);
    const { interaction, snap } = this.mapCoreDrawService.addDrawInteraction((initGeomType === 'line') ? 'point' : 'line', this.map, this.vectorLayer, { selectFeature: this.feature, getInteractions: true });
    this.interactionModify = interaction;
    this.map.addInteraction(snap);
    this.listenerKeyInteractionModify = this.interactionModify.on('drawend', (evt: any) => {
      const initGeom = this.mapUtils.getEWKTFromGeom(initFeature, this.map);
      const drawnGeom = this.mapUtils.getEWKTFromGeom(evt.feature.getGeometry(), this.map);
      this.getDividedGeom(initGeom, drawnGeom, Number(mapProj), initGeomType)
        .then((geoms: Array<string>) => {
          this.vectorSource.clear();
          this.feature = new Feature({ geometry: this.mapUtils.getGeomFromEWKTAndProj(geoms[0], mapProj) });
          disectedFeature = [new Feature({ geometry: this.mapUtils.getGeomFromEWKTAndProj(geoms[1], mapProj) })];
          if (initGeomType === 'line') {
            this.vectorSource.addFeatures([
              this.feature,
              ...disectedFeature,
              evt.feature
            ]);
            // retire le layer de surbrillance, qui cache la ligne, si il existe, sinon, devient le plus haut zIndex
            if (this.vitisCheck.isDefAndNotNull(this.map.get('popup_layer')) && this.map.get('popup_layer').getVisible()) {
              this.map.get('popup_layer').setVisible(false);
            } else {
              const layerArray: Array<any> = this.map.getLayers().getArray();
              this.vectorLayer.setZIndex(Math.max(...layerArray.map(o => o.getZIndex())) + 1);
            }
          } else {
            geoms.shift();
            disectedFeature = geoms.map((elem: string) => {
              return new Feature({ geometry: this.mapUtils.getGeomFromEWKTAndProj(elem, mapProj) });
            });
            this.vectorSource.addFeatures([
              this.feature,
              ...disectedFeature
            ]);
          }
          this.mapToolsService.refreshLayer(this.map, this.oLayer.layer_id, true);
          this.subscribeFeature.next(this.feature);
          this.subscribeDisectionFeature.next(disectedFeature);
        })
        .catch(() => {
          this.vectorSource.removeFeature(evt.feature);
          this.subscribeFeature.next(this.feature);
        });
    });
  };

  /**
   * Event appelé lors de la validation du formulaire 'Modifier les coordonnées'
   * @param event
   */
  private createModifyCoordGeometry(event) {
    let sEWKTGeom;
    switch (event.format) {
      case 'WKT':
        if (event.wkt) {
          sEWKTGeom = this.mapUtils.getEWKTFromWKT({ proj: event.projection, geom: event.wkt });
        }
        break;
      case 'Geojson':
        if (event.geojson) {
          sEWKTGeom = this.mapUtils.getEWKTFromGeojson(JSON.parse(event.geojson), event.projection);
        }
        break;
      case 'XY':
        if (event.longitude && event.latitude) {
          const coordinates = transform([event.longitude, event.latitude], this.map.getView().getProjection(), event.projection);
          sEWKTGeom = this.mapUtils.getEWKTFromGeojson({
            type: event.type,
            coordinates
          }, event.projection);
        }
        break;
    }
    if (sEWKTGeom) {
      const geometry: any = this.mapUtils.getGeomFromEWKT(sEWKTGeom, this.map);
      this.feature = new Feature({ geometry });
      this.subscribeFeature.next(this.feature);
      this.mapUtils.centerMapOnGeom(this.map, this.feature.getGeometry());
    }
  }

  /**
   * Supprime les interactions passées en paramètre
   * @param interactionsToDel Tableau avec les interactions à supprimer de la carte
   */
  public deleteCurrentInteraction(interactionsToDel: Array<string> = []): void {
    if (this.listenerKeyInteractionModify) {
      unByKey(this.listenerKeyInteractionModify);
    }
    if (this.listenerKeyInteractionAddFeature) {
      unByKey(this.listenerKeyInteractionAddFeature);
    }
    if (this.listenerKeyInteractionRemoveFeature) {
      unByKey(this.listenerKeyInteractionRemoveFeature);
    }
    if (this.listenerKeyInteractionRemoveVertex) {
      unByKey(this.listenerKeyInteractionRemoveVertex);
    }
    this.vmapToolsSetCircleRadiusService.eventDrawstartUnbind();
    this.vmapToolsSetCircleRadiusService.eventInnerDrawstartUnbind();
    if (this.vectorSource) {
      this.vectorSource.clear();
    }
    if (interactionsToDel.includes('select') && this.interactionRemove) {
      this.map.removeInteraction(this.interactionSelect);
      this.interactionSelect = null;
      if (this.vectorLayer) {
        this.map.removeLayer(this.vectorLayer);
        this.vectorLayer = null;
      }
    }
    if (interactionsToDel.includes('modify') && this.interactionModify) {
      this.map.removeInteraction(this.interactionModify);
      this.interactionModify = null;
    }
    if (interactionsToDel.includes('add') && this.interactionAdd) {
      this.map.removeInteraction(this.interactionAdd);
      this.interactionAdd = null;
    }
    if (interactionsToDel.includes('remove') && this.interactionRemove) {
      this.map.removeInteraction(this.interactionRemove);
      this.interactionRemove = null;
    }
    if (interactionsToDel.includes('snap') && this.interactionSnap) {
      this.map.removeInteraction(this.interactionSnap);
      this.interactionSnap = null;
      if (this.vectorLayerSnap) {
        this.vectorLayerSnap.setStyle(null);
        this.map.removeLayer(this.vectorLayerSnap);
        this.vectorLayerSnap = null;
      }
    }
    if (interactionsToDel.includes('removeVertex') && this.interactionRemoveVertex) {
      this.map.removeInteraction(this.interactionRemoveVertex);
      this.map.removeLayer(this.removeVertexPointsLayer);
      this.interactionRemoveVertex = null;
    }
  }

  /**
   * Récupère les zones d'accrochage de la visualisation en cous (carte extent)
   */
  public loadSnappingDatas() {
    this.isLoadingSnap = true;
    if (this.vectorLayerSnap) {
      // Supprime les features actuelles dans le vectorlayer du snap
      const features = this.vectorLayerSnap.getSource().getFeatures();
      if (features.length > 0) {
        features.forEach(feature => this.vectorLayerSnap.getSource().removeFeature(feature));
      }
    }

    if (this.xhrSnap) {
      this.currXhrExist.next(true);
    }

    const extent = this.map.getView().calculateExtent(this.map.getSize());
    const polygonGeom = fromExtent(extent);
    const sEWKTGeometry = this.mapUtils.getEWKTFromGeom(polygonGeom, this.map);

    // Buffer d'intersection
    const intersect_buffer = this.mapSelectService.getRealScaleBuffer_(this.oLayer, sEWKTGeometry, this.map);

    // Geométrie à intersecter
    const intersect_geom = sEWKTGeometry;

    // Projection en cours
    const result_srid = this.map.getView().getProjection().getCode().substring(5);

    // Paramètres
    const oParams = {
      limit: this.limitFeaturesSnap,
      groupBy: true,
      idList: this.layersSnapping,
      d: Date.now(),
      get_geom: true,
      intersect_buffer,
      intersect_geom,
      result_srid
    };

    const this_ = this;
    this.xhrSnap = this.vitisRequest.ajaxRequest({
      method: 'GET',
      url: `${environment.properties.app_api_url}/vmap/layers/query/layer`,
      params: oParams,
      ajaxLoader: false,
      success: (resp) => {
        let geoms = [];
        if (resp && resp.data && resp.data.data) {
          resp = resp.data.data;
          Object.keys(resp).forEach(layerName => {
            const geomfield = this_.mapSelectService.getSelectLayers().find(layer => layer.name === layerName).geomfield;
            geoms = [...geoms, ...resp[layerName].data.map(data => data[geomfield])];
          });
        }
        // Ajoute les nouvelles features dans le vectorlayer du snap
        this_.featuresSnap = geoms.map(geom => new Feature({ geometry: this_.mapUtils.getGeomFromEWKT(geom, this_.map) }));
        this_.vectorLayerSnap.getSource().addFeatures(this_.featuresSnap);
        this_.isLoadingSnap = false;
        return;
      },
      error: (resp) => {
        this_.isLoadingSnap = false;
      },
      abord: this.currXhrExist
    });
  }

  /**
   * Charge le formulaire à afficher pour les zones d'accrochage ou pour la modifications des coordonnées
   * @param formProperties
   */
  private loadForm(formProperties) {
    const { sDefinition, sFormName, sDatasForm } = formProperties;
    this.sDefinition = this.vitisClone.clone(sDefinition);
    this.sFormName = this.vitisClone.clone(sFormName);
    this.sDatasForm = this.vitisClone.clone(sDatasForm);
  }

  /**
   * Ferme le formulaire pour les zones d'accochages ou pour la modifications des coordonnées
   */
  public sendCloseForm() {
    this.geomEvent.next(this.mode === 'modify' ? 'modify_object' : 'add_object');
  }

  /**
   * Valide le formulaire pour les zones d'accrochage ou pour la modifications des coordonnées
   * @param event
   */
  public sendUpdateForm(event) {
    if (this.geomEventValue === 'snap') {
      this.createSnap(event);
    } else if (this.geomEventValue === 'modify_coord_object') {
      this.createModifyCoordGeometry(event);
    }
    this.geomEvent.next(this.mode === 'modify' ? 'modify_object' : 'add_object');
  }

  /**
   * Supprime toute les interactions de la cartes + la feature en cours et les subcribe
   */
  private deleteAllInteraction() {
    if (this.feature) {
      this.feature = null;
    }
    this.deleteCurrentInteraction(['snap', 'select', 'modify', 'add', 'remove', 'removeVertex']);
    if (this.geomEventSubscribe) {
      this.geomEventSubscribe.unsubscribe();
    }
    if (this.selectModeSubscribe) {
      this.selectModeSubscribe.unsubscribe();
    }
    if (this.rightPanelEmitterSubscribe) {
      this.rightPanelEmitterSubscribe.unsubscribe();
    }
  }

  /**
   * Ouvre la liste des différents géométrie qu'on peut ajouter
   * si la couche est de type Géométrie
   */
  public openDropdownGeomtypesList() {
    document.getElementById(this.listGeomtypesId).classList.toggle('show');
  }

  /**
   * Change le type de géométrie à ajouter si c'est une couche de type Géométrie
   * @param geomType point | linestring | polygon
   */
  public changeGeomType(geomType: string) {

    let isGeometryLayerMultiPolygon = false;
    // En mode GEOMETRY et multigeom
    if (this.isGeometryLayer && this.isMultiGeom) {
      // isGeometryLayerPolygon est variable en fonction du geomType passé en paramètre et du précédent geomType
      if (this.checkSpecialPolygonType(geomType) == 'polygon' && this.checkSpecialMultiPolygonType(this.geomType) == 'multipolygon') {
        isGeometryLayerMultiPolygon = true;
      }
    }

    // Si ce n'est un poylgon (simple ou spécifique) multiple on ne garde pas la feature
    if (!(this.isPolygon && this.isMultiGeom) && !isGeometryLayerMultiPolygon && this.feature) {
      this.feature = null;
    }

    this.deleteCurrentInteraction(['add', 'select', 'modify', 'remove', 'removeVertex']);
    if (this.isMultiGeom) {
      geomType = `MULTI${geomType}`;
    }
    this.geomType = geomType;
    if (this.isPolygon) {
      document.getElementById(this.listGeomtypesPolyId).classList.remove('show');
    } else {
      document.getElementById(this.listGeomtypesId).classList.remove('show');
    }

    if (this.geomEventValue === 'add_object') {
      this.initDrawing();
      this.addGeometry();
    } else {
      this.geomEvent.next('add_object');
    }
  }

  /**
   * Assigne le type de geométrie en fonction du type présent dans la table
   */
  private getGeometryTableField() {
    this.vitisRequest.ajaxRequestPromise({
      method: 'GET',
      url: `${environment.properties.app_api_url}/vmap/layers/query/fields`,
      params: {
        database: this.oLayer.database,
        schema: this.oLayer.schema,
        table: this.oLayer.table ? this.oLayer.table : this.oLayer.pg_table
      }
    }).then(resp => {
      if (resp && resp.data && resp.data.length > 0) {
        const field = resp.data.find(e => e.name === this.oLayer.geomfield);
        this.geomType = field ? field.geometry_type : null;

        if (this.geomType === 'GEOMETRY') {
          this.isGeometryLayer = true;
          this.geomType = 'POINT';
        }

        if (this.geomType.toLowerCase().includes('multi')) { this.isMultiGeom = true; }
        this.geomEvent.next(this.mode === 'modify' ? 'modify_object' : 'add_object');
      }
    }).catch(e => {
      console.error(e);
      this.translate.get('ERROR_INSERT_LAYER_OBJECT').subscribe(async (translation) => {
        vitisAlert.alert({
          text: translation,
          showCancelButton: true
        });
      });
    });
  }

  /**
   * Ouvre la liste des différents types de géométrie disponible pour le polygon
   * si la couche est de type Polygon
   */
  public openDropdownGeomPolygontypesList() {
    document.getElementById(this.listGeomtypesPolyId).classList.toggle('show');
  }

  /**
   * Si la geomtype passée en paramètre est un element particulier de "polygon", retourne la valeur "polygon"
   * @param geomType
   */
  private checkSpecialPolygonType(geomType: string): string {
    return this.aSpecialTypePolygon.includes(geomType.toLowerCase()) ? 'polygon' : geomType.toLowerCase();
  }

  /**
   * Si la geomtype passée en paramètre est un element particulier de "polygon" et qu'on est en multigeometry alors retourne la valeur "multipolygon"
   * @param geomType
   */
  private checkSpecialMultiPolygonType(geomType: string): string {
    geomType = this.checkSpecialPolygonType(this.isMultiGeom ? this.geomType.toLowerCase().substring(5) : this.geomType.toLowerCase());
    return this.isMultiGeom ? 'multi' + geomType : geomType;
  }

  /**
   * Lance le dessin du cercle quand on lui a spécifier une valeur dans la popup
   * @param feature
   */
  public setCircle(feature: Feature) {
    this.vectorSource.addFeature(feature);
    this.deleteCurrentInteraction(['add', 'select', 'modify', 'remove', 'removeVertex']);
    this.initDrawing();
    this.addGeometry();
  }

  /**
   * Suppression d'un vertex.
   * @returns void
   */
  private removeVertex(): void {

    // Ajoute la couche qui contiendra les sommets
    this.map.addLayer(this.removeVertexPointsLayer);
    const { interaction, select, snap } = this.mapCoreDrawService.addDrawInteraction('removeVertex', this.map, this.vectorLayer, {
      feature: this.feature,
      pointsLayer: this.removeVertexPointsLayer,
      getInteractions: true,
      onRemoveVertex: this.subscribeFeature,
    });
    this.interactionRemoveVertex = interaction;
  }
}
