import { Component, Input, OnInit, AfterViewInit, OnDestroy } from '@angular/core';
import { VitisCheck, VitisClone, VitisRequest, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { VmapSelectService, MapSelection } from 'src/modules/vmap/components/carto/services/vmap-select/vmap-select.service';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { environment } from 'src/environments/environment';
import { ActivatedRoute, Router } from '@angular/router';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { MapCorePopupService } from 'src/app/vitis/map-core/services/map-core-popup/map-core-popup.service';
import Feature from 'ol/Feature';
import { TranslateService } from '@ngx-translate/core';
import { VmapLayer } from 'src/app/vitis/map-core/services/map-models.model';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';
import { MapCoreDrawService } from 'src/app/vitis/map-core/services/map-core-draw/map-core-draw.service';
import { Subscription } from 'rxjs';
import { IsSmallScreenService } from 'src/app/vitis/core/services/utils/is-small-screen/is-small-screen.service';

declare let $: any;

@Component({
  selector: 'app-vmap-tools-select-right-panel',
  templateUrl: './vmap-tools-select-right-panel.component.html',
  styleUrls: ['./vmap-tools-select-right-panel.component.scss']
})
export class VmapToolsSelectRightPanelComponent implements OnInit, AfterViewInit, OnDestroy {

  @Input() map;

  public bOpen;
  public bOpenMobile;

  public requestorOpen: boolean = false;
  public requestorOpenMobile: boolean = false;

  private oParamsId;
  private oParamsLayerId;

  public formName = 'display';
  private mode;
  public selection: any | VmapLayer | Array<MapSelection>;
  public indexSelection;

  public formSize;
  public pixelFormSize: number = 0;

  public requestorFormSize: number = 0;

  public isSmallScreen;
  public isFullScreen = false;

  public isGeomExist: boolean = false;


  // Subscriptions
  private selectLayersSubscription: Subscription;
  private selectionSubscribe: Subscription;
  private selectModeSubscribe: Subscription;

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  /**
   * Constructeur
   */
  constructor(
    private mapSelectService: VmapSelectService,
    private vitisCheck: VitisCheck,
    private mapToolsService: VmapToolsService,
    private vitisClone: VitisClone,
    private vitisRequest: VitisRequest,
    private route: ActivatedRoute,
    private router: Router,
    private mapUtils: MapCoreUtilsService,
    private mapPopup: MapCorePopupService,
    private translate: TranslateService,
    private mapManagerService: MapCoreManagerService,
    private isSmallScreenService: IsSmallScreenService,
    private mapCoreDrawService: MapCoreDrawService
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {

    this.isSmallScreen = this.isSmallScreenService.getIsSmallScreen();
    this.subscriptions.push(this.isSmallScreenService.isSmallScreenObservable.subscribe(isSmallScreen => {
      this.isSmallScreen = isSmallScreen;
    }));

    // Traductions (obligatoire sinon ne fonctionne pas dans le module en lazy loading)
    this.translate.addLangs(['en', 'fr']);
    this.translate.setDefaultLang('en');
    this.translate.use(environment.properties.language ? environment.properties.language : 'en');

    this.subscriptions.push(this.route.data.subscribe(() => {
      this.subscriptions.push(this.route.queryParams.subscribe((oParams: any) => {
        this.oParamsId = oParams.id;
        this.oParamsLayerId = oParams.layer_id;
      }));
    }));

    this.getSelectionMode();
    this.map.on('change', () => {
      if (this.selectionSubscribe) {
        this.selectionSubscribe.unsubscribe();
      }
      if (this.selectModeSubscribe) {
        this.selectModeSubscribe.unsubscribe();
      }
      this.getSelectionMode();
    });

    this.subscriptions.push(this.mapToolsService.getCurrentOpenedRightPanel().subscribe(panel => {
      if (panel === 'tool-select-right-panel') {
        if (this.isSmallScreen) {
          this.bOpenMobile = true;
        } else {
          this.bOpen = true;
        }
      } else {
        this.bOpenMobile = false;
        this.bOpen = false;
        this.mapSelectService.closeSelections();
        this.clearUrl_();

        // Nettoyage de la couche des sélections
        if (this.map.get('select_layer')) {
          this.mapCoreDrawService.addDrawInteraction('removeAll', this.map, this.map.get('select_layer'));
        }
      }
    }));

    this.subscriptions.push(this.mapToolsService.getBottomPanelEmiter().subscribe(size => {
      this.requestorFormSize = size == 0 ? 0 : 500;
      if (this.selection &&
        (this.selection[0] || this.selection[this.indexSelection]) &&
        (this.selection[0].data || this.selection[this.indexSelection].data) &&
        (this.selection[0].data.geom || this.selection[this.indexSelection].data.geom)) {
        this.mapUtils.centerMapOnGeom(this.map, this.mapUtils.getGeomFromEWKT(this.mode === 'multiedit' ? this.selection[0].data.geom : this.selection[this.indexSelection].data.geom, this.map), [0, this.pixelFormSize, this.requestorFormSize, 0], true);
      }
    }));

    this.subscriptions.push(this.mapManagerService.mapVisibilityChange.subscribe((change: any) => {
      const oLayerId = change.oLayer.layer_id;
      if ((this.oParamsLayerId && (this.oParamsLayerId == oLayerId)) ||
        (this.selection && (this.selection.layer_id == oLayerId ||
          (this.selection[this.indexSelection] &&
            this.selection[this.indexSelection].oLayer.layer_id === oLayerId)))) {
        this.closeSelection();

        // Nettoyage de la couche des sélections
        if (this.map.get('select_layer')) {
          this.mapCoreDrawService.addDrawInteraction('removeAll', this.map, this.map.get('select_layer'));
        }
      }
    }));

  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
    if (this.selectionSubscribe) {
      this.selectionSubscribe.unsubscribe();
    }
    if (this.selectModeSubscribe) {
      this.selectModeSubscribe.unsubscribe();
    }
    if (this.selectLayersSubscription) {
      this.selectLayersSubscription.unsubscribe();
    }
  }

  /**
   * Récupère le mode et charge la selection en fonction de ce dernier
   */
  private getSelectionMode() {
    this.selectionSubscribe = this.mapSelectService.getSelectionObs().subscribe((selection: any) => {
      if (this.mode) {
        let init = false;
        switch (this.mode) {
          case 'add': // Event quand on veut ajouter un object sur une carte
            if (selection) {
              init = true;
            }
            break;
          case 'modify': // Event quand on modifie un object sur une carte
            if (selection && selection.length) {
              this.indexSelection = 0;
              init = true;
            }
            break;
          case 'multiedit': // Event quand on modifie plusieurs objets sur une carte
            if (selection) {
              init = true;
            }
            break;
        }
        if (init) {
          this.openSelection(selection);
        } else {
          this.closeSelection();
        }
      }
    });

    this.selectModeSubscribe = this.mapSelectService.getSelectModeObs().subscribe((mode: string) => {
      this.mode = mode;
    });
  }

  /**
   * Appelé après le chargement du DOM
   */
  ngAfterViewInit(): void {
    this.initSelection();
    this.map.on('change', () => {
      if (this.selectLayersSubscription) {
        this.selectLayersSubscription.unsubscribe();
      }
      this.initSelection();
    });
  }

  /**
   * Initialise la selection en cours si elle existe dans l'url
   */
  private initSelection() {
    this.selectLayersSubscription = this.mapSelectService.getObservableSelectLayers().subscribe(aSelectLayers => {
      // Permet de ne lancer qu'une fois
      if (this.selectLayersSubscription) {
        this.selectLayersSubscription.unsubscribe();
      }
      const initLayer = this.oParamsLayerId ? aSelectLayers.find(layer => layer.layer_id == +this.oParamsLayerId) : aSelectLayers.find(layer => !layer.layer_id);
      if (initLayer) {
        if (initLayer.layer_id && this.oParamsId) {
          this.vitisRequest.ajaxRequestPromise({
            method: 'GET',
            httpMethodOverride: true,
            url: `${environment.properties.app_api_url}/vmap/layers/${initLayer.layer_id}/query`,
            params: {
              filter: {
                relation: 'AND',
                operators: [{
                  column: initLayer.idfield,
                  compare_operator: '=',
                  value: this.oParamsId,
                  compare_operator_options: { 'case_insensitive': true }
                }]
              },
              result_srid: this.map.getView().getProjection().getCode().substring(5),
            }
          }).then((resp) => {
            if (resp && resp.data) {
              const aSelections = [];
              const geometries = [];
              resp.data.data.forEach((element, index) => {
                let aCoordinates: any = [];
                if (element && element[initLayer.geomfield] && this.vitisCheck.isDefAndNotNull(element[initLayer.geomfield])) {
                  const geom = this.mapUtils.getGeomFromEWKT(element[initLayer.geomfield], this.map);
                  if (geom) {
                    geometries[index] = geom;
                    aCoordinates = geometries[index].getCoordinates();
                    while (typeof aCoordinates[0] != 'number') {
                      aCoordinates = aCoordinates[0];
                    }
                  } else {
                    geometries[index] = null;
                    aCoordinates = [];
                  }
                } else {
                  geometries[index] = null;
                  aCoordinates = [];
                }
                aSelections.push({
                  data: element,
                  clickCoords: aCoordinates,
                  oLayer: initLayer,
                });
              });

              this.pixelFormSize = aSelections[0] && aSelections[0].oLayer && aSelections[0].oLayer.form_display_size_id ? this.mapToolsService.getDecimalFormSize(aSelections[0].oLayer.form_display_size_id) : this.mapToolsService.getDecimalFormSize(1);
              this.mapSelectService.setSelection(aSelections);

              // Affiche les popups
              this.mapSelectService.displayPopups(this.map, aSelections);

              if (geometries.length) {
                if (this.vitisCheck.isDefAndNotNull(geometries[0])) {
                  this.mapUtils.centerMapOnGeom(this.map, geometries[0], this.mapToolsService.getCenterPadding());
                }
                this.mapSelectService.setSelectMode('modify');
                this.indexSelection = 0;
                this.openSelection(aSelections);
              }
            }
          }).catch(e => {
            console.error(e);
            this.translate.get('ERROR_GET_LAYER_OBJECT').subscribe(async (translation) => {
              vitisAlert.alert({
                text: translation,
                showCancelButton: true
              });
            });
          });
        }
      }
    });
  }

  /**
   * Ferme la sélection en cours
   */
  public closeSelection() {
    this.mapToolsService.setRightPanel(0, '');
    this.isFullScreen = false;
  }

  /**
   * Ouvre le panel de droite et affiche le contenu pour modifier ou ajouter un object
   * @param selection
   */
  public openSelection(selection: any | VmapLayer | Array<MapSelection>) {
    this.selection = selection;
    this.isGeomExist = this.vitisCheck.isDefAndNotNull(this.selection) &&
      this.vitisCheck.isDefAndNotNull(this.indexSelection) &&
      this.vitisCheck.isDefAndNotNull(this.selection[this.indexSelection]) &&
      this.vitisCheck.isDefAndNotNull(this.selection[this.indexSelection].data) &&
      this.vitisCheck.isDefAndNotNull(this.selection[this.indexSelection].data.geom) &&
      this.mapUtils.getGeomFromEWKT(this.selection[this.indexSelection].data.geom, this.map) ? true : false;

    let formSize = 1;
    if (this.mode === 'modify') {
      if (selection && selection[this.indexSelection]) {
        const aSelection = selection[this.indexSelection];
        if (aSelection.oLayer) {
          // Si une taille de formulaire est fournie
          if (this.vitisCheck.isNumber(aSelection.oLayer.form_display_size_id)) {
            formSize = aSelection.oLayer.form_display_size_id;
          }

          if (aSelection.popupParams) {
            const { sPopupData, olPopupGeom } = aSelection.popupParams;
            aSelection.popup = this.mapPopup.displayPopup(sPopupData, new Feature({ geometry: olPopupGeom }), this.map, { hideCloseBtn: true });
          }
          const object_id = aSelection.data[aSelection.oLayer.idfield];
          // Met à jour l'url si l'id sélectionné est différent de celui dans l'url
          if (this.oParamsId != object_id) {
            this.router.navigate([],
              {
                relativeTo: this.route,
                queryParams: { layer_id: aSelection.oLayer.layer_id, id: object_id },
                queryParamsHandling: 'merge'
              });
          }
        }
      }
    } else if (this.mode === 'multiedit') {
      if (selection && selection[0]) {
        // Si une taille de formulaire est fournie
        if (this.vitisCheck.isNumber(selection[0].oLayer.form_update_size_id)) {
          formSize = selection[0].oLayer.form_update_size_id;
        }
      }
    } else {
      if (selection) {
        // Si une taille de formulaire est fournie
        if (this.vitisCheck.isNumber(selection.form_insert_size_id)) {
          formSize = selection.form_insert_size_id;
        }
      }
    }
    this.bOpen = true;
    this.isFullScreen = false;
    this.formSize = formSize;
    this.mapToolsService.closeTools(this.map, {
      bExceptBottom: true,
    });

    this.pixelFormSize = this.selection[0] && this.selection[0].oLayer && this.selection[0].oLayer.form_display_size_id ? this.mapToolsService.getDecimalFormSize(this.selection[0].oLayer.form_display_size_id) : this.mapToolsService.getDecimalFormSize(1);
    if (this.isGeomExist &&
      selection &&
      (selection[0] || selection[this.indexSelection]) &&
      (selection[0].data || selection[this.indexSelection].data) &&
      (selection[0].data.geom || selection[this.indexSelection].data.geom)) {
      this.mapUtils.centerMapOnGeom(this.map, this.mapUtils.getGeomFromEWKT(this.mode === 'multiedit' ? selection[0].data.geom : selection[this.indexSelection].data.geom, this.map), [0, this.pixelFormSize, this.requestorFormSize, 0], true);
    }
    this.mapToolsService.setRightPanel(formSize, 'tool-select-right-panel');
  }

  /**
   * Formate la sélection pour qu'elle soit plus facielement exploitable par la vue
   * @param oSelection
   * @returns
   */
  public getFormatedSelection(oSelection: any): any {
    const formatView = {
      title: '',
      selectionDataTitle: ''
    };
    switch (this.mode) {
      case 'add':
        formatView.title = oSelection.form_title;
        break;
      case 'modify':
        const selection = oSelection[this.indexSelection];
        if (selection) {
          formatView.title = selection.oLayer.form_title;
          formatView.selectionDataTitle = selection.data[selection.oLayer.idfield];
        }
        break;
      case 'multiedit':
        if (oSelection[0]) {
          formatView.title = oSelection[0].oLayer.form_title;
        }
        break;
    }
    return formatView;
  }

  /**
   * Récupère le nom du formulaire actuel ouvert dans l'enfant
   * @param event
   */
  public subscribeFormName(event) {
    this.formName = event;
  }

  /**
   * Fonction appelée pour passer à la sélection précédente
   */
  public changeToLeftSelection() {
    if (this.selection[this.indexSelection].popup) {
      this.selection[this.indexSelection].popup.close();
    }

    this.selection[this.indexSelection].popup = null;

    if (this.indexSelection === 0) {
      this.indexSelection = this.selection.length - 1;
    } else {
      this.indexSelection = this.indexSelection - 1;
    }

    this.openSelection(this.selection);
  }

  /**
   * Fonction appelée pour passer à la sélection suivante
   */
  public changeToRightSelection() {
    if (this.selection[this.indexSelection].popup) {
      this.selection[this.indexSelection].popup.close();
    }

    this.selection[this.indexSelection].popup = null;

    if (this.indexSelection === this.selection.length - 1) {
      this.indexSelection = 0;
    } else {
      this.indexSelection = this.indexSelection + 1;
    }

    this.openSelection(this.selection);
  }

  /**
   * Met à jour les coordonnées de la popup
   * @param event
   */
  public updatePopup(event) {
    let selections;
    if (event === null) {
      this.selection[this.indexSelection].popup.close();
      this.selection[this.indexSelection].popup = null;
      selections = [...this.selection.slice(0, this.indexSelection), ...this.selection.slice(this.indexSelection + 1, this.selection.length)];
    } else {
      this.selection[this.indexSelection].data = event;
      if (this.selection[this.indexSelection].oLayer.is_geometry_layer) {
        const geom = this.mapUtils.getGeomFromEWKT(event[this.selection[this.indexSelection].oLayer.geomfield], this.map);
        this.selection[this.indexSelection].clickCoords = geom.getCoordinates();
        if (this.vitisCheck.isDefAndNotNull(this.selection[this.indexSelection].popup)) {
          this.selection[this.indexSelection].popup.close();
        }
      }
      this.selection[this.indexSelection].popup = null;
      selections = this.vitisClone.clone(this.selection);
      this.closeSelection();
    }

    this.mapSelectService.setSelection(selections);
    if (this.selection[this.indexSelection].oLayer.is_geometry_layer) {
      this.mapSelectService.displayPopups(this.map, selections);
    }
  }

  /**
   * Set la taille du formulaire
   * @param event
   */
  public subscribeFormSize(event: number) {
    this.formSize = event;
  }

  /**
   * Nettoie le layer_id et id de l'URL
   */
  private clearUrl_() {
    this.router.navigate([],
      {
        relativeTo: this.route,
        queryParams: { layer_id: null, id: null },
        queryParamsHandling: 'merge'
      });
  }

  /**
   * Toggle en mode fullscreen
   */
  public toggleFullScreen() {
    this.isFullScreen = !this.isFullScreen;
  }
}
