import { Component, OnInit, Input, Inject, Optional, ViewContainerRef, EventEmitter, OnDestroy, createNgModuleRef, Injector, ViewChild, createNgModule } from '@angular/core';
import { Map } from 'ol';
import { nativeMapToolsDef } from './map-tools-buttons';
import { MapToolDef } from './map-tools.model';
import { Subject, Subscription } from 'rxjs';
import { DashboardComponent } from 'src/app/vitis/core/components/dashboard/dashboard.component';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { VitisCheck, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { DomSanitizer, SafeHtml } from '@angular/platform-browser';
import { environment } from 'src/environments/environment';
import { IsSmallScreenService } from 'src/app/vitis/core/services/utils/is-small-screen/is-small-screen.service';

@Component({
  selector: 'app-vmap-tools',
  templateUrl: './vmap-tools.component.html',
  styleUrls: ['./vmap-tools.component.scss']
})
export class VmapToolsComponent implements OnInit, OnDestroy {

  @Input() olMap: Map;
  @Input() isWidget: boolean;
  @Input() targetCompareId: string;
  @Input() tileSize: Array<number>;
  @Input() haveHelperTool: boolean;

  public oMapTools: any;
  public oMapToolsDef: any;
  public oMapPanelTabs: any = {};
  public bSidenavOpen: boolean = false;
  public iLeftPanelOpen: number = 0; // 0, 1, 2, 3
  public iRightPanelOpen: number = 0; // 0, 1, 2, 3
  public iBottomPanelOpen: number = 0; // 0, 1, 2, 3

  public isSmallScreen;

  // Gère les panneaux déjà ouverts pour les remplacer si besoin
  public sCurrentOpenLeftPanel: string = '';
  public sCurrentOpenRightPanel: string = '';
  public sCurrentOpenBottomPanel: string = '';

  // Mémorise toutes les souscriptions de manière à les desouscrire à la destruction du composant
  private subscriptions: Subscription[] = [];

  @ViewChild('lazyLoadCadastre', { static: false, read: ViewContainerRef }) lazyLoadCadastre!: ViewContainerRef;
  @ViewChild('lazyLoadCadastreRelevePropriete', { static: false, read: ViewContainerRef }) lazyLoadCadastreRelevePropriete!: ViewContainerRef;
  @ViewChild('lazyLoadCadastreFicheDescr', { static: false, read: ViewContainerRef }) lazyLoadCadastreFicheDescr!: ViewContainerRef;
  @ViewChild('lazyLoadCadastreFicheUrba', { static: false, read: ViewContainerRef }) lazyLoadCadastreFicheUrba!: ViewContainerRef;

  /**
   * Constructeur
   */
  constructor(
    public viewContainer: ViewContainerRef,
    public vmapToolsService: VmapToolsService,
    private isSmallScreenService: IsSmallScreenService,
    private vitisCheck: VitisCheck,
    private vitisclone: VitisClone,
    private auth: AuthService,
    private domSanitizer: DomSanitizer,
    private injector: Injector,
    @Optional() @Inject(DashboardComponent) private dashboardComponent: DashboardComponent,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {
    // Chargement des outils
    this.vmapToolsService.loadToolsDef().then(oMapToolsDef => {
      this.oMapToolsDef = this.setToolsVisibility(oMapToolsDef);

      this.isSmallScreen = this.isSmallScreenService.getIsSmallScreen();
      this.subscriptions.push(this.isSmallScreenService.isSmallScreenObservable.subscribe(isSmallScreen => {
        this.isSmallScreen = isSmallScreen;
      }));

      this.initVmapTools();
      this.olMap.on('change', () => {
        this.initVmapTools();
      });

      // Barre latérale Vitis (non visible en mode mobile)
      if (this.dashboardComponent) {
        this.bSidenavOpen = this.dashboardComponent.sidenavOpen;

        // Observe l'état du sidenav
        this.subscriptions.push(this.dashboardComponent.sidenavOpenChange.subscribe((next: boolean) => {
          this.bSidenavOpen = next;
        }));
      }

      // Observe l'état des bandeaux
      this.subscriptions.push(this.vmapToolsService.getLeftPanelEmiter().subscribe((next: number) => {
        this.iLeftPanelOpen = next;
      }));
      this.subscriptions.push(this.vmapToolsService.getRightPanelEmiter().subscribe((next: number) => {
        this.iRightPanelOpen = next;
      }));
      this.subscriptions.push(this.vmapToolsService.getBottomPanelEmiter().subscribe((next: number) => {
        this.iBottomPanelOpen = next;
      }));

      // Observe l'état du panneau de gauche actuellement ouvert
      this.subscriptions.push(this.vmapToolsService.getCurrentOpenedLeftPanel().subscribe((next: string) => {
        if (this.sCurrentOpenLeftPanel !== '' && !this.oMapToolsDef.hasOwnProperty(next)) {
          this.vmapToolsService.closePanelAt(this.olMap, 'left');
        }

        this.sCurrentOpenLeftPanel = next;
      }));

      // Observe l'état du panneau de droite actuellement ouvert
      this.subscriptions.push(this.vmapToolsService.getCurrentOpenedRightPanel().subscribe((next: string) => {
        if (this.sCurrentOpenRightPanel !== '' && !this.oMapToolsDef.hasOwnProperty(next)) {
          this.vmapToolsService.closePanelAt(this.olMap, 'right');
        }

        this.sCurrentOpenRightPanel = next;
      }));

      // Observe l'état du panneau du bas actuellement ouvert
      this.subscriptions.push(this.vmapToolsService.getCurrentOpenedBottomPanel().subscribe((next: string) => {
        if (this.sCurrentOpenBottomPanel != '' && !this.oMapToolsDef.hasOwnProperty(next)) {
          this.vmapToolsService.closePanelAt(this.olMap, 'bottom');
        }

        this.sCurrentOpenBottomPanel = next;
      }));

      // important : set des identifiants uniques à chaque tool
      this.setToolsIds_();

      // set les events (ex : ouverture/fermeture de l'outil)
      this.setToolsEvents_();

      // set panels tabs
      this.setPanelsTabs();
    });
  }

  /**
   * Fonction de destruction
   */
  ngOnDestroy(): void {
    this.subscriptions.forEach(subscription => subscription.unsubscribe());
  }

  /**
   * Initialise les outils au rechargement d'une carte
   */
  private initVmapTools() {
    const oMapDef = this.olMap.get('map_definition');

    // Outils de la carte
    if (oMapDef && oMapDef.maptools && oMapDef.maptools.tools) {
      this.oMapTools = this.vitisclone.clone(oMapDef.maptools.tools);
    } else {
      this.oMapTools = {};
    }

    if (this.haveHelperTool) {
      if (this.vitisCheck.isDefAndNotNull(this.oMapTools.upright)) {
        this.oMapTools.upright.unshift('helper');
      } else {
        this.oMapTools.upright = ['helper'];
      }
    }

    setTimeout(() => {
      this.lazyLoadTools_(this.oMapTools, this.oMapToolsDef);
    });

    // Ferme les outils ouverts
    this.vmapToolsService.closeTools(this.olMap);
  }

  /**
   * Affiche / cache un sous-menu
   * @param oTool
   */
  public toggleToolMenu(oTool: MapToolDef) {
    if (oTool.collapsed_ === false) {
      this.vmapToolsService.closeTool(this.olMap, oTool.name, true);
    } else {
      this.vmapToolsService.openTool(this.olMap, oTool.name, true);
    }
  }

  /**
   * Affiche / cache un outil de type panel
   * @param oTool
   */
  public toggleToolPanel(oTool: MapToolDef) {
    if (oTool.collapsed_ === false) {
      this.vmapToolsService.closeTool(this.olMap, oTool.name, true);
    } else {
      this.vmapToolsService.openTool(this.olMap, oTool.name, true);
    }
  }

  /**
   * Permet de changer la disposition d'un outil de type panel
   * @param oTool Outil
   * @param sPosition Nouvelle position
   * @param iPanelSize Nouvelle taille
   */
  public movePanelTo(oTool: MapToolDef, sPosition: string, iPanelSize: number) {
    // Vérification si l'outil change de position
    if (oTool.panelPosition !== sPosition) {
      // Récupération du conteneur de l'outil et fermeture pour adaptation
      const element: any = document.getElementById(oTool.id).parentNode.parentNode;
      element.classList.remove('open');

      // Adaptation de l'interface
      this.vmapToolsService.setPanel(oTool.panelPosition, 0, oTool.name);

      setTimeout(() => {
        // Désactivation temporaire des transitions
        element.style.transition = 'none';
        oTool.panelPosition = sPosition;
        oTool.panelSize = iPanelSize;

        // Réactivation des transitions
        setTimeout(() => {
          element.style.transition = '';
          element.classList.add('open');

          // Adaptation de l'interface
          this.vmapToolsService.setPanel(oTool.panelPosition, oTool.panelSize, oTool.name);
        }, 200);
      }, 500);
    } else {
      oTool.panelSize = iPanelSize;

      // Adaptation de l'interface
      this.vmapToolsService.setPanel(oTool.panelPosition, oTool.panelSize, oTool.name);
    }
  }

  /**
   * Apelle l'event d'un outil de type action
   * @param oTool
   */
  public callToolEvent(oTool: MapToolDef) {
    if (oTool.event && this.vmapToolsService[oTool.event]) {
      this.vmapToolsService[oTool.event](this.olMap);
    }
  }

  /**
   * set des identifiants uniques à chaque tool
   */
  private setToolsIds_() {
    for (const key in this.oMapToolsDef) {
      if (Object.prototype.hasOwnProperty.call(this.oMapToolsDef, key)) {
        this.oMapToolsDef[key].id = `map-tools-component-${key}-${Date.now()}`;

        if (this.oMapToolsDef[key].hasTabs) {
          this.oMapToolsDef[key].tabsId = `map-panel-tabs-${key}-${Date.now()}`;
        }
      }
    }
  }

  /**
   * set les events (ex : ouverture/fermeture de l'outil)
   */
  private setToolsEvents_() {
    for (const key in this.oMapToolsDef) {
      if (Object.prototype.hasOwnProperty.call(this.oMapToolsDef, key)) {
        this.oMapToolsDef[key].onOpen = new Subject<boolean>();
        this.oMapToolsDef[key].onClose = new Subject<boolean>();
        this.oMapToolsDef[key].open = () => {
          this.vmapToolsService.openTool(this.olMap, key, false);
        };
        this.oMapToolsDef[key].close = () => {
          this.vmapToolsService.closeTool(this.olMap, key, false);
        };
      }
    }
  }

  /**
   * Initialise les onglets des panneaux en comportant
   */
  private setPanelsTabs() {

    // si oMaptools n'est pas présent alors il n'y a rien à charger
    if (!this.oMapTools) {
      return;
    }

    for (const key in this.oMapToolsDef) {
      if (Object.prototype.hasOwnProperty.call(this.oMapToolsDef, key)) {
        const oTool: MapToolDef = this.oMapToolsDef[key];
        if (oTool.hasTabs && oTool.tabsId) {
          if (!this.vitisCheck.isDefAndNotNull(this.oMapPanelTabs[oTool.tabsId])) {
            this.oMapPanelTabs[oTool.tabsId] = {
              oTabsEmitter: new EventEmitter<Array<any>>(),
              oSelectedTabEmitter: new EventEmitter<number>()
            };
          }
        }
      }
    }
  }

  /**
   * Définition de la visibilité des outils.
   * @param  {any} oMapToolsDef
   */
  private setToolsVisibility(oMapToolsDef: any) {
    for (const key in oMapToolsDef) {
      if (Object.prototype.hasOwnProperty.call(oMapToolsDef, key)) {
        // L'outil est visible par défaut.
        if (!this.vitisCheck.isDefAndNotNull(oMapToolsDef[key].visible)) {
          oMapToolsDef[key].visible = true;
        }
        // Affichage suivant les privilèges de l'utilisateur.
        if (this.vitisCheck.isDefAndNotNull(oMapToolsDef[key].privilege)) {
          if (this.vitisCheck.isString(oMapToolsDef[key].privilege)) {
            oMapToolsDef[key].privilege = [oMapToolsDef[key].privilege];
          }
          const aUserPrivileges = this.auth.getPrivileges();
          const aMatch = aUserPrivileges.filter(item => oMapToolsDef[key].privilege.indexOf(item) !== -1);
          if (aMatch.length == 0) {
            oMapToolsDef[key].visible = false;
          }
        }
      }
    }
    return oMapToolsDef;
  }

  /**
   * Vérification pour la sécurité car la valeur vient de la bdd
   * @param html
   */
  public trustHTML(html: string): SafeHtml {
    return this.domSanitizer.bypassSecurityTrustHtml(html);
  }

  /**
   * Charge les outils en lazyLoading
   * @param oMapTools
   * @param oMapToolsDef
   */
  private async lazyLoadTools_(oMapTools: any, oMapToolsDef: any) {
    for (const pos in oMapTools) {
      if (Object.prototype.hasOwnProperty.call(oMapTools, pos)) {
        if (this.vitisCheck.isArray(oMapTools[pos])) {
          for (const sToolname of oMapTools[pos]) {
            if (oMapToolsDef[sToolname]) {
              this.lazyLoadTool_(oMapToolsDef[sToolname]);
            }
          }
        }
      }
    }
  }

  /**
   * Charge l'outil en lazyLoading
   * @param oMapTool
   */
  private async lazyLoadTool_(oMapTool: any) {
    if (oMapTool.component === 'app-vmap-tools-cadastre' &&
      this.lazyLoadCadastre) {
      const { VmapToolsCadastreModule } = await import('./components/vmap-tools-cadastre/vmap-tools-cadastre.module');
      const { VmapToolsCadastreComponent } = await import('./components/vmap-tools-cadastre/vmap-tools-cadastre.component');
      this.lazyLoadCadastre.clear();
      const { instance } = this.lazyLoadCadastre.createComponent(VmapToolsCadastreComponent, {
        injector: this.injector,
        ngModuleRef: createNgModule(VmapToolsCadastreModule, this.injector),
      });
      instance.olMap = this.olMap;
      instance.oTool = oMapTool;
    }
    if (oMapTool.component === 'app-vmap-cadastre-tools-releve-de-propriete' &&
      this.lazyLoadCadastreRelevePropriete) {
      const { VmapToolsCadastreModule } = await import('./components/vmap-tools-cadastre/vmap-tools-cadastre.module');
      const { VmapCadastreToolsReleveDeProprieteComponent } = await import('./components/vmap-tools-cadastre/vmap-cadastre-tools-releve-de-propriete/vmap-cadastre-tools-releve-de-propriete.component');
      this.lazyLoadCadastreRelevePropriete.clear();
      const { instance } = this.lazyLoadCadastreRelevePropriete.createComponent(VmapCadastreToolsReleveDeProprieteComponent, {
        injector: this.injector,
        ngModuleRef: createNgModule(VmapToolsCadastreModule, this.injector),
      });
      instance.olMap = this.olMap;
      instance.oTool = oMapTool;
    }
    if (oMapTool.component === 'app-vmap-cadastre-tools-fiche-urbanisme' &&
      this.lazyLoadCadastreFicheUrba) {
      const { VmapToolsCadastreModule } = await import('./components/vmap-tools-cadastre/vmap-tools-cadastre.module');
      const { VmapCadastreToolsFicheUrbanismeComponent } = await import('./components/vmap-tools-cadastre/vmap-cadastre-tools-fiche-urbanisme/vmap-cadastre-tools-fiche-urbanisme.component');
      this.lazyLoadCadastreFicheUrba.clear();
      const { instance } = this.lazyLoadCadastreFicheUrba.createComponent(VmapCadastreToolsFicheUrbanismeComponent, {
        injector: this.injector,
        ngModuleRef: createNgModule(VmapToolsCadastreModule, this.injector),
      });
      instance.olMap = this.olMap;
      instance.oTool = oMapTool;
    }
    if (oMapTool.component === 'app-vmap-cadastre-tools-fiche-descriptive' &&
      this.lazyLoadCadastreFicheDescr) {
      const { VmapToolsCadastreModule } = await import('./components/vmap-tools-cadastre/vmap-tools-cadastre.module');
      const { VmapCadastreToolsFicheDescriptiveComponent } = await import('./components/vmap-tools-cadastre/vmap-cadastre-tools-fiche-descriptive/vmap-cadastre-tools-fiche-descriptive.component');
      this.lazyLoadCadastreFicheDescr.clear();
      const { instance } = this.lazyLoadCadastreFicheDescr.createComponent(VmapCadastreToolsFicheDescriptiveComponent, {
        injector: this.injector,
        ngModuleRef: createNgModule(VmapToolsCadastreModule, this.injector),
      });
      instance.olMap = this.olMap;
      instance.oTool = oMapTool;
    }
  }
}
