import { Injectable } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Map } from 'ol';
import { Extent } from 'ol/extent';
import { transformExtent } from 'ol/proj';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';

@Injectable({
  providedIn: 'root'
})
export class VmapCartoService {

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private router: Router,
    private route: ActivatedRoute,
    private mapCoreManagerService: MapCoreManagerService
  ) { }

  /**
   * Load une carte
   */
  public loadMap(map_id: string | number, olMap: Map) {
    // Map id de la carte.
    const oQueryParams: any = {
      map_id: map_id
    };

    // Récupère le catalogue de cartes.
    this.mapCoreManagerService.getMapCatalog().subscribe((oMapCatalog: any) => {
      // Garde l'étendue de la carte affichée pour celle qui a été sélectionnée.
      if (environment.properties.keep_map_extent === true) {
        if (this.vitisCheck.isDefAndNotNull(oMapCatalog) &&
        this.vitisCheck.isArray(oMapCatalog.maps) && oMapCatalog.maps.length > 0) {
          const oSelectedMap = oMapCatalog.maps.find((map) => map.map_id == map_id);
          if (this.vitisCheck.isDefAndNotNull(oSelectedMap)) {
            // Système de coordonnées et étendue de la carte affichée.
            const sCurrentProjCode = olMap.getView().getProjection().getCode();
            let extent: Extent = olMap.getView().calculateExtent();
            // Conversion vers le système de coordonnées de la carte sélectionnée.
            if (sCurrentProjCode != oSelectedMap.coordsys_epsg) {
              extent = transformExtent(extent, sCurrentProjCode, oSelectedMap.coordsys_epsg);
            }
            // Garde l'étendue de la carte affichée pour celle sélectionnée.
            oQueryParams.map_ext = extent.join('|');
          }
        }
      }
      // Change le map_id et l'étendue dans l'URL.
      this.router.navigate([], { relativeTo: this.route, queryParams: oQueryParams, queryParamsHandling: 'merge' });
    });
  }
}
