import { Injectable } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { TranslateService } from '@ngx-translate/core';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { vitisAlert, VitisCheck, VitisClone, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';
import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { MapCoreDrawService } from 'src/app/vitis/map-core/services/map-core-draw/map-core-draw.service';
import { MapCoreJSONService } from 'src/app/vitis/map-core/services/map-core-json/map-core-json.service';
import { MapCoreManagerService } from 'src/app/vitis/map-core/services/map-core-manager/map-core-manager.service';
import { MapCorePopupOverlayService } from 'src/app/vitis/map-core/services/map-core-popup-overlay/map-core-popup-overlay.service';
import { MapCorePopupService } from 'src/app/vitis/map-core/services/map-core-popup/map-core-popup.service';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { environment } from 'src/environments/environment';
import { VmapCartoService } from 'src/modules/vmap/components/carto/services/vmap-carto/vmap-carto.service';
import { VmapSelectService } from 'src/modules/vmap/components/carto/services/vmap-select/vmap-select.service';
import { VmapToolsGeolocationService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-geolocation/vmap-tools-geolocation.service';
import { VmapToolsMeasureService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-measure/vmap-tools-measure.service';
import { VmapToolsPrintService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-print/vmap-tools-print.service';
import { VmapToolsReportService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools-report/vmap-tools-report.service';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import exportFromJSON from 'export-from-json';
import * as echarts from 'echarts';
import moment from 'moment-timezone';
import * as shpjs from 'shpjs';

@Injectable({
  providedIn: 'root'
})
export class VmapFormService {

  /**
   * Constructeur
   */
  constructor(
    private mapCoreManagerService: MapCoreManagerService,
    private mapCoreJSONService: MapCoreJSONService,
    private mapCoreDrawService: MapCoreDrawService,
    private mapCorePopupService: MapCorePopupService,
    private mapCorePopupOverlayService: MapCorePopupOverlayService,
    private mapCoreUtilsService: MapCoreUtilsService,
    private mapSelectService: VmapSelectService,
    private mapCartoService: VmapCartoService,
    private mapToolsService: VmapToolsService,
    private mapToolsReportService: VmapToolsReportService,
    private mapToolsPrintService: VmapToolsPrintService,
    private mapToolsMeasureService: VmapToolsMeasureService,
    private mapToolsGeolocationService: VmapToolsGeolocationService,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private vitisClone: VitisClone,
    private translate: TranslateService,
    private formreaderSrvc: FormreaderService,
    private route: ActivatedRoute,
    private router: Router,
    private auth: AuthService,
    private modalService: NgbModal,
  ) { }

  /**
  * Charge le js du formulaire
  * @param published_form_js_def
  * @param iObjectId
  * @param oFormReader
  */
  public loadJs(published_form_js_def: string, iObjectId: string | number, oFormReader: FormreaderComponent, oScope: any) {

    // Supprime les js précédement chargés
    const oldDiv = document.getElementsByClassName('vmap-form-js');
    for (let i = 0; i < oldDiv.length; i++) {
      oldDiv[i].remove();
    }

    if (published_form_js_def) {
      const oScript = document.createElement('script');
      oScript.textContent = published_form_js_def;
      oScript.setAttribute('class', 'vmap-form-js');
      oScript.setAttribute('type', 'text/javascript');
      document.getElementsByTagName('head')[0].appendChild(oScript);
    }
    this.runJS(oFormReader, oScope);
  }

  /**
   * Execute le javascript
   * @param oFormReader
   */
  private runJS(oFormReader: FormreaderComponent, oScope: any) {
    // Lance le JS associé
    if ((<any>window).constructor_form) {
      (<any>window).constructor_form(oScope, environment, {
        router: this.router,
        route: this.route,
        auth: this.auth,
        modalService: this.modalService,
        vitisRequest: this.vitisRequest,
        vitisCheck: this.vitisCheck,
        vitisAlert: vitisAlert,
        vitisClone: this.vitisClone,
        translate: this.translate,
        formreaderSrvc: this.formreaderSrvc,
        mapCoreManagerService: this.mapCoreManagerService,
        mapCoreJSONService: this.mapCoreJSONService,
        mapCoreDrawService: this.mapCoreDrawService,
        mapCorePopupService: this.mapCorePopupService,
        mapCorePopupOverlayService: this.mapCorePopupOverlayService,
        mapCoreUtilsService: this.mapCoreUtilsService,
        mapSelectService: this.mapSelectService,
        mapCartoService: this.mapCartoService,
        mapFormService: this,
        oFormReader: oFormReader,
        maptoolsService: this.mapToolsService,
        mapToolsService: this.mapToolsService,
        mapToolsReportService: this.mapToolsReportService,
        mapToolsPrintService: this.mapToolsPrintService,
        mapToolsMeasureService: this.mapToolsMeasureService,
        mapToolsGeolocationService: this.mapToolsGeolocationService,
        exportFromJSON: exportFromJSON,
        echarts: echarts,
        moment: moment,
        shpjs: shpjs,
      });
    }
  }

  /**
   * Permet de charger le css des formulaire d'une couche dans le html
   * @param published_form_css_def
   */
  public loadCss(published_form_css_def: string) {
    // Supprime les css précédement chargés
    this.removeCss();

    // Charge le nouveau css
    if (published_form_css_def) {
      const oStyle = document.createElement('style');
      oStyle.textContent = published_form_css_def;
      oStyle.setAttribute('class', 'vmap-form-css');
      document.getElementsByTagName('head')[0].appendChild(oStyle);
    }
  }

  /**
   * Supprime les css précédement chargés
   */
  public removeCss() {
    const oldDiv = document.getElementsByClassName('vmap-form-css');
    for (let i = 0; i < oldDiv.length; i++) {
      oldDiv[i].remove();
    }
  }
}
