import { Injectable } from '@angular/core';
import { MapCoreDrawService } from 'src/app/vitis/map-core/services/map-core-draw/map-core-draw.service';
import { MapCorePopupService } from 'src/app/vitis/map-core/services/map-core-popup/map-core-popup.service';
import { Feature } from 'ol';
import { Geometry, LineString, Point, Polygon, Circle } from 'ol/geom';
import { transform } from 'ol/proj';
import { Fill, Stroke, Style } from 'ol/style';
import CircleStyle from 'ol/style/Circle';

@Injectable({
  providedIn: 'root'
})
export class PrintMeasureService {

  /**
   * Constructeur
   */
  constructor(
    private mapCoreDrawService: MapCoreDrawService,
    private mapPopup: MapCorePopupService,
  ) { }

  private olMeasureStyle = new Style({
    fill: new Fill({
      color: 'rgba(255, 255, 255, 0.2)',
    }),
    stroke: new Stroke({
      color: '#ffcc33',
      width: 2,
    }),
    image: new CircleStyle({
      radius: 7,
      fill: new Fill({
        color: '#ffcc33',
      }),
    })
  });

  /**
   * Rajoute les mesures à la carte pour l'impression
   */
  public initPrintableMeasures(olMap, resolutionCoeff?: number) {
    olMap.set('measure_layer', this.mapCoreDrawService.initDrawLayer(olMap, null, this.olMeasureStyle));
    const infosMeasures = olMap.get('measure_features');
    if (infosMeasures && infosMeasures.length > 0) {
      infosMeasures.forEach(measure => {
        let feature: Feature<Geometry>;
        let sPopupData: any;
        switch (measure.geometry.type) {
          case 'Point':
            feature = new Feature({ geometry: new Point(transform(measure.geometry.coordinates, measure.projection, olMap.getView().getProjection())) });
            sPopupData = {
              Projection: measure.projection,
              X: measure.sPopupData.X,
              Y: measure.sPopupData.Y,
            };
            break;
          case 'LineString':
            feature = new Feature({ geometry: new LineString(measure.geometry.coordinates) });
            sPopupData = measure.sPopupData;
            break;
          case 'Polygon':
            feature = new Feature({ geometry: new Polygon(measure.geometry.coordinates) });
            sPopupData = measure.sPopupData;
            break;
          case 'Circle':
            feature = new Feature({ geometry: new Circle(measure.geometry.center, measure.radius) });
            sPopupData = measure.sPopupData;
            break;
        }
        if (feature) {
          // Ajout de la mesure à la carte
          olMap.get('measure_layer').getSource().addFeature(feature);
          // Ajout de la popup à la carte
          this.mapPopup.displayPopup(sPopupData, feature, olMap, { hideCloseBtn: true, resolutionCoeff });
        }
      });
    }
    return olMap;
  }
}
