import { Component, Input, OnInit, Output, EventEmitter, OnChanges } from '@angular/core';
import { LayerSymbologyService } from '../../../services/layer-symbology/layer-symbology.service';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { TranslateService } from '@ngx-translate/core';

declare let $: any;

@Component({
  selector: 'app-layer-symbology-form-text',
  templateUrl: './layer-symbology-form-text.component.html',
  styleUrls: ['./layer-symbology-form-text.component.scss']
})
export class LayerSymbologyFormTextComponent implements OnInit, OnChanges {

  @Input() public value: any;
  @Input() public label: any;
  @Input() public attribut?: string;
  @Input() public disabled: boolean = false;
  @Input() public placeholder: string = '';
  @Input() public type?: string;
  @Input() multipleclass?: number;
  @Input() formatFunc: any;
  @Input() public tooltipContent: string;
  @Input() public tooltipTitle: string;
  @Input() public pattern: string;
  @Input() public required: boolean;
  @Input() public errorMessagePattern: string;

  @Output() changeVal = new EventEmitter<string>();
  @Output() isValidField = new EventEmitter<boolean>();

  public typeInput: string;
  public classId: number;
  public isForceDisabled: boolean = false;

  /**
   * Constructeur
   */
  constructor(public service: LayerSymbologyService,
    private vitisCheck: VitisCheck,
    private translate: TranslateService) {
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    if (this.type && this.type !== 'color') {
      this.typeInput = this.type;
    } else if (this.type && this.type === 'color') { // pour les types color on ajoute une phrase générique
      this.translate.get([this.tooltipContent, 'COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS_LABEL_COLORS_TOOLTIP_CONTENT']).subscribe((aTranslate: Array<string>) => {
        this.tooltipContent = aTranslate[this.tooltipContent] + '<br><br>' + aTranslate['COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS_LABEL_COLORS_TOOLTIP_CONTENT'];
      });
    }

    this.isValidField.next(this.checkRequired(this.value));
  }

  /**
   * Appelé lorsqu'il y a des changements sur les input
   */
  ngOnChanges(changes) {
    if (changes) {

      if (changes.disabled && this.vitisCheck.isDefAndNotNull(changes.disabled.currentValue)) {
        this.isForceDisabled = changes.disabled.currentValue;
      }

      if (changes.formatFunc && this.vitisCheck.isDefAndNotNull(changes.formatFunc.currentValue) && !changes.disabled) {
        this.disabled = changes.formatFunc.currentValue ? true : false;
      }
      if (changes.multipleclass && this.vitisCheck.isDefAndNotNull(changes.multipleclass.currentValue)) {
        this.classId = this.multipleclass;
      }
    }
  }

  /**
   * Permet d'ouvrir l'éditeur de texte avancée
   * @param value Valeur de l'input
   */
  public openAvancedTextInput(value): void {
    $('#modal-avanced-text-input').modal('show', { value, label: this.label, multipleclass: this.classId });
    this.service.attributName = this.attribut;
  }

  /**
   * Vérifie si il a un pattern et si la valeur est valide par rapport à ce pattern
   * @param value Valeur de l'input
   */
  public inputControlChange(value: string) {
    if (this.vitisCheck.isDefAndNotNull(this.pattern) && this.pattern != '') {
      if (value !== '') {
        const match = value.match(RegExp(this.pattern));
        this.isValidField.next((match && value === match[0]) ?? false);
      } else {
        this.isValidField.next(this.checkRequired(value));
      }
    } else {
      this.isValidField.next(this.checkRequired(value));
    }
  }

  /**
   * Si required vaut true et qu'il n'y a pas de valeur, c'est une erreur sinon c'est bon.
   * @param value
   */
  public checkRequired(value: string) {
    return (
      this.vitisCheck.isDefAndNotNull(this.required) &&
      this.required === true &&
      (value === '' || !this.vitisCheck.isDefAndNotNull(value))
    ) ? false : true;
  }
}
