import { Component, Input, OnChanges, OnInit, Output, EventEmitter } from '@angular/core';
import { LayerSymbologyService } from '../../services/layer-symbology/layer-symbology.service';
import { VmapLayer } from 'src/app/vitis/map-core/services/map-models.model';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { JsonSymbologyLabelsShadow, JsonSymbologyLabelsWrap, JsonSymbologyLabelsSizeAndPosition, JsonSymbologyLabelsOther, JsonSymbologyLabelsBackgroundShadow, JsonSymbologyLabelsColorAndSizeOutline, JsonSymbologyLabelsVisibility } from '../../../models/json-symbology/json-symbology-label.model';
import { SubFields } from 'src/modules/vmap/components/models/json-symbology/json-symbology-label.model';

@Component({
  selector: 'app-layer-symbology-class-label-form',
  templateUrl: './layer-symbology-class-label-form.component.html',
  styleUrls: ['./layer-symbology-class-label-form.component.scss']
})
export class LayerSymbologyClassLabelFormComponent implements OnInit, OnChanges {

  @Input() public oDefinitionSymJson: any;
  @Input() public bClassOpen: boolean;

  public bLabelOpen: Array<any> = [];

  public sDragElementID: string;
  public placeholder: HTMLDivElement;


  public oSubType = this.service.getSubType();
  public aAvailableSubType = this.service.getAvailableSubTypeList('', true);
  public isOpenDropDown: boolean = false;

  @Input() public oLayer: VmapLayer;

  public bVisibilityOpen: boolean = false;
  public bFontOpen: boolean = false;
  public bIsPolygon: boolean = false;
  public bIsPoint: boolean = false;
  public bIsLine: boolean = false;

  public bShadowFontOpen: boolean = false;
  public bShadowFontYes: boolean = false;
  public bLabelClassOpen: boolean = false;

  @Output() isValidField = new EventEmitter<any>();

  @Output() deleteVal = new EventEmitter<any>();

  /**
   * Retourne la valeur de symbols
   */
  get symbols() {
    return this.service.symbols['_value'];
  }

  /**
   * Retourne la valeur de fonts
   */
  get fonts() {
    return this.service.fonts['_value'];
  }

  /**
   * Retourne la valeur de positions
   */
  get positions() {
    return this.service.positions['_value'];
  }


  /**
   * Constructeur
   */
  constructor(
    public service: LayerSymbologyService,
    private vitisCheck: VitisCheck,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    if (this.vitisCheck.isDefAndNotNull(this.oLayer) && this.vitisCheck.isDefAndNotNull(this.oLayer.ms_geomtype_id)) {
      switch (this.oLayer.ms_geomtype_id) {
        case 1: // point
          this.bIsPoint = true;
          break;
        case 2:
          this.bIsLine = true;
          break;
        case 3:
          this.bIsPolygon = true;
          break;
      }
    }
  }

  /**
   * Appelé lorsqu'il y a des changements sur les input
   */
  ngOnChanges(changes): void {
    if (changes.oDefinitionSymJson && changes.oDefinitionSymJson.currentValue) {
      this.oDefinitionSymJson = changes.oDefinitionSymJson.currentValue;
    }
  }

  /**
   * Supprime la liste des champs liés
   * @param aFields
   */
  deleteFields(aFields: Array<string>) {
    for (let i = 0; i < aFields.length; i++) {
      const field = aFields[i];
      this.deleteVal.next(this.getAttribut(field));
      this.isValidField.next({ field: true});
    }
  }

  /**
   * Ouvre la liste des labels pour l'ajout
   */
  public openAddLabels = (): void => {
    document.getElementById('labelsDropdown').classList.toggle('show');
    this.isOpenDropDown = !this.isOpenDropDown;
  };

  /**
   * Ajoute un label au table oDefinitionSymJson.otherFields
   * @param label type stype à ajouter
   */
  public addLabels = (label): void => {
    let jsonSymbology;
    const id = label;
    switch (label) {
      case 'shadow':
        jsonSymbology = { 'shadow': new JsonSymbologyLabelsShadow() };
        break;
      case 'shadowBackground':
        jsonSymbology = { 'shadowBackground': new JsonSymbologyLabelsBackgroundShadow() };
        break;
      case 'wrap':
        jsonSymbology = { 'wrap': new JsonSymbologyLabelsWrap() };
        break;
      case 'sizeAndPosition':
        jsonSymbology = { 'sizeAndPosition': new JsonSymbologyLabelsSizeAndPosition() };
        break;
      case 'outline':
        jsonSymbology = { 'outline': new JsonSymbologyLabelsColorAndSizeOutline() };
        break;
      case 'other':
        jsonSymbology = { 'other': new JsonSymbologyLabelsOther() };
        break;
      case 'visibility':
        jsonSymbology = { 'visibility': new JsonSymbologyLabelsVisibility() };
        break;
    }

    document.getElementById('labelsDropdown').classList.remove('show');
    this.oDefinitionSymJson.label.otherFields = { ...this.oDefinitionSymJson.label.otherFields, ...jsonSymbology};
  };

  /**
   * Ouvre ou ferme le label
   * @param id identifiant du label
   */
  public openEdit = (id): void => {
    if (this.bLabelOpen.includes(id)) {
      this.bLabelOpen = this.bLabelOpen.filter(open => open !== id);
    } else {
      this.bLabelOpen = [...this.bLabelOpen, id];
    }
  };

  public getAttribut = (indexLabel) => {
    return this.vitisCheck.isDefAndNotNull(this.oDefinitionSymJson.id) ?
      `class.${this.oDefinitionSymJson.id}.label.otherFields.${indexLabel}` : `label.otherFields.${indexLabel}`;
  };

  /**
   * Set la valeur dans otherFields en fonction du chemin dans event
   */
  public setJsonSymboValue(event: any) {
    this.oDefinitionSymJson.label = this.service.setOtherFieldValue(event, this.oDefinitionSymJson.label);
  }
}
