import { computeMsgId } from '@angular/compiler';
import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { Map } from 'ol';
import { JsonSymbologyExpression } from 'src/app/modules/vmap/models/json-symbology/json-symbology-expression.model';
import { VitisRequest, vitisAlert, VitisCheck, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { VmapLayer } from 'src/app/vitis/map-core/services/map-models.model';
import { environment } from 'src/environments/environment';
import { VmapToolsService } from 'src/modules/vmap/components/carto/services/vmap-tools/vmap-tools.service';
import { lineSimple } from 'src/modules/vmap/components/models/definition-sym-json';
import { JsonSymbologyClass } from 'src/modules/vmap/components/models/json-symbology/json-symbology-class.model';
import { lineSymbol, polygonSimple, polygonSymbolCenter, polygonSymbolRepeat, unflattenDefinitionSymJson } from 'src/modules/vmap/components/models/definition-sym-json';
import { JsonSymbologyCluster, JsonSymbologyClusterDef } from 'src/modules/vmap/components/models/json-symbology/json-symbology-cluster.model';
import { LayerSymbologyService } from './services/layer-symbology/layer-symbology.service';
import { clusterExpression } from '../models/definition-sym-json';
import { JsonSymbologyOpacity, JsonSymbologyUnits, JsonSymbologyVisibility } from '../models/json-symbology/json-symbology-params.model';

@Component({
  selector: 'app-layer-symbology',
  templateUrl: './layer-symbology.component.html',
  styleUrls: ['./layer-symbology.component.scss']
})
export class LayerSymbologyComponent implements OnInit {

  @Input() public olMap: Map;
  @Input() public oLayer: VmapLayer;
  @Input() public sClass: string;
  @Input() public testJSONEvent: EventEmitter<any>;
  @Output() public sendJSONevent: EventEmitter<any> = new EventEmitter();

  public bParamsOpen = true; // Statut d'ouverture du composant paramètre de la couche

  public isScaleVisible = true; // Statut de présence de l'échelle de référence, non nécessaire pour les formats fichiers
  public isFile = false; // Statut pour savoir s'il faut modifier le titre lorsque c'est un format fichier

  public oDefinitionSymJson: any = {}; // Object layer.definition_sym_json
  public oDefinitionSymJsonModified: any = {}; // Copie de l'object layer.definition_sym_json, en cours de modifications
  public isModaleAvancedEditorOpen: boolean = false; // Statut d'ouverture de la modale avec l'éditeur de texte simple/avancé
  public ClassType; // Model de la classe en cours : singleclass (JsonSymbologyClass) | expression (JsonSymbologyExpression)
  public editSymbology: 'layer' | 'class' = 'layer'; // Édition en cours de la page : layer | class
  public class: any = { // Class précharger quand on édite une classe
    index: null,
    content: null,
  };
  private isValidFields: any = {}; // Nom du champ: boolean
  public isLayerClassExist = false;
  private geomtype: String;
  public routeDatasource = {
    layer: {
      postgres: 'pglayers',
      file: 'filelayers'
    },
    publication: {
      postgres: 'pgpublications',
      file: 'filepublications'
    }
  };

  /**
   * Retourne columnsAvancedInput
   */
  get columnsAvancedInput() {
    return this.service.columnsAvancedInput['_value'];
  }
  private msGeomTypes = ['point', 'line', 'polygon'];
  public currGeomType = 'point';
  private def_sym_json_geometries = {};


  public styleTypes: any = {
    'singleclass': 'COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS_PARAMS_TYPE_SINGLECLASS',
    'expression': 'COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS_PARAMS_TYPE_EXPRESSION',
    // 'range': 'COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS_PARAMS_TYPE_RANGE',
  };

  public oSubType = this.service.getSubType();
  public aAvailableSubType = this.service.getAvailableSubTypeList(this.currGeomType, false, true);
  public isSymbolscaledenom: boolean = false;
  public isOpenDropDown: boolean = false;
  public bAutogenerate: boolean = false;

  /**
   * Constructeur
   */
  constructor(
    private vmapToolsService: VmapToolsService,
    private service: LayerSymbologyService,
    private vitisRequest: VitisRequest,
    private vitisCheck: VitisCheck,
    private vitisClone: VitisClone,
    private formReaderService: FormreaderService,
    private translate: TranslateService
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    if (this.oLayer['layertype'] === 'postgres' ||
      this.oLayer['publicationtype'] === 'postgres') {
      this.service.getSymbols();
      this.service.getFonts();
      // Initialise les données de bdd pour récupérer les colonnes liées à la couche
      this.service.bddDatas = {
        layer_id: this.oLayer['layer_id'],
        database: this.oLayer['database'],
        schema: this.oLayer['schema'],
        table: this.oLayer['table'] || this.oLayer['pg_table'],
        datasource_id: this.oLayer['datasource_id']
      };
      this.service.getColumns().catch(() => {
        console.error('Cannot get layer columns');
        vitisAlert.alert({
          text: 'Impossible de récupérer les colonnes de la couche',
          type: 'error',
        });
      });

      this.isLayerClassExist = true;

      this.geomtype = this.oLayer['ms_geomtype']['label'];
      // Vérifie si la symboloie a été générer
      if (this.vitisCheck.isDefAndNotNull(this.oLayer['definition_sym_json']) &&
        this.vitisCheck.isDefAndNotNull(this.oLayer['definition_sym_json'].autogenerate) &&
        this.oLayer['definition_sym_json'].autogenerate === true) {
        this.bAutogenerate = true;
      }

      let bNewSym = false;
      if (!this.oLayer.definition_sym_json) {
        bNewSym = true;
        this.oLayer.definition_sym_json = {};
      }

      if (this.hasGeomTypeChange()) { // Test s'il y a eu changement de type de géométrie
        // Alerte l'utilisateur que le type de géométrie à changé et lui propose de repartir de la symbologie par défaut
        this.alertGeomTypeChanged();
      } else if (this.isSymbologyValid()) { // Test si la définition semble ok
        // Initialise la symbologie
        this.initLayerSymbology(false);
      } else if (!this.isSymbologyValid() || bNewSym === true) { // Test si la définition est vide ou incorrecte
        this.alertSymbologyNotValid();
      } else {
        this.alertSymbologyNotValid();
      }
    } else {
      if (this.oLayer['layertype'] === 'file' ||
        this.oLayer['publicationtype'] === 'file') {
        this.isScaleVisible = false;
        this.isFile = true;
        let bNewSym = false;
        if (!this.oLayer.definition_sym_json) {
          bNewSym = true;
          this.oLayer.definition_sym_json = {};
        }

        this.oLayer.definition_sym_json = this.unflattenDefSymJson(this.vitisClone.clone(this.oLayer.definition_sym_json));
        this.initDefinitionSymJson(this.oLayer.definition_sym_json);


      }
    }

    /**
     * Subscribe sur le test de  JSON
     */
    this.testJSONEvent.subscribe(editorJson => {
      this.oLayer.definition_sym_json = editorJson;
      this.editSymbology = 'layer';
      this.initDefinitionSymJson(this.oLayer.definition_sym_json);
    });
  }

  /**
   * Initialise la symbologie de la couche, si generateSymbology est à true alors regénère la symbologie par défaut en fonction du type de géométrie
   * @param generateSymbology
   */
  public initLayerSymbology(generateSymbology: boolean) {

    // Si type de géométrie est égal à géométrie
    if (this.isGeometryGeomtype()) {
      this.oLayer.definition_sym_json.prevGeomType = this.geomtype;
      this.msGeomTypes.forEach(geomType => {
        this.def_sym_json_geometries[geomType] = this.vitisCheck.isDefAndNotNull(this.oLayer.definition_sym_json[geomType]) ?
          this.vitisClone.clone(this.oLayer.definition_sym_json[geomType]) : this.vitisClone.clone(this.oLayer.definition_sym_json);
        if (generateSymbology == true) {
          this.def_sym_json_geometries[geomType].styles = this.setDefaultValue(geomType);
        }
        this.def_sym_json_geometries[geomType].prevGeomType = this.geomtype;
      });
      this.oLayer.definition_sym_json = this.vitisClone.clone(this.def_sym_json_geometries);
      this.updateoLayerGeomType(this.currGeomType);
      // autre type de géométrie
    } else {
      this.currGeomType = this.oLayer.ms_geomtype.label.toLowerCase();
      if (this.currGeomType == 'point') {
        this.styleTypes.cluster = 'COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS_PARAMS_TYPE_CLUSTER';
      }
      if (generateSymbology == true) {
        const styleJSON = this.setDefaultValue(this.geomtype);
        this.oLayer.definition_sym_json.styles = styleJSON;
      }
      this.oLayer.definition_sym_json.prevGeomType = this.geomtype;
      this.oLayer.definition_sym_json = this.unflattenDefSymJson(this.vitisClone.clone(this.oLayer.definition_sym_json));
      this.initDefinitionSymJson(this.oLayer.definition_sym_json);
    }
  }

  /**
   * Initialisation de la définition symbologique de la couche
   * @param oLayerSymDef
   */
  private initDefinitionSymJson(oLayerSymDef) {

    this.aAvailableSubType = this.service.getAvailableSubTypeList(this.currGeomType, false, true, oLayerSymDef.type, this.isFile);
    this.initClassType(oLayerSymDef.type);
    this.oDefinitionSymJson = new this.ClassType(this.vitisClone.clone(oLayerSymDef));
    this.sendJSON(this.oDefinitionSymJson);
    this.oDefinitionSymJsonModified = new this.ClassType(this.oDefinitionSymJson);

    // Met à jour la prévisualisation
    setTimeout(() => {
      if (this.oLayer['layertype'] === 'postgres') {
        this.service.getColumns(this.oDefinitionSymJsonModified.type == 'cluster');
      }
      this.testModifications();
    });
  }

  /**
   * Renvoie le style spécifique à la geometrie
   * @param geomtype
   */
  public setDefaultValue(defGeom) {
    if (defGeom == 'POINT' || defGeom == 'point') {
      return environment.properties.default_symbology_point.styles;
    }
    if (defGeom == 'LINE' || defGeom == 'line') {
      return environment.properties.default_symbology_line.styles;
    }
    if (defGeom == 'POLYGON' || defGeom == 'polygon') {
      return environment.properties.default_symbology_polygon.styles;
    }
  }

  /**
   * Appelé au changement de type de géométrie
   * @param type
   */
  public updateoLayerGeomType(geomType): void {
    this.currGeomType = geomType;
    if (this.currGeomType == 'point') {
      this.styleTypes.cluster = 'COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS_PARAMS_TYPE_CLUSTER';
    }
    this.oLayer.definition_sym_json[geomType] = this.unflattenDefSymJson(this.vitisClone.clone(this.oLayer.definition_sym_json[geomType]));
    this.oLayer.definition_sym_json[geomType].geomType = geomType;
    this.initDefinitionSymJson(this.oLayer.definition_sym_json[geomType]);
  }

  /**
   * Appelé lorsque le json est initialiser, modifier ou mis à jour
   * @param object
   */
  public sendJSON(json) {
    setTimeout(() => {
      this.sendJSONevent.emit(json);
    });
  }

  /**
   * Initialise le type de classe
   * @param type
   */
  private initClassType(type?): void {
    switch (type) {
      case 'singleclass':
        this.ClassType = JsonSymbologyClass;
        break;
      case 'expression':
        this.ClassType = JsonSymbologyExpression;
        break;
      case 'cluster':
        this.ClassType = JsonSymbologyCluster;
        break;
      default:
        this.ClassType = JsonSymbologyClass;
        break;
    }
  }

  /**
   * Appelé au changement de type
   * @param type
   */
  public updateType(type): void {
    this.aAvailableSubType = this.service.getAvailableSubTypeList(this.currGeomType, false, true, type);
    this.initClassType(type);
    this.oDefinitionSymJsonModified = new this.ClassType(this.oDefinitionSymJsonModified);
    this.sendJSON(this.oDefinitionSymJsonModified);

    this.service.getColumns(this.oDefinitionSymJsonModified.type == 'cluster');
    if (this.isGeometryGeomtype()) {
      this.oDefinitionSymJsonModified.geomType = this.currGeomType;
      this.def_sym_json_geometries = {
        ...this.def_sym_json_geometries,
        [this.currGeomType]: this.oDefinitionSymJsonModified
      };
    }
  }

  /**
   * Ferme le menu
   */
  public closePanel() {
    this.vmapToolsService.setLeftPanel(1, 'control-layers');
  }

  /**
   * Met à jour le champ depuis l'éditeur de texte avancé
   * @param e  = { attribut, value } : attribut: chemin depuis l'object odefsymjson ; value: nouvelle valeur
   */
  public changeVal(e: any): void {
    const childs = e.attribut.match(/(\w*\d*)/ig).filter(child => child);
    let oDefClone = this.editSymbology === 'class' ? this.class.content : this.oDefinitionSymJsonModified; // Permet de mettre à jour avec la référence de la vararible
    for (let i = 0; i < childs.length; i++) {
      if (i !== childs.length - 1) {
        oDefClone = oDefClone[childs[i]];
      }
    }
    oDefClone[childs[childs.length - 1]] = e.value;
    if (this.editSymbology === 'class') {
      this.class.content = { ...this.class.content };
    } else if (this.editSymbology === 'layer') {
      this.oDefinitionSymJsonModified = { ...this.oDefinitionSymJsonModified }; // Réatribution du changement car l'objet a été modifié
    }
  }

  /**
   * Utiliser pour supprimer un element dans un tableau de l'objet odefSymJson
   * @param e chemin + index sous forme de string
   */
  public deleteVal(event): void {
    const { deleteClass, e, indexClass } = event;
    if (deleteClass && this.vitisCheck.isDefAndNotNull(indexClass)) { // SUPPRESSION D'UNE CLASSE
      this.oDefinitionSymJsonModified.classes = [...this.oDefinitionSymJsonModified.classes.slice(0, indexClass),
        ...this.oDefinitionSymJsonModified.classes.slice(Number(indexClass) + 1)];
    } else if (e) {
      const paths = e.match(/(\w*\d*)/ig).filter(child => child);

      if (this.vitisCheck.isDefAndNotNull(indexClass)) { // SUPPRESSION D'UN ELEMENT DANS UN TABLEAU -> MULTIPLE CLASSE
        if (paths.length == 5 && this.vitisCheck.isObject(this.class.content[paths[2]][paths[3]][paths[4]])) {
          // SUPPRESION D'UN ELEMENT DANS UN TABLEAU -> .otherFields
          delete this.class.content[paths[2]][paths[3]][paths[4]];
        } else {
          const property = paths[2];
          const indexToDelete = paths[3];
          const path = this.class.content[property];
          if (path) {
            this.class.content[property] = [...path.slice(0, indexToDelete), ...path.slice(Number(indexToDelete) + 1)];
          }
        }
      } else { // SUPPRESION D'UN ELEMENT DANS UN TABLEAU -> SINGLE CLASSE
        if (paths.length == 3 && this.vitisCheck.isObject(this.oDefinitionSymJsonModified[paths[0]][paths[1]][paths[2]])) { // SUPPRESION D'UN ELEMENT DANS UN TABLEAU -> .otherFields
          delete this.oDefinitionSymJsonModified[paths[0]][paths[1]][paths[2]];
        } else {
          const property = paths[0];
          const indexToDelete = paths[1];
          const path = this.oDefinitionSymJsonModified[property];
          this.oDefinitionSymJsonModified[property] = [...path.slice(0, indexToDelete), ...path.slice(Number(indexToDelete) + 1)];
        }
      }
    }
  }

  /**
   * Annule les modifications et reset les valeurs modifiées
   */
  public cancelModifications = (): void => {
    if (this.isGeometryGeomtype()) {
      this.aAvailableSubType = this.service.getAvailableSubTypeList(this.currGeomType, false, true, this.oLayer.definition_sym_json[this.currGeomType].type);
      this.initClassType(this.oLayer.definition_sym_json[this.currGeomType].type);
    } else {
      this.aAvailableSubType = this.service.getAvailableSubTypeList(this.currGeomType, false, true, this.oLayer.definition_sym_json.type);
      this.initClassType(this.oLayer.definition_sym_json.type);
    }
    this.oDefinitionSymJsonModified = new this.ClassType(this.oDefinitionSymJson);
    // Si on est en mode classe alors remonte sur l'édition de la couche
    if (this.editSymbology === 'class') {
      this.cancelClassModifications();
    } else {
      // lance le test
      this.testModifications();
    }
  };

  /**
   * Annule les modifications et reset les valeurs modifiées
   */
  public cancelClassModifications = (): void => {
    this.editSymbology = 'layer';
    this.class = { index: null, content: null };
    // lance le test
    this.testModifications();
  };

  /**
   * Reinitialise la symbologie
   */
  public resetJson() {
    this.oLayer.definition_sym_json = {};
    this.initLayerSymbology(true);
  }

  /**
   * Envoie la requête de modification de odefinition_sym_json OU modifie la classe éditer
   */
  public updateModifications = (): void => {
    this.sendJSON(this.oDefinitionSymJsonModified);
    if (this.editSymbology === 'class') {
      this.oDefinitionSymJsonModified.classes[this.class.index] = this.class.content;
    }
    if (this.formValidation(this.oDefinitionSymJsonModified, true)) {
      let oDefSymJson: any;
      let flatDefSymJson: any;
      if (this.isGeometryGeomtype()) { // Si type de géométrie est égal à géométrie
        oDefSymJson = {
          ...this.def_sym_json_geometries,
          [this.currGeomType]: this.oDefinitionSymJsonModified
        };
        flatDefSymJson = {
          ...this.def_sym_json_geometries,
          [this.currGeomType]: this.flattenDefSymJson(this.vitisClone.clone(this.oDefinitionSymJsonModified))
        };
      } else { // autre type de géométrie
        oDefSymJson = this.oDefinitionSymJsonModified;
        flatDefSymJson = this.flattenDefSymJson(this.vitisClone.clone(this.oDefinitionSymJsonModified));
      }
      let route = '';
      if (this.vitisCheck.isDefAndNotNull(this.oLayer.layer_id)) {
        route = `${this.routeDatasource.layer[this.oLayer['layertype']]}/${this.oLayer.layer_id}`;
      } else if (this.vitisCheck.isDefAndNotNull(this.oLayer.publication_id)) {
        route = `${this.routeDatasource.publication[this.oLayer['publicationtype']]}/${this.oLayer.publication_id}`;
      }
      this.vitisRequest.ajaxRequestPromise({
        url: `${environment.properties.app_api_url}/vmap/datasourcetreeview/${route}`,
        method: 'PUT',
        data: this.formReaderService.transformToDataForm({ definition_sym_json: flatDefSymJson }, true)
      }).then((resp: any) => {
        if (resp && resp.data && resp.data.status === 1) {
          if (this.isGeometryGeomtype()) { // Si type de géométrie est égal à géométrie
            this.oDefinitionSymJson = new this.ClassType(oDefSymJson[this.currGeomType]);
            this.oDefinitionSymJsonModified = new this.ClassType(oDefSymJson[this.currGeomType]);
            this.def_sym_json_geometries = this.vitisClone.clone(oDefSymJson);
            this.oLayer.definition_sym_json = this.vitisClone.clone(oDefSymJson);
          } else {
            this.oDefinitionSymJson = new this.ClassType(oDefSymJson);
            // Actualise l'object modifié pour le binding des héritage
            this.oDefinitionSymJsonModified = new this.ClassType(oDefSymJson);
          }
          // update ok
          this.translate.get('FORM_PROPERTIES_EDIT_OK').subscribe(async (translation) => {
            vitisAlert.alert({
              type: 'success',
              text: translation
            });
          });
          // lance le test
          this.testModifications();
        } else {
          // update ko
        }
      });
    }
  };

  /**
   * Envoie la requête de modification de odefinition_sym_json OU modifie la classe éditer
   */
  public updateClassModifications = (): void => {
    this.oDefinitionSymJsonModified.classes[this.class.index] = this.class.content;
    this.sendJSON(this.oDefinitionSymJsonModified);
    this.editSymbology = 'layer';
    this.updateModifications();
  };

  /**
   * Lance le test sur les modifications en cours
   */
  public testModifications() {
    if (this.editSymbology === 'class') {
      this.oDefinitionSymJsonModified.classes[this.class.index] = this.class.content;
    }
    if (this.oLayer['layertype'] === 'postgres') {
      this.oDefinitionSymJsonModified.prevGeomType = this.oLayer['ms_geomtype']['label'];
    }
    this.sendJSON(this.oDefinitionSymJsonModified);
    const flatDefSymJson = this.flattenDefSymJson(this.vitisClone.clone(this.oDefinitionSymJsonModified));
    this.service.runTest(flatDefSymJson);
  }

  /**
   * Lance le test sur les modifications en cours
   */
  public testClassModifications() {
    const oDefinitionSymJsonModified = this.vitisClone.clone(this.oDefinitionSymJsonModified);
    oDefinitionSymJsonModified.classes[this.class.index] = this.class.content;
    if (this.formValidation(oDefinitionSymJsonModified, true)) {
      this.sendJSON(oDefinitionSymJsonModified);
      const flatDefSymJson = this.flattenDefSymJson(this.vitisClone.clone(oDefinitionSymJsonModified));
      this.service.runTest(flatDefSymJson);
    }
  }

  /**
   * Ouvre l'éditeur pour ajouter une classe à la couche
   */
  public addClass = (): void => {
    this.class = {
      newClass: true,
      index: this.oDefinitionSymJsonModified.classes.length,
      content: new JsonSymbologyClass({ id: this.oDefinitionSymJsonModified.classes.length, styles: [this.service.setDefaultValue(this.currGeomType)] }, this.oDefinitionSymJsonModified.type)
    };
    this.editSymbology = 'class';
  };

  /**
   * Function qui prend les paramètres classe pour la modification
   * @param e { indexClass (index de la classe qu'on veut modifier), class (l'object class qu'on veut éditer)  }
   */
  public updateEdit(e): void {
    this.class = {
      index: e.indexClass,
      content: e.class
    };
    this.editSymbology = 'class';
  }

  /**
   * Vérifie si chaque champ qui possède [nom_de_la colonne], nom_de_la colonne fait bien parti de la table
   * @param form object JsonSymbologySingleClass modifié
   * @param formIsValide sert pour la recursivité
   * @returns boolean, true si le formulaire est valide false sinon
   */
  private formValidation(form: any, formIsValide: boolean): boolean {
    for (const key in form) {
      if (typeof form[key] === ('object')) {
        formIsValide = this.formValidation(form[key], formIsValide);
      } else if (typeof form[key] === 'string') {
        const isColumnExist = form[key].match(/[^[\]]+(?=])/g);
        if (isColumnExist) {
          for (let i = 0; i < isColumnExist.length; i++) {
            if (!this.columnsAvancedInput.find(column => column.column_name === isColumnExist[i])) {
              formIsValide = false;
              vitisAlert.alert({
                type: 'warning',
                text: `Veuillez entrer un nom de table valide. La table [${isColumnExist[i]}] n'existe pas.`
              });
            }
          }
        }

        if (this.vitisCheck.isDefAndNotNull(this.isValidFields[key]) && this.isValidFields[key] === false) {
          formIsValide = false;
          vitisAlert.alert({
            type: 'warning',
            text: this.translate.instant('COMPONENT_MAP_SYMBOLOGY_ERROR_INVALID_FORM')
          });
        }
      }
    }
    return formIsValide;
  }

  /**
   * Récupère le bon titre que l'on soit dans la création ou dans l'édition d'une classe
   * @returns
   */
  public getClassTitle = (): string => {
    let classTitle = '';
    this.translate.get(['COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS', 'COMPONENT_MAP_CREATE_SYMBOLOGY_CLASS'])
      .subscribe((translations: string) => {
        classTitle = this.oDefinitionSymJsonModified.classes[this.class.index] ?
          `${translations['COMPONENT_MAP_EDIT_SYMBOLOGY_CLASS']} : ${this.oDefinitionSymJsonModified.classes[this.class.index].legendName}` :
          translations['COMPONENT_MAP_CREATE_SYMBOLOGY_CLASS'];
      });
    return classTitle;
  };

  /**
   * Clonne une classe
   * @param e
   */
  public cloneClass(e: any) {
    const indexClass = e.indexClass;
    if (this.vitisCheck.isDefAndNotNull(indexClass) &&
      this.vitisCheck.isDefAndNotNull(this.oDefinitionSymJsonModified.classes[indexClass])) {
      const newClass = this.vitisClone.clone(this.oDefinitionSymJsonModified.classes[indexClass]);
      this.oDefinitionSymJsonModified.classes.splice(indexClass + 1, 0, newClass);
    }
  }

  /**
   * Remonce une classe
   * @param e
   */
  public moveUpClass(e: any) {
    const indexClass = e.indexClass;
    if (this.vitisCheck.isDefAndNotNull(indexClass) &&
      this.vitisCheck.isDefAndNotNull(this.oDefinitionSymJsonModified.classes[indexClass])) {
      this.moveClass(this.oDefinitionSymJsonModified.classes, indexClass, indexClass - 1);
    }
  }

  /**
   * Descend une classe
   * @param e
   */
  public moveDownClass(e: any) {
    const indexClass = e.indexClass;
    if (this.vitisCheck.isDefAndNotNull(indexClass) &&
      this.vitisCheck.isDefAndNotNull(this.oDefinitionSymJsonModified.classes[indexClass])) {
      this.moveClass(this.oDefinitionSymJsonModified.classes, indexClass, indexClass + 1);
    }
  }

  /**
   * Déplace une élement dans le tableau de classes
   * @param arr
   * @param old_index
   * @param new_index
   * @returns
   */
  private moveClass(arr: Array<any>, old_index: number, new_index: number) {
    if (new_index >= 0 && new_index < arr.length) {
      arr.splice(new_index, 0, arr.splice(old_index, 1)[0]);
    }
    return arr;
  }

  /**
   * Modifie la validité d'un champ
   * @param e
   */
  public isValidField(e: any) {
    Object.assign(this.isValidFields, e);
  }

  /**
   * Retourne true si la symbologie est de type geometry
   * @returns
   */
  private isGeometryGeomtype() {
    return this.oLayer['layertype'] === 'postgres' && this.oLayer['ms_geomtype']['label'] === 'GEOMETRY';
  }

  /**
   * Fonction appelée au changement de type
   * @param e
   */
  public changeType = (e) => {
    this.oDefinitionSymJsonModified.type = e;

    this.updateType(e);
  };

  /**
   * Set valeurs in otherField
   * @param event
   */
  public setJsonValue(event: any) {
    this.oDefinitionSymJsonModified = this.service.setOtherFieldValue(event, this.oDefinitionSymJsonModified);
  }

  /**
   * Vérifie si symbolscaledenom à une valeur quand il y a une modification
   * @param event
   */
  public updateSymbolScaleDeNom(event: any) {
    this.oDefinitionSymJsonModified.symbolscaledenom = event;
    this.isSymbolscaledenom = event == '' ? false : true;
  }

  /**
   * Fonction appelée lorsque le json est appliqué depuis l'onglet définition JSON
   * @param editorJson
   */
  public applyJSON(editorJson: string) {
    this.applyJSON(editorJson);
    this.oLayer.definition_sym_json = editorJson;
    this.testModifications();
    this.initDefinitionSymJson(this.oLayer.definition_sym_json);
  }

  /**
   * Passe d'une structure à plat à une structure utilisable dans le studio
   * @param oDefinitionSymJson
   */
  private unflattenDefSymJson(oDefinitionSymJson) {
    oDefinitionSymJson = this.unflattenItem(oDefinitionSymJson, 'general');
    if (this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.classes)) {
      for (let i = 0; i < oDefinitionSymJson.classes.length; i++) {
        oDefinitionSymJson.classes[i] = this.unflattenItem(oDefinitionSymJson.classes[i], 'classes');
        oDefinitionSymJson.classes[i] = this.unflattenClass(oDefinitionSymJson.classes[i]);
      }
    } else {
      oDefinitionSymJson = this.unflattenClass(oDefinitionSymJson);
    }

    return oDefinitionSymJson;
  }

  /**
   * Passe d'une structure à plat à une structure utilisable dans le studio
   * @param oDefinitionSymJson
   */
  private unflattenClass(oDefinitionSymJson: any) {
    if (this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.label)) {
      oDefinitionSymJson.label = this.unflattenItem(oDefinitionSymJson.label, 'labels');
    }
    if (this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.styles)) {
      for (let i = 0; i < oDefinitionSymJson.styles.length; i++) {
        oDefinitionSymJson.styles[i] = this.unflattenItem(oDefinitionSymJson.styles[i], 'styles');
      }
    }
    return oDefinitionSymJson;
  }

  /**
   * Passe d'une structure à plat à une structure utilisable dans le studio
   * @param oDefinitionSymJson
   * @param levelInObject
   * @returns
   */
  private unflattenItem(oDefinitionSymJson: any, levelInObject: string) {
    let keys: Array<string>;
    let localUnflattenDefinitionSymJson = unflattenDefinitionSymJson[levelInObject];
    // ajout un niveau avec le type de géométrie si besoin
    if (this.vitisCheck.isDefAndNotNull(localUnflattenDefinitionSymJson[this.currGeomType])) {
      localUnflattenDefinitionSymJson = localUnflattenDefinitionSymJson[this.currGeomType];
    }

    if (this.currGeomType == 'polygon') {
      if (Object.keys(oDefinitionSymJson).findIndex((key: string) => key === 'polygonSymbolRepeat') != -1) {
        localUnflattenDefinitionSymJson = { ...localUnflattenDefinitionSymJson, ...polygonSymbolRepeat };
      } else if (Object.keys(oDefinitionSymJson).findIndex((key: string) => key === 'polygonSymbolCenter') != -1) {
        localUnflattenDefinitionSymJson = { ...localUnflattenDefinitionSymJson, ...polygonSymbolCenter };
      } else {
        localUnflattenDefinitionSymJson = { ...localUnflattenDefinitionSymJson, ...polygonSimple };
      }
    }
    if (this.currGeomType == 'line') {
      if (Object.keys(oDefinitionSymJson).findIndex((key: string) => key === 'lineSymbol') != -1) {
        localUnflattenDefinitionSymJson = { ...localUnflattenDefinitionSymJson, ...lineSymbol };
      } else {
        localUnflattenDefinitionSymJson = { ...localUnflattenDefinitionSymJson, ...lineSimple };
      }
    }

    for (const key in oDefinitionSymJson) {
      if (Object.prototype.hasOwnProperty.call(oDefinitionSymJson, key)) {
        keys = [];
        if (!this.vitisCheck.isObject(oDefinitionSymJson[key])) {
          const element = oDefinitionSymJson[key];
          if (this.vitisCheck.isDefAndNotNull(localUnflattenDefinitionSymJson[key])) {
            if (this.vitisCheck.isDefAndNotNull(element) && element != '') {
              if (!this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.otherFields)) {
                oDefinitionSymJson.otherFields = {};
              }
              keys = localUnflattenDefinitionSymJson[key].split('.');

              if (keys.length == 2) {
                if (!this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.otherFields[keys[0]])) {
                  oDefinitionSymJson.otherFields[keys[0]] = {};
                }
                oDefinitionSymJson.otherFields[keys[0]][keys[1]] = this.vitisClone.clone(element);
              }
            }
            if (keys.length != 1) {
              delete oDefinitionSymJson[key];
            }
          }
        }
      }
    }
    return oDefinitionSymJson;
  }

  /**
   * Met à plat la définition de la symbologie en json pour la sauvegarde en bdd
   * @param oDefinitionSymJson
   */
  private flattenDefSymJson(oDefinitionSymJson: any) {
    oDefinitionSymJson = this.flattenOtherFields(oDefinitionSymJson);
    if (this.vitisCheck.isDefAndNotNull(oDefinitionSymJson.classes)) {
      for (let i = 0; i < oDefinitionSymJson.classes.length; i++) {
        oDefinitionSymJson.classes[i] = this.flattenOtherFields(oDefinitionSymJson.classes[i]);
        oDefinitionSymJson.classes[i] = this.flattenClass(oDefinitionSymJson.classes[i]);
      }
    } else {
      return this.flattenClass(oDefinitionSymJson);
    }
    return oDefinitionSymJson;
  }

  /**
   * met à plat une classe
   * @param oDefinitionSymJson
   */
  private flattenClass(oDefinitionSymJson: any) {
    oDefinitionSymJson.label = this.flattenOtherFields(oDefinitionSymJson.label);
    for (let i = 0; i < oDefinitionSymJson.styles.length; i++) {
      oDefinitionSymJson.styles[i] = this.flattenOtherFields(oDefinitionSymJson.styles[i]);
    }
    return oDefinitionSymJson;
  }

  /**
   * met à plat OtherFields
   * @param oDefinitionSymJson
   */
  private flattenOtherFields(oDefinitionSymJson: any) {
    if (Object.prototype.hasOwnProperty.call(oDefinitionSymJson, 'otherFields')) {
      for (const key in oDefinitionSymJson.otherFields) {
        if (Object.prototype.hasOwnProperty.call(oDefinitionSymJson.otherFields, key)) {
          const subArray = oDefinitionSymJson.otherFields[key];
          for (const subKey in subArray) {
            if (Object.prototype.hasOwnProperty.call(subArray, subKey)) {
              oDefinitionSymJson[subKey] = subArray[subKey];
            }
          }
        }
      }
      delete oDefinitionSymJson.otherFields;
    }
    return oDefinitionSymJson;
  }

  /**
   * Retourne true si la classe a une expression
   * @param oClass
   */
  public isClusterExpression(oClass: any) {
    return oClass.expression === clusterExpression[0];
  }

  /**
   * Ouvre la liste des params pour l'ajout
   */
  public openAddParams = (): void => {
    document.getElementById('paramsDropdown').classList.toggle('show');
    this.isOpenDropDown = !this.isOpenDropDown;
  };

  /**
   * Ajoute un param au table oDefinitionSymJsonModified.otherFields
   * @param param type stype à ajouter
   */
  public addParams = (param): void => {
    let jsonSymbology;
    const id = param;
    switch (param) {
      case 'visibility':
        jsonSymbology = { 'visibility': new JsonSymbologyVisibility() };
        break;
      case 'opacity':
        jsonSymbology = { 'opacity': new JsonSymbologyOpacity() };
        break;
      case 'units':
        jsonSymbology = { 'units': new JsonSymbologyUnits() };
        break;
      case 'cluster':
        jsonSymbology = { 'cluster': new JsonSymbologyClusterDef() };
        break;
    }

    document.getElementById('paramsDropdown').classList.remove('show');
    this.oDefinitionSymJsonModified.otherFields = { ...this.oDefinitionSymJsonModified.otherFields, ...jsonSymbology };
  };

  /**
   * Suppression de la valeur
   * @param event
   */
  public deleteParam(event) {
    this.translate.get([
      (this.oSubType[event] ? this.oSubType[event].label : ''),
      'COMPONENT_MAP_FORM_ASK_DELETE',
      'FORM_YES',
      'FORM_NO',
    ]).subscribe((translation) => {
      vitisAlert.alert({
        title: `${translation['COMPONENT_MAP_FORM_ASK_DELETE']} "${translation[(this.oSubType[event] ? this.oSubType[event].label : '')]}" ?`,
        type: 'warning',
        showCancelButton: true,
        confirmButtonColor: '#3085d6',
        cancelButtonColor: '#d33',
        confirmButtonText: translation['FORM_YES'],
        cancelButtonText: translation['FORM_NO']
      }).then((result: any) => {
        if (result.value) {
          for (const key in this.oDefinitionSymJsonModified.otherFields) {
            if (Object.prototype.hasOwnProperty.call(this.oDefinitionSymJsonModified.otherFields, key)) {
              if (key === event) {
                delete this.oDefinitionSymJsonModified.otherFields[key];
              }
            }
          }
        }
      });
    });
  }

  /**
   * Retourne true si la symbologie est ok
   * @returns
   */
  private isSymbologyValid(): boolean {
    if (this.oLayer.definition_sym_json &&
      this.oLayer.definition_sym_json.type) {
      if (this.oLayer.definition_sym_json.type === 'expression' ||
        this.oLayer.definition_sym_json.type === 'cluster') {
        if (this.vitisCheck.isArray(this.oLayer.definition_sym_json.classes)) {
          return true;
        } else {
          return false;
        }
      } else if (this.oLayer.definition_sym_json.type === 'singleclass') {
        if (this.vitisCheck.isArray(this.oLayer.definition_sym_json.styles)) {
          return true;
        } else {
          return false;
        }
      }
    } else if (this.oLayer.definition_sym_json && (this.oLayer.definition_sym_json.point.type ||
      this.oLayer.definition_sym_json.line.type || this.oLayer.definition_sym_json.polygon.type)) {
      if (this.oLayer.definition_sym_json.point.type === 'expression' ||
        this.oLayer.definition_sym_json.point.type === 'cluster' ||
        this.oLayer.definition_sym_json.line.type === 'expression' ||
        this.oLayer.definition_sym_json.line.type === 'cluster' ||
        this.oLayer.definition_sym_json.polygon.type === 'expression' ||
        this.oLayer.definition_sym_json.polygon.type === 'cluster') {
        if (this.vitisCheck.isArray(this.oLayer.definition_sym_json.point.classes) ||
          this.vitisCheck.isArray(this.oLayer.definition_sym_json.line.classes) ||
          this.vitisCheck.isArray(this.oLayer.definition_sym_json.polygon.classes)) {
          return true;
        } else {
          return false;
        }
      } else if (this.oLayer.definition_sym_json.point.type === 'singleclass' ||
        this.oLayer.definition_sym_json.line.type === 'singleclass' ||
        this.oLayer.definition_sym_json.polygon.type === 'singleclass') {
        if (this.vitisCheck.isArray(this.oLayer.definition_sym_json.point.styles) ||
          this.vitisCheck.isArray(this.oLayer.definition_sym_json.line.styles) ||
          this.vitisCheck.isArray(this.oLayer.definition_sym_json.polygon.styles)) {
          return true;
        } else {
          return false;
        }
      } else {
        return false;
      }
    } else {
      return false;
    }
  }
  /**
   * Retourne true s'il y a eu un changement de type de géométrie
   * @returns
   */
  private hasGeomTypeChange(): boolean {
    if (this.oLayer.definition_sym_json.prevGeomType) {
      if (this.oLayer.definition_sym_json.prevGeomType !== this.geomtype) {
        return true;
      } else {
        return false;
      }
    } else if (this.vitisCheck.isDefAndNotNull(this.oLayer.definition_sym_json['line']) || this.vitisCheck.isDefAndNotNull(this.oLayer.definition_sym_json['point']) || this.vitisCheck.isDefAndNotNull(this.oLayer.definition_sym_json['polygon'])) {
      if (this.oLayer.definition_sym_json['line'].prevGeomType !== this.geomtype && this.oLayer.definition_sym_json['point'].prevGeomType !== this.geomtype && this.oLayer.definition_sym_json['polygon'].prevGeomType !== this.geomtype) {
        return true;
      } else {
        return false;
      }
    } else {
      return false;
    }
  }

  /**
   * Alerte l'utilisateur que le type de géométrie à changé et lui propose de repartir de la symbologie par défaut
   */
  private alertGeomTypeChanged() {
    vitisAlert.alert({
      title: this.translate.instant('COMPONENT_MAP_EDIT_SYMBOLOGY_GEOM_TYPE_CHANGED'),
      text: this.translate.instant('COMPONENT_MAP_EDIT_SYMBOLOGY_RESET_SYMBOLOGY'),
      width: 800,
      showDenyButton: true,
      confirmButtonText: this.translate.instant('COMPONENT_MAP_EDIT_SYMBOLOGY_RESET_SYMBOLOGY_YES'),
      denyButtonText: this.translate.instant('COMPONENT_MAP_EDIT_SYMBOLOGY_RESET_SYMBOLOGY_NO'),
    }).then((result) => {
      if (result.isConfirmed) {
        this.oLayer.definition_sym_json = {};
        this.initLayerSymbology(true);
      } else {
        this.initLayerSymbology(false);
      }
    });
  }

  /**
   * Alerte l'utilisateur que la symbologie est innexistante ou non valide et lui propose d'utiliser la symbologie par défaut
   */
  private alertSymbologyNotValid() {
    vitisAlert.alert({
      title: this.translate.instant('COMPONENT_MAP_EDIT_SYMBOLOGY_NOT_VALID'),
      text: this.translate.instant('COMPONENT_MAP_EDIT_SYMBOLOGY_RESET_SYMBOLOGY'),
      width: 800,
      showDenyButton: true,
      confirmButtonText: this.translate.instant('COMPONENT_MAP_EDIT_SYMBOLOGY_RESET_SYMBOLOGY_YES'),
      denyButtonText: this.translate.instant('COMPONENT_MAP_EDIT_SYMBOLOGY_RESET_SYMBOLOGY_NO'),
    }).then((result) => {
      if (result.isConfirmed) {
        this.oLayer.definition_sym_json = {};
        this.initLayerSymbology(true);
      } else {
        this.initLayerSymbology(false);
      }
    });
  }
}
