import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { MultiTranslateHttpLoader } from 'ngx-translate-multi-http-loader';
import { TranslateLoader, TranslateModule } from '@ngx-translate/core';

import { SharedModule } from 'src/app/vitis/shared/shared.module';
import { FormreaderModule } from 'src/app/vitis/formreader/formreader.module';
import { environment } from 'src/environments/environment';

import { LayerSymbologyComponent } from './layer-symbology.component';
/**
 * Import components formulaire
 */
import {
  LayerSymbologyClassAvancedInputComponent,
  LayerSymbologyClassLabelFormComponent,
  LayerSymbologyClassComponent,
} from './components';

/**
 * Import champs réutilisable
 */
import {
  LayerSymbologyFormOpacityComponent,
  LayerSymbologyFormSelectComponent,
  LayerSymbologyFormTextComponent,
  LayerSymbologyFormVisibilityScalesComponent
} from './components/fields';
import { ContainerComponent } from './components/containers/container/container.component';
import { VisibilityComponent } from './components/containers/visibility/visibility.component';
import { OpacityComponent } from './components/containers/opacity/opacity.component';
import { LayerSymbologyStyleComponent } from './components/layer-symbology-style/layer-symbology-style.component';
import { SymbolOutlineComponent } from './components/containers/symbol-outline/symbol-outline.component';
import { SymbolOffsetComponent } from './components/containers/symbol-offset/symbol-offset.component';
import { SymbolSizeVariationComponent } from './components/containers/symbol-size-variation/symbol-size-variation.component';
import { LayerSymbologieFormScalesComponent } from './components/fields/layer-symbologie-form-scales/layer-symbologie-form-scales.component';
import { LabelShadowComponent } from './components/containers/label-shadow/label-shadow.component';
import { LabelShadowBackgroundComponent } from './components/containers/label-shadow-background/label-shadow-background.component';
import { LabelWrapComponent } from './components/containers/label-wrap/label-wrap.component';
import { LabelSizeAndPositionComponent } from './components/containers/label-size-and-position/label-size-and-position.component';
import { LabelOutlineComponent } from './components/containers/label-outline/label-outline.component';
import { LabelOtherComponent } from './components/containers/label-other/label-other.component';
import { DefaultFieldsPointComponent } from './components/containers/default-fields-point/default-fields-point.component';
import { DefaultFieldsLineComponent } from './components/containers/default-fields-line/default-fields-line.component';
import { DefaultFieldsPolygonComponent } from './components/containers/default-fields-polygon/default-fields-polygon.component';
import { SymbolRepeatComponent } from './components/containers/symbol-repeat/symbol-repeat.component';
import { PolygonSymbolComponent } from './components/containers/polygon-symbol/polygon-symbol.component';
import { LineSymbolComponent } from './components/containers/line-symbol/line-symbol.component';
import { LineDashComponent } from './components/containers/line-dash/line-dash.component';
import { ClusterComponent } from './components/containers/cluster/cluster.component';
import { UnitsComponent } from './components/containers/units/units.component';

/**
 * Chargement des fichiers de traduction
 * @param http
 * @returns
 */
export function HttpLoaderFactory(http: HttpClient) {
  return new MultiTranslateHttpLoader(http, environment.appStruct['translate']);
}

@NgModule({
  declarations: [
    LayerSymbologyComponent,
    LayerSymbologyClassComponent,
    LayerSymbologyClassAvancedInputComponent,
    LayerSymbologyClassLabelFormComponent,
    LayerSymbologyFormOpacityComponent,
    LayerSymbologyFormSelectComponent,
    LayerSymbologyFormTextComponent,
    LayerSymbologyFormVisibilityScalesComponent,
    ContainerComponent,
    VisibilityComponent,
    OpacityComponent,
    LayerSymbologyStyleComponent,
    SymbolOutlineComponent,
    SymbolOffsetComponent,
    SymbolSizeVariationComponent,
    LayerSymbologieFormScalesComponent,
    LabelShadowComponent,
    LabelShadowBackgroundComponent,
    LabelWrapComponent,
    LabelSizeAndPositionComponent,
    LabelOutlineComponent,
    LabelOtherComponent,
    DefaultFieldsPointComponent,
    DefaultFieldsLineComponent,
    DefaultFieldsPolygonComponent,
    SymbolRepeatComponent,
    PolygonSymbolComponent,
    LineSymbolComponent,
    LineDashComponent,
    ClusterComponent,
    UnitsComponent
  ],
  imports: [
    SharedModule,
    FormreaderModule,
    CommonModule,
    HttpClientModule,
    TranslateModule.forChild({
      loader: {
        provide: TranslateLoader,
        useFactory: HttpLoaderFactory,
        deps: [HttpClient]
      }
    }),
    FormsModule,
    ReactiveFormsModule,
  ],
  exports: [
    LayerSymbologyComponent
  ],
})
export class LayerSymbologyModule { }
