import { Component, OnInit, OnDestroy, ViewChild, Output, Input, EventEmitter, } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { VitisCheck, VitisClone, VitisRequest, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import { TranslateService } from '@ngx-translate/core';
import { LayerSymbologyService } from 'src/modules/vmap/components/layer-symbology/services/layer-symbology/layer-symbology.service';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { MapCoreComponent } from 'src/app/vitis/map-core/map-core.component';
import { Map } from 'ol';

@Component({
  selector: 'app-section-layer-symbology',
  templateUrl: './section-layer-symbology.component.html',
  styleUrls: ['./section-layer-symbology.component.scss']
})
export class SectionLayerSymbologyComponent implements OnInit, OnDestroy {

  @ViewChild(MapCoreComponent) mapCoreComponent: MapCoreComponent;

  public testJSONEvent = new EventEmitter();

  public oMode: any;
  public oParams: any;
  public oLayer: any;
  private layertype: any;
  public isDatasourceSection: boolean = false;
  public activeTestTab: string;
  public activeTestJson: boolean = false;

  // Options utilisées
  public oMapOptions = {};
  private olMap: Map;
  private aMapExtent: Array<number>;


  public routeDatasource = {
    layer: {
      postgres: 'pglayers',
      file: 'filelayers'
    },
    publication: {
      postgres: 'pgpublications',
      file: 'filepublications'
    }
  };

  // Options par défaut
  private oMapDefaultOptions_ = {
    type: 'vmap',
    controls: {
      mouse_position: false,
      zoom_controls: true,
      scale_line: true,
      map_projection: false,
      location_control: false
    },
    interactions: {},
    tree: {
      name: 'Carte de test',
      children: [
        {
          view: {
            zoom: 17,
            projection: 'EPSG:3857',
            extent: [-745018.5236871485, 4536769.427781392, 1179971.5966467303, 7259150.627186229]
          }
        },
        {
          name: 'Open Street Map',
          children: [
            {
              index: 0,
              name: 'Open Street Maps',
              visible: true,
              layerType: 'osm',
              url: 'https://{a-c}.tile.openstreetmap.org/{z}/{x}/{y}.png',
              select: true,
              displayedMenu: false,
              opacity: 1
            }
          ],
          visible: true
        }
      ]
    }
  };

  // Paramètres de l'éditeur de texte.
  public editorCode: string;
  public editorLogsCode: string;
  public logUrl: string;
  public editorOptions = {
    theme: 'vs',
    language: 'plaintext',
    readOnly: true,
    fontFamily: 'Arial',
    fontSize: 13,
    automaticLayout: true
  };
  public beforeEditorJson: string;
  public editorJson: string;
  public editorJsonOptions = {
    theme: 'vs',
    language: 'json',
    readOnly: false,
    fontFamily: 'Arial',
    fontSize: 13,
    automaticLayout: true
  };

  // Souscription à obsRunTest de LayerSymbologyService
  private testSubscription;

  /**
   * Constructeur
   */
  constructor(
    private route: ActivatedRoute,
    private vitisRequest: VitisRequest,
    private vitisCheck: VitisCheck,
    private vitisClone: VitisClone,
    private translate: TranslateService,
    private symbologyService: LayerSymbologyService,
    private formReaderService: FormreaderService,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {

    // Traductions (obligatoire sinon ne fonctionne pas dans le module en lazy loading)
    this.translate.addLangs(['en', 'fr']);
    this.translate.setDefaultLang('en');
    this.translate.use(environment.properties.language ? environment.properties.language : 'en');
    this.route.data.subscribe(oMode => {
      this.oMode = oMode;
      this.route.params.subscribe(oParams => {
        this.oParams = {
          'id': oParams.id,
          'form': oParams.form,
          'treeid': oParams.treeid,
        };

        if (oParams.treeid) {
          this.isDatasourceSection = true;
        }

        this.loadLayer();
      });
    });

    this.testSubscription = this.symbologyService.getObsRunTest().subscribe((oLayer: any) => {
      this.runTest(oLayer);
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy(): void {
    if (this.testSubscription) {
      this.testSubscription.unsubscribe();
    }
  }

  /**
   * Appelé lorsque l'étendue de la carte change
   * @param olMap
   */
  public changeExtentRecived(olMap: Map) {
    this.olMap = olMap;
    this.aMapExtent = olMap.getView().calculateExtent();
  }

  /**
   * Appeler lorsque l'enfant initialise, modifie, ou met à jour le Json
   * @param olMap
   */
  public receveidJSon(json) {
    this.beforeEditorJson = JSON.stringify(json, null, '\t');
    this.editorJson = JSON.stringify(json, null, '\t');
  }

  /**
   * Charge la couche
   * @returns
   */
  private loadLayer(): Promise<boolean> {
    return new Promise((resole, reject) => {
      if (this.vitisCheck.isDefAndNotNull(this.oMode.options)) {
        if (this.vitisCheck.isDefAndNotNull(this.oMode.options.ressource)) {
          const sRessourcePath = this.oMode.options.ressource;
          if (this.vitisCheck.isString(sRessourcePath)) {
            // Récupère l'id dans les paramètres de la route
            let sRouteId: any;
            if (this.vitisCheck.isDefAndNotNull(this.oParams)) {
              if (this.vitisCheck.isString(this.oParams.id)) {
                sRouteId = this.oParams.id;
              }
            }
            if (this.vitisCheck.isDefAndNotNull(this.oParams.id)) {

              this.vitisRequest.ajaxRequestPromise({
                method: 'GET',
                url: environment.properties.app_api_url + '/' + sRessourcePath + '/' + sRouteId,
                params: {
                  attributs: 'layer_id|publication_id|definition_sym_json|name|database|schema|pg_table|datasource_id|ms_geomtype_id|ms_geomtype|layertype|publicationtype'
                }
              }).then((resp) => {
                if (resp.data) {
                  if (resp.data.status === 1) {
                    if (resp.data.data) {

                      this.oLayer = resp.data.data;
                      this.layertype = this.oLayer.layertype || this.oLayer.publicationtype;
                      if (this.vitisCheck.isDefAndNotNull(this.oLayer.definition_sym_json)) {
                        this.oLayer.definition_sym_json = JSON.parse(this.oLayer.definition_sym_json);

                      }
                      resole(true);
                    }
                  }
                }
                resole(false);
              });
            }
          }
        }
      }
    });
  }

  /**
   * Déclenche le lancement du test sur la couche
   * @param oJsonSymbologyExpression
   */
  private runTest(oJsonSymbologyExpression: any) {
    this.activeTestTab = null;
    const data = {
      definition_sym_json: oJsonSymbologyExpression,
      layertype: this.layertype
    };

    if (this.oLayer['ms_geomtype'] && oJsonSymbologyExpression['geomType']) {
      data['geomtype'] = oJsonSymbologyExpression['geomType'];
    }
    this.vitisRequest.ajaxRequestPromise({
      url: `${environment.properties.app_api_url}/${this.oMode.options.ressourceTest}/${this.oLayer[this.oMode.options.id_field]}/test`,
      method: 'PUT',
      data: this.formReaderService.transformToDataForm(data, true)
    }).then((resp: any) => {
      if (resp && resp.data && resp.data.status === 1) {
        this.logUrl = resp.data.data.log;
        if (resp.data.data && resp.data.data.mapserver_def) {
          this.editorCode = resp.data.data.mapserver_def;

        } else {
          console.error('Layer test error : mapserver_def not defined', resp);
        }
        if (resp.data.data && resp.data.data.flux_url && resp.data.data.layer_name) {
          this.deleteLog().then(re => {
            this.refreshMapPreview(resp.data.data.flux_url, resp.data.data.layer_name);
          });
        } else {
          console.error('Layer test error : flux_url or layer_name not defined', resp);
        }
      } else {
        console.error('Layer test error', resp);
      }
    }).catch(err => {
      console.error('Layer test error', err);
    });
  }

  /**
   * onEditorInit function
   * @param  {any} editor
   * @returns void
   */
  public onEditorInit = function (editor: any, typeLanguage: String) {

    // Création d'un nouveau langage "mapserver" dans l'éditeur de texte.
    monaco.languages.register({ id: 'mapserver' });

    // Register a tokens provider for the language
    monaco.languages.setMonarchTokensProvider('mapserver', {
      tokenizer: {
        root: [
          [
            /\b(CLASS|CLUSTER|COMPOSITE|CONNECTIONOPTIONS|END|FEATURE|GRID|JOIN|LABEL|LAYER|LEADER|LEGEND|MAP|METADATA|OUTPUTFORMAT|PATTERN|POINTS|PROJECTION|QUERYMAP|REFERENCE|SCALEBAR|STYLE|SYMBOL|VALIDATION|WEB)\b/,
            'entity.name.function'
          ],
          [
            /\b(ALIGN|ANCHORPOINT|ANGLE|ANGLEITEM|ANTIALIAS|BACKGROUNDCOLOR|BANDSITEM|BINDVALS|BOM|BROWSEFORMAT|BUFFER|CHARACTER|CLASSITEM|CLASSGROUP|COLOR|COLORRANGE|COMPFILTER|COMPOSITE|COMPOP|CONFIG|CONNECTION|CONNECTIONTYPE|DATAPATTERN|DATA|DATARANGE|DEBUG|DRIVER|EMPTY|ENCODING|ERROR|EXPRESSION|EXTENT|EXTENSION|FEATURE|FILLED|FILTER|FILTERITEM|FONT|FOOTER|FORCE|FORMATOPTION|FROM|GAP|GEOMTRANSFORM|GRIDSTEP|GRATICULE|GROUP|HEADER|IMAGE|IMAGECOLOR|IMAGEPATH|IMAGEURL|INTERLACE|INTERVALS|IMAGETYPE|IMAGEMODE|INCLUDE|INITIALGAP|ITEMS|KEYIMAGE|KEYSIZE|KEYSPACING|LABELANGLEITEM|LABELCACHE|LABELITEM|LABELMAXSCALE|LABELMAXSCALEDENOM|LABELMINSCALE|LABELMINSCALEDENOM|LABELSIZEITEM|LOG|LABELREQUIRES|LABELFORMAT|LINECAP|LINEJOIN|LOG|LINEJOINMAXSIZE|MAXFEATURES|MAXGEOWIDTH|MAXSCALE|MAXSCALEDENOM|MAXSIZE|MAXTEMPLATE|MAXWIDTH|MINDISTANCE|MINFEATURESIZE|MINGEOWIDTH|MINSCALE|MINSCALEDENOM|MINSIZE|MINTEMPLATE|MINWIDTH|MAXARCS|MININTERVAL|MAXINTERVAL|MINSUBDIVIDE|MAXSUBDIVIDE|MIMETYPE|MARKER|MARKERSIZE|MINBOXSIZE|MAXBOXSIZE|NAME|OFFSET|OUTLINECOLOR|OUTLINEWIDTH|OFFSITE|OPACITY|OVERLAYOUTLINECOLOR|OVERLAYCOLOR|OVERLAYSYMBOL|OVERLAYSIZE|OVERLAYMINSIZE|OVERLAYMAXSIZE|OVERLAYBACKGROUNDCOLOR|PARTIALS|PRIORITY|POLAROFFSET|POSITION|POSTLABELCACHE|PROCESSING|QUERYITEM|RANGEITEM|REPEATDISTANCE|REQUIRES|SCALE|SHAPEPATH|SIZE|SPACING|STATUS|STYLED|SYMBOL|SYMBOLSCALE|SYMBOLSCALEDENOM|STYLEITEM|SIZEUNITS|SHADOWCOLOR|SHADOWSIZE|TABLE|TEMPLATE|TEXT|TILEINDEX|TILEITEM|TO|TOLERANCE|TOLERANCEUNITS|TRANSFORM|TRANSPARENT|TRANSPARENCY|TYPE|TEMPLATEPATTERN|UNITS|UTFDATA|UTFITEM|WIDTH|WKT|WRAP)\b/,
            'keyword.source'
          ],
          [
            /\b(AGG|AGGA|APPLICATION|AUTO|AUTO2|BEVEL|BITMAP|BUTT|BYTE|CAIRO|CC|CENTER|CHART|CIRCLE|CL|CONTOUR|CR|CSV|DD|DDMM|DDMMSS|DEFAULT|ELLIPSE|EMBED|FALSE|FEET|FLOAT32|FOLLOW|GD|GIANT|GIF|GTIFF|HATCH|HILITE|INCHES|INT16|INTERLACE|JPEG|KERNELDENSITY|KILOMETERS|KML|KMZ|LARGE|LC|LINE|LL|LOCAL|LR|MEDIUM|METERS|MILES|MITER|MIXED|MULTIPLE|MYGIS|MYSQL|NAUTICALMILES|NORMAL|OFF|OGR|ON|ONE-TO-MANY|ONE-TO-ONE|ORACLESPATIAL|PC256|PIXELS|PIXMAP|PNG|POINT|POLYGON|POSTGIS|POSTGRESQL|QUERY|RASTER|RGB|RGBA|ROUND|SDE|SELECTED|SIMPLE|SINGLE|SMALL|SQUARE|SVG|SWF|TINY|TRUE|TRUETYPE|UC|UL|UNION|UNION_SCALE_CHECK|UNION_SRCLAYER_CLOSE_CONNECTION|UNION_STATUS_CHECK|UR|UVRASTER|VALUES|VECTOR|VMS|WBMP|WFS|WMS|X-SHOCKWAVE-FLASH|XMP)\b/,
            'variable.parameter'
          ],
          [
            /\b(ANNOTATION|BACKGROUNDSHADOWCOLOR|BACKGROUNDSHADOWSIZE|CARTOLINE|DUMP|IMAGEQUALITY|INTERLACE|LABELANGLEITEM|LABELSIZEITEM|MAXSCALE|MINSCALE|MYGIS|OVERLAYBACKGROUNDCOLOR|OVERLAYMAXSIZE|OVERLAYMINSIZE|OVERLAYOUTLINECOLOR|OVERLAYSIZE|OVERLAYSYMBOL|SCALE|SYMBOLSCALE|TRANSPARENCY|TRIANGLE)\b/,
            'invalid'
          ],
          [
            /[-]?[0-9]*\.?[0-9]+/,
            'constant.numeric'
          ],
          [
            /(#).*$\n?/,
            'comment.line'
          ],
        ]
      }
    });

    // Création d'un nouveau thème "mapserverTheme" dans l'éditeur de texte.
    monaco.editor.defineTheme('mapserverTheme', {
      base: 'vs',
      inherit: false,
      rules: [
        { token: 'entity.name.function', foreground: '#30069c' },
        { token: 'keyword.source', foreground: '#7eb600' },
        { token: 'variable.parameter', foreground: '#0074ff' },
        { token: 'invalid', foreground: '#cc0000' },
        { token: 'constant.numeric', foreground: '#000000' },
        { token: 'comment.line', foreground: '#417c26' },
      ],
      colors: {}
    });
    if (typeLanguage == 'mapserver') {
      // Change le thème et le langage.
      monaco.editor.setTheme('mapserverTheme');
      monaco.editor.setModelLanguage(editor.getModel(), 'mapserver');
    }
  };

  /**
   * Met à jour la carte de prévisualisation
   * @param sFluxUrl
   */
  private refreshMapPreview = function (sFluxUrl: any, sLayerName: string) {

    const oJSONLayer = {
      LAYERS: sLayerName,
      opacity: 1,
      visible: true,
      index: 30,
      layerType: 'imagewms',
      url: sFluxUrl,
      params: {
        LAYERS: sLayerName,
        VERSION: '1.3.0',
        TIMESTAMP: Date.now(),
      },
      serverType: 'mapserver',
      hidpi: false
    };

    this.oMapOptions = this.vitisClone.clone(this.oMapDefaultOptions_);

    if (this.oMapOptions.tree &&
      this.oMapOptions.tree.children) {

      // Ajoute la couche à tester
      this.oMapOptions.tree.children.push(oJSONLayer);
      if (this.aMapExtent &&
        this.oMapOptions.tree.children[0] &&
        this.oMapOptions.tree.children[0].view) {

        // Utilise l'étendue précédente
        this.oMapOptions.tree.children[0].view.extent = this.aMapExtent;
      }
    }

    this.activeTestTab = 'map_preview';
  };

  /**
   * Appel le testJSON sur l'élément parent
   */
  public testJSON() {
    this.testJSONEvent.next(JSON.parse(this.editorJson));
  }

  /**
   * Réinitialise la valeur du json contenue dans l'editeur de texte
   */
  public cancelJSON() {
    this.editorJson = this.beforeEditorJson;
  }

  /**
   * Rafraichit le fichier de log
   */
  public refreshLog() {
    if (this.logUrl) {
      // Récupère le log
      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url: environment.properties.app_api_url + '/vmap/logs/content',
        params: {
          key: this.logUrl
        }
      }).then((resp: any) => {
        if (resp.data) {
          if (resp.data.status === 1) {
            this.editorLogsCode = resp.data.data.length == 0 ? ' ' : resp.data.data;
          } else {
            // Message d'erreur
            this.translate.get('LOADING_FILE_ERROR_LOGS').subscribe((translation) => {
              const options: any = {
                type: 'error',
                title: translation
              };
              vitisAlert.alert(options);
            });
          }
        }
      });
    } else {
      console.error('L\'url de log n\'es pas défini :>> ', this.logUrl);
    }
  }

  /**
   * Nettoie le fichier de log
   * @returns
   */
  public deleteLog(): Promise<boolean> {
    return new Promise((resolve, reject) => {
      if (this.logUrl) {
        // Suppression du fichier.
        this.vitisRequest.ajaxRequest({
          method: 'DELETE',
          url: environment.properties.app_api_url + '/vmap/logs/delete',
          params: {
            key: this.logUrl
          },
          displayErrorMessage: false,
          success: (resp: any) => {
            resolve(true);
            if (resp.data && resp.data.status === 1) {
              this.editorLogsCode = '';
            }
          },
          error: (resp: any) => {
            resolve(true);
            console.error('Fichier de test n\'a pas pu être supprimé', resp.data.errorMessage);
            this.editorLogsCode = '';
          }
        });
      } else {
        console.error('L\'url de log n\'es pas défini, impossible de supprimer le fichier :>> ', this.logUrl);
        resolve(false);
      }
    });
  }

  /**
   * onEditorInit function
   * @param  {any} editor
   * @returns void
   */
  public onLogEditorInit = function (editor: any) {

    // Création d'un nouveau langage "logs" dans l'éditeur de texte.
    monaco.languages.register({ id: 'logs' });

    // Register a tokens provider for the language
    monaco.languages.setMonarchTokensProvider('logs', {
      tokenizer: {
        root: [
          [/((e|E)rror)/, 'custom-error'], // Erreurs
          [/(notice|(w|W)arning|warn)/i, 'custom-notice'], // Warnings
          [/\[info.*/, 'custom-info'],
          [/^\[?[0-9a-zA-Z- :\/]+\]?/, 'custom-date'] // Date du log
        ]
      }
    });

    // Création d'un nouveau thème "logsTheme" dans l'éditeur de texte.
    monaco.editor.defineTheme('logsTheme', {
      base: 'vs',
      inherit: false,
      rules: [
        { token: 'custom-info', foreground: '808080' },
        { token: 'custom-error', foreground: 'ff0000', fontStyle: 'bold' },
        { token: 'custom-notice', foreground: 'FFA500' },
        { token: 'custom-date', foreground: '008800' },
      ],
      colors: {}
    });

    /**
     * getLogsProvider function
     * @param  {any} monaco_
     * @returns suggestions
     */
    this.getLogsProvider = function (monaco_: any) {
      return {
        provideCompletionItems: () => {
          const suggestions = [{
            label: 'simpleText',
            kind: monaco_.languages.CompletionItemKind.Text,
            insertText: 'simpleText'
          }, {
            label: 'testing',
            kind: monaco_.languages.CompletionItemKind.Keyword,
            insertText: 'testing(${1:condition})',
            insertTextRules: monaco_.languages.CompletionItemInsertTextRule.InsertAsSnippet
          }, {
            label: 'ifelse',
            kind: monaco_.languages.CompletionItemKind.Snippet,
            insertText: [
              'if (${1:condition}) {',
              '\t$0',
              '} else {',
              '\t',
              '}'
            ].join('\n'),
            insertTextRules: monaco_.languages.CompletionItemInsertTextRule.InsertAsSnippet,
            documentation: 'If-Else Statement'
          }];
          return { suggestions: suggestions };
        }
      };
    };

    monaco.languages.registerCompletionItemProvider('logs', this.getLogsProvider(monaco));

    // Change le thème et le langage.
    monaco.editor.setTheme('logsTheme');
    monaco.editor.setModelLanguage(editor.getModel(), 'logs');
  };
}
