import { TranslateService } from '@ngx-translate/core';
import { ActivatedRoute, Router } from '@angular/router';
import { environment } from 'src/environments/environment';
import { Component, OnInit, ViewChild } from '@angular/core';
import { ToolsEnabledComponent } from './tools-enabled/tools-enabled.component';
import { ControlsEnabledComponent } from './controls-enabled/controls-enabled.component';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { availableControls } from 'src/app/vitis/map-core/services/map-models.model';
import { VitisRequest, vitisAlert, VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

@Component({
  selector: 'app-map-tools-designer',
  templateUrl: './map-tools-designer.component.html',
  styleUrls: ['./map-tools-designer.component.scss']
})
export class MapToolsDesignerComponent implements OnInit {

  @ViewChild(ToolsEnabledComponent) oToolsEnabled: ToolsEnabledComponent;
  @ViewChild(ControlsEnabledComponent) oControlsEnabled: ControlsEnabledComponent;

  public mode: string;
  public oFormValues: any = {};

  private iMapId: number;

  /**
   * Constructeur
   */
  constructor(
    private route: ActivatedRoute,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private translate: TranslateService,
    private formreaderSrvc: FormreaderService,
    private router: Router
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {

    // Traductions (obligatoire sinon ne fonctionne pas dans le module en lazy loading)
    this.translate.addLangs(['en', 'fr']);
    this.translate.setDefaultLang('en');
    this.translate.use(environment.properties.language ? environment.properties.language : 'en');

    this.route.data.subscribe(oMode => {
      this.route.params.subscribe(oParams => {
        if (this.vitisCheck.isDefAndNotNull(oParams) && this.vitisCheck.isDefAndNotNull(oParams.id)) {
          this.iMapId = oParams.id;
        } else {
          vitisAlert.alert({
            type: 'error',
            title: 'Error'
          });
        }
      });
    });
  }

  /**
   * Retour à la liste.
   */
  public returnToList() {
    this.router.navigate(['/dashboard/admin/maps']);
  }

  /**
   * Récupère les contrôles du formulaire associé et formate le json à envoyer
   * @returns Objet contenant les contrôles à activer
   */
  private getControlsEnabled(): any {
    const aControlsEnabled: Array<string> = this.oControlsEnabled.oFormReader.oFormValues.value.controls_availables;
    const oControls: any = {};
    availableControls.forEach(sControl => {
      if (sControl != '') {
        oControls[sControl] = aControlsEnabled.indexOf(sControl) !== -1;
      }
    });

    return oControls;
  }

  /**
   * Fonction retournant les outils de la carte.
   * @returns Objet contenant les outils de la carte
   */
  private getToolsEnabled(): any {
    const oToolsEnabled: any = this.oToolsEnabled.oMapTools;
    return oToolsEnabled;
  }

  /**
   * Envoyer les données
   */
  public postData() {
    const oControls: any = this.getControlsEnabled();
    const oTools: any = this.getToolsEnabled();
    const aData: any = {
      maptools: {
        controls: oControls,
        tools: oTools
      }
    };

    this.vitisRequest.ajaxRequestPromise({
      method: 'PUT',
      url: environment.properties.app_api_url + '/vmap/maps/' + this.iMapId,
      data: this.formreaderSrvc.transformToDataForm(aData, true)
    }).then((resp) => {
      if (resp.data) {
        if (resp.data.status === 1) {
          // Message succès
          this.translate.get('COMPONENT_MAP_ENABLE_UTILS_SUCCESS_TOAST').subscribe((translation) => {
            vitisAlert.fireToast({
              icon: 'success',
              title: translation
            });
          });
        } else {
          // Message d'erreur.
          this.translate.get('COMPONENT_MAP_ENABLE_UTILS_ERROR_TOAST').subscribe((translation) => {
            vitisAlert.alert({
              type: 'error',
              title: translation
            });
          });
        }
      }
    });
  }

}
