import { JsonSymbologyStyle } from 'src/modules/vmap/components/models/json-symbology/json-symbology-style.model';
import { JsonSymbologyLabel } from './json-symbology-label.model';
import { JsonSymbologyOpacity, JsonSymbologyUnits, JsonSymbologyVisibility } from './json-symbology-params.model';


export class ClassSubFields {
  opacity?: JsonSymbologyOpacity;
  visibility?: JsonSymbologyVisibility;
  units?: JsonSymbologyUnits;
}

export class JsonSymbology {
  type?: string; // singleclass, valueclass, range, simpleexpression, advancedexpression, cluster
  classes?: Array<JsonSymbologyClass>;

  otherFields?: ClassSubFields = {};
  symbolscaledenom?: number;

  /**
   * Constructeur
   */
  constructor(oDefSym: any = {}) {
    this.otherFields = new ClassSubFields();
    oDefSym.otherFields = oDefSym.otherFields ?? new ClassSubFields(); // Evite le oDefSym.otherFields undefined
    this.otherFields.visibility = new JsonSymbologyVisibility(oDefSym.otherFields.visibility ? oDefSym.otherFields.visibility : {});
    this.symbolscaledenom = oDefSym.symbolscaledenom;
  }
}

export class ExpressionModel {
  operator?: string;
  sql_operator?: string;
  value?: string;
  table?: string;

  /**
   * Constructeur
   */
  constructor(obj: any = {}) {
    this.operator = obj.operator || null;
    this.sql_operator = obj.sql_operator || null;
    this.value = obj.value || null;
    this.table = obj.table || null;
  }
}

export class JsonSymbologyClass extends JsonSymbology {
  id?: number; // Utile si plusieurs classe
  legendName: string; // Nom de la légende
  expression?: string | Array<any>; // expression
  hasLabel: boolean; // Affiche un libellé
  hasStyle: boolean; // Affiche un style carto
  label?: JsonSymbologyLabel; // Définition de la classe
  styles?: Array<any>; // Définition des styles
  opacity?: number; // OPACITY (défaut 1)
  minscale?: number; // MINSCALEDENOM
  maxscale?: number; // MAXSCALEDENOM
  geomType?: string;
  prevGeomType: string; // Permet de connaitre l'avant dernier type de géométrie après un changement de géométrie
  symbolscaledenom?: number;

  /**
   * Constructeur
   */
  constructor(oDefSym: any = {}, typeClass) {
    super(oDefSym);
    this.styles = [];
    if (oDefSym) {
      if (typeClass == 'expression' || typeClass == 'cluster') {
        this.expression = '';
        if (oDefSym.expression) {
          if (typeof oDefSym.expression == 'object' && oDefSym.expression.length > 0) {
            this.expression = [];
            oDefSym.expression.forEach(expression => {
              this.expression = [...this.expression, new ExpressionModel(expression)];
            });
          } else {
            this.expression = oDefSym.expression;
          }
        }
        this.id = oDefSym.id;
      } else {
        this.type = oDefSym.type || 'singleclass';
        this.opacity = oDefSym.opacity || 1;

        if (oDefSym.minscale) {
          this.minscale = oDefSym.minscale;
        }

        if (oDefSym.maxscale) {
          this.maxscale = oDefSym.maxscale;
        }
      }
      this.legendName = oDefSym.legendName || '';
      this.hasLabel = oDefSym.hasLabel;
      this.hasStyle = oDefSym.hasStyle;
      this.geomType = oDefSym.geomType;
      this.prevGeomType  = oDefSym.prevGeomType ;
      this.label = new JsonSymbologyLabel(oDefSym.label || {});
      this.symbolscaledenom = oDefSym.symbolscaledenom;
      if (oDefSym.styles) {
        oDefSym.styles.forEach((style, index) => {
          style.id = index + 1;
          style.hasStyle = style.hasStyle ? true : false;
          this.styles.push(new JsonSymbologyStyle(style));
        });
      }

      // Unité.
      if (typeof (oDefSym.otherFields.units) != 'undefined') {
        this.otherFields.units = new JsonSymbologyUnits(oDefSym.otherFields.units);
      }

      // Opacité.
      if (typeof (oDefSym.otherFields.opacity) != 'undefined') {
        this.otherFields.opacity = new JsonSymbologyOpacity(oDefSym.otherFields.opacity);
      }
    }
  }
}
