import { clusterExpression } from '../definition-sym-json';
import { JsonSymbology, JsonSymbologyClass } from './json-symbology-class.model';
import { JsonSymbologyLabel } from './json-symbology-label.model';
import { JsonSymbologyOpacity, JsonSymbologyUnits, JsonSymbologyVisibility } from './json-symbology-params.model';

export class ClusterSubFields {
  opacity?: JsonSymbologyOpacity;
  visibility?: JsonSymbologyVisibility;
  cluster?: JsonSymbologyClusterDef;
  units?: JsonSymbologyUnits;
}

/**
 * JSON SYMBOLOGY SIMPLE CLUSTER MODEL
 */
export class JsonSymbologyCluster extends JsonSymbology {
  classes: Array<JsonSymbologyClass>;

  otherFields?: ClusterSubFields = {};
  symbolscaledenom?: number;
  labelitem = 'Cluster_FeatureCount';
  classitem = 'Cluster_FeatureCount';
  geomType?: string;

  /**
   * Constructeur
   */
  constructor(oDefSym: any = {}) {
    super();

    this.type = 'cluster';

    this.classes = [];
    let isPresentDefaultExpression = false;
    // Si les classes existe déjà on les récupères, sinon on créer les 2 par défaut
    if (oDefSym.classes && oDefSym.classes.length > 0) {
      oDefSym.classes.forEach((classDef, index) => {
        classDef.id = index;
        this.classes.push(new JsonSymbologyClass(classDef, 'expression'));
        if (classDef.expression == clusterExpression[0]) {
          isPresentDefaultExpression = true;
        }
      });

      if (isPresentDefaultExpression == false) {
        this.classes.unshift(this.getJsonSymbologyClassCluster(1, clusterExpression[0], 'Valeurs regroupées (Cluster)', this.getStyleCluster(), this.getLabelCluster()));
      }
    } else {
      this.classes.unshift(this.getJsonSymbologyClassCluster(1, clusterExpression[0], 'Valeurs regroupées (Cluster)', this.getStyleCluster(), this.getLabelCluster()));
      this.classes.push(this.getJsonSymbologyClassCluster(2, clusterExpression[1], 'Valeurs non regroupées', this.getStyleClusterUnique()));
    }

    this.otherFields = new ClusterSubFields();
    oDefSym.otherFields = oDefSym.otherFields ?? new ClusterSubFields(); // Evite le oDefSym.otherFields undefined
    this.otherFields.visibility = new JsonSymbologyVisibility(oDefSym.otherFields.visibility ? oDefSym.otherFields.visibility : {});

    // Opacité.
    if (typeof(oDefSym.otherFields.opacity) != 'undefined') {
      this.otherFields.opacity = new JsonSymbologyOpacity(oDefSym.otherFields.opacity);
    }

    // Unité.
    if (typeof(oDefSym.otherFields.units) != 'undefined') {
      this.otherFields.units = new JsonSymbologyUnits(oDefSym.otherFields.units);
    }

    this.symbolscaledenom = oDefSym.symbolscaledenom;

    this.otherFields.cluster = new JsonSymbologyClusterDef(oDefSym.otherFields.cluster ? oDefSym.otherFields.cluster : {});
    this.geomType = oDefSym.geomType;
  }

  /**
   * Retourne une JsonSymbologyClass pour cluster
   */
  private getJsonSymbologyClassCluster(id: number, expression: string|any, name: string, styles?: Array<any>, label?: JsonSymbologyLabel): JsonSymbologyClass {
    return new JsonSymbologyClass({
      id: id,
      legendName: name,
      expression: expression,
      styles: styles,
      label: label,
      hasLabel: label? true: false
    }, 'expression');
  }

  /**
   * retourne le style par défaut pour les cluster
   */
  private getStyleCluster(): Array<any> {
    return [
      {
        'otherFields': {
          'symbolOutline': {
            'outlineColor': '#fff0f0',
            'width': '2'
          }
        },
        'id': 1,
        'hasStyle': true,
        'symbol': 'circle',
        'fillcolor': '#ff0000',
        'size': '50'
      }
    ];
  }

  /**
   * retourne le label par défaut pour les cluster
   */
  private getLabelCluster(): JsonSymbologyLabel {
    return {
      'otherFields': {},
      'text': '',
      'font': 'arial',
      'size': '15',
      'symbol': '',
      'fillcolor': '#ffffff',
      'position': 'cc'
    };
  }

  /**
   * retourne le style par défaut pour les points non regroupé
   */
  private getStyleClusterUnique(): Array<any> {
    return [
      {
        'otherFields': {},
        'id': 1,
        'hasStyle': true,
        'symbol': 'circle',
        'fillcolor': '#000000',
        'size': '10'
      }
    ];
  }
}

export class JsonSymbologyClusterDef { // Cluster
  maxdistance: number;
  region: string;

  /**
   * Constructeur
   */
  constructor(style: any = {}) {
    if (style) {
      this.maxdistance = style.maxdistance || 50;
      this.region = style.region || 'ellipse';
    }
  }
}


