import { clusterExpression } from '../definition-sym-json';
import { JsonSymbology, JsonSymbologyClass } from './json-symbology-class.model';
import { JsonSymbologyOpacity, JsonSymbologyUnits, JsonSymbologyVisibility } from './json-symbology-params.model';


export class ExpressionSubFields {
  opacity?: JsonSymbologyOpacity;
  visibility?: JsonSymbologyVisibility;
  units?: JsonSymbologyUnits;
}

/**
 * JSON SYMBOLOGY SIMPLE EXPRESSION MODEL
 */
export class JsonSymbologyExpression extends JsonSymbology {
  classes: Array<JsonSymbologyClass>;

  otherFields?: ExpressionSubFields = {};
  symbolscaledenom?: number;
  geomType?: string;

  /**
   * Constructeur
   */
  constructor(oDefSym: any = {}) {
    super();

    this.type = 'expression';

    this.classes = [];
    if (oDefSym.classes) {
      oDefSym.classes.forEach((classDef, index) => {
        // En mode expression on retire les expressions avec [Cluster_FeatureCount] car cette valeur n'est disponible que en cluster
        if (!classDef.expression.includes('[Cluster_FeatureCount]')) {
          classDef.id = index;
          this.classes.push(new JsonSymbologyClass(classDef, 'expression'));
        }
      });
    }

    this.otherFields = new ExpressionSubFields();
    oDefSym.otherFields = oDefSym.otherFields ?? new ExpressionSubFields(); // Evite le oDefSym.otherFields undefined
    this.otherFields.visibility = new JsonSymbologyVisibility(oDefSym.otherFields.visibility ? oDefSym.otherFields.visibility : {});

    // Opacité.
    if (typeof(oDefSym.otherFields.opacity) != 'undefined') {
      this.otherFields.opacity = new JsonSymbologyOpacity(oDefSym.otherFields.opacity);
    }

    // Unité.
    if (typeof(oDefSym.otherFields.units) != 'undefined') {
      this.otherFields.units = new JsonSymbologyUnits(oDefSym.otherFields.units);
    }

    this.symbolscaledenom = oDefSym.symbolscaledenom;
    this.geomType = oDefSym.geomType;
  }
}
