/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  scope.initEvent = function () {
    scope.oFormValues.controls.definition_mapserver.valueChanges.subscribe(value => {
      // Affichage dynamique de l'url du flux si il en possède un
      scope.getFormField('definition_mapserver').visible = services.vitisCheck.isDefAndNotNull(value);
    });
  }

  scope.generateMapfile = function () {
    if (scope.oFormValues.value.publications.length > 0) {
      // Génération du map file.
      return services.vitisRequest.ajaxRequestPromise({
        method: 'POST',
        url: `${env.properties.app_api_url}/vmap/flux/${scope.oFormValues.value.flux_id}/generateMapfile`,
        data: {}
      }).then((resp) => {
        if (resp.data) {
          if (resp.data.status === 1) {
            return services.SimpleformComponent.sendSimpleForm({
              ...scope.oFormValues.value,
              ...scope.getFormValues(true),
              publications: JSON.stringify(scope.oFormValues.value.publications),
              definition_mapserver: resp.data.data.flux_url
            }).then(() => {
              services.SimpleformComponent.setMode('update');
            });
          } else {
            // Message error
            return services.vitisAlert.alert({
              type: 'error',
              title: this.translate.instant('GENERATE_FLUX_MAP_FILE_ERROR'),
            });
          }
        }
      });
    } else {
      // Alerte aucune couche dans le service
      return services.vitisAlert.alert({
        type: 'error',
        title: this.translate.instant('GENERATE_FLUX_MAP_FILE_NO_PUBLICATIONS'),
      });
    }
  }

  scope.openGetCapabilities = function () {
    const sUrl = scope.getFormField('definition_mapserver').getValue();
    // récupération du type de flux
    const oFluxtypeIdField = scope.getFormField('fluxtype_id');
    let sStreamType = 'WMS'
    if (services.vitisCheck.isDefAndNotNull(oFluxtypeIdField) &&
      services.vitisCheck.isArray(oFluxtypeIdField.options.choises) &&
      oFluxtypeIdField.options.choises.length > 0) {
      const oFlowType = oFluxtypeIdField.options.choises.find((element) => element.value == String(oFluxtypeIdField.getValue()));
      if (services.vitisCheck.isDefAndNotNull(oFlowType)) {
        sStreamType = oFlowType.label;
      }
    }

    if (services.vitisCheck.isDefAndNotNull(sUrl)) {
      const sVersion = (sStreamType === 'WFS') ? '1.1.0' : '1.3.0';

      const sGetCapabilitiesUrl = sUrl + '?SERVICE=' + sStreamType + '&VERSION=' + sVersion + '&REQUEST=GetCapabilities';
      window.open(sGetCapabilitiesUrl, '_blank').focus();
    } else {
      services.vitisAlert.alert({
        type: 'error',
        title: this.translate.instant('GENERATE_FLUX_NO_URL_FOR_GETCAPABILITIES'),
      })
    }
  }

  scope.updateflux = function () {
    return services.SimpleformComponent.sendSimpleForm({
      ...scope.oFormValues.value,
      ...scope.getFormValues(true),
      publications: JSON.stringify(scope.oFormValues.value.publications)
    });
  }
}