/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  var iGeomPoint = 0;
  var iGeomLine = 0;
  var iGeomPolygon = 0;
  var iGeomGeometry = 0;
  const aVmapColumnTypeList = [{
    key: 'string',
    label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_TEXT_LABEL'
  },
  {
    key: 'number',
    label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_NUMBER_LABEL'
  },
  {
    key: 'bool',
    label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_BOOLEAN_LABEL'
  },
    // {
    //     key: 'image',
    //     label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_IMAGE_LABEL'
    // },
    // {
    //     key: 'document',
    //     label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_DOCUMENT_LABEL'
    // },
    // {
    //     key: 'link',
    //     label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_LINK_LABEL'
    // },
    // {
    //     key: 'date',
    //     label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_DATE_LABEL'
    // }
  ];

  scope.initFormLayerPG = function () {
  }
  /**
  * Event lancé avant l'envoi du formulaire d'update
  */
  scope.beforeEventUpdate = function (oFormValues) {
    return new Promise((resolve, reject) => {
      scope.initValueLibelleFormUpdate(oFormValues);
      setTimeout(() => {
        resolve();
      }, 100);
    });
  }
  /**
   * Event lancé avant l'envoi du formulaire d'insert
   */
  scope.beforeEventInsert = function (oFormValues) {
    return new Promise((resolve, reject) => {
      scope.initSymbologie();
      // Attendre que la promesse de initValueLibelleForm soit résolue
      scope.initValueLibelleForm(oFormValues).then(() => {
        setTimeout(() => {
          resolve();
        }, 100);
      }).catch((error) => {
        reject(error);
      });
    });
  }

  /**
   * Met en place une symbologie par défaut
   */
  scope.initSymbologie = function () {
    const sPoint = JSON.stringify(env.properties.default_symbology_point);
    const sLine = JSON.stringify(env.properties.default_symbology_line);
    const sPolygon = JSON.stringify(env.properties.default_symbology_polygon);
    const sGeometry = JSON.stringify(env.properties.default_symbology_geometry);
    const acualSym = scope.getFormField('definition_sym_json').getValue();
    let geom = scope.getFormField('ms_geomtype_id').getValue();
    geom = parseInt(geom);

    if (services.vitisCheck.isDefAndNotNull(geom) && !services.vitisCheck.isDefAndNotNull(acualSym)) {
      let sSymbologie;
      switch (geom) {
        case iGeomPoint:
          sSymbologie = sPoint;
          break;
        case iGeomLine:
          sSymbologie = sLine;
          break;
        case iGeomPolygon:
          sSymbologie = sPolygon;
          break;
        case iGeomGeometry:
          sSymbologie = sGeometry;
          break;
      }
      if (sSymbologie) {
        const oSymbologie = JSON.parse(sSymbologie);
        oSymbologie.autogenerate = true;
        const sSymbologieAutogenerate = JSON.stringify(oSymbologie);
        scope.getFormField('definition_sym_json').setValue(sSymbologieAutogenerate);
      }
    }
  }
  scope.openJsonColumnsModal = function () {
    let sForm = '';
    let oFormValues = scope.oFormValues.value;

    const params = {
      database: oFormValues.database,
      schema: oFormValues.schema,
      table: oFormValues.pg_table,
      datasource_id: oFormValues.datasource_id
    };

    // Si une table specifique est utilisé pour le requeteur
    if (oFormValues.is_specific_table_requestor) {
      if (!services.vitisCheck.isDefAndNotNull(oFormValues.schema_requestor) ||
        oFormValues.schema_requestor == '' ||
        !services.vitisCheck.isDefAndNotNull(oFormValues.table_requestor) ||
        oFormValues.table_requestor == ''
      ) {
        services.translate.get('PG_LAYER_JSON_COLUMNS_MISSING_PARAMS').subscribe(async (translation) => {
          services.vitisAlert.alert({
            text: translation,
            showCancelButton: true
          });
        });
        return;
      } else {
        params.schema = oFormValues.schema_requestor;
        params.table = oFormValues.table_requestor;
      }
    }

    services.vitisRequest.ajaxRequest({
      method: 'GET',
      url: `${env.properties.app_api_url}/vmap/pglayers/columns`,
      params: params,
      success: async (resp) => {
        if (services.vitisCheck.isDefAndNotNull(resp.data)) {
          const oResult = resp.data;
          // $(document).off('click', '#table_field_list_selectAll');
          if (oResult.status == 1 && oResult.data && oResult.data.length >= 0) {
            // Déclaration function pour select all
            // $(document).on('click', '#table_field_list_selectAll', function () {
            //     if (document.getElementById("table_field_list_selectAll").checked === true) {
            //         for (let i = 0; i < oResult.data.length; i++) {
            //             if ((document.getElementById('table_field_list_' + oResult.data[i].column_name))) {
            //                 (document.getElementById('table_field_list_' + oResult.data[i].column_name)).checked = true;
            //             } else {
            //                 console.error('Erreur dans le chargement de la checbox: ', oResult.data[i].column_name);
            //             }
            //         }
            //     } else {
            //         for (let i = 0; i < oResult.data.length; i++) {
            //             if ((document.getElementById('table_field_list_' + oResult.data[i].column_name))) {
            //                 (document.getElementById('table_field_list_' + oResult.data[i].column_name)).checked = false;
            //             } else {
            //                 console.error('Erreur dans le chargement de la checbox: ', oResult.data[i].column_name);
            //             }
            //         }
            //     }
            // });

            let disabled = '';
            if (scope.sFormDefinitionName == 'display') {
              disabled = 'disabled';
            }

            const needTranslates = [
              'DATASOURCE_FORM_PG_LAYER_JSON_COLUMNS_MODAL_TITLE',
              'PG_LAYER_JSON_COLUMNS_LABEL',
              'PG_LAYER_JSON_COLUMNS_ALL_LIST',
              'CONFIRM_CONFIRM',
              'CONFIRM_CANCEL',
              'STUDIO_COMPONENT_LABEL_PLACEHOLDER',
              'PG_LAYER_JSON_COLUMNS_INDEX',
              'PG_LAYER_JSON_COLUMNS_TYPE'
            ];
            for (let i = 0; i < aVmapColumnTypeList.length; i++) {
              needTranslates.push(aVmapColumnTypeList[i].label);
            }

            services.translate.get(needTranslates).subscribe(async (translates) => {
              // Préparation du contenu de la modale
              let sForm = `
                <div style="text-align: left; height: 100%">
                  <!--Sélectionner tous-->
                  <!--
                  <div class="row">
                    <div class="checkbox checkbox-info margin-2 col-12" style="position: relative">
                      <input id="table_field_list_selectAll" type="checkbox" style="cursor:pointer; position: absolute;" ${disabled} />
                      <label for="table_field_list_selectAll" style="cursor:pointer"> ${translates['PG_LAYER_JSON_COLUMNS_ALL_LIST']} </label>
                    </div>
                  </div>
                  -->
                  <div class="row">
                    <div class="mb-2 col-2" style="position: relative">
                    </div>
                    <div class="mb-2 col-1" style="position: relative; padding: 0px; text-align: center;">
                    Requêteur
                    </div>
                    <div class="mb-2 col-1" style="position: relative; padding: 0px; text-align: center;">
                    Filtre
                    </div>
                    <div class="mb-2 col-2" style="position: relative">
                    </div>
                    <div class="mb-2 col-3" style="position: relative">
                    </div>
                    <div class="mb-2 col-2" style="position: relative">
                    </div>
                  </div>
                  <!--Liste-->
                  <div class="scrollbar-container">
              `;

              for (let i = 0; i < oResult.data.length; i++) {
                const column = oResult.data[i];
                let index = -1;
                let json_columns = [];
                if (oFormValues.json_columns && oFormValues.json_columns.length > 0) {
                  json_columns = [...JSON.parse(oFormValues.json_columns)];
                  for (let j = 0; j < json_columns.length; j++) {
                    if (json_columns[j].column == column.column_name) {
                      index = j;
                    }
                  }
                }

                sForm += `
                  <div class="row">
                    <!-- Nom colonne -->
                    <div class="checkbox checkbox-info mb-2 col-2" style="position: relative; text-align: right;">
                      <label> ${column.column_name} </label>
                    </div>
                    <!-- Checkbox requêteur -->
                    <div class="checkbox checkbox-info mb-2 col-1" style="position: relative; text-align: center;">
                      <input id="table_field_list_${column.column_name}" 
                        type="checkbox" 
                        style="cursor:pointer; position: absolute"
                        ${((index !== -1) ? json_columns[index].requestor === true ? 'checked' : '' : '')}
                        ${disabled}>
                      <label for="table_field_list_${column.column_name}"></label>
                    </div>
                    <!-- Checkbox filtre -->
                    <div class="checkbox checkbox-info mb-2 col-1" style="position: relative; text-align: center;">
                      <input id="table_field_filter_${column.column_name}" 
                        type="checkbox" 
                        style="cursor:pointer; position: absolute"
                        ${((index !== -1) ? json_columns[index].filter === true ? 'checked' : '' : '')}
                        ${disabled}>
                      <label for="table_field_filter_${column.column_name}"></label>
                    </div>
                    <!-- Label -->
                    <div class="input-group input-group-sm col-3 mb-2">
                      <div class="input-group-prepend">
                        <label class="input-group-text" id="table_field_label_label"> ${translates['PG_LAYER_JSON_COLUMNS_LABEL']} </label>
                      </div>
                      <input id="table_field_label_input_${column.column_name}" 
                        type="text" 
                        class="form-control" 
                        placeholder="${translates['STUDIO_COMPONENT_LABEL_PLACEHOLDER']}" 
                        required="" 
                        value="${((index !== -1) ? json_columns[index].name : column.column_name)}"
                        ${disabled}>
                    </div>
                    <!-- Type -->
                    <div class="input-group input-group-sm col-3 mb-2">
                      <div class="input-group-prepend">
                        <label class="input-group-text" id="table_field_index_label"> ${translates['PG_LAYER_JSON_COLUMNS_TYPE']} </label>
                      </div>
                      <select class="custom-select custom-select-sm"
                        id="table_field_type_input_${column.column_name}"
                        ${disabled}>`;

                // Contenu du select
                for (let k = 0; k < aVmapColumnTypeList.length; k++) {
                  const type = aVmapColumnTypeList[k];
                  if (index !== -1 && json_columns[index].type == type.key) {
                    sForm += '<option value="' + type.key + '" selected>' + translates[type.label] + '</option>';
                  } else {
                    sForm += '<option value="' + type.key + '">' + translates[type.label] + '</option>';
                  }
                }

                sForm += `  </select>
                  </div>
                  <!-- Index -->
                  <div class="input-group input-group-sm col-2 mb-2">
                    <div class="input-group-prepend">
                      <label class="input-group-text" id="table_field_index_label"> ${translates['PG_LAYER_JSON_COLUMNS_INDEX']} </label>
                    </div>
                    <input id="table_field_index_input_${column.column_name}" 
                      type="text" 
                      class="form-control" 
                      required=""
                      value="${((index !== -1) ? json_columns[index].index : i)}"
                      ${disabled}>
                  </div>
                </div>`;
              }


              sForm += `</div>
                                </div>`;

              // Si + de 13 éléments on ajoute le style pour la scrollbar
              if (oResult.data.length > 13) {
                sForm += '<style>';
                sForm += '.swal2-popup.swal2-modal.vitis-show { height: 80% }';
                sForm += '.swal2-popup.swal2-modal.vitis-show .swal2-content { height: 85% }';
                sForm += '.swal2-popup.swal2-modal.vitis-show .swal2-content .swal2-html-container { height: 100% }';
                sForm += '.scrollbar-container { overflow-x: scroll; height: 95% }';
                sForm += '.scrollbar-container .row { width: 101% }';
                sForm += '</style>';
              }

              // Ouverture de la modale
              const {
                value: aAlertFormValues
              } = await services.vitisAlert.alert({
                title: translates['DATASOURCE_FORM_PG_LAYER_JSON_COLUMNS_MODAL_TITLE'],
                width: 1200,
                html: sForm,
                focusConfirm: false,
                showCancelButton: true,
                confirmButtonText: translates['CONFIRM_CONFIRM'],
                cancelButtonText: translates['CONFIRM_CANCEL'],
                preConfirm: () => {
                  const preConfirm = [];
                  for (let i = 0; i < oResult.data.length; i++) {
                    preConfirm.push({
                      column: oResult.data[i],
                      requestor: (document.getElementById('table_field_list_' + oResult.data[i].column_name)).checked,
                      filter: (document.getElementById('table_field_filter_' + oResult.data[i].column_name)).checked,
                      value: (document.getElementById('table_field_label_input_' + oResult.data[i].column_name)).value,
                      index: (document.getElementById('table_field_index_input_' + oResult.data[i].column_name)).value,
                      type: (document.getElementById('table_field_type_input_' + oResult.data[i].column_name)).value,
                    });
                  }
                  return preConfirm;
                }
              });

              // Résultat de la modal
              if (aAlertFormValues) {
                const json_columns = [];
                for (let i = 0; i < aAlertFormValues.length; i++) {
                  const data = aAlertFormValues[i];
                  json_columns.push({
                    requestor: data.requestor,
                    filter: data.filter,
                    column: data.column.column_name,
                    name: data.value,
                    index: data.index,
                    type: data.type
                  });
                }

                scope.oFormValues.patchValue({
                  json_columns: JSON.stringify(json_columns)
                });
              }
            });
          } else {
            console.error('Impossible de récupérer les champs de la table');
            services.translate.get('PG_LAYER_JSON_COLUMNS_RESPONSE_NO_DATA').subscribe(async (translation) => {
              services.vitisAlert.alert({
                text: translation,
                showCancelButton: true
              });
            });
          }
        } else {
          console.error('Impossible de récupérer les champs de la table - resp', resp);
        }
      }
    });
  }

  /**
   * Chargement des infos WMS
   */
  scope.wmsPrepareData = function (oFormValues) {
    this.oOriginalFormValues = { ...oFormValues };
    const sLayerName = oFormValues.layer_name;
    setTimeout(() => {
      const oDatasource = scope.getFormField('datasource').getValue();

      if (oDatasource &&
        oDatasource.flux_url &&
        oDatasource.flux_version
      ) {
        scope.wmsGetCapabilities(oDatasource).then((oCapabilities) => {
          if (oCapabilities &&
            oCapabilities.json &&
            oCapabilities.json.Capability &&
            oCapabilities.json.Capability.Layer) {

            const oLayerOptionsField = scope.getFormField('layer_options');
            oLayerOptionsField.setValue(JSON.stringify(oCapabilities.json));

            // Remplit les listes déroulantes avec le contenu du getCapabilities
            scope.wmsFillSelectFields(oCapabilities.json, sLayerName);
          } else {
            console.error('Capabilities not valid', oCapabilities)
          }
        })
      } else {
        console.error('datasource not valid', oDatasource);
      }
      // Nom et url de la source.
      scope.setSourceFormField(oDatasource, 'wms');
    }, 500);
  }

  /**
   * Recharge les listes déroulantes lorsque l'utilisateur change de couche
   */
  scope.wmsLayerNameChange = function () {
    const sLayerNameVal = scope.getFormField('layer_name').getValue();
    const oLayerOptionsVal = scope.getFormField('layer_options').getValue();

    if (this.vitisCheck.isDefAndNotNull(sLayerNameVal) && sLayerNameVal != 'undefined') {
      let oJSONCapabilities;
      try {
        oJSONCapabilities = JSON.parse(oLayerOptionsVal)
      } catch (error) { }

      if (oJSONCapabilities) {
        // Ajout setTimeout pour que le champ du formulaire ai eu le temps de changer de valeurs
        setTimeout(() => scope.wmsFillSelectFields(oJSONCapabilities, sLayerNameVal), 100);
      }
    }
  }

  /**
   * Remplit les listes déroulantes avec le contenu du getCapabilities
   * @param {*} oJSONCapabilities
   * @param {string} sLayerName
   */
  scope.wmsFillSelectFields = function (oJSONCapabilities, sLayerName) {
    let oLayerNameField = scope.getFormField('layer_name');
    const aLayers = services.WmsService.searchLayers(oJSONCapabilities.Capability.Layer);

    // Remplit la liste des couches
    services.AdminDatasourceFormService.prepareSelectForm(aLayers, oLayerNameField, ['Title'], 'Name');

    // Couche sélectionnée
    const oSelectedLayer = aLayers.find((oLayer) => oLayer.Name === sLayerName);

    if (this.vitisCheck.isDefAndNotNull(oSelectedLayer)) {
      // Remplit la liste des styles
      if (oSelectedLayer.Style) {
        services.AdminDatasourceFormService.prepareSelectForm(oSelectedLayer.Style, scope.getFormField('layer_style'), ['Title'], 'Name');
      }

      // Remplit la liste des formats
      if (oSelectedLayer.Format) {
        services.AdminDatasourceFormService.prepareSelectForm(oSelectedLayer.Format, scope.getFormField('layer_format'));
      }

      // Remplit la liste des CRS
      if (oSelectedLayer.CRS || oSelectedLayer.SRS) {
        const aAvailableCoordsys = oSelectedLayer.CRS ? oSelectedLayer.CRS : oSelectedLayer.SRS
        scope.getFormField('crs_list').setValue(aAvailableCoordsys.join('|'));
      }
    } else {
      console.error('La couche ' + sLayerName + ' n\'a pas été trouvée');
    }

    // Uniquement si mode diaplay.
    if (scope.oParams.formName == 'display') {
      setTimeout(() => {
        // Select non modifiables.
        oLayerNameField.disabled = true;
        let oLayerStyleField = scope.getFormField('layer_style');
        oLayerStyleField.disabled = true;
        let oLayerFormatField = scope.getFormField('layer_format');
        oLayerFormatField.disabled = true;

        // Application des valeurs sauvées avant suppression.
        const oFormFields = scope.getFormFields();
        for (let i = 0; i < oFormFields.length; i++) {
          if (this.vitisCheck.isDefAndNotNull(this.oOriginalFormValues[oFormFields[i].name])) {
            oFormFields[i].setValue(this.oOriginalFormValues[oFormFields[i].name]);
          }
        }
      });
    }
  }

  /**
   * Effecture la requête getCapabilities
   * @param {*} oDatasource 
   * @returns 
   */
  scope.wmsGetCapabilities = function (oDatasource) {
    return new Promise((resolve, reject) => {
      services.WmsService.getAjaxCapabilitiesWMS({
        flux_url: oDatasource.flux_url,
        flux_version: oDatasource.flux_version,
        flux_login: oDatasource.flux_login,
        flux_password: oDatasource.flux_password,
        headers: oDatasource.headers
      }).then((oCapabilities) => {
        if (oCapabilities !== false) {
          resolve(oCapabilities);
        } else {
          services.translate.get('CANT_REFRESH_CAPABILITIES').subscribe((sTranslate) => {
            services.vitisAlert.fireToast({
              type: 'warning',
              title: sTranslate
            });
          });
          reject();
        }
      });

    });
  }

  /**
   * Chargement des infos WMTS
   */
  scope.wmtsPrepareData = function () {
    setTimeout(() => {
      const oDatasource = scope.getFormField('datasource').getValue();

      if (oDatasource &&
        oDatasource.flux_url &&
        oDatasource.flux_version
      ) {
        scope.wmtsGetCapabilities(oDatasource).then((oCapabilities) => {
          if (oCapabilities &&
            oCapabilities.Contents &&
            oCapabilities.Contents.Layer) {

            const oLayerOptionsField = scope.getFormField('layer_options');
            oLayerOptionsField.setValue(JSON.stringify(oCapabilities));

            // Remplit les listes déroulantes avec le contenu du getCapabilities
            scope.wmtsFillSelectFields(oCapabilities);

          } else {
            console.error('Capabilities not valid', oCapabilities)
          }
        })
      } else {
        console.error('datasource not valid', oDatasource);
      }
      // Nom et url de la source.
      scope.setSourceFormField(oDatasource, 'wmts');
    }, 500);
  }

  /**
   * Recharge les listes déroulantes lorsque l'utilisateur change de couche
   */
  scope.wmtsLayerNameChange = function () {
    const sLayerNameVal = scope.getFormField('layer_name').getValue();
    const oLayerOptionsVal = scope.getFormField('layer_options').getValue();

    if (this.vitisCheck.isDefAndNotNull(sLayerNameVal) && sLayerNameVal != 'undefined') {
      let oJSONCapabilities;
      try {
        oJSONCapabilities = JSON.parse(oLayerOptionsVal)
      } catch (error) { }

      if (oJSONCapabilities) {
        // Ajout setTimeout pour que le champ du formulaire ai eu le temps de changer de valeurs
        setTimeout(() => scope.wmtsFillSelectFields(oJSONCapabilities), 100);
      }
    }
  }

  /**
   * Remplit les listes déroulantes avec le contenu du getCapabilities
   * @param {*} oJSONCapabilities 
   */
  scope.wmtsFillSelectFields = function (oJSONCapabilities) {
    const oLayerNameField = scope.getFormField('layer_name');
    const aLayers = services.WmtsService.searchLayers(oJSONCapabilities.Contents.Layer);

    // Remplit la liste des couches
    services.AdminDatasourceFormService.prepareSelectForm(aLayers, oLayerNameField, ['Title'], 'Identifier');

    // Couche sélectionnée
    const oSelectedLayer = aLayers.find((oLayer) => oLayer.Identifier === oLayerNameField.getValue());

    if (this.vitisCheck.isDefAndNotNull(oSelectedLayer)) {
      // Remplit la liste des styles
      if (oSelectedLayer.Style) {
        services.AdminDatasourceFormService.prepareSelectForm(oSelectedLayer.Style, scope.getFormField('layer_style'), ['Title'], 'Identifier');
      }

      // Remplit la liste des formats
      if (oSelectedLayer.Format) {
        services.AdminDatasourceFormService.prepareSelectForm(oSelectedLayer.Format, scope.getFormField('layer_format'));
      }

      // Remplit la liste des matrices
      if (oSelectedLayer.TileMatrixSetLink) {
        const bSelectFirst = (scope.sFormDefinitionName === 'update' && (scope.getFormField('matrix_set').getValue() === null || scope.getFormField('matrix_set').getValue() === '')) ? false : true;
        services.AdminDatasourceFormService.prepareSelectForm(oSelectedLayer.TileMatrixSetLink, scope.getFormField('matrix_set'),
          ['TileMatrixSet', ' (', 'SupportedCRS', ')'], 'TileMatrixSet', bSelectFirst);
      }
    } else {
      console.error('La couche ' + oLayerNameField.getValue() + ' n\'a pas été trouvée');
    }
  }

  /**
   * Effecture la requête getCapabilities
   * @param {*} oDatasource 
   * @returns 
   */
  scope.wmtsGetCapabilities = function (oDatasource) {
    return new Promise((resolve, reject) => {
      services.WmtsService.getAjaxCapabilitiesWMTS({
        flux_type: oDatasource.flux_type,
        flux_url: oDatasource.flux_url,
        flux_version: oDatasource.flux_version,
        flux_login: oDatasource.flux_login,
        flux_password: oDatasource.flux_password,
        headers: oDatasource.headers,
      }).then((oCapabilities) => {
        if (oCapabilities !== false) {
          resolve(oCapabilities);
        } else {
          services.translate.get('CANT_REFRESH_CAPABILITIES').subscribe((sTranslate) => {
            services.vitisAlert.fireToast({
              type: 'warning',
              title: sTranslate
            });
          });
          reject();
        }
      });

    });
  }

  /**
   * Initialisation des champs de form. de la source.
   * @param {object} oDatasource 
   */
  scope.initFormLayerOsm = function (oFormValues) {
    // Nom et url de la source.
    scope.setSourceFormField(oFormValues.datasource, 'osm');
  }

  /**
   * Initialisation des champs de form. de la source.
   * @param {object} oDatasource 
   */
  scope.initFormLayerXyz = function (oFormValues) {
    // Nom et url de la source.
    scope.setSourceFormField(oFormValues.datasource, 'xyz');
  }

  /**
   * Initialisation des champs de form. de la source.
   * @param {object} oDatasource 
   * @param {string} sLayerType
   */
  scope.setSourceFormField = function (oDatasource, sLayerType) {
    // Nom et url de la source.
    if (this.vitisCheck.isDefAndNotNull(oDatasource)) {
      const oSourceUrl = scope.getFormField('source_url');
      const oSourceName = scope.getFormField('source_name');
      if (sLayerType == 'wms' || sLayerType == 'wmts') {
        oSourceUrl.setValue(oDatasource['flux_url']);
        oSourceName.setValue(oDatasource['name']);
      } else {
        var subscriptionSourceUrl = oSourceUrl.oninit.subscribe(() => {
          oSourceUrl.setValue(oDatasource['flux_url']);
          // Supprime la subscription à l'observable.
          subscriptionSourceUrl.unsubscribe();
        });
        var subscriptionSourceName = oSourceName.oninit.subscribe(() => {
          oSourceName.setValue(oDatasource['name']);
          // Supprime la subscription à l'observable.
          subscriptionSourceName.unsubscribe();
        });
      }
    }
  }

  /**
   * Label spécifique en mode display.
   * @param {object} oFormValues
   */
  scope.initEventLayerWmtsDisplay = function (oFormValues) {
    // Attend l'initialisation du champ "matrix_set".
    const oMatrixSetField = scope.getFormField('matrix_set');
    var subscriptionMatrixSet = oMatrixSetField.oninit.subscribe(() => {
      setTimeout(() => {
        // Ajoute les systèmes de coordonnées supportés par la matrice dans le libellé.
        const oMatrixSetElem = document.getElementById(oMatrixSetField.id);
        if (this.vitisCheck.isDefAndNotNull(oMatrixSetElem) && oMatrixSetElem.value != '') {
          const oCapabilities = scope.getFormValues().layer_options;
          if (this.vitisCheck.isDefAndNotNull(oMatrixSetElem) && oMatrixSetElem.value != '') {
            const oLayerTileMatrix = oCapabilities.Contents.TileMatrixSet.find((oTileMatrix) => oTileMatrix.Identifier == oMatrixSetElem.value);
            if (this.vitisCheck.isDefAndNotNull(oLayerTileMatrix)) {
              oMatrixSetField.setValue(`${oLayerTileMatrix.Identifier} (${oLayerTileMatrix.SupportedCRS})`);
            }
          }
        }
        // Supprime la subscription à l'observable.
        subscriptionMatrixSet.unsubscribe();
      });
    });
  }

  /**
   * Permet de charger un identifiant par défaut dans la colonne ID afin de limiter les confusions quant à l'utilisation de la colonne.
   */
  scope.loadDefaultId = (oFormValues) => {

    // Dans le cas ou le champ table s'appelle "pg_table"
    oFormValues['pg_table'] = oFormValues['pg_table'] ? oFormValues['pg_table'] : oFormValues['table'];

    if (!services.vitisCheck.isDefAndNotNull(oFormValues['database'])
      || !services.vitisCheck.isDefAndNotNull(oFormValues['schema'])
      || (!services.vitisCheck.isDefAndNotNull(oFormValues['table']) && !services.vitisCheck.isDefAndNotNull(oFormValues['pg_table']))) {
      console.error("Can't get default ID.");
      return;
    }

    services.vitisRequest.ajaxRequestPromise({
      method: 'GET',
      url: `${env.properties.app_api_url}/vmap/pglayers/columns_unique_key`,
      data: {
        ...oFormValues
      },
      httpMethodOverride: true
    })
      .then((data) => {
        if (services.vitisCheck.isDefAndNotNull(data)
          && services.vitisCheck.isDefAndNotNull(data.data)
          && services.vitisCheck.isDefAndNotNull(data.data.data)
          && services.vitisCheck.isDefAndNotNull(data.data.data.column_name)) {
          if (scope.getFormField('idfield')) {
            scope.getFormField('idfield').setValue(data.data.data.column_name);
          }
        }
      })
      .catch(() => {
        console.error("Can't get default ID.");
      })
  }

  /**
   * Lancer avant le chargement du formulaire .
   */
  scope.initEvent = function (oFormValues) {
    scope.getGeom(oFormValues);
    scope.loadDefaultId(oFormValues);
  }
  /**
   * Set différentes valeurs du formulaire avant son envoi .
   */
  scope.initValueLibelleForm = function (oFormValues) {
    return new Promise((resolve, reject) => {
      const sLayerName = scope.getFormField('name').getValue();
      const bGeometry = scope.getFormField('is_geometry_layer').getValue();

      if (bGeometry === true) {
        // Le test sur json_columns permet de savoir s'il s'agit d'une nouvelle couche et non d'un clone
        if (!services.vitisCheck.isDefAndNotNull(scope.getFormField('json_columns').getValue())) {
          scope.createJsonFilterRequestor(oFormValues)
            .then((jsonColumns) => {
              scope.getFormField('form_title').setValue(sLayerName);
              scope.getFormField('is_requestor_layer').setValue(bGeometry);
              scope.getFormField('is_filter').setValue(bGeometry);
              scope.getFormField('is_map_selection').setValue(bGeometry);
              scope.getFormField('is_data_grid').setValue(bGeometry);
              scope.getFormField('is_location').setValue(bGeometry);
              resolve();
            })
            .catch((error) => {
              console.error("Can't create filter");
              reject(error); // rejete la promesse si on n'a pas pu générer le json pour le filtre et requêteur
            });
          if (!services.vitisCheck.isDefAndNotNull(scope.getFormField('published_form_json_def').getValue())) {
            scope.createJsonForm(oFormValues).then(() => { }).catch((error) => {
              console.error("Can't create Form");
              reject(error); // rejete la promesse si on n'a pas pu générer les formulaires
            });
          }
        } else {
          services.oFormReader.setIsFormSubmitted();
          resolve();
        }
      } else {
        scope.getFormField('is_requestor_layer').setValue(bGeometry);
        scope.getFormField('is_map_selection').setValue(bGeometry);
        scope.getFormField('is_data_grid').setValue(bGeometry);
        scope.getFormField('is_location').setValue(bGeometry);
        scope.getFormField('is_filter').setValue(bGeometry);
        resolve(); // résout la promesse si la couche n'a pas de géométrie
      }
    });
  }
  /**
   * Lancer avant l'envoi du formulaire en update.
   */
  scope.initValueLibelleFormUpdate = function (oFormValues) {
    const bGeometry = scope.getFormField('is_geometry_layer').getValue();
    oFormValues.is_requestor_layer = bGeometry;
    oFormValues.is_filter = bGeometry;
    oFormValues.is_map_selection = bGeometry;
    oFormValues.is_data_grid = bGeometry;
    oFormValues.is_location = bGeometry;
  }
  /**
   * Lancer pour enregistrer le formulaire et rediriger l'utilisateur sur la symbologie ou sur information générale.
   */
  scope.eventInsert = function (oFormValues) {
    let oValues = services.oFormReader.getFormValues();
    let oData = {}
    for (const key in oValues) {
      if (services.vitisCheck.isDefAndNotNull(oValues[key])) {
        oData[key] = oValues[key];
      }
    }
    if (scope.isFormValid()) {
      let aSelect = [];
      scope.getListSearchMode().then(function (options) {
        aSelect = options;
        let iSearchMode;
        for (let i = 0; i < aSelect.length; i++) {
          if (aSelect[i].label === 'FORM_SEARCH_CENTER') {
            iSearchMode = aSelect[i].searchmode_id;
            break;
          }
        }
        oData['location_search_mode_id'] = iSearchMode;
        services.vitisRequest.ajaxRequestPromise({
          method: 'POST',
          url: env.properties.app_api_url + '/vmap/datasourcetreeview/pglayers?',
          data: oData
        }).then((resp) => {
          if (resp.data) {
            if (resp.data.status === 1 && services.vitisCheck.isDefAndNotNull(resp.data.data['layer_id'])) {
              let url;
              if (resp.data.data['is_geometry_layer'] === false) {
                url = '/dashboard/admin/layers/' + resp.data.data['layer_id'] + '/update/informations_generales'
              } else {
                url = '/dashboard/admin/layers/' + resp.data.data['layer_id'] + '/update/symbology'
              }
              services.router.navigate([url]);
            } else {
              services.vitisAlert.fireToast({
                type: 'error',
                title: this.translate.instant('CREATE_LAYER_ERROR'),
              });
            }
          }
        });
      }).catch(function (error) {
        console.error(error);
        services.vitisRequest.ajaxRequestPromise({
          method: 'POST',
          url: env.properties.app_api_url + '/vmap/datasourcetreeview/pglayers?',
          data: oData
        }).then((resp) => {
          if (resp.data) {
            if (resp.data.status === 1 && services.vitisCheck.isDefAndNotNull(resp.data.data['layer_id'])) {
              let url;
              if (resp.data.data['is_geometry_layer'] === false) {
                url = '/dashboard/admin/layers/' + resp.data.data['layer_id'] + '/update/informations_generales'
              } else {
                url = '/dashboard/admin/layers/' + resp.data.data['layer_id'] + '/update/symbology'
              }
              services.router.navigate([url]);
            } else {
              services.vitisAlert.fireToast({
                type: 'error',
                title: this.translate.instant('CREATE_LAYER_ERROR'),
              });
            }
          }
        });
      });
    }
  }
  /**
   * Lancer pour récupérer le type de géométrie et le set dans le formulaire.
   */
  scope.getGeom = function (oFormValues) {
    oFormValues['pg_table'] = oFormValues['pg_table'] ? oFormValues['pg_table'] : oFormValues['table'];

    if (!services.vitisCheck.isDefAndNotNull(oFormValues['database']) || !services.vitisCheck.isDefAndNotNull(oFormValues['schema'])
      || (!services.vitisCheck.isDefAndNotNull(oFormValues['table']) && !services.vitisCheck.isDefAndNotNull(oFormValues['pg_table']))) {
      return;
    }
    services.vitisRequest.ajaxRequestPromise({
      method: 'GET',
      url: `${env.properties.app_api_url}/vmap/datasourcetreeview/geom/pglayers`,
      data: {
        ...oFormValues
      },
      httpMethodOverride: true
    })
      .then((data) => {
        if (services.vitisCheck.isDefAndNotNull(data) && services.vitisCheck.isDefAndNotNull(data.data)
          && services.vitisCheck.isDefAndNotNull(data.data.data) && services.vitisCheck.isDefAndNotNull(data.data.data.f_geometry_column)
          && services.vitisCheck.isDefAndNotNull(data.data.data.srid) && services.vitisCheck.isDefAndNotNull(data.data.data.type)) {
          scope.setValueOnInit('geomfield', data.data.data.f_geometry_column);
          if (scope.getFormField('ms_geomtype_id')) {
            let aSelect = [];
            scope.getListGeometry().then(function (options) {
              aSelect = options;
              let iGeom;
              for (let i = 0; i < aSelect.length; i++) {
                if (aSelect[i].label === data.data.data.type) {
                  iGeom = aSelect[i].ms_geomtype_id;
                  break;
                }
              }
              scope.setValueOnInit('ms_geomtype_id', iGeom);
            }).catch(function (error) {
              console.error(error);
            });
          }
          scope.setValueOnInit('coordsys_id', data.data.data.srid);
        }
      })
      .catch((error) => {
        console.error("Can't get geom", error);

      })
  }

  scope.setValueOnInit = function (sField, oData) {
    oField = scope.getFormField(sField);
    if (oField && this.vitisCheck.isFunction(oField.setValue)) {
      oField.setValue(oData);
    } else if (oField) {
      oField.oninit.subscribe(() => {
        if (oField.setValue) {
          oField.setValue(oData);
        } else {
          setTimeout(() => {
            oField.setValue(oData);
          });
        }
      });
    }
  }

  /**
   * Lancer pour générer le Json du requêteur et filtre.
   */
  scope.createJsonFilterRequestor = function (oFormValues) {
    return new Promise((resolve, reject) => {
      let oData = {}
      for (const key in oFormValues) {
        if (services.vitisCheck.isDefAndNotNull(oFormValues[key])) {
          if (services.vitisCheck.isDefAndNotNull(oFormValues['pg_table'])) {
            oData['table'] = oFormValues['pg_table'];
          }
          oData[key] = oFormValues[key];
        }
      }
      if (!services.vitisCheck.isDefAndNotNull(oFormValues['database']) || !services.vitisCheck.isDefAndNotNull(oFormValues['schema'])
        || (!services.vitisCheck.isDefAndNotNull(oFormValues['table']) && !services.vitisCheck.isDefAndNotNull(oFormValues['pg_table']))) {
        reject(new Error('Missing required form values'));
      }
      services.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url: `${env.properties.app_api_url}/vmap/pglayers/columns`,
        data: oData,
        httpMethodOverride: true
      })
        .then((data) => {
          let json = [];
          let index = 0
          data.data.data.forEach(function (element) {
            let obj;
            let bFilter = false;
            let bRequestor = false;
            let sFilter_Type;
            let sRequestor_Type;
            if (element.column_name === oFormValues.idfield) {
              bRequestor = true;
            } else if (element.column_name === oFormValues.select_field) {
              bRequestor = true;
              bFilter = true;
            }
            if (bRequestor === true) {
              if (element.data_type === 'integer') {
                sFilter_Type = 'number';
                sRequestor_Type = 'number';
              } else {
                sFilter_Type = 'string';
                sRequestor_Type = 'string';
              }
              obj = {
                requestor: bRequestor,
                filter: bFilter,
                column: element.column_name,
                name: element.column_name,
                index: index,
                filter_type: sFilter_Type,
                request: false,
                requestor_type: sRequestor_Type
              };
            } else {
              obj = {
                requestor: bRequestor,
                filter: bFilter,
                column: element.column_name,
                name: element.column_name,
                index: index,
                request: false
              };
            }
            json.push(obj);
            index = index + 1;
          });
          scope.getFormField('json_columns').setValue(JSON.stringify(json));
          resolve();
        })
        .catch((err) => {
          console.error("Can't create filter", err);
          reject(err);
        })
    });
  }
  /**
   * Lancer pour générer le Json des formulaires.
   */
  scope.createJsonForm = function (oFormValues) {
    return new Promise((resolve, reject) => {
      const sForm = { "autoGenerated": true, "datasources": {}, "insert": { "input_size": "xxs", "javascript": false, "name": "custom-form", "nb_cols": 12, "title": "", "tabs": { "type": "tab", "list": [{ "label": "Tab 0", "elements": ["subtitle_1680793833864"] }] }, "rows": [{ "fields": [{ "type": "subtitle", "name": "subtitle_1680793833864", "label": "STUDIO_FORM_NOT_EDITED", "nb_cols": 12 }] }] }, "update": { "input_size": "xxs", "javascript": false, "name": "custom-form", "nb_cols": 12, "title": "", "tabs": { "type": "tab", "list": [{ "label": "Tab 0", "elements": ["subtitle_1680793833864"] }] }, "rows": [{ "fields": [{ "type": "subtitle", "name": "subtitle_1680793833864", "label": "STUDIO_FORM_NOT_EDITED", "nb_cols": 12 }] }] }, "search": { "input_size": "xxs", "javascript": false, "name": "custom-form", "nb_cols": 12, "title": "", "tabs": { "type": "tab", "list": [{ "label": "Tab 0", "elements": ["code", "nom"] }] }, "rows": [{ "fields": [{ "type": "text", "name": oFormValues['select_field'], "label": oFormValues['select_field'], "nb_cols": 12 }] }] }, "display": { "input_size": "xxs", "javascript": false, "name": "custom-form", "nb_cols": 12, "title": "", "tabs": { "type": "tab", "list": [{ "label": "Tab 0", "elements": ["code", "nom"] }] }, "rows": [{ "fields": [{ "type": "label", "name": oFormValues['idfield'], "label": oFormValues['idfield'], "nb_cols": 12 }] }, { "fields": [{ "type": "label", "name": oFormValues['select_field'], "label": oFormValues['select_field'], "nb_cols": 12 }] }] }, "multi_update": { "input_size": "xxs", "javascript": false, "name": "custom-form", "nb_cols": 12, "title": "", "tabs": { "type": "tab", "list": [{ "label": "Tab 0", "elements": [] }] }, "rows": [] } };
      sJson = JSON.stringify(sForm);
      scope.getFormField('default_form_json_def').setValue(sJson);
      scope.getFormField('published_form_json_def').setValue(sJson);
      scope.getFormField('custom_form_json_def').setValue(sJson);
      resolve();
    });
  }
  /**
   * Lancer pour récupérer la liste des type de géométries
   */
  scope.getListGeometry = function () {
    return new Promise(function (resolve, reject) {
      services.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url: `${env.properties.app_api_url}/vmap/datasourcetreeview/msgeomtype`,
        httpMethodOverride: true
      })
        .then((data) => {
          if (services.vitisCheck.isDefAndNotNull(data)) {
            aGeom = data.data.data;
            for (let i = 0; i < aGeom.length; i++) {
              switch (aGeom[i].label) {
                case 'POINT':
                  iGeomPoint = aGeom[i].ms_geomtype_id;
                  break;
                case 'LINE':
                  iGeomLine = aGeom[i].ms_geomtype_id;
                  break;
                case 'POLYGON':
                  iGeomPolygon = aGeom[i].ms_geomtype_id;
                  break;
                case 'GEOMETRY':
                  iGeomGeometry = aGeom[i].ms_geomtype_id;
                  break;
              }
            }
            resolve(aGeom);
          }
        })
        .catch(() => {
          console.error("Can't get list geom_type");
          reject();
        })
    });
  }
  /**
   * Lancer pour récupérer la liste des modes de recherche.
   */
  scope.getListSearchMode = function () {
    return new Promise(function (resolve, reject) {
      services.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url: `${env.properties.app_api_url}/vmap/datasourcetreeview/searchmode`,
        httpMethodOverride: true
      })
        .then((data) => {
          if (services.vitisCheck.isDefAndNotNull(data)) {
            resolve(data.data.data);
          }
        })
        .catch(() => {
          console.error("Can't get list search_mode");
          reject();
        })
    });
  }
}