/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  scope.prepareData = function (sType) {
    // Traitement pour un flux XYZ
    if (sType == 'xyz') {

      const oDatasourceId = scope.getFormField('datasource_id');
      const oMapField = scope.getFormField('xyz_map_layer');

      // Réinitialise la carte
      oMapField.map_options = this.vitisClone.clone(oMapField.map_options);

      var subscriptionDatasourceSet = oDatasourceId.oninit.subscribe(() => {
        services.vitisRequest.ajaxRequestPromise({
          method: 'GET',
          url: env.properties.app_api_url + '/vmap/datasources/' + oDatasourceId.getValue(),
        }).then((resp) => {
          if (services.vitisCheck.isObject(resp.data)) {
            if (resp.data.status == 1 && resp.data.data) {
              let oDatas = scope.oFormValues.value;
              let oResult = resp.data.data

              // Prépare les données pour la carte.
              const oService = this.prepareLayerXYZ(oResult, oDatas);

              // Suppression service si existe déjà
              const iChildIndex = oMapField.map_options.tree.children.findIndex((oChild) => oChild.name === oService.name);
              if (iChildIndex !== -1) {
                oMapField.map_options.tree.children.splice(iChildIndex, 1);
              }

              // Ajout du service
              oMapField.map_options.tree.children.push(oService);

              // Rafraîchi la carte
              oMapField.refreshMap();
            }
          }
        });
        // Supprime la subscription à l'observable.
        subscriptionDatasourceSet.unsubscribe();
      });

      // Traitement pour un flux WMTS
    } else if (sType == 'wmts') {

      // Traitement pour un flux WMTS
      const oDatasourceId = scope.getFormField('datasource_id');
      const oMapField = scope.getFormField('wmts_map_layer');

      // Réinitialise la carte
      oMapField.map_options = this.vitisClone.clone(oMapField.map_options);

      var subscriptionDatasourceSet = oDatasourceId.oninit.subscribe(() => {
        services.vitisRequest.ajaxRequestPromise({
          method: 'GET',
          url: env.properties.app_api_url + '/vmap/datasources/' + oDatasourceId.getValue(),
        }).then((resp) => {
          if (services.vitisCheck.isObject(resp.data)) {
            if (resp.data.status == 1 && resp.data.data) {
              if (resp.data.data &&
                resp.data.data.flux_url &&
                resp.data.data.flux_version
              ) {
                scope.wmtsGetCapabilities(resp.data.data).then((oCapabilities) => {
                  if (oCapabilities &&
                    oCapabilities.Contents &&
                    oCapabilities.Contents.Layer) {

                    let oDatas = scope.oFormValues.value;
                    const oCapabilitiesLayers = oCapabilities.Contents.Layer;

                    // Récupère la liste des couches dans le getCapabilities
                    const aLayers = services.WmtsService.searchLayers(oCapabilitiesLayers);

                    // Récupère la couche
                    const oLayer = aLayers.find((layer) => layer.Identifier === oDatas.layer_name);

                    this.getMapProjFromWMTSLayer(oLayer, oCapabilities).then((sProj) => {

                      // Prépare les données pour la carte et la grille à partir des couches du flux WMTS.
                      const oService = this.prepareLayerWMTS(oCapabilities, oDatas);

                      // Suppression service si existe déjà
                      const iChildIndex = oMapField.map_options.tree.children.findIndex((oChild) => oChild.name === oService.name);
                      if (iChildIndex !== -1) {
                        oMapField.map_options.tree.children.splice(iChildIndex, 1);
                      }

                      // Ajout du service
                      oMapField.map_options.tree.children.push(oService);

                      // Rafraîchi la carte
                      oMapField.refreshMap();

                    });
                  } else {
                    console.error('Capabilities not valid', oCapabilities)
                  }
                })
              } else {
                console.error('datasource not valid', resp.data.data);
              }


            }
          }
        });
        // Supprime la subscription à l'observable.
        subscriptionDatasourceSet.unsubscribe();
      });

    } else if (sType == 'wms') {
      // Traitement pour un flux WMS

      const oDatasourceId = scope.getFormField('datasource_id');
      const oMapField = scope.getFormField('wms_map_layer');

      // Réinitialise la carte
      oMapField.map_options = this.vitisClone.clone(oMapField.map_options);

      var subscriptionDatasourceSet = oDatasourceId.oninit.subscribe(() => {
        services.vitisRequest.ajaxRequestPromise({
          method: 'GET',
          url: env.properties.app_api_url + '/vmap/datasources/' + oDatasourceId.getValue(),
        }).then((resp) => {
          if (services.vitisCheck.isObject(resp.data)) {
            if (resp.data.status == 1 && resp.data.data) {
              let oDatas = scope.oFormValues.value;

              // Prépare les données pour la carte.
              const oService = this.prepareLayerWMS(oDatas);
              
              // Suppression service si existe déjà
              const iChildIndex = oMapField.map_options.tree.children.findIndex((oChild) => oChild.name === oService.name);
              if (iChildIndex !== -1) {
                oMapField.map_options.tree.children.splice(iChildIndex, 1);
              }

              // Ajout du service
              oMapField.map_options.tree.children.push(oService);

              // Rafraîchi la carte
              oMapField.refreshMap();

              const oLayerOptionsField = scope.getFormField('layer_options');
              if (services.vitisCheck.isDefAndNotNull(oLayerOptionsField)) {
                const oJsonCapabilities = oLayerOptionsField.getValue();
                if (services.vitisCheck.isDefAndNotNull(oJsonCapabilities) &&
                  services.vitisCheck.isDefAndNotNull(oJsonCapabilities.Capability) &&
                  services.vitisCheck.isDefAndNotNull(oJsonCapabilities.Capability.Layer)) {

                  const oCapabilitiesLayers = oJsonCapabilities.Capability.Layer;

                  // Récupère la liste des couches dans le getCapabilities
                  const aLayers = services.WmsService.searchLayers(oCapabilitiesLayers);
                  // Récupère la couche
                  const oLayer = aLayers.find((layer) => layer.Name === oDatas.layer_name);
                  const oBoundingBox = oLayer.BoundingBox.find((bbox) => bbox.crs === oMapField.map_options.proj);

                  if (services.vitisCheck.isDefAndNotNull(oBoundingBox)) {
                    setTimeout(() => {
                      // Evenement pour modifier la carte
                      services.MapCoreFormreaderService.emitMapFormreaderEvent(oMapField.id, {
                        action: 'zoomOnExtent',
                        extent: oBoundingBox.extent
                      });
                    });
                  }
                }
              }
            }
          }
        });
      });
    }

  }

  /**
   * Prépare la couche XYZ
   * @param oResult
   * @returns
   */
  scope.prepareLayerXYZ = function (oResult, oDatas) {

    const oChildrenService = {
      'name': 'Service XYZ',
      'children': [{
        'index': 1000000,
        'name': oDatas.name,
        'visible': 'true',
        'layerType': 'xyz',
        'url': oResult.flux_url
      }]
    };
    return oChildrenService;
  }

  /**
   * Prépare la couche WMTS
   * @returns
   */
  scope.prepareLayerWMTS = function (oCapabilities, oDatas) {
    const oChildrenService = {
      'name': 'Service WMTS',
      'children': [{
        'name': oDatas.layer_name,
        'index': 1,
        'layer': oDatas.layer_name,
        'visible': true,
        'layerType': 'wmts',
        'matrixSet': oDatas.matrix_set,
        'style': oDatas.layer_style,
        'format': oDatas.layer_format,
        'service_options': JSON.stringify(oCapabilities),
        'version': oDatas.datasource.flux_version,
        'url': oDatas.datasource.flux_url,
        'service_login': oDatas.datasource.flux_login,
        'service_password': oDatas.datasource.flux_password,
        'headers': oDatas.datasource.headers,
      }]
    };
    return oChildrenService;
  }

  /**
   * Prépare la couche pour la carte
   * @returns
   */
  scope.prepareLayerWMS = function (oDatas) {
    const oChildrenService = {
      'name': 'Service WMS',
      'children': [{
        'layer_id': oDatas.layer_id,
        'name': oDatas.layer_name,
        'layerType': 'imagewms',
        'visible': true,
        'select': true,
        'url': oDatas.datasource.flux_url,
        'params': {
          'LAYERS': oDatas.layer_name,
          'VERSION': '1.3.0',
          'STYLES': this.vitisCheck.isDefAndNotNull(oDatas.layer_style) ? oDatas.layer_style : '',
          'TIMESTAMP': new Date().getTime()
        },
        'service_options': oDatas.datasource.flux_options,
        'service_login': oDatas.datasource.flux_login,
        'service_password': oDatas.datasource.flux_password,
        'headers': oDatas.datasource.headers,
        'index': 1
      }]
    };
    return oChildrenService;
  }

  /**
     * Effecture la requête getCapabilities
     * @param {*} oDatasource 
     * @returns 
     */
  scope.wmtsGetCapabilities = function (oDatasource) {
    return new Promise((resolve, reject) => {
      services.WmtsService.getAjaxCapabilitiesWMTS({
        flux_type: oDatasource.flux_type,
        flux_url: oDatasource.flux_url,
        flux_version: oDatasource.flux_version,
        flux_login: oDatasource.flux_login,
        flux_password: oDatasource.flux_password,
        headers: oDatasource.headers,
      }).then((oCapabilities) => {
        if (oCapabilities !== false) {
          resolve(oCapabilities);
        } else {
          services.translate.get('CANT_REFRESH_CAPABILITIES').subscribe((sTranslate) => {
            services.vitisAlert.fireToast({
              type: 'warning',
              title: sTranslate
            });
          });
          reject();
        }
      });

    });
  }

  /**
   * Récupère la projection de la carte à partir d'une couche
   */
  scope.getMapProjFromWMTSLayer = function (oLayer, oCapabilities) {
    return new Promise((resolve, reject) => {
      services.MapCoreManagerService.getProjections().subscribe((oProjections) => {
        // Récupère l'index de projection à utiliser
        let sUsedProjectionIndex = oProjections.projections.findIndex((oProj) => oProj.code === 'EPSG:3857'); // par défaut 3857
        for (let i = 0; i < oProjections.projections.length; i++) {
          const index = services.WmtsService.getTileMatrixSetsIndex(oLayer, oCapabilities, oProjections.projections[i]);

          if (index !== -1) {
            sUsedProjectionIndex = this.vitisClone.clone(i);
            break;
          }
        }
        // Par sécurité si la valeur vaut -1 on prend l'element 0
        if (sUsedProjectionIndex === -1) {
          sUsedProjectionIndex = 0;
        }
        resolve(oProjections.projections[sUsedProjectionIndex].code);
      });
    });
  }

}