/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

    /**
     * Change l'étendue de la carte
     * @param {*} oMapField 
     * @param {*} sExtent 
     * @param {*} sCoordsys 
     * @param {*} olMap 
     */
    scope.setOlMapExtent = function (oMapField, sExtent, sCoordsys, olMap) {
        if (sExtent) {
            var oMapField = services.oFormReader.getFormField('map_extent');
            var olExtent = sExtent.split('|').map(coord => Number(coord));
            if (olExtent.length === 4) {
                var currMapProjection = olMap.getView().getProjection().getCode();
                var aNewExtent = oMapField.transformExtent(olExtent, `EPSG:${sCoordsys}`, currMapProjection);
                oMapField.setExtent(aNewExtent);
            }
        }
    }

    /**
     * Calcule l'étendue de la carte, la reprojette et la place sur le champ texte
     */
    scope.setFormExtentFromMap = function (olMap) {
        var oMapField = services.oFormReader.getFormField('map_extent');
        var oExtentField = services.oFormReader.getFormField('extent');
        var currMapExtent = olMap.getView().calculateExtent();
        var currMapProjection = olMap.getView().getProjection().getCode();
        var formProjection = 'EPSG:' + scope.oFormValues.get('coordsys_id').value;
        var aNewExtent = oMapField.transformExtent(currMapExtent, currMapProjection, formProjection);
        oExtentField.setValue(aNewExtent.join('|'));
    }

    /**
     * Initialise l'outil de gestion de l'étendue
     */
    scope.initMapForm = function () {

        // Set l'étendue de la carte
        var firstLoad = true;
        services.formreaderSrvc.mapChangeEventObservable.subscribe(olMap => {
            if (firstLoad) {
                // Set l'étendue du composant carto
                scope.setOlMapExtent(services.oFormReader.getFormField('map_extent'), scope.oFormValues.get('extent').value, scope.oFormValues.get('coordsys_id').value, olMap);
                firstLoad = false;
            }
        });

        // Valeurs dans couche de localisation par défaut
        var oDefaultLayerLocation = services.oFormReader.getFormField('default_layer_location_id');
        if (scope.vitisCheck.isDefAndNotNull(oDefaultLayerLocation) &&
            scope.vitisCheck.isDefAndNotNull(oDefaultLayerLocation.oninit)) {
            oDefaultLayerLocation.oninit.subscribe(() => {
                if (scope.sFormDefinitionName === 'update') {
                    scope.vitisRequest.ajaxRequestPromise({
                        url: env.properties.app_api_url + '/vmap/maps/layers/getAll',
                        method: 'GET',
                        params: {
                            filter: {
                                'relation': 'AND',
                                'operators': [{
                                    'column': 'map_id',
                                    'compare_operator': '=',
                                    'value': scope.oParams.id
                                }, {
                                    'column': 'is_location',
                                    'compare_operator': '=',
                                    'value': true
                                }]
                            }
                        }
                    }).then(resp => {
                        if (resp && resp.data && resp.data.data) {
                            const options = resp.data.data.map(layer => `${layer.layer_name}|${layer.layer_id}`);
                            const currentOptionsDatasource = scope.oFormDefinition.datasources.datasource_layers_localisable.options;
                            scope.oFormDefinition.datasources.datasource_layers_localisable.options = [...currentOptionsDatasource, ...options];
                            const currentOptionsSelect = oDefaultLayerLocation.options.choises;
                            oDefaultLayerLocation.options.choises = [...currentOptionsSelect, ...resp.data.data.map(layer => ({
                                "value": layer.layer_id.toString(),
                                "label": layer.layer_name
                            }))];
                        }
                    });
                }
            });
        }

        //Initialise maptools
        if (scope.oFormValues.controls['maptools']) {
            scope.oFormValues.controls['maptools'].setValue(JSON.stringify({
                controls: {
                    zoom_controls: true,
                    zoom_slider: true,
                    map_projection: true,
                    map_title: true,
                    numeric_scale: true,
                    graphic_scale: true,
                    mouse_coordinates: true,
                    map_overview: true
                }
            }));
        }
    }

    /**
     * Fonction avant l'update ou l'insert
     */
    scope.sendForm = function () {

        if (scope.oFormValues.controls['layers'] && scope.oFormValues.controls['layers'].value) {
            scope.oFormValues.controls['layers'].setValue(scope.oFormValues.value.layers.map(layer => JSON.stringify(layer)));
        }

        // Set la valeur de l'étendue
        var oMapField = services.oFormReader.getFormField('map_extent');
        if (oMapField.getMapComponent) {
            var oMapComponent = oMapField.getMapComponent();
            if (oMapComponent &&
                oMapComponent.olMap) {
                var olMap = oMapComponent.olMap;
                scope.setFormExtentFromMap(olMap);
            }
        }

        scope.oFormValues.markAsPristine();
        return services.SimpleformComponent.sendSimpleForm({
            ...scope.oFormValues.value,
            ...scope.getFormValues(true)
        });
    }
}