/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  /**
   * Envoie le formulaire pour insérer un widget en bdd
   * @param {*} oFormValues 
   */
  scope.sendInsert = function (oFormValues) {
    if (scope.isFormValid()) {
      scope.oFormValues.markAsPristine();

      // Supprime le token si l'accès est privé
      if (oFormValues.need_auth === true) {
        delete oFormValues.token;
      }

      // Supprime les vides
      for (const key in oFormValues) {
        if (Object.hasOwnProperty.call(oFormValues, key)) {
          if (oFormValues[key] === '') {
            delete oFormValues[key];
          }
        }
      }

      // Récupère l'id du type vmap_map pour l'associer au widget
      services.vitisRequest.ajaxRequest({
        method: 'GET',
        url: `${env.properties.app_api_url}/vitis/rt_type`,
        params: {
          filter: { relation: "AND", operators: [{ column: "name", compare_operator: "=", value: "vmap_map" }] }
        },
        success: (respType) => {
          if (respType && respType.data && respType.data.data) {
            if (respType.data.data.length > 0) {
              const oFormData = {
                ...oFormValues,
                comp_data: {
                  map_id: scope.oParams.id_parent,
                  layers_tool: oFormValues.layers_tool,
                  maps_tool: oFormValues.maps_tool,
                  helper_tool: oFormValues.helper_tool
                },
                type: respType.data.data[0].rt_type_id
              };
              // Post le widget
              services.vitisRequest.ajaxRequest({
                method: 'POST',
                url: `${env.properties.app_api_url}/vitis/widget`,
                data: services.formreaderSrvc.transformToDataForm(oFormData, true),
                success: (resp) => {
                  services.vitisAlert.fireToast({
                    type: 'success',
                    title: services.translate.instant('LIST_ADD_OK')
                  });
                  return services.router.navigate([`../${resp.data.data.widget_id}/update`], {
                    relativeTo: this.route.parent
                  });
                },
                error: (resp) => {
                  console.error(resp);
                }
              });
            } else {
              // alert
            }
          }
        },
        error: (resp) => {
          console.error(resp);
        }
      });
    }
  }

  /**
   * Envoie le formulaire pour mettre à jour le widget
   * @param {*} oFormValues 
   */
  scope.sendForm = function (oFormValues) {
    if (scope.isFormValid()) {
      // Récupère l'id du type vmap_map pour l'associer au widget
      services.vitisRequest.ajaxRequest({
        method: 'GET',
        url: `${env.properties.app_api_url}/vitis/rt_type`,
        params: {
          filter: { relation: "AND", operators: [{ column: "name", compare_operator: "=", value: "vmap_map" }] }
        },
        success: (resp) => {
          if (resp && resp.data && resp.data.data) {
            if (resp.data.data.length > 0) {
              const oFormData = {
                ...oFormValues,
                map_id: scope.oParams.id_parent
              };
              // Post le widget
              services.vitisRequest.ajaxRequest({
                method: 'PUT',
                url: `${env.properties.app_api_url}/vmap/maps/${oFormData.map_id}/widget/${oFormData.widget_id}`,
                data: services.formreaderSrvc.transformToDataForm(oFormData, true),
                success: (resp) => {
                  services.vitisAlert.fireToast({
                    type: 'success',
                    title: services.translate.instant('LIST_EDIT_OK')
                  });
                  scope.oFormValues.markAsPristine();
                },
                error: (resp) => {
                  console.error(resp);
                }
              });
            } else {
              // alert
            }
          }
        },
        error: (resp) => {
          console.error(resp);
        }
      });
    }
  }

  /**
   * Génère le code du widget
   */
  scope.generateCodeWidget = function () {
    setTimeout(() => {
      if (scope.isFormValid()) {
        const sWidgetUrl = scope.getWidgetURL();
        if (this.vitisCheck.isDefAndNotNull(sWidgetUrl)) {

          // Set le lien vers le widget
          const oLinkField = scope.getFormField('url_widget');
          oLinkField.visible = true;
          setTimeout(() => {
            oLinkField.setValue(sWidgetUrl);
            scope.oFormValues.markAsPristine();
          });

          // Set le champ monacoeditor
          const sHtml = scope.getWidgetIframe(sWidgetUrl);
          const oCodeField = scope.getFormField('html_code');
          oCodeField.visible = true;
          setTimeout(() => {
            oCodeField.setValue(sHtml);
            scope.oFormValues.markAsPristine();
          });
        }
      }
      if (scope.sFormDefinitionName === 'update') {
        setTimeout(() => {
          scope.oFormValues.controls.need_auth_value
            .setValue(services.translate.instant(scope.oFormValues.value.need_auth ? 'FORM_YES' : 'FORM_NO'));
          scope.oFormValues.markAsPristine();
        });
      }
    }, 500);
  }

  /**
   * Retourne l'URL du widget
   */
  scope.getWidgetURL = function () {

    const sClientUrl = env.properties.app_client_url;
    const bAuth = scope.getFormField('need_auth').getValue();
    const endpoint = scope.getFormField('endpoint').getValue();

    if (services.vitisCheck.isDefAndNotNull(sClientUrl) && sClientUrl !== '') {
      let sUrl = `${sClientUrl}/widget/vmap?widget_id=${endpoint}`;
      if (bAuth === false) {
        const sLoginToken = scope.getFormField('token').getValue();
        if (services.vitisCheck.isDefAndNotNull(sLoginToken) && sLoginToken !== '') {
          sUrl += `&token=${sLoginToken}`;
        }
      }
      return sUrl;
    }
  }

  /**
   * Génère un code endpoint unique
   */
  scope.generateUUID = function () {
    let uuid = '';
    const possibleChars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    for (let i = 0; i < 64; i++) {
      uuid += possibleChars.charAt(Math.floor(Math.random() * possibleChars.length));
    }

    services.vitisRequest.ajaxRequest({
      method: 'GET',
      url: `${env.properties.app_api_url}/vitis/widget/${uuid}`,
      success: (resp) => {
        // Vérifie que le endpoint n'existe pas
        if (resp.data.data && resp.data.data.length === 0) {
          scope.oFormValues.controls.endpoint.setValue(uuid);
          if (scope.sFormDefinitionName === 'update') {
            scope.generateCodeWidget();
          }
        } else {
          scope.generateUUID();
        }
      }
    });
  }

  /**
   * Retourne le code HTML pour utiliser le widget dans une iframe
   * @param {*} sWidgetUrl
   */
  scope.getWidgetIframe = function (sWidgetUrl) {
    const iHeight = scope.getFormField('iframe_height').getValue();
    const iWidth = scope.getFormField('iframe_width').getValue();
    const sTitle = scope.getFormField('name').getValue();
    const sHtml = `<iframe title="${sTitle}" width="${iWidth}" height="${iHeight}"
    style="border: 1px solid #39468A; border-radius: 5px 5px 0px 0px;"
    src = "${sWidgetUrl}" >
</iframe>`;

    return sHtml;
  }
}