import { AppModeConfig } from 'src/app/vitis/app.model';
import { WklBooleanIconComponent } from 'src/app/vitis/workspacelist/template/wkl-boolean-icon/wkl-boolean-icon.component';
import { WklDateComponent } from 'src/app/vitis//workspacelist/template/wkl-date/wkl-date.component';
// import { WmsService } from 'src/modules/vmap/components/admin-datasource/services/wms.service';

/**
 * Objet JDD du mode admin cartes
 */
export const vmapAdminDatasourceObject: AppModeConfig = {
  'name': 'vmap_admin_datasource',
  'label': 'TITLE_MODE_VMAP_ADMIN_DATASOURCE',
  'icon': 'icon-vitis-vm4vmp',
  'route': 'datasource',
  'type': 'module',
  'privilege': 'vmap_admin',
  'options': {
    'module_name': 'VmapAdminDatasourceModule'
  }
};

/**
 * Mode cartes et couches
 */

const vmapMapsGeneralSection: AppModeConfig = {
  name: 'vmap_maps_informations_generales_section',
  label: 'TITLE_SECTION_VMAP_MAP_GENERAL',
  route: 'informations_generales',
  type: 'simpleform',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/maps',
    id_field: 'map_id',
    form_path: 'modules/vmap/forms/maps/informations_generales/form.json',
    js_path: 'modules/vmap/forms/maps/informations_generales/form.js',
    update: true,
    insert: true,
    delete: true,
  }
};

const vmapMapsEnableUtilsSection: AppModeConfig = {
  name: 'vmap_maps_enable_utils_section',
  label: 'TITLE_SECTION_VMAP_MAP_ENABLE_UTILS',
  route: 'enable_utils',
  type: 'module',
  privilege: 'vmap_admin',
  options: {
    module_name: 'MapToolsDesignerModule',
    ressource: 'vmap/maps',
    id_field: 'map_id',
    update: true,
    insert: true,
    delete: true
  }
};

const vmapMapsLayersSection: AppModeConfig = {
  name: 'vmap_maps_layers_map_section',
  label: 'TITLE_SECTION_VMAP_MAP_LAYERS',
  route: 'layers',
  type: 'draganddrop-grid',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/maps/:id/layers',
    ressource_source: 'layers',
    id_field: 'layer_map_id',
    index_field: 'layer_index',
    name_field: 'name',
    form_path: 'modules/vmap/forms/layer_map/layer_map.json',
    js_path: 'modules/vmap/forms/layer_map/layer_map.js',
    update: true,
    insert: true,
    delete: true,
    edit: true,
    reversed: true,
    search: {
      custom: true,
      form: false,
      default: 'custom'
    },
    buttons: [],
    columns: [
      {
        title: 'IS_VISIBLE',
        field: 'is_visible',
        type: 'checkbox'
      },
      {
        title: 'IS_LOCATION',
        field: 'is_location',
        type: 'checkbox',
        enabled: { column: 'is_location_source', value: true }
      },
      {
        title: 'IS_MAP_SELECTION',
        field: 'is_map_selection',
        type: 'checkbox',
        enabled: { column: 'is_map_selection_source', value: true }
      },
      {
        title: 'IS_DATA_GRID',
        field: 'is_data_grid',
        type: 'checkbox',
        enabled: { column: 'is_data_grid_source', value: true }
      },
      {
        title: 'DATASOURCE_FORM_PG_LAYER_IS_FILTER',
        field: 'is_filter',
        type: 'checkbox',
        enabled: { column: 'is_filter_source', value: true }
      },
      {
        title: 'IS_MAP_HIDDEN',
        field: 'is_map_hidden',
        type: 'checkbox'
      },
      {
        title: 'MAP_PARENT_HIDDEN',
        field: 'map_parent_hidden',
        type: 'select',
        source: 'current',
        id_field: 'layer_id',
        name_field: 'name',
        enabled: { column: 'is_map_hidden', value: true }
      }],
    source: {
      ressource: 'vmap/layers',
      id_field: 'layer_id',
      id_ressource_field: 'map_id',
      titleModalInsert: 'Ajouter une couche',
      insert: true,
      columns: [{
        title: 'LAYER_ID',
        field: 'layer_id',
        data: 'layer_id',
        name: 'layer_id',
        align: 'left',
        resizeable: true,
        sortable: true,
        searchable: true,
        searchType: 'number'
      },
      {
        title: 'LAYERS_NAME',
        field: 'name',
        data: 'name',
        align: 'left',
        resizeable: true,
        sortable: true,
        searchable: true,
      },
      {
        title: 'DATASOURCE_FORM_LAYER_TYPE',
        field: 'layertype',
        data: 'layertype',
        align: 'left',
        resizeable: true,
        sortable: true,
        searchable: true,
        searchType: 'strict'
      }
      ],
    }
  }
};

const vmapMapsWidgetSection: AppModeConfig = {
  name: 'vmap_maps_widget_section',
  label: 'TITLE_SECTION_VMAP_MAP_WIDGET',
  route: 'widget',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/maps/:map_id/widget',
    id_field: 'widget_id',
    parent_id_field: 'map_id',
    disable_filter_parent: true,
    form_path: 'modules/vmap/forms/maps/widget/widget_form.json',
    js_path: 'modules/vmap/forms/maps/widget/widget_form.js',
    update: true,
    insert: true,
    delete: true,
    clone: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    columns: [{
      title: 'MAPS_WKLST_TITLE_WIDGET_ID',
      field: 'widget_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    },
    {
      title: 'MAPS_WKLST_TITLE_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'string'
    },
    {
      title: 'MAPS_WKLST_TITLE_EXPIRATION_DATE',
      field: 'expiration_date',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'date',
      template: WklDateComponent
    },
    {
      title: 'MAPS_WKLST_TITLE_ACTIVE',
      field: 'active',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchType: 'boolean',
      searchSuggestions: [{
        value: true,
        label: 'FORM_YES'
      }, {
        value: false,
        label: 'FORM_NO'
      }],
      template: WklBooleanIconComponent
    }]
  }
};

/**
 * Section "Commentaire" de l'onglet "Cartes".
 */
const vmapMapsCommentSection: AppModeConfig = {
  name: 'vmap_maps_comment_section',
  label: 'TITLE_SECTION_VMAP_MAP_COMMENT',
  route: 'comment',
  type: 'simpleform',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/maps',
    id_field: 'map_id',
    form_path: 'modules/vmap/forms/maps/comment/form.json',
    js_path: 'modules/vmap/forms/maps/comment/form.js',
    update: true
  }
};

const vmapMapsLayersMapsObject: AppModeConfig = {
  name: 'vmap_mapslayers_maps',
  label: 'TITLE_MODE_MAPSLAYERS_MAPS',
  route: 'maps',
  type: 'workspacelist',
  privilege: 'vmap_admin',
  sections: [
    vmapMapsGeneralSection,
    vmapMapsEnableUtilsSection,
    vmapMapsLayersSection,
    vmapMapsWidgetSection,
    vmapMapsCommentSection
  ],
  options: {
    module: 'vmap',
    ressource: 'vmap/maps',
    id_field: 'map_id',
    update: true,
    insert: true,
    delete: true,
    clone: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    columns: [{
      title: 'MAPS_ID',
      field: 'map_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'MAPS_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }, {
      title: 'MAPS_DESCRIPTION',
      field: 'description',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }, {
      title: 'MAPS_CATALOG_INDEX',
      field: 'catalog_index',
      align: 'left',
      resizeable: true,
      searchable: true,
      sortable: true,
      searchType: 'number'
    }, {
      title: 'MAPS_THEME_ID',
      field: 'theme_name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number',
      searchWebService: {
        resource: 'vmap/themes/maps',
        params: {
          attributs: 'thememap_id|label',
          order_by: 'thememap_id',
          sort_order: 'DESC'
        },
        label: 'label',
        value: 'thememap_id',
      }
    }, {
      title: 'MAPS_COORDSYS_ID',
      field: 'coordsys_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }]
  }
};

const vmapLayersGeneralesSection: AppModeConfig = {
  name: 'vmap_layers_informations_generales_section',
  label: 'TITLE_MODE_MAPSLAYERS_LAYERS_GENERAL',
  route: 'informations_generales',
  type: 'simpleform',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/layers',
    id_field: 'layer_id',
    update: true,
    insert: true,
    map_field: 'layertype',
    form_path: {
      'postgres': 'modules/vmap/forms/layers/layerpg_simple.json',
      'osm': 'modules/vmap/forms/layers/layerosm.json',
      'file': 'modules/vmap/forms/layers/layerfile_simple.json',
      'wms': 'modules/vmap/forms/layers/layerwms.json',
      'wmts': 'modules/vmap/forms/layers/layerwmts.json',
      'xyz': 'modules/vmap/forms/layers/layerxyz.json',
      'vectortiles': 'modules/vmap/forms/layers/layervectortiles.json',
    },
    js_path: 'modules/vmap/forms/layers/layer.js',
    js_services: {
      'WmsService': import('src/modules/vmap/components/admin-datasource/services/wms.service'),
      'WmtsService': import('src/modules/vmap/components/admin-datasource/services/wmts.service'),
      'AdminDatasourceFormService': import('src/modules/vmap/components/admin-datasource/services/admin-datasource-form.service'),
    },
  }
};

const vmapLayersAdvancedSection: AppModeConfig = {
  name: 'vmap_layers_advanced_section',
  label: 'TITLE_MODE_MAPSLAYERS_LAYERS_ADVANCED',
  route: 'advanced',
  type: 'simpleform',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/layers',
    id_field: 'layer_id',
    update: true,
    insert: true,
    map_field: 'layertype',
    form_path: {
      'postgres': 'modules/vmap/forms/layers/layerpg_advanced.json',
      'osm': 'modules/vmap/forms/layers/layerosm.json',
      'file': 'modules/vmap/forms/layers/layerfile_advanced.json',
      'wms': 'modules/vmap/forms/layers/layerwms.json',
      'wmts': 'modules/vmap/forms/layers/layerwmts.json',
      'xyz': 'modules/vmap/forms/layers/layerxyz.json',
      'vectortiles': 'modules/vmap/forms/layers/layervectortiles.json',
    },
    js_path: 'modules/vmap/forms/layers/layer.js',
  }
};

const vmapLayersFilterForm: AppModeConfig = {
  name: 'vmap_filter',
  label: 'TITLE_MODE_MAPSLAYERS_LAYERS_FILTER_COLUMNS',
  route: 'filter_columns',
  type: 'module',
  privilege: 'vmap_admin',
  options: {
    module_name: 'LayerFilterRequestFormModule',
    type: 'filter',
  }
};

const vmapLayersRequestorForm: AppModeConfig = {
  name: 'vmap_filter',
  label: 'TITLE_MODE_MAPSLAYERS_LAYERS_REQUESTOR_COLUMNS',
  route: 'requestor_columns',
  type: 'module',
  privilege: 'vmap_admin',
  options: {
    module_name: 'LayerFilterRequestFormModule',
    type: 'requestor',
  }
};

const vmapLayersSymbologie: AppModeConfig = {
  name: 'vmap_layers_symbology_section',
  label: 'TITLE_MODE_MAPSLAYERS_LAYERS_SYMBOLOGY',
  route: 'symbology',
  type: 'module',
  privilege: 'vmap_admin',
  visible: '{{is_geometry_layer}} === true || {{layertype}} === "file"',
  options: {
    module_name: 'SectionLayerSymbologyModule',
    ressource: 'vmap/layers',
    ressourceTest: 'vmap/datasourcetreeview/pglayers',
    id_field: 'layer_id',
    update: true,
    insert: true,
    delete: true,
  }
};

const vmapLayersStudioSection: AppModeConfig = {
  name: 'vmap_layers_studio_section',
  label: 'TITLE_MODE_MAPSLAYERS_LAYERS_FORM',
  route: 'form',
  type: 'studio',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/datasourcetreeview/pglayers/:id/form/:form',
    ressource_result_attr: 'data.json_form',
    js_ressource_result_attr: 'data.js_form',
    css_ressource_result_attr: 'data.css_form',
    available_form_names: ['insert', 'update', 'search', 'display', 'multi_update'],
    default_form_name: 'update',
    available_form_types: ['Default', 'Published', 'Perso'],
    default_form_type: 'Perso',
    field_list_ressource: 'vmap/pglayers/:id/columns',
    field_list_ressource_result_attr: 'data',
    // Liste complete des types d'attr-form-fields affiché
    attr_form_fields_list: [
      'radio', 'checkbox', 'button', 'map_bing', 'map_osm', 'map_vmap', 'date',
      'datetime', 'upload', 'image_wsdata', 'hr', 'title', 'subtitle', 'label',
      'list', 'select', 'double_select', 'searchable_select', 'integer', 'float', 'slider', 'text', 'textarea',
      'email', 'password', 'linkurl', 'imageurl', 'angularurl', 'hidden', 'color_picker', 'tinymce', 'dessin',
      'data_grid_vmap', 'text_body', 'formicon'
    ],
    attr_form_fields_specification: {
      upload: {
        multiple: true,
        display_only: true,
      },
      data_grid: {
        vmap_grid: true
      }
    }
  }
};

const vmapMapsLayersFMEWorkspacesObject: AppModeConfig = {
  name: 'vmap_fme_workbench_section',
  label: 'TITLE_MODE_MAPSLAYERS_LAYERS_FME_WORKSPACES',
  description: "TITLE_MODE_LAYER_FME_WORKSPACES_DESCRIPTION",
  route: 'fme-workspaces',
  type: 'draganddrop-grid',
  privilege: 'vmap_admin',
  visible: '@prop(enable_gtf_api)',
  options: {
    module: 'vmap',
    ressource: 'vmap/layers/:id/fme-workspaces',
    ressource_source: 'fme-workspaces',
    id_field: 'layer_workspace_id',
    index_field: 'layer_index',
    name_field: 'name',
    form_path: 'modules/vmap/forms/layers/layersfmeworkspace.json',
    js_path: 'modules/vmap/forms/layers/layersfmeworkspace.js',
    update: true,
    insert: true,
    delete: true,
    edit: true,
    reversed: true,
    search: {
      custom: true,
      form: false,
      default: 'custom'
    },
    buttons: [],
    columns: [{
      title: 'GTF_MODAL_PROJECT_ALLOW_DISPLAY_CARD',
      field: 'allow_display_card',
      type: 'checkbox'
    }, {
      title: 'GTF_MODAL_PROJECT_ALLOW_REQUESTOR',
      field: 'allow_requestor',
      type: 'checkbox'
    }, {
      title: 'GTF_MODAL_PROJECT_ALLOW_SELECT_ALL_DATA',
      field: 'allow_select_all_data',
      type: 'checkbox'
    }, {
      title: 'GTF_MODAL_PROJECT_ALLOW_SELECT_CURRENT_FILTER',
      field: 'allow_select_current_filter',
      type: 'checkbox'
    }, {
      title: 'GTF_MODAL_PROJECT_SELECT_MAX_ENTITIES',
      field: 'selection_max_entities',
      type: 'number'
    }],
    source: {
      ressource: 'vmap/gtf-workspaces',
      id_field: 'workspace_id',
      id_ressource_field: 'layer_id',
      titleModalInsert: 'TITLE_MODE_LAYER_FME_WORKSPACES_TITLE_MODAL_INSERT',
      insert: true,
      columns: [{
        title: 'TITLE_MODE_LAYER_FME_WORKSPACES_TITLE_MODAL_WORKSPACE_ID',
        field: 'workspace_id',
        data: 'workspace_id',
        name: 'workspace_id',
        resizeable: true,
        sortable: true,
        searchable: true,
        searchType: 'number'
      },
      {
        title: 'TITLE_MODE_LAYER_FME_WORKSPACES_TITLE_MODAL_WORKSPACE_NAME',
        field: 'name',
        data: 'name',
        align: 'left',
        resizeable: true,
        sortable: true,
        searchable: true,
      },
      {
        title: 'TITLE_MODE_LAYER_FME_WORKSPACES_TITLE_MODAL_WORKSPACE_FMW',
        field: 'fmw_file_name',
        data: 'fmw_file_name',
        align: 'left',
        resizeable: true,
        sortable: true,
        searchable: true,
        searchType: 'strict'
      }
      ],
    }
  }
};

const vmapLayersPrevisualisationSection: AppModeConfig = {
  name: 'vmap_layers_previsualisation_section',
  label: 'TITLE_MODE_MAPSLAYERS_LAYERS_PREVISU',
  route: 'preview',
  type: 'simpleform',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/layers',
    id_field: 'layer_id',
    display: true,
    update: true,
    insert: false,
    map_field: 'layertype',
    form_path: {
      'wms': 'modules/vmap/forms/layers/layerwms_preview.json',
      'wmts': 'modules/vmap/forms/layers/layerwmts_preview.json',
      'xyz': 'modules/vmap/forms/layers/layerxyz_preview.json',
    },
    js_path: 'modules/vmap/forms/layers/layer_preview.js',
    js_services: {
      'WmsService': import('src/modules/vmap/components/admin-datasource/services/wms.service'),
      'WmtsService': import('src/modules/vmap/components/admin-datasource/services/wmts.service'),
      'MapCoreManagerService': import('src/app/vitis/map-core/services/map-core-manager/map-core-manager.service'),
      'MapCoreFormreaderService': import('src/app/vitis/map-core/services/map-core-fromreader/map-core-formreader.service'),
    },
  }
};

const vmapMapsLayersLayersObject: AppModeConfig = {
  name: 'vmap_mapslayers_layers',
  label: 'TITLE_MODE_MAPSLAYERS_LAYERS',
  route: 'layers',
  type: 'workspacelist',
  privilege: 'vmap_admin',
  sections: {
    'postgres': [
      vmapLayersGeneralesSection,
      vmapLayersAdvancedSection,
      vmapLayersRequestorForm,
      vmapLayersFilterForm,
      vmapLayersSymbologie,
      vmapLayersStudioSection,
      vmapMapsLayersFMEWorkspacesObject,
    ],
    'osm': [
      vmapLayersGeneralesSection
    ],
    'file': [
      vmapLayersGeneralesSection,
      vmapLayersAdvancedSection,
      vmapLayersSymbologie,
    ],
    'wms': [
      vmapLayersGeneralesSection,
      vmapLayersPrevisualisationSection
    ],
    'wmts': [
      vmapLayersGeneralesSection,
      vmapLayersPrevisualisationSection
    ],
    'xyz': [
      vmapLayersGeneralesSection,
      vmapLayersPrevisualisationSection
    ],
    'vectortiles': [
      vmapLayersGeneralesSection
    ],
  },
  options: {
    module: 'vmap',
    ressource: 'vmap/layers',
    id_field: 'layer_id',
    map_field: 'layertype',
    display: true,
    update: true,
    insert: false, // L'ínsertion doit se faire par la partie sources de données, autrement ça crée des problèmes de conception
    delete: true,
    clone: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    columns: [{
      title: 'LAYERS_ID',
      field: 'layer_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchType: 'number'
    }, {
      title: 'LAYERS_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'DATASOURCE_FORM_LAYER_TYPE',
      field: 'layertype',
      align: 'left',
      resizeable: true,
      sortable: false,
      searchType: 'strict',
      searchWebService: false
    }]
  }
};

const vmapMapsLayersThemeMapsObject: AppModeConfig = {
  name: 'vmap_mapslayers_themes_maps',
  label: 'TITLE_MODE_MAPSLAYERS_THEMES_MAPS',
  route: 'maps',
  type: 'workspacelist',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/themes/maps',
    id_field: 'thememap_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'label'
    },
    form_path: 'modules/vmap/forms/theme_map/theme_map.json',
    columns: [{
      title: 'THEMEMAPS_ID',
      field: 'thememap_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'THEMEMAPS_LABEL',
      field: 'label',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }]
  }
};

const vmapMapsLayersThemeLayersObject: AppModeConfig = {
  name: 'vmap_mapslayers_themes_layers',
  label: 'TITLE_MODE_MAPSLAYERS_THEMES_LAYERS',
  route: 'layers',
  type: 'workspacelist',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/themes/layers',
    id_field: 'themelayer_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'label'
    },
    form_path: 'modules/vmap/forms/theme_layer/theme_layer.json',
    columns: [{
      title: 'THEMELAYERS_ID',
      field: 'themelayer_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'THEMELAYERS_LABEL',
      field: 'label',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }]
  }
};

const vmapMapsLayersThemesObject: AppModeConfig = {
  name: 'vmap_mapslayers_themes',
  label: 'TITLE_MODE_MAPSLAYERS_THEMES',
  route: 'theme',
  privilege: 'vmap_admin',
  objects: [vmapMapsLayersThemeMapsObject, vmapMapsLayersThemeLayersObject]
};

const vmapLayersSourcesObject: AppModeConfig = {
  name: 'vmap_layers_sources',
  label: 'TITLE_MODE_LAYERS_SOURCES',
  route: 'source',
  privilege: 'vmap_admin',
  type: 'workspacelist',
  options: {
    module: 'vmap',
    ressource: 'vmap/datasourcetreeview/source',
    id_field: 'source_id',
    form_path: 'modules/vmap/forms/source/source.json',
    update: true,
    insert: true,
    delete: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    columns: [{
      title: 'SOURCE_ID',
      field: 'source_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'SOURCE_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }]
  }
};

/**
 * Onglet "Systèmes de coordonnées" du mode "Cartes et couches".
 */
const vmapMapsLayersCoordsysObject: AppModeConfig = {
  name: 'vmap_maps_layers_coordsys',
  label: 'TITLE_MODE_MAPS_LAYERS_COORDSYS',
  route: 'coordsys',
  privilege: 'vmap_admin',
  type: 'workspacelist',
  options: {
    module: 'vmap',
    ressource: 'vmap/coordsys',
    id_field: 'coordsys_id',
    form_path: 'modules/vmap/forms/coordsys/coordsys.json',
    update: true,
    insert: true,
    delete: true,
    display: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    columns: [{
      title: 'LIST_MAPS_LAYERS_COORDSYS_COORDSYS_ID',
      field: 'coordsys_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'LIST_MAPS_LAYERS_COORDSYS_EPSG',
      field: 'epsg',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }, {
      title: 'LIST_MAPS_LAYERS_COORDSYS_LABEL',
      field: 'label',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }]
  }
};

export const vmapMapsLayersMode: AppModeConfig = {
  name: 'vmap_mapslayers',
  route: 'admin',
  label: 'TITLE_MODE_VMAP_ADMIN',
  icon: 'icon-vitis-map',
  privilege: 'vmap_admin',
  objects: [vmapMapsLayersMapsObject, vmapMapsLayersLayersObject, vmapMapsLayersThemesObject, vmapLayersSourcesObject, vmapMapsLayersCoordsysObject]
};

const vmapSymbolsList: AppModeConfig = {
  name: 'vmap_symbols',
  label: 'TITLE_MODE_SYMBOLS',
  route: 'symbols',
  type: 'workspacelist',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/symbols',
    id_field: 'symbol_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    clone: true,
    form_path: 'modules/vmap/forms/config_layer/symbol.json',
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'label'
    },
    columns: [{
      title: 'SYMBOL_ID',
      field: 'symbol_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'SYMBOL_LABEL',
      field: 'label',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }, {
      title: 'SYMBOL_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }, {
      title: 'SYMBOL_TYPE_SYMBOL',
      field: 'type',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }]
  }
};

const vmapFontsList: AppModeConfig = {
  name: 'vmap_fonts',
  label: 'TITLE_MODE_FONTS',
  route: 'fonts',
  type: 'workspacelist',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/fonts',
    id_field: 'font_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    clone: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    form_path: 'modules/vmap/forms/config_layer/font.json',
    columns: [{
      title: 'FONT_ID',
      field: 'font_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'FONT_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }]
  }
};

export const vmapSymbolsFontsMode: AppModeConfig = {
  name: 'vmap_symbolsfonts',
  route: 'layer-config',
  label: 'TITLE_MODE_VMAP_ADMIN_SYMBOLS_FONTS',
  icon: 'icon-vitis-font',
  privilege: 'vmap_admin',
  objects: [vmapSymbolsList, vmapFontsList]
};

/**
 * Section informations générales de l'objet template du mode modèle d'impression
 */
const vmapMapsPrintingsTemplatesInformationsGenerales: AppModeConfig = {
  name: 'vmap_maps_printings_informations_generales',
  label: 'TITLE_MODE_PRINTINGS_PRINT_TEMPLATES_GENERAL',
  route: 'informations-generales',
  type: 'simpleform',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/printtemplates',
    id_field: 'printtemplate_id',
    display: true,
    update: true,
    insert: true,
    form_path: 'modules/vmap/forms/print/templates.json'
  }
};

/**
 * Section formulaire de l'objet template du mode modèle d'impression
 */
const vmapMapsPrintingsTemplatesFormSection: AppModeConfig = {
  name: 'vmap_maps_printings_form_section',
  label: 'TITLE_MODE_PRINTINGS_PRINT_TEMPLATES_FORM',
  route: 'form',
  type: 'studio',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/printtemplates/:id/form/:form',
    ressource_result_attr: 'data.json_form',
    js_ressource_result_attr: 'data.js_form',
    available_form_names: ['insert'],
    default_form_name: 'insert',
    attr_form_fields_list: [
      'radio', 'checkbox', 'date', 'datetime', 'hr', 'select', 'searchable_select', 'integer', 'float', 'slider', 'text', 'textarea', 'email', 'linkurl', 'imageurl', 'color_picker', 'tinymce', 'dessin', 'data_grid', 'text_body'
    ],
    available_form_types: ['Published', 'Perso'],
    default_form_type: 'Perso'
  }
};

const vmapMapsPrintingsTemplatesObject: AppModeConfig = {
  name: 'vmap_maps_print_templates_list',
  label: 'TITLE_MODE_PRINTINGS_PRINT_TEMPLATES',
  route: 'templates',
  type: 'workspacelist',
  privilege: 'vmap_admin',
  sections: [
    vmapMapsPrintingsTemplatesInformationsGenerales,
    vmapMapsPrintingsTemplatesFormSection
  ],
  options: {
    module: 'vmap',
    ressource: 'vmap/printtemplates',
    id_field: 'printtemplate_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    clone: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    form_path: 'modules/vmap/forms/print/templates.json',
    columns: [{
      title: 'FORM_PRINT_TEMPLATE_ID',
      field: 'printtemplate_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'FORM_PRINT_TEMPLATE_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }, {
      title: 'FORM_PRINT_TEMPLATE_FORMAT',
      field: 'rt_format_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true
    }, {
      title: 'FORM_PRINT_TEMPLATE_ORIENTATION',
      field: 'rt_orientation_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true
    }, {
      title: 'FORM_PRINT_TEMPLATE_ORDER',
      field: 'printtemplate_order',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true
    }]
  }
};

const vmapMapsPrintingsStylesObject: AppModeConfig = {
  name: 'vmap_maps_print_templates_list',
  label: 'TITLE_MODE_PRINTINGS_PRINT_STYLES',
  route: 'styles',
  type: 'workspacelist',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/printstyles',
    id_field: 'printstyle_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    clone: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    form_path: 'modules/vmap/forms/print/styles.json',
    columns: [{
      title: 'FORM_PRINT_STYLE_ID',
      field: 'printstyle_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'FORM_PRINT_STYLE_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }]
  }
};

export const vmapMapPrintingsMode: AppModeConfig = {
  name: 'vmap_map_printings',
  route: 'printings',
  label: 'TITLE_MODE_PRINTINGS',
  icon: 'icon-vitis-print',
  privilege: 'vmap_admin',
  objects: [vmapMapsPrintingsTemplatesObject, vmapMapsPrintingsStylesObject]
};

export const vmapUsersSection: AppModeConfig = {
  name: 'vitis_users_vmap_details',
  label: 'TITLE_MODE_USERS_USER_VMAP',
  route: 'vmap',
  type: 'simpleform',
  privilege: 'vitis_admin',
  options: {
    module: 'vitis',
    ressource: 'vmap/useroptions',
    id_field: 'user_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    search: {
      custom: true,
      form: false,
      default: 'custom'
    },
    form_path: 'modules/vmap/forms/users/users_vitis_users_vmap_user.json',
    js_path: 'modules/vmap/forms/users/users_vitis_users_vmap_user.js',
  }
};

export const vmapUsersGroupsSpecSection: AppModeConfig = {
  name: 'vitis_users_groups_spec',
  label: 'TITLE_MODE_USERS_USER_VMAP',
  route: 'specificites_vmap',
  type: 'simpleform',
  privilege: ['vitis_admin'],
  options: {
    module: 'vitis',
    ressource: 'vmap/printgroup',
    id_field: 'group_id',
    display: true,
    update: true,
    insert: true,
    form_path: 'modules/vmap/forms/groups/groups_spec_vmap.json'
  }
};

const vmapPublicationsGeneralesSection: AppModeConfig = {
  name: 'vmap_publications_informations_generales_section',
  label: 'TITLE_MODE_PUBLICATIONS_GENERAL',
  route: 'informations_generales',
  type: 'simpleform',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/publications',
    id_field: 'publication_id',
    update: true,
    insert: true,
    map_field: 'publicationtype',
    form_path: {
      'postgres': 'modules/vmap/forms/publications/publicationpostgres_simple.json',
      'file': 'modules/vmap/forms/publications/publicationfile_simple.json'
    },
    js_services: {
      'WmsService': import('src/modules/vmap/components/admin-datasource/services/wms.service'),
      'WmtsService': import('src/modules/vmap/components/admin-datasource/services/wmts.service'),
      'AdminDatasourceFormService': import('src/modules/vmap/components/admin-datasource/services/admin-datasource-form.service'),
    }
  }
};

const vmapPublicationsAdvancedSection: AppModeConfig = {
  name: 'vmap_publications_advanced_section',
  label: 'TITLE_MODE_PUBLICATIONS_ADVANCED',
  route: 'advanced',
  type: 'simpleform',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/publications',
    id_field: 'publication_id',
    update: true,
    insert: true,
    map_field: 'publicationtype',
    form_path: {
      'postgres': 'modules/vmap/forms/publications/publicationpostgres_advanced.json',
      'file': 'modules/vmap/forms/publications/publicationfile_advanced.json'
    }
  }
};

const vmapPublicationsSymbologie: AppModeConfig = {
  name: 'vmap_publications_symbology_section',
  label: 'TITLE_MODE_PUBLICATIONS_SYMBOLOGY',
  route: 'symbology',
  type: 'module',
  privilege: 'vmap_admin',
  options: {
    module_name: 'SectionLayerSymbologyModule',
    ressource: 'vmap/publications',
    ressourceTest: 'vmap/datasourcetreeview/pgpublications',
    id_field: 'publication_id',
    update: true,
    insert: true,
    delete: true,
  }
};

const vmapPublicationsList: AppModeConfig = {
  name: 'vmap_publications',
  label: 'TITLE_MODE_PUBLICATIONS',
  route: 'publications',
  type: 'workspacelist',
  privilege: 'vmap_admin',
  sections: {
    'postgres': [
      vmapPublicationsGeneralesSection,
      vmapPublicationsAdvancedSection,
      vmapPublicationsSymbologie,
    ],
    'file': [
      vmapPublicationsGeneralesSection,
      vmapPublicationsAdvancedSection,
      vmapPublicationsSymbologie,
    ]
  },
  options: {
    module: 'vmap',
    ressource: 'vmap/publications',
    id_field: 'publication_id',
    map_field: 'publicationtype',
    display: true,
    update: true,
    insert: false, // Línsertion doit se faire par la partie sources de données, autrement ça crée des problèmes de conception
    delete: true,
    form_path: 'modules/vmap/forms/publications/publicationpostgres_simple.json',
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    columns: [{
      title: 'PUBLICATION_ID',
      field: 'publication_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'PUBLICATION_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    },
    {
      title: 'PUBLICATION_TITLE',
      field: 'title',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    },
    {
      title: 'PUBLICATION_TYPE',
      field: 'publicationtype',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchWebService: false
    },
    {
      title: 'PUBLICATION_ACTIVE',
      field: 'active',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchType: 'boolean',
      searchSuggestions: [{
        value: true,
        label: 'FORM_YES'
      }, {
        value: false,
        label: 'FORM_NO'
      }],
      template: WklBooleanIconComponent
    }]
  }
};

const vmapFluxGeneralSection: AppModeConfig = {
  name: 'vmap_flux_informations_generales_section',
  label: 'TITLE_MODE_FLUX_GENERAL',
  route: 'informations_generales',
  type: 'simpleform',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/flux',
    id_field: 'flux_id',
    form_path: 'modules/vmap/forms/flux/flux.json',
    js_path: 'modules/vmap/forms/flux/flux.js',
    update: true,
    insert: true,
    delete: true,
  }
};

const vmapFluxPublicationsSection: AppModeConfig = {
  name: 'vmap_flux_publications_section',
  label: 'TITLE_MODE_FLUX_PUBLICATIONS',
  route: 'publications',
  type: 'draganddrop-grid',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/flux/:id/publications',
    ressource_source: 'publications',
    id_field: 'flux_layer_id',
    name_field: 'name',
    edit: false,
    reversed: false,
    search: {
      custom: true,
      form: false,
      default: 'custom'
    },
    buttons: [],
    columns: [],
    source: {
      ressource: 'vmap/publications',
      id_field: 'publication_id',
      id_ressource_field: 'flux_id',
      titleModalInsert: 'Ajouter une publication',
      insert: true,
      columns: [{
        title: 'PUBLICATION_ID',
        field: 'publication_id',
        data: 'publication_id',
        name: 'publication_id',
        align: 'left',
        resizeable: true,
        sortable: true,
        searchable: true,
        searchType: 'number'
      },
      {
        title: 'PUBLICATION_NAME',
        field: 'name',
        data: 'name',
        align: 'left',
        resizeable: true,
        sortable: true,
        searchable: true,
      },
      {
        title: 'PUBLICATION_TYPE',
        field: 'publicationtype',
        data: 'publicationtype',
        align: 'left',
        resizeable: true,
        sortable: true,
        searchable: true,
        searchType: 'strict'
      }
      ],
    }
  }
};

const vmapFluxList: AppModeConfig = {
  name: 'vmap_flux',
  label: 'TITLE_MODE_FLUX',
  route: 'flux_wms',
  type: 'workspacelist',
  privilege: 'vmap_admin',
  sections: [
    vmapFluxGeneralSection,
    vmapFluxPublicationsSection,
  ],
  options: {
    module: 'vmap',
    ressource: 'vmap/flux',
    id_field: 'flux_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    columns: [{
      title: 'FLUX_ID',
      field: 'flux_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    },
    {
      title: 'FLUX_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    },
    {
      title: 'FLUXTYPE_NAME',
      field: 'fluxtype_name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number',
      searchWebService: {
        resource: 'vmap/fluxtype',
        params: {
          attributs: 'fluxtype_id|label',
          order_by: 'fluxtype_id',
          sort_order: 'DESC'
        },
        label: 'label',
        value: 'fluxtype_id',
      }
    },
    {
      title: 'FLUX_COORDSYS_ID',
      field: 'coordsys_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchType: 'number',
    }]
  }
};

export const vmapFluxMode: AppModeConfig = {
  name: 'vmap_flux',
  route: 'flux',
  label: 'TITLE_MODE_VMAP_FLUX',
  icon: 'icon-vitis-FDS_01',
  privilege: 'vmap_admin',
  objects: [vmapFluxList, vmapPublicationsList]
};

export const vmapMapReportsMode: AppModeConfig = {
  name: 'vmap_map_reports',
  label: 'TITLE_MODE_REPORTS',
  icon: 'icon-vitis-vmap_business_object',
  route: 'reports',
  type: 'workspacelist',
  privilege: 'vmap_admin',
  options: {
    module: 'vmap',
    ressource: 'vmap/printreport',
    id_field: 'printreport_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    clone: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    form_path: 'modules/vmap/forms/print/reports.json',
    columns: [{
      title: 'FORM_PRINTTEMPLATE_ID_VMAP_PRINTREPORT',
      field: 'printreport_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'FORM_NAME_VMAP_PRINTREPORT',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
    }, {
      title: 'FORM_FORMAT_VMAP_PRINTREPORT',
      field: 'rt_format_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true
    }, {
      title: 'FORM_ORIENTATION_VMAP_PRINTREPORT',
      field: 'rt_orientation_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true
    }, {
      title: 'FORM_OUTPUT_FORMATS_VMAP_PRINTREPORT',
      field: 'outputformats_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true
    }, {
      title: 'FORM_LAYER_VMAP_PRINTREPORT',
      field: 'layer_name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true
    }]
  }
};
