<?php

namespace App\Module\Vmap\WebService\Controller;

use App\Module\Vitis\WebService\Controller\GenericQueriesController;
use App\Module\Vitis\WebService\Repository\GenericRepository;
use App\Module\Vmap\WebService\Entity\Datasource;
use App\Module\Vmap\WebService\Entity\LayerPostgres;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller Intermédiaire entre vitis/genericqueries et PgLayerController et Datasourcetreeviewcontroller.
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */
abstract class AbstractVmapQueriesController extends GenericQueriesController {
  /**
   * Récupère les informations de connexion de la couche à partir de l'identifiant
   * @param mixed $id Identifiant de la datasource
   * @param bool $useCurrentUser
   * @param string|null $sDbName
   * @return array|null
   */
  protected function getDataConnectionFromDatasource($id, bool $useCurrentUser = false, ?string $sDbName = null): ?array{
    $aReturn = [
      'status' => 0,
      'errorMessage' => 'ERROR_GENERIC_CONTROLLER_GET',
    ];

    $oDsRepository = $this->oDatabaseConnection->getUserEntityManager()->getRepository(Datasource::class);
    $oDs = null;
    if ($oDsRepository instanceof GenericRepository) {
      $oDs = $oDsRepository->findById($id)[0];
    }

    if (empty($oDs)) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Pglayer getPgLayerFormColumns : can\'t get columns');
      $aReturn['message'] = 'Error during get column';
      $this->formatResponse($aReturn, Response::HTTP_UNAUTHORIZED);
      return null;
    }

    // Récupère les informations de connexion de la couche
    $aDsData = $oDs->getPostgresConnexionData(!$useCurrentUser, $sDbName);

    if (empty($aDsData)) {
      $aReturn['message'] = 'Missing database or schema or pg_table in datasource';
      $this->formatResponse($aReturn, Response::HTTP_UNAUTHORIZED);
      return null;
    }

    return $aDsData;
  }

  /**
   * Récupère les informations de connexion de la couche à partir de l'identifiant
   * @param mixed $id Identifiant de la couche
   * @param string|null $sSchemaType Type de schema/table (null = schema et pg_table; 'form' = schema_form et table_form; 'requestor' = schema_requestor et table_requestor )
   * @param bool $useCurrentUser
   * @return array|null
   */
  protected function getDataConnectionFromLayer($id, ?string $sSchemaType = null, bool $useCurrentUser = false): ?array{
    $aReturn = [
      'status' => 0,
      'errorMessage' => 'ERROR_GENERIC_CONTROLLER_GET',
    ];

    $oLayerRepository = $this->oDatabaseConnection->getAdminEntityManager()->getRepository(LayerPostgres::class);
    $oLayer = null;
    if ($oLayerRepository instanceof GenericRepository) {
      $oLayer = $oLayerRepository->findById($id)[0];
    }

    if (empty($oLayer)) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Pglayer getPgLayerFormColumns : can\'t get columns');
      $aReturn['message'] = 'Error during get column';
      $this->formatResponse($aReturn, Response::HTTP_UNAUTHORIZED);
      return null;
    }

    // Récupère les informations de connexion de la couche
    $aLayerData = $oLayer->getDataConnectionFromLayer(!$useCurrentUser, $sSchemaType);

    if (empty($aLayerData)) {
      $aReturn['message'] = 'Missing database or schema or pg_table in datasource';
      $this->formatResponse($aReturn, Response::HTTP_UNAUTHORIZED);
      return null;
    }

    return $aLayerData;
  }

  /**
   * Vérifie que les paramètres de connexion requis sont présent.
   *
   * @param mixed $aData
   * @return bool
   */
  protected function isConnectionDatasPresent($aData): bool {
    return $this->isFieldPresent($aData, 'login')
    && $this->isFieldPresent($aData, 'password')
    && $this->isFieldPresent($aData, 'server')
    && $this->isFieldPresent($aData, 'port');
  }

  /**
   * Vérifie qu'un paramètre est présent.
   *
   * @param array|null $aData
   * @param string $sField
   * @return bool
   */
  protected function isFieldPresent(?array $aData, string $sField): bool {

    if(empty($aData)) {
      return false;
    }

    if (!array_key_exists($sField, $aData)) {
      return false;
    }

    if (empty($aData[$sField]) || $aData[$sField] == 'null') {
      return false;
    }

    return true;
  }

  /**
   * Récupère le namespace de la couche à partir de l'identifiant
   * @param mixed $id Identifiant de la couche
   * @return array|null
   */
  protected function getNamespaceFromLayer($id): ?string{
    $aReturn = [
      'status' => 0,
      'errorMessage' => 'ERROR_GENERIC_CONTROLLER_GET',
    ];

    $oLayerRepository = $this->oDatabaseConnection->getAdminEntityManager()->getRepository(LayerPostgres::class);
    $oLayer = null;
    if ($oLayerRepository instanceof GenericRepository) {
      $oLayer = $oLayerRepository->findById($id)[0];
    }

    if (empty($oLayer)) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Pglayer getPgLayerFormColumns : can\'t get columns');
      $aReturn['message'] = 'Error during get column';
      $this->formatResponse($aReturn, Response::HTTP_UNAUTHORIZED);
      return null;
    }

    // Récupère le namespace
    return $oLayer->getNamespace();
  }
}
