<?php

namespace App\Module\Vmap\WebService\Controller;

use App\Module\Vitis\WebService\Controller\GenericController;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller pour le traitement des géométries
 *
 * @author Kyllian Beasse <kyllian.beasse@veremes.com>
 */
class GeometryController extends GenericController {
    
    /**
    * Retourne deux polygonne résultant du découpage d'un polygone et d'une ligne en entrée
    * @param string $entryPolygon
    * @param string $entryLine
    * @return Response
    */
    public function DisectPolygon() {
        $aData = $this->requestToArray();
        $aReturn = [
            'status' => 0,
          ];
        $aParams = [
            'GEOM1' => ['value' => $aData['geom1'] , 'type' => 'string'],
            'GEOM2' => ['value' => $aData['geom2'], 'type' => 'string'],
            'EPSG' => ['value' => intval($aData['result_srid']), 'type' => 'integer']
        ];
        $aReturn['message'] = $aData;
        if(!isset($aData['geom1']) && !isset($aData["geom2"])) {
            $aReturn['errorMessage'] = 'REQUEST_ERROR';
            $aReturn['message'] = "Missing geometries parameters";
            return $this->formatResponse($aReturn);
        }

        try {
            $qb = $this->oDatabaseConnection->getQueryAccessor()->createQueryBuilder();
            $qb2 = $this->oDatabaseConnection->getQueryAccessor()->createQueryBuilder();
            $qb->select('GeometryType(ST_GeomFromEWKT([GEOM1])) as geom_type;');
            $qb2->select('GeometryType(ST_GeomFromEWKT([GEOM2])) as geom_type;');
            try {
                $aResult = [
                    $qb->executeWithParams($aParams)->fetch()['geom_type'],
                    $qb2->executeWithParams($aParams)->fetch()['geom_type']
                ];

                if(!in_array($aResult[0], array('POLYGON', 'LINESTRING')) || !in_array($aResult[1], array('LINESTRING', 'POINT'))) {
                    $aReturn['format'] = $aResult[0];
                    $aReturn['message'] = 'Wrong geometry format';
                    $aReturn['errorMessage'] = 'WRONG_GEOM_FORMAT';
                    return $this->formatResponse($aReturn);
                }
                
                if(($aResult[0] == 'POLYGON' && $aResult[1] == 'LINESTRING')  || ($aResult[0] == 'LINESTRING' && $aResult[1] == 'POLYGON'))
                {
                    $qb->select('ST_AsEWKT(st_transform((ST_Dump(ST_Split(area, new_line))).geom, [EPSG]::integer)) As wkt, 
                    st_dwithin(ST_GeomFromEWKT(area),ST_GeomFromEWKT(new_line), 0.1) as is_in')
                    ->from('(
                        select area, 
                        (select ST_AsEWKT(ST_MakeLine(geom)) as new_line from
                        ((select array[0] as path, geom from ST_Dump(ST_Intersection(boundary, ST_Scale(first_segment, ST_MakePoint(dist/ST_length(first_segment), dist/ST_length(first_segment)), ST_EndPoint(first_segment)))) order by ST_Distance(geom, ST_StartPoint(first_segment)) limit 1)
                        union all
                        select * from ST_Dump(line)
                        union all
                        (select array [ST_NumPoints(line)+1] as path, geom 
                        from ST_Dump(ST_Intersection(boundary, ST_Scale(last_segment, ST_MakePoint(dist/ST_length(last_segment), dist/ST_length(last_segment)), ST_StartPoint(last_segment)))) 
                        order by ST_Distance(geom, ST_StartPoint(last_segment)) limit 1)) as points
                        )
                        from
                        (select area, line, 
                            ST_Length(ST_BoundingDiagonal(ST_Translate(ST_Scale(area, 2, 2), 
                            ST_X(ST_Centroid(area))*(1 - 2), 
                            ST_Y(ST_Centroid(area))*(1 - 2)))) as dist,
                            (select st_makeline(geom) from ST_DumpPoints(line) where path in (array[1],array[2])) as first_segment,
                            (select st_makeLine(geom) from ST_DumpPoints(line) where path in (array[ST_NPoints(line)],array[ST_NPoints(line)-1])) as last_segment,
                            ST_Boundary(ST_Translate(ST_Scale(area, 2, 2), 
                            ST_X(ST_Centroid(area))*(1 - 2), 
                            ST_Y(ST_Centroid(area))*(1 - 2))) as boundary
                            from (values ([GEOM1]::geometry,[GEOM2]::geometry)) as t(area,line)) as parts
                    ) as e');
                } else {
                    $qb->select('wkt, is_in')
                    ->from('(SELECT ST_AsEWKT(ST_Transform(ST_LineSubstring(line, 0, ST_LineLocatePoint(line, point)), [EPSG]::integer)) as line1,
                    ST_AsEWKT(ST_Transform(ST_LineSubstring(line, ST_LineLocatePoint(line, point), 1), [EPSG]::integer)) as line2,
                    st_dwithin(line,point, 0.1) as is_in
               FROM 
                   ST_GeomFromText([GEOM1]) as line, 
                   ST_GeomFromText([GEOM2]) as point) as foo
                   cross join lateral (values (line1), (line2)) as c(wkt);');
                }
                try {
                    if(isset($qb->executeWithParams($aParams)->fetch()['is_in']) && $qb->executeWithParams($aParams)->fetch()['is_in'] != true) {
                        $aReturn['message'] = 'Drawn geometry is not within the initial geometry extend';
                        $aReturn['errorMessage'] = 'REQUEST_ERROR';
                        return $this->formatResponse($aReturn, Response::HTTP_BAD_REQUEST);
                    }
                    $aResult = $qb->executeWithParams($aParams)->fetchall(\PDO::FETCH_COLUMN);
                    return $this->formatResponse($aResult);
                } catch(\Exception $e) {
                    $this->oLoggerManager->get('SQL_ERROR')->error('Error processing geometry' . $e->getMessage());
                    $aReturn['message'] = 'Error processing Geometry';
                    $aReturn['errorMessage'] = 'REQUEST_ERROR';
                    return $this->formatResponse($aReturn);
                }
            } catch(\Exception $e) {
                $this->oLoggerManager->get('SQL_ERROR')->error('Error processing geometry' . $e->getMessage());
                $aReturn['message'] = 'Error processing Geometry';
                $aReturn['errorMessage'] = 'REQUEST_ERROR';
                return $this->formatResponse($aReturn);
            }
        } catch (\Exception $e) {
            $aReturn['errorMessage'] = 'INTERNAL_ERROR';
            $aReturn['message'] = "ERROR";
            return $this->formatResponse($aReturn);
        }
    }
}