<?php

namespace App\Module\Vmap\WebService\Controller;

use App\Module\Vitis\WebService\Controller\GenericController;
use App\Module\Vmap\WebService\Entity\AbstractLayer;
use App\Module\Vmap\WebService\Entity\LayerMap;
use App\Module\Vmap\WebService\Entity\Map;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller pour les cartes
 *
 * @author Laurie Ambroise <laurie.ambroise@veremes.com>
 */
class MapController extends GenericController {

  /**
   * Ajoute une entité Map
   * @return Response réponse à retourner par l'API
   */
  public function postData(): Response {
    $aData = $this->requestToArray();
    // Sauvegarde en base de donnée
    $oEntity = $this->insertDataInDb(json_encode($aData));

    if ($oEntity == null) {
      return $this->formatGenericResponseForErrorOnInsert();
    }

    if ($oEntity['status'] == 0) {
      return $this->formatGenericResponseForErrorOnInsert();
    }

    // Si l'attribut layers existe lors que l'ajout d'une carte, on ajoute les layers dans la table layer_map
    if (isset($aData['layers']) && count($aData['layers']) > 0) {
      // Récupère la route layer_map
      $oLayerMapRoute = $this->oRouteConfig->getRequestParameters('vmap_maps_layers_post');
      $oLayerMapRoute['options']['schema'] = "@prop(schema_vmap)";
      $oLayerMapRoute['options']['table'] = "layer_map";

      $oDataAccessor = $this->getDataAccessor();
      $oDataAccessor->setOptions($oLayerMapRoute['options']);
      $this->oRouteConfig->setOptionsParameter($oLayerMapRoute['options']);

      // Récupère tous les setters de layer_map
      $aSetterMap = [];
      foreach ($oLayerMapRoute['options']['fields'] as $aField) {
        if (isset($aField['orm_set'])) {
          $aSetterMap[$aField['name']] = $aField['orm_set'];
        }
      }

      // Parcours le tableau de couches de la carte pour les ajouter en bdd
      foreach ($aData['layers'] as $layer) {
        $objLayer = (array) json_decode($layer);
        $objLayer['map_id'] = $oEntity['data']->getMapId();
        if (isset($objLayer['layer_id'])) {
          $layerMap = new LayerMap();
          $layerMap->setLayer($this->oDatabaseConnection->getAdminEntityManager()->find(AbstractLayer::class, $objLayer['layer_id']));
          $layerMap->setMap($this->oDatabaseConnection->getAdminEntityManager()->find(Map::class, $objLayer['map_id']));
          // Filtre les setters qui ont une valeur
          $allowedFilter = array_filter($aSetterMap, function (mixed $key) use ($objLayer) {
            return isset($objLayer[$key]);
          }, ARRAY_FILTER_USE_KEY);
          foreach ($allowedFilter as $key => $value) {
            //@phpstan-ignore-next-line
            $layerMap->$value($objLayer[$key]);
          }

          // Sauvegarde l'object layermap en bdd
          $this->oDatabaseConnection->getAdminEntityManager()->persist($layerMap);
        }
      }
      $this->oDatabaseConnection->getAdminEntityManager()->flush();
    }

    return $this->formatResponse($oEntity);
  }
}
