<?php

namespace App\Module\Vmap\WebService\Controller;

use App\Module\Vitis\WebService\Repository\GenericRepository;
use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Configuration\Route;
use App\Module\Vitis\WebService\Service\DataAccessor\DatabaseConnection;
use App\Module\Vitis\WebService\Service\FileAccessor\FileManager;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use App\Module\Vitis\WebService\Service\VitisRequest;
use App\Module\Vmap\WebService\Entity\AbstractPublication;
use App\Module\Vmap\WebService\Service\MapServerService;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller pour le datasource treeview de vMap.
 *
 * @author Laurie Ambroise <laurie.ambroise@veremes.com>
 */
class PgPublicationController extends AbstractVmapQueriesController {
  /**
   * @var MapServerService
   */
  private $oMapServerService;

  /**
   * @var Properties
   */
  private $oPropertiesSrvc;

  /**
   * Initialise le controler et instancie les services requis
   * @param Properties $oPropertiesSrvc Service d'accès aux properties
   * @param VitisRequest $oVitisRequest Service d'abstraction de la requête Symfony
   * @param Route $oRouteConfig Service d'accès à la configuration de la route
   * @param LoggerManager $oLoggerManager Service d'utilsation des logs
   * @param DatabaseConnection $oDatabaseConnection Service d'accès à la base de données
   * @param FileManager $oFileManager Service de gestion des fichiers
   * @param MapServerService $oMapServerService
   */
  public function __construct(Properties $oPropertiesSrvc, VitisRequest $oVitisRequest, Route $oRouteConfig, LoggerManager $oLoggerManager, DatabaseConnection $oDatabaseConnection, FileManager $oFileManager, MapServerService $oMapServerService) {
    parent::__construct($oPropertiesSrvc, $oVitisRequest, $oRouteConfig, $oLoggerManager, $oDatabaseConnection, $oFileManager);
    $this->oMapServerService = $oMapServerService;
    $this->oPropertiesSrvc = $oPropertiesSrvc;
  }

  /**
   * Retourne la définition de la layer
   * @param mixed $id
   * @return array|null
   */
  private function getPublicationDef($id): ?array{
    $aPublicationDef = [];
    $oPublicationRepository = $this->oDatabaseConnection->getAdminEntityManager()->getRepository(AbstractPublication::class);

    if (!$oPublicationRepository instanceof GenericRepository) {
      return null;
    }

    $oLayer = $oPublicationRepository->findById($id)[0];

    // Getters de l'entité AbstractPublication
    $oLayerRoute = $this->oRouteConfig->getRequestParameters('vmap_publication');
    $aPublicationDef = $oLayer->jsonSerialize($oLayerRoute);

    return $aPublicationDef;
  }

  /**
   * Test d'une couche
   * @param mixed $id
   * @return Response
   */
  public function testPublication($id): Response {
    $aData = $this->requestToArray();
    $aReturn = [
      'status' => 0,
    ];

    $aPublicationDef = $this->getPublicationDef($id);

    if (array_key_exists('publication_id', $aPublicationDef) &&
      array_key_exists('definition_sym_json', $aData) &&
      array_key_exists('layertype', $aData)) {
      $aPublicationDef['definition_sym_json'] = $aData['definition_sym_json'];
      $aPublicationDef['layertype'] = $aData['layertype'];
      // Retour de la définition mapserver
      switch ($aData['layertype']) {
        case 'file':
          $aMSLayerDef = $this->oMapServerService->getMSLayerRasterDef($aPublicationDef);
          break;
        default:
          if (!array_key_exists('coordsys_id', $aPublicationDef) ||
            !array_key_exists('ms_geomtype_id', $aPublicationDef) ||
            !array_key_exists('geomfield', $aPublicationDef)) {
            return $this->formatResponse($aReturn = [
              'status' => 0,
              'errorMessage' => 'ERROR_MISSING_GEOMETRY_PARAMS',
            ]);
          }

          if (array_key_exists("geomtype", $aData)) {
            $aPublicationDef['ms_geomtype']['label'] = strtoupper($aData['geomtype']);
            $aPublicationDef['multiGeom'] = true;
            $aMSLayerDef = $this->oMapServerService->getMSLayerPostgresDef($aPublicationDef, true);
          } else {
            $aMSLayerDef = $this->oMapServerService->getMSLayerPostgresDef($aPublicationDef);
          }
          // Générer le fichier fonts.list avec les données en bdd
          $this->oMapServerService->generateFontList();
      }
      $sMSLayerDef = join("\r\n", $aMSLayerDef);

      // Génération du mapfile de test
      $sMapfileName = $this->oMapServerService->createMapfile([
        'map_id' => 'test_map',
        'name' => 'test_map',
      ], [$aPublicationDef], null, true);

      // Construction de l'URL du flux
      $sOnlineResourceUrl = str_replace('@file', $sMapfileName, MapServerService::PUBLIC_WMS_MAIN_URL);
      $sOnlineResourceUrl = $this->oPropertiesSrvc->replacePropertiesInString($sOnlineResourceUrl);
      $sFluxUrl = $sOnlineResourceUrl;

      $aReturn = [
        'data' => [
          'mapserver_def' => $sMSLayerDef,
          'flux_url' => $sFluxUrl,
          'layer_name' => $this->oMapServerService->cleanLayerName(array_key_exists('uniquename', $aPublicationDef) ? $aPublicationDef['uniquename'] : $aPublicationDef['name']),
        ],
        'status' => 1,
      ];
    }

    return $this->formatResponse($aReturn);
  }
}
