<?php

namespace App\Module\Vmap\WebService\Controller;

use App\Module\Vitis\WebService\Controller\GenericController;
use Symfony\Component\HttpClient\HttpClient;
use Symfony\Component\HttpFoundation\Response;
use XMLReader;

/**
 * Controller pour le proxy de vMap.
 *
 * @author Ludivine Premont <ludivine.premont@veremes.com>
 */
class ProxyController extends GenericController {
  /**
   * @var array
   */
  protected const HTTP_CLIENT_OPTIONS = [
    'timeout' => 900, // Temps max file_get_contents (900 = 15min)
    'headers' => [],
  ];

  /**
   * Vérifie que le proxy est correcte
   * @return Response
   */
  public function checkProxy() {
    $aData = $this->requestToArray();
    $sUrl = $aData['url'];

    // preset de la structure de réponse
    $aReturn = ['status' => 0];

    foreach ($aData as $key => $value) {
      if ($key !== 'headers' && $key !== 'url') {
        if (strpos($sUrl, '?') == false) {
          $sUrl .= '?' . $key . '=' . urlencode($value);
        } else {
          $sUrl .= '&' . $key . '=' . urlencode($value);
        }
      }
    }

    $oOption = self::HTTP_CLIENT_OPTIONS;
    // activation ou non la vérification SSL
    $oOption['verify_peer'] = $this->oProperties->get('proxy_check_ssl');
    $oOption['verify_host'] = $this->oProperties->get('proxy_check_ssl');

    /**
     * Authentification basic
     */
    if (isset($aData['headers']) && $aData['headers'] != '') {
      foreach (json_decode($aData['headers']) as $key => $value) {
        $oOption['headers'][$key] = $value;
      }
    }

    $client = HttpClient::create();
    if (filter_var($sUrl, FILTER_VALIDATE_URL) != false && (strpos($sUrl, 'http://') === 0 || strpos($sUrl, 'https://') === 0)) {
      $oResponse = $client->request('GET', $sUrl, $oOption);
      if ($oResponse->getStatusCode() == Response::HTTP_OK) {
        $string = $oResponse->getContent();

        $aReturn = ['status' => 1, 'data' => $string];
        $this->formatResponse($aReturn);
        if ($this->isXML($string)) {
          return $this->oResponse;
        } else if ($this->isImage($string)) {
          if (strpos($string, 'PNG')) {
            $this->oResponse->headers->set('Content-Type', 'image/png');
          }
          if (strpos($string, 'JPG')) {
            $this->oResponse->headers->set('Content-Type', 'image/jpg');
          }
          if (strpos($string, 'JPEG')) {
            $this->oResponse->headers->set('Content-Type', 'image/jpeg');
          }
          return $this->oResponse;
        } else {
          $aReturn['message'] = 'Return format not valid';
          $aReturn['errorMessage'] = 'ERROR_CHECK_PROXY_FORBIDDEN';
          return $this->formatResponse($aReturn, Response::HTTP_FORBIDDEN);
        }
      } else {
        $aReturn['message'] = 'Not found';
        $aReturn['errorMessage'] = 'ERROR_CHECK_PROXY_HTTP_NOT_FOUND';
        return $this->formatResponse($aReturn, Response::HTTP_NOT_FOUND);
      }
    } else {
      $aReturn['message'] = 'forbidden';
      $aReturn['errorMessage'] = 'ERROR_CHECK_PROXY_FORBIDDEN';
      return $this->formatResponse($aReturn, Response::HTTP_FORBIDDEN);
    }
  }

  /**
   * Return true if the result is an XML
   * @param string $xml_string
   * @return boolean
   */
  protected function isXML(string $xml_string): bool {
    $aXmlArray = simplexml_load_string($xml_string);
    if (!$aXmlArray) {
      return false;
    }
    $xml = new XMLReader();
    if (!$xml->XML($xml_string, null, LIBXML_DTDVALID)) {
      $this->oLoggerManager->get('API_ERROR')->error('Proxy : XML not valid: load error ');
      return false;
    } else {
      return true;
    }
  }

  /**
   * Return true if the result is an image
   * @param string $image_string
   * @return bool
   */
  protected function isImage(string $image_string): bool {
    if (@is_array(getimagesizefromstring($image_string))) {
      return true;
    } else {
      return false;
    }
  }

}
