<?php

namespace App\Module\Vmap\WebService\Controller;

use App\Module\Vitis\WebService\Controller\GenericController;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller pour les symbols de vMap.
 *
 * @author Ludivine Premont <ludivine.premont@veremes.com>
 */
class SymbolController extends GenericController {
  /**
   * Met à jour une entité pour une route de type PUT Module/Objet/{{id}}.
   * @param mixed $id id de l'entité à trouver
   * @return Response réponse à retourner par l'API
   */
  public function updateData(mixed $id): Response {
    $aData = $this->requestToArray();

    $aData = $this->extractDefinition($id, $aData);
    if (!empty($aData) && array_key_exists('status', $aData) && $aData['status'] === 0) {
      return $this->formatGenericErrorRequestError($aData['message']);
    }
    // Sauvegarde en base de donnée
    $oEntity = $this->updateDataInDb($id, json_encode($aData));
    if ($oEntity['status'] == 0) {
      return $this->formatGenericResponseForErrorOnUpdate();
    }

    return $this->formatResponse($oEntity);
  }

  /**
   * Créé une entité pour une route de type POST Module/Objet.
   * @return Response réponse à retourner par l'API
   */
  public function postData(): Response {
    $aData = $this->requestToArray();

    // Pour la contrainte not null sur type lors de l'ajout
    if (empty($aData['type'])) {
      $aData['type'] = $aData['type_symbol'];
    }

    // Sauvegarde en base de donnée
    $oEntity = $this->insertDataInDb(json_encode($aData));

    if ($oEntity == null) {
      return $this->formatGenericResponseForErrorOnInsert();
    }

    $aRequestOptions = $this->getRequestParameters();
    $id = $oEntity['data'][$aRequestOptions['options']['field_id']];

    // Génère la définition et update
    $aData = $this->extractDefinition($id, $aData);
    if (!empty($aData) && array_key_exists('status', $aData) && $aData['status'] === 0) {
      return $this->formatGenericErrorRequestError($aData['message']);
    }

    $oEntity = $this->updateDataInDb($id, json_encode($aData));
    if ($oEntity['status'] == 0) {
      return $this->formatGenericResponseForErrorOnUpdate();
    }

    if ($oEntity['status'] == 0) {
      return $this->formatGenericResponseForErrorOnInsert();
    }
    return $this->formatResponse($oEntity);
  }

  /**
   * @param mixed $id
   * @param array $aData
   * @return array
   */
  private function extractDefinition(mixed $id, array $aData): array{
    $type = $aData['type'];
    if (empty($type)) {
      $type = $aData['type_symbol'];
    }

    switch ($type) {
      case 'font':
        $aData['definition'] = "SYMBOL\n";
        $aData['definition'] .= "\tNAME \"" . $aData['name'] . "\"\n";
        $aData['definition'] .= "\tTYPE TRUETYPE\n";
        $aData['definition'] .= "\tFONT " . $aData['font'] . "\n";
        $aData['definition'] .= "\tCHARACTER '" . $aData['font_character'] . "'\n";
        $aData['definition'] .= "\tANTIALIAS TRUE\n";
        $aData['definition'] .= "END\n";
        $aData['type'] = "font";
        break;
      case 'image':
        // Si l'image a été modifier on modifie la définition
        if (isset($_FILES['image'])) {
          $aData['definition'] = "SYMBOL\n";
          $aData['definition'] .= "\tNAME \"" . $aData['name'] . "\"\n";

          if ($this->getLocalFileAccessor()->getExtension($_FILES['image']['name']) == 'svg') {
            $aData['definition'] .= "\tTYPE SVG\n";
          } else {
            $aData['definition'] .= "\tTYPE PIXMAP\n";
          }

          $aData['definition'] .= "\tIMAGE \"../symbols/images/" . $id . '/' . $_FILES['image']['name'] . "\"\n";
          $aData['definition'] .= "\tTRANSPARENT 0\n";
          $aData['definition'] .= "END\n";
        } else {
          $aData['definition'] = preg_replace('/(NAME) (.*)/', '${1} "' . $aData['name'] . '"', $aData['definition']);
        }
        $aData['type'] = "image";
        break;
      case 'mapserver':
        $aData['type'] = 'mapserver';
        break;
      default:
        return ['status' => 0, 'message' => 'Type not valid : ' . $type];
    }

    return $aData;
  }
}
