<?php

namespace App\Module\Vmap\WebService\Controller;

use App\Module\Vitis\WebService\Controller\GenericController;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller pour la section vMap dans la gestion des utilisateurs
 *
 */
class UserOptionsController extends GenericController {
  /**
   * Retourne les options spécifiques à l'utilisateur
   * @param mixed $id user_id
   * @return Response réponse à retourner par l'API
   */
  public function getData($id): Response {
    $aData = $this->requestToArray();

    // chargement de la connexion à la base de donnée
    $oDataAccessor = $this->getDataAccessor();
    $oEntity = $oDataAccessor->getEntity($id, $aData);

    if ($oEntity['status'] === 0) {
      return $this->oResponse;
    }

    $this->formatResponse($oEntity, Response::HTTP_OK);
    return $this->oResponse;
  }

  /**
   * Met à jour une entité pour une route de type PUT Module/Objet/{{id}}.
   * @param mixed $id id de l'entité à trouver
   * @return Response réponse à retourner par l'API
   */
  public function updateData($id): Response {
    $sData = $this->requestToJson();
    $aData = $this->requestToArray();

    // Récupère les infos de l'utilisateur depuis user_defaultmap
    $oDataAccessor = $this->getDataAccessor();
    $oEntity = $oDataAccessor->getEntity($id, $aData);

    // Si status = 0 alors c'est que l'utilisateur n'est pas référencé sur la table user_defaultmap
    if ($oEntity['status'] === 0) {
      // Crée un enregistrement avec simplement le user_id
      $this->insertUser($id);
    }

    // Requête update
    $oEntity = $this->updateDataInDb($id, $sData);
    if ($oEntity['status'] == 0) {
      return $this->formatGenericResponseForErrorOnUpdate();
    }

    return $this->formatResponse($oEntity);
  }

  /**
   * Créé une entité avec simplement le user_id
   * c'est ce qui va permettre de faire l'update par la suite
   * @param mixed $iUserId
   * @return Response réponse à retourner par l'API
   */
  private function insertUser(mixed $iUserId): Response {
    $aData = [
      'user_id' => $iUserId,
    ];

    $oEntity = $this->insertDataInDb(json_encode($aData));

    if ($oEntity == null) {
      return $this->formatGenericResponseForErrorOnInsert();
    }

    if ($oEntity['status'] == 0) {
      return $this->formatGenericResponseForErrorOnInsert();
    }
    return $this->formatResponse($oEntity);
  }

}
