<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\OneToMany;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour coordsys
 *
 * @author Ludivine Premont <ludivine.premont@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="coordsys")
 */
class Coordsys extends AbstractEntity {
  /**
   * @var int
   * @Column(type="integer", name="coordsys_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).coordsys_coordsys_id_seq")
   */
  private $coordsys_id;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $label;

  /**
   * @var ArrayCollection<\App\Module\Vmap\WebService\Entity\LayerPostgres>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\LayerPostgres", mappedBy="coordsys")
   */
  private $aLayersPostgres;

  /**
   * @var ArrayCollection<\App\Module\Vmap\WebService\Entity\LayerFile>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\LayerFile", mappedBy="coordsys")
   */
  private $aLayersFiles;

  /**
   * @var ArrayCollection<\App\Module\Vmap\WebService\Entity\PublicationPostgres>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\PublicationPostgres", mappedBy="coordsys")
   */
  private $aPublicationsPostgres;

  /**
   * @var ArrayCollection<\App\Module\Vmap\WebService\Entity\PublicationFile>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\PublicationFile", mappedBy="coordsys")
   */
  private $aPublicationsFiles;

  public function __construct() {
    $this->aLayersPostgres = new ArrayCollection();
    $this->aLayersFiles = new ArrayCollection();
    $this->aPublicationsPostgres = new ArrayCollection();
    $this->aPublicationsFiles = new ArrayCollection();
  }

  /**
   * @return int identifiant du theme
   */
  public function getCoordsysId(): int {
    return $this->coordsys_id;
  }

  /**
   * @return string|null label du theme
   */
  public function getLabel(): ?string {
    return $this->label;
  }

  /**
   * @param string $label label du theme
   * @return self
   */
  public function setLabel(?string $label): self {
    $this->label = $label;
    return $this;
  }

  /**
   * retourne la liste des couches postgres
   * @return array
   */
  public function getLayersPostgres(): array{
    return $this->aLayersPostgres->toArray();
  }

  /**
   * retourne la liste des couches files
   * @return array
   */
  public function getLayersFiles(): array{
    return $this->aLayersFiles->toArray();
  }

  /**
   * retourne la liste des couches postgres
   * @return array
   */
  public function getPublicationsPostgres(): array{
    return $this->aPublicationsPostgres->toArray();
  }

  /**
   * retourne la liste des couches files
   * @return array
   */
  public function getPublicationsFiles(): array{
    return $this->aPublicationsFiles->toArray();
  }
}
