<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use App\Module\Vitis\WebService\Repository\GenericRepository;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\OneToMany;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;
use Doctrine\Persistence\Mapping\ClassMetadata;
use Doctrine\Persistence\ObjectManager;

/**
 * Objet ORM pour flux
 *
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="flux")
 */
class Flux extends AbstractEntity {
  /**
   * @var EntityManager|null
   */
  private $oEntityManager;
  /**
   * @var LoggerManager|null
   */
  private $oLoggerManager;

  /**
   * @var int
   * @Column(type="integer", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).flux_flux_id_seq")
   */
  private $flux_id;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $name;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $definition_json;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $definition_mapserver;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $public;

  /**
   * @var int|null
   * @Column(type="integer")
   */
  private $fluxdebuglevel_id;

  /**
   * @var FluxDebugLevel|null
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\FluxDebugLevel", inversedBy="flux")
   * @JoinColumn(name="fluxdebuglevel_id", referencedColumnName="fluxdebuglevel_id")
   */
  protected $fluxdebuglevel;

  /**
   * @var int|null
   * @Column(type="integer")
   */
  private $coordsys_id;

  /**
   * @var Coordsys|null
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\Coordsys", inversedBy="flux")
   * @JoinColumn(name="coordsys_id", referencedColumnName="coordsys_id")
   */
  protected $coordsys;

  /**
   * @var int|null
   * @Column(type="integer")
   */
  private $fluxtype_id;

  /**
   * @var FluxType|null
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\FluxType", inversedBy="flux")
   * @JoinColumn(name="fluxtype_id", referencedColumnName="fluxtype_id")
   */
  protected $fluxtype;

  /**
   * @var ArrayCollection<\App\Module\Vmap\WebService\Entity\PublicationFlux>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\PublicationFlux", mappedBy="flux")
   */
  protected $publications;

  public function __construct() {
    $this->publications = new ArrayCollection();
  }

  /**
   * @return int identifiant de la publication
   */
  public function getFluxId() {
    return $this->flux_id;
  }

  /**
   * @return string|null nom du flux
   */
  public function getName(): ?string {
    return $this->name;
  }

  /**
   * @param string $name nom du flux
   * @return self
   */
  public function setName(?string $name): self {
    $this->name = $name;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getDefinitionJson(): ?string {
    return $this->definition_json;
  }

  /**
   * @param string|null $definition_json
   * @return self
   */
  public function setDefinitionJson(?string $definition_json): self {
    $this->definition_json = $definition_json;
    return $this;
  }

  /**
   * @return string|null
   */
  public function getDefinitionMapserver(): ?string {
    return $this->definition_mapserver;
  }

  /**
   * @param string|null $definition_mapserver
   * @return self
   */
  public function setDefinitionMapserver(?string $definition_mapserver): self {
    $this->definition_mapserver = $definition_mapserver;
    return $this;
  }

  /**
   * @return bool|null
   */
  public function getPublic(): ?bool {
    return $this->public;
  }

  /**
   * @param bool|null $public
   * @return self
   */
  public function setPublic(?bool $public): self {
    $this->public = $public;
    return $this;
  }

  /**
   * @return int|null fluxdebuglevel_id
   */
  public function getFluxDebugLevelId(): ?int {
    return $this->fluxdebuglevel_id;
  }

  /**
   * @param int $fluxdebuglevel_id
   * @return self
   */
  public function setFluxDebugLevelId(?int $fluxdebuglevel_id): self {
    $this->fluxdebuglevel_id = $fluxdebuglevel_id;

    if (!isset($this->oEntityManager) || !isset($this->fluxdebuglevel_id)) {
      return $this;
    }
    try {
      $oFluxDebugLevelRepository = $this->oEntityManager->getRepository(FluxDebugLevel::class);

      if (!$oFluxDebugLevelRepository instanceof GenericRepository) {
        throw new \Exception("Repository is not GenericRepository", 1);
      }

      $oResult = $oFluxDebugLevelRepository->findById($this->fluxdebuglevel_id);

      if (!empty($oResult)) {
        $this->fluxdebuglevel = $oResult[0];
      }
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot find fluxdebuglevel id ' . $this->fluxdebuglevel_id . 'thown an error', [$this, $e]);
    } finally {
      return $this;
    }
  }

  /**
   * @return array|null fluxdebuglevel
   */
  public function getFluxDebugLevel(): ?array{
    if (!isset($this->fluxdebuglevel)) {
      return null;
    }
    // return context
    return [
      'fluxdebuglevel_id' => $this->fluxdebuglevel->getFluxDebugLevelId(),
      'label' => $this->fluxdebuglevel->getLabel(),
    ];
  }

  /**
   * @return int|null coordsys_id
   */
  public function getCoordsysId(): ?int {
    return $this->coordsys_id;
  }

  /**
   * @param int $coordsys_id
   * @return self
   */
  public function setCoordsysId(?int $coordsys_id): self {
    $this->coordsys_id = $coordsys_id;

    if (!isset($this->oEntityManager) || !isset($this->coordsys_id)) {
      return $this;
    }
    try {
      $oCoordSysRepository = $this->oEntityManager->getRepository(Coordsys::class);

      if (!$oCoordSysRepository instanceof GenericRepository) {
        throw new \Exception("Repository is not GenericRepository", 1);
      }

      $oResult = $oCoordSysRepository->findById($this->coordsys_id);

      if (!empty($oResult)) {
        $this->coordsys = $oResult[0];
      }
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot find coordsys id ' . $this->coordsys_id . 'thown an error', [$this, $e]);
    } finally {
      return $this;
    }
  }

  /**
   * @return array|null theme
   */
  public function getCoordsys(): ?array{
    if (!isset($this->coordsys)) {
      return null;
    }
    // return context
    return [
      'coordsys_id' => $this->coordsys->getCoordsysId(),
      'label' => $this->coordsys->getLabel(),
    ];
  }

  /**
   * @return int|null fluxtype_id
   */
  public function getFluxTypeId(): ?int {
    return $this->fluxtype_id;
  }

  /**
   * @param int $fluxtype_id
   * @return self
   */
  public function setFluxTypeId(?int $fluxtype_id): self {
    $this->fluxtype_id = $fluxtype_id;

    if (!isset($this->oEntityManager) || !isset($this->fluxtype_id)) {
      return $this;
    }
    try {
      $oFluxTypeRepository = $this->oEntityManager->getRepository(FluxType::class);

      if (!$oFluxTypeRepository instanceof GenericRepository) {
        throw new \Exception("Repository is not GenericRepository", 1);
      }

      $oResult = $oFluxTypeRepository->findById($this->fluxtype_id);

      if (!empty($oResult)) {
        $this->fluxtype = $oResult[0];
      }
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot find fluxtype id ' . $this->fluxtype_id . 'thown an error', [$this, $e]);
    } finally {
      return $this;
    }
  }

  /**
   * @return array|null theme
   */
  public function getFluxType(): ?array{
    if (!isset($this->fluxtype)) {
      return null;
    }
    // return context
    return [
      'fluxtype_id' => $this->fluxtype->getFluxTypeId(),
      'label' => $this->fluxtype->getLabel(),
    ];
  }

  /**
   * @return string|null fluxtype_name
   */
  public function getFluxTypeName(): ?string {
    if (!isset($this->fluxtype)) {
      return null;
    }
    // return context
    return $this->fluxtype->getLabel();
  }

  /**
   * Retourne la liste des identifiants de publication
   *
   * @return array
   */
  public function getPublicationsIdList() : array{
    $aReturn = [];
    foreach ($this->publications->toArray() as $oFluxPublication) {
      $aReturn[] = $oFluxPublication->getPublicationId();
    }

    return array_unique($aReturn);
  }

  /**
   * @return array liste des publications
   */
  public function getPublications(): array{
    $aReturn = [];
    foreach ($this->publications->toArray() as $oFluxPublication) {
      $oPublication = $this->oEntityManager->find(AbstractPublication::class, $oFluxPublication->getPublicationId());
      $aReturn[] = [
        'flux_layer_id' => $oFluxPublication->getFluxLayerId(),
        'publication_id' => $oFluxPublication->getPublicationId(),
        'type' => $oPublication->getPublicationtype(),
        'name' => $oPublication->getName(),
        'title' => $oPublication->getTitle(),
        'datasource_id' => $oPublication->getDatasourceId(),
        'datemaj' => $oPublication->getDatemaj(),
        'extent' => $oPublication->getExtent(),
        'min_layer_scale' => $oPublication->getMinLayerScale(),
        'max_layer_scale' => $oPublication->getMaxLayerScale(),
        'active' => $oPublication->getActive(),
      ];
    }
    return $aReturn;
  }

  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet la récupération d'un gestionnaire d'entité pour interagir avec la base
   * @return void
   */
  public function injectObjectManager(ObjectManager $objectManager, ClassMetadata $classMetadata): void {
    if ($objectManager instanceof EntityManager) {
      $this->oEntityManager = $objectManager;
    }
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void{
    $this->oLoggerManager = $oService;
  }
}
