<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\OneToMany;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour formsize
 *
 * @author Ludivine Premont <ludivine.premont@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="formsize")
 */
class FormSize extends AbstractEntity {
  /**
   * @var int
   * @Column(type="integer", name="formsize_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).formsize_formsize_id_seq")
   */
  private $formsize_id;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $label;

  /**
   * @var ArrayCollection<\App\Module\Vmap\WebService\Entity\LayerPostgres>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\LayerPostgres", mappedBy="form_display_size")
   */
  private $aDisplayLayersPostgres;

  /**
   * @var ArrayCollection<\App\Module\Vmap\WebService\Entity\LayerPostgres>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\LayerPostgres", mappedBy="form_insert_size")
   */
  private $aInsertLayersPostgres;

  /**
   * @var ArrayCollection<\App\Module\Vmap\WebService\Entity\LayerPostgres>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\LayerPostgres", mappedBy="form_update_size")
   */
  private $aUpdateLayersPostgres;

  public function __construct() {
    $this->aDisplayLayersPostgres = new ArrayCollection();
    $this->aInsertLayersPostgres = new ArrayCollection();
    $this->aUpdateLayersPostgres = new ArrayCollection();
  }

  /**
   * @return int identifiant du theme
   */
  public function getFormSizeId(): int {
    return $this->formsize_id;
  }

  /**
   * @return string|null label du theme
   */
  public function getLabel(): ?string {
    return $this->label;
  }

  /**
   * @param string $label label du theme
   * @return self
   */
  public function setLabel(?string $label): self {
    $this->label = $label;
    return $this;
  }

  /**
   * retourne la liste des couches postgres
   * @return array
   */
  public function getLayersPostgres(): array{
    return array_unique(
      array_merge(
        $this->aDisplayLayersPostgres->toArray(),
        $this->aInsertLayersPostgres->toArray(),
        $this->aUpdateLayersPostgres->toArray()
      )
    );
  }
}
