<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vmap\WebService\Entity\Coordsys;
use App\Module\Vmap\WebService\Entity\Fileformat;
use App\Module\Vmap\WebService\Service\MapServerService;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour layerfile
 *
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="layerfile")
 */
class LayerFile extends AbstractLayer {
  /**
   * @var string
   */
  public const TYPE = 'file';

  /**
   * on utilise pas celui du parent pour spécifier à doctrine que cette clé existe
   * dans les deux tables, les getters du parent seront applicable
   * @var int
   * @Id
   * @Column(type="integer", name="layer_id", nullable=false)
   */
  protected $layer_id;

  /**
   * @var string
   * @Column(type="string", nullable=true)
   */
  private $definition_sym_json;

  /**
   * @var string
   * @Column(type="string")
   */
  private $file_path;

  /**
   * @var Coordsys|null
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\Coordsys", inversedBy="layers_files")
   * @JoinColumn(name="coordsys_id", referencedColumnName="coordsys_id")
   */
  private $coordsys;

  /**
   * @var int
   * @column(type="integer")
   */
  private $coordsys_id;

  /**
   * @var Fileformat|null
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\Fileformat", inversedBy="layers_files")
   * @JoinColumn(name="fileformat_id", referencedColumnName="fileformat_id")
   */
  private $fileformat;

  /**
   * @var int
   * @column(type="integer")
   */
  private $fileformat_id;

  /**
   * @return string|null chemin du fichier
   */
  public function getFilePath(): ?string {
    return $this->file_path;
  }

  /**
   * @param string $file_path chemin du fichier
   * @return self
   */
  public function setFilePath(?string $file_path): self {
    $this->file_path = $file_path;
    return $this;
  }

  /**
   * @return string|null Définition de la symbologie de la couche file au format JSON
   */
  public function getDefinitionSymJson(): ?string {
    return $this->definition_sym_json;
  }

  /**
   * @param string $definition_sym_json Définition de la symbologie de la couche file au format JSON
   * @return self
   */
  public function setDefinitionSymJson(?string $definition_sym_json): self {
    $this->definition_sym_json = $definition_sym_json;
    return $this;
  }

  /**
   * @return array|null Système de coordonnées de la couche de type fichier
   */
  public function getCoordsys(): ?array{
    if (!isset($this->coordsys)) {
      return null;
    }
    // return context
    return [
      'coordsys_id' => $this->coordsys->getCoordsysId(),
      'label' => $this->coordsys->getLabel(),
    ];
  }

  /**
   * @param int $coordsys_id Système de coordonnées de la couche de type fichier
   * @return self
   */
  public function setCoordsysId(int $coordsys_id): self {
    $this->coordsys_id = $coordsys_id;
    if (isset($this->oEntityManager)) {
      $coordsys = $this->oEntityManager->getReference(Coordsys::class, $coordsys_id);
      $this->setCoordsys($coordsys);
    }
    return $this;
  }

  /**
   * @return int
   */
  public function getCoordsysId(): ?int {
    return $this->coordsys_id;
  }

  /**
   * @param Coordsys $coordsys Système de coordonnées de la couche de type fichier
   * @return self
   */
  public function setCoordsys(Coordsys $coordsys): self {
    $this->coordsys = $coordsys;
    return $this;
  }

  /**
   * @return array|null Format du fichier
   */
  public function getFileformat(): ?array{
    if (!isset($this->fileformat)) {
      return null;
    }
    // return context
    return [
      'fileformat_id' => $this->fileformat->getFileformatId(),
      'label' => $this->fileformat->getLabel(),
    ];
  }

  /**
   * @param int $fileformat_id Format du fichier
   * @return self
   */
  public function setFileformatId(int $fileformat_id): self {
    $this->fileformat_id = $fileformat_id;
    if (isset($this->oEntityManager)) {
      $fileformat = $this->oEntityManager->getReference(Fileformat::class, $fileformat_id);
      $this->setFileformat($fileformat);
    }
    return $this;
  }

  /**
   * @return int
   */
  public function getFileformatId(): ?int {
    return $this->fileformat_id;
  }

  /**
   * @param Fileformat $fileformat Format du fichier
   * @return self
   */
  public function setFileformat(Fileformat $fileformat): self {
    $this->fileformat = $fileformat;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function getLayerParams(array $aLayer): array{
    $aParamsLayer = [
      'layerType' => 'imagewms',
      'url' => $aLayer['url'],
      'params' => [
        'LAYERS' => MapServerService::cleanLayerName(array_key_exists('uniquename', $aLayer) ? $aLayer['uniquename'] : $aLayer['name']),
        'VERSION' => '1.3.0',
      ],
      'file_path' => $aLayer['file_path'],
      'coordsys' => $aLayer['coordsys'],
      'coordsys_id' => $aLayer['coordsys_id'],
      'fileformat' => $aLayer['fileformat'],
      'fileformat_id' => $aLayer['fileformat_id'],
      'definition_sym_json' => $aLayer['definition_sym_json'],
      'pictogram' => $aLayer['pictogram']
    ];
    return array_merge(parent::getLayerParams($aLayer), $aParamsLayer);
  }

  public function jsonSerialize(?array $aRouteConfiguration = null): array
  {
    $aData = parent::jsonSerialize($aRouteConfiguration);
    $aData["file_path"] = $this->getFilePath();
    $aData["coordsys"] = $this->getCoordsys();
    $aData["coordsys_id"] = $this->getCoordsysId();
    $aData["fileformat"] = $this->getFileformat();
    $aData["fileformat_id"] = $this->getFileformatId();
    $aData["definition_sym_json"] = $this->getDefinitionSymJson();
    $aData["pictogram"] = $this->getPictogram();
    return $aData;
  }

  public function getDeserializeMap(): array
  {
    $aData = parent::getDeserializeMap();
    $aData["file_path"] = "setFilePath";
    $aData["coordsys"] = "setCoordsys";
    $aData["coordsys_id"] = "setCoordsysId";
    $aData["fileformat"] = "setFileformat";
    $aData["fileformat_id"] = "setFileformatId";
    $aData["definition_sym_json"] = "setDefinitionSymJson";
    $aData["pictogram"] = "setPictogram";
    return $aData;
  }
}
