<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use App\Module\Vmap\WebService\Entity\Coordsys;
use App\Module\Vmap\WebService\Entity\Fileformat;
use Doctrine\Common\Persistence\ObjectManagerAware;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM et fonction de récupération des données pour les traitements FME liés aux couches
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="layer_workspace")
 */
class LayerFmeWorkbench extends AbstractEntity {

  /**
   * @var int
   * @Column(type="integer", name="layer_workspace_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).layer_workspace_layer_workspace_id_seq")
   */
  protected $layer_workspace_id;

  /**
   * @var int
   * @Column(type="integer", name="layer_id", nullable=false)
   */
  protected $layer_id;

  /**
   * @var int
   * @Column(type="integer", name="workspace_id", nullable=false)
   */
  private $workspace_id;

  /**
   * @var int|null
   * @Column(type="integer", name="layer_index", nullable=true)
   */
  private $layerIndex;

  /**
   * @var bool|null
   * @Column(type="boolean", name="allow_display_card", nullable=true)
   */
  private $allow_display_card;

  /**
   * @var bool|null
   * @Column(type="boolean", name="allow_requestor", nullable=true)
   */
  private $allow_requestor;

  /**
   * @var bool|null
   * @Column(type="boolean", name="allow_select_all_data", nullable=true)
   */
  private $allow_select_all_data;

  /**
   * @var bool|null
   * @Column(type="boolean", name="allow_select_current_filter", nullable=true)
   */
  private $allow_select_current_filter;

  /**
   * @var int|null
   * @Column(type="integer", name="selection_max_entities", nullable=true)
   */
  private $selection_max_entities;

  /**
   * @var int|null
   * @Column(type="integer", name="user_id", nullable=false)
   */
  private $userId;

  function __construct(){}

   /**
   * @return int|null identifiant de la table layer_workspace
   */
  public function getLayerWorkspaceId(): ?int {
    return $this->layer_workspace_id;
  }

  /**
   * @return self identifiant de la table layer_workspace
   */
  public function setLayerWorkspaceId(int $layerWorkspaceId): self {
    $this->layer_workspace_id = $layerWorkspaceId;
    return $this;
  }


  /**
   * @return int|null identifiant de la couche associée au traitement
   */
  public function getLayerId(): ?int {
    return $this->layer_id;
  }

  /**
   * @param int $layer_id : idenfifiant de la couche associée au traitement
   * @return self
   */
  public function setLayerId(int $layer_id): self {
    $this->layer_id = $layer_id;
    return $this;
  }

  /**
   * @return int|null identifiant du traitement associé à la couche
   *
   */
  public function getWorkspaceId(): ?int {
    return $this->workspace_id;
  }

  /**
   * @param int $workspace_id : idenfifiant du traitement associé à la couche
   * @return self
   */
  public function setWorkspaceId(int $workspace_id): self {
    $this->workspace_id = $workspace_id;
    return $this;
  }

  /**
   * @return bool|null allow_display_card
   *
   */
  public function getAllowDisplayCard(): ?int {
    return $this->allow_display_card;
  }

  /**
   * @param bool $allow_display_card
   * @return self
   */
  public function setAllowDisplayCard($allow_display_card): self {
    $this->allow_display_card = $allow_display_card;
    return $this;
  }

  /**
   * @return bool|null allow_requestor
   *
   */
  public function getAllowRequestor(): ?int {
    return $this->allow_requestor;
  }

  /**
   * @param bool $allow_requestor
   * @return self
   */
  public function setAllowRequestor($allow_requestor): self {
    $this->allow_requestor = $allow_requestor;
    return $this;
  }

  /**
   * @return bool|null allow_select_all_data
   *
   */
  public function getAllowSelectAllData(): ?int {
    return $this->allow_select_all_data;
  }

  /**
   * @param bool $allow_select_all_data
   * @return self
   */
  public function setAllowSelectAllData(int $allow_select_all_data): self {
    $this->allow_select_all_data = $allow_select_all_data;
    return $this;
  }

  /**
   * @return bool|null allow_select_current_filter
   *
   */
  public function getAllowSelectCurrentFilter(): ?int {
    return $this->allow_select_current_filter;
  }

  /**
   * @param bool $allow_select_current_filter
   * @return self
   */
  public function setAllowSelectCurrentFilter(int $allow_select_current_filter): self {
    $this->allow_select_current_filter = $allow_select_current_filter;
    return $this;
  }

  /**
   * @return int|null selection_max_entities
   *
   */
  public function getSelectionMaxEntities(): ?int {
    return $this->selection_max_entities;
  }

  /**
   * @param int $selection_max_entities
   * @return self
   */
  public function setSelectionMaxEntities(int $selection_max_entities): self {
    $this->selection_max_entities = $selection_max_entities;
    return $this;
  }

  /**
   * @return int|null index du traitement associé à la couche
   *
   */
  public function getLayerIndex(): ?int {
    return $this->layerIndex;
  }

  /**
   * @param int $layerIndex : index du traitement associé à la couche
   * @return self
   */
  public function setLayerIndex(int $layerIndex): self {
    $this->layerIndex = $layerIndex;
    return $this;
  }

  /**
   * @return int|null identifiant de l'utilisateur Vitis à qui appartient le traitement lié
   */
  public function getUserId(): ?int {
    return $this->userId;
  }

  /**
   * @param int $userId : identifiant de l'utilisateur Vitis auquel la couche sera rattaché
   * @return self 
   */
  public function setUserId(int $userId): self {
    $this->userId = $userId;
    return $this;
  }

};