<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use App\Module\Vitis\WebService\Repository\GenericRepository;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;
use Doctrine\Persistence\Mapping\ClassMetadata;
use Doctrine\Persistence\ObjectManager;

/**
 * Objet ORM pour layer_map
 *
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="layer_map")
 */
class LayerMap extends AbstractEntity {
  /**
   * @var EntityManager|null
   */
  protected $oEntityManager;

  /**
   * @var LoggerManager|null
   */
  protected $oLoggerManager;

  /**
   * @var int
   * @Column(type="integer", name="layer_map_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).layer_map_layer_map_id_seq")
   */
  private $layer_map_id;

  /**
   * @var int|null
   * @Column(type="integer", name="layer_id", nullable=false)
   */
  private $layer_id;

  /**
   * @var int|null
   * @Column(type="integer", name="map_id", nullable=false)
   */
  private $map_id;

  /**
   * @var AbstractLayer|null
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\AbstractLayer", inversedBy="aMaps")
   * @JoinColumn(name="layer_id", referencedColumnName="layer_id")
   */
  private $layer_name;

  /**
   * @var Map|null
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\Map", cascade={"persist"}, inversedBy="layers")
   * @JoinColumn(name="map_id", referencedColumnName="map_id")
   */
  protected $map;

  /**
   * @var array
   * @Column(type="json", name="definition_sym_json")
   */
  private $definition_sym_json;

  /**
   * @var string|null
   * @Column(type="text", name="definition_sym_mapserver", nullable=true)
   */
  private $definition_sym_mapserver;

  /**
   * @var int|null
   * @Column(type="integer", name="layer_index", nullable=true)
   */
  private $layer_index;

  /**
   * @var bool|null
   * @Column(type="boolean", name="is_visible", nullable=true)
   */
  private $is_visible;

  /**
   * @var bool|null
   * @Column(type="boolean", name="is_location", nullable=true)
   */
  private $is_location;

  /**
   * @var bool|null
   * @Column(type="boolean", name="is_map_selection", nullable=true)
   */
  private $is_map_selection;

  /**
   * @var bool|null
   * @Column(type="boolean", name="is_data_grid", nullable=true)
   */
  private $is_data_grid;

  /**
   * @var bool|null
   * @Column(type="boolean", name="is_map_hidden", nullable=true)
   */
  private $is_map_hidden;

  /**
   * @var int|null
   * @Column(type="integer", name="map_parent_hidden", nullable=true)
   */
  private $map_parent_hidden;

  /**
   * @var bool|null
   * @Column(type="boolean", name="is_filter", nullable=true)
   */
  private $is_filter;
  
  /**
   * @return int identifiant du layer_map
   */
  public function getLayerMapId(): ?int {
    return $this->layer_map_id;
  }

  /**
   * @return int identifiant du layer_id
   */
  public function getLayerId(): ?int {
    return $this->layer_id;
  }

  /**
   * @param int|null $layer_id
   * @return self
   */
  public function setLayerId(?int $layer_id): self {
    $this->layer_id = $layer_id;

    if (!isset($this->oEntityManager) || !isset($this->layer_id)) {
      return $this;
    }
    try {
      $oLayerRepository = $this->oEntityManager->getRepository(AbstractLayer::class);

      if (!$oLayerRepository instanceof GenericRepository) {
        throw new \Exception("Repository is not GenericRepository", 1);
      }

      $oResult = $oLayerRepository->findById($this->layer_id);

      if (!empty($oResult)) {
        $this->layer_name = $oResult[0];
      }
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot find layer id ' . $this->layer_id . 'thown an error', [$this, $e]);
    } finally {
      return $this;
    }
  }

  /**
   * @return string|null
   */
  public function getLayer(): ?string {
    if (!isset($this->layer_name)) {
      return null;
    }
    // return context
    return $this->layer_name->getName();
  }

  /**
   * @param AbstractLayer $layer_name
   * @return self
   */
  public function setLayer(AbstractLayer $layer_name): self {
    $this->layer_name = $layer_name;
    return $this;
  }

  /**
   * @return Map map
   */
  public function getMap(): ?Map {
    return $this->map;
  }

  /**
   * @return int identifiant du map_id
   */
  public function getMapId(): ?int {
    return $this->map_id;
  }

  /**
   * @param int $map_id
   * @return self
   */
  public function setMapId(?int $map_id): self {
    $this->map_id = $map_id;

    if (!isset($this->oEntityManager) || !isset($this->map_id)) {
      return $this;
    }
    try {
      $oMapRepository = $this->oEntityManager->getRepository(Map::class);

      if (!$oMapRepository instanceof GenericRepository) {
        throw new \Exception("Repository is not GenericRepository", 1);
      }

      $oResult = $oMapRepository->findById($this->map_id);

      if (!empty($oResult)) {
        $this->map = $oResult[0];
      }
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot find map id ' . $this->map_id . 'thown an error', [$this, $e]);
    } finally {
      return $this;
    }
  }

  /**
   * @param Map|null $map
   * @return self
   */
  public function setMap(?Map $map): self {
    $this->map = $map;
    return $this;
  }

  /**
   * @return array|null definition_sym_json
   */
  public function getDefinitionSymJSON(): ?array{
    return $this->definition_sym_json;
  }

  /**
   * @param array|string|null $definition_sym_json
   * @return self
   */
  public function setDefinitionSymJSON($definition_sym_json): self {
    if (is_string($definition_sym_json)) {
      $definition_sym_json = json_decode($definition_sym_json, true);
    }
    $this->definition_sym_json = $definition_sym_json;
    return $this;
  }

  /**
   * @return string|null definition_sym_mapserver
   */
  public function getDefinitionSymMapserver(): ?string {
    return $this->definition_sym_mapserver;
  }

  /**
   * @param string $definition_sym_mapserver
   * @return self
   */
  public function setDefinitionSymMapserver(?string $definition_sym_mapserver): self {
    $this->definition_sym_mapserver = $definition_sym_mapserver;
    return $this;
  }

  /**
   * @return int|null layer_index
   */
  public function getLayerIndex(): ?int {
    return $this->layer_index;
  }

  /**
   * @param int $layer_index
   * @return self
   */
  public function setLayerIndex(?int $layer_index): self {
    $this->layer_index = $layer_index;
    return $this;
  }

  /**
   * @return boolean|null is_visible
   */
  public function getIsVisible(): ?bool {
    return $this->is_visible;
  }

  /**
   * @param boolean $is_visible
   * @return self
   */
  public function setIsVisible(?bool $is_visible): self {
    $this->is_visible = $is_visible;
    return $this;
  }

  /**
   * @return boolean|null is_location
   */
  public function getIsLocation(): ?bool {
    return $this->is_location;
  }

  /**
   * @param boolean $is_location
   * @return self
   */
  public function setIsLocation(?bool $is_location): self {
    $this->is_location = $is_location;
    return $this;
  }

  /**
   * @return boolean|null is_map_selection
   */
  public function getIsMapSelection(): ?bool {
    return $this->is_map_selection;
  }

  /**
   * @param boolean $is_map_selection
   * @return self
   */
  public function setIsMapSelection(?bool $is_map_selection): self {
    $this->is_map_selection = $is_map_selection;
    return $this;
  }

  /**
   * @return boolean|null is_data_grid
   */
  public function getIsDataGrid(): ?bool {
    return $this->is_data_grid;
  }

  /**
   * @param boolean $is_data_grid
   * @return self
   */
  public function setIsDataGrid(?bool $is_data_grid): self {
    $this->is_data_grid = $is_data_grid;
    return $this;
  }

  /**
   * @return boolean|null is_map_hidden
   */
  public function getIsMapHidden(): ?bool {
    return $this->is_map_hidden;
  }

  /**
   * @param boolean $is_map_hidden
   * @return self
   */
  public function setIsMapHidden(?bool $is_map_hidden): self {
    $this->is_map_hidden = $is_map_hidden;
    return $this;
  }

  /**
   * @return int|null map_parent_hidden
   */
  public function getMapParentHidden(): ?int {
    return $this->map_parent_hidden;
  }

  /**
   * @param int $map_parent_hidden
   * @return self
   */
  public function setMapParentHidden(?int $map_parent_hidden): self {
    $this->map_parent_hidden = $map_parent_hidden;
    return $this;
  }

  /**
   * @return boolean|null is_filter
   */
  public function getIsFilter(): ?bool {
    return $this->is_filter;
  }

  /**
   * @param boolean $is_filter
   * @return self
   */
  public function setIsFilter(?bool $is_filter): self {
    $this->is_filter = $is_filter;
    return $this;
  }


  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet la récupération d'un gestionnaire d'entité pour interagir avec la base
   * @return void
   */
  public function injectObjectManager(ObjectManager $objectManager, ClassMetadata $classMetadata): void {
    if ($objectManager instanceof EntityManager) {
      $this->oEntityManager = $objectManager;
    }
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void {
    $this->oLoggerManager = $oService;
  }
}
