<?php

namespace App\Module\Vmap\WebService\Entity;

use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour layervectortiles
 *
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="layervectortiles")
 */
class LayerVectorTiles extends AbstractLayer {
  /**
   * @var string
   */
  public const TYPE = 'vectortiles';
  /**
   * @var int
   * @Id
   * @Column(type="integer", name="layer_id", nullable=false)
   */
  protected $layer_id;

  /**
   * @var array|null
   * @Column(type="json", nullable=true)

   */
  private $mapbox_gl_style;

  /**
   * @return string|null
   */
  public function getLayerMapboxGlStyle(): ?string {
    if (isset($this->mapbox_gl_style)) {
      return json_encode($this->mapbox_gl_style);
    }
    return null;
  }

  /**
   * @param string $mapbox_gl_style
   * @return self
   */
  public function setLayerMapboxGlStyle(?string $mapbox_gl_style): self {
    $this->mapbox_gl_style = json_decode($mapbox_gl_style, true);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function getLayerParams(array $aLayer): array{
    $aParamsLayer = [
      'layerType' => self::TYPE,
      'mapbox_gl_style' => $aLayer['mapbox_gl_style'],
      'headers' => $aLayer['datasource']['headers'],
      'pictogram' => $aLayer['pictogram']
    ];
    return array_merge(parent::getLayerParams($aLayer), $aParamsLayer);
  }

  /**
   * {@inheritdoc}
   */
  public function jsonSerialize(?array $aRouteConfiguration = null): array{
    $aData = parent::jsonSerialize($aRouteConfiguration);
    $aData['mapbox_gl_style'] = $this->getLayerMapboxGlStyle();
    $aData["pictogram"] = $this->getPictogram();
    return $aData;
  }

  /**
   * {@inheritdoc}
   */
  public function getDeserializeMap(): array{
    $aData = parent::getDeserializeMap();
    $aData['mapbox_gl_style'] = "setLayerMapboxGlStyle";
    $aData["pictogram"] = "setPictogram";
    return $aData;
  }
}
