<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use App\Module\Vitis\WebService\Repository\GenericRepository;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\JoinTable;
use Doctrine\ORM\Mapping\ManyToMany;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\OneToMany;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;
use Doctrine\Persistence\Mapping\ClassMetadata;
use Doctrine\Persistence\ObjectManager;

/**
 * Objet ORM pour map
 *
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="map")
 */
class Map extends AbstractEntity {
  /**
   * @var EntityManager|null
   */
  private $oEntityManager;
  /**
   * @var LoggerManager|null
   */
  private $oLoggerManager;

  /**
   * @var int
   * @Column(type="integer", name="map_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).map_map_id_seq")
   */
  private $map_id;

  /**
   * @var string
   * @Column(type="string", name="name")
   */
  private $name;

  /**
   * @var string
   * @Column(type="string", name="description")
   */
  private $description;

  /**
   * @var string
   * @Column(type="string", name="extent")
   */
  private $extent;

  /**
   * @var int
   * @Column(type="integer", name="catalog_index")
   */
  private $catalog_index;

  /**
   * @var string
   * @Column(type="string", name="thumbnail")
   */
  private $thumbnail;

  /**
   * @var int|null
   * @Column(type="integer", name="theme_id")
   */
  private $theme_id;

  /**
   * @var ThemeMap|null
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\ThemeMap")
   * @JoinColumn(name="theme_id", referencedColumnName="thememap_id")
   */
  protected $theme;

  /**
   * @var string
   * @Column(type="string", name="maptools")
   */
  private $maptools;

  /**
   * @var int|null
   * @Column(type="integer", name="coordsys_id")
   */
  private $coordsys_id;

  /**
   * @var Coordsys|null
   * @ManyToOne(targetEntity="App\Module\Vmap\WebService\Entity\Coordsys", inversedBy="maps")
   * @JoinColumn(name="coordsys_id", referencedColumnName="coordsys_id")
   */
  protected $coordsys;

  /**
   * @var ArrayCollection<\App\Module\Vmap\WebService\Entity\LayerMap>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\LayerMap", mappedBy="map")
   */
  protected $layers;

  /**
   * @var ArrayCollection<\App\Module\Vitis\WebService\Entity\Group>
   * @ManyToMany(targetEntity="App\Module\Vitis\WebService\Entity\Group")
   * @JoinTable(
   *   name="@prop(schema_vmap).map_group",
   *   joinColumns={@JoinColumn(name="map_id", referencedColumnName="map_id")},
   *   inverseJoinColumns={@JoinColumn(name="group_id", referencedColumnName="group_id")}
   * )
   */
  protected $aGroups;

  /**
   * @var string
   * @Column(type="string", name="default_layer_location_id")
   */
  private $default_layer_location_id;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  protected $comment_activation;

  /**
   * @var string
   * @Column(type="string", name="comment")
   */
  protected $comment;

  /**
   * @var string
   * @Column(type="string", name="comment_last_modification", nullable=true)
   */
  protected $comment_last_modification;

  public function __construct() {
    $this->layers = new ArrayCollection();
    $this->aGroups = new ArrayCollection();
  }

  /**
   * @return int identifiant du map_id
   */
  public function getMapId(): int {
    return $this->map_id;
  }

  /**
   * @return string|null name
   */
  public function getName(): ?string {
    return $this->name;
  }

  /**
   * @param string $name
   * @return self
   */
  public function setName(?string $name): self {
    $this->name = $name;
    return $this;
  }

  /**
   * @return string|null description
   */
  public function getDescription(): ?string {
    return $this->description;
  }

  /**
   * @param string $description
   * @return self
   */
  public function setDescription(?string $description): self {
    $this->description = $description;
    return $this;
  }

  /**
   * @return string|null extent
   */
  public function getExtent(): ?string {
    return $this->extent;
  }

  /**
   * @param string $extent
   * @return self
   */
  public function setExtent(?string $extent): self {
    $this->extent = $extent;
    return $this;
  }

  /**
   * @return int|null catalog_index
   */
  public function getCatalogIndex(): ?int {
    return $this->catalog_index;
  }

  /**
   * @param int $catalog_index
   * @return self
   */
  public function setCatalogIndex(?int $catalog_index): self {
    $this->catalog_index = $catalog_index;
    return $this;
  }

  /**
   * @return string|null thumbnail
   */
  public function getThumbnail(): ?string {
    return $this->thumbnail;
  }

  /**
   * @param string $thumbnail
   * @return self
   */
  public function setThumbnail(?string $thumbnail): self {
    $this->thumbnail = $thumbnail;
    return $this;
  }

  /**
   * @return int|null theme_id
   */
  public function getThemeId(): ?int {
    return $this->theme_id;
  }

  /**
   * @param int $theme_id
   * @return self
   */
  public function setThemeId(?int $theme_id): self {
    $this->theme_id = $theme_id;

    if (!isset($this->oEntityManager) || !isset($this->theme_id)) {
      return $this;
    }
    try {
      $oThemeMapRepository = $this->oEntityManager->getRepository(ThemeMap::class);

      if (!$oThemeMapRepository instanceof GenericRepository) {
        throw new \Exception("Repository is not GenericRepository", 1);
      }

      $oResult = $oThemeMapRepository->findById($this->theme_id);

      if (!empty($oResult)) {
        $this->theme = $oResult[0];
      }
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot find theme id ' . $this->theme_id . 'thown an error', [$this, $e]);
    } finally {
      return $this;
    }
  }

  /**
   * @return array|null theme
   */
  public function getTheme(): ?array{
    if (!isset($this->theme)) {
      return null;
    }
    // return context
    return [
      'theme_id' => $this->theme->getThemeId(),
      'label' => $this->theme->getLabel(),
    ];
  }

  /**
   * @return string|null theme_name
   */
  public function getThemeName(): ?string {
    if (!isset($this->theme)) {
      return null;
    }
    // return context
    return $this->theme->getLabel();
  }

  /**
   * @return string|null maptools
   */
  public function getMapTools(): ?string {
    return $this->maptools;
  }

  /**
   * @param string $maptools
   * @return self
   */
  public function setMapTools(?string $maptools): self {
    $this->maptools = $maptools;
    return $this;
  }

  /**
   * @return int|null coordsys_id
   */
  public function getCoordsysId(): ?int {
    return $this->coordsys_id;
  }

  /**
   * @param int $coordsys_id
   * @return self
   */
  public function setCoordsysId(?int $coordsys_id): self {
    $this->coordsys_id = $coordsys_id;

    if (!isset($this->oEntityManager) || !isset($this->coordsys_id)) {
      return $this;
    }
    try {
      $oCoordSysRepository = $this->oEntityManager->getRepository(Coordsys::class);

      if (!$oCoordSysRepository instanceof GenericRepository) {
        throw new \Exception("Repository is not GenericRepository", 1);
      }

      $oResult = $oCoordSysRepository->findById($this->coordsys_id);

      if (!empty($oResult)) {
        $this->coordsys = $oResult[0];
      }
    } catch (\Exception $e) {
      $this->oLoggerManager->get('SQL_ERROR')->error('Cannot find coordsys id ' . $this->coordsys_id . 'thown an error', [$this, $e]);
    } finally {
      return $this;
    }
  }

  /**
   * @return array|null theme
   */
  public function getCoordsys(): ?array{
    if (!isset($this->coordsys)) {
      return null;
    }
    // return context
    return [
      'coordsys_id' => $this->coordsys->getCoordsysId(),
      'label' => $this->coordsys->getLabel(),
    ];
  }

  /**
   * @return array liste des layers
   */
  public function getLayers(): array{
    $aReturn = [];
    foreach ($this->layers->toArray() as $oMapLayer) {
      $oLayer = $this->oEntityManager->find(AbstractLayer::class, $oMapLayer->getLayerId());
      $aReturn[] = [
        'layer_id' => $oMapLayer->getLayerId(),
        'definition_sym_json_map' => $oMapLayer->getDefinitionSymJSON(),
        'definition_sym_mapserver_map' => $oMapLayer->getDefinitionSymMapserver(),
        'layer_index' => $oMapLayer->getLayerIndex(),
        'is_visible' => $oMapLayer->getIsVisible(),
        'is_location' => $oMapLayer->getIsLocation(),
        'is_map_selection' => $oMapLayer->getIsMapSelection(),
        'is_data_grid' => $oMapLayer->getIsDataGrid(),
        'is_filter'=>$oMapLayer->getIsFilter(),
        'is_map_hidden' => $oMapLayer->getIsMapHidden(),
        'map_parent_hidden' => $oMapLayer->getMapParentHidden(),
        'pictogram' => $oLayer->getPictogram(),
      ];
    }
    return $aReturn;
  }

  /**
   * @return array liste des layers
   */
  public function getLayersWithEntity(): array{
    $aReturn = [];
    foreach ($this->layers->toArray() as $oMapLayer) {
      $oLayer = $this->oEntityManager->find(AbstractLayer::class, $oMapLayer->getLayerId());
      $aReturn[] = [
        'layer' => $oLayer,
        'layer_id' => $oMapLayer->getLayerId(),
        'definition_sym_json_map' => $oMapLayer->getDefinitionSymJSON(),
        'definition_sym_mapserver_map' => $oMapLayer->getDefinitionSymMapserver(),
        'layer_index' => $oMapLayer->getLayerIndex(),
        'is_visible' => $oMapLayer->getIsVisible(),
        'is_location' => $oMapLayer->getIsLocation(),
        'is_map_selection' => $oMapLayer->getIsMapSelection(),
        'is_data_grid' => $oMapLayer->getIsDataGrid(),
        'is_filter'=>$oMapLayer->getIsFilter(),
        'is_map_hidden' => $oMapLayer->getIsMapHidden(),
        'map_parent_hidden' => $oMapLayer->getMapParentHidden(),
      ];
    }
    return $aReturn;
  }

  /**
   * @param bool $bAddLayerValues défini s'il faut tenir compte des valeur de la couche
   * @return array liste des layers
   */
  public function getLayersSource(bool $bAddLayerValues = false): array{
    $aReturn = [];
    foreach ($this->layers->toArray() as $oMapLayer) {
      $oLayer = $this->oEntityManager->find(AbstractLayer::class, $oMapLayer->getLayerId());
      $aLayerSource = [
        'layer_map_id' => $oMapLayer->getLayerMapId(),
        'layer_id' => $oMapLayer->getLayerId(),
        'definition_sym_json_map' => $oMapLayer->getDefinitionSymJSON(),
        'definition_sym_mapserver_map' => $oMapLayer->getDefinitionSymMapserver(),
        'layer_index' => $oMapLayer->getLayerIndex(),
        'is_visible' => $oMapLayer->getIsVisible(),
        'is_location' => $oMapLayer->getIsLocation(),
        'is_map_selection' => $oMapLayer->getIsMapSelection(),
        'is_data_grid' => $oMapLayer->getIsDataGrid(),
        'is_filter'=>$oMapLayer->getIsFilter(),
        'is_map_hidden' => $oMapLayer->getIsMapHidden(),
        'map_parent_hidden' => $oMapLayer->getMapParentHidden(),
        'name' => $oLayer->getName(),
        'layertype' => $oLayer->getLayertype(),
        'theme' => $oLayer->getTheme(),
        'pictogram' => $oLayer->getPictogram(),
      ];

      if ($bAddLayerValues && $oLayer instanceof LayerPostgres) {
        $aLayerSource['is_map_selection_source'] = $oLayer->getIsMapSelection() ?? false;
        $aLayerSource['is_data_grid_source'] = $oLayer->getIsDataGrid() ?? false;
        $aLayerSource['is_location_source'] = $oLayer->getIsLocation() ?? false;
        $aLayerSource['is_filter_source'] = $oLayer->getIsFilter() ?? false;
      }

      $aReturn[] = $aLayerSource;
    }
    return $aReturn;
  }

  /**
   * @return int[] liste des user_id
   */
  public function getGroups(): array{
    $aGroupId = [];
    foreach ($this->aGroups as $oGroup) {
      $aGroupId[] = $oGroup->getGroupId();
    }
    return $aGroupId;
  }

  /**
   * @return string|null thumbnail
   */
  public function getDefaultLayerLocationId(): ?string {
    return $this->default_layer_location_id;
  }

  /**
   * @param string $default_layer_location_id
   * @return self
   */
  public function setDefaultLayerLocationId(?string $default_layer_location_id): self {
    $this->default_layer_location_id = $default_layer_location_id;
    return $this;
  }

  /**
   * @return bool|null Activation du commentaire de la carte
   */
  public function getCommentActivation(): ?bool {
    return $this->comment_activation;
  }

  /**
   * @param bool $comment_activation Activation du commentaire de la carte
   * @return self
   */
  public function setCommentActivation($comment_activation): self {
    $this->comment_activation = $comment_activation;
    return $this;
  }

  /**
   * @return string|null comment
   */
  public function getComment(): ?string {
    return $this->comment;
  }

  /**
   * @param string $comment
   * @return self
   */
  public function setComment(?string $comment): self {
    $this->comment = $comment;
    return $this;
  }

  /**
   * @return string|null 
   */
  public function getCommentLastModification(): ?string {
    return $this->comment_last_modification;
  }

  /**
   * @param string $comment_last_modification
   * @return self
   */
  public function setCommentLastModification(string $comment_last_modification): self {
    $this->comment_last_modification = $comment_last_modification;
    return $this;
  }

  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet la récupération d'un gestionnaire d'entité pour interagir avec la base
   * @return void
   */
  public function injectObjectManager(ObjectManager $objectManager, ClassMetadata $classMetadata): void {
    if ($objectManager instanceof EntityManager) {
      $this->oEntityManager = $objectManager;
    }
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void {
    $this->oLoggerManager = $oService;
  }
}
