<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\JoinTable;
use Doctrine\ORM\Mapping\ManyToMany;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour print style
 *
 * @author Mickaël Fernandez <mickael.fernandez@veremes.com>
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="printstyle")
 */
class PrintStyle extends AbstractEntity {
  /**
   * @var int
   * @Column(type="integer", name="printstyle_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).printstyle_printstyle_id_seq")
   */
  private $printstyle_id;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $name;

  /**
   * @var string
   * @Column(type="string", nullable=false)
   */
  private $definition;

  /**
   * @var ArrayCollection<\App\Module\Vitis\WebService\Entity\User>
   * @ManyToMany(targetEntity="App\Module\Vitis\WebService\Entity\User")
   * @JoinTable(
   *   name="@prop(schema_vmap).printstyle_user",
   *   joinColumns={@JoinColumn(name="printstyle_id", referencedColumnName="printstyle_id")},
   *   inverseJoinColumns={@JoinColumn(name="user_id", referencedColumnName="user_id")}
   * )
   */
  private $aUsers;

  /**
   * @return int
   */
  public function getPrintStyleId(): int {
    return $this->printstyle_id;
  }

  /**
   * @return string
   */
  public function getName(): string {
    return $this->name;
  }

  /**
   * @param string $name
   * @return self
   */
  public function setName(?string $name): self {
    $this->name = $name;
    return $this;
  }

  /**
   * @return string
   */
  public function getDefinition(): string {
    return $this->definition;
  }

  /**
   * @param string $definition
   * @return self
   */
  public function setDefinition(?string $definition): self {
    $this->definition = $definition;
    return $this;
  }

  /**
   * @return int[] liste des user_id
   */
  public function getUsers(): array{
    $aUsersId = [];

    if (!$this->aUsers->isEmpty()) {
      foreach ($this->aUsers as $oUser) {
        $aUsersId[] = $oUser->getUserId();
      }
    }

    return $aUsersId;
  }
}
