<?php

namespace App\Module\Vmap\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\OneToMany;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;

/**
 * Objet ORM pour thememap
 *
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vmap)",name="thememap")
 */
class ThemeMap extends AbstractEntity {
  /**
   * @var int
   * @Column(type="integer", name="thememap_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vmap).thememap_thememap_id_seq")
   */
  private $thememap_id;

  /**
   * @var string
   * @Column(type="string", length=100, nullable=true)
   */
  private $label;

  /**
   * @var ArrayCollection<\App\Module\Vmap\WebService\Entity\AbstractLayer>
   * @OneToMany(targetEntity="App\Module\Vmap\WebService\Entity\AbstractLayer", mappedBy="theme")
   */
  private $aLayers;

  public function __construct() {
    $this->aLayers = new ArrayCollection();
  }

  /**
   * @return int identifiant du theme
   */
  public function getThemeId(): int {
    return $this->thememap_id;
  }

  /**
   * @param int $thememap_id
   * @return self
   */
  public function setThemeId(?int $thememap_id): self {
    $this->thememap_id = $thememap_id;
    return $this;
  }

  /**
   * @return string|null label du theme
   */
  public function getLabel(): ?string {
    return $this->label;
  }

  /**
   * @param string $label label du theme
   * @return self
   */
  public function setLabel(?string $label): self {
    $this->label = $label;
    return $this;
  }

  /**
   * retourne la liste des couches
   * @return array
   */
  public function getLayers(): array{
    return $this->aLayers->toArray();
  }
}
