#!/usr/bin/env python3
# coding: utf-8

import json
from os import path
import os


class Command:
  def __init__(self, **kwargs):
    self.settings = kwargs.get('settings')
    self.logging = kwargs.get('logging')

  def execute(self):
    print('Script de preinstall')
    # print(self.settings)
    self.updateVault()


  def updateVault(self):
    '''
      Dans le cadre de la version 2022.08.00 la politique sur le stockage des clés de chiffrement a évolué
      L'ancienne clé JWK est donc déplacé dans la nouvelle structure
    '''
    vault_path = path.join(self.settings['INSTALL_PATH'], 'vas/.vitis_db')
    jwk_path = path.join(self.settings['INSTALL_PATH'], 'vas/.vitis')

    if path.exists(jwk_path) and not path.exists(vault_path) :
      self.logging.info(
          "Conversion de votre ancien fichier de clé de chiffrement")
      data = None
      with open(jwk_path, 'r', encoding='utf8') as jwk_file:
        data = jwk_file.read()

      if data is not None :
        sKey = self.settings.defineKeyForVault('jwk_key')
        sEncryptedJwk = self.settings.encryptDataForVault('jwk_key', data)

        vault_data = {sKey: sEncryptedJwk}
        sVault = json.dumps(vault_data, sort_keys=True, indent=2)
        self.logging.info(
            "Écrit le nouveau fichier contenant les clés de chiffrement sur le disque : \"%s\"" % vault_path)
        with open(vault_path, 'w', encoding='utf-8') as key_file:
          key_file.write(sVault)

        if path.exists(vault_path) : 
          self.logging.info(
              "Suppression de l'ancien fichier de clé de chiffrement : \"%s\"" % jwk_path)
          os.remove(jwk_path)
          if path.exists(jwk_path) :
            self.logging.info("Suppression réussie")
          else :
            self.logging.info("Suppression échouée")



