import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';

import { HttpClientModule, HttpClient } from '@angular/common/http';
import { TranslateModule, TranslateLoader } from '@ngx-translate/core';
import { MultiTranslateHttpLoader } from 'ngx-translate-multi-http-loader';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { ColorPickerModule } from 'ngx-color-picker';
import { RecaptchaModule, RecaptchaFormsModule } from 'ng-recaptcha';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { MarkdownModule } from 'ngx-markdown';

import { AppRoutingModule } from './app-routing';
import { AppComponent } from './app.component';
import { MonacoEditorModule, NgxMonacoEditorConfig } from 'ngx-monaco-editor';
import { environment } from 'src/environments/environment';
import { SharedModule } from './shared/shared.module';
import { FormreaderModule } from './formreader/formreader.module';
import { DataGridModule } from './data-grid/data-grid.module';
import { CanDeactivateGuard } from './core/services/deactivate/can-deactivate.guard';
import { CoreModule } from './core/core.module';

/**
 * Chargement des fichiers de traduction
 * @param http
 * @returns
 */
export function HttpLoaderFactory(http: HttpClient) {
  if (environment.appStruct['translate'] && environment.properties && environment.properties.build) {
    for (let i = 0; i < environment.appStruct['translate'].length; i++) {
      if (environment.appStruct['translate'][i].suffix &&
        environment.appStruct['translate'][i].suffix.indexOf('?') === -1) {
        environment.appStruct['translate'][i].suffix += `?${environment.properties.build}`;
      }
    }
  }
  return new MultiTranslateHttpLoader(http, environment.appStruct['translate']);
}

const monacoConfig: NgxMonacoEditorConfig = {
  defaultOptions: { scrollBeyondLastLine: false }
};

@NgModule({
  declarations: [
    AppComponent
  ],
  imports: [
    CommonModule,
    AppRoutingModule,
    HttpClientModule,
    FormsModule,
    ReactiveFormsModule,
    TranslateModule.forRoot({
      loader: {
        provide: TranslateLoader,
        useFactory: HttpLoaderFactory,
        deps: [HttpClient]
      }
    }),
    NgbModule,
    ColorPickerModule,
    RecaptchaModule,
    RecaptchaFormsModule,
    BrowserAnimationsModule,
    MarkdownModule.forRoot(),
    MonacoEditorModule.forRoot(monacoConfig),
    CoreModule,
    SharedModule,
    FormreaderModule,
    DataGridModule,
  ],
  providers: [CanDeactivateGuard],
  bootstrap: [AppComponent]
})
export class AppModule { }
