import { AppModeConfig } from './app.model';
import { WklBooleanIconComponent } from './workspacelist/template/wkl-boolean-icon/wkl-boolean-icon.component';
import { WklDateComponent } from './workspacelist/template/wkl-date/wkl-date.component';
import { WklStateTranslationComponent } from './workspacelist/template/wkl-state-translation/wkl-state-translation.component';
import { WklTooltipComponent } from './workspacelist/template/wkl-tooltip/wkl-tooltip.component';

/**
 * Objet utilisateurs du mode utilisateurs
 */
export const vitisUsersUsersObject: AppModeConfig = {
  name: 'vitis_users_users',
  label: 'TITLE_MODE_USERS',
  route: 'list',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  options: {
    module: 'vitis',
    ressource: 'vitis/users',
    id_field: 'user_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    form_path: 'assets/forms/users/users.json',
    js_path: 'assets/forms/users/users.js',
    columns: [{
      title: 'LIST_USERS_USERS_ID',
      field: 'user_id',
      align: 'right',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number',
    }, {
      title: 'LIST_USERS_USERS_USER',
      field: 'login',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_USERS_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_USERS_DOMAIN',
      field: 'domain',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'strict',
      searchWebService: {
        resource: 'vitis/domains',
        params: {
          attributs: 'domain_id|name',
          order_by: 'domain_id',
          sort_order: 'DESC'
        },
        label: 'name',
        value: 'domain_id',
      }
    }, {
      title: 'LIST_USERS_USERS_COMPANY',
      field: 'company',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_USERS_MAIL',
      field: 'email',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_USERS_DEPARTEMENT',
      field: 'department',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_USERS_LAST_CONNEXION',
      field: 'last_connection',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'date',
      template: WklDateComponent
    }, {
      title: 'LIST_USERS_USERS_IP',
      field: 'ip_constraint',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_USERS_ROLE',
      field: 'role',
      align: 'center',
      resizeable: true,
      sortable: false,
      template: WklBooleanIconComponent,
      template_options: {
        true_class: 'fa fa-check text-success',
        false_class: ' ',
      },
      searchable: false,
      searchType: 'boolean',
      searchSuggestions: [{
        value: true,
        label: 'FORM_YES'
      }, {
        value: false,
        label: 'FORM_NO'
      }]
    }]
  }
};

/**
 * Objet utilisateurs du mode utilisateurs
 */
export const vitisUsersRequestUsersObject: AppModeConfig = {
  name: 'vitis_users_users_requests',
  label: 'TITLE_MODE_USERS_REQUESTS_LIST',
  route: 'requests',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  visible: '@prop(sign_up) && @prop(advanced_login_page)',
  options: {
    module: 'vitis',
    ressource: 'vitis/user_requests',
    id_field: 'user_request_id',
    display: true,
    update: true,
    insert: false,
    delete: true,
    search: {
      custom: true,
      form: false,
      default: 'custom'
    },
    form_path: 'assets/forms/users_requests/users_requests.json',
    js_path: 'assets/forms/users_requests/users_requests.js',
    columns: [{
      title: 'LIST_USERS_REQUEST_USERS_USER_ID',
      field: 'user_request_id',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_REQUEST_USERS_USER',
      field: 'login',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_REQUEST_USERS_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_REQUEST_USERS_COMPANY',
      field: 'company',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_REQUEST_USERS_MAIL',
      field: 'email',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_REQUEST_USERS_DEPARTEMENT',
      field: 'department',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_REQUEST_USERS_STATUS',
      field: 'status',
      align: 'center',
      resizeable: true,
      sortable: true,
      template: WklStateTranslationComponent,
      template_options: {
        states: {
          1: 'LIST_USERS_REQUEST_USERS_STATUS_WIP',
          2: 'LIST_USERS_REQUEST_USERS_STATUS_OK',
          3: 'LIST_USERS_REQUEST_USERS_STATUS_KO',
        }
      },
      searchable: true,
      searchType: 'number',
      searchSuggestions: [{
        value: 1,
        label: 'LIST_USERS_REQUEST_USERS_STATUS_WIP'
      }, {
        value: 2,
        label: 'LIST_USERS_REQUEST_USERS_STATUS_OK'
      }, {
        value: 3,
        label: 'LIST_USERS_REQUEST_USERS_STATUS_KO'
      }],
    }, {
      title: 'LIST_USERS_REQUEST_USERS_DATE',
      field: 'request_date',
      align: 'center',
      template: WklDateComponent,
      resizeable: true,
      sortable: true,
    }]
  }
};

/**
 * Objet domaines du mode utilisateurs
 */
export const vitisUsersDomainsObject: AppModeConfig = {
  name: 'vitis_users_domains',
  label: 'TITLE_MODE_USERS_DOMAINS',
  route: 'domains',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  sections: [{
    name: 'vitis_users_domains_details',
    label: 'TITLE_SUB_MODE_VITIS_DOMAIN_INFO_GENERALES',
    route: 'details',
    type: 'simpleform',
    privilege: 'vitis_admin',
    options: {
      module: 'vitis',
      ressource: 'vitis/domains',
      id_field: 'domain_id',
      display: true,
      update: true,
      insert: true,
      form_path: 'assets/forms/domains/domains.json',
      js_path: 'assets/forms/domains/domains.js'
    }
  }, {
    name: 'vitis_users_domains_users',
    label: 'TITLE_SUB_MODE_VITIS_DOMAIN_USERS',
    route: 'users',
    type: 'domainlist',
    privilege: 'vitis_admin',
    options: {
      module: 'vitis',
      ressource: 'vitis/domains/:id/users',
      id_field: 'uid',
      display: false,
      update: false,
      insert: false,
      delete: false,
      search: {
        custom: true,
        form: false,
        default: 'custom'
      },
      form_path: 'assets/forms/domains/domains.json',
      js_path: 'assets/forms/domains/domains.js',
      columns: [{
        title: 'LIST_DOMAIN_IMPORT_USER_LOGIN',
        field: 'uid',
        align: 'left',
        resizeable: true,
        sortable: true,
      }, {
        title: 'LIST_DOMAIN_IMPORT_USER_NAME',
        field: 'displayname',
        align: 'left',
        resizeable: true,
        sortable: true,
      }, {
        title: 'LIST_DOMAIN_IMPORT_USER_DN',
        field: 'dn',
        align: 'left',
        resizeable: true,
        sortable: false,
        searchable: false,
      }, {
        title: 'LIST_DOMAIN_IMPORT_USER_MAIL',
        field: 'mail',
        align: 'left',
        resizeable: true,
        sortable: false,
        searchable: false,
      }]
    }
  }, {
    name: 'vitis_users_domains_groups',
    label: 'TITLE_SUB_MODE_VITIS_DOMAIN_GROUPS',
    route: 'groups',
    type: 'workspacelist',
    privilege: 'vitis_admin',
    options: {
      module: 'vitis',
      ressource: 'vitis/domains/:domain_id/groups',
      id_field: 'displayname',
      parent_id_field: 'domain_id',
      disable_filter_parent: true,
      display: false,
      update: false,
      insert: false,
      delete: false,
      search: {
        custom: true,
        form: false,
        default: 'custom'
      },
      js_path: 'assets/forms/domains/domains.js',
      buttons: [{
        type: 'button',
        name: 'domain_groups_actions_group_import',
        label: 'BTN_IMPORT_SELECTION_ACTIVE_DIRECTORY_USERS_VITIS_GROUP_IMPORT',
        icon: 'fa fa-plus',
        event: 'importGroups()'
      }, {
        type: 'group',
        label: 'BTN_IMPORT_SELECTION_ACTIVE_DIRECTORY_USERS_VITIS_GROUP_SYNCHRONIZATION',
        name: 'domain_groups_actions',
        class: 'btn-primary',
        buttons: [{
          type: 'button',
          name: 'domain_groups_actions_group_sync',
          label: 'BTN_IMPORT_SELECTION_ACTIVE_DIRECTORY_USERS_VITIS_GROUP_SYNC_ENABLE',
          icon: 'fa fa-check',
          event: 'syncEnableGroups()'
        }, {
          type: 'button',
          name: 'domain_groups_actions_group_sync',
          label: 'BTN_IMPORT_SELECTION_ACTIVE_DIRECTORY_USERS_VITIS_GROUP_SYNC_DISABLE',
          icon: 'fa fa-times',
          event: 'syncDisableGroups()'
        }, {
          type: 'button',
          name: 'domain_groups_actions_group_sync',
          label: 'BTN_IMPORT_SELECTION_ACTIVE_DIRECTORY_USERS_VITIS_GROUP_SYNC',
          icon: 'fa fa-refresh',
          event: 'syncGroups()'
        }]
      }],
      columns: [{
        title: 'LIST_DOMAIN_GROUP_NAME',
        field: 'cn',
        align: 'left',
        resizeable: true,
        sortable: true,
        searchable: true
      }, {
        title: 'LIST_DOMAIN_GROUP_DN',
        field: 'dn',
        align: 'left',
        resizeable: true,
        sortable: false,
        searchable: false
      }]
    }
  }],
  options: {
    module: 'vitis',
    ressource: 'vitis/domains',
    id_field: 'domain_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    form_path: 'assets/forms/domains/domains.json',
    js_path: 'assets/forms/domains/domains.js',
    columns: [{
      title: 'LIST_USERS_DOMAINS_ID',
      field: 'domain_id',
      align: 'right',
      resizeable: true,
      sortable: true,
      searchType: 'number'
    }, {
      title: 'LIST_USERS_DOMAINS_TYPE',
      field: 'type',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_DOMAINS_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_DOMAINS_SUFFIX',
      field: 'suffix',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_DOMAINS_SERVER',
      field: 'server',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_DOMAINS_PORT',
      field: 'port',
      align: 'right',
      resizeable: true,
      sortable: true,
      searchType: 'number'
    }]
  }
};

/**
 * Objet groupes du mode utilisateurs
 */
export const vitisUsersGroupsObject: AppModeConfig = {
  name: 'vitis_users_groups',
  label: 'TITLE_MODE_USERS_GROUPS',
  route: 'groups',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  options: {
    module: 'vitis',
    ressource: 'vitis/groups',
    id_field: 'group_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    form_path: 'assets/forms/groups/groups.json',
    js_path: 'assets/forms/groups/groups.js',
    columns: [{
      title: 'LIST_USERS_GROUPS_ID',
      field: 'group_id',
      align: 'right',
      resizeable: true,
      sortable: true,
      searchType: 'number'
    }, {
      title: 'LIST_USERS_GROUPS_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_GROUPS_NB_MEMBERS',
      field: 'nb_members',
      align: 'right',
      resizeable: true,
      sortable: true,
      searchable: false
    }, {
      title: 'LIST_USERS_GROUPS_DOMAIN',
      field: 'domain',
      align: 'left',
      resizeable: true,
      sortable: false,
      searchable: false
    }, {
      title: 'LIST_USERS_GROUPS_AUTO_REFRESH',
      field: 'auto_refresh',
      align: 'center',
      resizeable: true,
      sortable: false,
      searchable: false,
      searchType: 'boolean',
      searchSuggestions: [{
        value: true,
        label: 'FORM_YES'
      }, {
        value: false,
        label: 'FORM_NO'
      }],
      template: WklBooleanIconComponent,
      template_options: {
        true_class: 'fa fa-check text-success',
        false_class: ' ',
      },
    }]
  }
};

/**
 * Objet privilèges du mode utilisateurs
 */
export const vitisUsersPrivilegesObject: AppModeConfig = {
  name: 'vitis_users_privileges',
  label: 'TITLE_MODE_USERS_PRIVILEGES',
  route: 'privileges',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  options: {
    module: 'vitis',
    ressource: 'vitis/privileges',
    id_field: 'rolname',
    display: true,
    update: true,
    insert: true,
    delete: false,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'rolname'
    },
    form_path: 'assets/forms/privileges/privileges.json',
    js_path: 'assets/forms/privileges/privileges.js',
    buttons: [{
      type: 'button',
      label: 'PRIVILEGE_DELETE_BTN',
      icon: 'fa fa-trash',
      event: 'deletePrivilege'
    }],
    columns: [{
      title: 'LIST_USERS_PRIVILEGES_NAME',
      field: 'rolname',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_PRIVILEGES_DESCRIPTION',
      field: 'description',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_USERS_PRIVILEGES_LOCKED',
      field: 'locked',
      align: 'center',
      resizeable: true,
      sortable: true,
      template: WklBooleanIconComponent,
      template_options: {
        true_class: 'fa fa-check text-success',
        false_class: ' ',
      },
      searchable: true,
      searchType: 'boolean',
      searchSuggestions: [{
        value: true,
        label: 'FORM_YES'
      }, {
        value: false,
        label: 'FORM_NO'
      }]
    }]
  }
};

/**
 * Objet jetons de connexion du mode utilisateurs
 */
export const vitisUsersLoginTokenObject: AppModeConfig = {
  name: 'vitis_users_login_token',
  label: 'TITLE_MODE_USERS_LOGIN_TOKEN',
  route: 'login-tokens',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  options: {
    module: 'vitis',
    ressource: 'vitis/logintoken',
    id_field: 'token_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    form_path: 'assets/forms/login_token/login_token.json',
    order_by: [7, 'desc'],
    columns: [{
      title: 'USERS_LOGIN_TOKEN_ID',
      field: 'token_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchType: 'string'
    }, {
      title: 'USERS_LOGIN_TOKEN_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'USERS_LOGIN_TOKEN_USER',
      field: 'login',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'USERS_LOGIN_TOKEN_DATE_LIMIT',
      field: 'limit_date',
      align: 'center',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'date',
      template: WklDateComponent,
    }, {
      title: 'USERS_LOGIN_TOKEN_IP_RESTRICTION',
      field: 'ip_restriction',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'USERS_LOGIN_TOKEN_DATE_CREATION',
      field: 'creation_date',
      align: 'center',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'date',
      template: WklDateComponent,
    }, {
      title: 'USERS_LOGIN_TOKEN_COMMENT',
      field: 'comment',
      align: 'left',
      resizeable: true,
      sortable: true,
    }]
  }
};

/**
 * Objet type de restrictions
 */
export const vitisRestrictionRestrictionTypeObject: AppModeConfig = {
  name: 'vitis_users_restriction_type',
  label: 'TITLE_MODE_USERS_RESTRICTIONS_RESTRICTION_TYPE',
  route: 'restriction_type',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  options: {
    module: 'vitis',
    ressource: 'vitis/restriction_types',
    id_field: 'restriction_type_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    form_path: 'assets/forms/restriction/restriction_type.json',
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'restriction_type_name'
    },
    columns: [{
      title: 'RESTRICTION_RESTRICTION_TYPE_RESTRICTION_TYPE_ID',
      field: 'restriction_type_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'RESTRICTION_RESTRICTION_TYPE_RESTRICTION_NAME',
      field: 'restriction_type_name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'string'
    }, {
      title: 'RESTRICTION_RESTRICTION_TYPE_RESTRICTION_BDD',
      field: 'restriction_db',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'string'
    }, {
      title: 'RESTRICTION_RESTRICTION_TYPE_RESTRICTION_SCHEMA',
      field: 'restriction_schema',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'string'
    }, {
      title: 'RESTRICTION_RESTRICTION_TYPE_RESTRICTION_TABLE',
      field: 'restriction_table',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'string'
    }, {
      title: 'RESTRICTION_RESTRICTION_TYPE_COLUMN_NAME',
      field: 'restriction_column',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'string'
    }, {
      title: 'RESTRICTION_RESTRICTION_TYPE_LABEL',
      field: 'restriction_alias',
      align: 'left',
      resizeable: true,
      sortable: true,
    }]
  }
};

export const vitisRestrictionRestrictionObject: AppModeConfig = {
  name: 'vitis_users_restriction',
  label: 'TITLE_MODE_USERS_RESTRICTIONS_RESTRICTION',
  route: 'user_restrictions',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  options: {
    module: 'vitis',
    ressource: 'vitis/restrictions',
    id_field: 'restriction_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    form_path: 'assets/forms/restriction/restriction.json',
    js_path: 'assets/forms/restriction/restriction.js',
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'restriction_name'
    },
    columns: [{
      title: 'RESTRICTION_RESTRICTION_ID',
      field: 'restriction_id',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'number'
    }, {
      title: 'RESTRICTION_RESTRICTION_NAME',
      field: 'restriction_name',
      align: 'left',
      resizeable: true,
      sortable: true,
      searchable: true,
      searchType: 'string'
    }]
  }
};

/**
 * Mode restrictions
 */
export const vitisRestrictionsMode: AppModeConfig = {
  name: 'restrictions',
  label: 'TITLE_MODE_RESTRICTIONS',
  route: 'restrictions',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  visible: '@prop(enable_custom_restrictions)',
  objects: [vitisRestrictionRestrictionTypeObject, vitisRestrictionRestrictionObject]
};

/**
 * Mode utilisateurs
 */
export const vitisUsersMode: AppModeConfig = {
  name: 'vitis_users',
  label: 'TITLE_MODE_USERS',
  icon: 'icon-vitis-users',
  route: 'users',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  objects: [vitisUsersUsersObject, vitisUsersDomainsObject, vitisUsersGroupsObject, vitisUsersPrivilegesObject, vitisUsersLoginTokenObject, vitisUsersRequestUsersObject, vitisRestrictionsMode]
};

/**
 * Mode logs
 */
export const vitisLogsMode: AppModeConfig = {
  name: 'vitis_logs',
  label: 'TITLE_MODE_LOGS',
  icon: 'icon-vitis-logs',
  route: 'logs',
  type: 'logs',
  privilege: 'vitis_admin'
};

/**
 * Objet général du mode config
 */
export const vitisConfigVitisObject: AppModeConfig = {
  name: 'vitis_config_vitis',
  label: 'TITLE_MODE_CONFIGURATION_PROPERTIES_VITIS',
  route: 'vitis',
  type: 'simpleform',
  privilege: 'vitis_admin',
  options: {
    module: 'vitis',
    ressource: 'vitis/properties',
    id_field: 'module_name',
    form_path: 'assets/forms/configuration/configuration.json',
    js_path: 'assets/forms/configuration/configuration.js'
  }
};

/**
 * Objet répertoire partagé du mode config
 */
export const vitisConfigSharedDirObject: AppModeConfig = {
  name: 'vitis_config_supervision_shared_dir',
  label: 'VITIS_CONFIG_SUPERVISION_SHARED_DIR',
  icon: 'fa-folder',
  route: 'shared-dir',
  type: 'directory',
  privilege: 'vitis_shared',
  options: {
    module: 'vitis',
    ressource: 'vitis/exploitations/shareddirectory',
    parent_folder_name: '/shared',
    hide_first_folder: true,
    hover_download: true,
    hover_delete: true,
    subroute: {
      getTree: 'tree',
      createDirectory: 'directory',
      deleteItem: 'delete',
      sendFile: 'file'
    },
    buttons: [{
      type: 'group',
      buttons: [{
        type: 'button',
        name: 'load-tree',
        title: 'FORM_RELOAD_DIRECTORY',
        class: 'btn-primary',
        icon: 'fas fa-refresh',
        event: 'loadResourceData()'
      }, {
        type: 'button',
        name: 'download-resource',
        title: 'FORM_DOWNLOAD_FILE',
        class: 'btn-primary',
        icon: 'fas fa-download',
        event: 'downloadResource()'
      },
      {
        type: 'button',
        name: 'upload-file',
        title: 'FORM_BTN_UPLOAD_FILE',
        class: 'btn-primary',
        icon: 'fas fa-upload',
        event: 'uploadFile()',
        param: 'FORM_ADD_FILE_TITLE'
      }
      ]
    }, {
      type: 'button',
      name: 'create-folder',
      title: 'FORM_BTN_CREATE_DIRECTORY',
      class: 'btn-primary',
      icon: 'fas fa-folder',
      event: 'createFolder()'
    }, {
      type: 'button',
      name: 'delete-resource',
      title: 'FORM_DELETE_FILE',
      class: 'btn-danger',
      icon: 'fas fa-trash',
      event: 'deleteResource()'
    }]
  }
};

/**
 * Objet modeles e-mail du mode config
 */
export const vitisConfigEmailTemplatesObject: AppModeConfig = {
  name: 'vitis_publication_email_templates',
  label: 'TITLE_MODE_GTF_PUBLICATION_EMAIL_TEMPLATES',
  route: 'mail-templates',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  sections: [{
    name: 'vitis_publication_email_template_informations_generales',
    label: 'TITLE_SUB_MODE_GTF_PUBLICATION_EMAIL_TEMPLATES_INFO_GENERALES',
    route: 'general-informations',
    type: 'simpleform',
    privilege: 'vitis_admin',
    options: {
      module: 'vitis',
      ressource: 'vitis/emailtemplates',
      id_field: 'email_template_id',
      display: true,
      update: true,
      insert: true,
      form_path: 'assets/forms/emailtemplate/emailtemplate.json'
    }
  }, {
    name: 'gtf_publication_email_template_simple_def',
    label: 'TITLE_SUB_MODE_GTF_PUBLICATION_EMAIL_TEMPLATES_SIMPLE_DEF',
    route: 'definition',
    type: 'simpleform',
    privilege: 'vitis_admin',
    options: {
      module: 'vitis',
      ressource: 'vitis/emailtemplates',
      id_field: 'email_template_id',
      display: true,
      update: true,
      insert: false,
      form_path: 'assets/forms/emailtemplate/emailtemplate_simple_edition.json',
      js_path: 'assets/forms/emailtemplate/emailtemplate_simple_edition.js'
    }
  }],
  options: {
    module: 'vitis',
    ressource: 'vitis/emailtemplates',
    id_field: 'email_template_id',
    display: true,
    update: true,
    insert: true,
    delete: true,
    clone: true,
    search: {
      custom: true,
      form: false,
      default: 'custom',
      defaultColumn: 'name'
    },
    form_path: 'modules/gtf/forms/publication/emailtemplate/emailtemplate.json',
    js_path: 'modules/gtf/forms/publication/emailtemplate/emailtemplate.js',
    columns: [{
      title: 'LIST_PUBLICATION_EMAIL_TEMPLATES_EMAIL_TEMPLATE_ID',
      field: 'email_template_id',
      align: 'right',
      resizeable: true,
      sortable: true,
      searchType: 'number'
    }, {
      title: 'LIST_PUBLICATION_EMAIL_TEMPLATES_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_PUBLICATION_EMAIL_TEMPLATES_TO',
      field: 'to',
      align: 'center',
      resizeable: true,
      sortable: true,
      template: WklTooltipComponent,
      template_options: {
        icon: 'fa-envelope'
      }
    },
    {
      title: 'LIST_PUBLICATION_EMAIL_TEMPLATES_CC',
      field: 'cc',
      align: 'center',
      resizeable: true,
      sortable: true,
      template: WklTooltipComponent,
      template_options: {
        icon: 'fa-envelope'
      }
    }, {
      title: 'LIST_PUBLICATION_EMAIL_TEMPLATES_CCI',
      field: 'cci',
      align: 'center',
      resizeable: true,
      sortable: true,
      template: WklTooltipComponent,
      template_options: {
        icon: 'fa-envelope'
      }
    }
    ]
  }
};

/**
 * Objet interface du mode config
 */
export const vitisConfigInterfaceObject: AppModeConfig = {
  name: 'vitis_config_interface',
  label: 'TITLE_MODE_CONFIGURATION_PROPERTIES_VITIS_INTERFACE',
  route: 'interface',
  type: 'simpleform',
  privilege: 'vitis_admin',
  options: {
    module: 'vitis',
    ressource: 'vitis/properties',
    id_field: 'module_name',
    form_path: 'assets/forms/configuration/interface.json',
    js_path: 'assets/forms/configuration/interface.js'
  }
};

/**
 * Objet traductions personnalisées du mode config
 */
export const vitisConfigCustomTranslationsObject: AppModeConfig = {
  name: 'vitis_configuration_custom_translations',
  label: 'TITLE_MODE_CONFIGURATION_CUSTOM_TRANSLATIONS',
  description: 'TITLE_MODE_CONFIGURATION_CUSTOM_TRANSLATIONS_DESCRIPTION',
  route: 'custom-translations',
  type: 'customtranslation',
  privilege: 'vitis_admin',
  options: {
    module: 'vitis',
    ressource: 'vitis/customtranslations',
    id_field: 'translation_id',
    display: true,
    update: true,
    insert: false,
    delete: true,
    search: {
      custom: true,
      form: false,
      default: 'custom'
    },
    refresh: true,
    export: false,
    form_path: 'assets/forms/configuration/custom_translations.json',
    js_path: 'assets/forms/configuration/custom_translations.js',
    js_services: {
      'CustomTranslationService': import('src/app/vitis/customtranslation/services/customtranslation.service'),
    },
    columns: [{
      title: 'LIST_CONFIGURATION_CUSTOM_TRANSLATIONS_TRANSLATION_ID',
      field: 'translation_id',
      align: 'left',
      resizeable: true,
      sortable: true
    }, {
      title: 'LIST_CONFIGURATION_CUSTOM_TRANSLATIONS_ORIGINAL_TEXT',
      field: 'original_text',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_CONFIGURATION_CUSTOM_TRANSLATIONS_CUSTOM',
      field: 'text',
      align: 'left',
      resizeable: true,
      sortable: true,
    }, {
      title: 'LIST_CONFIGURATION_CUSTOM_TRANSLATIONS_CUSTOM_TRANSLATION',
      field: 'custom_translation',
      align: 'center',
      resizeable: true,
      sortable: false,
      template: WklBooleanIconComponent,
      template_options: {
        true_class: 'fa fa-check text-success',
        false_class: ' ',
      },
      searchable: true,
      searchType: 'boolean'
    }]
  }
};

/**
 * Service web vitis du mode API
 */
export const vitisAPIVitisWebService: any = {
  name: 'Vitis',
  api_definition_file_path: 'assets/api/swagger_vitis.json'
};


export const vitisUserHeaderInfoUpdate: any = {
  name: 'vitis_user_infos',
  label: 'TITLE_USER_INFOS',
  icon: 'icon-vitis-user',
  type: 'modalform',
  privilege: 'vitis_user',
  options: {
    module: 'vitis',
    ressource: 'vitis/users',
    id_field: 'user_id',
    form_path: 'assets/forms/user_infos/user_infos.json',
    js_path: 'assets/forms/user_infos/user_infos.js'
  }
};

export const vitisUserHeaderParams: any = {
  name: 'vitis_user_params',
  label: 'TITLE_USER_PARAMS',
  icon: 'icon-vitis-options',
  type: 'modalform',
  privilege: 'vitis_user',
  options: {
    module: 'vitis',
    ressource: 'vitis/users',
    id_field: 'user_id',
    form_path: 'assets/forms/user_params/user_params.json',
    js_path: 'assets/forms/user_params/user_params.js'
  }
};

export const vitisUserHeaderPassword: any = {
  name: 'vitis_user_password',
  label: 'TITLE_USER_PASSWORD',
  icon: 'icon-vitis-key',
  type: 'modalform',
  privilege: 'vitis_user',
  options: {
    module: 'vitis',
    ressource: 'vitis/users',
    id_field: 'user_id',
    form_path: 'assets/forms/user_password/user_password.json',
    js_path: 'assets/forms/user_password/user_password.js'
  }
};

export const vitisUserHeaderDisconnect: any = {
  name: 'vitis_user_disconect',
  label: 'TITLE_USER_DISCONECT',
  icon: 'icon-vitis-exit',
  type: 'function',
  privilege: 'vitis_user',
  options: {
    functionContainer: 'auth',
    function: 'disconnect()'
  }
};

/**
 * Header utilisateur
 */
export const vitisUserHeader: any = {
  name: 'vitis_user',
  title: 'USER_NAME',
  description: 'USER_COMPANY',
  icon: 'icon-vitis-user',
  type: 'dropdownMenu',
  privilege: 'vitis_user',
  objects: [
    vitisUserHeaderInfoUpdate,
    vitisUserHeaderParams,
    vitisUserHeaderPassword,
    vitisUserHeaderDisconnect
  ]
};

/**
 * Header help
 */
export const vitisHelpHeader: any = {
  name: 'vitis_help',
  title: 'TITLE_MODE_HELP',
  description: 'TITLE_MODE_HELP_DESCRIPTION',
  icon: 'icon-vitis-help',
  type: 'dropdownMenu',
  privilege: 'vitis_user',
  objects: [{
    name: 'vitis_help_documentation',
    label: 'TITLE_HELP_DOCUMENTATION',
    icon: 'icon-vitis-file-text',
    type: 'modalform',
    privilege: 'vitis_user',
    options: {
      size: 'large',
      form_path: 'assets/forms/help_documentation/help_documentation.json',
      js_path: 'assets/forms/help_documentation/help_documentation.js'
    }
  }, {
    name: 'vitis_help_credits',
    label: 'TITLE_HELP_CREDITS',
    icon: 'icon-vitis-star-full',
    type: 'modalform',
    privilege: 'vitis_user',
    options: {
      form_path: 'assets/forms/help_credits/help_credits.json',
      js_path: 'assets/forms/help_credits/help_credits.js'
    }
  }, {
    name: 'vitis_help_infos',
    label: 'TITLE_HELP_INFOS',
    icon: 'icon-vitis-info',
    type: 'modalform',
    privilege: 'vitis_user',
    options: {
      form_path: 'assets/forms/help_infos/help_infos.json',
      js_path: 'assets/forms/help_infos/help_infos.js'
    }
  }]
};

/**
 * Config translate vitis
 */
export const vitisTranslateConfig: any = {
  prefix: 'assets/lang/',
  suffix: '.json'
};


/**
 * Objet général du mode config
 */
export const vitisConfigList: AppModeConfig = {
  name: 'vitis_config_list',
  label: 'TITLE_MODE_CONFIGURATION_PROPERTIES_VITIS_LIST',
  route: 'list_settings',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  options: {
    module: 'vitis',
    ressource: 'vitis/admin_list_settings',
    id_field: 'list_settings_id',
    form_path: 'assets/forms/config_list/config_list.json',
    js_path: 'assets/forms/config_list/config_list.js',
    display: true,
    update: true,
    insert: false,
    delete: true,
    customColumns: true,
    search: {
      custom: true,
      form: false,
      default: 'custom'
    },
    columns: [{
      title: 'LIST_CONFIGURATION_LIST_SETTINGS_LIST_SETTINGS_ID',
      field: 'list_settings_id',
      align: 'left',
      resizeable: true,
      sortable: true
    },
    {
      title: 'LIST_CONFIGURATION_LIST_SETTINGS_NAME',
      field: 'name',
      align: 'left',
      resizeable: true,
      sortable: true
    },
    {
      title: 'LIST_CONFIGURATION_LIST_SETTINGS_USER_NAME',
      field: 'user_name',
      align: 'left',
      resizeable: true,
      sortable: true
    },
    {
      title: 'LIST_CONFIGURATION_LIST_SETTINGS_ROUTE',
      field: 'route',
      align: 'left',
      resizeable: true,
      sortable: true
    },
    {
      title: 'LIST_CONFIGURATION_LIST_SETTINGS_PUBLIC',
      field: 'ispublic',
      align: 'left',
      resizeable: true,
      searchType: 'boolean',
      searchSuggestions: [{
        value: true,
        label: 'FORM_YES'
      }, {
        value: false,
        label: 'FORM_NO'
      }],
      template: WklBooleanIconComponent
    },
    {
      title: 'LIST_CONFIGURATION_LIST_SETTINGS_DEFAULT',
      field: 'isdefault',
      align: 'left',
      resizeable: true,
      searchType: 'boolean',
      searchSuggestions: [{
        value: true,
        label: 'FORM_YES'
      }, {
        value: false,
        label: 'FORM_NO'
      }],
      template: WklBooleanIconComponent
    }]
  }
};

/**
 * Configuration des modules
 */
export const vitisAdditionalRouteObject: AppModeConfig = {
  privilege: ['vitis_user'],
  name: 'additional_route',
  type: 'additionalRoute',
  label: 'MODE_CONFIGURATION',
  visible: '@prop(enable_module_configuration)',
  route: 'additional-route',
  options: {}, // Obligatoire sinon ne fonctionne pas
  headless: false
};
