import { Component, OnInit } from '@angular/core';
import { environment } from 'src/environments/environment';
import { VitisCheck, VitisRequest, } from 'src/app/vitis/core/services/utils/vitis-utils';
import { TranslateService } from '@ngx-translate/core';

declare let $: any;
declare let monaco: any;

@Component({
  selector: 'app-supervision',
  templateUrl: './supervision.component.html',
  styleUrls: ['./supervision.component.scss']
})
export class ConfigurationSupervisionComponent implements OnInit {

  public oSupervisionData: any = {};
  public oInstallStateCssClass: any = {};

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private translate: TranslateService
  ) {}

  /**
   * Chargement du composant
   */
  ngOnInit() {
    this.loadSupervisionData();
  }

  /**
   * Charge les données de supervision.
   * @returns void
   */
  public loadSupervisionData () {
    this.vitisRequest.ajaxRequestPromise({
      method: 'GET',
      url: environment.properties.app_api_url + '/vitis/versions/'
    }).then((resp: any) => {
      if (resp.data) {
        if (resp.data.status === 1) {
          if (this.vitisCheck.isDefAndNotNull(resp.data) && this.vitisCheck.isObject(resp.data)) {
            // Données de supervision.
            this.oSupervisionData = resp.data;
            this.oSupervisionData.version = environment.properties.version;
            this.oSupervisionData.build = environment.properties.build;

            // Classes css pour les paramètres de l'état de l'installation.
            const aInstallStateKeys: any = ['php_version',
              'pdo_extension_loaded',
              'pdo_pgsql_extension_loaded',
              'curl_extension_loaded',
              'postgresql',
              'ldap_extension_loaded',
              'postgis'
            ];
            let sCssClass: String;
            for (let i = 0; i < aInstallStateKeys.length; i++) {
              if (this.vitisCheck.isDefAndNotNull(this.oSupervisionData[aInstallStateKeys[i]])) {
                if (this.oSupervisionData[aInstallStateKeys[i]] != '') {
                  sCssClass = 'fas fa-check-circle text-success';
                }
              } else {
                sCssClass = 'fas fa-exclamation-circle text-danger';
              }
              this.oInstallStateCssClass[aInstallStateKeys[i]] = sCssClass;
            }
          }
        }
      }
    });
  }
}
