import { Injectable } from '@angular/core';
import { fromEvent, Observable, Subject } from 'rxjs';
import { environment } from 'src/environments/environment';


@Injectable({
  providedIn: 'root'
})
export class IsSmallScreenService {

  public resizeScreenObservable: Observable<Event>;
  public isSmallScreenObservable = new Subject<boolean | null>();
  private isSmallScreen: boolean = null;
  private resizeTimeout = 2000;
  private resizeTimeoutPointer: any;

  /**
   * Constructeur
   */
  constructor() {
    this.setSmallScreen(window.innerWidth <= environment.properties.smallscreen_limit);
    this.resizeScreenObservable = fromEvent(window, 'resize');
    this.resizeScreenObservable.subscribe((event: any) => {
      clearTimeout(this.resizeTimeoutPointer);
      this.resizeTimeoutPointer = setTimeout(() => {
        if (event.target.innerWidth <= environment.properties.smallscreen_limit) {
          if (this.isSmallScreen == false) {
            this.setSmallScreen(true);
          }
        } else if (this.isSmallScreen == true) {
          this.setSmallScreen(false);
        }
      }, this.resizeTimeout);
    });
  }

  /**
   * Getter
   * @returns
   */
  public getIsSmallScreen() {
    return this.isSmallScreen;
  }

  /**
   * Setter quand on change l'état de la fenêtre
   * @param isSmallScreen
   */
  private setSmallScreen(isSmallScreen: boolean) {
    this.isSmallScreen = isSmallScreen;
    this.isSmallScreenObservable.next(isSmallScreen);
    environment.isSmallScreen = isSmallScreen;
  }
}
