import { Injectable } from '@angular/core';
import { CheckService } from '../check/check.service';
import { environment } from 'src/environments/environment';

class GetterSetterOptions {
  prefixByApp?: boolean;
  prefixByVersion?: boolean;
}

@Injectable({
  providedIn: 'root'
})
export class LocalStorageService {

  public container = localStorage;

  /**
   * Constructeur
   */
  constructor(
    public vitisCheck: CheckService
  ) { }

  /**
   * Retourne l'objet sous la clé fournie
   * @param sKey
   * @param bHandleVersion
   */
  public get(sKey: string, options: GetterSetterOptions = { prefixByApp: true, prefixByVersion: true }): any {

    // Pefixe
    const prefix = this.getPrefix_(options);

    // Récupère la valeur
    let value = this.container.getItem(prefix + sKey);

    // Gestion objets
    try {
      value = JSON.parse(value);
    } catch (error) {

    }

    return value;
  }

  /**
   * Set l'objet celons la clé fournie, écrase l'objet si il existe
   * @param sKey
   * @param value
   */
  public set(sKey: string, value: any, options: GetterSetterOptions = { prefixByApp: true, prefixByVersion: true }) {

    // Pefixe
    const prefix = this.getPrefix_(options);

    // Gestion des objets
    if (this.vitisCheck.isObject(value)) {
      try {
        value = JSON.stringify(value);
      } catch (error) {

      }
    }

    // Stocke la valeur
    this.container.setItem(prefix + sKey, value);
  }

  /**
   * Supprime un objet
   * @param sKey
   * @param options
   */
  public removeItem(sKey: string, options: GetterSetterOptions = { prefixByApp: true, prefixByVersion: true }) {
    // Pefixe
    const prefix = this.getPrefix_(options);

    // Stocke la valeur
    this.container.removeItem(prefix + sKey);
  }

  /**
   * Ajoute un attribut à l'objet
   * @param sKey
   * @param value
   * @param options
   */
  public add(sKey: string, value: any, options: GetterSetterOptions = { prefixByApp: true, prefixByVersion: true }) {

    const currentValue = this.get(sKey, options);

    // Merge des valeurs
    if (this.vitisCheck.isObject(value) &&
      this.vitisCheck.isObject(currentValue)) {
      value = { ...currentValue, ...value };
    }

    // Set la nouvelle valeur
    this.set(sKey, value, options);
  }

  /**
   * Supprime les valeurs
   * @param options
   */
  public clear(options: GetterSetterOptions = { prefixByApp: true, prefixByVersion: true }) {

    const prefix = this.getPrefix_(options);

    this.forEachKey_((key: string) => {
      if (key.startsWith(prefix)) {
        this.container.removeItem(key);
      }
    });
  }

  /**
   * Returne le prefixe
   * @param options
   */
  private getPrefix_(options: GetterSetterOptions = { prefixByApp: true, prefixByVersion: true }): string {
    let prefix = '';

    // prefixByApp
    if (options &&
      options.prefixByApp &&
      environment &&
      environment.properties &&
      environment.properties.application) {
      prefix += environment.properties.application + '_';
    }

    // prefixByVersion
    if (options &&
      options.prefixByVersion &&
      environment &&
      environment.properties &&
      environment.properties.version) {
      prefix += environment.properties.version.replace(/\./gm, '_') + '_';
    }

    return prefix;
  }

  /**
   * Parcourt l'ensemble des clés
   * @param callback
   */
  private forEachKey_(callback: any) {
    for (let i = this.container.length - 1; i >= 0; i--) {
      callback(this.container.key(i));
    }
  }
}
