import { Injectable } from '@angular/core';
import { AuthService } from '../../auth/auth.service';
import { VitisCheck } from '../vitis-utils';
import { PrivilegeArrayMode } from 'src/app/vitis/app.model';

@Injectable({
  providedIn: 'root'
})
export class RoutingService {

  /**
   *
   */
  constructor(
    private auth: AuthService,
    private vitisCheck: VitisCheck
  ) { }

  /**
   * Retourne true si routePrivilege est valide en fonction de aUserPrivileges
   * @param routePrivilege string ou tableau de string
   * @param routePrivilege string ou tableau de string
   * @returns boolean
   */
  public canAccessRoute(routePrivilege: any, privilegeArrayMode = PrivilegeArrayMode.OR): boolean {
    const aUserPrivileges = this.auth.getPrivileges();
    if (this.vitisCheck.isString(routePrivilege)) {
      if (aUserPrivileges.indexOf(routePrivilege) !== -1) {
        return true;
      }
    } else if (this.vitisCheck.isArray(routePrivilege)) {

      // cherche le match de privilège entre les deux tableaux
      const aMatch: Array<string> = aUserPrivileges.filter(item => routePrivilege.indexOf(item) > -1);
      const bHasRight: boolean = privilegeArrayMode === PrivilegeArrayMode.AND ? aMatch.length === routePrivilege.length : aMatch.length > 0;
      if (bHasRight) {
        return true;
      }
    } else {
      console.error('Mode privilege not string or array', routePrivilege);
    }
    return false;
  }
}
