import { Component, OnInit, Input, Output, EventEmitter } from '@angular/core';
import { WorkspaceListButton } from 'src/app/vitis/workspacelist/workspacelist.model';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { VitisCheck, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-btn-modal',
  templateUrl: './btn-modal.component.html',
  styleUrls: ['./btn-modal.component.scss']
})
export class BtnModalComponent implements OnInit {

  @Input() public button: WorkspaceListButton;
  @Input() public group?: boolean = false;
  @Input() public oDataGrid?: any;
  @Output() public modalClose: EventEmitter<any> = new EventEmitter<any>();

  /**
   * Constructeur
   */
  constructor(
    private modalService: NgbModal,
    private vitisCheck: VitisCheck,
    private translate: TranslateService,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
  }

  /**
   * Ouvre dans une modale le composant défini dans modal_component
   * @param oButton
   */
  public openBtnModal(oButton: WorkspaceListButton) {
    import('src/app/modules/modules-components').then((oModules) => {

      if (oButton &&
        oButton.modal_component &&
        oModules &&
        oModules.oModulesComponents &&
        oModules.oModulesComponents[oButton.modal_component]) {
        const component = oModules.oModulesComponents[oButton.modal_component];

        let canOpenComponent = true;
        // Si besoin, vérifie qu'au moins une ligne est sélectionné
        if (oButton.modal_need_selected_row === true) {
          const aSelectedLine = this.oDataGrid.getSelectedRows_();
          // Si aucun élément n'a été sélectionés affiche un message d'erreur
          if (aSelectedLine.length == 0) {
            let msgToTranslate = 'VITIS_WORKSPACELIST_NO_SELECT_ERROR';
            if (this.vitisCheck.isDefAndNotNull(oButton.modal_label_no_row_error) && oButton.modal_label_no_row_error !== '') {
              msgToTranslate = oButton.modal_label_no_row_error;
            }

            this.translate.get(msgToTranslate).subscribe((translation: string) => {
              vitisAlert.fireToast({
                icon: 'warning',
                title: translation
              });
            });
            canOpenComponent = false;
          }
        }

        if (canOpenComponent === true) {
          const modalRef = this.modalService.open(component, oButton.modal_options);
          if (this.vitisCheck.isDefAndNotNull(modalRef.componentInstance)) {
            modalRef.componentInstance.dataTableInstance = this.oDataGrid;
            modalRef.componentInstance.modalOptions = oButton.modal_options;
          }

          modalRef.result.then((res) => {
            this.modalClose.emit(true);
            if (oButton.refreshGridOnClose == true) {
              this.oDataGrid.reloadTable();
            }
          }).catch((err) => {});
        }
      } else {
        console.error('Component not found :', oButton.modal_component);
      }
    });
  }
}
