import { Component, OnInit, OnDestroy, ViewChild, Input } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { VitisCheck, VitisRequest, vitisAlert, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { AuthService } from '../core/services/auth/auth.service';
import { FormreaderComponent } from '../formreader/formreader.component';
import { environment } from 'src/environments/environment';
import { FormreaderService } from '../formreader/services/formreader.service';

@Component({
  selector: 'app-displayform',
  templateUrl: './displayform.component.html',
  styleUrls: ['./displayform.component.scss']
})
export class DisplayformComponent implements OnInit, OnDestroy {

  @ViewChild(FormreaderComponent, { static: false }) oFormReader: FormreaderComponent;
  public oMode: any;
  public oParams: any;
  public insertedId: any;
  public sFormName: string = 'insert'; // Affiche le formulaire insert mais sera mis en disabled par la suite
  public oFormDefinition: any;
  public oFormValues: any;

  /**
   * Permet de passer oInputMode en argument du template
   */
  @Input() set oInputMode(oMode: any) {
    if (oMode.options) {
      this.initForm();
    }
  }

  /**
   * Constructeur
   */
  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private vitisRequest: VitisRequest,
    private vitisCheck: VitisCheck,
    private vitisClone: VitisClone,
    private auth: AuthService,
    private translate: TranslateService,
    private formReaderService: FormreaderService,
  ) { }

  /**
   * Initialisation du composant
   */
  ngOnInit() {
    this.route.data.subscribe(oMode => {
      this.oMode = oMode;

      this.route.params.subscribe(oParams => {
        this.oParams = {
          'id': oParams.id,
          'form': oParams.form
        };
        // Initialise le formulaire
        this.initForm();
      });
    });
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy() {
    if ((<any>window).constructor_form) {
      (<any>window).constructor_form = () => { };
    }
  }

  /**
   * Initialise le formulaire
   */
  public initForm() {

    // Charge les données du formulaire
    this.loadFormValues_().then((oFormValues) => {
      this.oFormValues = oFormValues;

      // Charge le JS associé
      this.ajaxGetFormDef_().then((oFormDefinition) => {
        this.oFormDefinition = oFormDefinition;

        // Charge le formulaire
        this.loadJSONForm_();

        // Lance le JS associé
        if ((<any>window).constructor_form) {
          (<any>window).constructor_form(this.oFormReader, environment, {
            vitisRequest: this.vitisRequest,
            vitisCheck: this.vitisCheck,
            vitisAlert: vitisAlert,
            auth: this.auth,
            translate: this.translate
          });
        }
      });
    });
  }

  /**
   * Charge les valeurs du formulaire en fonction de l'identifiant passé dans l'URL
   * @returns Promise
   */
  private loadFormValues_(): Promise<boolean> {
    return new Promise((resolve, reject) => {
      let bValuesLoading = false;

      if (this.vitisCheck.isDefAndNotNull(this.oMode.options)) {
        if (this.vitisCheck.isDefAndNotNull(this.oMode.options.ressource_parent)) {

          const sRessourcePath = this.oMode.options.ressource_parent;

          if (this.vitisCheck.isString(sRessourcePath)) {

            const sUrl = this.router.url;
            if (this.vitisCheck.isString(sUrl)) {

              // Récupère l'id dans les paramètres de la route
              let sRouteId: any;
              if (this.vitisCheck.isDefAndNotNull(this.oParams)) {
                if (this.vitisCheck.isString(this.oParams.id)) {
                  sRouteId = this.oParams.id;
                }
              }

              // Recupère les données
              if (this.vitisCheck.isDefAndNotNull(sRouteId)) {
                bValuesLoading = true;
                this.vitisRequest.ajaxRequestPromise({
                  method: 'GET',
                  url: environment.properties.app_api_url + '/' + sRessourcePath + '/' + sRouteId
                }).then((resp) => {
                  if (resp.data) {
                    if (resp.data.status === 1) {
                      if (resp.data.data) {
                        const oResult = resp.data.data;
                        if (this.vitisCheck.isDefAndNotNull(oResult.wk_params)) {
                          try {
                            const oFormValues = JSON.parse(oResult.wk_params);
                            this.oParams.workspace_id = oResult[this.oMode.options.id_field];
                            resolve(oFormValues);
                          } catch (error) {
                            reject();
                          }
                        }
                      }
                    }
                  }
                });
              }
            }
          }
        }
      }
    });
  }

  /**
   * Charge le JS associé au mode
   * @returns Promise
   */
  private ajaxGetFormDef_(): Promise<boolean> {
    return new Promise((resolve, reject) => {

      const sRessourcePath = this.oMode.options.ressource;

      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url: environment.properties.app_api_url + '/' + sRessourcePath + '/' + this.oParams.workspace_id + '/form/published'
      }).then((resp) => {
        if (resp.data
          && resp.data.status === 1
          && resp.data.data
          && resp.data.data
          && resp.data.data['json_form']) {
          try {
            resolve(JSON.parse(resp.data.data['json_form']));
          } catch (error) {
            reject();
          }
        } else {
          reject();
        }
      });
    });
  }

  /**
   * Charge le formulaire
   * @returns Promise
   */
  private loadJSONForm_(): Promise<void> {
    return new Promise((resolve, reject) => {
      // Récupère le formulaire
      if (this.vitisCheck.isDefAndNotNull(this.oFormDefinition)
        && this.vitisCheck.isDefAndNotNull(this.oFormDefinition[this.sFormName])) {
        this.oFormReader.initForm(this.formReaderService.makeFormDisabled(
          this.oFormDefinition,
          this.sFormName
        ), this.sFormName, this.vitisClone.clone(this.oFormValues));
      }
      resolve();
    });
  }
}
