import { Component, OnInit, EventEmitter, ViewChild, Input } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { VitisCheck, VitisRequest, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { WorkspacelistComponent } from '../workspacelist/workspacelist.component';
import { environment } from 'src/environments/environment';

@Component({
  selector: 'app-domainlist',
  templateUrl: './domainlist.component.html',
  styleUrls: ['./domainlist.component.scss']
})
export class DomainlistComponent implements OnInit {

  @ViewChild(WorkspacelistComponent) oWorkspacelist: WorkspacelistComponent;
  public oMode: any;
  public oParams: any;

  /**
   * Permet de passer oInputMode en argument du template
   */
  @Input() set oInputMode(oMode: any) {
    if (oMode.options) {
      this.initWorkspacelist();
    }
  }

  /**
   * Constructeur
   */
  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private vitisRequest: VitisRequest,
    private vitisCheck: VitisCheck,
    private translate: TranslateService,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {
    this.route.data.subscribe(oMode => {
      this.oMode = oMode;
    });
    this.route.params.subscribe(oParams => {
      this.oParams = oParams;
      // Initialise le workspacelist
      this.initWorkspacelist();
    });
  }

  /**
   * Initialise le workspacelist
   */
  public initWorkspacelist() {
    this.oMode.type = 'workspacelist';
    // Ajout de bouton d'import
    this.oMode.options['buttons'] = [{
      'type': 'button',
      'name': 'import-user',
      'label': 'BTN_IMPORT_SELECTION_ACTIVE_DIRECTORY_USERS_VITIS_USERS',
      'class': 'btn-primary',
      'icon': 'fa fa-plus',
      'event': () => { this.importUsers(); }
    }];
    // construit le chemin vers la ressource
    this.oMode.options.ressource = 'vitis/domains/' + this.oParams['id'] + '/users';
  }

  /**
   * Redirige sur la liste des domaines
   */
  public returnToVitisUsersDomains() {
    this.router.navigate(['/dashboard/users/domains']);
  }

  /**
   * Redirige sur la liste des Utilisateurs
   */
  private returnToVitisUsersList() {
    this.router.navigate(['/dashboard/users/list']);
  }


  /**
   * Import les utilisateurs du domaine
   */
  public importUsers() {
    const rows: any = this.oWorkspacelist.oDataGrid.getSelectedRows_();

    this.translate.get([
      'LIST_NO_SELECTION',
      'SUCCESSFUL_OPERATION',
      'FORM_VALIDATION_ERROR',
      'CONFIRM_IMPORT_DOMAIN_USERS_VITIS_USERS',
      'SUCCESFULL_IMPORTATION_ACTIVE_DIRECTORY_USERS_VITIS_USERS',
      'FORM_YES',
      'FORM_NO',
      'IMPORTEDS',
      'IMPORTED'
    ]).subscribe((translation) => {
      // Si aucun enregistrements n'as été sélectionnés
      if (rows.length === 0) {
        vitisAlert.alert({
          title: translation['LIST_NO_SELECTION'],
          type: 'info',
          showCancelButton: false,
          confirmButtonText: 'OK'
        }).then(() => { });
        return;
      }
      vitisAlert.alert({
        title: translation['CONFIRM_IMPORT_DOMAIN_USERS_VITIS_USERS'],
        type: 'warning',
        showCancelButton: true,
        confirmButtonColor: '#3085d6',
        cancelButtonColor: '#d33',
        confirmButtonText: translation['FORM_YES'],
        cancelButtonText: translation['FORM_NO']
      }).then((result) => {
        if (result.value) {
          const domain_id = this.oParams['id'];
          this.vitisRequest.ajaxRequestPromise({
            method: 'POST',
            url: environment.properties.app_api_url + '/vitis/domains/' + domain_id + '/users/import',
            data: { ...rows, length: rows.length }
          }).then((resp) => {
            if (resp['data'] && resp['data']['status'] != 1) {
              vitisAlert.fireToast({
                type: 'error',
                title: translation['FORM_VALIDATION_ERROR']
              });
              return;
            }
            if (resp['data']['list_count'] === rows.length) {
              vitisAlert.fireToast({
                type: 'success',
                title: translation['SUCCESSFUL_OPERATION']
              });
              this.returnToVitisUsersList();
              return;
            }
            let imported = 'IMPORTEDS';
            if (resp['data']['list_count'] <= 1) {
              imported = 'IMPORTED';
            }
            vitisAlert.fireToast({
              type: 'warning',
              title: resp['data']['list_count'] + '/' + rows.length + ' ' + translation[imported]
            });
          });
        }
      });
    });
  }
}
