import { Component, OnInit } from '@angular/core';
import { Subject } from 'rxjs';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

@Component({
  selector: 'app-drag-and-drop-list',
  templateUrl: './drag-and-drop-list.component.html',
  styleUrls: ['./drag-and-drop-list.component.scss']
})
export class DragAndDropListComponent implements OnInit {

  /**
   * Données
   */
  public options: any;
  public oDragAndDropGridRows: Array<any> = [];

  /**
   * Elements pour le drag and drop
   */
  public placeholder: HTMLDivElement;
  public sDragElementID: string;
  private iDraggedElementIndex: number;
  private iOnDragElementIndex: number;

  /**
   * Event appelé lors du changement d'ordre
   */
  public onChange = new Subject();

  /**
   *
   */
  constructor(private vitisCheck: VitisCheck) { }

  /**
   *
   */
  ngOnInit(): void {
  }

  /**
   * Initialise la drag and drop liste avec ses options
   * @param options
   */
  public initOptions(options) {
    this.options = options;
    this.setDraggedRows();
  }

  /**
   *
   */
  public setDraggedRows(columns = this.options.columns) {
    this.oDragAndDropGridRows = columns.reduce((rows, column, index) => {
      const columnOptions = {
        index,
        label: column[this.options.label_field] ?? column.label,
        [this.options.id_field || 'id']: column[this.options.id_field] ?? column.id,
        visible: column[this.options.visible_field] ?? (column.visible ?? false)
      };
      return [...rows, columnOptions];
    }, []);
  }


  /**
   * Fonction déclenchée lors d'un début de drag and drop sur un item
   * @param event
   */
  public onDragStart(event, draggedIndexElement?) {
    // Tout dans le setTimeout (sinon bug après scroll sur chrome).
    setTimeout(() => {
      this.iDraggedElementIndex = draggedIndexElement;

      // On récupère la div element-item en cours de drag
      const draggingElement = event.target;
      event.preventDefault();

      // On lui applique quelques styles pour qu'il ne soit plus visible pendant le drag
      draggingElement.style.display = 'none';

      // On récupère tout le groupe de l'élément
      const element = draggingElement.parentNode;
      this.sDragElementID = element.id;

      // On crée la div de placeholder
      this.placeholder = document.createElement('div');
      this.placeholder.classList.add('placeholder');

      // Insertion du placeholder
      element.parentNode.insertBefore(
        this.placeholder,
        element.nextSibling
      );
    });
  }

  /**
   * Fonction déclenchée lors du survol d'un élément en cours de drag and drop
   * @param event
   */
  public onDragOver(event, draggedIndexElement?) {
    let focusedElement: any;
    const element = event.target.closest('.element');
    // Drag au dessus d'un element
    if (this.vitisCheck.isDefAndNotNull(element)) {
      focusedElement = element;
    }

    const { bottom, top } = focusedElement.getBoundingClientRect();
    const elHeigth = bottom - top;
    const offsetY = bottom - event.clientY;

    // Par en bas
    if (offsetY < elHeigth / 2) {
      focusedElement.parentNode.insertBefore(
        this.placeholder,
        focusedElement
      );
      this.iOnDragElementIndex = draggedIndexElement;
    } else if (offsetY > elHeigth / 2) {
      // Par en haut
      focusedElement.parentNode.insertBefore(
        this.placeholder,
        focusedElement.nextSibling
      );
      this.iOnDragElementIndex = draggedIndexElement;
    }
    if (this.vitisCheck.isDefAndNotNull(this.iOnDragElementIndex) && this.vitisCheck.isDefAndNotNull(this.iDraggedElementIndex)) {
      this.changeArrayIndex(this.oDragAndDropGridRows, this.iDraggedElementIndex, this.iOnDragElementIndex);
      this.iDraggedElementIndex = this.iOnDragElementIndex;
    }
  }

  /**
   * Fonction déclenchée lors du drop d'un élément
   * @param event
   */
  public onDragEnd(event) {
    const draggingElement = event.target;
    draggingElement.style.removeProperty('display');
    // On met l'élément que l'on déplacait à la place du placeholder
    if (this.placeholder.parentNode) {
      this.placeholder.parentNode.replaceChild(draggingElement.parentNode, this.placeholder);
    }
    this.updateIndexRows();
  }

  /**
  * Modifie notre tableau pour mettre à jour le déplacement des éléments via le drag and drop
  */
  private changeArrayIndex(array, fromIndex, toIndex) {
    const element = array[fromIndex];
    array.splice(fromIndex, 1);
    array.splice(toIndex, 0, element);
  }

  /**
   * Met à jour le champ index dans les lignes de oDragAndDropGridRows
   */
  private updateIndexRows() {
    for (let i = 0; i < this.oDragAndDropGridRows.length; i++) {
      this.oDragAndDropGridRows[i].index = i;
    }
    this.onChange.next(this.oDragAndDropGridRows);
  }

  /**
   * Change la visibilité d'un element
   * @param indexElement
   */
  public onChangeElementVisibility(indexElement) {
    this.oDragAndDropGridRows[indexElement].visible = !this.oDragAndDropGridRows[indexElement].visible;
    this.onChange.next(this.oDragAndDropGridRows);
  }

}
