import { Component, OnInit, Input, ElementRef } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { FormreaderField } from '../../formreader-field';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { Router } from '@angular/router';

/**
 * Type lien angular
 *
 * Arguments:
 * type : type de champ
 * name : attribut utilisé par le champ
 * label : libellé utilisé par le champ
 * nb_cols : nombre de colonnes à utiliser (de 1 à 12)
 * id? : id du champ à utiliser
 * text? : texte à afficher dans le lien, si vide vaut la valeur du lien
 * class? : Class à appliquer au lien ou au bouton
 *
 * Exemple:
 * {
 *   "type": "angularurl",
 *   "name": "url",
 *   "label": "URL",
 *   "nb_cols": 12,
 *   "text": "Mon url",
 *   "class": "btn btn-primary btn-sm"
 * }
 */
@Component({
  selector: 'app-angularurl',
  templateUrl: './angularurl.component.html',
  styleUrls: ['./angularurl.component.scss']
})
export class AngularurlComponent extends FormreaderField implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;

  /**
   * Constructeur
   */
  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    public element: ElementRef,
    public router: Router
  ) {
    super(translate, vitisCheck);
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    // Fonction getValue
    this.oField.getValue = () => {
      return this.getValue(this.oFormValues, this.oField);
    };

    // Fonction setValue
    this.oField.setValue = (oValue: any) => {
      return this.setValue(this.oFormValues, this.oField, oValue);
    };

    this.oField.refreshTooltip = () => {
      return this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
    };

    // Tooltip
    this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
  }

  /**
   * Lance la navigation
   * @param sUrl
   */
  public navigate(sUrl: string) {
    // Pour être valide l'url doit commencer par /dashboard
    if (this.vitisCheck.isDefAndNotNull(sUrl) && sUrl.substring(0, 10) == '/dashboard') {
      const oUrl = sUrl.split('?');
      // Si il y a des paramètres on les transformes en objet
      if (this.vitisCheck.isDefAndNotNull(oUrl[1])) {
        const aParams = oUrl[1].split('&');
        const oParams = {};
        for (let i = 0; i < aParams.length; i++) {
          const aParam = aParams[i].split('=');
          oParams[aParam[0]] = aParam[1];
        }

        this.router.navigate([oUrl[0]], { queryParams: oParams });
      } else {
        this.router.navigate([sUrl]);
      }
    } else {
      console.error('Le lien (' + sUrl + ') n\'est pas valide (Le lien doit commencer par /dashboard');
    }
  }


  /**
   * Return true if the element is a string not empty
   * @param {string} element
   * @returns {Boolean}
   */
  public isStringNotEmpty(element: String): boolean {
    if (!this.vitisCheck.isDefAndNotNull(element)) {
      return false;
    }
    if (!this.vitisCheck.isString(element)) {
      return false;
    }
    if (!(element.length > 0)) {
      return false;
    }
    return true;
  }

}

