import { Component, OnInit, Input, ElementRef } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { Validators } from '@angular/forms';
import { VitisCheck, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { FormreaderField } from '../../formreader-field';
import { environment } from 'src/environments/environment';
// https://www.npmjs.com/package/ng-recaptcha

/**
 * Type captcha
 *
 * Arguments:
 * To do
 *
 * Exemple:
 * To do
 */
@Component({
  selector: 'app-captcha',
  templateUrl: './captcha.component.html',
  styleUrls: ['./captcha.component.scss'],
})
export class CaptchaComponent extends FormreaderField implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;

  /**
   * Constructeur
   */
  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    public element: ElementRef
  ) {
    super(translate, vitisCheck);
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    // Fonction getValue
    this.oField.getValue = () => {
      return this.getValue(this.oFormValues, this.oField);
    };

    // Fonction setValue
    this.oField.setValue = (oValue: any) => {
      return this.setValue(this.oFormValues, this.oField, oValue);
    };

    this.oField.refreshTooltip = () => {
      return this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
    };

    // Tooltip
    this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);

    // Création d'un validateur spécifique
    this.updateFieldValidator_(this.oField);

    if (!this.vitisCheck.isDefAndNotNull(this.oField.key)) {
      this.oField.key = environment.properties.google_public_captcha;
    }
  }

  /**
   * Création d'un validateur spécifique
   * Le captcha est toujours required
   * @param oField
   */
  private updateFieldValidator_(oField: any) {

    let aValidators = [];

    // Récupère les validators inclus par FormReader
    if (this.vitisCheck.isArray(oField.validators_list)) {
      aValidators = oField.validators_list;
    }

    if (!oField.required) {

      oField.required = true;

      aValidators.push(Validators.required);
      this.oFormValues.controls[oField.name].setValidators(aValidators);

      if (!oField.validation_messages) {
        oField.validation_messages = [];
      }
      oField.validation_messages.push({
        type: 'required',
        message: this.translate.instant('NOCAPTCHA')
      });
    }
  }

  /**
   * Retourne true si le captcha est valide
   * @param oFormValues
   * @param oField
   * @returns
   */
  public isValid(oFormValues: any, oField: any): boolean {
    let result = false;
    if (this.vitisCheck.isDefAndNotNull(oFormValues.value[oField.name])) {
      result = true;
    }
    return result;
  }

  /**
   * Notifie l'utilisateur d'une erreur de validation
   * @param oFormValues
   * @param oField
   */
  public notifyValidationError(oFormValues: any, oField: any) {
    if (!this.isValid(oFormValues, oField)) {
      const sTitle = 'NOCAPTCHA';
      this.translate.get(sTitle).subscribe((sTranslation: string) => {
        vitisAlert.fireToast({
          icon: 'error',
          title: sTranslation
        });
      });
    }
  }

  /**
   * Remet à zéro le captcha
   * @param oFormValues
   * @param oField
   */
  public resetCaptcha(oFormValues: any, oField: any) {
    oFormValues.value[oField.name] = null;
    oFormValues.controls[oField.name].reset();
  }

}
