import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { VitisCheck, VitisRequest, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { FormReaderDataGridComponent } from '../data-grid/data-grid.component';
import { environment } from 'src/environments/environment';
import { TranslateService } from '@ngx-translate/core';
import { ActivatedRoute } from '@angular/router';
import { FormreaderService } from '../../services/formreader.service';

@Component({
  selector: 'app-data-grid-vmap',
  templateUrl: './data-grid-vmap.component.html',
  styleUrls: ['./data-grid-vmap.component.scss']
})
export class DataGridVmapComponent implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;
  @Input() oFormReader: any;

  /**
   * Enfant de type FormReaderDataGridComponent
   */
  @ViewChild(FormReaderDataGridComponent, { static: false, read: FormReaderDataGridComponent }) set oFormReaderDataGrid(oFormReaderDataGrid: FormReaderDataGridComponent) {
    setTimeout(() => {
      // Pas d'initialisation dans le studio.
      if (this.vitisCheck.isDefAndNotNull(oFormReaderDataGrid) &&
        this.vitisCheck.isDefAndNotNull(oFormReaderDataGrid.oDataGridOptions) &&
        this.bStudioMode === false) {
        // Définition des évènements des boutons.
        this.setButtonEvents(oFormReaderDataGrid);
      }
    }, 500);
  }

  // Couche associée.
  public oRelatedLayer: any;

  // Mode en cours d'utilisation = studio ?
  public bStudioMode: boolean;

  // Affichage du composant FormReaderDataGrid.
  public bDisplayDataGrid: boolean = false;

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private translate: TranslateService,
    private route: ActivatedRoute,
    private formreaderSrvc: FormreaderService,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    // Mode en cours d'utilisation.
    this.route.data.subscribe(mode => {
      this.bStudioMode = (mode.type == 'studio');
      if (!this.bStudioMode) {
        // Chargement des données de la couche liée (formulaire, id_field).
        setTimeout(() => {
          this.loadLayerData().then(() => {
            // Paramétrage de la grille de données.
            this.setDataGridParams();
            // Chargement des droits de l'utilisateur sur la table de la couche.
            this.loadLayerUserRights().then((oLayerUserRights) => {
              // Paramétrage des droits d'affichage des boutons.
              this.setButtonRights(oLayerUserRights);
              // Traite les options en fonction des boutons etc.
              this.handleDatatableOptions();
              // Affichage de la grille de données.
              this.bDisplayDataGrid = true;
            });
          });
        });
      } else {
        this.bDisplayDataGrid = true;
      }
    });
  }

  /**
   * Définition des évènements des boutons.
   * @param  {any} oFormReaderDataGrid
   * @returns void
   */
  public setButtonEvents(oFormReaderDataGrid: any): void {
    let oButton: any;
    // Bouton "Ajouter".
    if (this.oField.options.insert === true) {
      oButton = oFormReaderDataGrid.oDataGridOptions.buttons.filter(o => o.name === 'insert')[0];
      if (this.vitisCheck.isDefAndNotNull(oButton)) {
        // Surcharge l'évènement par défaut du bouton.
        oButton.event = () => {
          // Pré-remplissage de la valeur du champ de liaison.
          const oWebServiceGrid = oFormReaderDataGrid.oField.webservice_grid;
          const oFormValues: any = {};
          oFormValues[oWebServiceGrid.params.filter_attr] = oWebServiceGrid.params.values[oWebServiceGrid.params.filter_value];
          // Affichage du formulaire de création dans une fenêtre modale.
          this.displayModalSubForm(oFormReaderDataGrid, 'insert', oFormValues);
        };
      }
    }

    // Bouton "display".
    if (this.oField.options.display === true) {
      oButton = oFormReaderDataGrid.oDataGridOptions.iconButtons.filter(o => o.type === 'display')[0];
      // Affichage uniquement si l'utilisateur à les droits sur la table.
      if (this.vitisCheck.isDefAndNotNull(oButton)) {
        // Surcharge l'évènement par défaut du bouton.
        oButton.event = (data: any) => {
          // Chargement des données de la couche.
          this.loadSubFormValues(data).then((oFormValues: any) => {
            // Affichage du formulaire de visualisation dans une fenêtre modale.
            this.displayModalSubForm(oFormReaderDataGrid, 'display', oFormValues);
          });
        };
      }
    }

    // Bouton "update".
    if (this.oField.options.update === true) {
      oButton = oFormReaderDataGrid.oDataGridOptions.iconButtons.filter(o => o.type === 'update')[0];
      // Affichage uniquement si l'utilisateur à les droits sur la table.
      if (this.vitisCheck.isDefAndNotNull(oButton)) {
        // Surcharge l'évènement par défaut du bouton.
        oButton.event = (data: any) => {
          // Chargement des données de la couche.
          this.loadSubFormValues(data).then((oFormValues: any) => {
            // Affichage du formulaire de mise à jour dans une fenêtre modale.
            this.displayModalSubForm(oFormReaderDataGrid, 'update', oFormValues);
          });
        };
      }
    }
    // Bouton "Supprimer".
    if (this.oField.options.delete === true) {
      oButton = oFormReaderDataGrid.oDataGridOptions.buttons.filter(o => o.name === 'delete')[0];
      if (this.vitisCheck.isDefAndNotNull(oButton)) {
        // Surcharge l'évènement par défaut du bouton.
        oButton.event = () => {
          this.deleteDataGridRow();
        };
      }
    }
  }

  /**
   * Affichage du sous formulaire.
   * @param  {any} oFormReaderDataGrid
   * @param  {string} sFormMode
   * @param  {any} oFormValues
   * @returns void
   */
  public displayModalSubForm(oFormReaderDataGrid: any, sFormMode: string, oFormValues: any = {}): void {
    // Paramètres de la fenêtre modale du formulaire
    const oModalOptions = this.getModalSubFormOptions(oFormReaderDataGrid, sFormMode);
    // Paramètres du sous formulaire.
    this.setSubFormParams(this.oRelatedLayer.form_definition, sFormMode);
    // Affichage du formulaire dans une fenêtre modale.
    oFormReaderDataGrid.displayModalForm('', sFormMode, oFormValues, oModalOptions, this.oRelatedLayer.form_definition, this.oRelatedLayer.form_js);
  }

  /**
   * Retourne les paramètres de la fenêtre modale du formulaire.
   * @param  {any} oFormReaderDataGrid
   * @param  {string} sFormMode
   * @returns any
   */
  public getModalSubFormOptions(oFormReaderDataGrid: any, sFormMode: string): any {
    // Bouton "Annuler" dans tous les modes.
    const oModalOptions: any = {
      cancel_button: {
        label: 'FORM_READER_DATA_GRID_FORM_MODAL_CANCEL',
        event: () => {
          oFormReaderDataGrid.modalRef.componentInstance.closeModalForm();
        },
        visible: true
      }
    };
    // Boutons suivant le mode.
    if (sFormMode == 'insert') {
      oModalOptions.submit_button = {
        label: 'FORM_READER_DATA_GRID_FORM_MODAL_CREATE',
        event: () => {
          // Fonction de validation du Formulaire.
          if (!this.vitisCheck.isDefAndNotNull(oFormReaderDataGrid.modalRef.componentInstance.oFormReader.sendSubForm)) {
            oFormReaderDataGrid.modalRef.componentInstance.oFormReader.sendSubForm = (oFormValues: any) => {
              this.sendSubForm(oFormValues, oFormReaderDataGrid, sFormMode);
            };
          }
          // Envoi du formulaire.
          oFormReaderDataGrid.modalRef.componentInstance.oFormReader.submitForm();
        },
        visible: true
      };
    } else if (sFormMode == 'update') {
      oModalOptions.submit_button = {
        label: 'FORM_READER_DATA_GRID_FORM_MODAL_UPDATE',
        event: () => {
          // Fonction de validation du Formulaire.
          if (!this.vitisCheck.isDefAndNotNull(oFormReaderDataGrid.modalRef.componentInstance.oFormReader.sendSubForm)) {
            oFormReaderDataGrid.modalRef.componentInstance.oFormReader.sendSubForm = (oFormValues: any) => {
              this.sendSubForm(oFormValues, oFormReaderDataGrid, sFormMode);
            };
          }
          // Envoi du formulaire.
          oFormReaderDataGrid.modalRef.componentInstance.oFormReader.submitForm();
        },
        visible: true
      };
    }
    return oModalOptions;
  }

  /**
   * Paramétrage du formulaire.
   * @param  {any} oFormDefinition
   * @param  {string} sFormMode
   * @returns any
   */
  public setSubFormParams(oFormDefinition: any, sFormMode: string): any {
    // Titre de la modale.
    switch (sFormMode) {
      case 'insert':
        oFormDefinition[sFormMode].title = 'DATA_GRID_VMAP_INSERT_FORM_MODAL_TITLE';
        break;
      case 'update':
        oFormDefinition[sFormMode].title = 'DATA_GRID_VMAP_UPDATE_FORM_MODAL_TITLE';
        break;
      case 'display':
        oFormDefinition[sFormMode].title = 'DATA_GRID_VMAP_DISPLAY_FORM_MODAL_TITLE';
        break;
    }
    // Fonction d'envoi du formulaire.
    oFormDefinition[sFormMode].event = 'sendSubForm()';
  }

  /**
   * Valide l'envoi du sous formulaire.
   * @param  {any} oFormValues
   * @param  {any} oFormReaderDataGrid
   * @param  {string} sFormMode
   * @return {Promise<any>}
   */
  public sendSubForm(oFormValues: any, oFormReaderDataGrid: any, sFormMode: string): any {
    return new Promise<void>((resolve, reject) => {
      if (oFormReaderDataGrid.modalRef.componentInstance.oFormReader.isFormValid()) {
        if (this.vitisCheck.isDefAndNotNull(oFormValues)) {
          // Couche liée.
          if (this.vitisCheck.isDefAndNotNull(this.oRelatedLayer)) {
            // Paramètres de la requête suivant le mode du formulaire.
            let sRequestMethod: string;
            let sSuccessMsg: string;
            let sErrorMsg: string;
            let sRequestUrl = `${environment.properties.app_api_url}/vmap/layers/${this.oRelatedLayer.layer_id}/query`;
            switch (sFormMode) {
              case 'insert':
                sRequestMethod = 'POST';
                sSuccessMsg = 'DATA_GRID_VMAP_INSERT_FORM_SUCCESS';
                sErrorMsg = 'DATA_GRID_VMAP_INSERT_FORM_ERROR';
                break;
              case 'update':
                sRequestMethod = 'PUT';
                sSuccessMsg = 'DATA_GRID_VMAP_UPDATE_FORM_SUCCESS';
                sErrorMsg = 'DATA_GRID_VMAP_UPDATE_FORM_ERROR';
                sRequestUrl += '/' + oFormValues[this.oRelatedLayer.id_field];
                break;
            }
            // Id de la couche parente.
            oFormValues[this.oRelatedLayer.filter_attr] = this.oRelatedLayer.parent_layer_id;
            this.formreaderSrvc.manageLargesFiles(oFormValues).then((managedData: any) => {
              // Requête d'ajout/ mise à jour.
              this.vitisRequest.ajaxRequest({
                method: sRequestMethod,
                url: sRequestUrl,
                data: this.formreaderSrvc.transformToDataForm(managedData),
                success: (resp: any) => {
                  if (this.vitisCheck.isDefAndNotNull(resp.data)) {
                    if (resp.data.status === 1) {
                      // Message de succés.
                      vitisAlert.fireToast({
                        icon: 'success',
                        title: this.translate.instant(sSuccessMsg)
                      });
                      // Ferme la fenêtre modale.
                      oFormReaderDataGrid.modalRef.componentInstance.closeModalForm();
                      // Recharge les enregistrements de la grille de données.
                      this.reloadDataGrid();
                      resolve();
                    } else {
                      // Message d'erreur.
                      vitisAlert.alert({
                        type: 'error',
                        title: this.translate.instant(sErrorMsg)
                      });
                      reject();
                    }
                  }
                },
                error: (err: any) => {
                  console.error('error while loading layer parameters :', err);
                  // Message d'erreur.
                  vitisAlert.alert({
                    type: 'error',
                    title: this.translate.instant('REQUEST_ERROR')
                  });
                  reject();
                }
              });
            });
          }
        }
      }
    });
  }

  /**
   * Recharge les enregistrements de la grille de données.
   * @returns void
   */
  public reloadDataGrid(): void {
    this.oField.refreshGrid();
  }

  /**
   * Chargement des données du sous formulaire.
   * @return {Promise<any>}
   */
  public loadSubFormValues(data: any): any {
    return new Promise<any>((resolve, reject) => {
      // Charge les données de la couche sélectionnée.
      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: `${environment.properties.app_api_url}/vmap/layers/${this.oRelatedLayer.layer_id}/query`,
        params: {
          filter: {
            relation: 'AND',
            operators: [{
              column: this.oRelatedLayer.id_field,
              compare_operator: '=',
              value: data[this.oRelatedLayer.id_field]
            }]
          },
          limit: 1
        },
        success: (resp: any) => {
          if (this.vitisCheck.isDefAndNotNull(resp.data)) {
            if (resp.data.status === 1) {
              if (this.vitisCheck.isDefAndNotNull(resp.data.data) &&
                this.vitisCheck.isArray(resp.data.data) && resp.data.data.length > 0) {
                resolve(resp.data.data[0]);
              }
            }
          }
        },
        error: (err: any) => {
          console.error('error while loading layer form values :', err);
          // Message d'erreur.
          vitisAlert.alert({
            type: 'error',
            title: this.translate.instant('REQUEST_ERROR')
          });
          reject();
        }
      });
    });
  }

  /**
   * Supprime un enregistrement de la grille de données.
   * @returns void
   */
  public deleteDataGridRow(): void {
    // Lignes sélectionnées dans la grille de données.
    const aRows = this.oField.oDataGrid.getSelectedRows_();
    if (aRows.length > 0) {
      vitisAlert.alert({
        type: 'warning',
        title: this.translate.instant('LIST_DELETE_CONFIRM'),
        showCancelButton: true
      }).then((result: any) => {
        if (result.isConfirmed === true) {
          const sUrl = `${environment.properties.app_api_url}/vmap/layers/`;
          for (let i = 0; i < aRows.length; i++) {
            // Envoi requête
            this.vitisRequest.ajaxRequest({
              method: 'DELETE',
              url: `${sUrl}${this.oField.webservice_grid.params.layer_id}/query/${aRows[i][this.oRelatedLayer.id_field]}`,
              success: (resp: any) => {
                if (this.vitisCheck.isDefAndNotNull(resp.data)) {
                  if (resp.data.status === 1) {
                    // Message de succès.
                    vitisAlert.fireToast({
                      icon: 'success',
                      title: this.translate.instant('LIST_DELETE_OK')
                    });
                  }
                }
              },
              error: (err: any) => {
                console.error('error while deleting layer object :', err);
                vitisAlert.alert({
                  type: 'error',
                  title: this.translate.instant('FORM_VALIDATION_ERROR'),
                });
                i = aRows.length;
              }
            });
          }
          // Rechargement de la grille de données.
          this.reloadDataGrid();
        }
      });
    } else {
      vitisAlert.alert({
        type: 'error',
        title: this.translate.instant('LIST_DELETE_NO_SELECTION'),
      });
    }
  }

  /**
   * Chargement des données de la couche liée.
   * @return {Promise<any>}
   */
  public loadLayerData(): any {
    return new Promise<void>((resolve, reject) => {
      // Couche liée.
      const oWebServiceGrid = this.oField.webservice_grid;
      // Chargement des données de la couche liée.
      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: `${environment.properties.app_api_url}/vmap/pglayers/${this.oField.webservice_grid.params.layer_id}/infos`,
        success: (resp: any) => {
          if (this.vitisCheck.isDefAndNotNull(resp.data)) {
            if (resp.data.status === 1) {
              if (this.vitisCheck.isDefAndNotNull(resp.data.data)) {
                // Sauve les données de la couche liée.
                this.oRelatedLayer = {
                  layer_id: oWebServiceGrid.params.layer_id,
                  filter_attr: oWebServiceGrid.params.filter_attr,
                  filter_value: oWebServiceGrid.params.filter_value,
                  parent_layer_id: oWebServiceGrid.params.values[oWebServiceGrid.params.filter_value],
                  id_field: resp.data.data.idfield,
                  form_definition: JSON.parse(resp.data.data.published_form_json_def),
                  form_js: resp.data.data.published_form_js_def ?? null,
                };
                resolve();
              }
            }
          }
        },
        error: (err: any) => {
          console.error('error while loading layer parameters :', err);
          // Message d'erreur.
          vitisAlert.alert({
            type: 'error',
            title: this.translate.instant('REQUEST_ERROR')
          });
        }
      });
    });
  }

  /**
   * Chargement des droits de l'utilisateur sur la couche liée.
   * @return {Promise<any>}
   */
  public loadLayerUserRights(): any {
    return new Promise<any>((resolve, reject) => {
      // Chargement des données de la couche liée.
      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: `${environment.properties.app_api_url}/vmap/layers/${this.oField.webservice_grid.params.layer_id}/rights`,
        success: (resp: any) => {
          if (this.vitisCheck.isDefAndNotNull(resp.data)) {
            if (resp.data.status === 1) {
              if (this.vitisCheck.isDefAndNotNull(resp.data.data)) {
                resolve(resp.data.data);
              } else {
                reject();
              }
            }
          }
        },
        error: (err: any) => {
          console.error('error while loading user rights on the layer :', err);
          // Message d'erreur.
          vitisAlert.alert({
            type: 'error',
            title: this.translate.instant('REQUEST_ERROR')
          });
          reject();
        }
      });
    });
  }

  /**
   * Paramétrage de la grille de données.
   * @returns void
   */
  public setDataGridParams(): void {
    // Activation de la pagination.
    this.oField.options.paging = true;
    // Nombre d'enregistrements par page.
    this.oField.options.pageLength = environment.properties.rows_per_page;
    // Activation du chargement des données côté serveur (ajax).
    this.oField.options.bProcessing = true;
    this.oField.options.bServerSide = true;
    // Pas de chargement de données à l'initialisation de la grille.
    this.oField.options.bDeferLoading = 0;
    this.oField.options.rowId = this.oRelatedLayer.id_field;

    // Requête ajax
    this.oField.options.ajax = (data: any, callback: any, settings: any) => {

      this.getResourceData_(data)
        .then((oResponse: any) => {

          callback(oResponse);
        })
        .catch((err) => {
          console.error('error while loading values :', err);
          callback({
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          });
        });
    };
  }

  /**
   * Retourne les données de la grille de données
   * @returns ajax results
   */
  private getResourceData_(params): Promise<any> {
    const oReturn = {
      recordsTotal: 0,
      recordsFiltered: 0,
      data: []
    };

    const requestParams: any = {
      // Paramètres pour le web service (pagination, filtre, ordre...).
      limit: this.oField.options.pageLength,
      offset: params.start,
      filter: JSON.stringify({
        relation: 'AND',
        operators: [{
          column: this.oRelatedLayer.filter_attr,
          compare_operator: '=',
          value: this.oRelatedLayer.parent_layer_id
        }]
      }),
      attributs: this.oField.webservice_grid.params.attributs,
    };

    if (this.vitisCheck.isDefAndNotNull(params.order) && params.order.length > 0) {
      requestParams.order_by = params.columns[params.order[0].column].data;
      requestParams.sort_order = params.order[0].dir;
    }

    return new Promise((resolve, reject) => {
      // Lance la requête
      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: `${environment.properties.app_api_url}/vmap/layers/${this.oField.webservice_grid.params.layer_id}/query`,
        ajaxLoader: false,
        params: requestParams,
        success: (resp: any) => {

          if (this.vitisCheck.isDefAndNotNull(resp.data)) {
            if (this.vitisCheck.isDefAndNotNull(resp.data.status === 1)) {
              if (this.vitisCheck.isArray(resp.data.data)) {
                oReturn.data = resp.data.data;
              }
              if (this.vitisCheck.isDefAndNotNull(resp.data.total_row_number)) {
                oReturn.recordsTotal = resp.data.total_row_number;
                oReturn.recordsFiltered = resp.data.total_row_number;
              }
            }
          }

          resolve(oReturn);
        },
        error: (err: any) => {
          console.error('error while loading values :', err);
          vitisAlert.alert({
            type: 'error',
            title: this.translate.instant('ERROR_REQUEST_FORM_DATA_GRID'),
          });
          resolve(oReturn);
        }
      });
    });
  }

  /**
   * Paramétrage des droits d'affichage des boutons.
   * @param  {any} oLayerUserRights
   * @returns void
   */
  public setButtonRights(oLayerUserRights: any): void {
    // Bouton "Ajouter".
    if (oLayerUserRights['INSERT'] === false) {
      this.oField.options.insert = false;
    }
    // Bouton "display".
    if (oLayerUserRights['SELECT'] === false) {
      this.oField.options.display = false;
    }
    // Bouton "update".
    if (oLayerUserRights['UPDATE'] === false) {
      this.oField.options.update = false;
    }
    // Bouton "Supprimer".
    if (oLayerUserRights['DELETE'] === false) {
      this.oField.options.delete = false;
    }
  }

  /**
   * Traite les options en fonction des boutons etc.
   * @returns void
   */
  public handleDatatableOptions(): void {
    // Checkbox de sélection uniquement si bouton de suppression.
    if (this.oField.options.delete === true) {
      this.oField.options.select = true;
      this.oField.options.selectMode = 'multi';
    } else {
      this.oField.options.select = false;
    }

    const aColumns = this.oField.options.columns;
    this.oField.options.bOrder = [aColumns.find(column => column.data === this.oRelatedLayer.id_field).index, 'desc'];
  }
}
