import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { FormreaderComponent } from 'src/app/vitis/formreader/formreader.component';
import { vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';


@Component({
  selector: 'app-formreader-data-grid-modal',
  templateUrl: './data-grid-modal.component.html',
  styleUrls: ['./data-grid-modal.component.scss']
})
export class FormReaderDataGridModalComponent implements OnInit {

  @ViewChild(FormreaderComponent) oFormReader: FormreaderComponent;

  public oModalFormOptions: any;

  // Options par défaut de la fenêtre modale du formReader.
  public oModalOptions: any = {
    submit_button: {
      label: 'FORM_READER_DATA_GRID_FORM_MODAL_CONFIRM',
      event: (this_) => {
      },
      visible: false
    },
    cancel_button: {
      label: 'FORM_READER_DATA_GRID_FORM_MODAL_CANCEL',
      event: (this_) => {
        this.closeModalForm();
      },
      visible: true
    },
  };

  /**
   * Constructeur
   */
  constructor(
    private readonly activeModal: NgbActiveModal
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {
  }

  /**
   * Ferme la modale du formulaire.
   * @returns void
   */
  public closeModalForm(): void {
    this.activeModal.close();
  }

  /**
   * Merge this.oModalOptions avec options
   * @param options
   */
  public mergeModalOptions(options: any): void {
    this.oModalOptions = {
      ...this.oModalOptions,
      ...options
    };
  }

  /**
   * appelle la fonction
   */
  public callEvent(sButton) {
    const this_: any = { ...this.oModalOptions.this_ };
    this_.addRow = this.addRow;
    this_.oModalFormReader = this.oFormReader;
    this_.activeModal = this.activeModal;
    this.oModalOptions[sButton].event(this_);
  }

  /**
   * Ajoute les données du formulaire dans une nouvelle ligne.
   */
  public addRow(this_) {
    const aFields = this_.oModalFormReader.getFormFields(this_.oModalFormReader.oFormDefinition, this_.oModalFormReader.sFormDefinitionName);
    // Set la valeur à vide en utilisant les setters
    const aFormValues: any = {};
    // for (let i = 0; i < aFields.length; i++) {
    for (const oField of aFields) {
      // const oField = aFields[i];
      if (this_.vitisCheck.isDefAndNotNull(oField.getDisplayedValue)) {
        aFormValues[oField.name] = oField.getDisplayedValue();
      } else if (this_.vitisCheck.isDefAndNotNull(oField.getValue)) {
        aFormValues[oField.name] = oField.getValue();

        if (oField.required === true && !aFormValues[oField.name]) {
          vitisAlert.fireToast({
            type: 'error',
            title: this_.translate.instant('DATA_GRID_FORMREADERCOMPONENT_FIELD_REQUIRED') + ' ' +
              this_.translate.instant(oField.label),
          });
          return;
        }
      }
    }

    // Concatène la nouvelle valeur avec les anciennes puis set les valeurs dans le champs.
    let aValue = this_.oField.getValue();
    if (!this_.vitisCheck.isArray(aValue)) {
      aValue = [];
    }
    aValue.push(aFormValues);
    this_.oField.setValue(aValue, true);
    // Ferme la fenêtre modale.
    this_.activeModal.close();
  }
}
