import { Component, OnInit, Input, ElementRef } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { FormreaderField } from '../../formreader-field';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

/**
 * Type date
 *
 * Arguments:
 * type : type de champ (date, datetime)
 * name : attribut utilisé par le champ
 * label : libellé utilisé par le champ
 * nb_cols : nombre de colonnes à utiliser (de 1 à 12)
 * id? : id du champ à utiliser
 * required? : champ requis
 * default_value? : valeur par défaut
 * min_date? : date minimum
 * max_date? : date maximum
 *
 * Exemple:
 * {
 *   "type": "date",
 *   "name": "date",
 *   "label": "Date",
 *   "nb_cols": 12,
 *   "required": true
 * }
 */
@Component({
  selector: 'app-date',
  templateUrl: './date.component.html',
  styleUrls: ['./date.component.scss']
})
export class DateComponent extends FormreaderField implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;

  /**
   * Constructeur
   */
  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    public element: ElementRef
  ) {
    super(translate, vitisCheck);
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    this.oField.refreshTooltip = () => {
      return this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
    };

    // Tooltip
    this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
  }
}

