import { Component, ElementRef, Input, OnInit, ViewChild } from '@angular/core';
import { FormreaderField } from '../../formreader-field';
import { TranslateService } from '@ngx-translate/core';
import { VitisCheck, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import SignaturePad from 'signature_pad';
import { vitisLogsMode } from 'src/app/vitis/app.struct';
declare let $: any;


@Component({
  selector: 'app-dessin',
  templateUrl: './dessin.component.html',
  styleUrls: ['./dessin.component.scss']
})
export class DessinComponent extends FormreaderField implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;
  @ViewChild('sPad', { static: false }) signaturePadElement: ElementRef;

  private oSignaturePad: SignaturePad;

  /**
   * Constructeur
   */
  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    private vitisClone: VitisClone,
    public element: ElementRef
  ) {
    super(translate, vitisCheck);
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    // Fonction getValue
    this.oField.getValue = () => {
      return this.getValue(this.oFormValues, this.oField);
    };

    // Fonction setValue
    this.oField.setValue = (oValue: any) => {
      const oNewValue = this.setValue(this.oFormValues, this.oField, oValue);
      this.reloadImage();
      return oNewValue;
    };

    this.oField.refreshTooltip = () => {
      return this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
    };

    // Tooltip
    this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
  }

  /**
   * Affiche la modal de dessin
   */
  public displayDessinModal() {
    $('#dessin-form-reader-modal-' + this.oField.id).modal('show');
    this.loadCanvas();
  }

  /**
   * Cache la modal de dessin
   */
  public closeModalDessin() {
    $('#dessin-form-reader-modal-' + this.oField.id).modal('hide');
  }

  /**
   * Charge le canvas
   */
  private loadCanvas() {
    if (this.isDisplayedBtn() &&
      this.vitisCheck.isDefAndNotNull(this.signaturePadElement) &&
      this.vitisCheck.isDefAndNotNull(this.signaturePadElement.nativeElement)) {
      this.fitToContainer();

      this.oSignaturePad = new SignaturePad(this.signaturePadElement.nativeElement, {
        backgroundColor: 'rgb(255, 255, 255)' // necessary for saving image as JPEG; can be removed is only saving as PNG or SVG
      });
    }
  }

  /**
   * Vérifie si le bouton doit s'afficher
   */
  public isDisplayedBtn() {
    const isDisplayOnly = this.oField.disabled || this.oField.readonly || this.oField.displayOnly;
    const haveImg = this.vitisCheck.isDefAndNotNull(this.oFormValues.value) &&
      this.vitisCheck.isDefAndNotNull(this.oField.getValue()) && this.oField.getValue() != '';

    return (!isDisplayOnly && !haveImg) || (!isDisplayOnly && this.oField.editableDessin && haveImg);
  }

  /**
   * Responsive
   */
  private fitToContainer() {
    const canvas = $('#dessin-form-reader-modal-' + this.oField.id).find('canvas')[0];

    setTimeout(() => {
      canvas.style.width = '100%';
      canvas.style.height = '100%';
      if (canvas.offsetWidth != 0) {
        canvas.width = canvas.offsetWidth;
        canvas.height = canvas.offsetHeight;
      }
    }, 200);
  }

  /**
   * reload image on parent modal
   */
  public reloadImage() {
    if (this.vitisCheck.isDefAndNotNull(this.oFormValues.value)
      && this.oField.getValue()) { // Image fix
      let imgDessin = this.oField.getValue();
      if (typeof this.oField.getValue() == 'object') {
        imgDessin = this.oField.getValue()[0]._url;
      }
      return imgDessin;
    }
  }

  /**
   * Clear canvas
   */
  public clearCanvas() {
    if (this.vitisCheck.isDefAndNotNull(this.oSignaturePad)) {
      this.oSignaturePad.clear();
    }
  }

  /**
   * Vérifie si l'image doit s'afficher
   */
  public isDisplayedImage() {
    return this.vitisCheck.isDefAndNotNull(this.oFormValues.value) &&
      this.vitisCheck.isDefAndNotNull(this.oField.getValue());
  }

  /**
   * Sauvegarde la dessin sur oFormValues
   */
  public saveDessin() {
    if (this.vitisCheck.isDefAndNotNull(this.oSignaturePad) && !this.oSignaturePad.isEmpty()) {
      const imageUrlBase64 = this.oSignaturePad.toDataURL();

      const oBlob: any = this.b64toBlob(imageUrlBase64);
      oBlob.name = this.oField.name + '_dessin.png';
      oBlob._url = imageUrlBase64;

      if (!this.vitisCheck.isDefAndNotNull(this.oFormValues.value.vitis_thumbnail_image_files)) {
        this.oFormValues.value.vitis_thumbnail_image_files = {};
      }
      this.oFormValues.value.vitis_thumbnail_image_files[this.oField.name] = {};

      // Dessin à envoyer
      this.oField.setValue([oBlob]);

      // Reload image
      $('#dessin-form-reader-modal-' + this.oField.id).modal('hide');
    }
  }

  /**
   * Transforme l'url base 64 vers un blob
   * @param dataURI url de l'image en base 64
   */
  public b64toBlob(dataURI: string) {
    const byteString = atob(dataURI.split(',')[1]);
    const ab = new ArrayBuffer(byteString.length);
    const ia = new Uint8Array(ab);

    for (let i = 0; i < byteString.length; i++) {
      ia[i] = byteString.charCodeAt(i);
    }
    return new Blob([ab], {
      type: 'image/png'
    });
  }
}
