import { Component, OnInit, Input, ElementRef } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { FormreaderField } from '../../formreader-field';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

/**
 * Type markdown
 *
 * Arguments:
 * type : type de champ
 * name : attribut utilisé par le champ
 * label? : libellé utilisé par le champ
 * nb_cols : nombre de colonnes à utiliser (de 1 à 12)
 * id? : id du champ à utiliser
 * options?
 * options.type? : positionnement ("inline" pour affichage centé sur une ligne ou vide pour affichage normal sur deux lignes)
 *
 * Exemple:
 * {
 *   "type": "markdown",
 *   "name": "mymarkdown",
 *   "label": "My label",
 *   "nb_cols": 12
 * }
 */
@Component({
  selector: 'app-markdown',
  templateUrl: './markdown.component.html',
  styleUrls: ['./markdown.component.scss']
})
export class MarkdownComponent extends FormreaderField implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;

  public sContent: string = '';

  /**
   * Constructeur
   */
  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    public element: ElementRef
  ) {
    super(translate, vitisCheck);
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    if (!this.vitisCheck.isDefAndNotNull(this.oField.options)) {
      this.oField.options = {};
    }

    // Fonction getValue
    this.oField.getValue = () => {
      return this.getValue(this.oFormValues, this.oField);
    };

    // Fonction setValue
    this.oField.setValue = (oValue: any) => {
      if (this.vitisCheck.isString(oValue) && oValue.length > 0) {
        this.sContent = oValue;
      } else {
        this.sContent = '';
        console.error('Markdown valuenot valid', oValue);
      }
      this.setValue(this.oFormValues, this.oField, oValue);
    };

    this.oField.refreshTooltip = () => {
      return this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
    };

    // Tooltip
    this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
  }
}
