import { Component, OnInit, Input, ElementRef } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { FormreaderField } from '../../formreader-field';

/**
 * Type object-viewer
 *
 * Arguments:
 * type : type de champ (object-viewer)
 * name : attribut utilisé par le champ
 * nb_cols : nombre de colonnes à utiliser (de 1 à 12)
 * id? : id du champ à utiliser
 *
 * Exemple:
 * {
 *     "type": "object-viewer",
 *     "name": "name",
 *     "nb_cols": 12
 * }
 *
 * setValue: json
 * status: 1|0 valide ou non
 * title: Titre de la card
 * titleParams: Object contenant les paramètres pour la traduction du titre
 * values?: Objet avec clé valeur à affiché dans la liste (si valide)
 *
 * Exemple:
 * {
 *     "status": 1,
 *     "title": "Erreur : Code cron non valide (argument {{index_error_argument}})",
 *     "titleParams": {'index_error_argument': '5'}
 *     "values":
 *      {
 *          "CLE_A_TRADUIRE": "valeur",
 *          "CLE_A_TRADUIRE2": "valeur2"
 *      }
 * }
 */
@Component({
  selector: 'app-object-viewer',
  templateUrl: './object-viewer.component.html',
  styleUrls: ['./object-viewer.component.scss']
})
export class ObjectViewerComponent extends FormreaderField implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;

  public oValue: any;

  /**
   * Constructeur
   */
  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    public element: ElementRef
  ) {
    super(translate, vitisCheck);
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {
    // Fonction setValue
    this.oField.setValue = (oValue: any) => {
      this.oValue = oValue;
      return true;
    };

    this.oField.refreshTooltip = () => {
      return this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
    };

    this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
  }

}
