import { Component, OnInit, Input, ElementRef } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';

import { FormreaderField } from '../../formreader-field';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

/**
 * Type color_picker
 *
 * Arguments:
 * type : type de champ
 * name : attribut utilisé par le champ
 * label : libellé utilisé par le champ
 * nb_cols : nombre de colonnes à utiliser (de 1 à 12)
 * id? : id du champ à utiliser
 * required? : champ requis
 * default_value? : valeur par défaut
 * options?
 * options.min? : valeur min
 * options.max? : valeur max
 * options.precision? : precision (0.1)
 * options.step? : écarts disponibles (pour des raisons de performances si max est grand il est conceillé d'utiliser un écart grand)
 *
 * Exemple:
 * {
 *   "type": "slider",
 *   "name": "largeur",
 *   "label": "Largeur",
 *   "nb_cols": 12,
 *   "required": true,
 *   "options": {
 *     "min": 0,
 *     "max": 100,
 *     "precision": 1,
 *     "step": 1
 *   }
 * }
 */
@Component({
  selector: 'app-slider',
  templateUrl: './slider.component.html',
  styleUrls: ['./slider.component.scss']
})
export class SliderComponent extends FormreaderField implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;

  public options: any = {
    'min': 0,
    'max': 100,
    'step': 1,
  };

  /**
   * Constructeur
   */
  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    public element: ElementRef
  ) {
    super(translate, vitisCheck);
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    // Fonction getValue
    this.oField.getValue = () => {
      return this.getValue(this.oFormValues, this.oField);
    };

    // Fonction setValue
    this.oField.setValue = (oValue: any) => {
      return this.setValue(this.oFormValues, this.oField, oValue);
    };

    this.oField.refreshTooltip = () => {
      return this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
    };

    // Tooltip
    this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);

    // Options
    if (this.oField.options) {
      this.options = this.oField.options;
    }
  }

}
