import { Component, OnInit, Input, ElementRef } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { FormreaderField } from '../../formreader-field';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

/**
 * Type texte
 *
 * Arguments:
 * type : type de champ
 * name : attribut utilisé par le champ
 * label : libellé utilisé par le champ
 * nb_cols : nombre de colonnes à utiliser (de 1 à 12)
 * id? : id du champ à utiliser
 * required? : champ requis
 * default_value? : valeur par défaut
 * pattern? : regex à utiliser pour contraintre la saisie
 * maxLength? : Nombre de caractères maximum.
 *
 * Exemple:
 * {
 *     "type": "text",
 *     "name": "name",
 *     "label": "Nom",
 *     "required": false,
 *     "nb_cols": 12
 * }
 */
@Component({
  selector: 'app-text',
  templateUrl: './text.component.html',
  styleUrls: ['./text.component.scss']
})
export class TextComponent extends FormreaderField implements OnInit {

  @Input() sSize: string;
  @Input() oField: any;
  @Input() oFormValues: any;

  /**
   * Constructeur
   */
  constructor(
    translate: TranslateService,
    vitisCheck: VitisCheck,
    public element: ElementRef
  ) {
    super(translate, vitisCheck);
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    // Fonction getValue
    this.oField.getValue = () => {
      return this.getValue(this.oFormValues, this.oField);
    };

    // Fonction setValue
    this.oField.setValue = (oValue: any) => {
      return this.setValue(this.oFormValues, this.oField, oValue);
    };

    this.oField.refreshTooltip = () => {
      return this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
    };

    // Tooltip
    this.tooltip(this.oFormValues, this.oField, this.element.nativeElement);
  }
}
