import { TranslateService } from '@ngx-translate/core';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';

declare let $: any;

export interface SelectOptions {
  label: string,
  value: string,
  selected?: boolean
}

export class FormreaderField {

  public oField: any;

  /**
   * Constructeur
   */
  constructor(public translate: TranslateService, public vitisCheck: VitisCheck) {

    setTimeout(() => {
      const element = $('#' + this.oField.id);
      if (element.length) {
        this.oField.element = element;
      }

      if (this.oField && this.oField.oninit && this.oField.oninit.next) {
        this.oField.oninit.next();
        this.oField.oninit.complete();
      }
    });
  }

  /**
   * Getter de valeur du champ
   * @param oFormValues
   * @param oField
   *
   * @returns {any}
   */
  public getValue(oFormValues: any, oField: any): any {
    if (oField.disabled === true &&
      oFormValues.controls &&
      oFormValues.controls[oField.name]) {
      return oFormValues.controls[oField.name].value;
    }
    return oFormValues.value[oField.name];
  }

  /**
   * Setter de valeur du champ
   * @param  {any} oFormValues
   * @param  {any} oField
   * @param  {any} oValue
   */
  public setValue(oFormValues: any, oField: any, oValue: any) {

    const oPatchValues = {};
    if (this.vitisCheck.isDefAndNotNull(oField)) {
      if (this.vitisCheck.isDefAndNotNull(oField.name)) {
        oPatchValues[oField.name] = oValue;
      }
    }

    oFormValues.patchValue(oPatchValues);
  }

  /**
   * Initialisation info-bulle
   * @param oFormValues
   * @param oField
   * @param element
   */
  public tooltip(oFormValues: any, oField: any, element: any) {
    const this_ = this;

    // Supprime la popover si elle existe déjà
    if ((<any>window).$('.popover') && (<any>window).$(element).find('label .form-field-label-tooltip').length > 0) {
      (<any>window).$(element).find('label .form-field-label-tooltip')[0].remove();
      (<any>window).$('.popover').remove();

      const labelElem = $(element).find('label').first();
      if (this.vitisCheck.isDefAndNotNull(labelElem)) {
        labelElem.html(labelElem.html().replace('&nbsp;', ''));
      }
    }

    // Tooltip sur le label du champ de form. ?
    if (this.vitisCheck.isDefAndNotNull(oField.tooltip) && oField.visible !== false) {
      // Tooltip au survol de la souris (par défaut) ?
      if (!this.vitisCheck.isDefAndNotNull(oField.tooltip.trigger)) {
        oField.tooltip.trigger = 'hover';
      }

      // Création de l'élément qui contient le tooltip.
      const oTooltipElement = document.createElement('i');
      oTooltipElement.className = '\'fas fa-info-circle form-field-label-tooltip';
      const sTooltipElementId = oField.id + '_tooltip';
      oTooltipElement.id = sTooltipElementId;

      setTimeout(() => {
        const labelElem = $(element).find('label').first();
        if (this.vitisCheck.isDefAndNotNull(labelElem)) {

          labelElem.html(labelElem.text() + '&nbsp;');
          labelElem.append(oTooltipElement);

          if (!this.vitisCheck.isDefAndNotNull(oField.tooltip.title)) {
            oField.tooltip.title = '';
          }
          if (!this.vitisCheck.isDefAndNotNull(oField.tooltip.content)) {
            oField.tooltip.content = '';
          }

          // Évite la popover de clignoter
          oField.tooltip.boundary = oField.tooltip.boundary ?? 'window';

          // Placement auto par défaut
          oField.tooltip.placement = oField.tooltip.placement ?? 'right';

          this.translate.get([oField.tooltip.title, oField.tooltip.content]).subscribe((translations: string) => {
            oField.tooltip.title = translations[oField.tooltip.title];
            oField.tooltip.content = translations[oField.tooltip.content];

            // Création du toolltip
            const oPopover = (<any>window).$(element).find('label .form-field-label-tooltip').popover(oField.tooltip);

            switch (oField.tooltip.trigger) {
              case 'manual':
                // permet de passer le curseur sur la popover pour scroller si besoin
                oPopover.on('mouseenter', function () {
                  const _this = this;
                  $(this).popover('show');
                  $('.popover').on('mouseleave', function () {
                    $(_this).popover('hide');
                  });
                }).on('mouseleave', function () {
                  const _this = this;
                  setTimeout(function () {
                    if (!$('.popover:hover').length) {
                      $(_this).popover('hide');
                    }
                  }, 100);
                }).on('inserted.bs.popover', function () {
                  // La largeur est spécifiée ?
                  if (this_.vitisCheck.isDefAndNotNull(oField.tooltip.width)) {
                    $('.popover').css('width', oField.tooltip.width);
                    $('.popover').css('max-width', oField.tooltip.width);
                  }
                  $('.popover-body').css('height', 'auto');
                  $('.popover-body').css('max-height', '500px');
                  $('.popover-body').css('overflow-y', 'auto');
                  $('.popover-body').css('white-space', 'pre-wrap');
                });
                break;
              default:
                oPopover.on('inserted.bs.popover', function () {
                  if (this.constructor.name == 'HTMLElement') {
                    const oPopoverElem = document.getElementById(this.getAttribute('aria-describedby'));
                    if (this_.vitisCheck.isDefAndNotNull(oPopoverElem)) {
                      // La largeur est spécifiée ?
                      if (this_.vitisCheck.isDefAndNotNull(oField.tooltip.width)) {
                        (<any>window).$(oPopoverElem).css('width', oField.tooltip.width);
                        (<any>window).$(oPopoverElem).css('max-width', oField.tooltip.width);
                      }
                    }
                  }
                });
                break;
            }

          });
        }
      });
    }
  }

  /**
   * Permet de set les options à posteriori via le JS
   * @param options
   */
  public setOptions(oField: any, options: SelectOptions) {
    if (oField && oField.options) {
      oField.options.choises = options;
      if (oField.options.observableChoises) {
        oField.options.observableChoises.next(oField.options.choises);
      }
    }
  }
}
