import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { TranslateModule } from '@ngx-translate/core';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { ColorPickerModule } from 'ngx-color-picker';
import { RecaptchaModule, RecaptchaFormsModule } from 'ng-recaptcha';
import { MonacoEditorModule } from 'ngx-monaco-editor';
import { FormreaderComponent } from './formreader.component';
import { TextComponent } from './components/text/text.component';
import { TitleComponent } from './components/title/title.component';
import { LabelComponent } from './components/label/label.component';
import { EmailComponent } from './components/email/email.component';
import { RadioComponent } from './components/radio/radio.component';
import { PasswordComponent } from './components/password/password.component';
import { SelectComponent } from './components/select/select.component';
import { SearchableSelectComponent } from './components/searchable-select/searchable-select.component';
import { HiddenComponent } from './components/hidden/hidden.component';
import { ButtonComponent } from './components/button/button.component';
import { ListComponent } from './components/list/list.component';
import { CheckboxComponent } from './components/checkbox/checkbox.component';
import { SubtitleComponent } from './components/subtitle/subtitle.component';
import { HrComponent } from './components/hr/hr.component';
import { OffsetComponent } from './components/offset/offset.component';
import { DateComponent } from './components/date/date.component';
import { TextareaComponent } from './components/textarea/textarea.component';
import { ColorpickerComponent } from './components/colorpicker/colorpicker.component';
import { CaptchaComponent } from './components/captcha/captcha.component';
import { SliderComponent } from './components/slider/slider.component';
import { LinkurlComponent } from './components/linkurl/linkurl.component';
import { EditableListComponent } from './components/editable-list/editable-list.component';
import { FormFieldNgForDirective } from './directives/form-field-ng-for/form-field-ng-for.directive';
import { DoubleSelectComponent, DoubleSelectFilterPipe } from './components/double-select/double-select.component';
import { NumberComponent } from './components/number/number.component';
import { TagsComponent } from './components/tags/tags.component';
import { FormReaderDataGridComponent } from './components/data-grid/data-grid.component';
import { FormReaderDataGridModalComponent } from './components/data-grid/components/modal/data-grid-modal.component';
import { MonacoEditorComponent } from './components/monaco-editor/monaco-editor.component';
import { ObjectViewerComponent } from './components/object-viewer/object-viewer.component';
import { TinymceComponent } from './components/tinymce/tinymce.component';
import { FilePickerComponent } from './components/file-picker/file-picker.component';
import { GlobalFilePickerComponent } from './components/file-picker/global-file-picker/global-file-picker.component';
import { UrlComponent } from './components/url/url.component';
import { DataGridModule } from 'src/app/vitis/data-grid/data-grid.module';
import { MapFormreaderComponent } from './components/map-formreader/map-formreader.component';
import { MapCoreModule } from '../map-core/map-core.module';
import { TreeviewFormreaderComponent } from './components/treeview-formreader/treeview-formreader.component';
import { PreComponent } from './components/pre/pre.component';
import { SharedModule } from 'src/app/vitis/shared/shared.module';
import { EditorModule } from '@tinymce/tinymce-angular';
import { MarkdownComponent } from './components/markdown/markdown.component';
import { MarkdownModule } from 'ngx-markdown';
import { IconComponent } from './components/icon/icon.component';
import { ImageurlComponent } from './components/imageurl/imageurl.component';
import { DessinComponent } from './components/dessin/dessin.component';
import { MapWorkbenchComponent } from './components/map-workbench/map-workbench.component';
import { AngularurlComponent } from './components/angularurl/angularurl.component';
import { TextBodyComponent } from './components/text-body/text-body.component';
import { DataGridVmapComponent } from './components/data-grid-vmap/data-grid-vmap.component';
import { FormIconComponent } from './components/formicon/formicon.component';


@NgModule({
  imports: [
    CommonModule,
    FormsModule,
    ReactiveFormsModule,
    TranslateModule,
    NgbModule,
    ColorPickerModule,
    RecaptchaModule,
    RecaptchaFormsModule,
    MonacoEditorModule,
    DataGridModule,
    MapCoreModule,
    EditorModule,
    SharedModule,
    MarkdownModule.forRoot(),
  ],
  declarations: [
    FormreaderComponent,
    TextComponent,
    TitleComponent,
    LabelComponent,
    EmailComponent,
    RadioComponent,
    PasswordComponent,
    SelectComponent,
    SearchableSelectComponent,
    HiddenComponent,
    ButtonComponent,
    ListComponent,
    CheckboxComponent,
    SubtitleComponent,
    HrComponent,
    OffsetComponent,
    DateComponent,
    TextareaComponent,
    ColorpickerComponent,
    CaptchaComponent,
    SliderComponent,
    LinkurlComponent,
    EditableListComponent,
    FormFieldNgForDirective,
    DoubleSelectComponent,
    DoubleSelectFilterPipe,
    NumberComponent,
    TagsComponent,
    FormReaderDataGridComponent,
    FormReaderDataGridModalComponent,
    MonacoEditorComponent,
    ObjectViewerComponent,
    TinymceComponent,
    FilePickerComponent,
    GlobalFilePickerComponent,
    UrlComponent,
    MapFormreaderComponent,
    TreeviewFormreaderComponent,
    PreComponent,
    MarkdownComponent,
    IconComponent,
    ImageurlComponent,
    DessinComponent,
    MapWorkbenchComponent,
    AngularurlComponent,
    TextBodyComponent,
    DataGridVmapComponent,
    FormIconComponent,
  ],
  exports: [
    FormreaderComponent,
    TextComponent,
    TitleComponent,
    LabelComponent,
    EmailComponent,
    RadioComponent,
    PasswordComponent,
    SelectComponent,
    HiddenComponent,
    ButtonComponent,
    ListComponent,
    CheckboxComponent,
    SubtitleComponent,
    HrComponent,
    OffsetComponent,
    DateComponent,
    TextareaComponent,
    ColorpickerComponent,
    CaptchaComponent,
    SliderComponent,
    LinkurlComponent,
    EditableListComponent,
    FormFieldNgForDirective,
    DoubleSelectComponent,
    NumberComponent,
    TagsComponent,
    FormReaderDataGridComponent,
    FormReaderDataGridModalComponent,
    MonacoEditorComponent,
    ObjectViewerComponent,
    TinymceComponent,
    FilePickerComponent,
    GlobalFilePickerComponent,
    UrlComponent,
    MapFormreaderComponent,
    TreeviewFormreaderComponent,
    PreComponent,
    MarkdownComponent,
    IconComponent,
    ImageurlComponent,
    DessinComponent,
    MapWorkbenchComponent,
    FormIconComponent,
    AngularurlComponent,
    TextBodyComponent,
  ],
})
export class FormreaderModule { }
