import { Component, OnInit } from '@angular/core';
import { environment } from 'src/environments/environment';
import { ActivatedRoute, Params } from '@angular/router';
import { VitisRequest, VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { LoginConfig } from 'src/app/vitis/app.model';

@Component({
  selector: 'app-signup',
  templateUrl: './signup.component.html',
  styleUrls: ['./signup.component.scss']
})
export class SignupComponent implements OnInit {

  public sMessage: string;
  public roles: string;

  private config: LoginConfig = {
    confirm_ressource: 'vitis/accounts',
  };

  /**
   * Constructeur
   */
  constructor(
    private route: ActivatedRoute,
    private vitisRequest: VitisRequest,
    private vitisCheck: VitisCheck
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    if (environment.appStruct.login) {
      this.config = {
        ...this.config,
        ...environment.appStruct.login
      };
    }

    const sUrl = `${environment.properties.app_api_url}/${this.config.confirm_ressource}/signup/confirm`;

    this.route.queryParams.subscribe((oParams: Params) => {

      if (!this.vitisCheck.isDefAndNotNull(oParams.datakey)) {
        this.sMessage = 'MISSING_PARAMS';
      } else {
        this.vitisRequest.ajaxRequest({
          url: sUrl,
          method: 'POST',
          async: false,
          responseType: '',
          data: { datakey: oParams.datakey },
          displayErrorMessage: false,
          success: (resp: any) => {
            if (environment.properties.automated_sign_up) {
              this.sMessage = 'ACCOUNT_ACTIVED';
            } else {
              this.roles = environment.properties.sign_up_automated_roles.join(', ') + ', vitis_user';
              this.sMessage = 'ACCOUNT_ACTIVED_ADMIN';
            }
          }, error: (resp: any) => {
            const data = JSON.parse(resp.data);
            this.sMessage = data.errorMessage;
          }
        });
      }
    });
  }
}
