import { Component, OnInit, Input, Renderer2, ViewChild, ElementRef } from '@angular/core';
import { Map } from 'ol';
import { VitisCheck, VitisRequest } from 'src/app/vitis/core/services/utils/vitis-utils';

@Component({
  selector: 'app-core-controls-map-projection',
  templateUrl: './core-controls-map-projection.component.html',
  styleUrls: ['./core-controls-map-projection.component.scss']
})
export class CoreControlsMapProjectionComponent implements OnInit {

  @Input() olMap: Map;
  @ViewChild('mapProjectionElement', { static: true }) mapProjectionElem: ElementRef;

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private renderer: Renderer2
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    this.reloadMapProjection_();
    this.olMap.on('change', () => {
      this.reloadMapProjection_();
    });
  }

  /**
   * Récupère les données de projection provenant d'un JSON
   * @returns Promise
   */
  private getProjections_(): Promise<any> {
    return new Promise((resolve, reject) => {

      const url = 'assets/data/proj.json';

      // Récupère le formulaire
      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url
      }).then((resp) => {
        if (this.vitisCheck.isDefAndNotNull(resp.data)) {
          resolve(resp.data);
        } else {
          console.error('Error getting JSON form');
          reject(false);
        }
      });
    });
  }

  /**
   * Recharge le contenu de la div
   */
  private reloadMapProjection_() {
    if (this.vitisCheck.isDefAndNotNull(this.mapProjectionElem)) {
      this.getProjections_().then((oProjections) => {
        const sCurrentProjCode = this.olMap.getView().getProjection().getCode();
        const oCurrentProjection = oProjections.projections.find(proj => proj.code == sCurrentProjCode);
        if (oCurrentProjection !== -1) {
          this.renderer.setProperty(this.mapProjectionElem.nativeElement, 'innerHTML', oCurrentProjection.name);
        } else {
          this.renderer.setProperty(this.mapProjectionElem.nativeElement, 'innerHTML', sCurrentProjCode);
        }
      });
    }
  }
}
