import { Component, Input, OnInit } from '@angular/core';
import { Map } from 'ol';
import { ScaleLine } from 'ol/control';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';

@Component({
  selector: 'app-core-controls-scaleline',
  templateUrl: './core-controls-scaleline.component.html',
  styleUrls: ['./core-controls-scaleline.component.scss']
})
export class CoreControlsScalelineComponent implements OnInit {

  @Input() olMap: Map;

  public mapScale: number;
  public scaleLine: ScaleLine;

  /**
   * Constructeur
   */
  constructor(private mapUtils: MapCoreUtilsService) { }

  /**
   * Fonction appelée lors de l'intialisation
   */
  ngOnInit(): void {
    this.refreshScale_();
    this.olMap.on('moveend', () => {
      this.refreshScale_();
    });

    // Scale line
    this.scaleLine = new ScaleLine({
      target: document.getElementById('map-scale-line')
    });

    this.olMap.addControl(this.scaleLine);
  }


  /**
   * Refraichit le texte de l'échelle
   */
  private refreshScale_() {
    this.mapScale = Math.round(this.mapUtils.getScale(this.olMap, { pretty: false }));
  }
}
