import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { MapCoreComponent } from './map-core.component';
import { FormsModule } from '@angular/forms';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { TranslateLoader, TranslateModule } from '@ngx-translate/core';
import { environment } from 'src/environments/environment';
import { MultiTranslateHttpLoader } from 'ngx-translate-multi-http-loader';
import { CoreControlsComponent } from './components/core-controls/core-controls.component';
import { CoreControlsMapProjectionComponent } from './components/core-controls/components/core-controls-map-projection/core-controls-map-projection.component';
import { CoreControlsLocationComponent } from './components/core-controls/components/core-controls-location/core-controls-location.component';
import { CoreControlsZoomComponent } from './components/core-controls/components/core-controls-zoom/core-controls-zoom.component';
import { CoreControlsDefaultExtentComponent } from './components/core-controls/components/core-controls-default-extent/core-controls-default-extent.component';
import { CoreControlsGeolocationComponent } from './components/core-controls/components/core-controls-geolocation/core-controls-geolocation.component';
import { CoreControlsSearchComponent } from './components/core-controls/components/core-controls-search/core-controls-search.component';
import { CoreInteractionsComponent } from './components/core-interactions/core-interactions.component';
import { CoreControlsMapNameComponent } from './components/core-controls/components/core-controls-map-name/core-controls-map-name.component';
import { CoreDraganddropComponent } from './components/core-draganddrop/core-draganddrop.component';
import { CoreControlsMapCommentComponent } from './components/core-controls/components/core-controls-map-comment/core-controls-map-commentcomponent';
import { CoreControlsScalelineComponent } from './components/core-controls/components/core-controls-scaleline/core-controls-scaleline.component';

/**
 * Chargement des fichiers de traduction
 * @param http
 * @returns
 */
export function HttpLoaderFactory(http: HttpClient) {
  return new MultiTranslateHttpLoader(http, environment.appStruct['translate']);
}

@NgModule({
  imports: [
    FormsModule,
    CommonModule,
    HttpClientModule,
    TranslateModule.forChild({
      loader: {
        provide: TranslateLoader,
        useFactory: HttpLoaderFactory,
        deps: [HttpClient]
      }
    }),
  ],
  declarations: [
    MapCoreComponent,
    CoreControlsComponent,
    CoreControlsMapProjectionComponent,
    CoreControlsLocationComponent,
    CoreControlsZoomComponent,
    CoreControlsDefaultExtentComponent,
    CoreControlsGeolocationComponent,
    CoreControlsSearchComponent,
    CoreInteractionsComponent,
    CoreControlsMapNameComponent,
    CoreDraganddropComponent,
    CoreControlsMapCommentComponent,
    CoreControlsScalelineComponent,
  ],
  exports: [
    MapCoreComponent,
    CoreControlsMapNameComponent,
    CoreControlsZoomComponent,
    CoreControlsSearchComponent,
    CoreControlsMapCommentComponent
  ],
})
export class MapCoreModule { }
