import { Injectable } from '@angular/core';

import Map from 'ol/Map.js';
import { Vector as VectorLayer } from 'node_modules/ol/layer.js';
import { Vector as VectorSource } from 'ol/source.js';
import Style from 'ol/style/Style';
import Fill from 'ol/style/Fill';
import Stroke from 'ol/style/Stroke';
import Circle from 'ol/style/Circle';
import Collection from 'ol/Collection';
import { Geometry } from 'ol/geom';

@Injectable({
  providedIn: 'root'
})
export class MapCorePopupOverlayService {
  /**
   * Constructeur
   */
  constructor() { }

  /**
   * Initialise un vector layer pour afficher les features en lien avec une popup
   * @param  {Map} olMap
   * @returns {VectorLayer}
   */
  public initPopupOverlay(olMap: Map): VectorLayer<VectorSource<Geometry>> {
    return this.initLocationPopupOverlay_(olMap);
  }

  /**
   * Initialise un vector layer pour afficher les features en lien avec une popup
   * @param {Map} olMap
   * @returns {VectorLayer}
   */
  private initLocationPopupOverlay_(olMap: Map): VectorLayer<VectorSource<Geometry>> {
    const oLocationStyleOverlay: Style = new Style({
      fill: new Fill({
        color: 'rgba(255, 255, 255, 0.3)'
      }),
      stroke: new Stroke({
        color: '#337ab7',
        width: 2
      }),
      image: new Circle({
        radius: 10,
        stroke: new Stroke({
          color: '#337ab7',
          width: 2
        }),
        fill: new Fill({
          color: 'rgba(255, 255, 255, 0.2)'
        })
      })
    });

    return new VectorLayer({
      map: olMap,
      style: oLocationStyleOverlay,
      source: new VectorSource({
        features: new Collection(),
        useSpatialIndex: false
      }),
      zIndex: 1000,
    });
  }

}
