import { Injectable } from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { filter, map } from 'rxjs/operators';

@Injectable({
  providedIn: 'root'
})
export class MapCoreStudioService {

  private mapChangeEvent = new BehaviorSubject<any>(undefined);
  private mapOptionsChangeEvent = new BehaviorSubject<any>(undefined);
  /**
   * Constructeur
   */
  constructor() { }

  /**
   * Émet un event de changement de la carte
   * @param studioId
   * @param olMap
   */
  public emitMapEvent(studioId: any, olMap: any) {
    this.mapChangeEvent.next({ studioId, olMap });
  }

  /**
   * Retourne un observable permettant d'écouter les changements sur la carte
   * @param studioId
   * @returns
   */
  public mapEventListner(studioId: any) {
    return this.mapChangeEvent.asObservable()
      .pipe(
        filter(e => {
          if (!e) {
            return;
          }
          return e.studioId === studioId;
        }),
        map(e => e.olMap)
      );
  }

  /**
   * Émet un event lors du changement des options
   * @param studioId
   * @param change
   */
  public emitMapOptionsEvent(studioId: any, change: any) {
    this.mapOptionsChangeEvent.next({ studioId, change });
  }

  /**
   * Retourne un observable permettant d'écouter les modifications sur les options de la carte
   * @param studioId
   * @returns
   */
  public mapOptionsEventListner(studioId: any) {
    return this.mapOptionsChangeEvent.asObservable()
      .pipe(
        filter(e => {
          if (!e) {
            return;
          }
          return e.studioId === studioId;
        }),
        map(e => e.change)
      );
  }

}
