import { Layer } from 'ol/layer';
import { Source } from 'ol/source';

/**
 * Définition d'un carte FormReader
 */
export class MapJSON {
  type: string;
  tree?: any; // arbre de définition de la carte
  map_id?: number;
  center?: any;
  extent?: any;
  source?: any; // utilisé pour cartes Bing
  circle_radius?: number;
  contour_color?: string;
  contour_size?: number;
  controls: {
    map_projection?: boolean,
    mobile_gps_zoom?: boolean,
    mouse_position?: boolean,
    scale_line?: boolean,
    zoom_controls?: boolean,
    location_controls?: boolean,
    attribution?: boolean, // True par défaut
  };
  coord_accuracy?: number;
  draw_color?: string;
  interactions: {
    business_objects?: boolean,
    draw_point?: boolean,
    draw_line?: boolean,
    draw_polygon?: boolean,
    draw_multipolygon?: boolean,
    full_screen?: boolean,
    modify?: boolean,
    multi_geometry?: boolean,
    remove?: boolean,
    remove_all?: boolean,
  };
  output_format?: string;
  proj?: string;
  map_geocoders?: any;
  map_default_geocoders?: string;
}

/**
 * Défintion d'une carte
 */
export class MapDefinition {
  name: string;
  children: Array<any>;
  maptools: Array<any>;
  options?: {
    optimized?: boolean;
  };
}

/**
 * Catalogue de cartes
 */
export class MapCatalog {
  maps: Array<any>;
  services: any;
  usedMap?: number;
  status?: number;
  defaultMap?: number;
  defaultExtent?: string;
}

export class VmapLayer {
  id?: string;
  index?: number;
  layerType?: string;
  olLayer?: any | Layer<Source>;
  opacity?: number;
  params?: any;
  select?: boolean;
  tileLoadFunction?: any;
  type?: string;
  url?: string;
  version?: any;
  visible?: boolean;
  active?: boolean;
  coordsys_id?: number;
  datemaj?: any;
  definition_sym_json?: any;
  extent?: any;
  form_display_size_id?: number;
  form_insert_size_id?: number;
  form_title?: string;
  form_update_size_id?: number;
  geomfield?: string;
  idfield?: string;
  is_automatically_edit?: boolean;
  is_background_layer?: boolean;
  is_clone?: boolean;
  is_data_grid?: boolean;
  is_geom_selection?: boolean;
  is_geom_updatable?: boolean;
  is_geometry_layer?: boolean;
  is_location?: boolean;
  is_filter?: boolean;
  is_map_selection?: boolean;
  is_map_hidden?: boolean;
  map_parent_hidden?: number;
  is_multi_edit?: boolean;
  is_multi_geom?: boolean;
  is_requestor_layer?: boolean;
  is_specific_table_form?: boolean;
  is_specific_table_requestor?: boolean;
  is_sym_definition_json?: boolean;
  json_columns?: any;
  layer_id?: number;
  publication_id?: number;
  location_placeholder?: string;
  location_result_field?: string;
  location_search_field?: string;
  location_search_mode_id?: any;
  map_selection_buffer?: any;
  max_edition_scale?: any;
  max_layer_scale?: any;
  min_edition_scale?: any;
  min_layer_scale?: any;
  ms_geomtype_id?: any;
  ms_geomtype?: any;
  select_field?: any;
  name?: string;
  published_form_js_def?: any;
  published_form_css_def?: any;
  published_form_json_def?: any;
  schema_form?: any;
  schema_requestor?: any;
  source?: any;
  table_form?: any;
  table_requestor?: any;
  theme?: any;
  database?: any;
  schema?: any;
  table?: any;
  pg_table?: any;
  datasource?: any;
  have_select_right?: boolean;
  have_insert_right?: boolean;
  have_update_right?: boolean;
  have_delete_right?: boolean;
  have_requestor_select_right?: boolean;
  pictogram?: string;
  legend?: string;
}

export class GeoCoder {
  title: string;
  url: string;
  data_field: string;
  title_field: string;
  description_field?: string;
  geojson_field?: string;
  summary_fields?: Array<{ key: string, label: string }>;
}

export class StyleJSON {
  draw?: StyleDrawJSON;
  text?: StyleTextJSON;
}

export class StyleDrawJSON {
  symbol?: string;
  color?: string;
  outline_color?: string;
  size?: string;
  dash?: string;
}

export class StyleTextJSON {
  text?: string;
  size?: string;
  font?: string;
  color?: string;
  outline_color?: string;
  outline_size?: string;
  offsetX?: string;
  offsetY?: string;
  rotation?: string;
}

export const availableControls: string[] = [
  'zoom_controls', // Boutons de zoom
  'zoom_slider', // Slider de zoom
  'map_projection', // Système de projection
  'map_title', // Titre de la carte
  'numeric_scale', // Echelle numérique
  'graphic_scale', // Echelle graphique
  'mouse_coordinates', // Coordonnées de la souris
  'map_overview', // Miniature de la carte
  'layers_tree_tab', // Onglet de l'arbre des couches de la carte
  'layers_list_tab', // Onglet de la liste des couches de la carte
  'layers_legend_tab' // Onglet de la légende des couches de la carte
];

export const availableControlsMobile: string[] = [
  'zoom_controls', // Boutons de zoom
  'map_title', // Titre de la carte
  'numeric_scale', // Echelle numérique
  'graphic_scale', // Echelle graphique
  'layers_tree_tab', // Onglet de l'arbre des couches de la carte
  'layers_list_tab', // Onglet de la liste des couches de la carte
  'layers_legend_tab' // Onglet de la légende des couches de la carte
];
