import { Component, OnInit, Input, Output, EventEmitter, AfterViewInit } from '@angular/core';
import { VitisCheck, VitisRequest, VitisClone, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import moment from 'moment';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { environment } from 'src/environments/environment';
import { TranslateService } from '@ngx-translate/core';
import { ActivatedRoute } from '@angular/router';

declare let $: any;

@Component({
  selector: 'app-customfilter',
  templateUrl: './customfilter.component.html',
  styleUrls: ['./customfilter.component.scss']
})
export class CustomfilterComponent implements OnInit, AfterViewInit {

  @Output() submit = new EventEmitter<number>();

  @Input() oFilterTypes: any;
  @Input() oResults: any;
  @Input() oFilter: any;
  @Input() aColumns: any;
  @Input() sPlaceholder: string;
  @Input() sDefaultFilterColumn: string;

  public aSugestionColumns: Array<any> = [];
  public aSugestionValues: Array<any> = [];
  public aSugestionOperators: Array<string> = [];
  public aWsSugestionValues: Array<any> = [];
  public oSearchColumn: any;
  public sCompareOperator: any;
  public sElemId = 'workspacelist-customfilter-input-' + Math.floor(Math.random() * 1000000);

  // Set lorsque l'opérateur est IN
  public checkedValuesSet = new Set();

  public aCompareOperators: Array<string> = ['~', '=', '>', '<', 'IN', 'NOT IN', 'IS NULL', 'IS NOT NULL'];

  // Utilisé pour dates
  public oDatePickerField: any = {
    type: 'datetime',
    name: 'filterdate',
    label: '',
  };
  public showDateTimePicker: boolean = true;
  //
  public oOptions: any;

  // Pour chargement via webservice
  private iWsLastLoad = 0;
  public iWsTotalRows: number;
  public bWsLoading = false;

  /**
   * Constructeur
   */
  constructor(
    private vitisCheck: VitisCheck,
    private authService: AuthService,
    private vitisRequest: VitisRequest,
    private vitisClone: VitisClone,
    private translate: TranslateService,
    private route: ActivatedRoute
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {
    // Mode en cours d'utilisation
    this.route.data.subscribe(mode => {
      this.oOptions = mode.options;
    });
  }

  /**
   * Appelé après le chargement du DOM
   */
  ngAfterViewInit() {

    // Input de recherche
    const input: any = document.getElementById(this.sElemId);

    // Événements clavier
    input.addEventListener('keyup', (event: any) => {

      // Touche "Enter"
      if (event.keyCode === 13) {

        // Supprime l'action en cours si besoin
        event.preventDefault();

        // Valide la recherche
        if (this.sCompareOperator === '=' &&
          this.oSearchColumn &&
          this.oSearchColumn.searchWebService &&
          this.oSearchColumn.searchSuggestions &&
          this.oSearchColumn.searchSuggestions[0]) {
          // Si on est dans un web service alors prend la première valeur dispo
          this.onValidSearchString(`${this.oSearchColumn.translateTitle} ${this.sCompareOperator} ${this.oSearchColumn.searchSuggestions[0].value}`);
        } else {
          // Autrement prend se qui a été tapé
          this.onValidSearchString(input.value);
        }
      }
    });
  }

  /**
   * Fonction lancée au changement de la recherche
   * @param  {string} searchValue
   * @returns void
   */
  public onSearchChange(searchValue: string): void {

    let oSearchColumn = this.extractSearchColumn_(searchValue);
    const sSearchOperator = this.extractSearchOperator_(searchValue);

    // Vide this.oSearchColumn si l'utilisateur fait des backspace
    if (this.oSearchColumn &&
      !searchValue.toLowerCase().startsWith(this.oSearchColumn.translateTitle.toLowerCase())) {
      this.oSearchColumn = null;
    } else if (!oSearchColumn) {
      oSearchColumn = this.oSearchColumn;
    } else {
      this.oSearchColumn = oSearchColumn;
    }

    // Set this.sCompareOperator
    if (sSearchOperator) {
      this.sCompareOperator = sSearchOperator;
    } else {
      this.sCompareOperator = null;
    }

    // Suggestions de colonnes
    this.aSugestionColumns = this.getColumnSuggestions_(searchValue);
    if (this.oSearchColumn) {
      if (this.sCompareOperator) {
        this.aSugestionOperators = [];

        // Suggestions de valeurs
        this.aSugestionValues = this.getValueSuggestions_(searchValue);
      } else {

        // Suggestions d'opérateurs
        this.aSugestionOperators = this.getOperatorSuggestions_(searchValue);

        // Opérateur auto s'il y en a qu'un
        if (this.aSugestionOperators.length === 1) {
          this.setCompareOperator(this.aSugestionOperators[0]);
        }
      }
    }

    // Force l'affichage de la dropdown
    this.showSuggestionsDropdown();
  }

  /**
   * Fonction lancée à la validation de la recherche
   * @param  {string} searchValue
   * @returns void
   */
  public onValidSearchString(searchValue: string): void {
    if (this.vitisCheck.isString(searchValue)) {

      // Colonne
      this.oSearchColumn = this.extractSearchColumn_(searchValue);
      if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn)) {
        if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.translateTitle)
          && this.vitisCheck.isDefAndNotNull(this.oSearchColumn.field)) {

          // Valeur
          const sFilterValue = this.extractSearchValue_(searchValue, true);

          // operateur
          const sOperator = this.parseSearch_(searchValue).operator;

          this.addFilterNode(this.oSearchColumn, sOperator, sFilterValue);
        }
      } else {
        // Cas où la colonne ne soit pas renseignée : prend la colonne favorite
        const oSearchColumn = this.getColumnByField_(this.sDefaultFilterColumn);
        if (this.vitisCheck.isDefAndNotNull(oSearchColumn)) {
          this.oSearchColumn = oSearchColumn;

          let sCompareOperator = '~';
          switch (oSearchColumn.searchType) {
            case 'boolean':
            case 'date':
            case 'number':
            case 'strict':
              sCompareOperator = '=';
              break;
            default:
              break;
          }

          this.addFilterNode(this.oSearchColumn, sCompareOperator, searchValue);
        } else {
          // Cas où il n'y a pas non plus de colonne favorite
          this.aSugestionColumns = this.getColumnSuggestions_('');
          this.showSuggestionsDropdown();

          // Message d'erreur
          this.translate.get('VITIS_WORKSPACELIST_FILTER_NO_COLUMN').subscribe((translation) => {
            vitisAlert.fireToast({
              type: 'error',
              title: translation
            });
          });
        }
      }
    }
  }

  /**
   * Crée le noeud de filtre et l'ajoute à this.oFilter.nodes
   * @param  {any} oSearchColumn
   * @param  {String} sCompareOperator
   * @param  {String} sValue
   * @returns void
   */
  public addFilterNode(oSearchColumn: any, sCompareOperator: String, sValue: String | String[]): void {

    if (this.vitisCheck.isDefAndNotNull(oSearchColumn)
      && this.vitisCheck.isString(sCompareOperator)
      && (this.vitisCheck.isString(sValue) || this.vitisCheck.isArray(sValue))) {

      // Valeur d'affichage
      let displayValue = sValue;

      // Valeur d'affichage pour les tableaux (type IN)
      if (Array.isArray(sValue)) {
        const tmp = [];
        for (const val of sValue) {
          tmp.push(this.getDisplayedValue_(oSearchColumn, String(val)));
        }
        displayValue = tmp.join(', ');
      } else {
        // Valeur d'affichage si on a utilisé une liste clé / valeur
        displayValue = this.getDisplayedValue_(oSearchColumn, String(sValue));
      }

      const oFilterNode = {
        column: oSearchColumn,
        compareOperator: sCompareOperator,
        compareOperatorTranslate: this.translate.instant(`VITIS_WORKSPACELIST_FILTER_COMPARE_OPERATOR_${sCompareOperator}`),
        value: sValue,
        displayValue
      };

      // Gesiton tableau de valeur avec libellé
      if (this.vitisCheck.isArray(oSearchColumn.searchSuggestions)) {
        for (let i = 0; i < oSearchColumn.searchSuggestions.length; i++) {
          const suggestion = oSearchColumn.searchSuggestions[i];
          if (this.vitisCheck.isDefAndNotNull(suggestion.label) &&
            this.vitisCheck.isDefAndNotNull(suggestion.value)) {
            if (suggestion.value == sValue) {
              oFilterNode.displayValue = suggestion.label;
            }
          }
        }
      }

      // Gestion dates
      if (oSearchColumn.searchType === 'date') {
        let sDateFormat = 'DD/MM/YYYY';
        const oUser = this.authService.getUserInfos();
        if (this.vitisCheck.isDefAndNotNull(oUser)) {
          if (this.vitisCheck.isDefAndNotNull(oUser.formatdate_id)) {
            sDateFormat = oUser.formatdate_id;
          }
        }
        if (sValue) {
          oFilterNode.displayValue = moment.unix(Number(sValue)).format(sDateFormat + ' HH:mm');
        }
      }

      // Gestion numériques
      if (oSearchColumn.searchType === 'number') {
        if (!this.vitisCheck.isNumber(Number(sValue))) {
          console.error('value is not a number');
          // Message d'erreur
          this.translate.get('VITIS_WORKSPACELIST_FILTER_VALUE_NOT_NUMBER').subscribe((translation) => {
            vitisAlert.fireToast({
              type: 'error',
              title: translation
            });
          });
          return;
        }
      }

      // Relation automatique si c'est deux fois la même colonne
      this.oFilter.relationOperator = this.oFilter.relationOperator ?? 'AND';

      this.oFilter.nodes.push(oFilterNode);

      // Vide le champ de recherche
      const input: any = document.getElementById(this.sElemId);
      input.value = '';

      // Applique le filtre à la grille
      setTimeout(() => {
        this.oSearchColumn = undefined;
        this.submit.emit();
      });

      // Cache la dropdown
      $('.customfilter-input').find('.dropdown-menu.show').dropdown('toggle');
    }
  }

  /**
   * Supprime un noeud du filtre
   * @param  {any} oNode
   * @returns void
   */
  public removeFilterNode(oNode: any): void {

    const iNodeIndex = this.oFilter.nodes.indexOf(oNode);

    if (iNodeIndex !== -1) {
      this.oFilter.nodes.splice(iNodeIndex, 1);
    }

    // Applique le filtre à la grille
    this.submit.emit();
  }

  /**
   * Valide la colonne de recherche
   * @param  {any} oSearchColumn
   */
  public setSearchColumn(oSearchColumn: any) {

    this.oSearchColumn = oSearchColumn;

    const input: any = document.getElementById(this.sElemId);

    // Renseigne le nom de la colonne dans la valeur
    input.value = this.translate.instant(oSearchColumn.translateTitle) + ' ';

    // Focus l'input
    input.focus();

    // Affiche la dropdown
    setTimeout(() => {
      input.click();
    });
  }

  /**
   * Valide l'opérateur de recherche
   * @param  {any} oSearchColumn
   */
  public setCompareOperator(sOp: any) {
    if (this.oSearchColumn) {

      const input: any = document.getElementById(this.sElemId);

      let diff: string;
      try {
        const stringDiff = (diffMe: string, diffBy: string) => diffMe.split(diffBy).join('');
        diff = stringDiff(input.value, this.translate.instant(this.oSearchColumn.translateTitle) + ' ').trim();
      } catch (error) { }

      // Renseigne le nom de la colonne dans la valeur
      input.value = this.translate.instant(this.oSearchColumn.translateTitle) + ' ' + sOp + ' ';

      if (sOp === 'IS NULL' || sOp === 'IS NOT NULL') {
        this.setSearchValue('');
      } else if (diff) {
        this.setSearchValue(diff);
      } else {

        // Focus l'input
        input.focus();

        // Affiche la dropdown
        setTimeout(() => {
          input.click();
        });

        if (sOp === 'IN') {
          // Valeurs cochées pour IN
          this.checkedValuesSet.clear();
        }
      }
    }
  }

  /**
   * Valide la colonne de valeur
   * @param  {String} sSearchValue
   */
  public setSearchValue(sSearchValue: any) {

    if (this.vitisCheck.isNumber(sSearchValue)) {
      sSearchValue = String(sSearchValue);
    }

    if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn)) {
      if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.translateTitle)) {
        if (this.vitisCheck.isBoolean(sSearchValue)) {
          sSearchValue = sSearchValue.toString();
        }
        if (this.vitisCheck.isString(sSearchValue)) {

          const input: any = document.getElementById(this.sElemId);
          const sOperator = this.parseSearch_(input.value).operator;

          // Renseigne le nom de la colonne dans la valeur
          const sNewValue = this.translate.instant(this.oSearchColumn.translateTitle) + ' ' + sOperator + ' ' + sSearchValue;
          input.value = sNewValue;

          // Lance event de validation
          this.onValidSearchString(sNewValue);

          // Nettoie les listes après avoir fait un filtre
          this.clearLists_();
        }
      }
    }
  }

  /**
   * Force l'affichage de la dropdown de suggestions
   * @returns void
   */
  public showSuggestionsDropdown(): void {

    setTimeout(() => {

      const inputElem: any = $('#' + this.sElemId);
      const isOpen = inputElem.attr('aria-expanded') === 'true';

      if (!isOpen) {
        inputElem.dropdown('toggle');
      }
    });
  }

  /**
   * Utilisé pour les dates
   * @param iDate
   */
  public onDateSubmit(iDate: any) {

    const year = moment.unix(iDate).year();
    const month = moment.unix(iDate).month() + 1;
    const day = moment.unix(iDate).date();
    const hour = moment.unix(iDate).hour();
    const minute = moment.unix(iDate).minute();

    if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn)) {
      if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.translateTitle)
        && this.vitisCheck.isDefAndNotNull(this.oSearchColumn.field)) {

        const input: any = document.getElementById(this.sElemId);
        const sOperator = this.parseSearch_(input.value).operator;

        if (sOperator === '>' || sOperator === '<') {
          this.addFilterNode(this.oSearchColumn, sOperator,
            String(moment(`${year}-${month}-${day} 00:00:00`).utc().unix()));
        } else {
          if (hour === 0 &&
            minute === 0) {
            this.addFilterNode(this.oSearchColumn, '>=',
              String(moment(`${year}-${month}-${day} 00:00:00`).utc().unix()));
            this.addFilterNode(this.oSearchColumn, '<=',
              String(moment(`${year}-${month}-${day} 23:59:59`).utc().unix()));
          } else {
            this.addFilterNode(this.oSearchColumn, '>',
              String(moment(`${year}-${month}-${day} ${hour}:${minute}`, 'YYYY-MM-DD HH:mm').format('x')));
            this.addFilterNode(this.oSearchColumn, '<',
              String(moment(`${year}-${month}-${day} ${hour}:${minute + 1}`, 'YYYY-MM-DD HH:mm').format('x')));
          }
        }
      }
    }
  }

  /**
   * Donne aux lignes (en mode in) si elles sont check
   * @param value
   * @returns
   */
  public isCheckedValue(value: any) {
    return this.checkedValuesSet.has(value);
  }

  /**
   * En mode IN, permet de check une valeur
   * @param value
   */
  public checkValue(value: any) {
    if (this.isCheckedValue(value)) {
      this.checkedValuesSet.delete(value);
    } else {
      this.checkedValuesSet.add(value);
    }
    this.showSuggestionsDropdown();
  }

  /**
   * En mode IN permet de valider la sélection
   */
  public validateCheckedValues() {
    const input: any = document.getElementById(this.sElemId);
    this.onValidSearchString(input.value);
    this.clearLists_();
  }

  /**
   * Changement de l'opérateur
   * @param sOperator (AND / OR)
   * @returns
   */
  public changeRelationOperator(sOperator: string) {
    if (this.oFilter.relationOperator === sOperator) {
      return;
    }
    this.oFilter.relationOperator = sOperator;

    // Applique le filtre à la grille
    this.submit.emit();
  }

  /**
   * Retourne les colonnes qui matchent avec searchValue
   * @param  {any} searchValue
   * @returns Array
   */
  private getColumnSuggestions_(searchValue: any): Array<any> {

    const aSugestionColumns = [];

    if (this.vitisCheck.isString(searchValue)) {

      // Ajoute la liste des colonnes dans aSugestionColumns
      if (this.aColumns) {
        for (const column of this.aColumns) {

          // Titre
          column.translateTitle = column.translateTitle ?? column.title;
          let columnTitle = this.translate.instant(column.translateTitle);

          // Patch si le nom de la colonne est un chiffre
          if (!this.vitisCheck.isString(searchValue)) {
            searchValue = String(searchValue);
          }
          if (!this.vitisCheck.isString(columnTitle)) {
            columnTitle = String(columnTitle);
          }

          // Ajout de la colonne dans la liste aSugestionColumns
          if (this.vitisCheck.isString(columnTitle)) {

            // Si la colonne est recherchable
            if (column.searchable !== false) {

              // Si la recherche coïncide
              if (columnTitle.toLowerCase().search(searchValue.toLowerCase()) !== -1) {
                aSugestionColumns.push(column);
              }
            }
          }
        }
      }
    }

    return aSugestionColumns;
  }

  /**
   * Retourne les valeurs de la vue actuelle qui matchent avec searchValue
   * @param  {any} searchValue
   * @returns Array
   */
  private getValueSuggestions_(searchValue: any): Array<any> {

    let aSugestionValues = [];

    // Pas de suggestion si compare op vaut ~ ou si la colonne est de type nombre
    if (this.sCompareOperator === '~' ||
      (this.oSearchColumn.searchType === 'number' && !this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchSuggestions))) {
      return aSugestionValues;
    }

    if (this.vitisCheck.isString(searchValue)) {

      const sSearchValueValue = this.extractSearchValue_(searchValue);

      // Ajoute la liste des valeurs
      if (this.oSearchColumn) {

        // Type booléen
        if (this.oSearchColumn.searchType === 'boolean') {
          if (!this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchSuggestions)) {
            this.oSearchColumn.searchSuggestions = [{
              value: 'true',
              label: 'FORM_YES'
            }, {
              value: 'false',
              label: 'FORM_NO'
            }];
          }
        }

        // Type date
        if (this.oSearchColumn.searchType === 'date') {
          this.oSearchColumn.searchSuggestions = false;
          return aSugestionValues;
        }

        if (this.oSearchColumn.field) {
          if (this.oSearchColumn.searchSuggestions !== false) {
            if (this.vitisCheck.isArray(this.oSearchColumn.searchSuggestions)) {
              aSugestionValues = this.oSearchColumn.searchSuggestions;
            } else {
              if (this.vitisCheck.isDefAndNotNull(searchValue)) {
                if (this.oResults) {
                  if (this.oResults.data) {
                    for (let i = 0; i < this.oResults.data.length; i++) {
                      const oRow = this.oResults.data[i];

                      if (this.vitisCheck.isDefAndNotNull(oRow[this.oSearchColumn.field])) {

                        const sRowValue = oRow[this.oSearchColumn.field];
                        if (this.searchInString(sRowValue, sSearchValueValue)) {
                          if (aSugestionValues.indexOf(sRowValue) === -1) {
                            aSugestionValues.push(sRowValue);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }

          if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchWebService)) {
            if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchWebService.resource) &&
              this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchWebService.value)) {

              let oParams: any = {};
              if (this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchWebService.params)) {
                oParams = { ...this.oSearchColumn.searchWebService.params };
              }

              if (sSearchValueValue.length > 0) {
                const oFilter = {
                  column: this.oSearchColumn.searchWebService.label,
                  compare_operator: 'LIKE',
                  value: `%${sSearchValueValue}%`,
                  compare_operator_options: {
                    case_insensitive: true
                  }
                };
                if (oParams.filter) {
                  oParams.filter = {
                    relation: 'AND',
                    operators: [
                      oParams.filter,
                      oFilter
                    ]
                  };
                } else {
                  oParams.filter = oFilter;
                }
              }


              if (!this.vitisCheck.isDefAndNotNull(this.oSearchColumn.searchWebService.label)) {
                this.oSearchColumn.searchWebService.label = this.oSearchColumn.searchWebService.value;
              }

              // Évite de lancer la requête trop souvent
              const date = Date.now();
              this.iWsLastLoad = date;
              this.bWsLoading = true;
              setTimeout(() => {
                if (date === this.iWsLastLoad) {
                  aSugestionValues = [];
                  this.sendSearchWebServiceRequest_(oParams);
                }
              }, 500);
            }
          }

        }
      }
    }

    return aSugestionValues;
  }

  /**
   * Retourne les operateurs disponibles pour la recherche actuelle
   * @param  {any} searchValue
   * @returns Array
   */
  private getOperatorSuggestions_(searchValue: any): Array<any> {
    let operators = [];

    if (this.vitisCheck.isArray(this.oSearchColumn.searchOperators)) {
      return [...this.oSearchColumn.searchOperators];
    }

    if (this.vitisCheck.isArray(this.oSearchColumn.searchSuggestions)) {
      return ['='];
    }

    if (this.oSearchColumn) {
      switch (this.oSearchColumn.searchType) {
        case 'boolean':
        case 'strict':
          operators = ['='];
          break;
        case 'date':
        case 'number':
          operators = ['=', '>', '<', 'IS NULL', 'IS NOT NULL'];
          break;
        default:
          operators = [
            '~',
            '=',
            'IN',
            'IS NULL',
            'IS NOT NULL'];
          break;
      }
    }

    return operators;
  }

  /**
   * Récupère la colonne depuis ce qu'il y a écrit dans le champ de recherche
   * @param  {string} searchValue
   * @returns any
   */
  private extractSearchColumn_(searchValue: string): any {

    let oSearchColumn: any;

    if (this.vitisCheck.isString(searchValue)) {

      const oSearch = this.parseSearch_(searchValue);

      if (this.vitisCheck.isDefAndNotNull(oSearch)) {
        if (this.vitisCheck.isDefAndNotNull(oSearch.column)) {
          oSearchColumn = this.getColumnByTitle_(oSearch.column);
        }
      }
    }

    return oSearchColumn;
  }

  /**
   * Récupère l'opérateur depuis ce qu'il y a écrit dans le champ de recherche
   * @param  {string} searchValue
   * @returns any
   */
  private extractSearchOperator_(searchValue: string): any {
    let sReturn: any;

    if (this.vitisCheck.isString(searchValue)) {

      const oSearch = this.parseSearch_(searchValue);

      if (this.vitisCheck.isDefAndNotNull(oSearch)) {
        if (this.vitisCheck.isDefAndNotNull(oSearch.operator)) {
          sReturn = oSearch.operator;
        }
      }
    }

    return sReturn;
  }

  /**
   * Récupère la valeur depuis ce qu'il y a écrit dans le champ de recherche
   * @param  {string} searchValue
   * @param  {boolean} parsed si true alors envoie la version parsée (tableau etc.) sinon retourne en string
   * @returns any string ou array de string
   */
  private extractSearchValue_(searchValue: string, parsed = false): any {
    let sReturn: any;

    if (this.vitisCheck.isString(searchValue)) {

      const oSearch = this.parseSearch_(searchValue);

      if (this.vitisCheck.isDefAndNotNull(oSearch)) {
        if (this.vitisCheck.isDefAndNotNull(oSearch.value)) {
          sReturn = oSearch.value;
        }
      }
    }

    if (parsed && this.sCompareOperator === 'IN') {
      return Array.from(this.checkedValuesSet);
    }

    return sReturn;
  }

  /**
   * Parse la chaine de recherche et retourne un objet contenant column, operator, value
   * @param  {string} sSearch
   * @returns any
   */
  private parseSearch_(sSearch: string): any {

    let oReturn: any;
    let aSearchValue: any;
    let sOperator: any;
    let sColumn: String;
    let sValue: String;

    // Teste l'ensembles des opérateurs et extrait un tableau
    for (let i = 0; i < this.aCompareOperators.length; i++) {

      const tmpOperator = this.aCompareOperators[i];
      const tmpTest = sSearch.split(tmpOperator);

      if (tmpTest.length > 1) {
        aSearchValue = tmpTest;
        sOperator = this.aCompareOperators[i];
        sColumn = tmpTest[0];
        sValue = tmpTest[1];

        const aValue = [];
        for (let ii = 1; ii < tmpTest.length; ii++) {
          aValue.push(tmpTest[ii]);
        }
        sValue = aValue.join(this.aCompareOperators[i]);
        break;
      }
    }

    if (this.vitisCheck.isArray(aSearchValue)) {
      if (aSearchValue.length > 1) {
        oReturn = {
          column: sColumn.trim(),
          operator: sOperator,
          value: sValue.trim()
        };
      }
    }

    return oReturn;
  }

  /**
   * Recherche dans les colonnes du mode celle dont le titre correspond à sColumnTitle
   * @param  {String} sColumnTitle
   * @returns any
   */
  private getColumnByTitle_(sColumnTitle: String): any {

    if (this.aColumns) {

      // Pour chaque colonne
      for (let i = 0; i < this.aColumns.length; i++) {
        const column = this.aColumns[i];

        if (this.translate.instant(column.translateTitle).trim().toLowerCase() === sColumnTitle.trim().toLowerCase()) {
          return this.vitisClone.clone(column);
        }
      }
    }
  }

  /**
   * Recherche dans les colonnes du mode celle dont le nom correspond à sColumnField
   * @param  {String} sColumnTitle
   * @returns any
   */
  private getColumnByField_(sColumnField: String): any {
    if (this.aColumns) {
      for (let i = 0; i < this.aColumns.length; i++) {
        const column = this.aColumns[i];
        if (column.field === sColumnField) {
          return this.vitisClone.clone(column);
        }
      }
    }
  }

  /**
   * Nettoie les listes après avoir fait un filtre
   */
  private clearLists_() {
    this.aSugestionColumns.length = 0;
    this.aSugestionOperators.length = 0;
    this.aSugestionValues.length = 0;
    this.aWsSugestionValues.length = 0;
  }

  /**
   * Fonction de recherche côté front
   * @param sInput
   * @param sSearch
   * @returns
   */
  private searchInString(sInput: any, sSearch: any): boolean {
    // Patch si la valeur est un chiffre
    if (!this.vitisCheck.isString(sInput)) {
      sInput = String(sInput);
    }

    if (sInput.length > 0) {

      // Patch si la valeur est un chiffre
      if (!this.vitisCheck.isString(sSearch)) {
        sSearch = String(sSearch);
      }

      // Ajoute au tableau si il n'y est pas déjà
      if (sInput.toLowerCase().search(sSearch.toLowerCase()) !== -1) {
        return true;
      }
    }
    return false;
  }

  /**
   * Lance la requête qui permet de chercher des valeurs
   * @param oParams
   */
  private sendSearchWebServiceRequest_(oParams: any) {
    this.bWsLoading = true;
    this.iWsTotalRows = null;
    if (this.oSearchColumn && this.oSearchColumn.searchWebService) {
      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: this.oSearchColumn.searchWebService.resource.startsWith(environment.properties.app_api_url) ? this.oSearchColumn.searchWebService.resource : environment.properties.app_api_url + '/' + this.oSearchColumn.searchWebService.resource,
        ajaxLoader: false,
        params: oParams,
        success: (resp: any) => {
          this.bWsLoading = false;
          if (this.vitisCheck.isDefAndNotNull(resp.data.data)) {
            this.aSugestionValues = [];
            for (const row in resp.data.data) {
              if (resp.data.data[row]) {
                const element = resp.data.data[row];
                if (this.oSearchColumn && this.oSearchColumn.searchWebService) {
                  if (this.vitisCheck.isDefAndNotNull(element[this.oSearchColumn.searchWebService.value]) &&
                    this.vitisCheck.isDefAndNotNull(element[this.oSearchColumn.searchWebService.label])) {
                    this.aSugestionValues.push({
                      label: element[this.oSearchColumn.searchWebService.label],
                      value: element[this.oSearchColumn.searchWebService.value]
                    });
                  }
                }
              }
            }

            // mémorisation pour affichage de l'alias
            this.aWsSugestionValues = [...this.aSugestionValues];
            if (this.oSearchColumn) {
              this.oSearchColumn.searchSuggestions = [...this.aSugestionValues];
            }
            // Nombre total
            if (resp.data.total_row_number) {
              this.iWsTotalRows = resp.data.total_row_number;
            }
          }
        },
        error: (err: any) => {
          this.bWsLoading = false;
          console.error('error while loading values :', err);
          vitisAlert.alert({
            type: 'error',
            title: this.translate.instant('ERROR_REQUEST_FILTER'),
          });
        }
      });
    }
  }

  /**
   * Retourne la valeur (humaine) à afficher
   * @param oSearchColumn
   * @param sValue
   * @returns
   */
  private getDisplayedValue_(oSearchColumn: any, sValue: string) {
    let displayValue = sValue;
    if (oSearchColumn && oSearchColumn.searchWebService && this.aSugestionValues) {
      const oClickedRow = this.aSugestionValues.find(row => {
        return row.value === sValue;
      });
      if (oClickedRow) {
        displayValue = oClickedRow.label;
      }
    }
    return displayValue;
  }
}
