import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { RouterModule } from '@angular/router';
import { MonacoEditorModule } from 'ngx-monaco-editor';
import { TranslateModule } from '@ngx-translate/core';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { BreadcrumbComponent } from './components/breadcrumb/breadcrumb.component';
import { CustomfilterComponent } from './components/customfilter/customfilter.component';
import { UserDatePipe } from './filters/userDate/user-date.pipe';
import { StatisticsComponent } from './components/statistics/statistics.component';
import { NgChartsModule } from 'ng2-charts';
import { FileSizePipe } from './filters/filesize/filesize.pipe';
import { TreeviewComponent } from './components/treeview/treeview.component';
import { ModalHeaderComponent } from './components/modal-header/modal-header.component';
import { MessagePageComponent } from './components/message-page/message-page.component';
import { AppLogoComponent } from './components/app-logo/app-logo.component';
import { DatetimepickerComponent } from './components/datetimepicker/datetimepicker.component';
import { ClickOutsideDirective } from './directives/click-outside/click-outside.directive';
import { BannerComponent } from './components/banner/banner.component';
import { TooltipComponent } from './components/tooltip/tooltip.component';

/**
 * Permet le partage des composants à travers les modules
 *
 * Pour utiliser un composant dans plusieurs modules il faudra le mettre dans declarations et dans exports de SharedModule
 * ensuite AppModule et les autres modules importent SharedModule et les composants sont utilisables partout
 */
@NgModule({
  imports: [
    CommonModule,
    FormsModule,
    RouterModule,
    ReactiveFormsModule,
    TranslateModule,
    MonacoEditorModule,
    NgChartsModule,
    NgbModule,
  ],
  declarations: [
    BreadcrumbComponent,
    CustomfilterComponent,
    UserDatePipe,
    StatisticsComponent,
    FileSizePipe,
    TreeviewComponent,
    ModalHeaderComponent,
    MessagePageComponent,
    AppLogoComponent,
    DatetimepickerComponent,
    ClickOutsideDirective,
    BannerComponent,
    TooltipComponent,
  ],
  exports: [
    BreadcrumbComponent,
    CustomfilterComponent,
    UserDatePipe,
    StatisticsComponent,
    FileSizePipe,
    TreeviewComponent,
    ModalHeaderComponent,
    MessagePageComponent,
    AppLogoComponent,
    DatetimepickerComponent,
    ClickOutsideDirective,
    BannerComponent,
    TooltipComponent,
  ],
})
export class SharedModule { }
