import { Component, OnInit, AfterViewInit, Input, OnDestroy, ViewChild, ChangeDetectorRef } from '@angular/core';
import { StudioService } from '../../services/studio.service';
import { VitisCheck, vitisAlert, VitisRequest, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { TranslateService } from '@ngx-translate/core';
import { environment } from 'src/environments/environment';
import { NgForm } from '@angular/forms';
import { MapCoreStudioService } from 'src/app/vitis/map-core/services/map-core-studio/map-core-studio.service';
import { MapCoreUtilsService } from 'src/app/vitis/map-core/services/map-core-utils/map-core-utils.service';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { ActivatedRoute } from '@angular/router';
import { DataGridStudioService } from 'src/app/vitis/data-grid/services/data-grid-studio.service';

@Component({
  selector: 'app-attr-form',
  templateUrl: './attr-form.component.html',
  styleUrls: ['./attr-form.component.scss']
})
export class StudioAttrFormComponent implements OnInit, AfterViewInit, OnDestroy {

  @Input() public oEditedField: any;
  @Input() public sFormName_: any;
  @ViewChild('form', { static: true }) ngForm: NgForm;

  public oFieldTypeDef: any;
  public oFormDefinition: any;
  public aDatasourceIds: Array<string> = [];
  public aDatasourceColumns: Array<string> = [];
  public bDisplayForm = true;
  public sSelectedFormType: any;

  public bDatasourceParents: boolean = false;
  public aFormRows: any;  // Champs du formulaire (par ligne).
  public aDataSourceParents: any = [];  // Champs compatibles pour être des parents de listes.
  public aEditedFieldParents: any = [];

  private bTypeChanged = false;
  private oOriginalField: any;
  private fieldChangesSubscription: any;
  private formChangesSubscription: any;
  private getStudioFormNameSubscription: any;

  public sMapCenteringMode = 'C&Z';
  public mapSubscription: any;

  // VMAP
  public vmapObjects: Array<any> = [];
  private sRessouceId: string;
  public vmapFilterAttr: Array<any> = [];
  public vmapFilterValue: Array<any> = [];
  public vmapObjectColumns: Array<any> = [];
  public aVmapColumnTypeList: Array<any> = [
    { key: 'string', label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_TEXT_LABEL' },
    { key: 'number', label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_NUMBER_LABEL' },
    { key: 'image', label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_IMAGE_LABEL' },
    { key: 'document', label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_DOCUMENT_LABEL' },
    { key: 'link', label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_LINK_LABEL' },
    { key: 'date', label: 'STUDIO_COMPONENT_WEBSERVICE_GRID_TYPE_DATE_LABEL' }
  ];

  public iconList: Array<any>;
  public availableIcons: Array<any>;
  public emptyIcon: any = {
    name: '',
    css_class: '',
    tags: ['vide', 'empty'],
  };

  /**
   * Constructeur
   */
  constructor(
    private studioService: StudioService,
    private translate: TranslateService,
    private vitisCheck: VitisCheck,
    private vitisRequest: VitisRequest,
    private vitisClone: VitisClone,
    private mapCoreStudioService: MapCoreStudioService,
    private mapCoreUtilsService: MapCoreUtilsService,
    private formService: FormreaderService,
    protected changeDetectorRef: ChangeDetectorRef,
    private route: ActivatedRoute,
    private dataGridStudioService: DataGridStudioService,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {
    // Format par défaut pour le "color picker".
    if (this.oEditedField.type == 'color_picker' && typeof (this.oEditedField.options) == 'undefined') {
      this.oEditedField.options = {
        format: 'rgba'
      };
    }
    // Mémorise la définition originale
    this.oOriginalField = this.vitisClone.clone(this.oEditedField);

    // initialise this.oFieldTypeDef
    this.initFieldTypeDef();

    // url et paramètres par défaut pour le "data grid vmap".
    if (this.isLayerVmapGrid_()) {
      if (!this.vitisCheck.isDefAndNotNull(this.oEditedField.webservice_grid)) {
        this.oEditedField.webservice_grid = {
          params: {}
        };
      }
      if (this.vitisCheck.isUndefined(this.oEditedField.webservice_grid.url)) {
        this.oEditedField.webservice_grid.url = '/vmap/formvalues';
      }
      if (!this.vitisCheck.isDefAndNotNull(this.oEditedField.options) || this.vitisCheck.isEmptyObject(this.oEditedField.options)) {
        // Options par défaut
        this.oEditedField.options = {
          info: true
        };
      }
    }

    // Apellé si on change le type de champ où qu'on recharge le formulaire
    // Recharge la définition du type de champ
    this.fieldChangesSubscription = this.studioService.getStudioEditedField().subscribe((oEditedField) => {

      // Cache le formulaire
      this.bDisplayForm = false;

      // Spécifie que le type a changé
      this.bTypeChanged = true;

      // Attend que attr-list set this.oEditedField
      setTimeout(() => {

        // Recharge les tooltip
        this.studioService.initTooltip();

        // initialise this.oFieldTypeDef
        this.initFieldTypeDef();

        // charge les colonnes si datasource de type web_service
        this.onDatasourceChange();

        // Change le champ dans la prévisualisation si on est en edition
        if (this.oEditedField.studio_id_) {
          this.studioService.updateField(this.oEditedField, this.bTypeChanged, this.sFormName_);
        }

        // Affiche le formulaire
        this.bDisplayForm = true;

        // Récupère l'étendue, le centre et l'echelle après un mouvement sur la carte
        setTimeout(() => {
          this.handleMapExtentChange_();
        });
      });
    });

    // Définition du formulaire
    this.oFormDefinition = this.studioService.getStudioFormSync();

    // Sources de données
    if (this.vitisCheck.isDefAndNotNull(this.oFormDefinition) &&
      this.vitisCheck.isDefAndNotNull(this.oFormDefinition.datasources)) {
      this.aDatasourceIds = Object.keys(this.oFormDefinition.datasources);
    }

    // Detection changement de nom
    this.handleFormChanges_();

    // Récupère l'étendue, le centre et l'echelle après un mouvement sur la carte
    this.handleMapExtentChange_();

    // Appellé lorsqu'on change de datasource
    this.onDatasourceChange();

    // Changement de nom de formulaire (display, insert, update...)
    this.getStudioFormNameSubscription = this.studioService.getStudioFormName().subscribe((sFormName: string) => {
      this.cancelEdit();
    });

    // Changement de type de formulaire (default, custom, published...)
    this.sSelectedFormType = this.studioService.getStudioFormTypeSync();
  }

  /**
   * Appelé après le chargement du DOM
   */
  ngAfterViewInit() {
    // Activation de tous les tooltips.
    this.studioService.initTooltip();

    // Récupération des lignes du formulaire
    this.aFormRows = this.formService.getFormRows(this.oFormDefinition, this.sFormName_);
    this.aDataSourceParents = this.getDataSourceParents();

    // Pour l'element select, si il y a déjà un datasource parent on l'affiche
    if ((this.oEditedField.type === 'select' ||
      this.oEditedField.type === 'list' ||
      this.oEditedField.type === 'treeview-list' ||
      this.oEditedField.type === 'double_select' ||
      this.oEditedField.type === 'searchable_select') &&
      this.vitisCheck.isDefAndNotNull(this.oEditedField.datasource) &&
      this.vitisCheck.isDefAndNotNull(this.oEditedField.datasource.parents)) {
      this.bDatasourceParents = true;
      this.checkDataSourceParents(this.oEditedField.datasource, this.bDatasourceParents);
      this.changeDetectorRef.detectChanges();
    }

    // Déplace le conteneur des boutons d'édition (Annuler les modifs. et retour à la liste).
    document.querySelector('.studio-attributs-section').appendChild(document.querySelector('.edit-field-buttons'));
    // Déplace le conteneur du bouton d'édition (retour à la liste).
    document.querySelector('.studio-attributs-section').insertBefore(document.querySelector('.edit-field-return-button'),
      document.querySelector('.studio-attributs-section').children[0]);

    // Valeur par défaut à afficher pour les champs date et datetime.
    if ((this.oEditedField.type == 'date' || this.oEditedField.type == 'datetime')
      && this.vitisCheck.isDefAndNotNull(this.oEditedField.default_value)) {
      try {
        this.oEditedField.setValue(this.oEditedField.default_value);
      } catch (error) {
        console.error('Could not set value of datetimepicker', error);
      }
    }
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy() {
    this.fieldChangesSubscription.unsubscribe();
    this.formChangesSubscription.unsubscribe();
    this.getStudioFormNameSubscription.unsubscribe();
    if (this.isMapNeedSubscribe_() && this.vitisCheck.isDefAndNotNull(this.mapSubscription)) {
      this.mapSubscription.unsubscribe();
    }
    // Supprime le conteneur des boutons d'édition (Annuler les modifs. et retour à la liste).
    document.querySelector('.studio-attributs-section').removeChild(document.querySelector('.edit-field-buttons'));
    // Supprime le conteneur du bouton d'édition (retour à la liste).
    document.querySelector('.studio-attributs-section').removeChild(document.querySelector('.edit-field-return-button'));
  }

  /**
   * initialise this.oFieldTypeDef
   */
  public initFieldTypeDef() {
    if (this.oEditedField &&
      this.oEditedField.type) {
      this.oFieldTypeDef = this.studioService.getFieldTypeDef(this.oEditedField.type);

      // Trouve le premier des dynnamic
      let iDynamicIndex: any;
      for (let i = 0; i < this.oFieldTypeDef.attributes.length; i++) {
        if (this.oFieldTypeDef.attributes[i].startsWith('dynamic_')) {
          iDynamicIndex = i;
          break;
        }
      }
      if (iDynamicIndex &&
        this.oFieldTypeDef.attributes.indexOf('dynamic_fields_advanced_options') === -1) {
        this.oFieldTypeDef.attributes.splice(iDynamicIndex, 0, 'dynamic_fields_advanced_options');
      }

      this.studioService.getFieldList().then((aData) => {
        this.oFieldTypeDef.nameOptions = aData;
      });

      if (this.oEditedField.type == 'data_grid_vmap') {
        const oParams = this.route.snapshot.params;
        if (this.vitisCheck.isDefAndNotNull(oParams) && this.vitisCheck.isDefAndNotNull(oParams.id)) {
          this.sRessouceId = oParams.id;
        }
        if (this.sRessouceId) {
          if (!this.vitisCheck.isDefAndNotNull(this.vmapObjects) || this.vmapObjects.length < 1) {
            this.setVmapObjectsValues();
          }
        }
      }

      if (this.oEditedField.typeIcon && this.oEditedField.typeIcon === 'icon') {
        // Liste des icones disponibles
        this.getAvailableIcons_().then((availableIcons) => {
          this.availableIcons = availableIcons;
          this.iconList = this.getIconList_(this.availableIcons, '');
        });
      }
    }
  }

  /**
   * Annule l'édition
   */
  public cancelEdit() {

    // Enlève la selection et hover
    this.oOriginalField.studio_selected_ = false;
    this.oOriginalField.studio_hover_ = false;

    // Remet l'ancienne definition
    this.oEditedField = this.oOriginalField;

    // Met à jour le champ si on est en edition
    if (this.oOriginalField.studio_id_) {
      this.studioService.updateField(this.oOriginalField, true, this.sFormName_);
    }
    this.studioService.setStudioEditedField(null);
  }

  /**
   * Sauvegarde les modifications
   */
  public saveEditedField() {

    // Cas grille vmap
    if (this.isLayerVmapGrid_()) {
      if (!this.ngForm.valid) {
        this.translate.get(['STUDIO_ERROR_FORM_NOT_VALID']).subscribe((translations: any) => {
          vitisAlert.alert({
            type: 'error',
            text: translations['STUDIO_ERROR_FORM_NOT_VALID']
          });
        });
        return false;
      }

      if (!this.vitisCheck.isDefAndNotNull(this.oEditedField)
        || !this.vitisCheck.isDefAndNotNull(this.oEditedField.options)
        || !this.vitisCheck.isDefAndNotNull(this.oEditedField.options.columns)
        || this.oEditedField.options.columns.length < 1) {
        this.translate.get(['STUDIO_ERROR_EMPTY_COLUMNS']).subscribe((translations: any) => {
          vitisAlert.alert({
            type: 'error',
            text: translations['STUDIO_ERROR_EMPTY_COLUMNS']
          });
        });
        return false;
      }
    }

    // Paramètre 'name' automatique pour les champs qui n'ont pas le champ name.
    if (this.oFieldTypeDef.attributes.indexOf('name') == -1) {
      this.oEditedField.name = this.oFieldTypeDef.type + '_' + new Date().getTime();
    }

    // Contrôle du champ name pour les autres
    if (!this.vitisCheck.isDefAndNotNull(this.oEditedField.name) ||
      this.oEditedField.name === '') {
      this.translate.get(['STUDIO_ERROR_EMPTY_NAME']).subscribe((translations: any) => {
        vitisAlert.alert({
          type: 'error',
          text: translations['STUDIO_ERROR_EMPTY_NAME']
        });
      });
    } else if (!new RegExp(/^[A-Za-z0-9_]+$/).test(this.oEditedField.name)) {
      // Pas de caractères spéciaux dans le nom de l'attribut (sauf "_").
      this.translate.get(['STUDIO_ERROR_INVALID_NAME']).subscribe((translations: any) => {
        vitisAlert.alert({
          type: 'error',
          text: translations['STUDIO_ERROR_INVALID_NAME']
        });
      });
    } else {
      // Fichier de définition de la carte vMap obligatoire (mode création).
      if (this.oEditedField.type == 'map_vmap' && !this.vitisCheck.isDefAndNotNull(this.oEditedField.id)) {
        const oFileElem = document.getElementById('studio_element_form_vmap_downloader');
        if (this.vitisCheck.isDefAndNotNull(oFileElem) && oFileElem['files'].length == 0) {
          vitisAlert.alert({
            type: 'error',
            text: this.translate.instant('STUDIO_ERROR_MISSING_MAP_FILE')
          });
          return false;
        }
      }

      // Force le typage si la valeur par défaut est de type "booléen".
      if (this.oEditedField.default_value == 'true') {
        this.oEditedField.default_value = true;
      } else if (this.oEditedField.default_value == 'false') {
        this.oEditedField.default_value = false;
      }

      // Sort du mode édition
      this.studioService.setStudioEditedField(null);

      // Valeur par défaut à récupérer pour les champs date et datetime.
      if (this.oEditedField.type == 'date' || this.oEditedField.type == 'datetime') {
        try {
          this.oEditedField.default_value = this.oEditedField.getValue();
        } catch (error) {
          console.error('Could not get value of datetimepicker', error);
        }
      }

      // Nouveau champ
      if (!this.oEditedField.studio_id_) {
        this.studioService.addField(this.oEditedField);
      } else {
        this.studioService.updateField(this.oEditedField, this.bTypeChanged);
      }
    }
  }

  /**
   * Appellé lorsqu'on change de datasource
   */
  public onDatasourceChange() {

    if (this.oEditedField &&
      this.oEditedField.datasource &&
      this.oEditedField.datasource.datasource_id &&
      this.oFormDefinition.datasources) {

      const oDatasource = this.oFormDefinition.datasources[this.oEditedField.datasource.datasource_id];

      // set aDatasourceColumns
      this.aDatasourceColumns = [];
      this.aEditedFieldParents = [];
      if (oDatasource && oDatasource.type === 'web_service') {
        this.getWebServiceAttributs_(oDatasource).then((aAttrs: Array<string>) => {
          this.aDatasourceColumns = aAttrs;
        });
      }
    }
  }

  /**
   * Lance une requête avec limit 1 dans le but de récupérer les attributs
   * @param oDatasource
   */
  private getWebServiceAttributs_(oDatasource: any) {
    return new Promise((resolve, reject) => {
      if (oDatasource.type === 'web_service') {

        // Lance une requête avec limit 1
        this.vitisRequest.ajaxRequestPromise({
          method: 'GET',
          url: environment.properties.app_api_url + '/' + oDatasource.ressource_id,
          params: {
            ...oDatasource.parameters,
            ...{ limit: 1 }
          },
          timeout: 5000,
          headers: {
            Accept: 'application/x-vm-json'
          }
        }).then((resp: any) => {

          if (!this.vitisCheck.isDefAndNotNull(resp)) {
            console.error('resp not defined');
            reject();
            return;
          }
          if (!this.vitisCheck.isDefAndNotNull(resp.data)) {
            console.error('resp.data not defined');
            reject();
            return;
          }
          if (!this.vitisCheck.isDefAndNotNull(resp.data.data)) {
            console.error('resp.data.data not defined');
            reject();
            return;
          }
          if (!this.vitisCheck.isDefAndNotNull(resp.data.data[0])) {
            console.error('resp.data.data[0] not defined');
            reject();
            return;
          }

          let data: any;

          // si un sous objet est renseigné
          if (oDatasource['dataType'] === 'businessObject' && this.vitisCheck.isDefAndNotNull(oDatasource['businessObjectRequest'])) {
            data = [];
            for (let i = 0; i < resp.data.data.length; i++) {
              data.push(resp.data.data[i]['bo_' + oDatasource['businessObjectRequest']]);
            }
          } else {
            data = resp.data.data;
          }

          resolve(Object.keys(data[0]));
        });
      }
    });
  }

  /**
   * Détecte les changements de nom et rafraichit la définition du formulaire
   * permet entre autres de gérer les problèmes avec les onglets
   */
  private handleFormChanges_() {
    let sOldName: string = this.oOriginalField.name;
    let changing = 0;
    let changed = 0;

    this.formChangesSubscription = this.ngForm.form.valueChanges.subscribe(x => {

      // Options avant le changement
      let oldOptions = null;
      if (this.vitisCheck.isDefAndNotNull(this.oEditedField.options)) {
        try {
          oldOptions = JSON.stringify(this.oEditedField.options);
        } catch (error) { }
      }

      let sNewName = x.studio_element_form_name_input;
      changing++;

      setTimeout(() => {

        // Détection de modification d'options
        try {
          if (this.vitisCheck.isDefAndNotNull(oldOptions)) {
            if (oldOptions !== JSON.stringify(this.oEditedField.options)) {
              if (this.oEditedField.onOptionsChanges) {
                this.oEditedField.onOptionsChanges(this.oEditedField.options);
              }
            }
          }
        } catch (error) { }

        // Si pas de changements pendant 100ms
        changed++;
        if (changing === changed) {

          // Changement le name
          if (sNewName && sNewName !== sOldName) {

            // Vérifie que le nom soit unique
            for (let i = 0; i < this.aFormRows.length; i++) {
              for (let j = 0; j < this.aFormRows[i].fields.length; j++) {
                if (this.aFormRows[i].fields[j].studio_id_ != this.oEditedField.studio_id_ &&
                  this.aFormRows[i].fields[j].name === sNewName) {
                  sNewName = sNewName + '_' + new Date().getTime();
                  this.oEditedField.name = sNewName;
                }
              }
            }

            if (this.vitisCheck.isDefAndNotNull(sOldName)) {
              // Recharge le formulaire avec le nouveau nom
              const oNewForm = this.studioService.onChangeName(sOldName, sNewName);
              this.studioService.reloadStudioForm(oNewForm);
            }

            sOldName = sNewName;
          }
        }
      }, 100);
    });
  }

  /**
   * Ajoute une option dans un champ "radio".
   * @param  {any} aChoices
   */
  public addRadioOption(aChoices: any) {
    aChoices.push({
      'label': '',
      'value': ''
    });
  }

  /**
   * Supprime une option dans un champ "radio".
   * @param {any} aChoices
   * @param {number} iIndex
   */
  public removeRadioOption(aChoices: any, iIndex: number) {
    aChoices.splice(iIndex, 1);
  }

  /**
   * Récupère l'étendue, le centre et l'echelle après un mouvement sur la carte
   */
  private handleMapExtentChange_() {
    if (this.isMapNeedSubscribe_()) {
      this.mapSubscription = this.mapCoreStudioService.mapEventListner(this.oEditedField.studio_id_).subscribe((olMap) => {
        if (this.mapSubscription &&
          this.mapSubscription.closed === false) {

          const olView = olMap.getView();
          const aExtent = olView.calculateExtent();
          if (aExtent) {
            if (aExtent.length === 4) {
              this.oEditedField.map_options.center.extent = aExtent;
            }
          }
          const aCenter = olView.getCenter();
          if (aCenter) {
            this.oEditedField.map_options.center.coord = aCenter;
          }

          const scale = this.mapCoreUtilsService.getScale(olMap, { pretty: false });
          if (scale) {
            this.oEditedField.map_options.center.scale = scale;
          }
        }
      });
    }
  }

  /**
   * determine si il y a besoin de souscrire au mapEventListner
   * @returns {boolean}
   */
  private isMapNeedSubscribe_(): boolean {
    return this.vitisCheck.isDefAndNotNull(this.oEditedField) &&
      this.vitisCheck.isDefAndNotNull(this.oEditedField.type) &&
      this.oEditedField.studio_id_ && // cas update
      (this.oEditedField.type == 'map_osm' || this.oEditedField.type == 'map_bing' || this.oEditedField.type == 'map_vmap');
  }

  /**
   * Récupère le contenu du fichier chargé et l'attribut à l'arbre.
   * @param {Array<any>} files
   */
  public setFileValue(files: Array<any>) {
    // Si il y a un fichier, on enregistre le contenu du fichier dans l'arbre
    if (this.vitisCheck.isDefAndNotNull(files[0])) {
      const this_ = this;
      const reader = new FileReader();
      reader.onload = function (evt: any) {
        this_.oEditedField.map_options.tree = JSON.parse(evt.target.result);
        this_.updateMap('reload');
      };
      reader.readAsText(files[0]);
      // Si il n'y a pas de fichier mais qu'il y a une valeur d'origine, on réattribue la valeur d'origine
    } else if (this.vitisCheck.isDefAndNotNull(this.oOriginalField.map_options)
      && this.vitisCheck.isDefAndNotNull(this.oOriginalField.map_options.tree)) {
      this.oEditedField.map_options.tree = this.oOriginalField.map_options.tree;
      this.updateMap('reload');
    } else {
      // Sinon on vide la valeur
      this.oEditedField.map_options.tree = [];
      this.updateMap('reload');
    }
  }

  /**
   * Ajout les px au changement de hauteur
   */
  public onHeightChange() {
    this.oEditedField.style.height = this.oEditedField.style.height + 'px';
  }

  /**
   * Modifie les informations convernant le datasource
   */
  public updateDatasourceInfos() {
    if (!this.vitisCheck.isDefAndNotNull(this.oEditedField.datasource)) {
      return;
    }

    const oDatasource = this.oEditedField.datasource;
    // Valeurs par défaut
    oDatasource.sort_order = this.vitisCheck.isDefAndNotNull(oDatasource.sort_order) ? oDatasource.sort_order : 'ASC';
    oDatasource.distinct = this.vitisCheck.isDefAndNotNull(oDatasource.distinct) ? oDatasource.distinct : 'true';

    // Enlève les espaces
    if (this.vitisCheck.isDefAndNotNull(oDatasource.id_key)) {
      oDatasource.id_key = oDatasource.id_key.replace(/\s/g, '');
    }
    if (this.vitisCheck.isDefAndNotNull(oDatasource.label_key)) {
      oDatasource.label_key = oDatasource.label_key.replace(/\s/g, '');
    }

    // Place les nouveaux paramètres
    if (this.vitisCheck.isDefAndNotNull(oDatasource.id_key) && this.vitisCheck.isDefAndNotNull(oDatasource.label_key)) {
      oDatasource.attributs = oDatasource.id_key + '|' + oDatasource.label_key;
    }
    if (this.vitisCheck.isDefAndNotNull(oDatasource.label_key)) {
      oDatasource.order_by = oDatasource.label_key;
    }

  }

  /**
   * Met à jour la carte après une modification sur le formulaire
   * @param {string} refreshType
   */
  public updateMap(refreshType: string) {
    if (this.vitisCheck.isDefAndNotNull(this.oEditedField) && this.vitisCheck.isDefAndNotNull(this.oEditedField.studio_id_)) {
      this.mapCoreStudioService.emitMapOptionsEvent(this.oEditedField.studio_id_, refreshType);
    }
  }

  /**
   * Vérifie le paramètre "parents" d'une source de données.
   * @param  {any} oDataSource
   * @param  {boolean} bUseParents
   * @returns void
   */
  public checkDataSourceParents(oDataSource: any, bUseParents: boolean) {
    if (bUseParents) {
      if (!this.vitisCheck.isDefAndNotNull(oDataSource.parents)) {
        oDataSource.parents = [];
      } else {
        for (let i = 0; i < oDataSource.parents.length; i++) {
          this.aEditedFieldParents.push(oDataSource.parents[i].name);
        }
      }
    } else if (!bUseParents) {
      delete oDataSource.parents;
    }
  }

  /**
   * Retourne la liste des parents possibles pour une source de données.
   * @returns any
   */
  public getDataSourceParents() {
    const aParentFields = [];
    for (let i = 0; i < this.aFormRows.length; i++) {
      for (let j = 0; j < this.aFormRows[i].fields.length; j++) {
        if (this.aFormRows[i].fields[j].type == 'select' ||
          this.aFormRows[i].fields[j].type == 'list' ||
          this.aFormRows[i].fields[j].type == 'treeview-list' ||
          this.aFormRows[i].fields[j].type == 'double_select' ||
          this.aFormRows[i].fields[j].type == 'searchable_select') {
          if (this.oEditedField.name !== this.aFormRows[i].fields[j].name) {
            aParentFields.push(this.aFormRows[i].fields[j]);
          }
        }
      }
    }
    return aParentFields;
  }

  /**
   * Définition du paramètre "parents" de la source de donnée suivant les parents sélectionnés.
   * @param  {any} aDataSouceParents
   * @param  {any} aSelectedParents
   * @returns void
   */
  public setDataSourceParents(aDataSouceParents: any, aSelectedParents: any) {
    const aParents = [];
    let bParentExist: boolean;
    for (let i = 0; i < aSelectedParents.length; i++) {
      bParentExist = false;
      if (this.vitisCheck.isDefAndNotNull(aDataSouceParents)) {
        for (let j = 0; j < aDataSouceParents.length; j++) {
          // Sauve le parent déja existant.
          if (aDataSouceParents[j].name == aSelectedParents[i]) {
            bParentExist = true;
            aParents.push(aDataSouceParents[j]);
            break;
          }
        }
      }
      // Ajoute le nouveau parent sélectionné.
      if (!bParentExist) {
        aParents.push({
          name: aSelectedParents[i],
          filter_attr: '',
          filter_equality: '=',
          wait_for_parent: true
        });
      }
    }
    aDataSouceParents = aParents;
    return aParents;
  }

  /**
   * Ajoute ou supprime le tooltip
   */
  public addOrRemoveTooltip() {
    if (this.vitisCheck.isDefAndNotNull(this.oEditedField.tooltip)) {
      delete this.oEditedField.tooltip;
      this.updateTooltip();
    } else {
      this.oEditedField.tooltip = {
        container: 'body'
      };
    }
  }

  /**
   * met à jour le tooltip
   */
  public updateTooltip() {
    if (this.oEditedField.studio_id_) {
      if (this.vitisCheck.isFunction(this.oEditedField.refreshTooltip)) {
        this.oEditedField.refreshTooltip();
      } else {
        console.error('refreshTooltip function doesn\'t exist on type: ', this.oEditedField.type);
      }
    }
  }

  /**
   * Récupère la liste des couches
   */
  public setVmapObjectsValues() {
    if (this.vitisCheck.isDefAndNotNull(this.sRessouceId)) {
      const sUrl = `${environment.properties.app_api_url}/vmap/datasourcetreeview/layers/${this.sRessouceId}/datasourcelayer`;
      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        params: {
          order_by: 'name',
          attributs: 'layer_id|name'
        },
        method: 'GET'
      }).then((resp: any) => {
        if (resp.data && resp.data.status === 1) {
          this.vmapObjects = resp.data.data;
        }
      });
    }
  }

  /**
   * Récupère les listes des colonnes
   * @param onlyGetObjectColumnsTable
   */
  public setVmapObjectsColumnsValues(onlyGetObjectColumnsTable: boolean = false) {
    if (this.vitisCheck.isDefAndNotNull(this.sRessouceId)) {
      if (!onlyGetObjectColumnsTable) {
        this.getVmapColumns_(this.sRessouceId).then((vmapFilterValue) => {
          this.vmapFilterValue = vmapFilterValue;
        });
      }
      this.getVmapColumns_(this.oEditedField.webservice_grid.params.layer_id).then((vmapFilterAttr) => {
        if (!onlyGetObjectColumnsTable) {
          this.vmapFilterAttr = vmapFilterAttr;
        }
        this.vmapObjectColumns = vmapFilterAttr;
      });
    } else {
      console.error('setVmapObjectsColumnsValues ressourceId is missing');
    }
  }

  /**
   * Récupère les colonnes de la table liée a une couche
   * @param id Identifiant de la couche
   */
  private getVmapColumns_(id: string): Promise<any> {
    return new Promise((resolve, reject) => {
      const sUrl = `${environment.properties.app_api_url}/vmap/pglayers/${id}/columns`;
      this.vitisRequest.ajaxRequestPromise({
        url: sUrl,
        method: 'GET'
      }).then((resp: any) => {
        if (resp.data && resp.data.status === 1) {
          resolve(resp.data.data);
        }
      });
    });
  }

  /**
   * Récupère l'emplacement de la colonne passé en paramètre (issue de la base) dans les options.columns
   * @param oColumn
   */
  public getVmapColumnIndex(oColumn: any) {
    // Aucune colonne existante
    if (!this.vitisCheck.isDefAndNotNull(this.oEditedField)
      || !this.vitisCheck.isDefAndNotNull(this.oEditedField.options)
      || !this.vitisCheck.isDefAndNotNull(this.oEditedField.options.columns)
      || this.oEditedField.options.columns.length < 1) {
      return -1;
    }

    return this.oEditedField.options.columns.findIndex((dataGridColumn: any) => dataGridColumn.data === oColumn.name);
  }

  /**
   * Evenement quand une checkbox est coché/décoché
   * @param eventTarget
   * @param oColumn
   */
  public vmapCheckColumn(eventTarget: any, oColumn: any) {
    if (eventTarget.checked) {
      if (!this.vitisCheck.isDefAndNotNull(this.oEditedField.options.columns)) {
        this.oEditedField.options.columns = [];
      }

      // Ajout de la colonne
      this.oEditedField.options.columns.push({
        data: oColumn.name,
        title: oColumn.name,
        index: this.oEditedField.options.columns.length,
        type: 'string'
      });


    } else {
      // Si la colonne existe on l'enlève
      if (this.getVmapColumnIndex(oColumn) !== -1) {
        this.oEditedField.options.columns.splice(this.oEditedField.options.columns.findIndex((dataGridColumn: any) => dataGridColumn.data === oColumn.name), 1);
      }
    }
  }

  /**
   * Retourne true si le champ édité est de type data_grid_vmap
   */
  private isLayerVmapGrid_() {
    return this.oEditedField.type == 'data_grid_vmap';
  }

  /**
   * Met à jour la grille après une modification sur le formulaire
   */
  public updateDataGrid() {
    if (this.vitisCheck.isDefAndNotNull(this.oEditedField) && this.vitisCheck.isDefAndNotNull(this.oEditedField.studio_id_)) {
      this.dataGridStudioService.emitDataGridOptionsEvent(this.oEditedField.studio_id_);
    }
  }

  /**
   * Vide les valeurs de lien et les colonnes
   */
  public emptyLinkAndColumns() {
    this.oEditedField.webservice_grid.params.filter_attr = '';
    this.oEditedField.webservice_grid.params.filter_value = '';
    this.oEditedField.options.columns = [];
  }

  /**
   * Télécharge la liste des icones disponibles
   */
  private getAvailableIcons_(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url: environment.properties.app_api_url + '/vitis/icons',
      }).then((resp: any) => {
        if (resp.data &&
          resp.data.data &&
          resp.data.data) {

          const aIcons = [this.emptyIcon, ...resp.data.data];
          for (let i = 0; i < aIcons.length; i++) {
            const icon = aIcons[i];
            try {
              icon.tags = JSON.parse(icon.tags);
            } catch (error) {
              icon.tags = [];
            }
          }

          resolve(aIcons);
        } else {
          console.error('could not get userorders', resp);
          reject();
        }
      }).catch((err) => {
        console.error('could not get userorders', err);
        reject();
      });
    });
  }

  /**
   * Retourne la liste filtrée
   * @param availableIcons
   * @param sFilter
   */
  public getIconList_(availableIcons: Array<any>, sFilter: string): Array<any> {
    if (!availableIcons || !sFilter || sFilter === '') {
      return availableIcons;
    }
    return availableIcons.filter(icon =>
      (icon.name.indexOf(sFilter.toLowerCase().normalize('NFD').replace(/[\u0300-\u036f]/g, '')) !== -1) ||
      (icon.tags.find((tag: string) => tag.indexOf(sFilter.toLowerCase().normalize('NFD').replace(/[\u0300-\u036f]/g, '')) !== -1)));
  }

  /**
   * Sélectionne l'icon dans la liste des icons font-awesome
   * @param icon
   */
  public selectIcon(icon) {
    this.oEditedField.icon = icon.css_class;
  }

  /**
   * Event lors de la modification du champ de filtre pour les icons
   * @param e
   */
  public updateFilterIcon(e) {
    this.iconList = this.getIconList_(this.availableIcons, e.target.value);
  }

  /**
   * Event appelé lors de l'upload de l'image pour la sauvegarde de l'url
   * @param oFiles
   */
  public uploadImageFile(oFiles) {
    const oFile = oFiles[0];
    if (this.vitisCheck.isDefAndNotNull(oFile.src)) {
      this.oEditedField.url = oFile.src;
    }
  }
}
