import { Component, OnInit, Input } from '@angular/core';
import { fieldsDef } from '../attr-form/attr-form-fields';
import { StudioService } from '../../services/studio.service';
import { TranslateService } from '@ngx-translate/core';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

@Component({
  selector: 'app-attr-type-list',
  templateUrl: './attr-type-list.component.html',
  styleUrls: ['./attr-type-list.component.scss']
})
export class StudioAttrTypeListComponent implements OnInit {

  @Input() public oEditedField: any;

  public fieldsDef = fieldsDef;
  public sSearchFilter: string = '';
  public aFilteredFieldsDef: any = [];
  public attrFormFieldsList: Array<string>;

  /**
   * Constructeur
   */
  constructor(
    private studioService: StudioService,
    private translate: TranslateService,
    private vitisCheck: VitisCheck,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    this.studioService.getAttrFormFieldsList().then((attrFormFieldsList?: Array<string>) => {
      if (!this.vitisCheck.isDefAndNotNull(attrFormFieldsList)) {
        console.error('Missing option: attr_form_fields_list');
      } else {
        this.attrFormFieldsList = attrFormFieldsList;

        // Liste des catégories et champs non filtrés.
        this.aFilteredFieldsDef = this.filterFieldsDef(this.fieldsDef);
      }

    });

  }

  /**
   * Retourne la définition d'un type de champ
   * @param  {string} sFieldType
   * @returns {any}
   */
  public getFieldTypeDef(sFieldType: string): any {
    return this.studioService.getFieldTypeDef(sFieldType);
  }

  /**
   * Filtrage des types de champs de chaque catégorie.
   * @param  {any} aFieldsDef
   * @param  {string} sSearchFilter
   * @returns {any}
   */
  public filterFieldsDef(aFieldsDef: any, sSearchFilter: string = ''): any {
    if (sSearchFilter == '') {
      return aFieldsDef;
    } else {
      const aFilteredFieldsDef = [];
      let oCategory: any;
      let sTranslation: string;
      for (let i = 0; i < aFieldsDef.length; i++) {
        oCategory = {};
        for (let j = 0; j < aFieldsDef[i].fields.length; j++) {
          sTranslation = this.translate.instant(aFieldsDef[i].fields[j].label).toLocaleLowerCase();
          if (sTranslation.indexOf(sSearchFilter.toLocaleLowerCase()) != -1) {
            if (Object.keys(oCategory).length == 0) {
              oCategory = JSON.parse(JSON.stringify(aFieldsDef[i]));
              oCategory.fields.length = 0;
            }
            oCategory.fields.push(aFieldsDef[i].fields[j]);
          }
        }
        if (Object.keys(oCategory).length > 0) {
          aFilteredFieldsDef.push(oCategory);
        }
      }
      return aFilteredFieldsDef;
    }
  }

  /**
   * Change le type de champ
   * @param sType
   */
  public selectType(sType: string) {
    if (this.oEditedField) {
      const oNewField = this.studioService.changeFieldType(this.oEditedField, sType);
      if (oNewField) {
        this.studioService.setStudioEditedField(oNewField);
      }
    } else {
      const oFieldDef = this.getFieldTypeDef(sType);
      if (oFieldDef &&
        oFieldDef.definition) {
        const oNewField = oFieldDef.definition;
        oNewField.nb_cols = 12;
        this.studioService.setStudioEditedField(oNewField);
      }
    }
  }

  /**
   * Vérifie si le champs est actif
   * @param oField
   */
  public isActiveField(oField: any): boolean {
    return (oField.disabled !== true
      && this.vitisCheck.isDefAndNotNull(this.attrFormFieldsList)
      && this.attrFormFieldsList.indexOf(oField.type) != -1);
  }

  /**
   * Vérifie si la catégorie est vide (aucun champ actif).
   * @param oCategory
   */
  public isEmptyCategory(oCategory: any): boolean {
    for (let i = 0; i < oCategory.fields.length; i++) {
      if (this.isActiveField(oCategory.fields[i])) {
        return false;
      }
    }
    return true;
  }
}
