import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { VitisCheck, VitisClone } from 'src/app/vitis/core/services/utils/vitis-utils';
import { FormreaderService } from 'src/app/vitis/formreader/services/formreader.service';
import { StudioService } from '../../services/studio.service';

@Component({
  selector: 'app-field-list-modal',
  templateUrl: './field-list-modal.component.html',
  styleUrls: ['./field-list-modal.component.scss']
})
export class FieldListModalComponent implements OnInit {

  public oMode: any;
  public oModalOptions: any;
  private oFormDefinition: any;
  private oInitialFormDatas: any;

  /**
   * Constructeur
   */
  constructor(
    private route: ActivatedRoute,
    private studioService: StudioService,
    private formReaderService: FormreaderService,
    private vitisCheck: VitisCheck,
    private vitisClone: VitisClone,
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {

    // récupération du mode
    this.route.data.subscribe(oMode => {
      this.oMode = oMode;
    });

    // Permet de récupérer les options de la modale
    this.studioService.getStudioFieldListModalOptions().subscribe((oModalOptions) => {
      this.oModalOptions = oModalOptions;
      this.oInitialFormDatas = this.vitisClone.clone(this.oModalOptions.formFieldList);
    });

    // Observe la modification du formulaire
    this.studioService.getStudioForm().subscribe((oFormDefinition: any) => {
      this.oFormDefinition = oFormDefinition;
    });
  }

  /**
   * Selectionne tout les champs pour un type donné
   * @param sFormName
   */
  public selectFields(sFormName: string) {
    for (let i = 0; i < this.oModalOptions.fieldList.length; i++) {
      const object = this.oModalOptions.fieldList[i];
      if (object.checked.indexOf(sFormName) === -1) {
        object.checked.push(sFormName);
      }
    }
  }

  /**
   * Déselectionne tout les champs pour un type donné
   * @param sFormName
   */
  public unselectFields(sFormName: string) {
    for (let i = 0; i < this.oModalOptions.fieldList.length; i++) {
      const object = this.oModalOptions.fieldList[i];
      const index = object.checked.indexOf(sFormName);
      if (index !== -1) {
        object.checked.splice(index, 1);
      }
    }
  }

  /**
   * Met à jour un champs pour un type donné
   * @param object
   * @param sFormName
   */
  public updateSelection(object: any, sFormName: string) {
    const index = object.checked.indexOf(sFormName);
    if (index === -1) {
      object.checked.push(sFormName);
    } else {
      object.checked.splice(index, 1);
    }
  }

  /**
   * Selectionne tous les types pour un champs donné
   * @param object
   */
  public selectTypes(object: any, isDbObject: boolean = true) {
    // Clone pour que les options ne disparaissent pas quand on décoche une case
    if (isDbObject) {
      object.checked = this.vitisClone.clone(this.oMode.options.available_form_names);
    } else {
      object.checked = this.vitisClone.clone(this.oInitialFormDatas[object.name].checked);
    }
  }

  /**
   * Déselectionne tous les types pour un champs donné
   * @param object
   */
  public unselectTypes(object: any) {
    object.checked = [];
  }

  /**
   * Ferme le formulaire sans enregistrer, par contre si le formulaire n'existe pas on en créer un vide
   */
  public cancelUpdateForm() {
    if (!this.vitisCheck.isDefAndNotNull(this.oFormDefinition) || this.oModalOptions.init) {
      this.studioService.useEmptyForm();
    }
  }

  /**
   * Lance la mise à jour du formulaire en fonction du resultat de la modal
   * Si oFormDefinition n'existe pas, on en créer un vide avant de le mettre à jour
   */
  public updateForm() {
    let oNewForm = this.vitisClone.clone(this.oFormDefinition);
    // Si le formulaire n'existe pas ou si on est en mode init on utilise un formulaire vide
    if (!this.vitisCheck.isDefAndNotNull(oNewForm) || this.oModalOptions.init) {
      this.studioService.useEmptyForm().then((result: boolean) => {
        if (result === true) {
          oNewForm = this.vitisClone.clone(this.oFormDefinition);
          this.updateForm_(oNewForm);
        }
      });
    } else {
      this.updateForm_(oNewForm);
    }
  }

  /**
   * Ajoute et/ou supprime des champs en fonction du résultat de la modale
   * @param oNewForm
   */
  private updateForm_(oNewForm: any) {

    // Parcourt la liste des champs récupéré par le webservice
    for (let i = 0; i < this.oModalOptions.fieldList.length; i++) {
      const oField = this.oModalOptions.fieldList[i];
      // Parcourt le type de formulaire valide
      for (let j = 0; j < this.oMode.options.available_form_names.length; j++) {

        const sFormName = this.oMode.options.available_form_names[j];
        const aFormRows = this.formReaderService.getFormRows(oNewForm, sFormName);
        const iRowIndex = this.studioService.findRowIndexWithName(aFormRows, oField);
        const iFieldIndex = this.studioService.findFieldIndexWithName(aFormRows, oField, iRowIndex);

        // Si le champ a été coché dans la modal
        if (oField.checked.indexOf(sFormName) !== -1) {
          // Si le champ n'existe pas dans oFormDefinition, on l'ajoute
          if (iRowIndex === -1) {
            const oAddedField = this.vitisClone.clone(oField);
            if (sFormName === 'display') {
              oAddedField.type = 'label';
            }
            delete oAddedField.checked;
            delete oAddedField.originalLabel;
            // Ajoute dans les lignes
            oNewForm[sFormName].rows.push({
              fields: [oAddedField]
            });

            // Ajoute dans le premier onglets
            if (oNewForm[sFormName] &&
              oNewForm[sFormName].tabs &&
              oNewForm[sFormName].tabs.list &&
              oNewForm[sFormName].tabs.list[0] &&
              oNewForm[sFormName].tabs.list[0].elements) {
              oNewForm[sFormName].tabs.list[0].elements.push(oField.name);
            }
          } else if (oField.originalLabel !== oField.label) { // Si il existe et que le label a été modifié
            oNewForm[sFormName].rows[iRowIndex].fields[iFieldIndex].label = oField.label;
          }

        // Si le champ existe dans oFormDefinition et qu'il a été décoché dans la modale
        // On le supprime
        } else if (iRowIndex !== -1 && oField.checked.indexOf(sFormName) === -1) {
          oNewForm = this.removeFieldInNewForm_(oNewForm, oField, sFormName, aFormRows, iRowIndex, iFieldIndex);
        }
      }
    }

    // Parcourt la liste des champs présent dans le formulaire mais pas dans le webservice
    for (const sFieldName in this.oModalOptions.formFieldList) {
      if (Object.prototype.hasOwnProperty.call(this.oModalOptions.formFieldList, sFieldName)) {
        const oField = this.oModalOptions.formFieldList[sFieldName];
        // Parcourt le type de formulaire où était présent les champs
        for (let j = 0; j < this.oInitialFormDatas[oField.name].checked.length; j++) {
          const sFormName = this.oInitialFormDatas[oField.name].checked[j];
          const aFormRows = this.formReaderService.getFormRows(oNewForm, sFormName);
          const iRowIndex = this.studioService.findRowIndexWithName(aFormRows, oField);
          const iFieldIndex = this.studioService.findFieldIndexWithName(aFormRows, oField, iRowIndex);

          // Si le champs existe dans oFormDefinition et qu'il a été décoché dans la modal
          // On le supprime
          if (iRowIndex !== -1 && oField.checked.indexOf(sFormName) === -1) {
            oNewForm = this.removeFieldInNewForm_(oNewForm, oField, sFormName, aFormRows, iRowIndex, iFieldIndex);
          }
        }
      }
    }

    // Ajout des boutons d'actions à tous les modes du formulaire (onglets de type 'container').
    if (this.vitisCheck.isDefAndNotNull(this.studioService.oStudioOptions_.generate_action_buttons)
    &&  this.studioService.oStudioOptions_.generate_action_buttons === true) {
      for (let i = 0; i < this.oMode.options.available_form_names.length; i++) {
        const sFormName = this.oMode.options.available_form_names[i];
        if (sFormName != 'search') {
          oNewForm[sFormName].rows.push(this.getFormActionButtons(sFormName));
          oNewForm[sFormName].tabs.list.push(this.studioService.getFormActionButtonsTab(sFormName));
        }
      }
    }

    // Met à jour le formulaire
    if (oNewForm !== this.oFormDefinition) {
      this.studioService.setStudioIsBeingModified(true);
      this.studioService.setStudioForm(oNewForm);
      this.studioService.reloadStudioForm(oNewForm);
      this.studioService.setStudioIsBeingModified(true);

      if (this.oModalOptions.init === true) {
        this.saveInitialeForm_();
      }
    }
  }

  /**
   * Envoi le formulaire avec param init pour sauvegarder tous les types
   */
  private saveInitialeForm_() {
    // Envoi le formulaire avec param init pour sauvegarder tous les types
    this.studioService.ajaxSaveStudioForm('init').then((oForm: any) => {
      this.studioService.reloadStudioForm(oForm);
    });
  }

  /**
   * Décode ou coche toutes les cases du second tableau
   * @param event
   */
  public updateAllSelection(event: any) {
    if (event.target.checked) {
      for (const sName in this.oModalOptions.formFieldList) {
        if (Object.prototype.hasOwnProperty.call(this.oModalOptions.formFieldList, sName)) {
          this.oModalOptions.formFieldList[sName].checked = this.vitisClone.clone(this.oInitialFormDatas[sName].checked);
        }
      }
    } else {
      for (const sName in this.oModalOptions.formFieldList) {
        if (Object.prototype.hasOwnProperty.call(this.oModalOptions.formFieldList, sName)) {
          this.oModalOptions.formFieldList[sName].checked = [];
        }
      }
    }
  }

  /**
   * Supprime un element d'un formulaire
   * @param oNewForm
   * @param oField
   * @param sFormName
   * @param aFormRows
   * @param iRowIndex
   * @param iFieldIndex
   */
  private removeFieldInNewForm_(oNewForm: any, oField: any, sFormName: string, aFormRows: Array<any>, iRowIndex: number, iFieldIndex: number) {

    let removeRow = false;
    const oRow = aFormRows[iRowIndex];

    if (this.vitisCheck.isArray(oRow.fields)) {
      if (oRow.fields.length == 1) {
        removeRow = true;
      }

      // Supprime oField de la ligne
      oRow.fields.splice(iFieldIndex, 1);

      // Supprime la ligne
      if (removeRow) {
        aFormRows.splice(iRowIndex, 1);
      }
      // Supprime des tabs
      this.studioService.removeElementFromTabs(oNewForm[sFormName].tabs, oField.name);

      if (oNewForm[sFormName] && oNewForm[sFormName].rows) {
        oNewForm[sFormName].rows = aFormRows;
      }
    }

    return oNewForm;
  }

  /**
   * Vérifie si il y a des données dans la formList (2eme tableau)
   */
  public isFilledFormList() {
    return !this.vitisCheck.isEmptyObject(this.oModalOptions.formFieldList);
  }

  /**
   * Retourne les boutons d'actions suivant le mode du formulaire.
   * @param {string} sFormMode
   * @returns {any}
   */
  private getFormActionButtons(sFormMode: string): any {
    // Définition des boutons.
    const aFormActionButtons: any = [{
      type: 'submit',
      name: 'form_submit',
      label: (sFormMode == 'insert') ? 'FORM_CREATE' : 'FORM_UPDATE',
      class: 'btn-primary'
    }, {
      type: 'button',
      name: 'return_list',
      label: 'FORM_RETURN_LIST',
      class: 'btn-default',
      event: 'setMode("search")'
    }];

    // Ajout des boutons suivant le mode du formulaire.
    let aButtons: Array<any> = [];
    if (sFormMode == 'insert' || sFormMode == 'update') {
      aButtons = aFormActionButtons;
    } else {
      aButtons.push(aFormActionButtons[1]);
    }
    return ({
      fields: [{
        type: 'button',
        class: 'btn-ungroup btn-group-sm',
        nb_cols: 12,
        buttons: aButtons
      }]
    });
  }
}
