import { Injectable } from '@angular/core';
import { StudioOptions } from '../studio.model';
import { Subject } from 'rxjs';
import { VitisRequest, VitisCheck, VitisClone, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { FormreaderService } from '../../formreader/services/formreader.service';
import { fieldsDef } from '../components/attr-form/attr-form-fields';
import { environment } from 'src/environments/environment';
import { search as jmespathSearch } from 'jmespath';
import { first } from 'rxjs/operators';
import { TranslateService } from '@ngx-translate/core';


declare const $: any;

@Injectable({
  providedIn: 'root'
})
export class StudioService {

  // Observables
  public obsStudioOptions = new Subject<StudioOptions>();
  public obsStudioForm = new Subject<any>();
  public obsStudioFormType = new Subject<any>();
  public obsStudioFormName = new Subject<any>();
  public obsStudioEditedField = new Subject<any>();
  public obsStudioJsField = new Subject<any>();
  public obsStudioCssField = new Subject<any>();
  public obsStudioIsBeingModified = new Subject<boolean>();
  public obsStudioFieldListModalOptions = new Subject<any>();

  // Listernner permettant de relancer l'initialisation du formulaire
  public obsStudioReloadFormListerner = new Subject<any>();

  // Variables fixes
  private sAPIRessource_: string;
  private sFormType_: string;
  private sFormName_: string;
  private oFormDefinition_: any;
  private sJSContent_: any;
  private sCSSContent_: any;
  public oStudioOptions_: StudioOptions;
  private JMESPATHFormResultAttr_: string;
  private JMESPATHJsResultAttr_: string;
  private JMESPATHCssResultAttr_: string;
  private sAPIFieldListRessource_: string;
  private JMESPATHFieldListResultAttr_: string;
  private aAvailableFormNames_: Array<string>;
  public attrFormFieldsList: Array<string>;
  public attrFormFieldsSpecification: any;
  private aFieldsList_: Array<any>;

  /**
   * Constructeur
   */
  constructor(
    private vitisRequest: VitisRequest,
    private formReaderService: FormreaderService,
    private vitisCheck: VitisCheck,
    private vitisClone: VitisClone,
    private translate: TranslateService
  ) {

    // Modification options
    this.obsStudioOptions.subscribe((oStudioOptions) => {

      // Options du studio
      this.oStudioOptions_ = oStudioOptions;

      // Url vers le fichier formulaire
      this.sAPIRessource_ = oStudioOptions.ressource;

      // Type de formulaire
      this.sFormType_ = oStudioOptions.default_form_type ? oStudioOptions.default_form_type : 'custom';

      // Nom du formulaire
      this.sFormName_ = oStudioOptions.default_form_name ? oStudioOptions.default_form_name : 'insert';

      // Emplacement du formulaire dans le retour de la requête
      this.JMESPATHFormResultAttr_ = oStudioOptions.ressource_result_attr;

      // Emplacement du javascript dans le retour de la requête
      this.JMESPATHJsResultAttr_ = oStudioOptions.js_ressource_result_attr;

      // Emplacement du css dans le retour de la requête
      this.JMESPATHCssResultAttr_ = oStudioOptions.css_ressource_result_attr;

      if (oStudioOptions.field_list_ressource) {
        this.sAPIFieldListRessource_ = oStudioOptions.field_list_ressource;
      }

      if (oStudioOptions.field_list_ressource_result_attr) {
        this.JMESPATHFieldListResultAttr_ = oStudioOptions.field_list_ressource_result_attr;
      }

      this.aAvailableFormNames_ = oStudioOptions.available_form_names;

      this.attrFormFieldsList = oStudioOptions.attr_form_fields_list ? oStudioOptions.attr_form_fields_list : null;
      this.attrFormFieldsSpecification = oStudioOptions.attr_form_fields_specification;
      this.aFieldsList_ = null; // réinitialise aFieldsList_

      // timeout permet d'attendre que tous les composants soient chargés
      setTimeout(() => {

        // Set les valeurs des observables
        this.setStudioFormType(this.sFormType_);
        this.setStudioFormName(this.sFormName_);

        // Charge le formulaire
        this.ajaxLoadStudioForm();
      });
    });

    // Modification formulaire
    this.obsStudioForm.subscribe((oStudioForm) => {

      // Défintion du formulaire
      this.oFormDefinition_ = oStudioForm;
    });

    // Modification type
    this.obsStudioFormType.subscribe((sStudioFormType: string) => {
      this.sFormType_ = sStudioFormType;
    });

    // Modification nom
    this.obsStudioFormName.subscribe((sStudioFormName: string) => {
      this.sFormName_ = sStudioFormName;
    });

    // Modification Contenu js
    this.obsStudioJsField.subscribe((sStudioJsField: string) => {
      this.sJSContent_ = sStudioJsField;
    });

    // Modification Contenu css
    this.obsStudioCssField.subscribe((sStudioCssField: string) => {
      this.sCSSContent_ = sStudioCssField;
    });
  }

  /**
   * Initialisation des tooltips du studio.
   * @returns void
   */
  public initTooltip() {
    setTimeout(() => {
      (<any>$('.studio [data-toggle="popover"]')).popover({
        trigger: 'hover'
      });
    }, 300);
  }

  /**
   * obsStudioOption Getter
   */
  public getStudioOptions(): Subject<StudioOptions> {
    return this.obsStudioOptions;
  }

  /**
   * obsStudioOption Getter
   */
  public getStudioOptionsSync(): StudioOptions {
    return this.oStudioOptions_;
  }

  /**
   * obsStudioOption Setter
   */
  public setStudioOptions(oStudioOptions: StudioOptions) {
    this.obsStudioOptions.next(oStudioOptions);
  }

  /**
   * obsStudioForm Getter
   */
  public getStudioForm(): Subject<any> {
    return this.obsStudioForm;
  }

  /**
   * oFormDefinition Getter
   */
  public getStudioFormSync(): Subject<any> {
    return this.oFormDefinition_;
  }

  /**
   * obsStudioForm Setter
   */
  public setStudioForm(oStudioForm: any) {
    oStudioForm = this.setFormIds_(oStudioForm);
    this.obsStudioForm.next(oStudioForm);
  }

  /**
   * obsStudioFormType Getter
   */
  public getStudioFormType(): Subject<any> {
    return this.obsStudioFormType;
  }

  /**
   * sFormType_ Getter
   */
  public getStudioFormTypeSync(): String {
    return this.sFormType_;
  }

  /**
   * obsStudioFormType Setter
   */
  public setStudioFormType(sStudioFormType: any) {
    this.obsStudioFormType.next(sStudioFormType);
  }

  /**
   * obsStudioFormName Getter
   */
  public getStudioFormName(): Subject<any> {
    return this.obsStudioFormName;
  }

  /**
   * obsStudioFormName Setter
   */
  public setStudioFormName(sStudioFormName: any) {
    this.obsStudioFormName.next(sStudioFormName);
  }

  /**
   * obsStudioEditedField Getter
   */
  public getStudioEditedField(): Subject<any> {
    return this.obsStudioEditedField;
  }

  /**
   * obsStudioEditedField Setter
   */
  public setStudioEditedField(oStudioEditedField: any) {
    // Le formulaire a été modifié
    this.setStudioIsBeingModified(true);

    this.obsStudioEditedField.next(oStudioEditedField);
  }

  /**
   * obsStudioJsField Getter
   */
  public getStudioJsField(): Subject<any> {
    return this.obsStudioJsField;
  }

  /**
   * obsStudioJsField Setter
   */
  public setStudioJsField(sStudioJsField: any) {
    this.obsStudioJsField.next(sStudioJsField);
  }

  /**
   * obsStudioCssField Getter
  **/
  public getStudioCssField(): Subject<any> {
    return this.obsStudioCssField;
  }

  /**
   * obsStudioCssField Setter
   */
  public setStudioCssField(sStudioCssField: any) {
    this.obsStudioCssField.next(sStudioCssField);
  }

  /**
   * obsStudioIsBeingModified Getter
   */
  public getStudioIsBeingModified(): Subject<any> {
    return this.obsStudioIsBeingModified;
  }

  /**
   * obsStudioIsBeingModified Setter
   */
  public setStudioIsBeingModified(bStudioIsBeingModified: any) {
    this.obsStudioIsBeingModified.next(bStudioIsBeingModified);
  }

  /**
   * obsStudioReloadFormListerner Getter
   */
  public getStudioReloadFormListerner(): Subject<any> {
    return this.obsStudioReloadFormListerner;
  }

  /**
   * obsStudioReloadFormListerner Setter
   */
  public setStudioReloadFormListerner(oStudioReloadFormListerner: any) {
    this.obsStudioReloadFormListerner.next(oStudioReloadFormListerner);
  }

  /**
   * obsStudioFieldListModalOptions Getter
   */
  public getStudioFieldListModalOptions(): Subject<any> {
    return this.obsStudioFieldListModalOptions;
  }

  /**
   * obsStudioFieldListModalOptions Setter
   */
  public setStudioFieldListModalOptions(oFieldListModalOptions: any) {
    this.obsStudioFieldListModalOptions.next(oFieldListModalOptions);
  }

  /**
   * Retourne la définition d'un type de champ
   * @param  {string} sFieldType
   * @returns {any}
   */
  public getFieldTypeDef(sFieldType: string): any {
    for (let i = 0; i < fieldsDef.length; i++) {
      for (let ii = 0; ii < fieldsDef[i].fields.length; ii++) {
        const oFieldDef = fieldsDef[i].fields[ii];
        if (oFieldDef.type == sFieldType) {
          // Si il y a des champs qui ont des spécificités on remplace la valeur true/false dans la définition du type de champ
          if (this.vitisCheck.isDefAndNotNull(this.attrFormFieldsSpecification)
            && this.vitisCheck.isDefAndNotNull(this.attrFormFieldsSpecification[oFieldDef.type])) {
            const structAttributes = this.attrFormFieldsSpecification[oFieldDef.type];
            for (const key in structAttributes) {
              if (Object.prototype.hasOwnProperty.call(structAttributes, key)
                && this.vitisCheck.isBoolean(structAttributes[key])
                && this.vitisCheck.isDefAndNotNull(oFieldDef.obj_attributes[key])) {
                oFieldDef.obj_attributes[key] = structAttributes[key];
              }
            }
          }
          const attributes: Array<string> = [];
          // Transforme les attributes d'objet en tableau en ne gardant que ceux qui valent true
          for (const key in oFieldDef.obj_attributes) {
            if (Object.prototype.hasOwnProperty.call(oFieldDef.obj_attributes, key)) {
              if (oFieldDef.obj_attributes[key] === true) {
                attributes.push(key);
              }
            }
          }
          oFieldDef.attributes = attributes;
          return this.vitisClone.clone(oFieldDef);
        }
      }
    }
  }

  /**
   * Charge un formulaire
   */
  public ajaxLoadStudioForm(sFormType?: string) {
    this.ajaxGetStudioForm(sFormType).then((oForm: any) => {
      this.reloadStudioForm(oForm[this.JMESPATHFormResultAttr_]);
      this.setStudioJsField(oForm[this.JMESPATHJsResultAttr_]);
      this.setStudioCssField(oForm[this.JMESPATHCssResultAttr_]);
      if (!this.vitisCheck.isDefAndNotNull(this.sJSContent_) || this.sJSContent_ == '') {
        this.setStudioJsField(this.getDefaultJs_());
      }
    });
  }

  /**
   * Récupère la définition d'un formulaire
   */
  public ajaxGetStudioForm(sFormType?: string) {
    return new Promise((resolve, reject) => {

      sFormType = this.vitisCheck.isDefAndNotNull(sFormType) ? sFormType : this.sFormType_;

      let url = environment.properties.app_api_url + '/' + this.sAPIRessource_;
      url = url.replace(':form', sFormType);

      // Téléchargement du fichier.
      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url
      }).then((resp: any) => {
        if (resp.status === 200 &&
          resp.data) {
          this.formatGetForm(resp.data).then((oForm: any) => {
            const isAutoGenerated = oForm[this.JMESPATHFormResultAttr_].autoGenerated === true;
            if (oForm[this.JMESPATHFormResultAttr_] == '' || isAutoGenerated) {
              // Si il n'y a pas de résultat et qu'une url sAPIFieldListRessource_ existe on propose la liste à l'utilisateur
              if (this.vitisCheck.isDefAndNotNull(this.sAPIFieldListRessource_)) {
                this.translate.get(['STUDIO_ALERT_FORM_ALREADY_GENERATE_TITLE', 'STUDIO_ALERT_FORM_ALREADY_GENERATE_TEXT', 'STUDIO_ALERT_NO_FORM', 'CONFIRM_CANCEL', 'STUDIO_ALERT_NO_FORM_EMPTY_GENERATE', 'STUDIO_ALERT_NO_FORM_GENERATE'])
                  .subscribe((translations: any) => {
                    vitisAlert.alert({
                      title: isAutoGenerated ? translations['STUDIO_ALERT_FORM_ALREADY_GENERATE_TITLE'] : translations['STUDIO_ALERT_NO_FORM'],
                      text: isAutoGenerated ? translations['STUDIO_ALERT_FORM_ALREADY_GENERATE_TEXT'] : '',
                      width: 800,
                      showDenyButton: true,
                      showCancelButton: false,
                      confirmButtonText: translations['STUDIO_ALERT_NO_FORM_GENERATE'],
                      denyButtonText: translations['STUDIO_ALERT_NO_FORM_EMPTY_GENERATE'],
                      cancelButtonText: translations['CONFIRM_CANCEL'],
                      confirmButtonColor: '#2778c4',
                      denyButtonColor: '#2778c4'
                    }).then((result) => {
                      if (result.isConfirmed) {
                        this.openListFieldsCanBeAdded(true, oForm[this.JMESPATHFormResultAttr_]);
                      } else if (result.isDenied) {
                        this.useEmptyForm().then(() => {
                          // Envoi le formulaire avec param init pour sauvegarder tous les types
                          this.ajaxSaveStudioForm('init').then((oEmptyForm: any) => {
                            this.reloadStudioForm(oEmptyForm);
                          });
                        });
                      } else if (result.isDismissed) {
                        this.useEmptyForm();
                      }
                    });
                  });
              } else {
                this.useEmptyForm();
              }
            } else {
              resolve(oForm);
            }
          }).catch((err) => {
            reject();
          });
        } else {
          reject();
        }
      }).catch((err) => {
        reject();
      });
    });
  }

  /**
   * Trigger obsStudioReloadFormListerner
   */
  public reloadStudioForm(oFormDefinition: any = this.oFormDefinition_) {
    if (this.vitisCheck.isDefAndNotNull(oFormDefinition)) {
      this.setStudioReloadFormListerner(oFormDefinition);
      setTimeout(() => {
        this.setStudioForm(oFormDefinition);
      });
    } else {
      this.setStudioReloadFormListerner(oFormDefinition);
    }
  }

  /**
   * Retourne l'index de la ligne où se trouve le champ
   * @param oField
   */
  public findRowIndex(aFormRows: Array<any>, oField: any): number {

    let rowIndex = -1;
    if (aFormRows) {
      rowIndex = aFormRows.findIndex((row: any) => {
        if (row.fields) {
          return (row.fields.findIndex((field: any) =>
            field.studio_id_ === oField.studio_id_) !== -1);
        }
        return false;
      });
    }

    return rowIndex;
  }

  /**
   * Retourne l'index de la colonne où se trouve le champ
   * @param oField
   */
  public findFieldIndex(aFormRows: Array<any>, oField: any, iRowIndex?: number): number {

    iRowIndex = iRowIndex ? iRowIndex : this.findRowIndex(aFormRows, oField);

    let fieldIndex = -1;
    if (aFormRows &&
      aFormRows[iRowIndex]) {
      const row = aFormRows[iRowIndex];
      if (row.fields) {
        fieldIndex = row.fields.findIndex((field: any) =>
          field.studio_id_ === oField.studio_id_);
      }
    }

    return fieldIndex;
  }

  /**
   * Met à jour les attributs d'un champ
   * @param oField
   * @param bErase
   */
  public updateField(oField: any, bErase = false, sFormName?: string) {

    sFormName = this.vitisCheck.isDefAndNotNull(sFormName) ? sFormName : this.sFormName_;

    let oNewForm = this.vitisClone.clone(this.oFormDefinition_);
    const aFormRows = this.formReaderService.getFormRows(oNewForm, sFormName);
    const iRowIndex = this.findRowIndex(aFormRows, oField);
    const iFieldIndex = this.findFieldIndex(aFormRows, oField, iRowIndex);

    if (oNewForm &&
      oNewForm[sFormName] &&
      oNewForm[sFormName].rows &&
      oNewForm[sFormName].rows[iRowIndex] &&
      oNewForm[sFormName].rows[iRowIndex].fields &&
      oNewForm[sFormName].rows[iRowIndex].fields[iFieldIndex]) {

      const oldField = oNewForm[sFormName].rows[iRowIndex].fields[iFieldIndex];

      if (oldField.name !== oField.name) {
        oNewForm = this.onChangeName(oldField.name, oField.name, oNewForm);
      }

      if (bErase) {
        // Écrase le champ par le nouveau
        oNewForm[sFormName].rows[iRowIndex].fields[iFieldIndex] = oField;
      } else {
        // Assigne les nouveaux attributs
        Object.assign(oldField, oField);
      }

      // Recharge le formulaire
      this.reloadStudioForm(oNewForm);
    }
  }

  /**
   * Ajoute un champ  à la fin du formulaire
   * @param oField
   */
  public addField(oField: any) {

    const oNewForm = this.vitisClone.clone(this.oFormDefinition_);
    const aFormRows = this.formReaderService.getFormRows(oNewForm, this.sFormName_);

    // Set id
    oField.studio_id_ = this.getUniqueId_();

    // Ajoute dans les lignes
    aFormRows.push({
      fields: [oField]
    });

    // Ajoute dans le premier onglets
    if (oNewForm[this.sFormName_] &&
      oNewForm[this.sFormName_].tabs &&
      oNewForm[this.sFormName_].tabs.list &&
      oNewForm[this.sFormName_].tabs.list[0] &&
      oNewForm[this.sFormName_].tabs.list[0].elements) {
      oNewForm[this.sFormName_].tabs.list[0].elements.push(oField.name);
    }

    // Le formulaire a été modifié
    this.setStudioIsBeingModified(true);

    this.reloadStudioForm(oNewForm);
  }

  /**
   * Change le type de champ et retourne le dit champ
   * @param oField
   * @param sFieldType
   */
  public changeFieldType(oField: any, sFieldType: string) {

    // Récupère la définition du nouveau type de champ.
    const oFieldDef = this.getFieldTypeDef(sFieldType);

    if (oFieldDef &&
      oFieldDef.definition) {

      const oNewField = this.vitisClone.clone(oFieldDef.definition);

      // Copie les paramètres obligatoires de l'ancien champ.
      oNewField.studio_id_ = oField.studio_id_;
      oNewField.name = oField.name;
      oNewField.label = oField.label;
      oNewField.nb_cols = oField.nb_cols;

      if (oField.default_value) {
        // on ne tiens pas compte des champs par défaut spécifique
        // uniquement les claurs textuelles
        if (oFieldDef.attributes.indexOf('default') > -1) {
          oNewField.default_value = oField.default_value;
        }
      }

      return oNewField;
    }
  }

  /**
   * Retourne la définition JSON du formulaire
   */
  public getJSONFormDefinition(): string {
    const oFormDefinition = this.vitisClone.clone(this.oFormDefinition_);
    const aPropertiesToDelete = ['validators_list', 'validation_messages', 'element', 'studio_selected_', 'studio_hover_',
      'studio_id_', '_parentsListened', '_visible', '_datasourcePulled', 'oninit', 'onvisible', 'oninitcomplete', 'id'];
    const aOptionsToDelete = ['choises', 'observableChoises'];
    const aOptionsColumnsToDelete = ['mData', 'sDefaultContent', 'sWidth', 'sClass', 'sTitle', 'sType'];
    const aWebserviceGridParamsToDelete = ['values', 'attributs'];

    const oNewFormDefinition = {};

    // Permet de faire un clone complet (car vitisClone ne suffit pas à casser le lien avec FormReader)
    // tout en supprimant les éléments non désirables dans le définition JSON
    if (this.vitisCheck.isDefAndNotNull(oFormDefinition)) {

      for (const sFormName in oFormDefinition) {
        if (oFormDefinition.hasOwnProperty(sFormName)) {
          if (this.vitisCheck.isDefAndNotNull(oFormDefinition[sFormName])) {

            // Nouveau formulaire
            oNewFormDefinition[sFormName] = {};

            // Copie de tout sauf des rows
            for (const key in oFormDefinition[sFormName]) {
              if (oFormDefinition[sFormName].hasOwnProperty(key)) {
                if (key !== 'rows') {
                  oNewFormDefinition[sFormName][key] = this.vitisClone.clone(oFormDefinition[sFormName][key]);
                }
              }
            }

            // Copie des rows
            if (this.vitisCheck.isArray(oFormDefinition[sFormName].rows)) {

              const aRows = oFormDefinition[sFormName].rows;
              const aNewRows = [];

              // Copie des rows
              if (this.vitisCheck.isArray(aRows)) {
                for (let i = 0; i < aRows.length; i++) {
                  if (this.vitisCheck.isArray(aRows[i].fields)) {

                    const aNewFields = [];

                    for (let ii = 0; ii < aRows[i].fields.length; ii++) {
                      const oField = aRows[i].fields[ii];
                      const oNewField: any = {};

                      for (const key in oField) {
                        if (oField.hasOwnProperty(key)) {
                          if (this.vitisCheck.isFunction(oField[key]) ||
                            this.vitisCheck.isObservabe(oField[key]) ||
                            aPropertiesToDelete.indexOf(key) !== -1) {
                            continue;
                          } else if (key === 'options') {

                            // nouvelles options
                            oNewField.options = {};

                            // Suppression des options indésirables
                            for (const key2 in oField.options) {
                              if (oField.options.hasOwnProperty(key2)) {
                                if (aOptionsToDelete.indexOf(key2) === -1) {
                                  if (key2 === 'columns') {
                                    oNewField.options.columns = [];
                                    for (let iColumnIndex = 0; iColumnIndex < oField.options.columns.length; iColumnIndex++) {
                                      const oColumn = oField.options.columns[iColumnIndex];
                                      const oNewColumn = {};
                                      // On ne garde que les colonnes qui ont un data
                                      if (this.vitisCheck.isDefAndNotNull(oColumn.data)) {
                                        for (const key3 in oColumn) {
                                          if (oColumn.hasOwnProperty(key3)) {
                                            if (aOptionsColumnsToDelete.indexOf(key3) === -1) {
                                              oNewColumn[key3] = oColumn[key3];
                                            }
                                          }
                                        }
                                        oNewField.options.columns.push(oNewColumn);
                                      }
                                    }
                                  } else {
                                    oNewField.options[key2] = oField.options[key2];
                                  }
                                }
                              }
                            }
                            // Suppression des options si vides
                            if (Object.keys(oNewField.options).length === 0) {
                              delete oNewField.options;
                            }
                          } else if (key === 'webservice_grid') {

                            oNewField.webservice_grid = {};
                            for (const key2 in oField.webservice_grid) {
                              if (oField.webservice_grid.hasOwnProperty(key2)) {
                                if (key2 === 'params') {
                                  oNewField.webservice_grid.params = {};
                                  for (const key3 in oField.webservice_grid.params) {
                                    if (oField.webservice_grid.params.hasOwnProperty(key3)) {
                                      if (aWebserviceGridParamsToDelete.indexOf(key3) === -1) {
                                        oNewField.webservice_grid.params[key3] = oField.webservice_grid.params[key3];
                                      }
                                    }
                                  }
                                } else {
                                  oNewField.webservice_grid[key2] = oField.webservice_grid[key2];
                                }
                              }
                            }
                          } else {
                            // Suppression des paramètres circulaires
                            try {
                              if (oField.type === 'map_vmap' && key === 'map_options') {
                                // Si on est sur un champs map_vmap on enlève olLayer qui est le paramètre circulaire
                                oNewField[key] = JSON.parse(JSON.stringify(oField[key], (key2, value) => {
                                  if (typeof value === 'object' && value !== null) {
                                    if (key2 === 'olLayer') {
                                      return;
                                    }
                                  }
                                  return value;
                                }));
                              } else {
                                const tmp = JSON.stringify(oField[key]);
                                oNewField[key] = oField[key];
                              }
                            } catch (error) { }
                          }
                        }
                      }

                      // Ajoute le field
                      aNewFields.push(oNewField);
                    }

                    // Ajoute la ligne
                    aNewRows.push({
                      fields: aNewFields
                    });
                  }
                }
              }

              oNewFormDefinition[sFormName].rows = aNewRows;
            }
          }
        }
      }
    }

    return JSON.stringify(oNewFormDefinition, null, 4);
  }

  /**
   * Enregistre le brouillon d'un formulaire
   */
  public ajaxSaveStudioForm(sFormType?: string): Promise<any> {
    return new Promise((resolve, reject) => {
      sFormType = this.vitisCheck.isDefAndNotNull(sFormType) ? sFormType : this.sFormType_;

      let url = environment.properties.app_api_url + '/' + this.sAPIRessource_;
      url = url.replace(':form', sFormType);

      const sForm = this.getJSONFormDefinition();

      // Téléchargement du fichier.
      this.vitisRequest.ajaxRequestPromise({
        method: 'PUT',
        url,
        data: {
          json: sForm,
          js: this.sJSContent_,
          css: this.sCSSContent_
        }
      }).then((resp: any) => {
        if (resp.status === 200 &&
          resp.data) {
          // Le formulaire a été sauvegarder et donc n'est plus en modification
          this.setStudioIsBeingModified(false);
          resolve(JSON.parse(sForm));
        } else {
          reject();
        }
      }).catch((err) => {
        reject();
      });
    });
  }

  /**
   * Gère les changements de noms et retourne la nouvelle définition
   * @param sOldName
   * @param sNewName
   * @param oFormDef
   */
  public onChangeName(sOldName: string, sNewName: string, oFormDef?: any): any {

    oFormDef = oFormDef ? oFormDef : this.vitisClone.clone(this.oFormDefinition_);

    if (sOldName !== sNewName) {

      // Change le nom dans les onglets
      if (oFormDef[this.sFormName_].tabs) {
        const tabs = oFormDef[this.sFormName_].tabs.list;
        if (this.vitisCheck.isArray(tabs)) {
          for (let i = 0; i < tabs.length; i++) {
            const tab = tabs[i];
            if (this.vitisCheck.isArray(tab.elements)) {
              const oldIndex = tab.elements.indexOf(sOldName);
              if (oldIndex !== -1) {
                tab.elements[oldIndex] = sNewName;
              }
            }
          }
        }
      }
    }

    return oFormDef;
  }

  /**
   * Récupère le formulaire à partir du data d'un retour de requête
   * @param {any} oData
   */
  private formatGetForm(oData: any): Promise<any> {
    return new Promise((resolve, reject) => {
      const response = {};

      let oForm = oData;
      // recherche JMESPATH form
      if (this.vitisCheck.isDefAndNotNull(this.JMESPATHFormResultAttr_)) {
        oForm = jmespathSearch(oData, this.JMESPATHFormResultAttr_);
      }

      if (this.vitisCheck.isArray(oForm) &&
        this.vitisCheck.isDefAndNotNull(oForm[0]) &&
        this.vitisCheck.isDefAndNotNull(oForm[0][this.sFormName_])) {
        response[this.JMESPATHFormResultAttr_] = oForm[0];
      } else if (!this.vitisCheck.isArray(oForm)) {
        try {
          if (oForm != '') {
            response[this.JMESPATHFormResultAttr_] = JSON.parse(oForm);
          } else {
            response[this.JMESPATHFormResultAttr_] = oForm;
          }
        } catch (e) {
          console.error('could not parse form from request');
        }
      } else {
        console.error('could not get form from request');
        reject();
      }

      // recherche JMESPATH form
      if (this.vitisCheck.isDefAndNotNull(this.JMESPATHJsResultAttr_)) {
        response[this.JMESPATHJsResultAttr_] = jmespathSearch(oData, this.JMESPATHJsResultAttr_);
      }

      if (this.vitisCheck.isDefAndNotNull(this.JMESPATHCssResultAttr_)) {
        response[this.JMESPATHCssResultAttr_] = jmespathSearch(oData, this.JMESPATHCssResultAttr_);
      }

      resolve(response);
    });
  }

  /**
   * Set des identifiants à tous les éléments
   * @param oFormDef
   */
  private setFormIds_(oFormDef: any) {

    if (oFormDef &&
      oFormDef[this.sFormName_]) {

      const aFields = this.formReaderService.getFormFields(oFormDef, this.sFormName_);

      for (let i = 0; i < aFields.length; i++) {
        const oField = aFields[i];
        oField.studio_id_ = this.vitisCheck.isDefAndNotNull(oField.studio_id_) ? oField.studio_id_ : this.getUniqueId_();
      }
    }

    return oFormDef;
  }

  /**
   * Trouve un identifiant unique à un élément
   * @returns string
   */
  public getUniqueId_(): string {
    let id = '_' + Math.random().toString(36).substring(2, 11);
    if (document.getElementById(id)) {
      id = this.getUniqueId_();
    }
    return id;
  }


  /**
   * Récupère un js de formulaire par défaut avec le constructeur.
   */
  private getDefaultJs_(): string {
    return '/**' + '\n' +
      '* constructor_form' + '\n' +
      '* Fonction lancée au chargement du formulaire' + '\n' +
      '* @param { object } scope Formreader' + '\n' +
      '* @param { object } env environment' + '\n' +
      '* @param { object } services' + '\n' +
      '* @returns { undefined } ' + '\n' +
      '*/' + '\n' +
      'var constructor_form = function (scope, env, services) {' + '\n' +
      ' ' + '\n' +
      '}';
  }

  /**
   * Retourne la liste des fields
   * @returns
   */
  public getFieldList(bLoadIfNotExists = false): Promise<any> {
    return new Promise((resolve, reject) => {
      if (this.vitisCheck.isDefAndNotNull(this.sAPIFieldListRessource_)) {
        if (this.vitisCheck.isDefAndNotNull(this.aFieldsList_)) {
          resolve(this.aFieldsList_);
        } else if (bLoadIfNotExists) {
          if (this.sAPIFieldListRessource_ == 'auto') {
            // Affichage de la fenêtre modale de sélection d'une table.
            (<any>$('#studio_select_table_database_modal')).modal('show').one('table.fields.list', (event: Event, sFieldsList: string) => {
              this.aFieldsList_ = JSON.parse(sFieldsList);
              resolve(this.aFieldsList_);
            });
          } else {
            const url = environment.properties.app_api_url + '/' + this.sAPIFieldListRessource_;

            // Téléchargement du fichier.
            this.vitisRequest.ajaxRequestPromise({
              method: 'GET',
              url
            }).then((resp: any) => {
              if (resp.status === 200 && resp.data) {
                let aData = resp.data;

                // recherche JMESPATH form
                if (this.vitisCheck.isDefAndNotNull(this.JMESPATHFieldListResultAttr_)) {
                  aData = jmespathSearch(resp.data, this.JMESPATHFieldListResultAttr_);
                }

                this.aFieldsList_ = aData;
                resolve(aData);
              } else {
                reject();
              }
            }).catch((err) => {
              reject();
            });
          }
        }
      }
    });
  }

  /**
   * Ouvre la liste des champs pouvant être ajouté
   * @param bIsInit Cas d'initilialisation
   * @param oForm Formulaire
   */
  public openListFieldsCanBeAdded(bIsInit?: boolean, oFormDef?: any): Promise<any> {
    return new Promise((resolve, reject) => {
      this.getFieldList(true).then((aData) => {

        // Vérifie que aData est un tableau et on vérifie qu'au moins le premier element ai le format requis
        if (this.vitisCheck.isArray(aData)
          && this.vitisCheck.isDefAndNotNull(aData[0])
          && this.vitisCheck.isDefAndNotNull(aData[0].name)
          && this.vitisCheck.isDefAndNotNull(aData[0].label)) {

          // formate les données
          this.formateDataForFieldListModal(aData, bIsInit, oFormDef).then((aFormatedData: any) => {
            const oFieldListModalOptions: any = {
              sModalTitle: 'FIELD_LIST_MODAL_TITLE',
              fieldList: this.vitisClone.clone(aFormatedData.databaseData),
              formFieldList: this.vitisClone.clone(aFormatedData.formData),
              sformDataTitle: 'FORM_FIELD_LIST_MODAL_TITLE'
            };
            if (this.vitisCheck.isDefAndNotNull(bIsInit) && bIsInit === true) {
              oFieldListModalOptions.init = true;
            }
            // Donne la configuration à la modal
            this.setStudioFieldListModalOptions(oFieldListModalOptions);

            // Attend que le getStudioFieldListModalOptions et *ngIf="oModalOptions" se soient activé (studio.component)
            setTimeout(() => {
              // ouvre la modale
              (<any>$('#field-list-modal')).modal('show');
            });
          });
        } else {
          reject();
        }
      });
    });
  }

  /**
   * Formate les données pour les afficher correctement dans la modal
   * => ajout option pour pré-coché les champs déjà présent
   * => ajout des labels pour les champs déjà présent
   * => récupération des champs présent dans le formulaire mais pas dans la base
   * @param aDbData Données pour la modal
   * @param bIsInit
   * @param oFormDef définition du formulaire
   */
  private formateDataForFieldListModal(aDbData: any, bIsInit?: any, oFormDef?: any): Promise<any> {
    return new Promise((resolve, reject) => {
      // Données présente dans le formulaire mais pas dans la base
      const oFormData = {};
      // Ajout des données par défaut pour les champs venant de la base
      for (let i = 0; i < aDbData.length; i++) {
        if (this.vitisCheck.isDefAndNotNull(oFormDef)) {
          aDbData[i].checked = [];
          for (const sFormName in oFormDef) {
            if (Object.prototype.hasOwnProperty.call(oFormDef, sFormName)) {
              const form = oFormDef[sFormName];
              if (form && form.rows) {
                const fields = this.formReaderService.getFormFields(oFormDef, sFormName);
                if (fields && fields.find && fields.find(field => field.name === aDbData[i].name)) {
                  aDbData[i].checked.push(sFormName);
                }
              }
            }
          }
        } else if (this.vitisCheck.isDefAndNotNull(bIsInit) && bIsInit === true) {
          aDbData[i].checked = this.aAvailableFormNames_;
        } else {
          aDbData[i].checked = [];
        }
        aDbData[i].studio_id_ = this.getUniqueId_();
        aDbData[i].originalLabel = this.vitisClone.clone(aDbData[i].label);
      }
      // Si c'est un init, on ne modifie pas les données
      if (this.vitisCheck.isDefAndNotNull(bIsInit) && bIsInit === true) {
        resolve({
          databaseData: aDbData,
          formData: oFormData
        });
      } else {
        const oNewForm = this.vitisClone.clone(this.oFormDefinition_);

        for (const sFormName in oNewForm) {
          if (Object.prototype.hasOwnProperty.call(oNewForm, sFormName)) {
            const aFormRows = this.formReaderService.getFormRows(oNewForm, sFormName);

            // Récupère la liste des champs qui existe dans le formulaire mais pas dans la base
            for (let j = 0; j < aFormRows.length; j++) {
              for (let k = 0; k < aFormRows[j].fields.length; k++) {
                const oField = aFormRows[j].fields[k];

                const iDataIndex = aDbData.findIndex((data: any) => {
                  if (data.name) {
                    return data.name === oField.name;
                  }
                  return false;
                });
                // Si le champ oField du formulaire n'est pas présent dans aDbData on le garde
                if (iDataIndex === -1) {
                  if (!this.vitisCheck.isDefAndNotNull(oFormData[oField.name])) {
                    oFormData[oField.name] = {
                      name: oField.name,
                      checked: []
                    };
                  }
                  oFormData[oField.name].checked.push(sFormName);
                  // Récupère un label (le dernier trouvé)
                  oFormData[oField.name].label = oField.label;

                }
              }
            }


            // Coche les champs issue de la base déjà présent
            for (let i = 0; i < aDbData.length; i++) {
              const iRowIndex = this.findRowIndexWithName(aFormRows, aDbData[i]);
              const iFieldIndex = this.findFieldIndexWithName(aFormRows, aDbData[i], iRowIndex);

              if (iRowIndex !== -1 && iFieldIndex !== -1) {
                aDbData[i].checked.push(sFormName);

                // Récupère un label (le dernier trouvé)
                if (oNewForm &&
                  oNewForm[sFormName] &&
                  oNewForm[sFormName].rows &&
                  oNewForm[sFormName].rows[iRowIndex] &&
                  oNewForm[sFormName].rows[iRowIndex].fields &&
                  oNewForm[sFormName].rows[iRowIndex].fields[iFieldIndex]) {

                  aDbData[i].label = oNewForm[sFormName].rows[iRowIndex].fields[iFieldIndex].label;
                }
              }
            }
          }
        }

        resolve({
          databaseData: aDbData,
          formData: oFormData
        });
      }
    });
  }

  /**
   * Génère un formulaire vide
   */
  public useEmptyForm(): Promise<boolean> {
    return new Promise((resolve, reject) => {
      const emptyForm = {
        'input_size': 'xxs',
        'javascript': false,
        'name': 'custom-form',
        'nb_cols': 12,
        'rows': [],
        'title': '',
        'tabs': {
          'type': 'tab',
          'list': [
            {
              'label': 'FORM_GENERAL_SECTION',
              'elements': []
            }
          ]
        }
      };

      // Onglets de type 'container'.
      if (this.vitisCheck.isDefAndNotNull(this.oStudioOptions_.tabs_type)
        && this.oStudioOptions_.tabs_type == 'container') {
        emptyForm.tabs.type = 'container';
        emptyForm.tabs.list[0]['collapsed'] = false;
      }

      const aAvailableFormNames_ = this.aAvailableFormNames_ ? this.aAvailableFormNames_ : ['update', 'insert', 'display'];

      const oData = {
        'datasources': {}
      };
      for (let i = 0; i < aAvailableFormNames_.length; i++) {
        const formName = aAvailableFormNames_[i];

        oData[formName] = emptyForm;
      }

      this.obsStudioForm.pipe(first()).subscribe(() => {
        resolve(true);
      });

      this.setStudioForm(oData);

    });
  }

  /**
   * Retourne l'index de la ligne où se trouve le champ à partir du nom
   * @param oField
   */
  public findRowIndexWithName(aFormRows: Array<any>, oField: any): number {

    let rowIndex = -1;
    if (aFormRows) {
      rowIndex = aFormRows.findIndex((row: any) => {
        if (row.fields) {
          return (row.fields.findIndex((field: any) =>
            field.name === oField.name) !== -1);
        }
        return false;
      });
    }

    return rowIndex;
  }

  /**
   * Retourne l'index de la colonne où se trouve le champ à partir du nom
   * @param oField
   */
  public findFieldIndexWithName(aFormRows: Array<any>, oField: any, iRowIndex?: number): number {

    iRowIndex = iRowIndex ? iRowIndex : this.findRowIndexWithName(aFormRows, oField);

    let fieldIndex = -1;
    if (aFormRows &&
      aFormRows[iRowIndex]) {
      const row = aFormRows[iRowIndex];
      if (row.fields) {
        fieldIndex = row.fields.findIndex((field: any) =>
          field.name === oField.name);
      }
    }

    return fieldIndex;
  }

  /**
   * Supprime un element des tabs
   * @param oFormTabs
   * @param sFieldName
   */
  public removeElementFromTabs(oFormTabs: any, sFieldName: string) {
    let aTabIndex = [];
    if (oFormTabs) {
      aTabIndex = oFormTabs.list.reduce((a: Array<number>, tab: any, index: number) => {
        if (tab.elements.indexOf(sFieldName) !== -1) {
          a.push(index);
        }
        return a;
      }, []);
    }

    for (let i = 0; i < aTabIndex.length; i++) {
      const iTabIndex = aTabIndex[i];
      if (iTabIndex !== -1) {
        const elementIndex = oFormTabs.list[iTabIndex].elements.indexOf(sFieldName);
        if (elementIndex !== -1) {
          oFormTabs.list[iTabIndex].elements.splice(elementIndex, 1);
        }
      }
    }
  }

  /**
   * Get option attr_form_fields_list
   */
  public getAttrFormFieldsList(): Promise<any> {
    return new Promise((resolve, reject) => {
      if (!this.vitisCheck.isUndefined(this.attrFormFieldsList)) {
        resolve(this.attrFormFieldsList);
      } else {
        this.obsStudioOptions.pipe(first()).subscribe((oStudioOptions: any) => {
          resolve(oStudioOptions.attr_form_fields_list);
        });
      }
    });
  }

  /**
   * Retourne l'onglet des boutons d'actions suivant le mode du formulaire.
   * @param {string} sFormMode
   * @returns {any}
   */
  public getFormActionButtonsTab(sFormMode: string): any {
    const oTab: any = {
      elements: [
        'return_list'
      ],
      class: 'tabs-container-transparent',
      action_buttons: true,
    };
    if (sFormMode != 'display') {
      oTab.elements.push('form_submit');
    }
    return oTab;
  }
}
