import { Component, OnInit, AfterViewInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { StudioService } from './services/studio.service';
import { VitisCheck, VitisClone, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { Observable, of } from 'rxjs';
import { CanComponentDeactivate } from 'src/app/vitis/core/services/deactivate/can-deactivate.guard';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-studio',
  templateUrl: './studio.component.html',
  styleUrls: ['./studio.component.scss']
})
export class StudioComponent implements OnInit, AfterViewInit, CanComponentDeactivate {

  public oMode: any;
  public oParams: any;
  public oFormId: any;
  public sPreviewMode: string = 'form';
  public oEditedField: any;

  private bIsBeingModified_: boolean;

  /**
   * Constructeur
   */
  constructor(
    private route: ActivatedRoute,
    private studioService: StudioService,
    private vitisClone: VitisClone,
    private vitisCheck: VitisCheck,
    private translate: TranslateService
  ) { }

  /**
   * canDeactivate
   * @returns
   */
  canDeactivate(): Observable<boolean> | Promise<boolean> | boolean {
    if (this.bIsBeingModified_) {
      return new Promise((resolve) => {
        this.translate.get(['FORM_MODIFIED_WARNING_TITLE',
          'FORM_MODIFIED_WARNING',
          'CONFIRM_CANCEL',
          'CONFIRM_CONFIRM']
        ).subscribe((translations) => {
          vitisAlert.alert({
            title: translations['FORM_MODIFIED_WARNING_TITLE'],
            text: translations['FORM_MODIFIED_WARNING'],
            icon: 'warning',
            showCancelButton: true,
            confirmButtonText: translations['CONFIRM_CONFIRM'],
            cancelButtonText: translations['CONFIRM_CANCEL'],
          }).then((result) => {
            resolve(result.value);
          });
        });
      });
    }

    return true;
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    this.studioService.getStudioIsBeingModified().subscribe((bIsBeingModified) => {
      this.bIsBeingModified_ = bIsBeingModified;
    });

    // Permet de savoir si un champ est en édition
    this.studioService.getStudioEditedField().subscribe((oEditedField) => {
      this.oEditedField = oEditedField;
    });

    // récupération du mode
    this.route.data.subscribe(oMode => {
      this.oMode = oMode;
    });
    this.route.params.subscribe(oParams => {
      this.oParams = oParams;

      const oStudioOptions = this.vitisClone.clone(this.oMode.options);

      // Remplace les paramètres dans la ressource
      if (this.vitisCheck.isString(oStudioOptions.ressource)) {
        if (this.vitisCheck.isDefAndNotNull(oParams.id)) {
          oStudioOptions.ressource = oStudioOptions.ressource.replace(':id', oParams.id);
          this.oFormId = oParams.id;
          if (oStudioOptions.field_list_ressource) {
            oStudioOptions.field_list_ressource = oStudioOptions.field_list_ressource.replace(':id', oParams.id);
          }
        }
      }

      // Initialise le studio
      setTimeout(() => {
        this.studioService.setStudioOptions(oStudioOptions);
      });
    });
  }

  /**
   * Appelé après la génération de la vue
   */
  ngAfterViewInit() {
    // Activation de tous les tooltips.
    this.studioService.initTooltip();
  }

  /**
   * Appelé lors du changement du mode de prévisualisation
   * @param sPreviewMode
   */
  public onPreviewModeChanges(sPreviewMode: string) {
    this.sPreviewMode = sPreviewMode;
  }
}
