import { Component, Inject, OnInit } from '@angular/core';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';
import { environment } from 'src/environments/environment';
import moment from 'moment-timezone';

@Component({
  selector: 'app-wkl-date',
  templateUrl: './wkl-date.component.html',
  styleUrls: ['./wkl-date.component.scss']
})
export class WklDateComponent implements OnInit {

  public timestamp: any;
  public type: string; // date, date_hour, hour, seconds, from_now
  public date: string;

  /**
   * Constructeur
   */
  constructor(
    @Inject('DatagridProvider') public providedValues,
    private vitisCheck: VitisCheck,
    private auth: AuthService
  ) {
    this.timestamp = this.providedValues.oRow[this.providedValues.oColumn.field];
    this.type = this.providedValues.type;
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {
    if (this.vitisCheck.isNumber(this.timestamp)) {
      const oUserInfos = this.auth.getUserInfos();

      switch (this.type) {
        case 'date':
          this.date = moment.unix(this.timestamp).tz(oUserInfos['timezone_id']).format(oUserInfos['formatdate_id']);
          break;
        case 'from_now':
          // chargement des traduction pour moment
          // peut avoir des conséquences sur le formatage des formats L, LL, LLL, LTS, ...
          moment.locale(environment.properties.language);
          this.date = moment.unix(this.timestamp).tz(oUserInfos['timezone_id']).fromNow();
          break;
        case 'hour':
          this.date = moment.unix(this.timestamp).format('HH:mm:ss');
          break;
        case 'seconds':
          this.date = moment.utc(this.timestamp * 1000).format('HH:mm:ss');
          break;
        default:
          this.date = moment.unix(this.timestamp).tz(oUserInfos['timezone_id']).format(oUserInfos['formatdate_id'] + ' HH:mm:ss');
          break;
      }
      // Supprime l'espace entre le "T" et l'heure (ISO 8601).
      this.date = String(this.date).replace('T ', 'T');
    } else {
      this.date = this.timestamp;
    }
  }

}
