import { Component, Inject, OnInit } from '@angular/core';

/**
 * template_options :
 * - url : url de base à utiliser pour les redirection interne possibilité de binder des valeur via des balide :mon_champ
 * - label : si le label est statique
 * - label_field : champs à afficher comme label
 */


@Component({
  selector: 'app-wkl-external-link',
  templateUrl: './wkl-external-link.component.html',
  styleUrls: ['./wkl-external-link.component.scss']
})
export class WklExternalLinkComponent implements OnInit {

  public url: string;
  public label: string;

  /**
   * Constructeur
   */
  constructor(
    @Inject('DatagridProvider') public providedValues,
  ) {

    // Si URL n'est pas passé en paramètre alors prend la valeur depuis oRow
    if (!this.providedValues.url) {
      this.providedValues.url = this.providedValues.oRow[this.providedValues.oColumn.data];
    }

    if (this.providedValues.url) {
      this.url = this.urlFormater(this.providedValues.oRow, this.providedValues.url);
    }

    if (this.providedValues.label) {
      this.label = this.providedValues.label;
    } else if (this.providedValues.label_field) {
      this.label = this.providedValues.oRow[this.providedValues.label_field];
    } else {
      this.label= this.url;
    }
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {}

  /**
   * Prépare une url pour le lien dans le composant
   * @param {any} aData data pour binder les données
   * @param {string} sUrl url à bind
   * @returns {string} url à utiliser
   */
  protected urlFormater(aData: any, sUrl: string): string {
    if (sUrl && sUrl.matchAll) {
      const sRegExp = /:(?<url_key>[^\/\s]+)/g;
      const matchs = sUrl.matchAll(sRegExp);

      for (const match of matchs) {
        const sKey = match.groups['url_key'];
        if (aData[sKey]){
          sUrl = sUrl.replace(new RegExp('\:' + sKey), `${aData[sKey]}`);
        }
      }

      return sUrl;
    }
  }

}
