import { Component, Inject, OnInit } from '@angular/core';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { WklExternalLinkComponent } from '../wkl-external-link/wkl-external-link.component';

/**
 * template_options :
 * - url : url de base à utiliser pour les redirection interne possibilité de binder des valeur via des balide :mon_champ
 * - label : si le label est statique
 * - label_field : champs à afficher comme label
 * - privilege : permet de switcher entre lien et label en focntions des roles
 */


@Component({
  selector: 'app-wkl-internal-link',
  templateUrl: './wkl-internal-link.component.html',
  styleUrls: ['./wkl-internal-link.component.scss']
})
export class WklInternalLinkComponent extends WklExternalLinkComponent implements OnInit {

  public bShowLink: boolean = false;

  private privileges: string[] = [];

  /**
   * Constructeur
   */
  constructor(
    @Inject('DatagridProvider') public providedValues,
    private auth: AuthService,
  ) {
    super(providedValues);

    // l'url interne doit forcément commencer par /
    if (this.url.indexOf('/') > 0) {
      this.url = '/' + this.url;
    }

    if (this.providedValues.privilege) {
      this.privileges = (Array.isArray(this.providedValues.privilege)) ? this.providedValues.privilege : [this.providedValues.privilege];
    }

    this.bShowLink = this.checkRole();
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {}

  /**
   * permet de savoir s'il faut afficher un lien ou un label
   * @returns {boolean}
   */
  private checkRole(): boolean {
    if (this.privileges.length === 0) {
      return true;
    }

    const aUserPrivileges = this.auth.getPrivileges();
    const aMatch = aUserPrivileges.filter(item => this.privileges.indexOf(item) > -1);
    if (aMatch.length > 0) {
      return true;
    }

    return false;
  }

}
