import { Component, OnInit, Input, AfterViewInit, ViewChild, OnDestroy, Inject } from '@angular/core';
import { VitisCheck } from 'src/app/vitis/core/services/utils/vitis-utils';

declare let $: any;

@Component({
  selector: 'app-wkl-params',
  templateUrl: './wkl-params.component.html',
  styleUrls: ['./wkl-params.component.scss']
})
export class WklParamsComponent implements OnInit, AfterViewInit, OnDestroy {

  public params: any;

  @ViewChild('popover', { static: false }) popoverElem: any;

  public aDisplayParams: Array<string> = [];

  /**
   * Constructeur
   */
  constructor(
    @Inject('DatagridProvider') public providedValues,
    private vitisCheck: VitisCheck
  ) {
    this.params = this.providedValues.oRow[this.providedValues.oColumn.field];
  }

  /**
   * Chargement du composant
   */
  ngOnInit() {
    // Initialise le contenu
    this.initContent();
  }

  /**
   * Appelé après le chargement du DOM
   */
  ngAfterViewInit() {
    // initialise la popover
    if (this.vitisCheck.isDefAndNotNull(this.popoverElem)) {
      this.initPopover(this.popoverElem.nativeElement);
    }
  }

  /**
   * Destruction du composant
   */
  ngOnDestroy() {
    // détruit la popover
    if (this.vitisCheck.isDefAndNotNull(this.popoverElem)) {
      this.destroyPopover(this.popoverElem.nativeElement);
    }
  }

  /**
   * Initialise le contenu
   */
  public initContent() {
    try {
      const oParams = JSON.parse(this.params);
      for (const key in oParams) {
        if (oParams.hasOwnProperty(key)) {
          this.aDisplayParams.push(`<b>${key}</b> = ${oParams[key]}`);
        }
      }
    } catch (error) {
      this.aDisplayParams = this.params.split('|');
    }
  }

  /**
   * Initialise la popover
   * @param elem élément dom
   */
  public initPopover(elem: any) {
    const popoverContainer: any = $(elem);
    popoverContainer.popover({
      content: this.aDisplayParams.join('<br>'),
      trigger: 'hover',
      html: true
    });
  }

  /**
   * Supprime la popover
   * @param elem élément dom
   */
  public destroyPopover(elem: any) {
    const popoverContainer: any = $(elem);
    popoverContainer.popover('hide');
    popoverContainer.popover('disable');
  }
}
