import { AfterViewInit, Component, ElementRef, Inject, OnInit, ViewChild } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';

declare let $: any;

interface TooltipConfig {
  container?: string;
  content?: string;
  html?: boolean;
  title?: string;
  trigger?: string;
}

@Component({
  selector: 'app-wkl-tooltip',
  templateUrl: './wkl-tooltip.component.html',
  styleUrls: ['./wkl-tooltip.component.scss']
})
export class WklTooltipComponent implements OnInit {

  public content: string;
  public title: string;
  public icon: string;
  public invisible: boolean = false;

  @ViewChild('tooltip', { static: true, read: ElementRef }) tooltip: ElementRef;

  /**
   * Constructeur
   */
  constructor(
    @Inject('DatagridProvider') public providedValues,
    private translate: TranslateService,
  ) {
    this.content = this.providedValues.oColumn.field ? this.providedValues.oRow[this.providedValues.oColumn.field] : this.providedValues.oRow[this.providedValues.oColumn.data];
    this.title = this.providedValues.title ?? this.providedValues.oColumn.title;
    this.icon = this.providedValues.icon ?? 'fa-info-circle fas vitis-tooltip';
  }

  /**
   * Chargement du composant
   */
  ngOnInit(): void {
    let iConfig: TooltipConfig;
    if (this.content) {
      this.invisible = false;
      iConfig = {
        container: 'body',
        content: this.content ? this.content : '',
        html: true,
        title: this.title ? this.translate.instant(this.title) : '',
        trigger: 'hover'
      };
      $(this.tooltip.nativeElement).popover(iConfig);
    } else {
      this.invisible = true;
    }
  }
}
