import { Component, EventEmitter } from '@angular/core';
import { NgbModalOptions } from '@ng-bootstrap/ng-bootstrap';


/*
 * Form_path ne faisant pas aprti des options necessaire pour ngbmodal mais pour nous
 * On l'ajoute
 */
export interface WkListBtnModalOptions extends NgbModalOptions{
  form_path?: string;
  ressource?: string;
  id_field?: string;
  modal_title?: string;
}

/**
 * Options boutons
 */
export class WorkspaceListButton {
  event?: any;
  type: string; // button / group / modal
  label: string;
  sEvent?: string;
  name?: string;
  class?: string;
  icon?: string;
  buttons?: Array<WorkspaceListButton>;
  modal_component?: string;
  modal_options?: WkListBtnModalOptions; // https://ng-bootstrap.github.io/#/components/modal/api#NgbModalOptions
  refreshGridOnClose?: boolean; // recharge la grille lors de la fermeture (default: false)
  eventOnClose?: string; // Événement à lancer à la fermeture
  modal_need_selected_row?: boolean; // Vérifie qu'au moins 1 ligne de la grille est sélectionné (default: false)
  modal_label_no_row_error?: string; // Message d'erreur à afficher si aucune ligne n'a été sélectionné alors que c'était néccessaire
  privilege?: string | Array<string>; // Affichage du bouton en fonction d'un ou de plusieurs privilèges.
  display?: boolean; // Affichage du bouton (string = privilège obligatoire).
}

/**
 * Options des colonnes Workspacelist
 */
export class WorkspaceListColumn {
  title: string; // Titre à afficher
  field: string; // Nom de la colonne
  align?: string; // Alignement (right, left, center)
  sortable?: boolean; // Ordre modifiable
  translateTitle?: string; // Titre à traduire
  template?: any; // Composant Angular à utiliser
  template_options?: any; // Options à passer au composant
  resizeable?: boolean; // Non disponible
  width?: string; // Non disponible
  visible?: boolean; // Colonne visible (mais utilisable) vaut true par défaut
  searchable?: boolean; // Permet de désectiver le filtre personnalisé sur cette colonne
  searchType?: string; // Type de données, permet de mieux gérer les filtres : strict, number, date, boolean, text
  searchOperators?: string[]; // permet de faire un filtre sur les operateurs disponibles (['~', '=', '>', '<', 'IN', 'NOT IN', 'IS NULL', 'IS NOT NULL'])
  searchColumn?: string; // permet de filtrer sur un autre champ de la base
  searchWebService?: WorkspaceListSearchWSColumn; // permet de définir une datasource pour remplir les suggestions
  searchSuggestions?: any; // Suggestions (false ou tableau)
  privilege?: string | Array<string>; // Affichage de la colonne en fonction d'un ou plusieurs privilèges
}

/**
 * Option pour utiliser un webservice comme suggestion d'un champs
 */
export class WorkspaceListSearchWSColumn {
  label?: string; // si non défini la valeur sera utilisé
  value: string;
  resource: string; // module/objet
  params?: any; // sera encodé en json et utilisé pour la requête GET
}

/**
 * Option value statusFilter
 */
export class WorkspaceListStatusFilterValue {
  label: string;
  value: string;
  textStyle?: string; // text-primary / text-danger / text-success / text-dark / text-info / text-secondary / text-warning
  showCounter?: boolean;
  annimation?: boolean;
}

/**
 * Options pour composant de type WorkspaceList
 */
export class WorkspaceListOptions {
  module?: string; // (ex: gtf) permet de charger les composant du module
  ressource?: string; // (ex: gtf/orders) ressource API à attaquer
  id_field?: string; // (ex: order_id) colonne id de l'API
  parent_id_field?: string; // cas de l'utilisation de workspacelist dans une section, permet soit de faire un remplacement dans l'URL avec :id_parent soit de faire un filtre sur l'élément parent
  disable_filter_parent?: boolean; // Permet de désactiver le filtrage via les noeud sur le parent automatiquement
  display?: boolean | string | Array<string>; // affichage du bouton fiche descriptive (string = privilège obligatoire)
  update?: boolean | string | Array<string>; // affichage du bouton mise à jour (string = privilège obligatoire)
  insert?: boolean | string | Array<string>; // affichage du bouton ajout (string = privilège obligatoire)
  delete?: boolean | string | Array<string>; // affichage du bouton suppression (string = privilège obligatoire)
  select?: boolean; // affichage des checkbox de selection
  clone?: boolean | string; // affichage du bouton clone (string = privilège obligatoire)
  refresh?: boolean; // affichage du bouton rafraichissement (true par défaut)
  autorefresh?: WorkspaceListAutorefreshOptions; // Options pour rafraichissement automatique
  form_path?: string; // Chemin vers le fichier formulaire
  js_path?: string; // Chemin vers le fichier formulaire
  translations?: any;
  order_by?: Array<any>; // [numbero de la colonne, asc / desc]
  filter?: any; // filtre en mode clé valeur
  search?: {
    custom?: boolean
    form?: boolean
    default?: string // custom/form
    defaultColumn?: string; // Nom de la colonne à utiliser en cas où l'utilisateur oublie de la saisir
    statusFilter?: {
      request?: {
        ressource?: string // ressource à utiliser pour les valeurs de filtres
        counter_key?: string // clé à utiliser pour la répartition
        params?: object // paramètre pour la requête de récupération des données
      }
      searchColumn: string
      values: Array<WorkspaceListStatusFilterValue>
    }
  };
  extra_attributs?: Array<string>; // élément à récupérer en plus des colonnes pour certains composants par exemple
  columns: Array<WorkspaceListColumn>;
  buttons: Array<WorkspaceListButton>;
  edition_component?: string; // Utilisation d'un composant autre que simpleform pour l'insertion et l'edition
  edition_module?: string; // même chose que edition_component mais pour un module chargé en lazy loading
  edition_options?: any; // options à utiliser pour edition_module et edition_component
  height?: number; // Hauteur de la grille
  singleSelect?: boolean; // Désactivation de la selection multiple
  max_rows?: number; // Nombre maximum d'enregistrements à afficher
}

/**
 * Options pour rafraichissement automatique
 */
export class WorkspaceListAutorefreshOptions {
  interval: number; // Intervalle de rafraichissement
  compare_columns?: Array<string>; // Colonnes à comparer
}

/**
 * Options pour workspacelist embarqué
 */
export class WorkspaceListEmbeddedOptions {
  mode: {
    name: string;
    label: string;
    route: string;
    type: string;
    privilege: string;
    options: WorkspaceListOptions;
  };
  events?: {
    onRefresh?: EventEmitter<any>;
    onSelect?: EventEmitter<any>;
    onDeselect?: EventEmitter<any>;
    onEdit?: EventEmitter<any>;
    onDisplay?: EventEmitter<any>;
    onCreate?: EventEmitter<any>;
    onDelete?: EventEmitter<any>;
  };
}
