import { Component, OnInit, ViewChild, EventEmitter } from '@angular/core';
import { DataGridComponent } from '../data-grid/data-grid.component';
import { environment } from 'src/environments/environment';
import { WorkspaceRelationListOptions } from './workspacerelationlist.model';
import { ActivatedRoute, Router } from '@angular/router';
import { VitisCheck, LocalStorage, SessionStorage, VitisRequest, vitisAlert } from 'src/app/vitis/core/services/utils/vitis-utils';
import { TranslateService } from '@ngx-translate/core';
import { AuthService } from 'src/app/vitis/core/services/auth/auth.service';
import { DataGridOptions } from 'src/app/vitis/data-grid/data-grid.model';
import { FormreaderComponent } from '../formreader/formreader.component';
import { WorkspacerelationlistService } from './workspacerelationlist.service';

declare let $: any;

@Component({
  selector: 'app-workspacerelationlist',
  templateUrl: './workspacerelationlist.component.html',
  styleUrls: ['./workspacerelationlist.component.scss']
})
export class WorkspacerelationlistComponent implements OnInit {

  @ViewChild(DataGridComponent, { static: true }) oDataGrid: DataGridComponent;

  // Options de la grille de données.
  public oDataGridOptions: DataGridOptions = {
    datatableOptions: {},
    datagridOptions: WorkspaceRelationListOptions
  };

  dataTableUpdated: any[] = [];

  // Largeur de page
  public pageLength: Number = environment.properties.rows_per_page;

  openDataSourceModal: boolean = false;

  // Mode en cours
  public mode: any;
  public oParams: any;

  // Résultat
  public oResults: any = {
    recordsTotal: 0,
    recordsFiltered: 0,
    data: []
  };

  // Paramètres passés dans l'URL
  public urlParams_: any;

  // Paramètres à passer à la requête
  public datatableRessourceParams_: any;

  public onRefresh = new EventEmitter<any>();

  // Modal avec un formreader
  public modalTitle: string = '';
  public modalTitleSourceDataGrid: string = '';
  @ViewChild(FormreaderComponent, { static: false }) oFormReader: FormreaderComponent;

  // Modale affichée si aucun enregistrement dans la liste.
  public bDisplayNoRecordModal: boolean = false;
  public sNoRecordLabel: string = 'VITIS_WORKSPACELIST_NO_RECORD';
  public sInsertButtonLabel: string = 'VITIS_WORKSPACELIST_INSERT_BTN';

  // Variables privées
  private localStorageValues_: any;
  private sessionStorageValues_: any;
  private clickedCell_: any;
  private queryCache_: any;

  /**
   * Constructeur
   */
  constructor(
    private route: ActivatedRoute,
    private vitisCheck: VitisCheck,
    private router: Router,
    private localStorage: LocalStorage,
    private sessionStorage: SessionStorage,
    private vitisRequest: VitisRequest,
    private translate: TranslateService,
    private auth: AuthService,
    private service: WorkspacerelationlistService
  ) { }

  /**
   * Chargement du composant
   */
  ngOnInit() {

    // Nombre d'éléments à afficher sur la grille
    this.pageLength = Number(environment.properties.rows_per_page);

    // Permet à oDataGrid de connaitre les colonnes etc..
    this.oDataGrid.oDataGridOptions = this.oDataGridOptions;

    // Mode en cours d'utilisation
    this.route.data.subscribe(mode => {
      this.mode = mode;

      // options app.json
      if (mode.options) {
        this.oDataGridOptions.datagridOptions = mode.options;

        // Texte optionnel pour la modale de création (si aucun enregistrement dans la liste).
        if (this.vitisCheck.isDefAndNotNull(mode.options.no_record_modal)) {
          if (this.vitisCheck.isDefAndNotNull(mode.options.no_record_modal.label)) {
            this.sNoRecordLabel = mode.options.no_record_modal.label;
          }
          if (this.vitisCheck.isDefAndNotNull(mode.options.no_record_modal.button_label)) {
            this.sInsertButtonLabel = mode.options.no_record_modal.button_label;
          }
        }
      }

      // Initialise la grille de données
      this.initDataGrid();

      const self = this;
      // Event champs editable grid data
      // $('body').on('input click keyup', '[contenteditable]', function () {
      //   const cell = $('#' + self.oDataGrid.tableId).DataTable().cell(this);
      //   self.updateCell(self, cell, $(this).html());
      // });
    });

    // Paramètres de la route en cours d'utilisation
    this.route.params.subscribe(oParams => {
      this.oParams = oParams;
      this.initWorkspaceRelationList();
    });

    this.service.modalDataSourceGridObservable
      .subscribe(statut => {
        // //Update data ressource quand on ferme la modale contenant le data grid de source
        if (!statut) {
          $('#workspacerelationlist-form-modal').removeClass('show').trigger('classChange');
          $('#workspacerelationlist-form-modal').modal('hide');
          this.oDataGrid.reloadTable();
        }
      });
    // Survéillance du filtre dans les paramètres
    this.route.queryParams.subscribe(oParams => {

      // mémorise les paramètres URL
      this.urlParams_ = oParams;

    });
  }
  /**
   * Fonction pour update les valeurs changer depuis la data grid
   * @param val jquery selector
   */
  private updateCell(self, cell, newValue) {
    const fields = {
      ressource: this.mode.options.id_field,
      source: this.mode.options.source.id_field
    };
    const cellIndex = cell.index();
    const table = $('#' + self.oDataGrid.tableId).dataTable();
    if (cellIndex) {
      const rowUpdate = table.fnGetData(cellIndex.row);
      const oSettings = table.fnSettings();
      const column = oSettings.aoColumns[cellIndex.column];
      const updateField = this.mode.options.fields_source.includes(column.name) ? 'source' : 'ressource';
      const rowExist = this.dataTableUpdated.find(data => data[fields[updateField]] === rowUpdate[fields[updateField]]);
      if (rowExist) {
        rowExist[column.name] = newValue;
      } else {
        this.dataTableUpdated.push({
          [fields[updateField]]: rowUpdate[fields[updateField]],
          [column.name]: newValue
        });
      }
    }
  }

  /**
 * Initialise le workspacerelationlist
 */
  public initWorkspaceRelationList() {
    this.mode.type = 'workspacerelationlist';
    // construit le chemin vers la ressource
    const r = this.mode.options.ressource;
    this.mode.options.ressource = this.processUrl();
  }

  /**
   * remplace :id dans la route ou cherche un id numérique
   * @returns {string}
   */
  private processUrl(): string {
    // construit le chemin vers la ressource
    const r = this.mode.options.ressource;
    const sSearch = r.search(':id') !== -1 ? ':id' : new RegExp('[0-9]+');
    return r.replace(sSearch, this.oParams['id']);
  }

  /**
   * Parse les params envoyés par DataTables pour créer des paramètres interprétables par l'API Vitis
   * @param  {any} params
   * @returns any
   */
  private getRequestParams_(params: any): any {

    const requestParams: any = {};

    // Order
    if (this.vitisCheck.isDefAndNotNull(params.order)) {
      if (this.vitisCheck.isDefAndNotNull(params.order[0])) {
        if (this.vitisCheck.isString(params.order[0].dir)) {
          requestParams.sort_order = params.order[0].dir.toUpperCase();
        }
        if (this.vitisCheck.isNumber(params.order[0].column)) {
          const iColumnId = params.order[0].column;
          if (this.vitisCheck.isDefAndNotNull(params.columns[iColumnId])) {
            if (this.vitisCheck.isString(params.columns[iColumnId].data)) {
              requestParams.order_by = params.columns[iColumnId].data;
            }
          }
        }
      }
    }

    // Limit
    if (this.vitisCheck.isNumber(params.length)) {
      requestParams.limit = params.length;
    }

    // Offset
    if (this.vitisCheck.isNumber(params.start)) {
      requestParams.offset = params.start;
    }

    // Attributs
    if (this.vitisCheck.isDefAndNotNull(params.columns)) {
      requestParams['attributs'] = params.columns.filter(
        (aColumn: any) => this.vitisCheck.isString(aColumn.data)
      ).map(
        (aColumn: any) => aColumn.data
      ).join('|');
    }

    return requestParams;
  }

  /**
   * Récupère les donéées au format DataTables
   * @param  {any} params
   * @returns Promise
   */
  public getResourceData_(params: any): Promise<any> {
    return new Promise((resolve, reject) => {

      // Mémorise les paramètres
      this.datatableRessourceParams_ = params;

      // Mémorisation des paramètres de la table
      this.localStorage.add(this.mode.name, {
        datatableParams: params,
      });
      this.sessionStorage.add(this.mode.name, {
        datatableParams: params,
      });

      this.oDataGridOptions.loadingData_ = true;

      // Lance la requête
      this.vitisRequest.ajaxRequest({
        method: 'GET',
        url: this.getRessourceUrl(),
        ajaxLoader: false,
        params: this.getRequestParams_(params),
        success: (resp: any) => {

          this.oDataGridOptions.loadingData_ = false;

          const oReturn = {
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          };

          if (this.vitisCheck.isDefAndNotNull(resp.data)) {
            if (this.vitisCheck.isDefAndNotNull(resp.data.status === 1)) {
              if (this.vitisCheck.isArray(resp.data.data)) {
                oReturn.data = resp.data.data;
              }
              if (this.vitisCheck.isDefAndNotNull(resp.data.total_row_number)) {
                oReturn.recordsTotal = resp.data.total_row_number;
                oReturn.recordsFiltered = resp.data.total_row_number;
              }
            }
          }
          resolve(oReturn);
        },
        error: (err: any) => {
          console.error('error while loading values :', err);

          this.oDataGridOptions.loadingData_ = false;

          const oReturn = {
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          };

          resolve(oReturn);
        }
      });
    });
  }

  /**
 * Génére l'url pour récupérer les données
 * @returns {string}
 */
  protected getRessourceUrl(): string {
    let sRessourceUrl = this.oDataGridOptions.datagridOptions.ressource;

    // Gestion du parent en cas de workspacelist dans une section
    if (this.mode.options &&
      this.mode.options.parent_id_field &&
      this.oParams &&
      this.oParams.id) {

      if (sRessourceUrl.indexOf(`:${this.mode.options.parent_id_field}`) !== -1) {
        // Remplacement dans l'URL
        sRessourceUrl = sRessourceUrl.replace(`:${this.mode.options.parent_id_field}`, String(this.oParams.id));
      }
    }

    return environment.properties.app_api_url + '/' + sRessourceUrl;
  }

  /**
   * Initialise la grille de données
   */
  public initDataGrid() {
    /**
     * Calcule la hauteur de la table
     */
    const calcDataTableHeight = () => {
      if (this.vitisCheck.isNumber(this.mode.options.height)) {
        return this.mode.options.height;
      } else {
        const dashboardHeight = $('#dashboard-content').height();
        const titleHeight = $('.mode-title').height();
        const filterHeight = $('.grid-filter').height();
        const menusHeight = 150;
        const tableHeight = dashboardHeight - (menusHeight + filterHeight + titleHeight);
        return tableHeight;
      }
    };
    // Ordre
    let order = [2, 'desc'];

    // Offset
    let displayStart = 0;

    let rowId = '';

    // Options app.json
    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions)) {
      if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.order_by)) {
        order = this.oDataGridOptions.datagridOptions.order_by;
      }
      if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.id_field)) {
        rowId = this.oDataGridOptions.datagridOptions.id_field;
      }
    }

    // LocalStorage
    if (this.localStorageValues_ &&
      this.localStorageValues_.datatableParams) {

      // Order
      if (this.localStorageValues_.datatableParams.order &&
        this.localStorageValues_.datatableParams.order[0] &&
        this.localStorageValues_.datatableParams.order[0].column &&
        this.localStorageValues_.datatableParams.order[0].dir) {
        order = [this.localStorageValues_.datatableParams.order[0].column,
          this.localStorageValues_.datatableParams.order[0].dir];
      }
    }

    // SessionStorage
    if (this.sessionStorageValues_ &&
      this.sessionStorageValues_.datatableParams) {

      // start
      if (this.vitisCheck.isDefAndNotNull(this.sessionStorageValues_.datatableParams.start)) {
        displayStart = this.sessionStorageValues_.datatableParams.start;
      }
    }

    // Paramètres URL
    if (this.urlParams_ &&
      this.urlParams_.wkgstart &&
      this.urlParams_.wkglength) {
      displayStart = Number(this.urlParams_.wkgstart);
    }

    // Paramètres de la grille de données.
    this.oDataGridOptions.datatableOptions = {
      select: true,
      selectMode: this.mode.options.singleSelect ? 'single' : 'multi',
      processing: true,
      serverSide: true,
      searching: false,
      lengthMenu: this.getLengthMenu_(),  // Tableau choix du nombre d'éléments par page
      pageLength: this.getPageLength_(),  // Nombre d'éléments par page
      order: order, // Ordre
      displayStart: displayStart,  // Offset
      rowId: rowId,
      selected: 'selected',
      // Return tous les champs editables
      editableColumns: this.mode.options.columns.filter(column => column.editable).map(column => column.field),
      ajax: (data: any, callback: any, settings: any) => {

        // Cache utilisé pour le rafraichissement auto
        this.queryCache_ = {
          options: data,
          date: Date.now(),
        };

        this.getResourceData_(data).then((oResponse: any) => {

          // Set oResults
          this.oResults = oResponse;

          // Vide viewContainer
          this.oDataGrid.viewContainer.clear();

          // Remplace les données
          callback(this.oResults);

          // Remplit le cache
          this.queryCache_.result = oResponse;

          // event onRefresh
          if (this.onRefresh) {
            this.onRefresh.emit();
          }

          // Si aucun enregistrement : affichage de la modale de création si l'onglet à un mode "insert".
          if (this.mode.options.insert) {
            if (this.oResults.recordsTotal == 0) {
              this.bDisplayNoRecordModal = true;
            } else {
              this.bDisplayNoRecordModal = false;
            }
          }
        }).catch(() => {
          this.oResults = {
            recordsTotal: 0,
            recordsFiltered: 0,
            data: []
          };
          callback(this.oResults);
        });
      }
    };

    // Initialise les boutons
    this.initButtons();

    // Initialise les boutons
    this.initIconButtons();

    // Charge le JS
    this.loadModeJavaScript_(this.mode);

    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions)) {
      if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.columns)) {

        // Enregistre le titre traduit
        for (let i = 0; i < this.oDataGridOptions.datagridOptions.columns.length; i++) {
          const column = this.oDataGridOptions.datagridOptions.columns[i];

          // Index par défaut de la colonne
          column.index = i;

          // Traduction
          column.translateTitle = this.translate.instant(column.title);

          column.data = column.field;

          // Visibilité de la colonne
          column.visible = column.visible === false ? false : true;

          // Colonne type
          column.type = column.template ? 'html' : 'string';

          // Alignement du contenu de la colonne.
          column.className = 'table-align-' + (column.align ? column.align : 'left');
        }


        // Initialise la table
        setTimeout(() => {

          this.oDataGridOptions.datatableOptions.tableHeight = calcDataTableHeight();

          // Création de la grille.
          this.oDataGrid.initTable(this.oDataGridOptions);
        });
      }
    }
  }

  /**
   * Affiche le formulaire correspondant
   * @param  {any} oRow
   * @param  {String} sFormName
   */
  private displayRowForm_(oRow: any, sFormName: String) {
    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions)) {
      if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.id_field)) {
        if (this.vitisCheck.isDefAndNotNull(oRow)) {
          if (this.vitisCheck.isDefAndNotNull(oRow[this.oDataGridOptions.datagridOptions.id_field])) {
            if (this.vitisCheck.isDefAndNotNull(sFormName)) {

              const sId = oRow[this.oDataGridOptions.datagridOptions.id_field];
              const aModeRoute = this.mode.route.split('/');

              // Avec sections
              if (this.vitisCheck.isDefAndNotNull(this.mode.sections)) {
                if (this.vitisCheck.isDefAndNotNull(this.mode.sections[0])) {
                  if (this.vitisCheck.isDefAndNotNull(this.mode.sections[0].route)) {

                    const aSectionRoute = this.mode.sections[0].route.split('/');
                    for (let i = aSectionRoute.length - 1; i >= 0; i--) {
                      if (aSectionRoute[i] === ':id') {
                        aSectionRoute[i] = sId;
                      }
                      if (aSectionRoute[i] === ':form') {
                        aSectionRoute[i] = sFormName;
                      }
                      if (aModeRoute.indexOf(aSectionRoute[i]) !== -1) {
                        aSectionRoute.splice(i, 1);
                      }
                    }
                    this.router.navigate(aSectionRoute, { relativeTo: this.route.parent });
                  }
                }
              } else {
                this.router.navigate([sId, sFormName], { relativeTo: this.route.parent });
              }
            }
          }
        }
      }
    }
  }

  /**
   * Ouvre une modale avec tableau pour ajouter un élément
   */
  private displayDataGridSource(): void {
    this.modalTitleSourceDataGrid = this.mode.options.source.titleModalInsert;
    // Permet de trigger le statut de la modale depuis la modale elle même
    $('#workspacerelationlist-form-modal').addClass('show').trigger('classChange');
    $('#workspacerelationlist-form-modal').modal('show');
    this.openDataSourceModal = true;
    this.service.modalDataSourceGrid.next(true);
  }

  /**
   * Update les données du tableau modifier selon la ressource
   */
  private updateDataGrid(): void {
    if (!this.dataTableUpdated.length) {
      return vitisAlert.alert({
        type: 'error',
        title: this.translate.instant('LIST_NO_ELEMENT_SELECTED'),
      });
    }
    const options = {
      // Permet la gestion de modifition dans la grid pour les champs de type ressource et de type source
      source: {
        url: this.mode.options.source.ressource,
        field: this.mode.options.source.id_field
      },
      ressource: {
        url: this.processUrl(),
        field: this.mode.options.id_field
      }
    };
    const updatedArr = this.dataTableUpdated.reduce((acc, curr) => {
      if (curr[options.ressource.field]) {
        return { ...acc, ressource: [...acc.ressource, curr] };
      } else if (curr[options.source.field]) {
        return { ...acc, source: [...acc.source, curr] };
      }
    }, { source: [], ressource: [] });
    for (const key in updatedArr) {
      if (updatedArr[key].length) {
        // Lancement de la requête PUT
        this.vitisRequest.ajaxRequestPromise({
          method: 'PUT',
          url: environment.properties.app_api_url + '/' + options[key].url,
          data: {
            data: updatedArr[key]
          }
        }).then((resp) => {
          if (resp.data) {
            if (resp.data.status === 1) {
              this.translate.get('LIST_EDIT_OK').subscribe((translation) => {
                vitisAlert.fireToast({
                  icon: 'success',
                  title: translation
                });
              });
            } else {
              vitisAlert.alert({
                type: 'error',
                title: this.translate.instant('LIST_NO_RESSOURCE_SOURCE'),
              });
            }
          }
        });
      }
    }
  }

  /**
   * Déclenche le processus de suppression des lignes sélectionnées
   */
  private deleteSelectedRows_(): void {

    const aIds = this.oDataGrid.getSelectedIds_();

    if (!(aIds.length > 0)) {
      vitisAlert.alert({
        type: 'error',
        title: this.translate.instant('LIST_DELETE_NO_SELECTION'),
      });
    } else {

      if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions)) {
        if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.ressource)) {

          // Chemin de la ressource REST
          let sRessourceURL = environment.properties.app_api_url;
          sRessourceURL += '/' + this.oDataGridOptions.datagridOptions.ressource;

          vitisAlert.alert({
            type: 'warning',
            title: this.translate.instant('LIST_DELETE_CONFIRM'),
            showCancelButton: true
          }).then((result) => {
            if (result.value) {
              this.runDelete_(sRessourceURL, aIds);
            }
          });
        }
      }
    }
  }

  /**
   * Lance une requête delete sur l'API
   * @param  {string} sRessourceURL
   * @param  {Array<string>} aIds
   * @returns Promise
   */
  private runDelete_(sRessourceURL: string, aIds: Array<string>): Promise<void> {
    return new Promise((resolve, reject) => {

      // Lancement de la requête DELETE
      this.vitisRequest.ajaxRequestPromise({
        method: 'DELETE',
        url: sRessourceURL,
        data: {
          idList: aIds
        }
      }).then((resp) => {
        if (resp.data) {
          if (resp.data.status === 1) {

            // Message succès
            this.translate.get('LIST_DELETE_OK').subscribe((translation) => {
              vitisAlert.fireToast({
                icon: 'success',
                title: translation
              });
              resolve();
            });
          } else {
            reject();
          }

          // Recharge les données
          this.oDataGrid.reloadTable();
        }
      });
    });
  }

  /**
   * Initialise les boutons
   * @returns void
   */
  public initButtons(): void {
    const aButtons: any = [];

    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions)) {
      // Bouton insert
      if (this.oDataGridOptions.datagridOptions.insert === true) {
        aButtons.unshift({
          type: 'button',
          class: 'btn-primary',
          label: 'LIST_BUTTON_ADD',
          icon: 'fa fa-plus',
          event: () => {
            this.displayDataGridSource();
          }
        });
      }

      if (this.oDataGridOptions.datagridOptions.edit === true) {
        aButtons.unshift({
          type: 'button',
          class: 'btn-secondary',
          label: 'LIST_BUTTON_SAVE',
          icon: 'fa fa-save',
          event: () => {
            this.updateDataGrid();
          }
        });
      }

      // Bouton delete
      if (this.oDataGridOptions.datagridOptions.delete === true) {
        aButtons.unshift({
          type: 'button',
          class: 'btn-default',
          label: 'LIST_BUTTON_DELETE',
          icon: 'fa fa-trash',
          event: () => {
            this.deleteSelectedRows_();
          }
        });
      }

      // Autres boutons
      if (this.vitisCheck.isArray(this.oDataGridOptions.datagridOptions.buttons)) {
        // Ajoute les boutons
        for (let i = 0; i < this.oDataGridOptions.datagridOptions.buttons.length; i++) {
          const button = this.oDataGridOptions.datagridOptions.buttons[i];
          this.oDataGrid.initButtonEvent_(button, this);
          this.oDataGrid.initButtonDisplay_(button, this);
          aButtons.unshift(button);
        }
      }
    }

    this.oDataGridOptions.buttons = aButtons;
  }

  /**
   * Initialise les boutons présent dans chaque ligne du tableau
   */
  public initIconButtons(): void {
    const aButtons: any = [];
    let buttonUpdateExist = false;
    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.update) &&
      this.oDataGridOptions.datagridOptions.update === true) {
      aButtons.push({
        type: 'update',
        icon: 'icon-vitis-draw',
        event: (data: any) => {
          this.displayRowForm_(data, 'update');
        }
      });
      buttonUpdateExist = true;
    }

    if (this.vitisCheck.isDefAndNotNull(this.oDataGridOptions.datagridOptions.display) &&
      this.oDataGridOptions.datagridOptions.display === true) {
      // Vérifie qu'il y a un bouton update
      if (environment.properties.data_grid_update_btn_only === true && buttonUpdateExist) {
        this.oDataGridOptions.datagridOptions.display = false;
      } else {
        aButtons.push({
          type: 'display',
          icon: 'icon-vitis-search',
          event: (data: any) => {
            this.displayRowForm_(data, 'display');
          }
        });
      }
    }

    this.oDataGridOptions.iconButtons = aButtons;
  }

  /**
   * Charge le JS associé au mode
   * @param  {any} oMode
   * @returns Promise
   */
  private loadModeJavaScript_(oMode: any): Promise<boolean> {
    return new Promise((resolve, reject) => {

      // Si le mode contient un JS associé
      if (this.vitisCheck.isDefAndNotNull(oMode.options)) {
        if (this.vitisCheck.isDefAndNotNull(oMode.options.js_path)) {

          // Charge le JS du mode
          this.vitisRequest.loadExternalJs(oMode.options.js_path).then(() => {

            // Lance le JS associé
            if ((<any>window).constructor_form) {
              (<any>window).constructor_form(this, environment, {
                vitisRequest: this.vitisRequest,
                vitisCheck: this.vitisCheck,
                vitisAlert: vitisAlert,
                auth: this.auth,
                translate: this.translate
              });
            }

            resolve(true);
          });
        } else {
          resolve(false);
        }
      } else {
        resolve(false);
      }
    });
  }

  /**
   * Retourne le nombre d'éléments à afficher
   */
  private getPageLength_(): Number {
    let pageLength = this.pageLength;

    // SessionStorage
    if (this.sessionStorageValues_ &&
      this.sessionStorageValues_.datatableParams) {

      // length
      if (this.vitisCheck.isDefAndNotNull(this.sessionStorageValues_.datatableParams.length)) {
        pageLength = this.sessionStorageValues_.datatableParams.length;
      }
    }

    // Paramètres URL
    if (this.urlParams_ &&
      this.urlParams_.wkgstart &&
      this.urlParams_.wkglength) {
      pageLength = Number(this.urlParams_.wkglength);
    }

    // Concordance avec le tableau de choix
    const aLengthMenu = this.getLengthMenu_();
    if (aLengthMenu.indexOf(pageLength) === -1) {
      if (pageLength > aLengthMenu[aLengthMenu.length - 1]) {
        pageLength = aLengthMenu[aLengthMenu.length - 1];
      } else {
        pageLength = 20;
      }
    }

    return pageLength;
  }

  /**
   * Retourne le tableau contenant le nombre de lignes affichables
   */
  private getLengthMenu_(): Array<Number> {

    const lengthMenu = [5, 10, 20, 50, 100];

    if (this.oDataGridOptions &&
      this.oDataGridOptions.datagridOptions &&
      this.oDataGridOptions.datagridOptions.max_rows) {

      const iMaxRows = this.oDataGridOptions.datagridOptions.max_rows;

      for (let i = lengthMenu.length - 1; i >= 0; i--) {
        if (lengthMenu[i] > iMaxRows) {
          lengthMenu.splice(i, 1);
        }
      }
    }

    return lengthMenu;
  }

  /**
   * Charge le formulaire
   * @param oConfig
   * @param oConfig.form_path chemin vers le formulaire
   * @param oConfig.mode inser/update/display
   * @param oConfig.eventContainer eventContainer
   */
  public compileFormReaderModal(oConfig: any) {
    if (!this.vitisCheck.isDefAndNotNull(oConfig.form_path)) {
      console.error('Empty form_path for modal');
      return;
    }
    const oFormMode = this.vitisCheck.isDefAndNotNull(oConfig.mode) ? oConfig.mode : 'insert';
    // Chargement formulaire
    const url = oConfig.form_path;
    // Récupère le formulaire
    this.getJSONForm_(oConfig.form_path).then((oForm) => {
      if (this.vitisCheck.isDefAndNotNull(oConfig.eventContainer)) {
        oForm[oFormMode].eventContainer = oConfig.eventContainer;
      }
      this.oFormReader.initForm(oForm, oFormMode);
    });
  }

  /**
   * affiche la modale contenant le formreader
   */
  public showFormReaderModal() {
    $('#workspacelist-form-modal').modal('show');
  }

  /**
   * affiche la modale contenant le formreader
   */
  public hideFormReaderModal() {
    $('#workspacelist-form-modal').modal('hide');
  }

  /**
 * Récupère la définition JSON d'un formulaire
 * @param  {string} url
 * @returns Promise
 */
  private getJSONForm_(url: string): Promise<any> {
    return new Promise((resolve, reject) => {
      // Récupère le formulaire
      this.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url
      }).then((resp) => {
        if (this.vitisCheck.isDefAndNotNull(resp.data)) {
          resolve(resp.data);
        } else {
          console.error('Error getting JSON form');
          reject(false);
        }
      });
    });
  }

}
