/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  /**
  * Fonction appelée à l'affichage du formulaire.
  */
  scope.initConfigurationCustomTranslationForm = function (oFormValues) {
    // Pas de traduction personnalisée dans la base.
    if (!services.vitisCheck.isDefAndNotNull(oFormValues.custom_translation_id)) {
      var sTranslationId = String(scope.oParams.id).toUpperCase();
      // Traduction Française.
      var oTextFrField = scope.getFormField('text_fr');
      var subscriptionTextFr = oTextFrField.oninit.subscribe(() => {
        setTimeout(function () {
          oTextFrField.setValue(services.translate.translations['fr'][sTranslationId]);
          // Supprime la subscription à l'observable.
          subscriptionTextFr.unsubscribe();
        });
      });
      // Traduction Anglaise.
      var oTextEnField = scope.getFormField('text_en');
      var subscriptionTextEn = oTextEnField.oninit.subscribe(() => {
        setTimeout(function () {
          oTextEnField.setValue(services.translate.translations['en'][sTranslationId]);
          // Supprime la subscription à l'observable.
          subscriptionTextEn.unsubscribe();
        });
      });
    }
  }

  /**
  * Fonction appelée avant l'envoi du formulaire.
  */
  scope.beforeConfigurationCustomTranslationForm = function (oFormValues) {
    oFormValues.translation_id = String(scope.oParams.id).toUpperCase();
    return true;
  }

  /**
  * Fonction appelée après l'envoi du formulaire.
  */
  scope.afterConfigurationCustomTranslationForm = function (oFormValues) {
    // Remplace les traductions par les personnalisées.
    let oTranslations = { ...services.translate.translations };
    for (const sLang in oTranslations) {
      oTranslations[sLang][oFormValues.translation_id] =
        oFormValues['text_' + sLang];
      services.translate.setTranslation(sLang, oTranslations[sLang]);
    }
    // Rechargement de la grille et des traductions personnalisées.
    if (services.vitisCheck.isDefAndNotNull(services.CustomTranslationService.dataTableInstance)) {
      services.CustomTranslationService.dataTableInstance.reloadTable();
    }
    // Retour à la liste.
    services.SimpleformComponent.setMode('search');
  }
}