/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  /**
   * Fonction appelée avant l'envoi du formulaire
   * @return {boolean|promise}
   */
  scope.beforeEvent = function () {
    const oFormValues = scope.getFormValues();
    return true
  }

  /**
   * Fonction appelée après l'envoi du formulaire
   * @return {boolean|promise}
   */
  scope.afterEvent = function () {
    const oFormValues = scope.getFormValues();
    return true;
  }

  /**
   * Interdit à l'utilisateur en cours de s'enlever les privilèges liés à vitis
   */
  scope.getPrivilegesFilter = function () {
    // Privilèges que l'on ne peut pas s'enlever soi-même
    const unallowedPrivileges = ['vitis_user', 'vitis_admin'];
    const oFormValues = scope.getFormValues();

    // Si le privilège édité est un de ceux interdits alors ajoute un filtre pour ne pas voir son propre utilisateur
    if (unallowedPrivileges.indexOf(oFormValues.rolname) !== -1) {
      return {
        "column": "user_id",
        "compare_operator": "!=",
        "value": services.auth.user_id
      }
    }
  }

  /**
   * Suppression d'un privilège
   */
  scope.deletePrivilege = function () {
    const aIds = scope.oDataGrid.getSelectedIds_();
    if (!(aIds.length > 0)) {
      services.vitisAlert.alert({
        type: 'error',
        title: services.translate.instant('LIST_DELETE_NO_SELECTION'),
      });
    } else {
      services.translate.get(['PRIVILEGE_DELETE_ALERT',
        'PRIVILEGE_DELETE_ALERT_YES',
        'PRIVILEGE_DELETE_ALERT_NO',
        'PRIVILEGE_DELETE_ALERT_CANCEL',
      ]).subscribe((translations) => {
        services.vitisAlert.alert({
          title: translations['PRIVILEGE_DELETE_ALERT'],
          type: 'warning',
          showConfirmButton: true,
          showDenyButton: true,
          showCancelButton: true,
          confirmButtonColor: '#d33',
          denyButtonColor: '#2778c4',
          confirmButtonText: translations['PRIVILEGE_DELETE_ALERT_YES'],
          denyButtonText: translations['PRIVILEGE_DELETE_ALERT_NO'],
          cancelButtonText: translations['PRIVILEGE_DELETE_ALERT_CANCEL'],
        }).then((result) => {
          if (!result.isDismissed) {
            const sRessourceURL = result.value === true ? env.properties.app_api_url + '/vitis/privilegeswithrole' : env.properties.app_api_url + '/vitis/privileges';
            services.vitisRequest.ajaxRequestPromise({
              method: 'DELETE',
              url: sRessourceURL,
              data: {
                idList: aIds
              }
            }).then((resp) => {
              if (resp.data) {
                if (resp.data.status === 1) {

                  // Message succès
                  services.translate.get('LIST_DELETE_OK').subscribe((translation) => {
                    services.vitisAlert.fireToast({
                      icon: 'success',
                      title: translation
                    });
                  });
                }

                scope.oDataGrid.dataTableInstance.ajax.reload();
              }
            });
          }
        });
      });
    }
  }
}