/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
 var constructor_form = function (scope, env, services) {

  /**
   * Initialisation des valeurs par défaut
   */
  scope.initEvent = function () {

    // Affichage de "Se souvenir de moi" d'apprès les properties
    if (this.vitisCheck.isDefAndNotNull(env.properties.login_remember_me)) {
      const oRememberField = scope.getFormField('remember_me');
      oRememberField.visible = env.properties.login_remember_me === true ? true : false;
    }

    // Valeur de "Se souvenir de moi" d'apprès les properties
    if (this.vitisCheck.isDefAndNotNull(env.properties.login_remember_me_default)) {
      scope.oFormValues.patchValue({
        remember_me: env.properties.login_remember_me_default === true ? true : false
      });
    }

    // Affichage de "Mot de passe oublié" d'apprès les properties
    if (this.vitisCheck.isDefAndNotNull(env.properties.password_forgotten)) {
      const oForgotenField = scope.getFormField('forgotten_password');
      oForgotenField.visible = env.properties.password_forgotten === true ? true : false;
    }
  }

  /**
   * Fonction appelée après l'envoi du formulaire
   * @return {boolean|promise}
   */
  scope.afterEvent = function () {

    // Cache la modale
    $('#dashboard-simpleform-modal').modal('hide');

    // Rafraichit la page en cours
    services.router.navigate(['/login']).then(() => {
      services.router.navigate(['**'], {
        queryParamsHandling: 'merge'
      });
    });

    return true
  }

  /**
   * Login avec les nouvelles infos
   * @returns 
   */
  scope.login = function () {
    return new Promise((resolve, reject) => {
      const oFormValues = scope.getFormValues();

      if (oFormValues.user_login && oFormValues.user_password) {
        let login = oFormValues.user_login;
        // if (this.suffix != null && this.suffix.length > 0) {
        //   login = oFormValues.user_login + this.suffix;
        // }
        services.auth.login(login, oFormValues.user_password, oFormValues.remember_me)
          .subscribe(
            (result) => {
              if (result === true) {

                // Relance le chargement des infos utilisateur
                services.auth.refreshUserInfos().then(() => {

                  // Set isPublicUser = false
                  const oUserInfos = services.auth.getUserInfos();
                  oUserInfos.isPublicUser = false;
                  services.auth.setUserInfos(oUserInfos);

                  resolve();
                });

              } else {
                // Message d'erreur à afficher.
                let sErrorTranslationId = 'ERROR_LOGIN';
                if (services.vitisCheck.isDefAndNotNull(result.data) &&
                  services.vitisCheck.isDefAndNotNull(result.data.errorMessage)) {
                  sErrorTranslationId = result.data.errorMessage;
                }
                // Affichage du message d'erreur.
                services.vitisAlert.alert({
                  text: services.translate.instant(sErrorTranslationId),
                  type: 'error'
                });
              }
            }
          );
      }
    });
  }

  /**
   * Affiche le formulaire de mot de passe oublié
   */
  scope.showForgottenPasswordForm = function (sForm) {

    const oFields = [];
    if (sForm === 'email') {
      this.sModalTitle = 'FORM_LOGIN_FORGET_PASSWORD';
      oFields.push({
        fields: [{
          type: 'email',
          name: 'fpwd_email',
          label: 'SIGNUPMAIL',
          nb_cols: 12,
          required: true,
          id: 'domain_1_1',
          pattern: '^(([a-zA-Z]|[0-9])|([-]|[_]|[.]))+[@](([a-zA-Z0-9])|([-])){2,63}[.](([a-zA-Z0-9]){2,63})+$'
        }]
      });
    }

    if (sForm === 'login') {
      this.sModalTitle = 'FORM_LOGIN_FORGET_PASSWORD';
      oFields.push({
        fields: [{
          type: 'subtitle',
          name: 'email_several_exist',
          label: 'SEVERAL_MAIL_FIND',
          nb_cols: 12,
        }, {
          type: 'text',
          name: 'fpwd_login',
          id: 'login_form_user_login',
          label: 'FORM_LOGIN_USER',
          autofocus: true,
          required: true,
          nb_cols: 12,
          pattern: '^[^@]+$'
        }]
      });
    }

    oFields.push({
      fields: [{
        type: 'button',
        class: 'btn-ungroup btn-group-sm',
        nb_cols: 12,
        buttons: [{
          type: 'submit',
          name: 'btn_send',
          label: 'FORM_FORGOTTEN_PASSWORD_SEND',
          class: 'btn-primary btn-block'
        }]
      }]
    });

    const oForm = {
      update: {
        name: 'vitis_forgotten_password_update_form',
        eventContainer: this,
        event: 'sendFormForgottenPassword()',
        input_size: 'xs',
        nb_cols: 12,
        rows: oFields
      }
    };

    // Remplace la définition du formulaire
    scope.initForm(oForm, 'update');
  }

  /**
   * Lance la requête de mot de passe oublié
   */
  scope.sendFormForgottenPassword = function () {
    if (scope.isFormValid()) {
      const values = scope.getFormValues();

      // Récupération des données du form
      const formData = services.formreaderSrvc.transformToDataForm(values);

      services.vitisRequest.ajaxRequest({
        method: 'POST',
        url: env.properties.app_api_url + '/vitis/accounts/fpwd',
        data: formData,
        displayErrorMessage: false,
        success: (resp) => {
          if (resp.data && resp.data.status == 1) {
            // Le mail a été envoyé
            services.translate.get('CHECKMAIL').subscribe((translate) => {
              services.vitisAlert.fireToast({
                type: 'success',
                title: translate
              });

              // Ferme la modale
              scope.afterEvent();
            });
          }
        },
        error: (resp) => {
          if (resp.data) {
            if (resp.data.status == 0 && resp.data.errorMessage === 'SEVERAL_USER_FOUNDED') {
              scope.showForgottenPasswordForm('login');
            } else {
              services.translate.get(resp.data.errorMessage).subscribe((translate) => {
                services.vitisAlert.alert({
                  type: 'error',
                  text: translate
                });
              });
            }
          } else {
            services.translate.get('ERROR_SERVER_REQUEST_500').subscribe((translate) => {
              services.vitisAlert.alert({
                type: 'error',
                text: translate
              });
            });
          }
        }
      });
    }
  }
}
