/**
 * Exemple récupération puis modification de la valeur d'un champ
 *
 * Fonction lancée au chargement du formulaire
 * @param { object } scope Formreader
 * @param { object } env environment
 * @param { object } services
 * @returns { undefined } 
 */
var constructor_form = function (scope, env, services) {

  // Permet de debug depuis la console du navigateur
  window.scope = scope;
  window.env = env;
  window.services = services;

  var debounceTimer;
  var debounce = function (callback, delay) {
    var args = arguments;
    var context = this;
    clearTimeout(debounceTimer);
    debounceTimer = setTimeout(function () {
      callback.apply(context, args);
    }, delay)
  }

  var initialValues; // Valeurs à l'initialisation

  scope.initEvent = function (oFormValues) {
    oFormValues.parent1 = services.vitisCheck.isString(oFormValues.parent1) ? oFormValues.parent1.split('|') : [];
    oFormValues.parent2 = services.vitisCheck.isString(oFormValues.parent2) ? oFormValues.parent2.split('|') : [];
    initialValues = JSON.parse(JSON.stringify(oFormValues));

    scope.getFormField('values').firstload = true;
    scope.getFormField('values').initialValue = initialValues.values;

    scope.getFormField('parent1').firstload = true;
    scope.getFormField('parent1').initialValue = initialValues.parent1;

    scope.getFormField('parent2').firstload = true;
    scope.getFormField('parent2').initialValue = initialValues.parent2;
  }

  /**
   * Fonction appelée en beforeEvent
   * @param {*} oFormValues 
   * @returns 
   */
  scope.beforeEvent = function (oFormValues) {
    return new Promise((resolve, reject) => {
      oFormValues.parent1 = services.vitisCheck.isArray(oFormValues.parent1) ? oFormValues.parent1.join('|') : null;
      oFormValues.parent2 = services.vitisCheck.isArray(oFormValues.parent2) ? oFormValues.parent2.join('|') : null;
      setTimeout(() => {
        resolve(oFormValues);
      })
    });
  }

  /**
   * Fonction appelée au changement de restriction
   */
  scope.onRestrictionChange = function () {
    debounce(function () {
      const oValues = scope.getFormValues();
      if (oValues.restriction_type_id && Number(oValues.restriction_type_id) > 0) {
        scope.getRestrictionTypeDef(Number(oValues.restriction_type_id)).then(oRestrictionType => {

          scope.getFormField('parent1').visible = false;
          scope.getFormField('parent2').visible = false;

          scope.oRestrictionType = oRestrictionType;

          if (oRestrictionType.parent2_child_column &&
            oRestrictionType.parent2_restriction_alias &&
            oRestrictionType.parent2_restriction_column &&
            oRestrictionType.parent2_restriction_db &&
            oRestrictionType.parent2_restriction_schema &&
            oRestrictionType.parent2_restriction_table &&
            oRestrictionType.parent1_child_column &&
            oRestrictionType.parent1_restriction_alias &&
            oRestrictionType.parent1_restriction_column &&
            oRestrictionType.parent1_restriction_db &&
            oRestrictionType.parent1_restriction_schema &&
            oRestrictionType.parent1_restriction_table) {

            scope.getFormField('parent2').visible = true;
            scope.getFormField('parent1').visible = true;
            scope.getRestrictionValues({
              restriction_db: oRestrictionType.parent2_restriction_db,
              restriction_schema: oRestrictionType.parent2_restriction_schema,
              restriction_table: oRestrictionType.parent2_restriction_table,
              restriction_column: oRestrictionType.parent2_restriction_column,
              restriction_alias: oRestrictionType.parent2_restriction_alias,
            }).then(oValues => {
              scope.setFieldOptions(scope.getFormField('parent2'), oValues.map(obj => ({
                label: obj[scope.oRestrictionType.parent2_restriction_alias],
                value: obj[scope.oRestrictionType.parent2_restriction_column],
                selected: false
              })));
            });

          } else if (oRestrictionType.parent1_child_column &&
            oRestrictionType.parent1_restriction_alias &&
            oRestrictionType.parent1_restriction_column &&
            oRestrictionType.parent1_restriction_db &&
            oRestrictionType.parent1_restriction_schema &&
            oRestrictionType.parent1_restriction_table) {

            scope.getFormField('parent1').visible = true;
            scope.getRestrictionValues({
              restriction_db: oRestrictionType.parent1_restriction_db,
              restriction_schema: oRestrictionType.parent1_restriction_schema,
              restriction_table: oRestrictionType.parent1_restriction_table,
              restriction_column: oRestrictionType.parent1_restriction_column,
              restriction_alias: oRestrictionType.parent1_restriction_alias,
            }).then(oValues => {
              scope.setFieldOptions(scope.getFormField('parent1'), oValues.map(obj => ({
                label: obj[scope.oRestrictionType.parent1_restriction_alias],
                value: obj[scope.oRestrictionType.parent1_restriction_column],
                selected: false
              })));
            });

          } else {
            scope.getRestrictionValues(oRestrictionType).then(oValues => {
              scope.setFieldOptions(scope.getFormField('values'), oValues.map(obj => ({
                label: obj[scope.oRestrictionType.restriction_alias],
                value: obj[scope.oRestrictionType.restriction_column],
                selected: false
              })));
            })
          }
        });
      }
    }, 250);
  }

  /**
   * Lorsque parent2 change
   * @param {*} oParent2Values 
   */
  scope.onParent2Change = function (oParent2Values) {
    if (scope.oRestrictionType) {
      debounce(function () {
        oParent2Values = scope.getFormField('parent2').getValue();
        if (services.vitisCheck.isArray(oParent2Values) && oParent2Values.length > 0) {
          scope.getFormField('parent1').visible = true;
          scope.getRestrictionValues({
            restriction_db: scope.oRestrictionType.parent1_restriction_db,
            restriction_schema: scope.oRestrictionType.parent1_restriction_schema,
            restriction_table: scope.oRestrictionType.parent1_restriction_table,
            restriction_column: scope.oRestrictionType.parent1_restriction_column,
            restriction_alias: scope.oRestrictionType.parent1_restriction_alias,
          }, {
            relation: 'AND',
            operators: [{
              column: scope.oRestrictionType.parent2_child_column,
              compare_operator: "IN",
              value: oParent2Values
            }]
          }).then(oValues => {
            scope.setFieldOptions(scope.getFormField('parent1'), oValues.map(obj => ({
              label: obj[scope.oRestrictionType.parent1_restriction_alias],
              value: obj[scope.oRestrictionType.parent1_restriction_column],
              selected: false
            })));
          });
        } else {
          scope.setFieldOptions(scope.getFormField('parent1'), []);
        }
      }, 500);
    }
  }

  /**
   * Lorsque parent1 change
   * @param {*} oParentValues 
   */
  scope.onParentChange = function (oParentValues) {
    if (scope.oRestrictionType) {
      debounce(function () {
        oParentValues = scope.getFormField('parent1').getValue();
        if (services.vitisCheck.isArray(oParentValues) && oParentValues.length > 0) {
          scope.getRestrictionValues(scope.oRestrictionType, {
            relation: 'AND',
            operators: [{
              column: scope.oRestrictionType.parent1_child_column,
              compare_operator: "IN",
              value: oParentValues
            }]
          }).then(oValues => {
            scope.setFieldOptions(scope.getFormField('values'), oValues.map(obj => ({
              label: obj[scope.oRestrictionType.restriction_alias],
              value: obj[scope.oRestrictionType.restriction_column],
              selected: false
            })));
          })
        } else {
          scope.setFieldOptions(scope.getFormField('values'), []);
        }
      }, 500);
    }
  }

  /**
   * Change les options du champ
   * @param {*} oField 
   * @param {*} oOptions 
   */
  scope.setFieldOptions = function (oField, oOptions) {
    let aValue = [];
    if (oField.firstload && oField.initialValue && oOptions.length > 0) {
      aValue = oField.initialValue;
      oField.firstload = false;
      oField.initialValue = [];
    } else {
      aValue = JSON.parse(JSON.stringify(oField.getValue()));
    }

    oField.setOptions(oOptions);
    setTimeout(() => {
      oField.setValue(aValue);
    });
  }

  /**
   * Récupère les infos d'un type de restriction
   * @param {*} restriction_type_id 
   */
  scope.getRestrictionTypeDef = function (restriction_type_id) {
    return new Promise((resolve, reject) => {
      services.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url: env.properties.app_api_url + '/vitis/restriction_types/' + restriction_type_id,
        params: {}
      }).then((resp) => {
        if (resp.data && resp.data.data) {
          resolve(resp.data.data)
        }
      });
    });
  }

  /**
   * Récupère les valeurs de restriction
   * @param {*} oRestrictionType 
   * @returns 
   */
  scope.getRestrictionValues = function (oRestrictionType, oFilter = null) {
    return new Promise((resolve, reject) => {

      const params = {
        database: oRestrictionType.restriction_db,
        schema: oRestrictionType.restriction_schema,
        table: oRestrictionType.restriction_table,
        attributs: oRestrictionType.restriction_column + "|" + oRestrictionType.restriction_alias
      }

      if (oFilter) {
        params.filter = oFilter
      }

      services.vitisRequest.ajaxRequestPromise({
        method: 'GET',
        url: env.properties.app_api_url + '/vitis/genericquerys',
        params
      }).then((resp) => {
        if (resp.data && resp.data.data) {
          resolve(resp.data.data)
        }
      });
    });
  }
}