/**
 * constructor_form
 * Fonction lancée au chargement du formulaire
 * @param {object} scope Formreader
 * @param {object} env environment
 * @param {object} services
 * @returns {undefined} 
 */
var constructor_form = function (scope, env, services) {

  /**
   * Fonction appelée avant l'envoi du formulaire
   * @return {boolean|promise}
   */
  scope.beforeEvent = function () {
    return new Promise((resolve, reject) => {
      const oFormValues = scope.getFormValues();

      // Test un administrateur ne peut s'enlever vitis_user
      if (oFormValues.user_id === services.auth.user_id) {
        if (services.auth.privileges &&
          services.auth.privileges.indexOf('vitis_admin') &&
          this.vitisCheck.isDefAndNotNull(oFormValues.privileges)) {

          const privileges = this.vitisCheck.isString(oFormValues.privileges) ? oFormValues.privileges.split('|') : oFormValues.privileges;

          // Administrateur non vitis_user
          if (privileges.indexOf('vitis_user') === -1) {
            services.translate.get('ERROR_ADMIN_NO_VITIS_USER').subscribe((res) => {
              services.vitisAlert.alert({
                'type': 'error',
                'text': res
              });
            });
            reject();
          }

          // Administrateur non vitis_admin
          if (privileges.indexOf('vitis_admin') === -1) {
            services.translate.get('ERROR_ADMIN_NO_VITIS_ADMIN').subscribe((res) => {
              services.vitisAlert.alert({
                'type': 'error',
                'text': res
              });
            });
            reject();
          }
        }
      }

      // Tests modification mot de passe
      if (services.vitisCheck.isString(oFormValues.password)) {
        if (oFormValues.password.length > 0) {

          // Message d'erreur si le mot de passe et la confirmation sont différents
          if (oFormValues.password !== oFormValues.password_confirm) {
            services.translate.get('ERROR_PASSWORD_CONFIRM_USER_USER').subscribe((res) => {
              services.vitisAlert.alert({
                'type': 'error',
                'text': res
              });
            });
          } else {

            // Si l'utilisateur modifié est l'utilisateur en cours
            if (oFormValues.user_id === services.auth.user_id) {

              // Message warning de changement de mot de passe
              services.translate.get('UPDATE_PASSWORD_USER_USER').subscribe((res) => {
                services.vitisAlert.alert({
                  'type': 'warning',
                  'html': res,
                  'showCancelButton': true
                }).then((result) => {
                  if (result.value) {
                    resolve();
                  }
                });
              });
            } else {
              resolve();
            }
          }
        } else {
          resolve();
        }
      } else {
        resolve();
      }
    });
  }

  /**
   * Fonction appelée après l'envoi du formulaire
   * @return {boolean|promise}
   */
  scope.afterEvent = function () {
    const oFormValues = scope.getFormValues();

    // Si l'utilisateur modifié est l'utilisateur en cours
    if (oFormValues.user_id === services.auth.user_id) {

      // Reset les nouvelles infos utilisateur dans AuthService
      services.auth.setUserInfos(oFormValues);

      // Si l'admin a modifié le mdp
      if (services.vitisCheck.isString(oFormValues.password)) {
        if (oFormValues.password.length > 0) {

          // Déconnecte l'utilisateur
          services.auth.disconnect();
        }
      }
    }

    // Retour à la liste des utilisateurs.
    setTimeout(() => {
      services.SimpleformComponent.setMode('search');
    }, 500);    
    return true
  }

  /**
   * Fonction appelée à l'initialisation du formulaire
   * @return {boolean|promise}
   */
  scope.initEvent = function () {
    setTimeout(function(){
      const oFormValues = scope.getFormValues();
      // Pas d'affichage de la section "Mot de passe" si l'utilisateur est associé à un domaine.
      if (services.vitisCheck.isDefAndNotNull(oFormValues.domain) && scope.sFormDefinitionName == 'update') {
        scope.getFormField('password').disabled = true;
        scope.getFormField('password_confirm').disabled = true;
        scope.oFormDefinition[scope.sFormDefinitionName].tabs.list[2].elements.length = 0;
      }

      // En insert on set la valeur par défaut en fonction des properties
      if (scope.sFormDefinitionName == 'insert') {
        scope.getFormField('formatdate_id').setValue(env.properties.default_formatdate_id ?? 'DD-MM-YYYY');
      }

    }, 500);
    return true
  }
}
