# Composants des modules Vitis

Nous avons vu dans le chapitre précédent que nous pouvions créer des modules Angular dans Vitis et renseigner `app-struct.ts` pour charger ces derniers depuis le menu principal.

Dans ce chapitre nous allons voir comment effectuer la même opération mais en utilisant des composants placés dans des modules de Vitis.

**Remarque:** ne pas confondre **module Vitis** avec **module Angular**

## Arborescence

Les modules Vitis sont placés dans `src/module_***` lors du montage de l'application plusieurs dossiers seront montés à plusieurs endroits.

Dans `client/src/modules` seront placés les contenus FrontEnd du module par exemple pour le module GTF on se retrouvera avec le contenu de `gtf/src/module_gtf/module`.

Dans `client/src/app/modules` seront placés les composants du module par exemple pour le module GTF on se retrouvera avec le contenu de `gtf/src/module_gtf/module/components`. Cette opération est essentielle car Angular a besoin que les fichiers soient placés dans le répertoire `app/` pour pouvoir les compiler.

Une fois les points de montage faits on se retrouve avec une arborescence de ce type

![image](./images/arborescence_modules_components.jpg)

On voit apparaître dans le dossier `src/app/modules` un dossier `gtf` contenant un module Angular, un router ainsi que des composants. Remarquons aussi les fichiers `modules-components.ts` et `modules-components.sample.ts`.

**Remarque:** dans le dossier `conf/` de l'application il y a aussi un fichier `modules-components.ts`

## Fonctionnement

Lors du montage de l'application le fichier `conf/modules-components.ts` et copié dans `src/app/modules`.

Pour comprendre le fonctionnement regardons d'abord le contenu du fichier.

```js
import {
  oGTFModules,
  oGTFModulesComponents,
  oGTFWkListComponents
} from './gtf/gtf.exports';

export const oModules = { ...oGTFModules };
export const oModulesComponents = { ...oGTFModulesComponents };
export const oWkListComponents = { ...oGTFWkListComponents };
```

En réalité ce fichier permet d'exporter les modules et composants Angular des différents modules Vitis.

Vitis va alors charger `oModules`, `oModulesComponents`, `oWkListComponents` et ainsi intégrer les composants du module GTF.

## Utilisation dans app-struct.ts

Une fois les éléments renseignés dans `modules-components.ts` il est possible de les utiliser pour les charger depuis les menus de l'application.

Prennons exemple du mode statistiques de GTF

```js
export const gtfStatisticsMode: AppModeConfig = {
  'name': 'gtf_statistics',
  'label': 'TITLE_MODE_GTF_STATISTICS',
  'icon': 'icon-vitis-statistics',
  'route': 'gtf_statistics',
  'privilege': 'gtf_admin',
  'type': 'module_component',
  'options': {
    'component_name': 'GtfStatisticsComponent',
    'route': 'gtf/statistics',
    'columns': {
      'data': 'length_sec',
      'legend': 'workspace',
      'date': 'execution_date'
    }
  }
};
```

Nous avons utilisé le type `module_component` puis renseigné le composant à utiliser dans l'attribut `component_name`.

## Conclusion

À l'aide du fichier `modules-components.ts` il est possible d'intégrer à Vitis le code des différents modules Vitis.

Ceci permet notamment de charger ces composants depuis `app-struct.ts`.
