# Composants Vitis

Nous avons vu dans le chapitre précédent que le développeur peut définir dans **app.struct.ts** des modes, objets, sections etc.

## Utilisation de app.struct.ts pour charger des composants

En regardant le modèle défini dans `client/src/app/vitis/app.model.ts` on remarque que les objets/modes se définissent notamment avec une route et un type.

Un type peut correspondre à plusieurs entités, il peut être simplement un separator, ou un composant Vitis ou un module Angular voir même un composant de module Vitis. Dans ce chapitre nous nous intéresseront en particulier aux composants Vitis.

Le module Angular routing applicatif `client/src/app/vitis/app-routing.ts` va lors du chargement de l'application parcourir la structure définie dans `app.struct.ts` et créer les routes associées. En décrivant un objet/mode dans la structure applicative nous allons par ce fait créer une nouvelle route pointant sur un composant en particulier.

Prenons l'exemple de l'objet Users permettant de liste l'ensemble des utilisateurs
```js
const vitisUsersUsersObject: AppModeConfig = {
  name: 'vitis_users_users',
  label: 'TITLE_MODE_USERS_LIST',
  route: 'list',
  type: 'workspacelist',
  privilege: 'vitis_admin',
  options: {...}
};
```

On remarque ici qu'on utilise le type workspacelist, c'est donc ce composant qui sera utilisé.

## Structure modulaire et lazy loading

Dans le chapitre précédent nous avons vu que nous pouvions charger un composant Vitis simplement en le mentionnant dans `app.struct.ts`. Dans ce chapitre nous allons voir comment créer un composant et le charger dans la structure.

Pour cela regardons d'abord comment fonctionne `client/src/app/vitis/app-routing.ts` notamment la fonction **getModeRoute_**.

Basiquement il y a un switch/case où nous trouvons plusieurs cas d'utilisation. Le plus simple d'entre eux est le chargement d'un composant contenu dans le module Angular principal de l'application.

Prenons l'exemple de `config_supervision`

```js
case 'config_supervision':
  modeComponent = ConfigurationSupervisionComponent;
  break;
```

Dans ce cas le composant `ConfigurationSupervisionComponent` fait partie du module principal et peut être chargé très simplement.

Le problème c'est le volume de code: en définissant un composant de la sorte, lors de la compilation son code se retrouvera dans le fichier **main.js** chargé au lancement. Anciennement les composants étaient tous définis de la sorte, le problème c'est que cette architecture surcharge **main.js**. La bonne pratique est de créer un module Angular et de l'appeler en lazy loading.

Le lazy loading consiste à affecter un module Angular à une route, lors de la compilation ce module aura son propre fichier `xxx.js` et ce dernier ne sera pas chargé au lancement de l'application mais plutôt lorsque l'utilisateur ira sur la route associée. De cette manière nous pouvons étaler la charge sur plusieurs fichiers rendant l'application plus légère et plus simple à compiler.

Prenons l'exemple de `studio`

```js
case 'studio':
  return [{
    path: oMode.route,
    loadChildren: () => import('./studio/studio.module').then(m => m.StudioModule),
    canActivate: [RoleGuard],
    data: oMode
  }];
  break;
```

Dans le code ci-dessus nous remarquons que lors de l'utilisation de la route `studio` nous chargeons dynamiquement le module Angular `StudioModule`.

## Arborescence et fonctionnement des modules

Lors du changement au lazy loading l’arborescence de Vitis a été changée

![image](./images/arborescence.jpg)

Dans **app/vitis** nous retrouvons un dossier par module puis les fichiers définissant le module app (module Angular principal).

Sur chacun des dossiers se trouve un module Angular qui sera potentiellement chargé en lazy loading.

Prenons exemple sur le module studio, à l'intérieur du dossier nous retrouvons les éléments suivants :
* components: dossier contenant les composants
* services: dossier contenant les services
* studio.component.*: composant principal
* studio.model.ts: contient les différents modèles utilisés dans le module
* **studio.module.ts**: définition du module
* **studio.routing.ts**: router du module

Vous remarquerez que le module contient son propre router, pour fonctionner en lazy loading c'est essentiel, il faut déclarer des sous-routes.

```js
export const routes: Routes = [
  {
    path: '',
    component: StudioComponent,
    canDeactivate: [CanDeactivateGuard]
  }, // composant par défaut
];

export const routing: ModuleWithProviders<any> = RouterModule.forChild(routes);
```

En regardant de plus près le fichier on se rend compte que tout ce qu'il fait c'est charger StudioComponent par défaut. Rien de compliqué mais c'est nécessaire au fonctionnement.

## Conclusion

La structure de Vitis permet de charger n'importe quel composant depuis `app-struct.ts` néanmoins il est impératif de bien organiser son
code et de passer au maximum par des modules Angular chargés lazy loading
