import { Logger } from "vitis-engine-lib";
import { Db } from "vitis-engine-lib";
import { LocalFS } from "vitis-engine-lib";
import fs from 'fs';
import { PlaceHolder } from "./signPdf/placeholder";
import { Signature } from "./signPdf/signature";


/**
 * Génére un excel
 */
export async function signPdf_(oProperties: any, oDb: Db, oLogger: Logger, oLocalFS: LocalFS, args: any) {
  let inputPdfPath: string = args.inputFile;
  let certPath: string = args.certFile;
  let outputPath: string = args.outputFile;

  let reason: string = args.reason;
  let contactInfo: string = args.contact;
  let name: string = args.name;
  let location: string = args.location;
  let signatureLength: number = args.certLength

  const oPlaceholderOptions: any = {
    reason,
    contactInfo,
    name,
    location,
    signatureLength,
  }
 try {

   const oPlaceHolder = new PlaceHolder(inputPdfPath, oPlaceholderOptions);
   const aPdfData: Buffer = oPlaceHolder.getBuffer()
   
   const oSignatureOptions: any = {};
   
   if (typeof args.certPass === 'string'){
     oSignatureOptions['passphrase'] = args.certPass
    }
    
    const oSignature = new Signature(certPath, oSignatureOptions)
    const aData = oSignature.signPdf(aPdfData);
    
    if (aData) {
      fs.writeFileSync(outputPath, aData);
      oLogger.log('Le PDF a été signé', null, 'INFO');
      return true;
    }
  } catch (e) {
    oLogger.log('Une erreur est survenue pendant la signature', e, 'ERROR');
    return false
  }
  oLogger.log('Le PDF n\'a pas pu être signé', null, 'ERROR');
  return false;
}