<?php

use App\Kernel;
use Symfony\Component\ErrorHandler\Debug;
use Symfony\Component\HttpFoundation\Request;

require dirname(__DIR__).'/config/bootstrap.php';

/**
 * Point d'entrée de l'API généré par symfony et légérement customisée.
 *
 * @author Anthony Borghi   <anthony.borghi@veremes.com>
 * @author Armand Bahi   <aramand.bahi@veremes.com>
 */

if ($_SERVER['APP_DEBUG']) {
    umask(0000);

    Debug::enable();
}

if ($trustedProxies = $_SERVER['TRUSTED_PROXIES'] ?? $_ENV['TRUSTED_PROXIES'] ?? false) {
    Request::setTrustedProxies(explode(',', $trustedProxies), Request::HEADER_X_FORWARDED_FOR | Request::HEADER_X_FORWARDED_PORT | Request::HEADER_X_FORWARDED_PROTO );
}

if ($trustedHosts = $_SERVER['TRUSTED_HOSTS'] ?? $_ENV['TRUSTED_HOSTS'] ?? false) {
    Request::setTrustedHosts([$trustedHosts]);
}

$kernel = new Kernel($_SERVER['APP_ENV'], (bool) $_SERVER['APP_DEBUG']);
$request = Request::createFromGlobals();

// Set de manière temporaire le fichier de destination error_log() vers vas/var/log/php.log
// Une fois le logger instancié alors les logs iront vers vas/var/log/[date]/php/error.log
ini_set('error_log', $kernel->getProjectDir() . '/var/log/php.log');

$response = $kernel->handle($request);
$response->send();
$kernel->terminate($request, $response);
