<?php

namespace App\Module\Vitis\WebService\Controller;

use App\Module\Vitis\WebService\Controller\AbstractVitisController;
use App\Module\Vitis\WebService\Service\Configuration\Documentation;
use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Configuration\Route;
use App\Module\Vitis\WebService\Service\DataAccessor\DatabaseConnection;
use App\Module\Vitis\WebService\Service\FileAccessor\FileManager;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use App\Module\Vitis\WebService\Service\VitisRequest;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller retourne la doc de l'API extrait du fichier de routage
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */
class ApiDocController extends AbstractVitisController {
  /**
   * Undocumented variable
   *
   * @var Documentation
   */
  private $oDocParser;

  /**
   * Initialise le controler et instancie les services requis
   * @param Properties $oPropertiesSrvc Service d'accès aux properties
   * @param VitisRequest $oVitisRequest Service d'abstraction de la requête Symfony
   * @param Documentation $oRouteConfig Service d'accès à la configuration de la route et parseur de documentation
   * @param LoggerManager $oLoggerManager Service d'utilsation des logs
   * @param DatabaseConnection $oDatabaseConnection Service d'accès à la base de données
   * @param FileManager $oFileManager Service de gestion des fichiers
   */
  public function __construct(Properties $oPropertiesSrvc, VitisRequest $oVitisRequest, Documentation $oRouteConfig, LoggerManager $oLoggerManager, DatabaseConnection $oDatabaseConnection, FileManager $oFileManager) {
    parent::__construct($oPropertiesSrvc, $oVitisRequest, $oRouteConfig, $oLoggerManager, $oDatabaseConnection, $oFileManager);
    $this->oDocParser = $oRouteConfig;

  }

  /**
   * Retourne la définition swagger de l'api
   *
   * @return Response
   */
  public function getDefiniton(): Response {
    try {
      $aDef = $this->oDocParser->getSwaggerDef();

      return $this->formatResponse(['status' => 1, 'data' => $aDef]);
    } catch (\Exception $e) {
      return $this->formatGenericError('API_DEF_ERROR', '', $e->getMessage());
    }
  }
}
