<?php

namespace App\Module\Vitis\WebService\Controller;

use App\Module\Vitis\WebService\Controller\GenericController;
use App\Module\Vitis\WebService\Service\Mail\Mail;
use App\Module\Vitis\WebService\Service\PrintService\HtmlRenderer;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller qui permet de gérer le dossier partagé.
 *
 * @author Ludivine Premont <ludivine.premont@veremes.com>
 */
class MailTemplateController extends GenericController {
  /**
   * get workspace directory tree
   * @param integer $id
   * @return Response
   */
  public function testMailTemplate($id): Response {
    $aData = $this->requestToArray();

    $aEmailTemplate = $this->getDataWithRequestParams($aData, $id);

    // Si le body n'est pas défini alors prend celui du template
    if (empty($aData['body'])) {
      $aData['body'] = $aEmailTemplate['data']['body'];
    }

    if (empty($aData['body'])) {
      return $this->formatGenericError('FORM_TEST_MAIL_EMPTY_PUBLICATION_EMAIL_TEMPLATE_EMAIL_TEMPLATE_SIMPLE_EDITION',
        '', 'The message of the mail template is empty', Response::HTTP_OK);
    }

    try {
      // Récupère les données du template de mail.
      $qb = $this->oDatabaseConnection->getQueryAccessor()->createQueryBuilder();
      $qb->select('context_documentation', 'name', 'body', 'json_scope', 'json_includes')
        ->from('[SCHEMA]."v_email_template"', 'vet')
        ->where($qb->expr()->eq('vet.email_template_id', '[TEMPL_ID]'));

      $aParams = [
        'SCHEMA' => ['value' => $this->oProperties->get('schema_vitis'), 'type' => 'schema_name'],
        'TEMPL_ID' => ['value' => $id, 'type' => 'integer'],
      ];
      try {
        $aResult = $qb->executeWithParams($aParams)->fetch();
      } catch (\Exception $e) {
        $this->oLoggerManager->get('SQL_ERROR')->error('Error getting mail template parameters' . $e->getMessage());
        return $this->formatGenericErrorRequestError('Error getting mail template parameters', Response::HTTP_NOT_FOUND);
      }

      // Données d'exemple.
      $aContextDoc = json_decode($aResult['context_documentation'], true);
      $aExamples = [];
      foreach ($aContextDoc as $aParam) {
        if (isset($aParam['example'])) {
          $aExamples[$aParam['label']] = $aParam['example'];
        }
      }

      // get HTML body
      $sHTMLBody = $aData['body'];

      // get scope
      if (empty($aData['scope'])) {
        $aScope = json_decode($aResult['json_scope'], true);
        if (!is_array($aScope)) {
          $aScope = [];
        }

        // merge avec données bouchons
        $aScope = array_merge_recursive($aExamples, $aScope);
      } else {
        $aScope = is_string($aData['scope']) ? json_decode($aData['scope'], true) : $aData['scope'];
      }

      // get include
      $aIncludes = json_decode($aResult['json_includes'], true);
      if (!is_array($aIncludes)) {
        $aIncludes = [];
      }

      // Conversion des données en php pour les variables
      foreach ($aScope as $sKey => $value) {
        if (strpos($sKey, '.')) {
          $aParam = explode('.', $sKey);
          $aScope[$aParam[0]][$aParam[1]] = $value;
        }
      }
      // instanciation du service HTMLRenderer
      $oHtmlRenderer = new HtmlRenderer($this->oLoggerManager, $this->oProperties, $this->oFileManager);
      // remplacement des variables avec phantomJS.
      $sBodyRendered = $oHtmlRenderer->render($sHTMLBody, $aScope, $aIncludes);
      $sSubjectRendered = $oHtmlRenderer->render($aData['subject'], $aScope, $aIncludes);
      // Email de l'utilisateur connecté.
      $sToken = $this->oVitisRequest->getToken();
      $oUser = $this->getCurrentUserEntity($sToken);
      $sUserEmail = $oUser->getEmail();
      // Envoi le mail avec le template de test.
      if (empty($sUserEmail)) {
        return $this->formatGenericError('FORM_TEST_MISSING_USER_EMAIL_PUBLICATION_EMAIL_TEMPLATE_EMAIL_TEMPLATE_SIMPLE_EDITION',
          '', 'Error while sending mail : the user has not entered an email address', Response::HTTP_OK);
      }
      $oMail = new Mail($this->oProperties, $this->oLoggerManager, $this->oFileManager);
      $aMailStruct = [
        'subject' => $sSubjectRendered,
        'body' => $sBodyRendered,
        'to' => $sUserEmail,
        'aAttachments' => '',
        'cc' => '',
        'cci' => '',
      ];
      $oMail->loadFromArray($aMailStruct);
      if ($oMail->send()) {
        return $this->formatResponse(['status' => 1]);
      } 
      // si le mail n'a pas pu être encoyé je léve un erreur pour passer dans le catch
      throw new \Exception('Cannot send mail');
    } catch (\Exception $e) {
      $sMessage= 'Error while sending mail : ' . $e->getMessage();
      return $this->formatGenericError('FORM_TEST_MAIL_ERROR_PUBLICATION_EMAIL_TEMPLATE_EMAIL_TEMPLATE_SIMPLE_EDITION',
        '', $sMessage, Response::HTTP_OK);
    }
  }
}
