<?php

namespace App\Module\Vitis\WebService\Controller;

use App\Module\Vitis\WebService\Controller\GenericController;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller pour générer les widget vmap
 */
class WidgetController extends GenericController {

  /**
   * Renvoie un widget grace à son endpoint
   */
  public function getData($id): Response {
    $aData = $this->requestToArray();
    $oParamsUrl = key_exists('params_url', $aData) ? json_decode($aData['params_url'], true) : [];

    // chargement de la connexion à la base de donnée
    $oEntity = $this->getDataWithRequestParams($aData, $id);
    if ($oEntity['status'] == 0) {
      if ($oEntity['message'] === 'entity is empty') {
        return $this->formatResponse([
          'status' => 1,
          'data' => [],
        ], Response::HTTP_OK);
      }
      return $this->formatGenericResponseForErrorOnSelect();
    }

    $widget = $oEntity['data'];
    if (empty($widget['active']) || $widget['active'] == false) {
      // Le widget n'est pas activé
      return $this->formatResponse([
        'status' => 0,
        'message' => 'Le widget n\'est pas activé',
      ], Response::HTTP_UNAUTHORIZED);
    } else if (!empty($widget['expiration_date']) && $widget['expiration_date'] < time()) {
      // Le widget a une date d'expiration inférieur à la date du jour
      return $this->formatResponse([
        'status' => 0,
        'message' => 'Le widget est expiré',
      ], Response::HTTP_UNAUTHORIZED);
    } else if ($widget['need_auth'] != true && (!key_exists('token', $oParamsUrl) ||
      empty($oParamsUrl['token']))) {
      // Le widget ne necessite pas de connexion, le token dans
      // l'url est donc obligatoire => Vérification de sa présence
      return $this->formatResponse([
        'status' => 0,
        'message' => 'Un token est obligatoire pour acceder à ce widget',
      ], Response::HTTP_UNAUTHORIZED);
    } else if ($widget['need_auth'] == true && key_exists('token', $oParamsUrl)) {
      // Le widget necessite une connexion, vérification de l'absence de token dans l'url
      return $this->formatResponse([
        'status' => 0,
        'message' => 'Une connexion est obligatoire pour acceder à ce widget, le token n\'est pas autorisé',
      ], Response::HTTP_UNAUTHORIZED);
    } else if ($this->userCanUseRoute() == false) {
      // L'utilisateur connecté n'est pas autorisé à accédès au widget
      // Retour d'erreur déjà géré dans ce cas
      return $this->formatResponse($this->oResponse, Response::HTTP_UNAUTHORIZED);
    }

    $aSerializerContext = $this->getSerializerContext();

    // transforme le tableau de réponse dans le résultat souhaité
    return $this->formatResponse($oEntity, Response::HTTP_OK, $aSerializerContext);
  }
}
