<?php

namespace App\Module\Vitis\WebService\Entity;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\OneToMany;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;
use Doctrine\ORM\Mapping\EntityListeners;
use Symfony\Component\Ldap\Adapter\CollectionInterface;
use Symfony\Component\Ldap\Entry;
use Symfony\Component\Ldap\Ldap;

/**
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\DomainRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vitis)",name="domain")
 */
class Domain extends AbstractEntity {
  /**
   * @var string []
   */
  public const AUTHORIZED_KEYS_FOR_GROUPS = [
    'cn',
    'name',
    'mail',
    'member',
    'memberUid',
    'objectSid',
    'objectGUID',
    'description',
    'objectClass',
    'objectCategory',
    'sAMAccountName',
  ];
  /**
   * @var string []
   */
  public const AUTHORIZED_KEYS_FOR_USERS = [
    'cn',
    'mail',
    'memberOf',
    'objectSid',
    'objectGUID',
    'description',
    'displayName',
    'sAMAccountName',
    'userPrincipalName',
  ];

  /**
   * @var string
   */
  private const DEFAULT_GROUP_FILTER = '(|(&(|(objectClass=container)(objectClass=group))(!(isCriticalSystemObject=TRUE))(!(showInAdvancedViewOnly=TRUE)))(objectClass=posixGroup))';
  /**
   * @var string
   */
  private const DEFAULT_USER_FILTER = '(&(@uid=*)(objectclass=person))';

  /**
   * @var Properties|null
   */
  private $oProperties;
  /**
   * @var LoggerManager|null
   */
  private $oLoggerManager;

  /**
   * @var Ldap|null
   */
  private $oLdap;
  /**
   * @var int
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vitis).seq_common")
   * @Column(type="integer", name="domain_id", nullable=false)
   */
  private $domain_id;

  /**
   * @var string|null
   * @Column(type="string", length=64, nullable=true)
   */
  private $name;

  /**
   * @var string|null
   * @Column(type="string", length=64, nullable=true)
   */
  private $login_role;

  /**
   * @var string|null
   * @Column(type="string", length=64, nullable=true)
   */
  private $suffix;

  /**
   * @var string|null
   * @Column(type="string", length=100, nullable=true)
   */
  private $prefix;

  /**
   * @var string|null
   * @Column(type="string", length=100, nullable=true)
   */
  private $uid;

  /**
   * @var string|null
   * @Column(type="string", length=100, nullable=true)
   */
  private $displayname;

  /**
   * @var string|null
   * @Column(type="string", length=255, nullable=true)
   */
  private $mail;

  /**
   * @var string|null
   * @Column(type="string", length=100, nullable=true)
   */
  private $server;

  /**
   * @var int|null
   * @Column(type="integer", nullable=true)
   */
  private $port;

  /**
   * @var string|null
   * @Column(type="string", length=255, nullable=true)
   */
  private $dn_search_user;

  /**
   * @var string
   * @Column(type="string")
   */
  private $filter_user;

  /**
   * @var string
   * @Column(type="string", length=255)
   */
  private $dn_search_group;

  /**
   * @var string|null
   * @Column(type="string", nullable=true)
   */
  private $filter_group;

  /**
   * @var string|null
   * @Column(type="string", length=50, nullable=true)
   */
  private $login;

  /**
   * @var string|null
   * @Column(type="string", length=255, nullable=true)
   */
  private $password;

  /**
   * @var string|null
   * @Column(type="string", length=10, nullable=true)
   */
  private $type;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $verify_rights;

  /**
   * @var bool|null
   * @Column(type="boolean", nullable=true)
   */
  private $checkcert;

  /**
   * @var string|null
   * @Column(type="string", length=10, nullable=true)
   */
  private $encrypttype;

  /**
   * @var ArrayCollection<\App\Module\Vitis\WebService\Entity\User>
   * @OneToMany(targetEntity="App\Module\Vitis\WebService\Entity\User", mappedBy="domain")
   */
  private $users;

  /**
   * @var ArrayCollection<DomainGroup>
   * @OneToMany(targetEntity=DomainGroup::class, mappedBy="domain", cascade={"persist", "remove"})
   */
  private $domainGroups;

  public function __construct() {
    $this->users = new ArrayCollection();
    $this->domainGroups = new ArrayCollection();
    $this->filter_group = self::DEFAULT_GROUP_FILTER;
  }

  /**
   * @return integer|null identifiant du domaine
   */
  public function getDomainId(): ?int {
    return $this->domain_id;
  }

  /**
   * @return string|null
   */
  public function getName(): ?string {
    return $this->name;
  }

  /**
   * @param string|null $name nom du domaine
   * @return self
   */
  public function setName(?string $name): self {
    $this->name = $name;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getLoginRole(): ?string {
    return $this->login_role;
  }

  /**
   * @param string|null $login_role
   * @return self
   */
  public function setLoginRole(?string $login_role): self {
    $this->login_role = $login_role;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getSuffix(): ?string {
    return $this->suffix;
  }

  /**
   * @param string|null $suffix
   * @return self
   */
  public function setSuffix(?string $suffix): self {
    $this->suffix = $suffix;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getPrefix(): ?string {
    return $this->prefix;
  }

  /**
   * @param string|null $prefix
   * @return self
   */
  public function setPrefix(?string $prefix): self {
    $this->prefix = $prefix;

    return $this;
  }

  /**
   * @return string retourne le nom de la colonne uid ou un nom par défaut en fonction du type
   */
  public function getUid(): string {
    $sUID = $this->uid;
    if (empty($sUID)) {
      switch ($this->getType()) {
        case 'LDAP':
          $sUID = 'sAMAccountName';
          break;
        case 'AD':
        default:
          $sUID = 'userPrincipalName';
          break;
      }
    }
    return $sUID;
  }

  /**
   * @param string|null $uid
   * @return self
   */
  public function setUid(?string $uid): self {
    $this->uid = $uid;

    if (empty($this->filter_user)) {
      $this->filter_user = $this->getDefaultUserFilter();
    }

    return $this;
  }

  private function getDefaultUserFilter(): string {
    if (empty($this->uid)) {
      return '';
    }

    return str_replace('@uid', $this->uid, self::DEFAULT_USER_FILTER);
  }

  /**
   * @return string|null
   */
  public function getDisplayName(): ?string {
    return $this->displayname;
  }

  /**
   * @param string|null $displayname
   * @return self
   */
  public function setDisplayName(?string $displayname): self {
    $this->displayname = $displayname;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getMail(): ?string {
    return $this->mail;
  }

  /**
   * @param string|null $mail
   * @return self
   */
  public function setMail(?string $mail): self {
    $this->mail = $mail;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getServer(): ?string {
    return $this->server;
  }

  /**
   * @param string|null $server
   * @return self
   */
  public function setServer(?string $server): self {
    $this->server = $server;

    return $this;
  }

  /**
   * @return integer|null
   */
  public function getPort(): ?int {
    return $this->port;
  }

  /**
   * @param integer|null $port
   * @return self
   */
  public function setPort(?int $port): self {
    $this->port = $port;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getDnSearchUser(): ?string {
    return $this->dn_search_user;
  }

  /**
   * @param string|null $dn_search_user
   * @return self
   */
  public function setDnSearchUser(?string $dn_search_user): self {
    $this->dn_search_user = $dn_search_user;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getFilterUser(): ?string {
    return $this->filter_user;
  }

  /**
   * @param string|null $filter_user
   * @return self
   */
  public function setFilterUser(?string $filter_user): self {
    $this->filter_user = $filter_user;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getDnSearchGroup(): ?string {
    return $this->dn_search_group;
  }

  /**
   * @param string|null $dn_search_group
   * @return self
   */
  public function setDnSearchGroup(?string $dn_search_group): self {
    $this->dn_search_group = $dn_search_group;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getFilterGroup(): ?string {
    return $this->filter_group;
  }

  /**
   * @param string|null $filter_group
   * @return self
   */
  public function setFilterGroup(?string $filter_group): self {
    $this->filter_group = $filter_group;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getLogin(): ?string {
    return $this->login;
  }

  /**
   * @param string|null $login
   * @return self
   */
  public function setLogin(?string $login): self {
    $this->login = $login;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getPassword(): ?string {
    return $this->password;
  }

  /**
   * @param string|null $password
   * @return self
   */
  public function setPassword(?string $password): self {
    $this->password = $password;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getType(): ?string {
    return $this->type;
  }

  /**
   * @param string|null $type
   * @return self
   */
  public function setType(?string $type): self {
    $this->type = $type;

    // autofille des champs au moment de set le type
    if (empty($this->mail)) {
      $this->mail = 'mail';
    }
    if (empty($this->uid)) {
      // va aussi le filtre utilisateur
      $this->setUid(($type == 'AD') ? 'userPrincipalName' : 'sAMAccountName');
    }
    if (empty($this->displayname)) {
      $this->displayname = ($type == 'AD') ? 'displayname' : 'cn';
    }

    if (empty($this->filter_group)) {
      $this->filter_group = self::DEFAULT_GROUP_FILTER;
    }

    return $this;
  }

  /**
   * @return bool|null
   */
  public function getVerifyRights(): ?bool {
    return $this->verify_rights;
  }

  /**
   * @param bool|null $verify_rights
   * @return self
   */
  public function setVerifyRights(?bool $verify_rights): self {
    $this->verify_rights = $verify_rights;

    return $this;
  }

  /**
   * @return bool|null
   */
  public function getCheckcert(): ?bool {
    return $this->checkcert;
  }

  /**
   * @param bool|null $checkcert
   * @return self
   */
  public function setCheckcert(?bool $checkcert): self {
    $this->checkcert = $checkcert;

    return $this;
  }

  /**
   * @return string|null
   */
  public function getEncrypttype(): ?string {
    return $this->encrypttype;
  }

  /**
   * @param string|null $encrypttype
   * @return self
   */
  public function setEncrypttype(?string $encrypttype): self {
    $this->encrypttype = $encrypttype;

    return $this;
  }

  /**
   * @return Collection<User>
   */
  public function getUsers(): Collection {
    return $this->users;
  }

  /**
   * retourne la config pour se connecter à un LDAP/AD
   * @return array
   */
  public function getLdapConfig(): array{
    $aConfig = [
      'host' => $this->server,
      'debug' => $this->oProperties->get('log_level') === 'DEBUG',
      'version' => 3,
      'referrals' => false,
      'encryption' => 'none',
      'options' => [
        // ancienne variable d'environnement pour GTF
        'x_tls_require_cert' => LDAP_OPT_X_TLS_NEVER,
      ],
    ];
    // none, ssl, tls
    if (!empty($this->encrypttype) && in_array($this->encrypttype, ['ssl', 'tls'])) {
      $aConfig['encryption'] = $this->encrypttype;
    }
    if (isset($this->port)) {
      $aConfig['port'] = $this->port;
    }
    return $aConfig;
  }

  /**
   * Configuration pour la pagination de la récupération des entrées
   *
   * @return array
   */
  public function getLdapQueryOptions(): array{
    return [
      'pageSize' => $this->oProperties->get('ldap_entries_per_page'),
      'maxItems' => $this->oProperties->get('ldap_max_entries'),
    ];
  }

  /**
   * @return Collection<DomainGroup>
   */
  public function getDomainGroups(): Collection {
    return $this->domainGroups;
  }

  /**
   * @param DomainGroup $domainGroup groupe à ajouter
   * @return self
   */
  public function addDomainGroup(DomainGroup $domainGroup): self {
    if (!$this->domainGroups->contains($domainGroup)) {
      $this->domainGroups[] = $domainGroup;
      $domainGroup->setDomain($this);
    }

    return $this;
  }

  /**
   * @param DomainGroup $domainGroup groupe à supprimer
   * @return self
   */
  public function removeDomainGroup(DomainGroup $domainGroup): self {
    if ($this->domainGroups->removeElement($domainGroup)) {
      // set the owning side to null (unless already changed)
      if ($domainGroup->getDomain() === $this) {
        $domainGroup->setDomain(null);
      }
    }

    return $this;
  }

  /**
   * Analyse l'objet retouner par LDAP/AD et le prépare pour être renvoyer au client pour un utilisateur
   * @param array $aData Donnée pour une ligne venant de decodeEntries
   * @return array $aData préparé pour le client
   */
  public function convertEntriesDataForUser(array $aData): array{
    $aData['uid'] = $this->testAndSetValueFromEntrie($aData, 'uid', $this->getUid());
    $aData['displayName'] = $this->testAndSetValueFromEntrie($aData, 'displayName', $this->getDisplayName());
    $aData['mail'] = $this->testAndSetValueFromEntrie($aData, 'mail', $this->getMail());
    $aData['domain_id'] = $this->getDomainId();
    return $aData;
  }

  /**
   * Analyse l'objet retouner par LDAP/AD et le prépare pour être renvoyer au client pour un groupe
   * @param array $aData Donnée pour une ligne venant de decodeEntries
   * @return array $aData préparé pour le client
   */
  public function convertEntriesDataForGroup(array $aData): array{
    $aData['mail'] = $this->testAndSetValueFromEntrie($aData, 'mail', $this->getMail());
    $aData['domain_id'] = $this->getDomainId();
    return $aData;
  }

  /**
   * @param array $aData Enties data
   * @param string $sClientKey Clé coté client pour datatable
   * @param string $sDomainKey clé dans la données
   * @return mixed
   */
  private function testAndSetValueFromEntrie(array $aData, string $sClientKey, ?string $sDomainKey) {
    // si la clé est déjà défini on ne change rien
    if (isset($aData[$sClientKey])) {
      return $aData[$sClientKey];
    }
    // si une clé spécifique est passé on fait le mappage
    if ($sDomainKey != null) {
      return $aData[$sDomainKey] ?? '';
    }
    // si rien ne match on retourne chaine vide
    return '';
  }

  /**
   * Récupère la liste des groupes du domaine
   *
   * @param  int         $iLimit
   * @param  int         $iOffset
   * @param  string|null $sClientFilter
   * @return array|null
   */
  public function getDomainGroupsList(int $iLimit = 20, int $iOffset = 0, ?string $sClientFilter = null): ?array{
    $sSearch = $this->prepareGroupFilter($sClientFilter);
    $sDnSearchGroups = $this->getDnSearchGroup() ?? '';
    $aAuthorizedKeys = array_unique(self::AUTHORIZED_KEYS_FOR_GROUPS);

    try {
      $aData = $this->fetchLdapData($sDnSearchGroups, $sSearch, $aAuthorizedKeys);
      return $this->prepareDataForUser($aData, $iLimit, $iOffset);

    } catch (\Exception $e) {
      $this->oLoggerManager->get('API_ERROR')->error('Cannot connect to LDAP/AD server : ' . $e->getMessage(), [$this->getLdapConfig()]);
      return null;
    }
  }

  /**
   * retourne les membres d'un groupe
   *
   * @param  string     $sGroupDn
   * @return array|null
   */
  public function getDomainGroupMembers(string $sGroupDn): ?array{
    try {
      $sSearch = $this->prepareGroupFilter();
      $aAuthorizedKeys = array_unique(self::AUTHORIZED_KEYS_FOR_GROUPS);
      $aData = $this->fetchLdapData($sGroupDn, $sSearch, $aAuthorizedKeys);
      // S'il n'y a pas de group venant de l'ad avec ce nom
      if (count($aData) == 0) {
        return [];
      }

      // recherche des utilisateurs AD/Ldap
      $aUsers = [];
      // Les attributes autorisés pour des utilisateurs
      $aAuthorizedKeysUsers = array_merge(array_unique(self::AUTHORIZED_KEYS_FOR_USERS), [
        $this->getUid(),
        $this->getDisplayName(),
        $this->getMail(),
      ]);
      foreach ($aData as $aGroup) {
        if (isset($aGroup['member'])) {
          $sSearch = $this->prepareUserFilter();
          foreach ($aGroup['member'] as $sMemberDn) {
            $sDnSearch = $sMemberDn;
            $aUserData = $this->fetchLdapData($sDnSearch, $sSearch, $aAuthorizedKeysUsers);
            foreach ($aUserData as $aUser) {
              $aUsers[$aUser[$this->getUid()]] = $aUser;
            }
          }
        }
        if (isset($aGroup['memberUid'])) {
          $sDnSearch = $this->getDnSearchUser() ?? '';
          foreach ($aGroup['memberUid'] as $sMemberUid) {
            $sSearch = $this->prepareUserFilter(null, 'uid=' . $sMemberUid);
            $aUserData = $this->fetchLdapData($sDnSearch, $sSearch, $aAuthorizedKeysUsers);
            foreach ($aUserData as $aUser) {
              $aUsers[$aUser[$this->getUid()]] = $aUser;
            }
          }
        }
        $sDnSearch = $aGroup['dn'];
        $sSearch = $this->prepareUserFilter();

        $aUserData = $this->fetchLdapData($sDnSearch, $sSearch, $aAuthorizedKeysUsers);
        foreach ($aUserData as $aUser) {
          $aUsers[$aUser[$this->getUid()]] = $aUser;
        }
      }

      return $aUsers;
    } catch (\Exception $e) {
      $this->oLoggerManager->get('API_ERROR')->error('Search error to LDAP/AD server : ' . $e->getMessage(), );
      return null;
    }
  }

  /**
   * Undocumented function
   *
   * @param  int         $iLimit
   * @param  int         $iOffset
   * @param  string|null $sClientFilter
   * @return array|null
   */
  public function getDomainUsersList(int $iLimit = 20, int $iOffset = 0, ?string $sClientFilter = null): ?array{
    $sSearch = $this->prepareUserFilter($sClientFilter);
    $sDnSearchUser = $this->getDnSearchUser() ?? '';
    $aAuthorizedKeys = array_merge(array_unique(self::AUTHORIZED_KEYS_FOR_USERS), [
      $this->getUid(),
      $this->getDisplayName(),
      $this->getMail(),
    ]);

    try {
      $aData = $this->fetchLdapData($sDnSearchUser, $sSearch, $aAuthorizedKeys);
      return $this->prepareDataForUser($aData, $iLimit, $iOffset);
    } catch (\Exception $e) {
      $this->oLoggerManager->get('API_ERROR')->error('Cannot connect to LDAP/AD server : ' . $e->getMessage(), [$this->getLdapConfig()]);
      return null;
    }
  }

  /**
   * Undocumented function
   *
   * @param  string $sDnSearch
   * @param  string $sSearch
   * @param  array  $aAuthorizedKeys
   * @return array
   * @throws \Exception
   */
  private function fetchLdapData(string $sDnSearch, string $sSearch, array $aAuthorizedKeys): array{
    $this->oLdap = $this->oLdap ?? Ldap::create('ext_ldap', $this->getLdapConfig());
    $this->oLdap->bind($this->getLogin(), $this->getPassword());
    $query = $this->oLdap->query($sDnSearch, $sSearch, $this->getLdapQueryOptions());
    $results = $query->execute()->toArray();

    // Transforme le résultat en array
    return $this->decodeEntries($results, $aAuthorizedKeys);
  }

  /**
   * Undocumented function
   *
   * @param  array $aData
   * @param  int   $iLimit
   * @param  int   $iOffset
   * @return array
   */
  private function prepareDataForUser(array $aData, int $iLimit, int $iOffset): array{
    $iListCount = intval($iOffset + $iLimit);
    if ($iListCount > count($aData) || $iLimit < 0) {
      $iListCount = count($aData);
    }
    $aResponse = [
      'data' => [],
      'total_row_number' => count($aData),
      'list_count' => $iListCount,
      'status' => 1,
    ];
    // Filtrage et ajout de 'domain_id'
    for ($i = $iOffset; $i < $iListCount; $i++) {
      $aResponse['data'][] = $this->convertEntriesDataForUser($aData[$i]);
    }

    return $aResponse;

  }
  /**
   * prépare le filtre pour la rechercher
   *
   * @param  string|null $sClientFilter filtre venant du client
   * @param  string|null $LdapFilter artéfact de l'ancienne version voir avec alex
   * @return string
   */
  private function prepareGroupFilter(?string $sClientFilter = null, ?string $LdapFilter = null): string {
    $sFilterGroup = $this->getFilterGroup();
    // Initialisation du filtre
    $sFilter = '';

    // si aucun filtre n'est défini on prend le filtre par défaut
    if (empty($sFilterGroup)) {
      $sFilterGroup = self::DEFAULT_GROUP_FILTER;
    }

    if (!empty($LdapFilter)) {
      $sFilterGroup = '(&' . $sFilterGroup . '(' . $LdapFilter . '))';
    }

    if (empty($sClientFilter)) {
      return $sFilterGroup;
    }

    $aFilter = json_decode(str_replace('%', '*', $sClientFilter), true);

    // Retourne filter du domain
    if (empty($aFilter)) {
      return $sFilterGroup;
    }

    switch ($aFilter['relation']) {
      case 'AND':
      case 'OR':
        $sFilter = '&';
        break;
    }
    foreach ($aFilter['operators'] as $aColumn) {
      $sColumn = $aColumn['column'];
      if (!isset($sColumn)) {
        // TODO faire correspondance des colonnes d'un groupe
        continue;
      }
      $sFilter = $sFilter . '(' . $sColumn . '=' . $aColumn['value'] . ')';
    }
    return '(' . $sFilter . $sFilterGroup . ')';
  }

  private function prepareUserFilter(?string $sClientFilter = null, string $sLdapFilter = null): ?string{
    // Filtre du domaine
    $sFilterUser = $this->getFilterUser();
    if (empty($this->getFilterUser())) {
      $sFilterUser = $this->getDefaultUserFilter();
    }
    if (!empty($sLdapFilter)) {
      $sFilterUser = '(&' . $sFilterUser . '(' . $sLdapFilter . '))';
    }
    // Initialisation du filtre
    $sFilter = '';

    // Retourne filter du domain
    if (empty($sClientFilter)) {
      return $sFilterUser;
    }

    $aFilter = json_decode(str_replace('%', '*', $sClientFilter), true);
    // Retourne filter du domain
    if (!isset($aFilter) || count($aFilter) == 0) {
      return $sFilterUser;
    }
    switch ($aFilter['relation']) {
      case 'AND':
        $sFilter = '&';
        break;
      case 'OR':
        $sFilter = '&';
        break;
    }
    foreach ($aFilter['operators'] as $aColumn) {
      $sColumn = null;
      switch ($aColumn['column']) {
        case 'displayname':
          $sColumn = null !== $this->getDisplayName() ? $this->getDisplayName() : 'displayname';
          break;
        case 'mail':
          $sColumn = null !== $this->getMail() ? $this->getMail() : 'mail';
          break;
        case 'uid':
          $sColumn = (!empty($this->getUid())) ? $this->getUid() : 'uid';
          break;
      }
      if (!isset($sColumn)) {
        continue;
      }
      $sFilter = $sFilter . '(' . $sColumn . '=' . $aColumn['value'] . ')';
    }
    return '(' . $sFilter . $sFilterUser . ')';
  }

  /**
   * Decode Ldap Entry
   *
   * @param CollectionInterface|array $aEntry
   * @param array $aAuthorizedKeys
   * @return array
   */
  private function decodeEntries(iterable $aEntry, array $aAuthorizedKeys): array{
    $aDecoded = [];
    foreach ($aEntry as $sKey => $sValue) {
      // decode 'objectSid'
      if ($sKey === 'objectSid') {
        $aDecoded[$sKey] = $this->decodeSID($sValue[0]);
        continue;
      }
      // decode 'objectGUID'
      if ($sKey === 'objectGUID') {
        $aDecoded[$sKey] = $this->decodeGUID($sValue[0]);
        continue;
      }
      // decode 'logonHours'
      if ($sKey === 'logonHours') {
        $aDecoded[$sKey] = [];
        $aDecoded[$sKey] = bin2hex($sValue[0]);
        continue;
      }
      // decode des membres d'un groupe
      if ($sKey === 'member' || $sKey === 'memberUid') {
        $aDecoded[$sKey] = $this->decodeEntries($sValue, $aAuthorizedKeys);
        continue;
      }
      if ($sValue instanceof Entry) {
        /* @var Entry $sValue */
        $aDecoded[$sKey] = $this->decodeEntries($sValue->getAttributes(), $aAuthorizedKeys);
        $aDecoded[$sKey]['dn'] = $sValue->getDn();
        // cn=demandeur_ldap2,ou=Groups,dc=ldap,dc=veremes,dc=net
        $sDisplayName = explode('=', explode(',', $aDecoded[$sKey]['dn'])[0])[1];
        $aDecoded[$sKey]['displayname'] = isset($aDecoded[$sKey]['displayname']) ? $aDecoded[$sKey]['displayname'] : $sDisplayName;
        continue;
      }
      // si c'est une liste
      if (is_int($sKey)) {
        $aDecoded[$sKey] = $sValue;
      }
      // Filtre les attributes envoyés par la requête LDAP
      if (!in_array($sKey, $aAuthorizedKeys)) {
        continue;
      }
      if (is_array($sValue) && count($sValue) > 1) {
        $aDecoded[$sKey] = $this->decodeEntries($sValue, $aAuthorizedKeys);
      }
      if (is_array($sValue) && count($sValue) <= 1) {
        $aDecoded[$sKey] = $this->decodeEntries($sValue, $aAuthorizedKeys)[0];
      }
      if (is_string($sValue)) {
        $aDecoded[$sKey] = $sValue;
      }
    }
    return $aDecoded;
  }

  /**
   * Decode binary SID
   *
   * @param string $bSID
   * @return string
   */
  private function decodeSID($bSID) {
    /**
     * revision - 8bit unsigned int (C1)
     * count - 8bit unsigned int (C1)
     * 2 null bytes
     * ID - 32bit unsigned long, big-endian order
     */
    $sid = unpack('C1rev/C1count/x2/N1id', $bSID);
    $subAuthorities = [];

    if (!isset($sid['id']) || !isset($sid['rev'])) {
      throw new \UnexpectedValueException(
        'The revision level or identifier authority was not found when decoding the SID.'
      );
    }

    $revisionLevel = $sid['rev'];
    $identifierAuthority = $sid['id'];
    $subs = isset($sid['count']) ? $sid['count'] : 0;

    // The sub-authorities depend on the count, so only get as many as the count, regardless of data beyond it
    for ($i = 0; $i < $subs; $i++) {
      // Each sub-auth is a 32bit unsigned long, little-endian order
      $subAuthorities[] = unpack('V1sub', hex2bin(substr(bin2hex($bSID), 16 + ($i * 8), 8)))['sub'];
    }

    // Tack on the 'S-' and glue it all together...
    return 'S-' . $revisionLevel . '-' . $identifierAuthority . implode(
      preg_filter('/^/', '-', $subAuthorities)
    );
  }

  /**
   * Decode binary GUID
   *
   * @param string $bGUID
   * @return string
   */
  private function decodeGUID($bGUID) {
    $unpacked = unpack('Va/v2b/n2c/Nd', $bGUID);
    return sprintf('%08X-%04X-%04X-%04X-%04X%08X', $unpacked['a'], $unpacked['b1'], $unpacked['b2'], $unpacked['c1'], $unpacked['c2'], $unpacked['d']);
  }

  /* ---------------------------------------------------------------- */
  /* -------------------------- Interfaces -------------------------- */
  /* ---------------------------------------------------------------- */

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @param Properties $oService
   * @return void
   */
  public function setPropertiesService(Properties $oService): void {
    $this->oProperties = $oService;
  }

  /**
   * Permet au gestionnaire d'event de doctrine de charger le service des logger lors du chargement
   * @return void
   */
  public function setLoggerManagerService(LoggerManager $oService): void {
    $this->oLoggerManager = $oService;
  }
}
