<?php

namespace App\Module\Vitis\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use App\Module\Vitis\WebService\Entity\Domain;
use App\Module\Vitis\WebService\Entity\Group;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\ManyToOne;
use Doctrine\ORM\Mapping\OneToOne;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;

/**
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\DomainGroupRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vitis)",name="domain_group")
 */
class DomainGroup extends AbstractEntity {
  /**
   * @var int|null
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vitis).seq_common")
   * @Column(type="integer", name="domain_group_id", nullable=false)
   */
  private $domainGroupId;

  /**
   * @var Group
   * @OneToOne(targetEntity=Group::class, inversedBy="domainGroup", cascade={"persist", "remove"})
   * @JoinColumn(name="vitis_group_id", referencedColumnName="group_id")
   */
  private $vitisGroup;

  /**
   * @var Domain|null
   * @ManyToOne(targetEntity=Domain::class, inversedBy="domainGroups", cascade={"persist", "remove"})
   * @JoinColumn(name="domain_id", referencedColumnName="domain_id")
   */
  private $domain;

  /**
   * @var string
   * @Column(type="string", length=255, nullable=true)
   */
  private $dn;

  /**
   * @var bool
   * @Column(name="auto_refresh",type="boolean", nullable=false,options={"default"="FALSE"})
   */
  private $autoRefresh = false;

  /**
   * @return array
   */
  public function toArray(): array{
    return [
      'domain_group_id' => $this->getDomainGroupId(),
      'name' => $this->vitisGroup->getName(),
      'displayname' => $this->vitisGroup->getName(),
      'domain' => $this->domain->getName(),
      'group_id' => $this->vitisGroup->getGroupId(),
      'domain_id' => $this->domain->getDomainId(),
      'dn' => $this->getDn(),
    ];
  }

  /**
   * @return int|null
   */
  public function getDomainGroupId(): ?int {
    return $this->domainGroupId;
  }
  /**
   * @return Group|null
   */
  public function getVitisGroup(): ?Group {
    return $this->vitisGroup;
  }
  /**
   * @param  Group|null $VitisGroup
   * @return self
   */
  public function setVitisGroup(?Group $VitisGroup): self {
    $this->vitisGroup = $VitisGroup;

    return $this;
  }
  /**
   *
   * @return Domain|null
   */
  public function getDomain(): ?Domain {
    return $this->domain;
  }
  /**
   *
   * @param Domain|null $Domain
   * @return self
   */
  public function setDomain(?Domain $Domain): self {
    $this->domain = $Domain;

    return $this;
  }
  /**
   *
   * @return string|null
   */
  public function getDn(): ?string {
    return $this->dn;
  }
/**
 * @param string|null $dn
 * @return self
 */
  public function setDn(?string $dn): self {
    $this->dn = $dn;

    return $this;
  }
/**
 *
 * @return bool|null
 */
  public function getAutoRefresh(): ?bool {
    return $this->autoRefresh;
  }
/**
 * @param bool $autoRefresh
 * @return self
 */
  public function setAutoRefresh(bool $autoRefresh): self {
    $this->autoRefresh = $autoRefresh;

    return $this;
  }
}
