<?php

namespace App\Module\Vitis\WebService\Entity;

use App\Module\Vitis\WebService\Entity\AbstractEntity;
use App\Module\Vitis\WebService\Entity\DomainGroup;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping\Column;
use Doctrine\ORM\Mapping\Entity;
use Doctrine\ORM\Mapping\EntityListeners;
use Doctrine\ORM\Mapping\GeneratedValue;
use Doctrine\ORM\Mapping\Id;
use Doctrine\ORM\Mapping\JoinColumn;
use Doctrine\ORM\Mapping\JoinTable;
use Doctrine\ORM\Mapping\ManyToMany;
use Doctrine\ORM\Mapping\OneToOne;
use Doctrine\ORM\Mapping\SequenceGenerator;
use Doctrine\ORM\Mapping\Table;

/**
 *  Objet ORM pour v_restriction
 *
 * @Entity(repositoryClass="App\Module\Vitis\WebService\Repository\GenericRepository")
 * @EntityListeners({"App\Module\Vitis\WebService\EventListener\VitisEntityListener"})
 * @Table(schema="@prop(schema_vitis)",name="v_restriction")
 */
class Restriction extends AbstractEntity {
  /**
   * @var int|null
   * @Column(type="integer", name="restriction_id", nullable=false)
   * @Id
   * @GeneratedValue(strategy="SEQUENCE")
   * @SequenceGenerator(sequenceName="@prop(schema_vitis).seq_common")
   */
  private $restriction_id;

  /**
   * @var string|null
   * @Column(type="string", length=255)
   */
  private $restriction_name;

  /**
   * @var int|null
   * @Column(type="integer", nullable=true)
   */
  private $restriction_type_id;

  /**
   * @var ArrayCollection<User>
   * @ManyToMany(targetEntity="User")
   * @JoinTable(
   *   name="@prop(schema_vitis).v_user_restriction",
   *   joinColumns={@JoinColumn(name="restriction_id", referencedColumnName="restriction_id")},
   *   inverseJoinColumns={@JoinColumn(name="user_id", referencedColumnName="user_id")}
   * )
   */
  private $aUsers;

  public function __construct() {
    $this->aUsers = new ArrayCollection();
  }

  /**
   * @return int identifiant de la restriction
   */
  public function getRestrictionId(): int {
    return $this->restriction_id;
  }

  /**
   * @return int identifiant du type de la restriction
   */
  public function getRestrictionTypeId(): int {
    return $this->restriction_type_id;
  }

  /**
   * @return string nom de la restriction
   */
  public function getRestrictionName(): string {
    return $this->restriction_name;
  }

  /**
   * @param string $restriction_name nom de la restriction
   * @return self
   */
  public function setRestrictionName(string $restriction_name): self {
    if (empty($restriction_name)) {
      // random
    } else {
      $this->restriction_name = $restriction_name;
    }
    return $this;
  }

  /**
   * @return int[] liste des user_id
   */
  public function getUsers(): array{
    $aUserId = [];
    foreach ($this->aUsers as $oUser) {
      $aUserId[] = $oUser->getUserId();
    }
    return $aUserId;
  }
}
