<?php

namespace App\Module\Vitis\WebService\Query\Functions;

use Doctrine\ORM\Query\AST\ArithmeticFactor;
use Doctrine\ORM\Query\AST\Functions\FunctionNode;
use Doctrine\ORM\Query\AST\Node;
use Doctrine\ORM\Query\Lexer;
use Doctrine\ORM\Query\Parser;
use Doctrine\ORM\Query\SqlWalker;

/**
 * Class :  PgpSymEncrypt
 *
 * Usage : pgp_sym_encrypt() : string
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */
class PgpSymEncrypt extends FunctionNode {
  /**
   * @var ArithmeticFactor[]
   */
  protected $expressions = [];
  /**
   * Parse la requête DQL pour extraire l'appel d'une fonction
   *
   * @param Parser $parser Parser DQL, utilise Doctrine\ORM\Query\Lexer pour parser le DQL
   * @return void
   */
  public function parse(Parser $parser) {
    $parser->match(Lexer::T_IDENTIFIER); // pgp_sym_Encrypt
    $parser->match(Lexer::T_OPEN_PARENTHESIS); //()

    $this->expressions[] = $parser->ArithmeticFactor(); // field/value

    $parser->match(Lexer::T_COMMA); // ,

    $this->expressions[] = $parser->ArithmeticFactor(); // password

    $lexer = $parser->getLexer();
    if (($lexer->lookahead['type'] ?? null) === Lexer::T_COMMA) {
      $parser->match(Lexer::T_COMMA);
      $this->expressions[] = $parser->ArithmeticFactor(); // options
    }

    $parser->match(Lexer::T_CLOSE_PARENTHESIS); //)
  }
  /**
   * Retourne le texte à utiliser pour le SQL pour la classe parente
   *
   * @param SqlWalker $sqlWalker permet de parcourir la requete SQL
   * @return string chaine de caractère à utiliser à travers DBAL pour le SQL
   */
  public function getSql(SqlWalker $sqlWalker) {
    $arguments = [];
    /** @var Node $expression */
    foreach ($this->expressions as $expression) {
      $arguments[] = $expression->dispatch($sqlWalker);
    }

    return 'pgp_sym_encrypt(' . implode(', ', $arguments) . ')';

  }
}
