<?php
namespace App\Module\Vitis\WebService\Query;

use Doctrine\ORM\Query\AST\CoalesceExpression;
use Doctrine\ORM\Query\AST\Literal;
use Doctrine\ORM\Query\AST\PathExpression;
use Doctrine\ORM\Query\SqlWalker;

/**
 * Permet d'affciher les nulls en dernier dans les requêtes DQL
 * 
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */

class SortableNullValueWalker extends SqlWalker {
  /**
   * @var string
   */
  public const HINT = 'sortableNulls.fields';
  /**
   * @var string
   */
  public const NULLS_FIRST = 'NULLS FIRST';
  /**
   * @var string
   */
  public const NULLS_LAST = 'NULLS LAST';

  private const TYPES_ALLOWED = [
    PathExpression::TYPE_STATE_FIELD,
    PathExpression::TYPE_SINGLE_VALUED_ASSOCIATION,
  ];

  /**
   * Parcours les orderby pour ajouter le tri des valeurs null en fin ou en debut de retour
   * {@inheritdoc}
   */
  public function walkOrderByItem($orderByItem) : string {
    // génération du SQL à partir du DQL
    $sql = parent::walkOrderByItem($orderByItem);
    // récupération de la configuration pour la surcharge
    $hint = $this->getQuery()->getHint(self::HINT);
    // Récupération de l'attribut pour ordonner et du type de tri
    $expr = $orderByItem->expression;
    $type = strtoupper($orderByItem->type);
    // si on a un modificateur pour la surcharge
    if (is_array($hint) && count($hint) > 0) {
      // si l'attribut est du bon type et que sont type peut être surchargé
      if ($expr instanceof PathExpression && in_array($expr->type, self::TYPES_ALLOWED)) {
        // génération de ce qui doit être recherché pour modification dans le sql natif
        $search = $this->walkPathExpression($expr) . ' ' . $type;
        // génération de l'index pour récupération de la bonne configuration pour la surcharge
        $index = $expr->identificationVariable . '.' . $expr->field;
        // surcharge du SQL natif
        $sql = str_replace($search, $search . ' ' . $hint[$index], $sql);
      }
    }

    return $sql;
  }
}
