<?php

namespace App\Module\Vitis\WebService\Repository;

use App\Module\Vitis\WebService\Entity\Domain;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Mapping\ClassMetadata;
use App\Module\Vitis\WebService\Repository\GenericRepository;

/**
 * @method DomainGroup|null find($id, $lockMode = null, $lockVersion = null)
 * @method DomainGroup|null findOneBy(array $criteria, array $orderBy = null)
 * @method DomainGroup[]    findAll()
 * @method DomainGroup[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class DomainGroupRepository extends GenericRepository
{
  /**
   * {@inheritdoc}
   */
  public function __construct(EntityManagerInterface $em, ClassMetadata $class) {
    parent::__construct($em, $class);
  }

  /**
   * Retourne les roles système pour l'utilisateur
   * @param Domain $oDomain Domaine
   * @param string $sGroupName Nom du groupe
   * @return array|null
   */
  public function findOneByDomainAndGroupName(Domain $oDomain, string $sGroupName): ?array{
    $oMetaData = $this->getClassMetadata();
    /* @var $em EntityManager */
    $qb = $this->getEntityManager()->getConnection()->createQueryBuilder();
    $sDomainGroupTable = $oMetaData->getSchemaName() . '.' . $oMetaData->getTableName();
    $sGroupTable = $oMetaData->getSchemaName() . '.group';
    $qb->select('a.*')
      ->from($sDomainGroupTable, 'a')
      ->leftJoin('a', $sGroupTable, 'b', 'a.vitis_group_id = b.group_id')
      ->where('b.name=:groupname')
      ->andWhere('a.domain_id = :domain_id')
      ->setParameter('groupname', $sGroupName)
      ->setParameter('domain_id', $oDomain->getDomainId());
    try {
      $aResult = $qb->executeQuery()->fetchAllAssociative();
    } catch (\Exception $e) {
      error_log('DomainGroup : can\'t get DomainGroup object : ' . $e->getMessage());
      $aResult = null;
    }
    return $aResult;
  }

}
