<?php

namespace App\Module\Vitis\WebService\Repository;

use App\Module\Vitis\WebService\Entity\UserRequest;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Mapping\ClassMetadata;

/**
 * Repository pour les demandes de compte de Vitis.
 *
 * @author Alexandre Mironenkov   <alexandre.mironenkov@veremes.com>
 * @author Anthony Borghi   <anthony.borghi@veremes.com>
 */
class UserRequestRepository extends GenericRepository {
  /**
   * {@inheritdoc}
   */
  public function __construct(EntityManagerInterface $em, ClassMetadata $class) {
    parent::__construct($em, $class);
  }

  /**
   * Retourne l'utilisateur récupéré avec son login
   * @param string $sLogin Login de l'utilisateur
   * @return array
   * @throws \Exception
   */
  public function findByLogin(string $sLogin): array{
    $oMetaData = $this->getClassMetadata();
    $sUserTable = $oMetaData->getSchemaName() . '.' . $oMetaData->getTableName();
    $conn = $this->_em->getConnection();
    /* @var $em EntityManager */
    $qb1 = $conn->createQueryBuilder();
    $qb1->select('a.*')
      ->from($sUserTable, 'a')
      ->where('a.login = :login')
      ->setParameter('login', $sLogin);

    $aResults = $qb1->executeQuery()->fetchAllAssociative();

    return (array) $aResults;
  }

  /**
   * vrai recherche par login
   *
   * @param  string    $sLogin
   * @return UserRequest|null
   */
  public function findByLoginEntity(string $sLogin): ?UserRequest{
    $qb = $this->createQueryBuilder('a');
    $qb->where($qb->expr()->eq('a.login', ':sLogin'))
      ->setParameter('sLogin', $sLogin);

    $oResult = $qb->getQuery()->getResult();

    if (empty($oResult)) {
      return null;
    }

    if (!$oResult[0] instanceof UserRequest) {
      return null;
    }

    return $oResult[0];

  }

  /**
   * Retourne l'utilisateur récupéré avec son login sans tenir conmpte de la casse
   * @param string $sLogin Login de l'utilisateur
   * @return array
   * @throws \Exception
   */
  public function findByLoginWithoutCase(string $sLogin): array{
    $oMetaData = $this->getClassMetadata();
    $sUserTable = $oMetaData->getSchemaName() . '.' . $oMetaData->getTableName();
    $conn = $this->_em->getConnection();
    /* @var $em EntityManager */
    $qb1 = $conn->createQueryBuilder();
    $qb1->select('a.*')
      ->from($sUserTable, 'a')
      ->where('LOWER(a.login) = LOWER(:login)')
      ->setParameter('login', $sLogin);

    $aResults = $qb1->executeQuery()->fetchAllAssociative();

    return (array) $aResults;
  }

  /**
   * Retourne l'utilisateur récupéré avec son email
   * @param string $sEmail Email de l'utilisateur
   * @return array
   * @throws \Exception
   */
  public function findByEmail(string $sEmail): array{
    $oMetaData = $this->getClassMetadata();
    $sUserTable = $oMetaData->getSchemaName() . '.' . $oMetaData->getTableName();
    $conn = $this->_em->getConnection();
    /* @var $em EntityManager */
    $qb1 = $conn->createQueryBuilder();
    $qb1->select('a.*')
      ->from($sUserTable, 'a')
      ->where('a.email = :email')
      ->setParameter('email', $sEmail);

    $aResults = $qb1->executeQuery()->fetchAllAssociative();

    return (array) $aResults;
  }
}
