<?php

namespace App\Module\Vitis\WebService\Service\DataAccessor;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Configuration\Route;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use App\Module\Vitis\WebService\Service\Serializer\VitisSerializer;
use App\Module\Vitis\WebService\Service\Token\JweTokenManager;
use Doctrine\DBAL\Query\QueryBuilder;
use Doctrine\ORM\EntityManager;
use Monolog\Logger;

/**
 * Permet d'accéder aux données d'une connexion Oracle
 *
 * @author Ludivine Premont   <ludivine.premont@veremes.com>
 */
abstract class AbstractQueryAccessor extends AbstractDataAccessor {

  /**
   * @var string
   */
  protected const MODE = 'QUERY';
  /**
   * @var string
   */
  protected const LOGGER_KEY = 'SQL_ERROR';
  /**
   * @var string
   */
  protected const RETURNING_STATEMENT = ' RETURNING *;';

  /**
   * Le nom de la table
   * @var string|null
   */
  protected $sTableName;

  /**
   * @param EntityManager $oEntityManager Gestionnaire d'entité Doctrine
   * @param Route $oRouteConfigurator service de configuration de la route
   * @param VitisSerializer $oSerializer Objet permettant la désearialization dans une couche ORM
   * @param LoggerManager $oLogger Service d'accès aux logger
   * @param Properties $oProperties Service d'accès aux properties
   * @param JweTokenManager $oJweTokenManager accès au Token et encryptage
   */
  public function __construct(EntityManager $oEntityManager, Route $oRouteConfigurator, VitisSerializer $oSerializer, LoggerManager $oLogger, Properties $oProperties, JweTokenManager $oJweTokenManager) {
    parent::__construct($oEntityManager, $oRouteConfigurator, $oSerializer, $oLogger, $oProperties, $oJweTokenManager);
  }

  /**
   * recharge le nom de la table
   *
   * @return void
   */
  protected function reloadTableNameFromOptions(): void {
    if (isset($this->oOptions['table'])) {
      $sTableName = $this->oOptions['table'];
      if (isset($this->oOptions['schema'])) {
        $sTableName = $this->oOptions['schema'] . '.' . $sTableName;
      }
      $this->sTableName = $this->oPropertiesSrvc->replacePropertiesInString($sTableName);
    }
  }

  /**
   * @return VitisNativeQueryBuilder|QueryBuilder QueryBuilder vitis
   */
  abstract public function createQueryBuilder(): VitisNativeQueryBuilder | QueryBuilder;
}
