<?php

namespace App\Module\Vitis\WebService\Service\DataAccessor\Fields;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Configuration\Route;
use App\Module\Vitis\WebService\Service\DataAccessor\Fields\AbstractCustomField;
use App\Module\Vitis\WebService\Service\DataAccessor\Fields\FieldInterface;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;

/**
 * Classe abstraite pour gérer un champ de type boolean.
 * returne la valeur correspondante au boolean.
 *
 *  Configuration du champ
 *    "type"      (string) : toujours boolean
 *    "name"      (string) : Nom du champ
 *  exemple :
 *    {
 *      "type": "boolean",
 *      "name": "my_field"
 *    }
 *
 * @author alexandre Mironenkov   <alexandre.mironenkov@veremes.com>
 */
class BooleanField extends AbstractCustomField implements FieldInterface {

  /**
   * @var string
   * Valeur du champs type dans route.json
   */
  public const TYPE = 'boolean';

  /**
   * {@inheritdoc}
   */
  public function __construct(Properties $oPropertiesSrvc, Route $oRoute, LoggerManager $oLoggerManager) {
    parent::__construct($oPropertiesSrvc, $oRoute, $oLoggerManager);
  }

  /**
   * {@inheritdoc}
   */
  public function updateFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    if (array_key_exists($aFieldOptions['name'], $aRequestData)) {
      $sValue = $aRequestData[$aFieldOptions['name']] ?? null;
      if ($sValue !== null) {
        $aEntity[$aFieldOptions['name']] = (strtolower($sValue) === 'true' || $sValue === 1 || $sValue === true);
      } else {
        $aEntity[$aFieldOptions['name']] = null;
      }
      
    }
    return $aEntity;
  }

  /**
   * {@inheritdoc}
   */
  public function updateMassiveFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $this->updateFieldValue($aEntity, $aRequestData, $aFieldOptions);
  }

  /**
   * {@inheritdoc}
   */
  public function insertFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $this->updateFieldValue($aEntity, $aRequestData, $aFieldOptions);
  }

}
