<?php

namespace App\Module\Vitis\WebService\Service\DataAccessor\Fields;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Configuration\Route;
use App\Module\Vitis\WebService\Service\DataAccessor\Fields\AbstractCustomField;
use App\Module\Vitis\WebService\Service\DataAccessor\Fields\FieldInterface;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use \DateTime;

/**
 * Classe abstraite pour gérer un champ de type date.
 * ne fait que des return sans modification.
 *
 *  Configuration du champ
 *    "type"      (string) : vaut toujours date pour ce champ
 *    "name"      (string) : Nom du champ
 *    "mode"      (string) : valeurs possible : unchanged, nullify (default : nullify)
 *  exemple :
 *    {
 *      "type": "date",
 *      "name": "my_field"
 *    }
 *
 * @author Armand Bahi   <armand.bahi@veremes.com>
 */

class DateField extends AbstractCustomField implements FieldInterface {
  /**
   * @var string
   * Valeur du champs type dans route.json
   */
  public const TYPE = 'date';

  /**
   * @var string
   */
  public const DEFAULT_FORMAT = 'c';
  /**
   * @var string
   */
  public const DEFAULT_DB_FORMAT = 'U';

  /**
   * {@inheritdoc}
   */
  public function __construct(Properties $oPropertiesSrvc, Route $oRoute, LoggerManager $oLoggerManager) {
    parent::__construct($oPropertiesSrvc, $oRoute, $oLoggerManager);
  }

  /**
   * Retourne un timestamp
   * {@inheritdoc}
   */
  public function getFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    $oDateTime = null;
    // dans le cas de l'ORM on peut retourner un DateTime directement
    if ($aEntity[$aFieldOptions['name']] instanceof \DateTime) {
      $oDateTime = $aEntity[$aFieldOptions['name']];
    } else if ($aEntity[$aFieldOptions['name']] > 0) {
      $oDateTime  = new DateTime($aEntity[$aFieldOptions['name']]);
    } else {
      unset($aEntity[$aFieldOptions['name']]);
    }

    if($oDateTime !== null){
      $aEntity[$aFieldOptions['name']] = intval($oDateTime->format(self::DEFAULT_DB_FORMAT));
    }

    return $aEntity;
  }

  /**
   * Récupère un timestamp pour écrire une date
   * {@inheritdoc}
   */
  public function insertFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    if (is_numeric($aEntity[$aFieldOptions['name']])) {
      $aEntity[$aFieldOptions['name']] = date(self::DEFAULT_FORMAT, $aEntity[$aFieldOptions['name']]);
    } else if ($aEntity[$aFieldOptions['name']] === ""){
      $aEntity[$aFieldOptions['name']] = null;
    }
    return $aEntity;
  }

  /**
   * {@inheritdoc}
   */
  public function afterInsertFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $this->insertFieldValue($aEntity, $aRequestData, $aFieldOptions);
  }

  /**
   * {@inheritdoc}
   */
  public function updateFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $this->insertFieldValue($aEntity, $aRequestData, $aFieldOptions);
  }

  /**
   * {@inheritdoc}
   */
  public function updateMassiveFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $this->insertFieldValue($aEntity, $aRequestData, $aFieldOptions);
  }
}