<?php

namespace App\Module\Vitis\WebService\Service\DataAccessor\Fields;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Configuration\Route;
use App\Module\Vitis\WebService\Service\DataAccessor\Fields\AbstractCustomField;
use App\Module\Vitis\WebService\Service\DataAccessor\Fields\FieldInterface;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use App\Module\Vitis\WebService\Service\Crypto\VitisKeyManager;


/**
 * Classe abstraite pour gérer un champ de type crypted.
 * ne fait que des return sans modification.
 *
 *  Configuration du champ
 *    "type"      (string) : vaut toujours date pour ce champ
 *    "name"      (string) : Nom du champ
 *  exemple :
 *    {
 *      "type": "encrypted",
 *      "name": "my_field"
 *    }
 *
 * @author Armand Bahi   <armand.bahi@veremes.com>
 */

class EncryptField extends AbstractCustomField implements FieldInterface {
  /**
   * @var string
   * Valeur du champs type dans route.json
   */
  public const TYPE = 'encrypted';
  /**
   * @var string
   * Valeur du champs type dans route.json
   */
  private const KEY_NAME = 'pgcrypto';

  /**
   * {@inheritdoc}
   */
  public function __construct(Properties $oPropertiesSrvc, Route $oRoute, LoggerManager $oLoggerManager) {
    parent::__construct($oPropertiesSrvc, $oRoute, $oLoggerManager);
  }

  /**
   * Retourne l'expression utilisée en tant que colonne dans le requête GET
   * {@inheritdoc}
   */
  public function getFieldColumn(string $sSchema, string $sField): string {
    $sKey = $this->processKey($sField);
    $sColumn = "\"$sSchema\".\"$sField\"";
    return "pgp_sym_decrypt($sColumn, '$sKey') as $sField";
  }

  /**
   * Retourne la fonction utilisée en tant que colonne dans le requête PUT ou insert
   * {@inheritdoc}
   */
  public function getFieldColumnFunction(string $sField): ?string{
    $sKey = $this->processKey($sField);
    $sReturn = "pgp_sym_encrypt(:$sField, '$sKey')";
    return $sReturn;
  }

  /**
   * Retourne un EWKT
   * {@inheritdoc}
   */
  public function getFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $aEntity;
  }

  /**
   * Récupère un timestamp pour écrire une date
   * {@inheritdoc}
   */
  public function insertFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $aEntity;
  }

  /**
   * {@inheritdoc}
   */
  public function afterInsertFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $this->insertFieldValue($aEntity, $aRequestData, $aFieldOptions);
  }

  /**
   * {@inheritdoc}
   */
  public function updateFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $this->insertFieldValue($aEntity, $aRequestData, $aFieldOptions);
  }

  /**
   * {@inheritdoc}
   */
  public function updateMassiveFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array{
    return $this->insertFieldValue($aEntity, $aRequestData, $aFieldOptions);
  }

  /**
   * calcul la clé de cryptage pour les champs à crypter de la base
   *
   * @param  string $sField
   * @return string
   */
  private function processKey(string $sField): string {
    $oKeyManager = new VitisKeyManager($this->oProperties, $this->oLoggerManager);
    $sAppSecret = $oKeyManager->getKeyByName(self::KEY_NAME);

    if (empty($sAppSecret)) {
      $this->oLoggerManager->get('API_ERROR')->error('app_secret to encrypt column is empty, this data will be encrypt with low security key');
    }

    $sKey = $sAppSecret . '%' . $sField . '%vitis';
    return bin2hex($sKey);
  }
}
