<?php

namespace App\Module\Vitis\WebService\Service\DataAccessor\Fields;

/**
 * Interface permettant de définir les fonctions à implémenter pour modifier un type de champ
 *
 * @author Anthony Borghi   <anthony.borghi@veremes.com>
 */
interface FieldInterface {
  /**
   * Retourne l'expression utilisée en tant que colonne dans le requête GET
   * @param string $sSchema
   * @param string $sField
   * @return string
   */
  public function getFieldColumn(string $sSchema, string $sField): string;
  /**
   * Retourne la fonction utilisée en tant que colonne dans le requête PUT ou insert
   * @param string $sField nom du field
   * @return string|null
   */
  public function getFieldColumnFunction(string $sField): ?string;
  /**
   * @param array $aEntity Retour de la base de données
   * @param array $aRequestData données provenant de la requête
   * @param array $aFieldOptions configuration du champ
   * @return array structure provenant de la base de donnée modifiée pour le GET
   */
  public function getFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array;
  /**
   * @param array $aEntity Retour de la base de données
   * @param array $aRequestData données provenant de la requête
   * @param array $aFieldOptions configuration du champ
   * @return array structure provenant de la base de donnée modifiée pour le POST
   */
  public function insertFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array;
  /**
   * @param array $aEntity Retour de la base de données
   * @param array $aRequestData données provenant de la requête
   * @param array $aFieldOptions configuration du champ
   * @return array structure provenant de la base de donnée modifiée pour le POST
   */
  public function afterInsertFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array;
  /**
   * @param array $aEntity Retour de la base de données
   * @param array $aRequestData données provenant de la requête
   * @param array $aFieldOptions configuration du champ
   * @return array structure provenant de la base de donnée modifiée pour le PUT
   */
  public function updateFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array;
  /**
   * @param array $aEntity Retour de la base de données
   * @param array $aRequestData données provenant de la requête
   * @param array $aFieldOptions configuration du champ
   * @return array structure provenant de la base de donnée modifiée pour le PUT
   */
  public function updateMassiveFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array;
  /**
   * @param array $aEntity Retour de la base de données
   * @param array $aRequestData données provenant de la requête
   * @param array $aFieldOptions configuration du champ
   * @return array structure provenant de la base de donnée modifiée pour le DELETE
   */
  public function deleteFieldValue(array $aEntity, array $aRequestData, array $aFieldOptions): array;

  /**
   * Définit s'il faut forcer un type d'attribut pour la modification
   * @param string $sMethodName nom de la méthode à forcer
   * @return bool true s'il faut forcer l'attribut
   */
  public function forceAttribute(string $sMethodName): bool;
  /**
   * retourne la valeur à utiliser pour forcer le champ
   * @return mixed
   */
  public function getForceValue();
  /**
   * Définit si une valeur est set pour le champ
   * @param array $aEntity Entité
   * @param array $aFieldOptions configuration du champ
   * @return bool true si le champ est set
   */
  public function fieldIsSet(array $aEntity, array $aFieldOptions): bool;
}

?>