<?php

namespace App\Module\Vitis\WebService\Service\Encoder;

use App\Module\Vitis\WebService\Service\Normalizer\VitisNormalizer;
use Symfony\Component\Serializer\Encoder\XmlEncoder;

/**
 * Surcouche de la classe  XmlEncoder de symfony pour gérer le distinct et forcer la normalisation.
 *
 * @author Anthony Borghi   <anthony.borghi@veremes.com>
 */

class VitisXmlEncoder extends XmlEncoder {
  /**
   * @var VitisNormalizer
   */
  private $oNormalizer;

  /**
   * @param VitisNormalizer $oNormalizer
   */
  public function __construct(VitisNormalizer $oNormalizer) {
    parent::__construct();
    $this->oNormalizer = $oNormalizer;
  }

  /**
   * {@inheritdoc}
   */
  public function encode($data, $format, array $context = []): string {
    $aData = $data;
    // on force la normalisation car le XML ne le fait pas par défaut, si possible
    if ($this->oNormalizer->supportsNormalization($data)) {
      $aData = $this->oNormalizer->normalize($data, $format, $context);
      // suppression des doublons si DISTINCT
      if (!empty($context) && isset($context[VitisNormalizer::DISTINCT])) {
        if ($context[VitisNormalizer::DISTINCT] == true) {
          $aData['data'] = array_unique($aData['data'], SORT_REGULAR);
        }
      }
    }

    return parent::encode($aData, $format, $context);
  }

}
