<?php

namespace App\Module\Vitis\WebService\Service\Logger;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use Monolog\Handler\StreamHandler;
use Monolog\Logger;
use Monolog\LogRecord;

/**
 * Permet de débiter le fichier de log
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */

class CustomStreamHandler extends StreamHandler {
  /**
   * @var Properties
   */
  private $oPropertiesSrvc;

  /**
   * @param Properties $PropertiesSrvc Un objet pour accéder aux properties de l'application
   * @param resource|string $stream
   * @param bool            $bubble         Whether the messages that are handled can bubble up the stack or not
   * @param int|null        $filePermission Optional file permissions (default (0644) are only for owner read/write)
   * @param bool            $useLocking     Try to lock log file before doing any writes
   *
   * @throws \Exception                If a missing directory is not buildable
   * @throws \InvalidArgumentException If stream is not a resource or string
   */
  public function __construct(Properties $PropertiesSrvc, $stream, $bubble = true, $filePermission = null, $useLocking = false) {
    // Sauvegarde du service de properties
    $this->oPropertiesSrvc = $PropertiesSrvc;

    parent::__construct($stream, $this->oPropertiesSrvc->get('log_level'), $bubble, $filePermission, $useLocking);

  }

  /**
   * {@inheritdoc}
   * @param LogRecord $record
   * @return void
   */
  protected function write(LogRecord $record): void {
    // récupère le chemin absolu du fichier
    $sFile = $this->getUrl();
    if (file_exists($sFile)) {
      // détermine la taille du fichier de log en octet
      $iLogFileSize = filesize($sFile);
      // calcul la taille du fichier de log aprés écriture en octet
      $iRowWroten = strlen((string) $record->formatted);
      $iFileSizeAfter = $iLogFileSize + $iRowWroten;
      // Si le fichier aprés écriture dépasse la taille limite on renomme le fichier pour écrire dans un nouveau
      if (($this->oPropertiesSrvc->get('log_size') - 1) <= $iFileSizeAfter) {
        $sFileSplitName = str_replace('.log', '.' . date('Y_m_d_U') . '.log', $sFile);
        if (!rename($sFile, $sFileSplitName)) {
          throw new \Exception('Error while moving log file from ' . $sFile . ' to ' . $sFileSplitName);
        }
      }
    }

    parent::write($record);
  }

}
