<?php

namespace App\Module\Vitis\WebService\Service\PrintService;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\FileAccessor\FileManager;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use App\Module\Vitis\WebService\Service\PrintService\AbstractCaller;
use App\Module\Vitis\WebService\Service\VitisSystem;
use Symfony\Component\Process\Exception\ProcessFailedException;


abstract class AbstractPrinter extends AbstractCaller {
  /**
   * Fonction à appeler
   * @var array
   */
  protected const NODE_SCRIPT_PATH = [
    'command_lin' => '@prop(vitis_engine_dir)/printService',
    'command_win' => '@prop(vitis_engine_dir)/printService.exe',
  ];
  /**
   * @var string
   */
  protected $sTempDir;
  /**
   * @var \Monolog\Logger
   */
  protected $oDebugLogger;
  /**
   * @var \Monolog\Logger
   */
  protected $oErrorLogger;
  /**
   * {@inheritdoc}
   */
  public function __construct(LoggerManager $oLogger, Properties $properties, FileManager $oFileManager) {
    parent::__construct($oLogger, $properties, $oFileManager);
    $this->oDebugLogger = $oLogger->get('COMMON_DEBUG');
    $this->oErrorLogger = $oLogger->get('API_ERROR');
  }

  /**
   * Récupère la commande à utiliser pour accéder au print server
   * @throws \Exception si le chemin n'arrive pas à être résolu
   * @return string
   */
  private function getNodeCommand(): string {
    //exécution de la commande
    $sCommand = self::NODE_SCRIPT_PATH['command_lin'];
    if (VitisSystem::runOnWindows()) {
      $sCommand = self::NODE_SCRIPT_PATH['command_win'];
    }
    $sNodeScriptPath = $this->oProperties->replacePropertiesInString($sCommand);
    if ($sNodeScriptPath === null) {
      throw new \Exception('Cannot process path to node script');
    }

    return $sNodeScriptPath;
  }

  /**
   * call printService
   *
   * @param  array  $aArguments arguments à utiliser pour l'engine
   * @throws ProcessFailedException si le process termine en erreur
   * @return string retour de la commande
   */
  protected function callPrinter(array $aArguments): string {
    $sNodeScriptPath = $this->getNodeCommand();
    return $this->call($sNodeScriptPath, $aArguments);
  }
}
