<?php

namespace App\Module\Vitis\WebService\Service\PrintService;

class HtmlRenderer extends AbstractPrinter {
  /**
   * Evalue un code HTML en tuilisant un scope json
   * @param string $sHtmlBody HTML à évaluer
   * @param array $aScope donnée utilisé dans le template HTML
   * @param array $aIncludes images à inclure dans l'HTML
   * @param bool $bPaged si true alors active pajedjs lors de l'impression
   * @return string|null HTML évalué ou null en cas d'erreur
   */
  public function render(string $sHtmlBody, array $aScope, array $aIncludes, bool $bPaged=false): ?string {
    try {
      // génération du tableau des arguments
      $aArguments = $this->prepareArguments($sHtmlBody, $aScope, $aIncludes, $bPaged);
      //exécution de la commande
      $sReturn = $this->callPrinter($aArguments);
      $this->oDebugLogger->debug($sReturn);
      $sHtml = $this->oLocalFileAccessor->file_get_contents($this->sTempDir . '/rendered.html');
    } catch (\Exception $e) {
      $this->oErrorLogger->error('Impossible to render HTML : ' . $e->getMessage());
      return null;
    } finally {
      if ($this->oLocalFileAccessor->file_exists($this->sTempDir)) {
        $this->oLocalFileAccessor->clearDir($this->sTempDir);
      }
    }
    return $sHtml;
  }

  /**
   * Prépare les arguments pour la CLI
   * @param string $sHtmlBody HTML à évaluer
   * @param array $aScope donnée utilisé dans le template HTML
   * @param array $aIncludes images à inclure dans l'HTML
   * @param bool $bPaged si true alors active pajedjs lors de l'impression
   * @throws \Exception si le dossier temporaire ne peut pas être créé
   * @return array tableau d'argument à utiliser pour appeler phantomjs
   */
  private function prepareArguments(string $sHtmlBody, array $aScope, array $aIncludes, bool $bPaged): array{
    $this->sTempDir = $this->oLocalFileAccessor->getTempDir();
    if ($this->oLocalFileAccessor->mkdir($this->sTempDir, 0777, true) !== true) {
      throw new \Exception('Cannot create Tempdir : ' . $this->sTempDir);
    }
    // génération d'un dossier temporaire
    $aArguments = ['renderHtml'];
    // chemin vers l'entrée HTML
    $sHtmlPath = $this->sTempDir . '/input.html';
    $this->oLocalFileAccessor->file_put_contents($sHtmlPath, $sHtmlBody);
    $aArguments[] = '--HTMLInputFileName="' . $sHtmlPath . '"';
    // chemin vers la sortie HTML
    $aArguments[] = '--OutputFileName="' . $this->sTempDir . '/rendered.html' . '"';
    // chemin vers les includes
    $sIncludesPath = $this->sTempDir . '/includes.json';
    $this->oLocalFileAccessor->file_put_contents($sIncludesPath, json_encode($aIncludes));
    $aArguments[] = '--JsonIncludes="' . $sIncludesPath . '"';
    // chemin vers le scope
    $sScopePath = $this->sTempDir . '/scope.json';
    $this->oLocalFileAccessor->file_put_contents($sScopePath, json_encode($aScope));
    $aArguments[] = '--JsonScope="' . $sScopePath . '"';
    $aArguments[] = '--Paged="' . ( $bPaged === true ? 'true' : 'false' ) . '"';

    return $aArguments;
  }

}
