<?php

namespace App\Module\Vitis\WebService\Service;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use Symfony\Component\HttpClient\HttpClient;
use Symfony\Contracts\HttpClient\HttpClientInterface;

/**
 * Service permettant d'intégrer l'utilisation du proxy sur l'httpclient de symfony
 *
 * @author Frédéric Carretero <frederic.Carretero@veremes.com>
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */

class VitisHttpClient {
  /**
   *
   * @var Properties
   */
  private $oProperties;

  /**
   * Initialise le controler et instancie les services requis
   * @param Properties $oPropertiesSrvc Service d'accès aux properties
   */
  public function __construct(Properties $oPropertiesSrvc) {
    $this->oProperties = $oPropertiesSrvc;
  }

  /**
   * encapsule httpClient pour éviter les oubli sur le proxy
   * @param array $aHttpClientParams paramètre pour le client http
   * @return HttpClientInterface
   */
  public function createClient( array $aHttpClientParams = []): HttpClientInterface {
    $sProxyUrl = $this->oProperties->get('proxy_server_url');
    $sProxyPort = $this->oProperties->get('proxy_server_port');
    if (!empty($sProxyUrl) && !empty($sProxyPort)) {
      $aProxyUrlParams = parse_url($sProxyUrl);
      $sProxy = $aProxyUrlParams['host'] . ':' . $sProxyPort;
      $sProxyLogin = $this->oProperties->get('proxy_server_login');
      $sProxyPassword = $this->oProperties->get('proxy_server_password');
      if (!empty($sProxyLogin) && !empty($sProxyPassword)) {
        $sProxy = $sProxyLogin . ':' . $sProxyPassword . '@' . $sProxy;
      }
      $sProxy = $aProxyUrlParams['scheme'] . '://' . $sProxy;
      $aHttpClientParams['proxy'] = $sProxy;
    }

    // Lance la requête pour vérifier le captcha
    return HttpClient::create($aHttpClientParams);
  }
}
