<?php

namespace App\Module\Vitis\WebService\Service;

use App\Module\Vitis\WebService\Service\Configuration\Properties;
use App\Module\Vitis\WebService\Service\Logger\LoggerManager;
use Monolog\Logger;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Process\Process;

/**
 * Service permettant d'intéragir avec process et de l'encadrer
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */

class VitisProcess {
  /**
   * @var Properties
   */
  private $oProperties;

  /**
   * @var Logger
   */
  private $oLogger;

  /**
   * @var Process
   */
  private $oProcess;

  /**
   * @param array $aCommandLine voir parent
   * @param string $sCwd dossier d'exécution
   * @param Properties $properties service d'accés aux properties
   * @param LoggerManager $oLogger service d'accés aux logger
   */
  public function __construct(array $aCommandLine, string $sCwd = null, Properties $properties, LoggerManager $oLogger) {
    $this->oProperties = $properties;
    $this->oLogger = $oLogger->get('API_ERROR');
    $this->oProcess = new Process($aCommandLine, $sCwd);

    // Valeur du timeout en fonction de la propertie max_execution_time
    $iTimeout = $this->oProperties->get('max_execution_time');
    if (is_numeric($iTimeout)) {
      $this->oProcess->setTimeout($iTimeout);
    }
  }

  /**
   * @return Process
   */
  public function getProcess(): Process {
    return $this->oProcess;
  }

  /**
   * Identique à start mais retourne un string
   * @param callable $callback voir start dans le parent
   * @param array $env environnement, voir start dans le parent
   * @return string dernière ligne provenant de STDOUT
   * @throws ProcessFailedException
   * @deprecated
   */
  public function startWithReturn(callable $callback = null, array $env = []): string {

    $this->logProcess();
    $this->oProcess->start($callback, $env);

    $sLastLine = '';
    foreach ($this->oProcess as $sType => $sData) {
      if ($this->oProcess::OUT === $sType) {
        $this->oLogger->debug('Read from stdout : ' . $sData);
        // force sdata en string
        $tmp = explode("\n", trim("$sData"));
        $sLastLine = end($tmp);
      } else { // parent::ERR === $sType
        $this->oLogger->error('Read from stderr : ' . $sData);
        throw new ProcessFailedException($this->oProcess);
      }
    }

    return trim($sLastLine);
  }

  /**
   * {@inheritDoc}
   * @deprecated
   */
  public function runWithAdapter(callable $callback = null, array $env = []): int {

    $this->logProcess();
    return $this->oProcess->run($callback, $env);
  }

  /**
   * Lance la commande simplement avec du php natif
   *
   * @return array|null un tableau contenant les retours de commande si tout va bien null en cas d'erreur
   */
  public function execute(): ?array{
    $output = [];
    $result_code = null;

    $sCwd = $this->oProcess->getWorkingDirectory();
    $sFinalCommandLine = "cd \"$sCwd\" && " . $this->oProcess->getCommandLine() . ' 2>&1';

    $this->logProcess();
    exec($sFinalCommandLine, $output, $result_code);

    // on supprime les row vide du tablea de sortie
    $output = array_filter($output, fn(mixed $value) => !empty(trim($value)));

    if ($result_code > 0) {
      $sMessage = (array_key_exists($result_code, $this->oProcess::$exitCodes)) ? ': ' . $this->oProcess::$exitCodes[$result_code] : '';
      $this->oLogger->error("Error occured while executing command $sMessage (code : $result_code)", $output);
      return null;
    }

    return $output;
  }

  /**
   * log la commande qui va être lancée
   *
   * @return void
   */
  private function logProcess(): void {
    $sFinalCommandLine = $this->oProcess->getCommandLine();
    $sCwd = $this->oProcess->getWorkingDirectory();
    // $env = $this->oProcess->getEnv();
    // $input = $this->oProcess->getInput();
    // $iTimeout = $this->oProcess->getTimeout();

    $this->oLogger->debug("Run command $sFinalCommandLine from $sCwd"/*, [
      'timeout' => $iTimeout,
      'env' => $env,
      'input' => $input,
    ]*/);

  }
}
