<?php

namespace App\Module\Vitis\WebService\Entity;

use App\Module\Vitis\WebService\Service\Configuration\Route;
use App\Module\Vitis\WebService\Service\Normalizer\VitisDeserializerInterface;
use \JsonSerializable;

/**
 * Objet ORM Abstrait pour exploiter le fichier route.json
 *
 * @author Anthony Borghi <anthony.borghi@veremes.com>
 */

abstract class AbstractEntity implements JsonSerializable, VitisDeserializerInterface {
  /**
   * @var array
   */
  protected $aRequestData;
  /**
   * @var Route
   */
  protected $oRouteConfigurator;
  /**
   *  @var array
   */
  protected $aRouteConfiguration = [];

  public function setRouteConfiguratorService(Route $oService): void {
    $this->oRouteConfigurator = $oService;
    $this->aRouteConfiguration = $oService->getRequestParameters();
  }

  /**
   * @param  array $aData
   * @return void
   */
  public function setRequestData(array $aData) : void {
    $this->aRequestData = $aData;
  }

  /**
   * Retourne un tableau pour la serialization en json (JsonSerializable) à partir du route.json
   * @param array|null $aRouteConfiguration
   * @return array
   */
  public function jsonSerialize(?array $aRouteConfiguration = null): array{
    $aDataReturn = [];

    if ($aRouteConfiguration === null) {
      $aRouteConfiguration = $this->aRouteConfiguration;
    }

    if (!empty($aRouteConfiguration)) {
      foreach ($aRouteConfiguration['options']['fields'] as $aField) {
        $fn = $aField['orm_get'];
        if (isset($fn)) {
          $aDataReturn[$aField['name']] = $this->$fn();
        }
      }
    }

    return $aDataReturn;
  }

  /**
   * Retourne un tableau pour la deserialization en json (VitisDeserializerInterface) à partir du route.json
   * @return array
   */
  public function getDeserializeMap(): array{
    $aSetterMap = [];
    if (!empty($this->aRouteConfiguration)) {
      foreach ($this->aRouteConfiguration['options']['fields'] as $aField) {
        if (isset($aField['orm_set'])) {
          $aSetterMap[$aField['name']] = $aField['orm_set'];
        }
      }
    }
    return $aSetterMap;
  }

}